/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

/*
 * $Revision: 1.13 $
 *
 * Generic sysinfo types
 */

#ifndef __mcgeneric_h__
#define __mcgeneric_h__ 

#include "mcsi.h"

/*
 * List Info type
 */
struct _ListInfo {
    int				IntKey;		/* Int key */
    char		       *Name;		/* Keyword name */
    char		       *Desc;		/* Description */
    void		      (*List)();	/* List function */
    int				CompatList[16];	/* Compatibility List */
};
typedef struct _ListInfo	ListInfo_t;

/*
 * Header Format type
 */
typedef struct {
#define HDR_FMT_MAX		16
    char		       *Headers[HDR_FMT_MAX]; 	/* Headers */
    size_t			HeadersCount;		/* No. of Headers */
    int				Minimums[HDR_FMT_MAX]; 	/* Min field size */
    size_t			MinimumsCount;		/* No. of Minimums */
    int				Percents[HDR_FMT_MAX]; 	/* Percent field size*/
    size_t			PercentsCount;		/* No. of Percents */
    int				MinOvers[HDR_FMT_MAX]; 	/* Override Minimums */
    size_t			MinOversCount;		/* No. of MinOver... */
} HeaderFormat_t;

/*
 * Show Information
 */
typedef struct {
    char		       *Name;		/* Name */
    char		       *Label;		/* Label/description */
    int				GetType;	/* What to get */
    int				Enabled;	/* Show this item? */
} ShowInfo_t;

/*
 * Class Information
 */
typedef struct {
    char		       *Name;		/* Class name */
    char		       *Aliases[8];	/* Aliases for Name */
    int			      (*Show)();	/* Show function */
    void		      (*List)();	/* List function */
    ShowInfo_t *		ShowInfo;	/* Show info list */
    char *			ShowUsage;	/* CLI usage info for --show */
    char		       *Label;		/* It's label/description */
    char		       *Banner;		/* Banner */
    HeaderFormat_t	       *Headers;	/* Normal header info */
    HeaderFormat_t	       *HeadersAll;	/* -msglevel all Headers */
    int				Enabled;	/* Is enabled or not */
    int				UserEnabled;	/* User enabled */
    MCSIopts_t		       *Options;	/* Options */
} ClassInfo_t;
#define CLASS_UNSUPPORTED	-2

/*
 * Description
 */
struct _Desc {
    char		       *Label;		/* Label of description */
    char		       *Desc;		/* Description */
    int				Flags;		/* Is primary description */
    struct _Desc	       *Next;		/* Pointer to next entry */
};
typedef struct _Desc 		Desc_t;
#define	DevDesc_t		Desc_t		/* Backwards compat */
#define AddDesc			DescAdd		/* Backwards compat */

/*
 * Description Action
 */
#define DA_APPEND		0x01		/* Append entry */
#define DA_INSERT		0x02		/* Insert entry */
#define DA_PRIME		0x10		/* Indicate primary */

/*
 * Unique Identifier
 */
#define IDT_UNKNOWN		0		/* Unknown format */
#define IDT_ASCII		1		/* ASCII data in Identifier */
#define IDT_BINARY		2		/* Binary (unprintable) */
typedef struct {
    int				Type;		/* Type of Identifier IDT_* */
    size_t			Length;		/* Length of Identifier */
    unsigned char	       *Identifier;	/* Unique ID */
} Ident_t;

/*
 * Generic Name/Value table
 */
typedef struct {
    int			value;			/* Value field */
    char       	       *name;			/* Corresponding name */
    char       	       *valuestr;		/* Value string field */
} NameTab_t;

/*
 * Conditions for matching criteria
 */
struct _Condition {
    char		       *Key;		/* Key word */
    char		       *ValStr;		/* String Value */
    int				ValInt;		/* Int Value */
    char		      **ValArray;	/* Array of string values */
    int				Matches;	/* Matches? */
    int				Flags;		/* Flags */
    struct _Condition	       *Next;		/* Next */
};
typedef struct _Condition	Condition_t;

#define CONFL_SETONLY		0x01		/* Entry only used for set vars */

/*
 * Definetion information
 */
struct _Define {
    char	       *KeyStr;			/* Key string */
    long	        KeyNum;			/* Key numeric */
    char	       *ValStr1;		/* Value string 1 */
    char	       *ValStr2;		/* Value string 2 */
    char	       *ValStr3;		/* Value string 3 */
    char	       *ValStr4;		/* Value string 4 */
    char	       *ValStr5;		/* Value string 5 */
    int			ValInt1;		/* Value int 1 */
    Condition_t	       *Conditions;		/* Matching conditions */
    struct _Define     *Next;			/* Next member */
};
typedef struct _Define Define_t;

/*
 * Definetion List
 */
struct _DefineList {
    char	       *Name;			/* Name of list */
    Define_t	       *Defines;		/* Definetion list */
    struct _DefineList *Next;			/* Next member */
};
typedef struct _DefineList DefineList_t;
    
/*
 * Generic Key/Value table
 */
typedef struct {
    char       	       *Key;			/* Corresponding name */
    long		Lvalue;			/* Long value field */
    char       	       *Svalue;			/* Value string field */
} KeyTab_t;

#endif	/* __mcgeneric.h */
