/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

/*
 * $Revision: 1.15 $
 *
 * NetIf definetions
 */

#ifndef __mcnetif_h__
#define __mcnetif_h__ 

/*
 * Flag info
 */
struct _NetIfFlag_t {
    unsigned long		Flag;		/* Numeric value */
    char *			Name;		/* 1 word name */
    char *			Desc;		/* Description */
    struct _NetIfFlag_t *	Next;
};
typedef struct _NetIfFlag_t	NetIfFlag_t;

/*
 * Network Interface specific data
 * XXX When adding members, need to update NetIfDestroy() in netif.c
 */
struct _netif {
    char *		Name;			/* Interface device name */
    char *		IfType;			/* Interface Type */
    char *		AddrType;		/* Name of address type */
    char *		HostAddr;		/* Host address */
    char *		HostName;		/* Host name */
    char *		MACaddr;		/* Current MAC address */
    char *		MACname;		/* Current MAC name */
    char *		FacMACaddr;		/* Factory MAC address */
    char *		FacMACname;		/* Factory MAC name */
    char *		NetAddr;		/* Network address */
    char *		NetName;		/* Network name */
    char *		NetMask;		/* Netmask value */
    char * 		BroadAddr;		/* Broadcast address */
    int			MtuSize;		/* MTU Size */
    char *		MediaType;		/* Type of Media */
    char *		MediaSubType;		/* Sub Type of Media */
    char *		Speed;			/* Speed of link */
    char *		Duplex;			/* FULL or HALF */
    NetIfFlag_t *       Flags;			/* Flags for this if */
    /* Values used internally */
    int			Unit;			/* Unit # */
    char *		Model;			/* Hardware model */
    /* Link list pointers */
    struct _netif *	NextAddr;		/* Next address for this IF */
    struct _netif *	Next;			/* Pointer to next element */
};
typedef struct _netif NetIF_t;

/*
 * Values for NetIF_t.MediaType
 */
#define IFMT_ETHERNET	"ethernet"		/* Ethernet */
#define IFMT_FDDI	"fddi"			/* FDDI */
#define IFMT_TOKEN	"tokenring"		/* Token Ring */
#define IFMT_80211	"wifi"			/* IEEE 802.11 WiFi */

/*
 * Values for NetIF_t.IfType
 */
#define IFT_PHYSICAL	"physical"		/* Physical interface */
#define IFT_VIRTUAL	"virtual"		/* Virtual interface */

/*
 * Values for NetIF_t.Duplex
 */
#define IFD_FULL	"full"			/* Full duplex */
#define IFD_HALF	"half"			/* Half duplex */

/*
 * Address family table
 */
typedef struct {
    int			Type;			/* Type value */
    char	       *Name;			/* Name value */
    NetIF_t	     *(*GetNetIF)();		/* Function to set netif */
} AddrFamily_t;

#endif	/* __mcnetif_h__ */
