/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

/*
 * $Revision: 1.19 $
 *
 * MCSI definetions
 */

#ifndef __mcsi_h__
#define __mcsi_h__ 

/*
 * Values for MCSIquery_t->Cmd
 */
#define MCSI_HOSTNAME		1		/* Host Name */
#define MCSI_HOSTALIASES	2		/* Host Aliases */
#define MCSI_HOSTADDRS		3		/* Host Addresses */
#define MCSI_HOSTID		4		/* Host ID */
#define MCSI_SERIAL		5		/* Serial # */
#define MCSI_MANSHORT		6		/* Manufacturer (Short) */
#define MCSI_MANLONG		7		/* Manufacturer (Long) */
#define MCSI_MAN		8		/* Manufacturer (Short+Long) */
#define MCSI_MODEL		9		/* System Model */
#define MCSI_PHYSMEM		10		/* Physical Memory */
#define MCSI_VIRTMEM		11		/* Virtual Memory */
#define MCSI_ROMVER		12		/* ROM Version */
#define MCSI_NUMCPU		13		/* # of CPUs */
#define MCSI_CPUTYPE		14		/* CPU Type */
#define MCSI_APPARCH		15		/* Application Arch */
#define MCSI_KERNARCH		16		/* Kernel Arch */
#define MCSI_OSNAME		17		/* OS Name */
#define MCSI_OSVER		18		/* OS Version */
#define MCSI_OSDIST		19		/* OS Distribution */
#define MCSI_KERNVER		20		/* Kernel Version */
#define MCSI_BOOTTIME		21		/* Boot Time */
#define MCSI_CURRENTTIME	22		/* Current Time */
#define MCSI_CPUSPEED		23		/* CPU Speed */
#define MCSI_CPUINFO		24		/* CpuInfo_t */
#define MCSI_LIBCNAME		25		/* Libc Name */
#define MCSI_LIBCVER		26		/* Libc version */
#define MCSI_KERNBITS		27		/* Kernel Bit Size */
#define MCSI_OSDISTNAME		28		/* OS Distribution Name */
#define MCSI_OSDISTNAMESH	29		/* OS Distribution Name */
#define MCSI_OSDISTVER		30		/* OS Distribution Version */
#define MCSI_NUMCPUVIRT		31		/* # of CPUs (Virtual) */
#define MCSI_DEVTREE		40		/* Device Tree */
#define MCSI_KERNELVAR		50		/* Kernel Variables */
#define MCSI_SYSCONF		51		/* sysconf() info */
#define MCSI_SOFTINFO		60		/* Software Tree */
#define MCSI_PARTITION		70		/* Partition Info Tree */
#define MCSI_PRINTER		80		/* Printer Tree */
#define MCSI_LICENSE		90		/* License Tree */
#define MCSI_NETWORK		91		/* Network Tree */
#define MCSI_NETIF		92		/* Network Interfaces */

/*
 * Values for MCSIquery_t->Op (Operator)
 */
#define MCSIOP_CREATE		0		/* Create (get) req. data */
#define MCSIOP_DESTROY		1		/* Destroy previous data */
#define MCSIOP_PROGRAM		2		/* Set program name */
#define MCSIOP_SETOPTS		3		/* Set options */

/*
 * Values for MCSIquery_t->Flags (Option Flags)
 */
#define MCSIF_DATA		0x000		/* Return basic data type */
#define MCSIF_STRING		0x001		/* Return single string */

/*
 * Type for passing options to specific MCSI_* commands.
 */
struct _MCSIopts {
    int			Op;		/* Operator this option set is for */
    Opaque_t		Options;	/* Operator specific options */
    struct _MCSIopts   *Next;
};
typedef struct _MCSIopts	MCSIopts_t;

/*
 * Query type for mcSysInfo() calls
 */
typedef struct {
    /* Caller's Input */
    int			Op;		/* MCSIOP_* operator */
    int			Cmd;		/* MCSI_* command to perform */
    int			Flags;		/* Option Flags */
    char	      **SearchExp;	/* Search Expression */
    Opaque_t	        In;		/* Ptr to Input */
    size_t		InSize;		/* Size of In */
    MCSIopts_t	       *Options;	/* Options */
    long		SubClasses;	/* Class specific request flags */
    /* Results */
    Opaque_t	        Out;		/* Ptr to Output */
    size_t		OutSize;	/* Size of Out */
} MCSIquery_t;

#endif	/* __mcsi_h__ */
