#
# $Id: mcDevInfo.pm,v 1.28 2003/04/01 16:53:43 mcooper Exp $
#
# DevInfo object
#

package mcDevInfo;
require 5.003;

use mcMisc;
use mcDebug;
use mcGetSize;
use mcCacheInfo;
use mcCpuInfo;
use mcNetIfInfo;

my $PresentOrderTrack = 1;
my $Summaries;
my $DescKey = 'Desc000';

my $Descriptions =
	{
	 'Name'		=> {Order=>10, Desc=>'Name'},
	 'Driver'	=> {Order=>11, Desc=>'Driver'},
	 'AltName'	=> {Order=>12, Desc=>'AKA Name'},
	 'Aliases'	=> {Order=>13, Desc=>'Aliases'},
	 'Vendor'	=> {Order=>20, Desc=>'Manufacturer'},
	 'Model'	=> {Order=>21, Desc=>'Model'},
	 'ModelDesc'	=> {Order=>22, Desc=>'Model Description'},
	 'Part'		=> {Order=>23, Desc=>'Part #'},
	 'Serial'	=> {Order=>24, Desc=>'Serial #'},
	 'Ident'	=> {Order=>25, Desc=>'Identifier'},
	 'Revision'	=> {Order=>26, Desc=>'Revision'},
	 'Type'		=> {Order=>30, Desc=>'Type'},
	 'ClassType'	=> {Order=>31, Desc=>'Class Type'},
	 'ClassDesc'	=> {Order=>32, Desc=>'Class Description'},
	 'Capacity'	=> {Order=>50, Desc=>'Capacity'},
	 'Speed'	=> {Order=>51, Desc=>'Speed'},
	 'Unit'		=> {Order=>60, Desc=>'Unit'},
	 'NodeID'	=> {Order=>61, Desc=>'Node ID'},
	 'Bus'		=> {Order=>62, Desc=>'Bus #'},
	 'DevNum'	=> {Order=>63, Desc=>'Device #'},
	 'Port'		=> {Order=>64, Desc=>'Port #'},
	 'Files'	=> {Order=>70, Desc=>'Device Files'},
	 # CPU
	 'Arch'		=> {Order=>80, Desc=>'Architecture'},
	 'ArchVer'	=> {Order=>81, Desc=>'Architecture Version'},
	 # DiskDrive values
	 'rpm'		=> {Order=>90, Desc=>'RPM'},
	 'cylskew'	=> {Order=>91, Desc=>'Cylinder Skew'},
	 'trackskew'	=> {Order=>92, Desc=>'Track Skew'},
	 'phycyl'	=> {Order=>93, Desc=>'Physical Cylinders'},
	 'datacyl'	=> {Order=>94, Desc=>'Data Cylinders'},
	 'sectsize'	=> {Order=>95, Desc=>'Sector Size (bytes)'},
	 # Don't show these values
	 'Caches'	=> {NoShow=>1},
	 'DescList'	=> {NoShow=>1},
	 'Master'	=> {NoShow=>1},
	 'DevSpec'	=> {NoShow=>1},
	 'Slaves'	=> {NoShow=>1},
	 'Next'		=> {NoShow=>1},
	 'Partitions'	=> {NoShow=>1},
	};

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"Name"} = undef;
    $self->{"Driver"} = undef;
    $self->{"Aliases"} = [];
    $self->{"AltName"} = undef;
    $self->{"Files"} = [];
    $self->{"Type"} = undef;
    $self->{"ClassType"} = undef;
    $self->{"ClassDesc"} = undef;
    $self->{"Vendor"} = undef;
    $self->{"Model"} = undef;
    $self->{"ModelDesc"} = undef;
    $self->{"Part"} = undef;
    $self->{"Serial"} = undef;
    $self->{"Ident"} = undef;
    $self->{"Revision"} = undef;
    $self->{"DescList"} = [];
    $self->{"Unit"} = undef;
    $self->{"NodeID"} = undef;
    $self->{"Capacity"} = undef;
    $self->{"Speed"} = undef;
    $self->{"Bus"} = undef;
    $self->{"DevNum"} = undef;
    $self->{"Port"} = undef;
    $self->{"DevSpec"} = undef;
    $self->{"Slaves"} = [];		# Names of slaves
    $self->{"Master"} = undef;		# Name of master
    $self->{"Next"} = undef;		# Name of next (sibling) entry
    $self->{"SlaveNodes"} = [];	# DevInfo of slaves
    $self->{"MasterNode"} = undef;	# DevInfo master
    $self->{"NextNode"} = undef;	# DevInfo next (sibling) entry
    $self->{"Partitions"} = [];

    $self->{"Desc"} = undef;
    $self->{"ORDER"} = undef;
    $self->{"SUMMARY"} = undef;

    bless ($self, $class);

    return $self;
}

sub Name {
    my $self = shift;

    if (!@_) {
	return $self->{"Name"};
    }

    $self->{"Name"} = shift;

    return;
}

sub Driver {
    my $self = shift;

    if (!@_) {
	return $self->{"Driver"};
    }

    $self->{"Driver"} = shift;

    return;
}

sub Aliases {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"Aliases"} };
    }

    @{ $self->{"Aliases"} } = @_;

    return;
}

sub AltName {
    my $self = shift;

    if (!@_) {
	return $self->{"AltName"};
    }

    $self->{"AltName"} = shift;

    return;
}

sub Files {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"Files"} };
    }

    @{ $self->{"Files"} } = @_;

    return;
}

sub DescList {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"DescList"} };
    }

    @{ $self->{"DescList"} } = @_;

    return;
}

sub Type {
    my $self = shift;

    if (!@_) {
	return $self->{"Type"};
    }

    $self->{"Type"} = shift;

    return;
}

sub ClassType {
    my $self = shift;

    if (!@_) {
	return $self->{"ClassType"};
    }

    $self->{"ClassType"} = shift;

    return;
}

sub ClassDesc {
    my $self = shift;

    if (!@_) {
	return $self->{"ClassDesc"};
    }

    $self->{"ClassDesc"} = shift;

    return;
}

sub Vendor {
    my $self = shift;

    if (!@_) {
	return $self->{"Vendor"};
    }

    $self->{"Vendor"} = shift;

    return;
}

sub Model {
    my $self = shift;

    if (!@_) {
	return $self->{"Model"};
    }

    $self->{"Model"} = shift;

    return;
}

sub ModelDesc {
    my $self = shift;

    if (!@_) {
	return $self->{"ModelDesc"};
    }

    $self->{"ModelDesc"} = shift;

    return;
}

sub Part {
    my $self = shift;

    if (!@_) {
	return $self->{"Part"};
    }

    $self->{"Part"} = shift;

    return;
}

sub Serial {
    my $self = shift;

    if (!@_) {
	return $self->{"Serial"};
    }

    $self->{"Serial"} = shift;

    return;
}

sub Ident {
    my $self = shift;

    if (!@_) {
	return $self->{"Ident"};
    }

    $self->{"Ident"} = shift;

    return;
}

sub Revision {
    my $self = shift;

    if (!@_) {
	return $self->{"Revision"};
    }

    $self->{"Revision"} = shift;

    return;
}

sub Unit {
    my $self = shift;

    if (!@_) {
	return $self->{"Unit"};
    }

    $self->{"Unit"} = shift;

    return;
}

sub NodeID {
    my $self = shift;

    if (!@_) {
	return $self->{"NodeID"};
    }

    $self->{"NodeID"} = shift;

    return;
}

sub Capacity {
    my $self = shift;

    if (!@_) {
	return $self->{"Capacity"};
    }

    $self->{"Capacity"} = shift;

    return;
}

sub Speed {
    my $self = shift;

    if (!@_) {
	return $self->{"Speed"};
    }

    $self->{"Speed"} = shift;

    return;
}

sub Bus {
    my $self = shift;

    if (!@_) {
	return $self->{"Bus"};
    }

    $self->{"Bus"} = shift;

    return;
}

sub DevNum {
    my $self = shift;

    if (!@_) {
	return $self->{"DevNum"};
    }

    $self->{"DevNum"} = shift;

    return;
}

sub Port {
    my $self = shift;

    if (!@_) {
	return $self->{"Port"};
    }

    $self->{"Port"} = shift;

    return;
}

sub Partitions {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"Partitions"} };
    }

    @{ $self->{"Partitions"} } = @_;

    return;
}

sub Next {
    my $self = shift;

    if (!@_) {
	return $self->{"Next"};
    }

    $self->{"Next"} = shift;

    return;
}

sub Slaves {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"Slaves"} };
    }

    @{ $self->{"Slaves"} } = @_;

    return;
}

sub Master {
    my $self = shift;

    if (!@_) {
	return $self->{"Master"};
    }

    $self->{"Master"} = shift;

    return;
}

sub NextNode {
    my $self = shift;

    if (!@_) {
	return $self->{"NextNode"};
    }

    $self->{"NextNode"} = shift;

    return;
}

sub SlaveNodes {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"SlaveNodes"} };
    }

    @{ $self->{"SlaveNodes"} } = @_;

    return;
}

sub MasterNode {
    my $self = shift;

    if (!@_) {
	return $self->{"MasterNode"};
    }

    $self->{"MasterNode"} = shift;

    return;
}

sub DevSpec {
    my $self = shift;

    if (!@_) {
	return $self->{"DevSpec"};
    }

    $self->{"DevSpec"} = shift;

    return;
}

#
# Add Description
#
sub AddDesc {
    my $self = shift;
    my $Key = shift;
    my $Value = shift;

    if (!length($Value)) {
	return;
    }

    my @list = $self->DescList;
    push(@list, {Key=>$Key, Value=>$Value});
    $self->DescList(@list);
}

sub Num {
    my $Str = shift;

    $Str =~ s/ //g;		# Strip whitespace

    return $Str;
}

#
# Decode partition entry
#
sub DecodePart {
    my $self = shift;
    my @Fields = @_;

    my $Part = mcPartInfo->New;
    $Part->Name($self->Name . $Fields[4]);
    $Part->Slice($Fields[4]);
    $Part->StartSect(Num($Fields[5]));
    $Part->NumSect(Num($Fields[6]));
    $Part->Size(Num($Fields[7]));
    $Part->Type($Fields[8]);
    $Part->MntName($Fields[9]);

    my @p = $self->Partitions;
    push(@p, $Part);
    $self->Partitions(@p);
}

#
# Fetch description
#
sub Desc {
    my $self = shift;
    my $Key = shift;

    if ($Descriptions->{$Key}->{'NoShow'}) {
	return;
    }

    return $Descriptions->{$Key}->{'Desc'};
}

#
# Make a Summary line for a DevInfo entry
#
sub MakeSummary {
    my $DevInfo = shift;
    my $Args = shift;
    my $Encode = $Args->{Encode};
    my $Name = $DevInfo->Name;

    # Set summary
    my @s;

    if (Eq($Encode, 'html')) {
	push(@s, '<b>' . $Name . '</b>');
    } else {
	push(@s, $Name);
    }
    if ($DevInfo->AltName) {
	push(@s, '(' . $DevInfo->AltName . ')');
    }

    push(@s, "is a");

    #
    # Set start quote and end quote
    #
    my $sq = '"';
    my $eq = '"';
    if (Eq($Encode, 'html')) {
	$sq = '<i>';
	$eq = '</i>';
    }

    if ($DevInfo->Vendor && $DevInfo->Model) {
	push(@s, $sq . $DevInfo->Vendor . ' ' . $DevInfo->Model . $eq);
    } elsif ($DevInfo->Vendor) {
	push(@s, $sq . $DevInfo->Vendor . $eq);
    } elsif ($DevInfo->Model) {
	push(@s, $sq . $DevInfo->Model . $eq);
    }

    if ($DevInfo->ClassDesc) {
	push(@s, $DevInfo->ClassDesc);
    } elsif ($DevInfo->ClassType) {
	push(@s, $DevInfo->ClassType);
    }

    if ($DevInfo->ModelDesc &&
	(!$DevInfo->ClassType || !Eq($DevInfo->ClassType,
				     $DevInfo->ModelDesc))) {
	push(@s, $DevInfo->ModelDesc);
    }

    if ($DevInfo->Capacity &&
	(!$DevInfo->ModelDesc || !Eq($DevInfo->Capacity,
				     $DevInfo->ModelDesc))) {
	push(@s, $DevInfo->Capacity);
    }

    if ($DevInfo->Speed &&
	(!$DevInfo->ModelDesc || !Eq($DevInfo->Speed,
				     $DevInfo->ModelDesc))) {
	push(@s, $DevInfo->Speed);
    }

    if ($DevInfo->Type &&
	(!$DevInfo->ModelDesc || !Eq($DevInfo->Type,
				     $DevInfo->ModelDesc))) {
	push(@s, $DevInfo->Type);
    }

    return join(' ', @s);
}

#
# Fetch/set summary of this item
#
sub SUMMARY {
    my $self = shift;
    my $Desc;
    my $Args;

    foreach my $a (@_) {
	if (ref($a) eq 'HASH') {
	    $Args = $a;
	} elsif (!ref($a)) {
	    $Desc = $a;
	}
    }

    if ($Desc) {
	# Set description
	$Summaries->{$self->Name} = $Desc;
	return $Desc;
    }

    # Return summary
    return $Summaries->{$self->Name} || $self->MakeSummary($Args);
}

#
# Order item is presented in
#
sub ORDER {
    my $self = shift;
    my $Key = shift;
    my $Order = shift;

    if ($Key && $Order) {
	# Set order
	$PresentOrder->{$Key} = $Order;
	return $Order;
    }

    # Return order
    return $PresentOrder->{$Key} || $Descriptions->{$Key}->{Order};
}

#
# Decode an array of general fields
#
sub Decode {
    my $self = shift;
    my @Fields = @_;

    my $key = $Fields[2];

    if (Eq($Fields[1], 'name')) {
	$self->Name($Fields[2]);
	$self->AltName($Fields[3]);
	$self->Master($Fields[4]);
	if (length($Fields[5])) {
	    $self->Files(split(',', $Fields[5]));
	}
	# Field 6 - Type Name (e.g. DT_DISKDRIVE)
	$self->Type($Fields[7]);
	$self->Model($Fields[8]);
	$self->ModelDesc($Fields[9]);
	$self->Unit($Fields[10]);
	# Field 11 - Address
	# Field 12 - Prio
	# Field 13 - Vec
	$self->NodeID($Fields[14]);
	$self->ClassType($Fields[15]);
	$self->ClassDesc($Fields[16]);
	$self->Vendor($Fields[17]);
	$self->Serial($Fields[18]);
	$self->Revision($Fields[19]);
	$self->Ident($Fields[20]);
	$self->Part($Fields[21]);
	$self->Next($Fields[22]);
	if (length($Fields[23])) {
	    $self->Slaves(split(',', $Fields[23]));
	}
	$self->Capacity($Fields[24]);
	$self->Speed($Fields[25]);
	$self->Bus($Fields[26]);
	$self->DevNum($Fields[27]);
	$self->Port($Fields[28]);	

    } elsif (Eq($Fields[1], 'desc')) {
	$self->Name($Fields[2]);
	my $Key = $Fields[3];
	my $Value = $Fields[4];
	if (!length($Key)) {
	    # Make sure Key points to something unique
	    $Key = ++$DescKey;
	}
	if (Eq($Key, 'part')) {
	    $self->DecodePart(@_);
	} elsif (Eq($Key, 'internet')) {
	    my $NetIfInfo = mcNetIfInfo->New;
	    $NetIfInfo->DecodeDevInfo(@_);
	    $self->DevSpec($NetIfInfo);
	} else {
	    my $Tag;
	    # Use the object's Desc function to lookup its description
	    if (!$Tag &&  exists($self->{'Desc'})) {
		$Tag = $self->Desc($Key);
	    }
	    # Change display tag 'DescXXXX' to 'Description'
	    if (!$Tag && $Key =~ /Desc.[0-9]/) {
		$Tag = 'Description';
	    }
	    # Default to the key
	    if (!$Tag) {
		$Tag = ucfirst($Key);
	    }

	    #
	    # Some entries, such as diskdrive's, have additional "class"
	    # identifiers for $Key.  In the case of a diskdrive this is
	    # 'OS' or 'HW' indicating the source of the data.
	    #
	    if (length($Fields[5])) {
		$Key = $Fields[5] . ' ' . $Key;
		if ($Tag) {
		    $Tag = $Fields[5] . ' ' . $Tag;
		}
	    }

	    my @list = $self->DescList;
	    push(@list, {Key=>$Key, Tag=>$Tag, Value=>$Value } );
	    $self->DescList(@list);
	}
    } elsif (Eq($Fields[1], 'cpuinfo')) {
	$self->Name($Fields[2]);
	my $CpuInfo = mcCpuInfo->New;
	$CpuInfo->Decode(@_);
	$self->DevSpec($CpuInfo);
    } elsif (Eq($Fields[1], 'cacheinfo')) {
	$self->Name($Fields[2]);
	my $CacheInfo = mcCacheInfo->New;
	$CacheInfo->Decode(@_);
	# Add the cache info to the CPU Info
	my $CpuInfo = $self->DevSpec;
	my @Caches = $CpuInfo->Caches;
	push(@Caches, $CacheInfo);
	$CpuInfo->Caches(@Caches);
    } elsif (Eq($Fields[1], 'totaldisk')) {
	$self->Capacity(sprintf($Fields[3]));
	$self->ModelDesc(sprintf($Fields[3]));	# Backwards compat
	$self->Type('disk drive');
    } else {
	Debug("mcDevInfo->Decode: Keyword <%s> is unknown", $Fields[1]);
    }

    # Set presentation order
    $self->ORDER($key, $PresentOrderTrack++);

    return $self;
}

1; # Make require/use happy
