#
# $Id: mcFileCache.pm,v 1.1 2003/04/08 21:32:01 mcooper Exp $
#
# MagniComp File Cache
#

package mcFileCache;
require 5.003;

use mcGlobals;
use mcMisc;
use mcDebug;
use File::stat;

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $Args = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"Dir"} = $Args->{Dir};		# Cache directory
    $self->{"Name"} = $Args->{Name};		# Name of item to cache
    $self->{"ExpireTime"} = $Args->{ExpireTime};
    $self->{"DataFile"} = undef;
    $self->{"InfoFile"} = undef;

    bless ($self, $class);

    if ($self->Dir && $self->Name) {
	Debug("mcSysInfo->New: Name=<%s> Dir=<%s> ExpireTime=<%s>",
	      $self->Name, $self->Dir, $self->ExpireTime);
	$self->Init;
    }

    return $self;
}

sub Dir {
    my $self = shift;

    if (!@_) {
	return $self->{"Dir"};
    }

    $self->{"Dir"} = shift;

    return;
}

sub Name {
    my $self = shift;

    if (!@_) {
	return $self->{"Name"};
    }

    $self->{"Name"} = shift;

    return;
}

sub DataFile {
    my $self = shift;

    if (!@_) {
	return $self->{"DataFile"};
    }

    $self->{"DataFile"} = shift;

    return;
}

sub InfoFile {
    my $self = shift;

    if (!@_) {
	return $self->{"InfoFile"};
    }

    $self->{"InfoFile"} = shift;

    return;
}

sub ExpireTime {
    my $self = shift;

    if (!@_) {
	return $self->{"ExpireTime"};
    }

    $self->{"ExpireTime"} = shift;

    return;
}

#
# Save $Data to the DataFile
#
sub WriteData {
    my $self = shift;
    my $Data = shift;
    my $fh;

    if (!$Data || $self->{WFHerror}) {
	return;
    }

    if (!$self->{WFH}) {
	$fh = GetFH();
	unlink($self->DataFile);
	if (!open($fh, '>', $self->DataFile)) {
	    Debug("mcSysInfo->WriteData: %s: Open for write failed: %s",
		  $self->DataFile, $!);
	    $self->{WFHerror} = 1;	# Mark this obj with an error
	    return;
	}
	$self->{WFH} = $fh;
    } else {
	$fh = $self->{WFH};
    }

    if (!printf $fh ("%s", $Data)) {
	Error("mcSysInfo->StreamRead: Write cache file (%s) failed: %s",
	      $self->DataFile, $!);
	# Delete Cache data so we don't try again
	delete $Stream->{Cache};
    }
}

#
# Read in the InfoFile
#
sub InfoRead {
    my $self = shift;
    my $Info = {};

    if (!$self->InfoFile) {
	return;
    }

    if (!open($FH, '<', $self->InfoFile)) {
	Debug("mcFileCache->InfoRead: %s: Open for read failed: %s",
	      $self->InfoFile, $!);
	return;
    }

    while (my $Line = <$FH>) {
	if ($Line =~ /^\#/) {
	    next;
	}
	chomp($Line);
	my @parts = split('=', $Line);
	my $key = $parts[0];
	my $value = $parts[1];

	if (Eq($key, 'size')) {
	    $Info->{Size} = $value;
	} elsif (Eq($key, 'cachefile')) {
	    $Info->{CacheFile} = $value;
	}
    }

    close($FH);

    $self->{"Info"} = $Info;
}

#
# Close and save our state
#
sub Close {
    my $self = shift;
    my $Update = 0;

    if ($self->{WFH}) {
	# If WFH is open, we'll need to update the info file
	$Update = 1;
	close($self->{WFH});
	delete($self->{WFH});
    }
    if ($self->{RFH}) {
	close($self->{RFH});
	delete($self->{RFH});
    }

    #
    # No need to update info file
    #
    if (!$Update) {
	return;
    }

    Debug("mcFileCache->Close: Saving cache data to %s",
	  $self->InfoFile);

    my $st = stat($self->DataFile);
    if (!$st) {
	Debug("mcFileCache->Close: %s: Stat cache data failed: %s",
	      $self->DataFile, $!);
	return;
    }

    unlink($self->InfoFile);
    if (!open($FH, '>', $self->InfoFile)) {
	Debug("mcFileCache->Close: %s: Open for write/create failed: %s",
	      $self->InfoFile, $!);
	return;
    }

    printf $FH "cachefile=%s\n", $self->DataFile;
    printf $FH "size=%ld\n", $st->size;

    close($FH);
}

#
# Check to see if cache is valid.
# Returns 1 on success.
# Returns undef on error.
#
sub Valid {
    my $self = shift;

    my $st = stat($self->DataFile);
    if (!$st) {
	Debug("mcFileCache->Valid: %s: Stat cache data failed: %s",
	      $self->DataFile, $!);
	return;
    }

    # Check file size
    if ($st->size != $self->{Info}->{Size}) {
	Debug("mcFileCache->Valid: %s: File size mismatch (expect=%d actual=%d)",
	      $self->DataFile, $self->{Info}->{Size}, $st->size);
	return;
    }

    # Check ExpireTime
    if ($self->ExpireTime && (time - $st->mtime > $self->ExpireTime)) {
	Debug("mcFileCache->Valid: %s: Cache file expired (%d secs old)",
	      $self->DataFile, time - $st->mtime);
	return;
    }

    Debug("mcFileCache->Valid: %s: Cache file is valid.", $self->DataFile);

    return 1;
}

#
# Initialize $self
#
sub Init {
    my $self = shift;

    if (!$self->DataFile) {
	$self->DataFile($self->Dir . '/' . $self->Name . '.cdata');
    }
    if (!$self->InfoFile) {
	$self->InfoFile($self->Dir . '/' . $self->Name . '.cinfo');
	$self->InfoRead;
    }
}

#
# Setup cache info.
# Returns $self on success.
# Returns undef on error.
#
sub Setup {
    my $self = shift;

    if (!$self->Dir || !$self->Name) {
	Debug("mcFileCache->Create: Missing params.");
	return;
    }

    $self->Init;

    if (!(-d $self->Dir)) {
	my $cmd = sprintf("mkdir -p %s", $self->Dir);
	if (system($cmd) != 0) {
	    Debug("mcFileCache->Create: mkdir command (%s) failed.", $cmd);
	    return;
	}
    }

    return $self;
}

#
# Read a line of data from the current DataFile
#
sub ReadLine {
    my $self = shift;
    my $fh;

    if (!$self->{RFH}) {
	$fh = GetFH();
	if (!open($fh, '<', $self->DataFile)) {
	    Debug("mcFileCache->ReadLine: %s: Open data cache for read failed: %s",
		  $self->DataFile, $!);
	    return;
	}
	$self->{RFH} = $fh;
    } else {
	$fh = $self->{RFH};
    }

    my $line = <$fh>;
    chomp($line);

    return $line;
}

1;
