#
# $Id: mcHardware.pm,v 1.12 2003/03/10 01:23:56 mcooper Exp $
#
# Hardware object
#

package mcHardware;
require 5.003;

use mcDebug;
use mcMisc;
use mcDevInfo;

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"Entries"} = undef;
    $self->{"Tree"} = undef;
    $self->{"POSTPROCESS"} = undef;

    bless ($self, $class);

    return $self;
}

#
# Entries is a "horozontal" list of all hardware devices
#
sub Entries {
    my $self = shift;
    my $New;

    for my $Arg (@_) {
	if (ref($Arg) eq 'HASH') {
	    $New = $Arg;
	} else {
	    # Lookup and return result
	    return $self->{"Entries"}->{$Arg};
	}
    }

    if (!@_) {
	return $self->{"Entries"};
    }

    my $h = $self->{"Entries"};
    my @NewKey = keys %$New;
    my $NewVal = $New->{$NewKey[0]};
    $h->{$NewKey[0]} = $NewVal; # Overwrite old value
    $self->{"Entries"} = $h;

    return;
}

#
# Tree is a hierarchical tree list of all hardware devices
#
sub Tree {
    my $self = shift;

    if (!@_) {
	return $self->{"Tree"};
    }

    $self->{"Tree"} = shift;

    return;
}

#
# Post Process in any info we need to post decode.
#
sub POSTPROCESS {
    my $self = shift;
    my $TotalDisk = 'TOTAL DISK';

    Debug("mcHardware->POSTPROCESS: START");

    #
    # Update all DevInfo Entries to have appropriate NextNode, SlaveNodes,
    # and MasterNode members set to represent hierarchical tree of hardware.
    #
    Debug("mcHardware->POSTPROCESS: Building Tree");

    my $Tree = mcDevInfo->New;		# Create root of tree
    #$Tree->Type('** ROOT OF TREE **');	# Debugging message

    my $Entries = $self->{"Entries"};
    my @Names = keys %$Entries;

    #
    # Foreach DevInfo name (in sorted order), add the node's sibling
    # and child node info.
    #
    my $TotalDiskInfo;
    foreach my $name (sort {$Tree->ORDER($Entries->{$a}->Name) <=>
				$Tree->ORDER($Entries->{$b}->Name)} @Names) {

	my $DevInfo = $Entries->{$name};

	Debug("mcHardware->POSTPROCESS: Node=<%s> Next=<%s> Slaves=<%s> Master=<%s>",
	      $name, $DevInfo->Next, join(',', $DevInfo->Slaves),
	      $DevInfo->Master);

	if (Eq($name, $TotalDisk)) {
	    $TotalDiskInfo = $DevInfo;
	    $TotalDiskInfo->Name($TotalDisk);
	    $TotalDiskInfo->SUMMARY("TOTAL DISK CAPACITY IS " . $TotalDiskInfo->Capacity);
	    next;
	}

	#
	# Add the next (sibling) node if known
	#
	if ($DevInfo->Next) {
	    if (my $next = $Entries->{$DevInfo->Next}) {
		$DevInfo->NextNode($next);
	    } else {
		Debug("\tCannot find next node <%s> for node <%s>",
		      $DevInfo->Next, $name);
	    }
	}

	#
	# Find the master for this node
	#
	my $Master = $Tree;	# Masterless nodes get the root
	if ($DevInfo->Master) {
	    $Master = $Entries->{$DevInfo->Master};
	    if (!$Master) {
		Debug("\tCannot find master <%s> for node <%s>",
		      $DevInfo->Master, $name);
		# Default to root node
		$Master = $Tree;
	    }
	}
	my $mn = ($Master && $Master->Name) ? $Master->Name : $Master->Type;
	Debug("\tMaster set to <%s>", $mn);
	$DevInfo->MasterNode($Master);

	my @s = $Master->SlaveNodes;
	push(@s, $DevInfo);
	$Master->SlaveNodes(@s);
    }

    # Add total disk
    if ($TotalDiskInfo) {
	my @s = $Tree->SlaveNodes;
	push(@s, $TotalDiskInfo);
	$Tree->SlaveNodes(@s);
    }

    $self->Tree($Tree);

    Debug("mcHardware->POSTPROCESS: Tree Top Slaves:");
    foreach my $di ($Tree->SlaveNodes) {
	Debug("\tNode is <%s>", $di->Name);
    }

    Debug("mcHardware->POSTPROCESS: END");
}

#
# Decode an array of DevInfo fields
#
sub Decode {
    my $self = shift;
    my @Fields = @_;

    if (!Eq($Fields[0], "device")) {
	return;
    }

    my $DevName = $Fields[2];
    if (Eq($Fields[1], 'totaldisk')) {
	$DevName = 'TOTAL DISK';
    }

    my $DevInfo = mcDevInfo->New;
    # Lookup the DevInfo entry (if any)
    my $di = $self->Entries($DevName);
    if (!$di) {
	# Need a new object
	$di = $DevInfo;
    }
    # Decode and Set
    $di->Decode(@Fields);
    # Add it back
    $self->Entries({$DevName => $di});

    return $self;
}

1; # Make require/use happy
