#
# $Id: mcLicense.pm,v 1.2 2002/07/09 15:53:51 mcooper Exp $
#
# License object
#

package mcLicense;
require 5.003;

use mcDebug;
use mcMisc;
use mcLicenseInfo;

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"Entries"} = undef;
    $self->{"POSTPROCESS"} = undef;

    bless ($self, $class);

    return $self;
}

sub Entries {
    my $self = shift;
    my $New;

    for my $Arg (@_) {
	if (ref($Arg) eq 'HASH') {
	    $New = $Arg;
	} else {
	    # Lookup and return result
	    return $self->{"Entries"}->{$Arg};
	}
    }

    if (!@_) {
	return $self->{"Entries"};
    }

    my $h = $self->{"Entries"};
    my @NewKey = keys %$New;
    my $NewVal = $New->{$NewKey[0]};
    $h->{$NewKey[0]} = $NewVal; # Overwrite old value
    $self->{"Entries"} = $h;

    return;
}

#
# Post Process in any info we need to post decode.
#
sub POSTPROCESS {
    my $self = shift;

    Debug("mcLicense->PostProcess: START");

    my @Names = keys %{ $self->{"Entries"} };
    foreach my $name (@Names) {
	#Debug("mcLicense->PostProcess: Name=<%s>", $name);
	my $License = $self->Entries->{$name};

	# Set summary
	my @s;
	push(@s, $name);
	if ($License->SoftVer) {
	    push(@s, $License->SoftVer);
	}
	if ($License->Count) {
	    push(@s, 'provides');
	    push(@s, $License->Count);
	}
	if ($License->Type) {
	    push(@s, $License->Type);
	    push(@s, 'licenses');
	}
	if ($License->ExpireDate) {
	    push(@s, 'expires');
	    push(@s, $License->ExpireDate);
	}
	$License->SUMMARY(join(' ', @s));
	Debug("\tSUMMARY: %s", $License->SUMMARY);
    }

    Debug("mcLicense->PostProcess: END");
}

#
# Decode an array of license fields
#
sub Decode {
    my $self = shift;
    my @Fields = @_;

    if (!Eq($Fields[0], "license")) {
	return;
    }

    my $Name = $Fields[2];
    my $LicenseInfo = mcLicenseInfo->New;
    # Lookup the LicenseInfo entry (if any)
    my $li = $self->Entries($Name);
    if (!$li) {
	# Need a new object
	$li = $LicenseInfo;
    }
    # Decode and Set
    $li->Decode(@Fields);
    # Add it back
    $self->Entries({$Name => $li});

    return $self;
}

1; # Make require/use happy
