#
# $Id: mcMagniCompLic.pm,v 1.3 2003/03/08 01:44:21 mcooper Exp $
#
# MagniComp License
#
# This object is about MagniComp's proprietary license, not about
# the generalized SysInfo License class.
#

package mcMagniCompLic;
require 5.003;

use mcDebug;
use mcMisc;
use mcLicenseInfo;

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"Entries"} = undef;			# Hash of mcLicenseInfo objs
    $self->{"EntriesReport"} = undef;		# Licenses in Report format
    $self->{"Decode"} = undef;

    bless ($self, $class);

    return $self;
}

sub Decode {
    my $self = shift;

    if (!@_) {
	return $self->{"Decode"};
    }

    $self->{"Decode"} = shift;

    return;
}

sub Entries {
    my $self = shift;
    my $New;

    for my $Arg (@_) {
	if (ref($Arg) eq 'HASH') {
	    $New = $Arg;
	} else {
	    # Lookup and return result
	    return $self->{"Entries"}->{$Arg};
	}
    }

    if (!@_) {
	return $self->{"Entries"};
    }

    my $h = $self->{"Entries"};
    my @NewKey = keys %$New;
    my $NewVal = $New->{$NewKey[0]};
    $h->{$NewKey[0]} = $NewVal; # Overwrite old value
    $self->{"Entries"} = $h;

    return;
}

sub EntriesReport {
    my $self = shift;
    my $New;

    for my $Arg (@_) {
	if (ref($Arg) eq 'HASH') {
	    $New = $Arg;
	} else {
	    # Lookup and return result
	    return $self->{"EntriesReport"}->{$Arg};
	}
    }

    if (!@_) {
	return $self->{"EntriesReport"};
    }

    my $h = $self->{"EntriesReport"};
    my @NewKey = keys %$New;
    my $NewVal = $New->{$NewKey[0]};
    $h->{$NewKey[0]} = $NewVal; # Overwrite old value
    $self->{"EntriesReport"} = $h;

    return;
}

#
# Decode an array of license fields
#
sub Decode {
    my $self = shift;
    my @Fields = @_;

    if (!Eq($Fields[0], "mcl")) {
	return;
    }

    #
    # Create a unique Name
    #
    my $Name = join('_', $Fields[1], $Fields[2], $Fields[3], $Fields[4],
		    $Fields[12], $Fields[13]);

    #
    # Add entry as is to Report Entries
    #
    $self->EntriesReport({$Name => join('|', @Fields)});

    # See if we have an existing entry already
    if ($self->Entries($Name)) {
	# Found existing
	return $self;
    }

    #
    # See sie/mcl.c -> MCLtoStr() for field order
    #
    my $l = mcLicenseInfo->New;
    $l->SoftName($Fields[1]);
    $l->SoftVer($Fields[2]);
    $l->Type($Fields[3]);
    $l->Serial($Fields[4]);
    if ($Fields[5]) {
	$l->Count($Fields[5]);
    }
    $l->AccountNumber($Fields[6]);
    $l->AccountName($Fields[7]);
    $l->Key($Fields[8]);
    $l->IssueDate($Fields[9]);
    $l->ExpireDate($Fields[10]);
    $l->DaysLeft($Fields[11]);
    if ($Fields[12]) {
	$l->MinCPU($Fields[12]);
    }
    if ($Fields[13]) {
	$l->MaxCPU($Fields[13]);
    }
    $l->Comment($Fields[14]);
    $l->LicenseURL($Fields[15]);
    $l->HomePageURL($Fields[16]);
    $l->Status($Fields[17]);
    $l->HostName($Fields[18]);

    # Add it back
    $self->Entries({$Name => $l});

    return $self;
}

1; # Make require/use happy
