#
# $Id: mcNetIf.pm,v 1.2 2003/03/28 19:38:57 mcooper Exp $
#
# NetIf object
#

package mcNetIf;
require 5.003;

use mcDebug;
use mcMisc;
use mcNetIfInfo;

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"Entries"} = undef;
    $self->{"POSTPROCESS"} = undef;

    bless ($self, $class);

    return $self;
}

sub Entries {
    my $self = shift;
    my $New;

    for my $Arg (@_) {
	if (ref($Arg) eq 'HASH') {
	    $New = $Arg;
	} else {
	    # Lookup and return result
	    return $self->{"Entries"}->{$Arg};
	}
    }

    if (!@_) {
	return $self->{"Entries"};
    }

    my $h = $self->{"Entries"};
    my @NewKey = keys %$New;
    my $NewVal = $New->{$NewKey[0]};
    $h->{$NewKey[0]} = $NewVal; # Overwrite old value
    $self->{"Entries"} = $h;

    return;
}

#
# Post Process in any info we need to post decode.
#
sub POSTPROCESS {
    my $self = shift;

    Debug("mcNetIf->PostProcess: START");

    my @Names = keys %{ $self->{"Entries"} };

    # Calculate Padding with for $name
    my $Padding = 0;
    foreach my $name (@Names) {
	if (length($name) > $Padding) {
	    $Padding = length($name);
	}
    }

    foreach my $name (@Names) {
	#Debug("mcNetIf->PostProcess: Name=<%s>", $name);
	my $NetIf = $self->Entries->{$name};

	# Set summary
	my @s;
	push(@s, sprintf("%-*s", $Padding, $name));
	if ($NetIf->HostAddr) {
	    push(@s, 'addr');
	    push(@s, $NetIf->HostAddr);
	}
	if ($NetIf->NetMask) {
	    push(@s, 'mask');
	    push(@s, $NetIf->NetMask);
	}
	if ($NetIf->BroadAddr) {
	    push(@s, 'bcast');
	    push(@s, $NetIf->BroadAddr);
	}
	if ($NetIf->Speed) {
	    push(@s, $NetIf->Speed);
	    if ($NetIf->Duplex) {
		push(@s, $NetIf->Duplex . '-duplex');
	    }
	}
	$NetIf->SUMMARY(join(' ', @s));
	Debug("\tSUMMARY: %s", $NetIf->SUMMARY);
    }

    Debug("mcNetIf->PostProcess: END");
}

#
# Decode an array of NetIf fields
#
sub Decode {
    my $self = shift;
    my @Fields = @_;

    if (!Eq($Fields[0], "netif")) {
	return;
    }

    my $Name = $Fields[2];
    my $NetIfInfo = mcNetIfInfo->New;
    # Lookup the NetIfInfo entry (if any)
    my $li = $self->Entries($Name);
    if (!$li) {
	# Need a new object
	$li = $NetIfInfo;
    }
    # Decode and Set
    $li->Decode(@Fields);
    # Add it back
    $self->Entries({$Name => $li});

    return $self;
}

1; # Make require/use happy
