#
# $Id: mcNetIfInfo.pm,v 1.7 2003/04/29 20:56:24 mcooper Exp $
#
# NetIf Info object
#

package mcNetIfInfo;
require 5.003;

use mcMisc;

#
# Every object value must have a description
# These are only used for description when called
# from mcDevInfo.
#
my $Descriptions =
	{
	 'Type'		=> {Order=>10, Desc=>'NIC Address Type'},
	 'HostAddr'	=> {Order=>12, Desc=>'NIC Host Address'},
	 'HostName'	=> {Order=>14, Desc=>'NIC Host Name'},
	 'MACaddr'	=> {Order=>16, Desc=>'NIC MAC Current Address'},
	 'MACname'	=> {Order=>18, Desc=>'NIC MAC Current Name'},
	 'FacMACaddr'	=> {Order=>20, Desc=>'NIC MAC Factory Address'},
	 'FacMACname'	=> {Order=>22, Desc=>'NIC MAC Factory Name'},
	 'NetAddr'	=> {Order=>24, Desc=>'NIC Network Address'},
	 'NetName'	=> {Order=>26, Desc=>'NIC Network Name'},
	 'NetMask'	=> {Order=>28, Desc=>'NIC Network Mask'},
	 'BroadAddr'	=> {Order=>30, Desc=>'NIC Broadcast Address'},
	 'MtuSize'	=> {Order=>32, Desc=>'NIC MTU Size'},
	 'Speed'	=> {Order=>34, Desc=>'NIC Link Speed'},
	 'Duplex'	=> {Order=>36, Desc=>'NIC Link Duplex'},
	};

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"Name"} = undef;
    $self->{"IfType"} = undef;
    $self->{"Type"} = undef;
    $self->{"HostAddr"} = undef;
    $self->{"HostName"} = undef;
    $self->{"MACaddr"} = undef;
    $self->{"MACname"} = undef;
    $self->{"FacMACaddr"} = undef;
    $self->{"FacMACname"} = undef;
    $self->{"NetAddr"} = undef;
    $self->{"NetMask"} = undef;
    $self->{"NetName"} = undef;
    $self->{"BroadAddr"} = undef;
    $self->{"MtuSize"} = undef;
    $self->{"MediaType"} = undef;
    $self->{"MediaSubType"} = undef;
    $self->{"Speed"} = undef;
    $self->{"Duplex"} = undef;
    $self->{"Flags"} = [];

    $self->{"Desc"} = undef;
    $self->{"ORDER"} = undef;
    $self->{"SUMMARY"} = undef;

    bless ($self, $class);

    return $self;
}

sub Name {
    my $self = shift;

    if (!@_) {
	return $self->{"Name"};
    }

    $self->{"Name"} = shift;

    return;
}

sub Type {
    my $self = shift;

    if (!@_) {
	return $self->{"Type"};
    }

    $self->{"Type"} = shift;

    return;
}

sub IfType {
    my $self = shift;

    if (!@_) {
	return $self->{"IfType"};
    }

    $self->{"IfType"} = shift;

    return;
}

sub HostAddr {
    my $self = shift;

    if (!@_) {
	return $self->{"HostAddr"};
    }

    $self->{"HostAddr"} = shift;

    return;
}

sub HostName {
    my $self = shift;

    if (!@_) {
	return $self->{"HostName"};
    }

    $self->{"HostName"} = shift;

    return;
}

sub MACaddr {
    my $self = shift;

    if (!@_) {
	return $self->{"MACaddr"};
    }

    $self->{"MACaddr"} = shift;

    return;
}

sub MACname {
    my $self = shift;

    if (!@_) {
	return $self->{"MACname"};
    }

    $self->{"MACname"} = shift;

    return;
}

sub FacMACaddr {
    my $self = shift;

    if (!@_) {
	return $self->{"FacMACaddr"};
    }

    $self->{"FacMACaddr"} = shift;

    return;
}

sub FacMACname {
    my $self = shift;

    if (!@_) {
	return $self->{"FacMACname"};
    }

    $self->{"FacMACname"} = shift;

    return;
}

sub NetAddr {
    my $self = shift;

    if (!@_) {
	return $self->{"NetAddr"};
    }

    $self->{"NetAddr"} = shift;

    return;
}

sub NetMask {
    my $self = shift;

    if (!@_) {
	return $self->{"NetMask"};
    }

    $self->{"NetMask"} = shift;

    return;
}

sub NetName {
    my $self = shift;

    if (!@_) {
	return $self->{"NetName"};
    }

    $self->{"NetName"} = shift;

    return;
}

sub BroadAddr {
    my $self = shift;

    if (!@_) {
	return $self->{"BroadAddr"};
    }

    $self->{"BroadAddr"} = shift;

    return;
}

sub MtuSize {
    my $self = shift;

    if (!@_) {
	return $self->{"MtuSize"};
    }

    $self->{"MtuSize"} = shift;

    return;
}

sub Speed {
    my $self = shift;

    if (!@_) {
	return $self->{"Speed"};
    }

    $self->{"Speed"} = shift;

    return;
}

sub MediaType {
    my $self = shift;

    if (!@_) {
	return $self->{"MediaType"};
    }

    $self->{"MediaType"} = shift;

    return;
}

sub MediaSubType {
    my $self = shift;

    if (!@_) {
	return $self->{"MediaSubType"};
    }

    $self->{"MediaSubType"} = shift;

    return;
}

sub Duplex {
    my $self = shift;

    if (!@_) {
	return $self->{"Duplex"};
    }

    $self->{"Duplex"} = shift;

    return;
}

sub Flags {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"Flags"} };
    }

    @{ $self->{"Flags"} } = @_;

    return;
}

#
# Fetch/set summary of this item
#
sub SUMMARY {
    my $self = shift;
    my $Desc = shift;

    if ($Desc) {
	# Set description
	$Summaries->{$self->Name} = $Desc;
	return $Desc;
    }

    # Return summary
    return $Summaries->{$self->Name};
}

#
# Order item is presented in
#
sub ORDER {
    my $self = shift;
    my $Key = shift;
    my $Order = shift;

    if ($Key && $Order) {
	# Set order
	$PresentOrder->{$Key} = $Order;
	return $Order;
    }

    # Return order
    return $PresentOrder->{$Key} || $Descriptions->{$Key}->{"Order"};
}

#
# Set/Fetch description
#
sub Desc {
    my $self = shift;
    my $Key = shift;
    my $Desc = shift;

    if ($Key && $Desc) {
	# Set description
	$Descriptions->{$Key}->{'Desc'} = $Desc;
	return $Desc;
    }

    if ($Descriptions->{$Key}->{'NoShow'}) {
	return;
    }

    # Return description
    return $Descriptions->{$Key}->{'Desc'};
}

#
# Decode NetIf info from a DevInfo entry
#
sub DecodeDevInfo {
    my $self = shift;
    my @Fields = @_;

    $self->Type($Fields[3]);
    $self->HostAddr($Fields[4]);
    $self->HostName($Fields[5]);
    $self->NetAddr($Fields[6]);
    $self->NetName($Fields[7]);
    $self->MACaddr($Fields[8]);
    $self->MACname($Fields[9]);
    $self->FacMACaddr($Fields[10]);
    $self->FacMACname($Fields[11]);
    $self->NetMask($Fields[12]);
    $self->BroadAddr($Fields[13]);

    return $self;
}

#
# Decode an array of "netif" fields
#
sub Decode {
    my $self = shift;
    my @Fields = @_;

    if (Eq($Fields[1], 'name')) {
	$self->Name($Fields[2]);
    } elsif (Eq($Fields[1], 'prop')) {
	$self->Name($Fields[2]);

	my $key = $Fields[3];
	my $desc = $Fields[4];
	my $v = $Fields[5];
	my $k = $key;
	my $altkey;

	if (Eq($k, 'addrtype')) {
	    $self->Type($v);
	    $altkey = 'type';
	} elsif (Eq($k, 'iftype')) {
	    $self->IfType($v);
	} elsif (Eq($k, 'hostaddr')) {
	    $self->HostAddr($v);
	} elsif (Eq($k, 'hostname')) {
	    $self->HostName($v);
	} elsif (Eq($k, 'macaddr')) {
	    $self->MACaddr($v);
	} elsif (Eq($k, 'macname')) {
	    $self->MACname($v);
	} elsif (Eq($k, 'facmacaddr')) {
	    $self->FacMACaddr($v);
	} elsif (Eq($k, 'facmacname')) {
	    $self->FacMACname($v);
	} elsif (Eq($k, 'netaddr')) {
	    $self->NetAddr($v);
	} elsif (Eq($k, 'netname')) {
	    $self->NetName($v);
	} elsif (Eq($k, 'netmask')) {
	    $self->NetMask($v);
	} elsif (Eq($k, 'broadcast')) {
	    $self->BroadAddr($v);
	} elsif (Eq($k, 'mtu')) {
	    $self->MtuSize($v);
	    $altkey = 'mtusize';
	} elsif (Eq($k, 'speed')) {
	    $self->Speed($v);
	} elsif (Eq($k, 'duplex')) {
	    $self->Duplex($v);
	} elsif (Eq($k, 'mediatype')) {
	    $self->MediaType($v);
	} elsif (Eq($k, 'mediasubtype')) {
	    $self->MediaSubType($v);
	} elsif (Eq($k, 'flag')) {
	    $key = $Fields[4];
	    my @list = $self->Flags;
	    # Create a hash compatable with GetDescProps()
	    push(@list, {Key=>$key, Tag=>$key, Value=>$Fields[5]});
	    $self->Flags(@list);
	}

	# Set presentation order
	$self->ORDER($key, $PresentOrderTrack++);
	if ($altkey) {
	    # Add the alternate key
	    $self->ORDER($altkey, $PresentOrderTrack - 1);
	}

	# Set description value for this key
	$self->Desc($key, $desc);
	if ($altkey) {
	    # Add the alternate key
	    $self->Desc($altkey, $desc);
	}
    }

    return $self;
}

1;
