#
# $Id: mcNetwork.pm,v 1.6 2003/04/21 21:02:34 mcooper Exp $
#
# SysInfo Network class object
#

package mcNetwork;
require 5.003;

use mcMisc;
use mcDebug;
use mcNetworkInfo;

my $MyClassName = 'Network';

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"Entries"} = undef;
    $self->{"Members"} = undef;

    bless ($self, $class);

    return $self;
}

#
# Populate Members with the object Name
#
sub MembersPopulate {
    my $self = shift;
    my $Members = $self->{"Members"};
    my $SpecialMap = {};		# Nothing for now
    my $ni = mcNetworkInfo->New;	# This is were the member names are

    foreach my $mname (keys %$Members) {
	my $Found;
	# Check the special map first
	if (my $name = $SpecialMap->{lc($mname)}) {
	    $Found = $name;
	} else {
	    # Look through all members
	    foreach my $name (keys %$ni) {
		if (Eq($mname, $name)) {
		    $Found = $name;
		    last;
		}
	    }
	}
	if ($Found) {
	    $self->{"Members"}->{$mname}->Name($Found);
	} else {
	    Debug("mcGeneral->Members: Cannot identify member <%s>",
		  $mname);
	}
    }
}

#
# Set/Get mcMember info for our data members
#
sub Members {
    my $self = shift;
    my $Args = shift;
    my $HashValue = shift;

    if (!$HashValue) {
	# If we're not set yet, retrieve and set the data
	if (!$self->{"Members"}) {
	    my $sysinfo = mcSysInfo->New;
	    $sysinfo->Class($MyClassName);
	    if (!$sysinfo->RetrieveMembers($Args)) {
		# The retrieve failed
		return;
	    }
	    my $Members = $sysinfo->{$MyClassName}->Members;
	    # Copy the member info from $sysinfo to $self
	    $self->{"Members"} = $Members;
	    # Populate the Name member of each Members
	    $self->MembersPopulate;
	}
	return $self->{"Members"};
    }

    $self->{"Members"} = $HashValue;

    return;
}

sub Entries {
    my $self = shift;
    my $New;

    for my $Arg (@_) {
	if (ref($Arg) eq 'HASH') {
	    $New = $Arg;
	} else {
	    # Lookup and return result
	    return $self->{"Entries"}->{$Arg};
	}
    }

    if (!@_) {
	return $self->{"Entries"};
    }

    my $h = $self->{"Entries"};
    my @NewKey = keys %$New;
    my $NewVal = $New->{$NewKey[0]};
    $h->{$NewKey[0]} = $NewVal; # Overwrite old value
    $self->{"Entries"} = $h;

    return;
}

#
# Decode an array of network fields
#
sub Decode {
    my $self = shift;
    my @Fields = @_;

    if (!Eq($Fields[0], "network")) {
	return;
    }

    my $Name = $MyClassName;
    my $NetworkInfo = mcNetworkInfo->New;
    # Lookup the NetworkInfo entry (if any)
    my $li = $self->Entries($Name);
    if (!$li) {
	# Need a new object
	$li = $NetworkInfo;
    }
    # Decode and Set
    $li->Decode(@Fields);
    # Add it back
    $self->Entries({$Name => $li});

    return $self;
}

1; # Make require/use happy
