#
# $Id: mcNetworkInfo.pm,v 1.3 2003/04/29 20:56:04 mcooper Exp $
#
# SysInfo Network Info class object
#

package mcNetworkInfo;
require 5.003;

use mcMisc;
use mcDebug;

my $PresentOrderTrack = 1;
my $Summaries;
# List of non Data returning object members
my @NonDataMembers = ('New', 'Desc', 'ORDER', 'MemberNames', 'Entries');

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"DefGatewayIP"} = undef;
    $self->{"DefGatewayName"} = undef;
    $self->{"DnsDomain"} = undef;
    $self->{"DnsServerIPs"} = [];
    $self->{"DnsServerNames"} = [];
    $self->{"NisDomain"} = undef;
    $self->{"NisServerMethod"} = undef;
    $self->{"NisServerIPs"} = [];
    $self->{"NisServerNames"} = [];
    $self->{"NisServerCurrent"} = undef;
    $self->{"NisMaster"} = undef;

    $self->{"Desc"} = undef;
    $self->{"ORDER"} = undef;
    $self->{"MemberNames"} = undef;

    bless ($self, $class);

    return $self;
}

sub DefGatewayIP {
    my $self = shift;

    if (!@_) {
	return $self->{"DefGatewayIP"};
    }

    $self->{"DefGatewayIP"} = shift;

    return;
}

sub DefGatewayName {
    my $self = shift;

    if (!@_) {
	return $self->{"DefGatewayName"};
    }

    $self->{"DefGatewayName"} = shift;

    return;
}

sub DnsDomain {
    my $self = shift;

    if (!@_) {
	return $self->{"DnsDomain"};
    }

    $self->{"DnsDomain"} = shift;

    return;
}

sub DnsServerIPs {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"DnsServerIPs"} };
    }

    @{ $self->{"DnsServerIPs"} } = @_;

    return;
}

sub DnsServerNames {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"DnsServerNames"} };
    }

    @{ $self->{"DnsServerNames"} } = @_;

    return;
}

sub NisDomain {
    my $self = shift;

    if (!@_) {
	return $self->{"NisDomain"};
    }

    $self->{"NisDomain"} = shift;

    return;
}

sub NisServerMethod {
    my $self = shift;

    if (!@_) {
	return $self->{"NisServerMethod"};
    }

    $self->{"NisServerMethod"} = shift;

    return;
}

sub NisServerIPs {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"NisServerIPs"} };
    }

    @{ $self->{"NisServerIPs"} } = @_;

    return;
}

sub NisServerNames {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"NisServerNames"} };
    }

    @{ $self->{"NisServerNames"} } = @_;

    return;
}

sub NisServerCurrent {
    my $self = shift;

    if (!@_) {
	return $self->{"NisServerCurrent"};
    }

    $self->{"NisServerCurrent"} = shift;

    return;
}

sub NisMaster {
    my $self = shift;

    if (!@_) {
	return $self->{"NisMaster"};
    }

    $self->{"NisMaster"} = shift;

    return;
}

sub Entries {
    my $self = shift;
    my $New;

    for my $Arg (@_) {
	if (ref($Arg) eq 'HASH') {
	    $New = $Arg;
	} else {
	    # Lookup and return result
	    return $self->{"Entries"}->{$Arg};
	}
    }

    if (!@_) {
	return $self->{"Entries"};
    }

    my $h = $self->{"Entries"};
    my @NewKey = keys %$New;
    my $NewVal = $New->{$NewKey[0]};
    $h->{$NewKey[0]} = $NewVal; # Overwrite old value
    $self->{"Entries"} = $h;

    return;
}

#
# Set/Fetch description of $Key
# If $Desc is given, then sets the description for $Key to $Desc.
# Always returns the description found.
#
sub Desc {
    my $self = shift;
    my $Key = shift;
    my $Desc = shift;

    #
    # If there is no entry as requested, try a lower case $Key
    #
    if (!exists($Descriptions->{$Key})) {
	$Key = lc($Key);
    }

##    Debug("mcNetwork->Desc: Key=<%s> Desc=<%s> Found=<%s>",
##	  $Key, $Desc, $Descriptions->{$Key}->{'Desc'});

##    if (!$Key && !$Desc && !exists($Descriptions->{$Key})) {
##	Debug("mcNetwork->Desc: Cannot locate description for <%s>", $Key);
##    }

    if ($Key && $Desc) {
	# Set description
	$Descriptions->{$Key}->{'Desc'} = $Desc;
    }

    if (exists($Descriptions->{$Key}->{'NoShow'}) &&
	$Descriptions->{$Key}->{'NoShow'}) {
	return;
    }

    # Return description
    return $Descriptions->{$Key}->{'Desc'};
}

#
# Order item is presented in
#
sub ORDER {
    my $self = shift;
    my $Key = shift;
    my $Order = shift;

    if ($Key && $Order) {
	# Set order
	$PresentOrder->{$Key} = $Order;
	return $Order;
    }

    # Return order
    return $PresentOrder->{$Key};
}

#
# Decode an array of network fields
#
sub Decode {
    my $self = shift;
    my @Fields = @_;

    my $k = $Fields[2];
    my $d = $Fields[3];
    my $v = $Fields[4];
    my $key = $k;
    my $altkey;

    if (Eq($k, 'defgwip')) {
	$self->DefGatewayIP($v);
	$altkey = 'defgatewayip';
    } elsif (Eq($k, 'defgwname')) {
	$self->DefGatewayName($v);
	$altkey = 'defgatewayname';
    } elsif (Eq($k, 'dnsdomain')) {
	$self->DnsDomain($v);
    } elsif (Eq($k, 'dnsserverips')) {
	$self->DnsServerIPs(split(',', $v));
    } elsif (Eq($k, 'dnsservernames')) {
	$self->DnsServerNames(split(',', $v));
    } elsif (Eq($k, 'nisdomain')) {
	$self->NisDomain($v);
    } elsif (Eq($k, 'nisservermethod')) {
	$self->NisServerMethod($v);
    } elsif (Eq($k, 'nisserverips')) {
	$self->NisServerIPs(split(',', $v));
    } elsif (Eq($k, 'nisservernames')) {
	$self->NisServerNames(split(',', $v));
    } elsif (Eq($k, 'nisserver')) {
	$self->NisServerCurrent($v);
	$altkey = 'nisservercurrent';
    } elsif (Eq($k, 'nismaster')) {
	$self->NisMaster($v);
    } else {
	Debug("NetworkInfo->Decode: Key <%s> is unknown.", $k);
	return;
    }

    # Set presentation order
    $self->ORDER($key, $PresentOrderTrack++);
    if ($altkey) {
	# Add the alternate key
	$self->ORDER($altkey, $PresentOrderTrack - 1);
    }

    # Set description value for this key
    $self->Desc($key, $d);
    if ($altkey) {
	# Add the alternate key
	$self->Desc($altkey, $d);
    }

    return $self;
}

#
# Return array of our member names.
# We take the entire list and then filter out what are not
# actual data members.
#
sub MemberNames {
    my $self = shift;
    my @List;

    foreach my $key (keys %$self) {
	if (ArrayFind($key, @NonDataMembers)) {
	    next;
	}
	push(@List, $key);
    }

    return @List;
}

1; # Make require/use happy
