#
# $Id: mcPartition.pm,v 1.4 2002/07/10 15:54:56 mcooper Exp $
#
# Partition object
#

package mcPartition;
require 5.003;

use mcDebug;
use mcMisc;
use mcPartInfo;
use mcGetSize;

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"Entries"} = undef;
    $self->{"POSTPROCESS"} = undef;

    bless ($self, $class);

    return $self;
}

sub Entries {
    my $self = shift;
    my $New;

    for my $Arg (@_) {
	if (ref($Arg) eq 'HASH') {
	    $New = $Arg;
	} else {
	    # Lookup and return result
	    return $self->{"Entries"}->{$Arg};
	}
    }

    if (!@_) {
	return $self->{"Entries"};
    }

    my $h = $self->{"Entries"};
    my @NewKey = keys %$New;
    my $NewVal = $New->{$NewKey[0]};
    $h->{$NewKey[0]} = $NewVal; # Overwrite old value
    $self->{"Entries"} = $h;

    return;
}

#
# Post Process in any info we need to post decode.
#
sub POSTPROCESS {
    my $self = shift;

    Debug("mcPartition->PostProcess: START");

    my @Names = keys %{ $self->{"Entries"} };
    foreach my $name (@Names) {
	#Debug("mcPartition->PostProcess: Name=<%s>", $name);
	my $PartInfo = $self->Entries->{$name};

	# Set summary
	my @s;
	push(@s, $name);
	push(@s, 'is a');
	if ($PartInfo->Size) {
	    push(@s, GetSizeStr($PartInfo->Size, $MBYTES));
	}
	if ($PartInfo->TypeDesc) {
	    push(@s, $PartInfo->TypeDesc);
	} elsif ($PartInfo->Type) {
	    push(@s, $PartInfo->Type);
	} elsif ($PartInfo->Usage) {
	    push(@s, $PartInfo->Usage);
	}

	push(@s, 'partition');

	if ($PartInfo->MntName) {
	    push(@s, 'in use as');
	    push(@s, $PartInfo->MntName);
	}
	$PartInfo->SUMMARY(join(' ', @s));
	Debug("\tSUMMARY: %s", $PartInfo->SUMMARY);
    }

    Debug("mcPartition->PostProcess: END");
}

#
# Decode an array of general fields
#
sub Decode {
    my $self = shift;
    my @Fields = @_;

    if (!Eq($Fields[0], "partition")) {
	return;
    }

    my $Name = $Fields[4];
    my $PartInfo = mcPartInfo->New;
    # Lookup the PartInfo entry (if any)
    my $pi = $self->Entries($Name);
    if (!$pi) {
	# Need a new object
	$pi = $PartInfo;
    }
    # Decode and Set
    $pi->Decode(@Fields);
    # Add it back
    $self->Entries({$Name => $pi});

    return $self;
}

1; # Make require/use happy
