#
# $Id: mcPrinterInfo.pm,v 1.9 2003/05/07 18:08:25 mcooper Exp $
#
# PrinterInfo object
#

package mcPrinterInfo;
require 5.003;

use mcMisc;
use mcDebug;

my $Descriptions =
	{
	 'Name'			=> {Order=>10, Desc=>'Queue Name' },
	 'Aliases'		=> {Order=>11, Desc=>'Queue Aliases' },
	 'Descript'		=> {Order=>12, Desc=>'Description'},
	 'Vendor'		=> {Order=>13, Desc=>'Vendor'},
	 'Model'		=> {Order=>14, Desc=>'Model'},
	 'Proto'		=> {Order=>15, Desc=>'Protocol'},
	 'Device'		=> {Order=>20, Desc=>'Device'},
	 'Server'		=> {Order=>21, Desc=>'Server'},
	 'RemoteQueue'		=> {Order=>22, Desc=>'Remote Queue'},
	 'Langs'		=> {Order=>23, Desc=>'Languages'},
	 'SpoolDir'		=> {Order=>24, Desc=>'Spool Directory'},
	 'MaxJobSize'		=> {Order=>25, Desc=>'Max Job Size'},
	 'DescList'		=> {NoShow=>1},
	};
#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;
    $self->{"Name"} = undef;
    $self->{"Aliases"} = [];
    $self->{"Descript"} = undef;
    $self->{"Vendor"} = undef;
    $self->{"Model"} = undef;
    $self->{"Proto"} = undef;
    $self->{"Device"} = undef;
    $self->{"Server"} = undef;
    $self->{"RemoteQueue"} = undef;
    $self->{"Langs"} = [];
    $self->{"SpoolDir"} = undef;
    $self->{"MaxJobSize"} = undef;
    $self->{"DescList"} = [];

    $self->{"Desc"} = undef;
    $self->{"ORDER"} = undef;
    $self->{"SUMMARY"} = undef;

    bless ($self, $class);

    return $self;
}

sub Name {
    my $self = shift;

    if (!@_) {
	return $self->{"Name"};
    }

    $self->{"Name"} = shift;

    return;
}

sub Descript {
    my $self = shift;

    if (!@_) {
	return $self->{"Descript"};
    }

    $self->{"Descript"} = shift;

    return;
}

sub Vendor {
    my $self = shift;

    if (!@_) {
	return $self->{"Vendor"};
    }

    $self->{"Vendor"} = shift;

    return;
}

sub Model {
    my $self = shift;

    if (!@_) {
	return $self->{"Model"};
    }

    $self->{"Model"} = shift;

    return;
}

sub Proto {
    my $self = shift;

    if (!@_) {
	return $self->{"Proto"};
    }

    $self->{"Proto"} = shift;

    return;
}

sub Device {
    my $self = shift;

    if (!@_) {
	return $self->{"Device"};
    }

    $self->{"Device"} = shift;

    return;
}

sub Server {
    my $self = shift;

    if (!@_) {
	return $self->{"Server"};
    }

    $self->{"Server"} = shift;

    return;
}

sub RemoteQueue {
    my $self = shift;

    if (!@_) {
	return $self->{"RemoteQueue"};
    }

    $self->{"RemoteQueue"} = shift;

    return;
}

sub SpoolDir {
    my $self = shift;

    if (!@_) {
	return $self->{"SpoolDir"};
    }

    $self->{"SpoolDir"} = shift;

    return;
}

sub MaxJobSize {
    my $self = shift;

    if (!@_) {
	return $self->{"MaxJobSize"};
    }

    $self->{"MaxJobSize"} = shift;

    return;
}

sub Aliases {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"Aliases"} };
    }

    @{ $self->{"Aliases"} } = @_;

    return;
}

sub Langs {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"Langs"} };
    }

    @{ $self->{"Langs"} } = @_;

    return;
}

sub DescList {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"DescList"} };
    }

    @{ $self->{"DescList"} } = @_;

    return;
}

#
# Fetch description
#
sub Desc {
    my $self = shift;
    my $Key = shift;

    if ($Descriptions->{$Key}->{'NoShow'}) {
	return;
    }

    return $Descriptions->{$Key}->{'Desc'};
}

#
# Make a Summary
#
sub MakeSummary {
    my $self = shift;
    my $Args = shift;
    my $Encode = $Args->{Encode};
    my $Printer = $self;

    Debug("mcPrinterInfo->MakeSummary: START");

    # Set summary
    my @s;

    if (Eq($Encode, 'html')) {
	push(@s, '<b>' . $self->Name . '</b>');
    } else {
	if (my $m = $self->{MaxNameLen}) {
	    push(@s, sprintf("%-*s", $m, $Printer->Name));
	} else {
	    push(@s, $Printer->Name);
	}
    }

    if ($Printer->Vendor || $Printer->Model) {
	push(@s, 'is a');
	
	if ($Printer->Vendor) {
	    push(@s, $Printer->Vendor);
	}
	if ($Printer->Model) {
	    push(@s, $Printer->Model);
	}
    }

    if ($Printer->Desc) {
	push(@s, '"' . $Printer->Desc . '"');
    }
    if ($Printer->Proto) {
	push(@s, $Printer->Proto);
    }
    if ($Printer->Langs) {
	my @a = $Printer->Langs;
	push(@s, $a[0]);
	}
    if ($Printer->Device) {
	push(@s, 'on');
	push(@s, $Printer->Device);
    }
    if ($Printer->Server) {
	push(@s, 'on');
	push(@s, $Printer->Server);
    }

    my $sum = join(' ', @s);
    Debug("\tSUMMARY: %s", $sum);

    return $sum;
}

#
# Fetch/set summary of this item
#
sub SUMMARY {
    my $self = shift;
    my $Desc;
    my $Args;

    foreach my $a (@_) {
	if (ref($a) eq 'HASH') {
	    $Args = $a;
	} elsif (!ref($a)) {
	    $Desc = $a;
	}
    }

    if ($Desc) {
	# Set description
	$Summaries->{$self->Name} = $Desc;
	return $Desc;
    }

    # Return summary
    return $Summaries->{$self->Name} || $self->MakeSummary($Args);
}

#
# Order item is presented in
#
sub ORDER {
    my $self = shift;
    my $Key = shift;
    my $Order = shift;

    if ($Key && $Order) {
	# Set order
	$PresentOrder->{$Key} = $Order;
	return $Order;
    }

    # Return order
    return $PresentOrder->{$Key} || $Descriptions->{$Key}->{"Order"};
}

#
# Decode an array of general fields
#
sub Decode {
    my $self = shift;
    my @Fields = @_;

    if (Eq($Fields[1], 'name')) {
	my $key = $Fields[2];

	$self->Name($Fields[2]);
	$self->Aliases(split(',',$Fields[3]));
	$self->Descript($Fields[4]);
	$self->Vendor($Fields[5]);
	$self->Model($Fields[6]);
	$self->Proto($Fields[7]);
	$self->Device($Fields[8]);
	$self->Server($Fields[9]);
	$self->RemoteQueue($Fields[10]);
	$self->Langs(split(',', $Fields[11]));
	$self->SpoolDir($Fields[12]);
	$self->MaxJobSize($Fields[13]);	

	# Set presentation order
	$self->ORDER($key, $PresentOrderTrack++);
    } elsif (Eq($Fields[1], 'desc')) {
	$self->Name($Fields[2]);
	push(@{ $self->DescList }, $Fields[3] => $Fields[4]);
    }

    return $self;
}

1; # Make require/use happy
