#
# $Id: mcSoftFile.pm,v 1.4 2002/07/15 00:32:42 mcooper Exp $
#
# SoftFile object
#

package mcSoftFile;
require 5.003;

use mcMisc;
use mcGetSize;

my $PresentOrderTrack = 1;
my $Summaries;

my $Descriptions =
	{
	 'Path'			=> {Order=>11, Desc=>'Path'},
	 'Type'			=> {Order=>10, Desc=>'File Type'},
	 'LinkTo'		=> {Order=>12, Desc=>'Links To'},
	 'FileSize'		=> {Order=>13, Desc=>'File Size'},
	 'MD5'			=> {Order=>16, Desc=>'MD5 Checksum'},
	 'Checksum'		=> {Order=>17, Desc=>'Checksum'},
	 'PkgNames'		=> {Order=>18, Desc=>'Packages'},
	};

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"Path"} = undef;
    $self->{"Type"} = undef;
    $self->{"LinkTo"} = undef;
    $self->{"FileSize"} = undef;
    $self->{"MD5"} = undef;
    $self->{"Checksum"} = undef;
    $self->{"PkgNames"} = [];

    $self->{"Desc"} = undef;
    $self->{"SUMMARY"} = undef;

    bless ($self, $class);

    return $self;
}

sub Type {
    my $self = shift;

    if (!@_) {
	return $self->{"Type"};
    }

    $self->{"Type"} = shift;

    return;
}

sub Path {
    my $self = shift;

    if (!@_) {
	return $self->{"Path"};
    }

    $self->{"Path"} = shift;

    return;
}

sub LinkTo {
    my $self = shift;

    if (!@_) {
	return $self->{"LinkTo"};
    }

    $self->{"LinkTo"} = shift;

    return;
}

sub FileSize {
    my $self = shift;

    if (!@_) {
	return $self->{"FileSize"};
    }

    $self->{"FileSize"} = shift;

    return;
}

sub MD5 {
    my $self = shift;

    if (!@_) {
	return $self->{"MD5"};
    }

    $self->{"MD5"} = shift;

    return;
}

sub Checksum {
    my $self = shift;

    if (!@_) {
	return $self->{"Checksum"};
    }

    $self->{"Checksum"} = shift;

    return;
}

sub PkgNames {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"PkgNames"} };
    }

    @{ $self->{"PkgNames"} } = @_;

    return;
}

#
# Fetch description
#
sub Desc {
    my $self = shift;
    my $Key = shift;

    if ($Descriptions->{$Key}->{'NoShow'}) {
	return;
    }

    return $Descriptions->{$Key}->{'Desc'};
}

#
# Make softfile summary
#
sub MakeFileSummary {
    my $self = shift;

    if (!$self || !$self->Path) {
	return;
    }

    my @s;
    push(@s, $self->Path);
    if ($self->LinkTo) {
	push(@s, '->');
	push(@s, $self->LinkTo);
    }
    if ($self->FileSize) {
	push(@s, sprintf("SIZE: %s", GetSizeStr($self->FileSize, 1)));
    }
    if ($self->MD5) {
	push(@s, "MD5: " . $self->MD5);
    }
    if ($self->Checksum) {
	push(@s, "CHKSUM: " . $self->Checksum);
    }

    return join(' ', @s);
}

#
# Fetch/set summary of this item
#
sub SUMMARY {
    my $self = shift;
    my $Desc = shift;

    if ($Desc) {
	# Set description
	$Summaries->{$self->Path} = $Desc;
	return $Desc;
    }

    # Return summary if it exists or generate it on the fly
    return $Summaries->{$self->Path} || MakeFileSummary($self);
}

#
# Decode an array of general fields
#
sub Decode {
    my $self = shift;
    my @Fields = @_;

    if (Eq($Fields[1], 'file')) {
	# Field 2 is the package name
	$self->Type($Fields[3]);
	$self->Path($Fields[4]);
	$self->LinkTo($Fields[5]);
	$self->FileSize($Fields[6]);
	$self->MD5($Fields[7]);
	$self->Checksum($Fields[8]);
	$self->PkgNames(split(',', $Fields[9]));
    }

    return $self;
}

1; # Make require/use happy
