#
# $Id: mcSoftInfo.pm,v 1.9 2003/04/25 01:16:35 mcooper Exp $
#
# SoftInfo object
#

package mcSoftInfo;
require 5.003;

use mcMisc;

$SoftMaxName = 0;
$SoftMaxVer = 0;

my $PresentOrderTrack = 1;
my $Summaries;

my $Descriptions =
	{
	 'EntryType'		=> {Order=>10, Desc=>'Entry Type'},
	 'Name'			=> {Order=>11, Desc=>'Name'},
	 'Version'		=> {Order=>12, Desc=>'Version'},
	 'Revision'		=> {Order=>13, Desc=>'Revision'},
	 'URL'			=> {Order=>16, Desc=>'URL'},
	 'License'		=> {Order=>17, Desc=>'License'},
	 'Copyright'		=> {Order=>18, Desc=>'Copyright'},
	 'Category'		=> {Order=>19, Desc=>'Category'},
	 'SubCategory'		=> {Order=>30, Desc=>'Sub-Category'},
	 'OSname'		=> {Order=>31, Desc=>'Requires OS Name'},
	 'OSversion'		=> {Order=>32, Desc=>'Requires OS Version'},
	 'Arch'			=> {Order=>33, Desc=>'Requires Architecture'},
	 'ISArch'		=> {Order=>34, Desc=>'Requires Inst Set Arch'},
	 'InstDate'		=> {Order=>35, Desc=>'Installation Date'},
	 'BuildDate'		=> {Order=>36, Desc=>'Build Date'},
	 'ProdStamp'		=> {Order=>37, Desc=>'Production Stamp'},
	 'BaseDir'		=> {Order=>38, Desc=>'Base Directory'},
	 'DiskUsage'		=> {Order=>39, Desc=>'Disk Usage'},
	 'FileList'		=> {Order=>60, Desc=>'File List'},
	 'PkgDeps'		=> {Order=>61, Desc=>'Package Dependencies'},
	 'VendorName'		=> {Order=>62, Desc=>'Vendor Name'},
	 'VendorEmail'		=> {Order=>63, Desc=>'Vendor Email'},
	 'VendorPhone'		=> {Order=>64, Desc=>'Vendor Phone #'},
	 'VendorStock'		=> {Order=>65, Desc=>'Vendor Stock #'},
	 'Description'		=> {Order=>100, Desc=>'Description'},
	 'DescVerbose'		=> {Order=>101, Desc=>'Description (Verbose)'},
	 'DescList'		=> {NoShow=>1},
	 'Master'		=> {NoShow=>1},
	 'MasterVer'		=> {NoShow=>1},
	 'MasterRev'		=> {NoShow=>1},
	 'Slaves'		=> {NoShow=>1},
	 'Next'			=> {NoShow=>1},
	};

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"Name"} = undef;
    $self->{"EntryType"} = undef;
    $self->{"Version"} = undef;
    $self->{"Revision"} = undef;
    $self->{"Description"} = undef;
    $self->{"DescVerbose"} = undef;
    $self->{"URL"} = undef;
    $self->{"License"} = undef;
    $self->{"Copyright"} = undef;
    $self->{"Category"} = undef;
    $self->{"SubCategory"} = undef;
    $self->{"OSname"} = undef;
    $self->{"OSversion"} = undef;
    $self->{"Arch"} = undef;
    $self->{"ISArch"} = undef;
    $self->{"InstDate"} = undef;
    $self->{"BuildDate"} = undef;
    $self->{"ProdStamp"} = undef;
    $self->{"BaseDir"} = undef;
    $self->{"PkgDeps"} = [];
    $self->{"DiskUsage"} = undef;
    $self->{"FileList"} = [];
    $self->{"DescList"} = [];
    $self->{"VendorName"} = undef;
    $self->{"VendorEmail"} = undef;
    $self->{"VendorPhone"} = undef;
    $self->{"VendorStock"} = undef;
    $self->{"Master"} = undef;
    $self->{"MasterNode"} = undef;
    $self->{"MasterVer"} = undef;
    $self->{"MasterRev"} = undef;
    $self->{"Slaves"} = [];
    $self->{"SlaveNodes"} = [];
    $self->{"Next"} = undef;
    $self->{"NextNode"} = undef;

    $self->{"Desc"} = undef;
    $self->{"ORDER"} = undef;
    $self->{"SUMMARY"} = undef;

    bless ($self, $class);

    return $self;
}

sub Name {
    my $self = shift;

    if (!@_) {
	return $self->{"Name"};
    }

    my $n = shift;
    $n =~ s/&/+/g; # To avoid $TreePathSep problems
    $n =~ s/\//_/g; # Avoid path problems
    $n =~ s/\+//g; # Avoid regexp problems
    $self->{"Name"} = $n;

    return;
}

sub EntryType {
    my $self = shift;

    if (!@_) {
	return $self->{"EntryType"};
    }

    $self->{"EntryType"} = shift;

    return;
}

sub Version {
    my $self = shift;

    if (!@_) {
	return $self->{"Version"};
    }

    $self->{"Version"} = shift;

    return;
}

sub Revision {
    my $self = shift;

    if (!@_) {
	return $self->{"Revision"};
    }

    $self->{"Revision"} = shift;

    return;
}

sub Description {
    my $self = shift;

    if (!@_) {
	return $self->{"Description"};
    }

    $self->{"Description"} = shift;

    return;
}

sub DescVerbose {
    my $self = shift;

    if (!@_) {
	return $self->{"DescVerbose"};
    }

    $self->{"DescVerbose"} = shift;

    return;
}

sub URL {
    my $self = shift;

    if (!@_) {
	return $self->{"URL"};
    }

    $self->{"URL"} = shift;

    return;
}

sub License {
    my $self = shift;

    if (!@_) {
	return $self->{"License"};
    }

    $self->{"License"} = shift;

    return;
}

sub Copyright {
    my $self = shift;

    if (!@_) {
	return $self->{"Copyright"};
    }

    $self->{"Copyright"} = shift;

    return;
}

sub Category {
    my $self = shift;

    if (!@_) {
	return $self->{"Category"};
    }

    $self->{"Category"} = shift;

    return;
}

sub SubCategory {
    my $self = shift;

    if (!@_) {
	return $self->{"SubCategory"};
    }

    $self->{"SubCategory"} = shift;

    return;
}

sub OSname {
    my $self = shift;

    if (!@_) {
	return $self->{"OSname"};
    }

    $self->{"OSname"} = shift;

    return;
}

sub OSversion {
    my $self = shift;

    if (!@_) {
	return $self->{"OSversion"};
    }

    $self->{"OSversion"} = shift;

    return;
}

sub Arch {
    my $self = shift;

    if (!@_) {
	return $self->{"Arch"};
    }

    $self->{"Arch"} = shift;

    return;
}

sub ISArch {
    my $self = shift;

    if (!@_) {
	return $self->{"ISArch"};
    }

    $self->{"ISArch"} = shift;

    return;
}

sub InstDate {
    my $self = shift;

    if (!@_) {
	return $self->{"InstDate"};
    }

    $self->{"InstDate"} = shift;

    return;
}

sub BuildDate {
    my $self = shift;

    if (!@_) {
	return $self->{"BuildDate"};
    }

    $self->{"BuildDate"} = shift;

    return;
}

sub ProdStamp {
    my $self = shift;

    if (!@_) {
	return $self->{"ProdStamp"};
    }

    $self->{"ProdStamp"} = shift;

    return;
}

sub BaseDir {
    my $self = shift;

    if (!@_) {
	return $self->{"BaseDir"};
    }

    $self->{"BaseDir"} = shift;

    return;
}

sub DiskUsage {
    my $self = shift;

    if (!@_) {
	return $self->{"DiskUsage"};
    }

    $self->{"DiskUsage"} = shift;

    return;
}

sub FileList {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"FileList"} };
    }

    @{ $self->{"FileList"} } = @_;

    return;
}

sub DescList {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"DescList"} };
    }

    @{ $self->{"DescList"} } = @_;

    return;
}

sub VendorName {
    my $self = shift;

    if (!@_) {
	return $self->{"VendorName"};
    }

    $self->{"VendorName"} = shift;

    return;
}

sub VendorEmail {
    my $self = shift;

    if (!@_) {
	return $self->{"VendorEmail"};
    }

    $self->{"VendorEmail"} = shift;

    return;
}

sub VendorPhone {
    my $self = shift;

    if (!@_) {
	return $self->{"VendorPhone"};
    }

    $self->{"VendorPhone"} = shift;

    return;
}

sub VendorStock {
    my $self = shift;

    if (!@_) {
	return $self->{"VendorStock"};
    }

    $self->{"VendorStock"} = shift;

    return;
}

sub Master {
    my $self = shift;

    if (!@_) {
	return $self->{"Master"};
    }

    $self->{"Master"} = shift;

    return;
}

sub MasterNode {
    my $self = shift;

    if (!@_) {
	return $self->{"MasterNode"};
    }

    $self->{"MasterNode"} = shift;

    return;
}

sub MasterVer {
    my $self = shift;

    if (!@_) {
	return $self->{"MasterVer"};
    }

    $self->{"MasterVer"} = shift;

    return;
}

sub MasterRev {
    my $self = shift;

    if (!@_) {
	return $self->{"MasterRev"};
    }

    $self->{"MasterRev"} = shift;

    return;
}

sub Next {
    my $self = shift;

    if (!@_) {
	return $self->{"Next"};
    }

    $self->{"Next"} = shift;

    return;
}

sub NextNode {
    my $self = shift;

    if (!@_) {
	return $self->{"NextNode"};
    }

    $self->{"NextNode"} = shift;

    return;
}

sub Slaves {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"Slaves"} };
    }

    @{ $self->{"Slaves"} } = @_;

    return;
}

sub PkgDeps {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"PkgDeps"} };
    }

    @{ $self->{"PkgDeps"} } = @_;

    return;
}

sub SlaveNodes {
    my $self = shift;

    if (!@_) {
	return @{ $self->{"SlaveNodes"} };
    }

    @{ $self->{"SlaveNodes"} } = @_;

    return;
}

#
# Fetch description
#
sub Desc {
    my $self = shift;
    my $Key = shift;

    if ($Descriptions->{$Key}->{'NoShow'}) {
	return;
    }

    return $Descriptions->{$Key}->{'Desc'};
}

#
# Make a Summary line
#
sub MakeSummary {
    my $SoftInfo = shift;
    my $Args = shift;
    my $Encode = $Args->{Encode};
    my $Name = $SoftInfo->Name;

    # Set summary
    my @s;

    if (Eq($Encode, 'html')) {
	if (Eq($SoftInfo->EntryType, 'product')) {
	    push(@s, '<h2>' . $Name . '</h2>');
	} else {
	    push(@s, '<b>' . $Name . '</b>');
	}
	if ($SoftInfo->Version) {
	    push(@s, '<i>' . $SoftInfo->Version . '</i>');
	}
    } else {
	push(@s, sprintf("%-*s", $SoftMaxName, $Name));

	if ($SoftInfo->Version) {
	    push(@s, sprintf("%-*s", $SoftMaxVer, $SoftInfo->Version));
	}
    }

    if ($SoftInfo->Description) {
	push(@s, $SoftInfo->Description);
    } elsif ($SoftInfo->DescVerbose) {
	my $d = $SoftInfo->DescVerbose;
	$d =~ s/\n.*//;		# Strip everything after first newline
	push(@s, $d);
    }

    return join(' ', @s);
}

#
# Fetch/set summary of this item
#
sub SUMMARY {
    my $self = shift;
    my $Desc;
    my $Args;

    foreach my $a (@_) {
	if (ref($a) eq 'HASH') {
	    $Args = $a;
	} elsif (!ref($a)) {
	    $Desc = $a;
	}
    }

    if ($Desc) {
	# Set description
	$Summaries->{$self->Name} = $Desc;
	return $Desc;
    }

    # Return summary
    return $Summaries->{$self->Name} || $self->MakeSummary($Args);
}

#
# Order item is presented in
#
sub ORDER {
    my $self = shift;
    my $Key = shift;
    my $Order = shift;

    if ($Key && $Order) {
	# Set order
	$PresentOrder->{$Key} = $Order;
	return $Order;
    }

    # Return order
    return $PresentOrder->{$Key} || $Descriptions->{$Key}->{Order};
}

#
# Decode an array of general fields
#
sub Decode {
    my $self = shift;
    my @Fields = @_;

    my $key = $Fields[2];

    if (Eq($Fields[1], 'name')) {
	$self->Name($Fields[2]);
	$self->EntryType($Fields[3]);
	# Field 4 - Duplicate of field 2
	$self->Version($Fields[5]);
	$self->Revision($Fields[6]);
	$self->Description($Fields[7]);
	$self->URL($Fields[8]);
	$self->License($Fields[9]);
	$self->Category($Fields[10]);
	$self->SubCategory($Fields[11]);
	$self->OSname($Fields[12]);
	$self->OSversion($Fields[13]);
	$self->Arch($Fields[14]);
	$self->ISArch($Fields[15]);
	$self->InstDate($Fields[16]);
	$self->BuildDate($Fields[17]);
	$self->ProdStamp($Fields[18]);
	$self->BaseDir($Fields[19]);
	$self->DiskUsage($Fields[20]);
	$self->VendorName($Fields[21]);
	$self->VendorEmail($Fields[22]);
	$self->VendorPhone($Fields[23]);
	$self->VendorStock($Fields[24]);
	$self->Master($Fields[25]);
	$self->MasterVer($Fields[26]);
	$self->MasterRev($Fields[27]);
	$self->DescVerbose($Fields[28]);
	$self->Copyright($Fields[29]);
	$self->Next($Fields[30]);
	$self->Slaves(split(',', $Fields[31]));
	$self->PkgDeps(split(',', $Fields[32]));
    } elsif (Eq($Fields[1], 'desc')) {
	$self->Name($Fields[2]);
	push(@{ $self->DescList }, $Fields[3] => $Fields[4]);
    }

    # Set presentation order
    $self->ORDER($key, $PresentOrderTrack++);

    return $self;
}

1; # Make require/use happy
