#
# $Id: mcSoftware.pm,v 1.8 2003/03/10 01:24:09 mcooper Exp $
#
# Software object
#

package mcSoftware;
require 5.003;

use mcDebug;
use mcMisc;
use mcSoftInfo;
use mcSoftFile;
use mcGetSize;

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"Entries"} = undef;
    $self->{"Tree"} = undef;
    $self->{"POSTPROCESS"} = undef;

    bless ($self, $class);

    return $self;
}

#
# Entries is a "horozontal" list of all software devices
#
sub Entries {
    my $self = shift;
    my $New;

    for my $Arg (@_) {
	if (ref($Arg) eq 'HASH') {
	    $New = $Arg;
	} else {
	    # Lookup and return result
	    return $self->{"Entries"}->{$Arg};
	}
    }

    if (!@_) {
	return $self->{"Entries"};
    }

    my $h = $self->{"Entries"};
    my @NewKey = keys %$New;
    my $NewVal = $New->{$NewKey[0]};
    $h->{$NewKey[0]} = $NewVal; # Overwrite old value
    $self->{"Entries"} = $h;

    return;
}

#
# Tree is a hierarchical tree list of all software devices
#
sub Tree {
    my $self = shift;

    if (!@_) {
	return $self->{"Tree"};
    }

    $self->{"Tree"} = shift;

    return;
}

#
# Post Process in any info we need to post decode.
#
sub POSTPROCESS {
    my $self = shift;

    Debug("mcSoftware->POSTPROCESS: START");

    my $Entries = $self->{"Entries"};
    my @Names = sort keys %$Entries;
    my $Tree = mcSoftInfo->New;			# Create root of tree
    my $SoftMaxName = 0;
    my $SoftMaxVer = 0;
    #$Tree->EntryType('** ROOT OF TREE **');	# Debugging message

    foreach my $name (@Names) {
	Debug("mcSoftware->POSTPROCESS: Name=<%s>", $name);
	my $SoftInfo = $self->Entries->{$name};

	# Calculate max length of Name and Version
	if (Eq($SoftInfo->EntryType, 'pkg')) {
	    if (length($SoftInfo->Name) > $SoftMaxName) {
		$SoftMaxName = length($SoftInfo->Name);
	    }
	    if (length($SoftInfo->Version) > $SoftMaxVer) {
		$SoftMaxVer = length($SoftInfo->Version);
	    }
	}

	#
	# Update all SoftInfo Entries to have appropriate NextNode, SlaveNodes,
	# and MasterNode members set to represent hierarchical tree of
	# software.
	#
	Debug("mcSoftware->POSTPROCESS: Node=<%s> Next=<%s> Slaves=<%s> Master=<%s>",
	      $name, $SoftInfo->Next, join(',', $SoftInfo->Slaves),
	      $SoftInfo->Master);

	#
	# Add the next (sibling) node if known
	#
	if ($SoftInfo->Next) {
	    if (my $next = $Entries->{$SoftInfo->Next}) {
		$SoftInfo->NextNode($next);
	    } else {
		Debug("\tCannot find next node <%s> for node <%s>",
		      $SoftInfo->Next, $name);
	    }
	}

	#
	# Find the master for this node
	#
	my $Master = $Tree;	# Masterless nodes get the root
	if ($SoftInfo->Master) {
	    $Master = $Entries->{$SoftInfo->Master};
	    if (!$Master) {
		Debug("\tCannot find master <%s> for node <%s>",
		      $SoftInfo->Master, $name);
		# Default to root node
		$Master = $Tree;
	    }
	}
	my $mn = ($Master && $Master->Name) ? $Master->Name :
	    $Master->EntryType;
	Debug("\tMaster set to <%s>", $mn);
	$SoftInfo->MasterNode($Master);

	my @s = $Master->SlaveNodes;
	push(@s, $SoftInfo);
	$Master->SlaveNodes(@s);
    }

    $self->Tree($Tree);
    # Set Max lengths for SoftInfo->MakeSummary
    $mcSoftInfo::SoftMaxName = $SoftMaxName;
    $mcSoftInfo::SoftMaxVer = $SoftMaxVer;

    Debug("mcSoftware->POSTPROCESS: Tree Top Slaves:");
    foreach my $di ($Tree->SlaveNodes) {
	Debug("\tNode is <%s>", $di->Name);
    }

    Debug("mcSoftware->POSTPROCESS: END");
}

#
# Decode an array of general fields
#
sub Decode {
    my $self = shift;
    my @Fields = @_;

    if (!Eq($Fields[0], "software")) {
	return;
    }

    my $Name = $Fields[2];
    # Lookup the SoftInfo entry (if any)
    my $Use = $self->Entries($Name);
    if (!$Use) {
	# Need a new object
	$Use = mcSoftInfo->New;
    }

    if (Eq($Fields[1], 'name')) {
	# Decode and Set
	$Use->Decode(@Fields);
	# Add it back
	$self->Entries({$Name => $Use});
    } elsif (Eq($Fields[1], 'file')) {
	my $sf = mcSoftFile->New;
	if ($sf->Decode(@Fields)) {
	    if (Debug > 3) {
		# This is a very expensive function
		my @a = $Use->FileList;
		Debug("Adding <%s> to FileList of <%s> Existing=%d",
		      $sf->Path, $Use->Name, $#a);
	    }
	    push(@{ $Use->{"FileList"} }, $sf);
	}
    }

    return $self;
}

1; # Make require/use happy
