#
# $Id: mcStreamCache.pm,v 1.1 2003/04/19 21:41:20 mcooper Exp $
#
# Stream driver for the Cache
#

package mcStreamCache;
require 5.003;

use mcGlobals;
use mcMisc;
use mcDebug;
use mcStatus;
use mcFileCache;

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"Open"} = undef;
    $self->{"Close"} = undef;
    $self->{"ReadLine"} = undef;

    bless ($self, $class);

    return $self;
}

#
# Open a connection to an existing cache.  If the cache is valid we
# return $Status->Code = SUCCESS.  We return FAILURE for all other cases.
# Returns mcStatus
#
sub Open {
    my $self = shift;
    my $SysInfo = shift;
    my $Args = shift;
    my $Class = $Args->{Class};
    my $Info = $Args->{Info};
    my $Refresh = $Args->{Refresh};
    my $Status = mcStatus->New;

    # Check to see if the Cache option was set and if so was it disabled
    if (exists($Args->{Cache}) && $Args->{Cache} == 0) {
	Debug("mcSysInfo->CacheSetup: Cache option is disabled; not caching.");
	$Status->Code($Status->FAILURE);
	return $Status;
    }

    # Main cache directory
    my $maindir = $Args->{CacheDir} || $SysInfoCacheDir;
    # Expire time
    my $expire = $Args->{ExpireTime} || $CacheExpireTime;
    # Need the hostname for this data
    my $host = $SysInfo->Host || 'local';

    #
    # Make sure the name differs from mcsysinfod's cachedir
    # The dir name must include a user unique member since multiple
    # users may run us on the same system.  We use the real UID ($<)
    # to achive this.
    #
    my $cdir = $maindir . '/' . 'mcsysinfoui.cache.' . $< . '/' . $host;
    my $name = $Info || $Class;
    my $Cache = mcFileCache->New({Name=>lc($name), Dir=>$cdir,
				  ExpireTime=>$expire});

    if (!$Cache->Setup) {
	$Status->Code($Status->FAILURE);
	return $Status;
    }

    # Save $Cache for use by other Stream functions
    $self->{FileCache} = $Cache;

    if ($Refresh) {
	Debug("mcSysInfo->CacheSetup: Refresh requested; not reading cache.");
	$Status->Code($Status->FAILURE);
	return $Status;
    }

    if ($Cache->Valid) {
	$Status->Code($Status->SUCCESS);
	Debug("mcSysInfo->CacheSetup: Data will be read from cache.");
    } else {
	$Status->Code($Status->FAILURE);
    }

    return $Status;
}

#
# Read one line
#
sub ReadLine {
    my $self = shift;

    if (exists($self->{FileCache})) {
	return $self->{FileCache}->ReadLine;
    }
}

#
# Close an open cache file
#
sub Close {
    my $self = shift;

    if (exists($self->{FileCache})) {
	$self->{FileCache}->Close;
    }
}
