#
# $Id: mcStreamSie.pm,v 1.3 2003/05/02 19:59:54 mcooper Exp $
#
# Stream driver for SIE
#

package mcStreamSie;
require 5.003;

use mcGlobals;
use mcMisc;
use mcDebug;
use mcStatus;

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"Open"} = undef;
    $self->{"Close"} = undef;
    $self->{"ReadLine"} = undef;

    bless ($self, $class);

    return $self;
}

#
# Read one line
#
sub ReadLine {
    my $self = shift;
    my $fh = $self->{FH};

    if (!$fh) {
	return;
    }

    if (eof($fh)) {
	return undef;
    }

    my $Line = <$fh>;
    if (length($Line)) {
	chop($Line);
    }

    return $Line || "";
}

#
# Is $What in @List?
#
sub InList {
    my $What = shift;
    my @List = @_;

    foreach my $l (@List) {
	if (Eq($l, $What)) {
	    return 1;
	}
    }

    return 0;
}

#
# Open a stream to SIE
#
sub Open {
    my $self = shift;
    my $SysInfo = shift;
    my $Args = shift;
    my $RedirectSTDERR = $Args->{RedirectSTDERR};
    my $Info = $Args->{Info};
    my $Status = mcStatus->New;
    my @Classes;
    my $Path;

    if ($Args->{Class}) {	# Specific override
	push(@Classes, $Args->{Class});
    } else {			# Default to all set classes
	@Classes = $SysInfo->Class;
    }

    if (!$Path && $SysInfo) {
	$Path = $SysInfo->Program;
    }
    if (!$Path && exists($Args->{SysInfo})) {
	$Path = $Args->{SysInfo}->Program || $SysInfo->FindSysInfo;
    }
    if (!$Path) {
	$Status->Code($Status->FAILURE);
	$Status->Value($Status->HARD);
	return $Status;
    }
    Debug("StreamSie->Open: Direct/local retrieve from <%s>", $Path);

    my @Cmd;
    push(@Cmd, $Path);

    # Add caller supplied arguments (if any)
    my @a;
    if (exists($Args->{SysInfo})) {
	@a = $Args->{SysInfo}->ProgramArgs;
    }
    if ($#a < 0) {
	@a = $SysInfo->ProgramArgs;
    }
    if ($#a >= 0) {
	push(@Cmd, @a);
    }

    #
    # Only set repsep if the caller did not and if the
    # caller did not specify '-format report'.
    #
    if (!InList('-repsep', @a) && !InList('--repsep', @a) &&
        !InList('report', @a)) {
	push(@Cmd, '--repsep');	push(@Cmd, "'|'");
    }
    push(@Cmd, '--format'); 	push(@Cmd, "report");
    if (!$Info) {
	push(@Cmd, '--msglevel'); 	push(@Cmd, "all");
    }
    if ($Args->{SwFiles}) {
	push(@Cmd, "+swfiles");
    }
    if ($Debug) {
	push(@Cmd, '--debug');
    }

    # Map 'hardware' -> 'device' for backwards compat
    my @cl = @Classes;
    for (my $i = 0; $i <= $#cl; ++$i) {
	if (lc($cl[$i]) eq 'hardware') {
	    $cl[$i] = 'device';
	}
	
    }
    if ($Info) {
	if (Eq($Info, $RI_MCL)) {
	    push(@Cmd, '--license');
	} elsif (Eq($Info, $RI_MEMBERS)) {
	    push(@Cmd, '--members');
	} elsif (Eq($Info, $RI_TYPES)) {
	    push(@Cmd, '--list');
	    push(@Cmd, 'type');
	} elsif (Eq($Info, $RI_VERSION)) {
	    push(@Cmd, '--version');
	    push(@Cmd, '--msglevel');	push(@Cmd, 'terse');
	} else {
	    Error("StreamSie->Open: Unknown Info type <%s>",
		  $Info);
	    return;
	}
    } else {
	push(@Cmd, '--class');	push(@Cmd, lc(join(',', @cl)));
    }

    # Check for args which require quoting
    for (my $i = 0; $i <= $#Cmd; ++$i) {
	my $QuoteNext = 0;
	if ($Cmd[$i] =~ /repsep/) {
	    $QuoteNext = 1;
	}
	if ($QuoteNext) {
	    if ($Cmd[$i+1] && $Cmd[$i+1] !~ /[\'\"]/) {
		$Cmd[$i+1] = '\'' . $Cmd[$i+1] . '\'';
	    }
	}
    }

    my $Command = join(' ', @Cmd);
    Debug("Running <%s>", $Command);

    if ($RedirectSTDERR) {
	$SaveSTDERR = GetFH();
	if (!open($SaveSTDERR, ">&STDERR")) {
	    Error("Cannot save STDERR: %s", $!);
	    return;
	}
	
	my $fn = fileno($RedirectSTDERR);
	if (!open(STDERR, "<&=$fn")) {
	    Error("Redirect STDERR failed: %s", $!);
	    return;
	}
    }

    $self->{FH} = GetFH();
    if (!open($self->{FH}, '-|', $Command)) {
	Error("%s: Run command failed: %s", $Command, $!);
	return;
    }

    $Status->Code($Status->SUCCESS);

    return $Status;
}

#
# Close a stream
#
sub Close {
    my $self = shift;

    if ($self->{FH}) {
	close($self->{FH});
	$self->{FH} = undef;
    }
}
