#
# $Id: mcStreamSsp.pm,v 1.1 2003/04/19 21:43:56 mcooper Exp $
#
# Stream driver for remote SSP
#

package mcStreamSsp;
require 5.003;

use mcGlobals;
use mcMisc;
use mcDebug;
use mcStatus;

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"Open"} = undef;
    $self->{"Close"} = undef;
    $self->{"ReadLine"} = undef;

    bless ($self, $class);

    return $self;
}

#
# Setup a new SSP connection to Host
#
sub SSPconnect {
    my $self = shift;
    my $SysInfo = shift;
    my $Args = shift;

    if (!$Args || !$SysInfo->Host)  {
	return;
    }

    Debug("mcSysInfo->SSPconnect: Creating connection to <%s>",
	  $SysInfo->Host);

    my $SSP = mcSSP->New({Host=>$SysInfo->Host,
			  Port=>$SysInfo->Port,
			  Username=>$SysInfo->Username,
			  Password=>$SysInfo->Password});

    if ($SSP) {
	# Save it to avoid reconnects later
	$SysInfo->SSP($SSP);
    }

    return $SSP;
}

#
# Retrieve Classes remotely from Host.
# Returns ref to $SSP for stream containing data on success.
# Returns undef on error.
#
sub RetrieveRemote {
    my $self = shift;
    my $SysInfo = shift;
    my $Args = shift;
    my $SSP = $SysInfo->SSP;
    my $Opts;
    my @ClassList;

    if (!$SSP) {
	$SSP = $self->SSPconnect($SysInfo, $Args);
	if (!$SSP) {
	    return;
	}
    }

    #
    # Set options in SSP format per SETOPTION
    #
    if ($Args->{SysInfoOpts}->{MsgLevel}) {
	$Opts->{'msglevel'} = $Args->{SysInfoOpts}->{MsgLevel};
    }
    if ($Args->{SysInfoOpts}->{LicPorts}) {
	$Opts->{'licports'} = $Args->{SysInfoOpts}->{LicPorts};
    }
    if ($Args->{SysInfoOpts}->{NoSerial}) {
	$Opts->{'noserial'} = undef;
    }
    if ($Args->{SysInfoOpts}->{SwFiles}) {
	$Opts->{'SwFiles'} = undef;
    }

    if ($Args->{Class}) {	# Specific override
	push(@ClassList, $Args->{Class});
    } else {			# Default to all set classes
	@ClassList = $SysInfo->Class;
    }

    if ($SSP->Retrieve({ClassList=>\@ClassList,
		        Info=>$Args->{Info},
			Options=>$Opts}) == 0) {
	# Success
	return $SSP;
    }

    # Failure
    return;
}

#
# Open a stream to SIE
#
sub Open {
    my $self = shift;
    my $SysInfo = shift;
    my $Args = shift;
    my $Host = $SysInfo->Host;
    my $Status = mcStatus->New;

    Debug("StreamSsp->Open: Remote retrieve from host <%s>", $Host);

    $self->{SSP} = $self->RetrieveRemote($SysInfo, $Args);

    if (!$self->{SSP}) {
	$Status->Code($Status->FAILURE);
	$Status->Value(-2);	# Hard error
	return $Status;
    }

    $Status->Code($Status->SUCCESS);

    return $Status;
}

#
# Read one line
#
sub ReadLine {
    my $self = shift;
    my $SSP = $self->{SSP};

    if (!$SSP) {
	return;
    }

    return $SSP->GetLine;
}

#
# Close down
#
sub Close {
    my $self = shift;

    # Do not shutdown the connection via SSP->Destroy as we want to leave
    # it up for future callers to save startup time.
}
