#
# $Id: Gui.pm 1.109 2003/05/04 01:08:21 mcooper Exp $
#
# SysInfo GUI functions
#

package Gui;

require 5.003;
require Exporter;

use POSIX;

use mcDebug;
use uiKVF;
use uiGlobals;
use uiMisc;
use uiPath;
use uiClass;
use uiFormat;
use uiDataColumn;
use uiDataTree;
use uiStyles;
use uiTableAdd;
use uiRetrieve;

$UIArgs = undef;
$SysInfo = undef;
$SysInfoPath = undef;
$ShowHostEntry = undef;
$ShowHostName = undef;
my $LoadModulesDone = undef;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   GUI
		   $StatusMessage $MainWindow
		  );
}

#
# Load modules when called
#
sub LoadModules {
    if (!$LoadModulesDone && GuiMode) {
	Debug("Gui: Loading GUI modules ...");
	$LoadModulesDone = 1;

	eval 'use Tk';
	eval 'use Tk::Tree';
	eval 'use Tk::HList';
	eval 'use Tk::ItemStyle';
	eval 'use Tk::Balloon';
	eval 'use Tk::NoteBook';
	eval 'use Tk::LabFrame';
	eval 'use Tk::Adjuster';
	eval 'use Tk::ErrorDialog';	# Tk errors appear in popup windows

	eval 'use GuiMeter';
	eval 'use GuiWindow';
	eval 'use GuiDataDisplay';
	eval 'use GuiWidget';
	eval 'use GuiDiag';
	eval 'use GuiButtons';
	eval 'use GuiOpen';
	eval 'use GuiReport';
	eval 'use GuiIcon';
	eval 'use GuiLicense';
    }
}

#
# Configure Scrollbar properties
#
sub ConfigScroll {
    my $Widget = shift;

# Don't need these anymore
#    $Widget->Subwidget("xscrollbar")->configure(-background=>$BGcolor);
#    $Widget->Subwidget("xscrollbar")->configure(-troughcolor=>$BGcolorOff);
#    $Widget->Subwidget("yscrollbar")->configure(-background=>$BGcolor);
#    $Widget->Subwidget("yscrollbar")->configure(-troughcolor=>$BGcolorOff);
}

#
# Handle toggles of the Format Type buttons
#
sub FormatChange {
    my $FormatName = FormatName;

    Debug("FormatChange(%s) Current=<%s> New=<%s>",
	  join(', ', @_), $FormatName, $FormatSelected);

    foreach my $Class (@$Classes) {
	ClassHideWidget({Class=>$Class, FormatName=>$FormatName});
    }

    FormatSet($FormatSelected);
    ClassShow({FormatChange=>1});
}

#
# Toggle showing properties on and off
#
sub ToggleProperties {
    Debug("ToggleProperties(%s) PropertiesButton=<%s>", 
	  join(', ', @_), $PropertiesButton);

    my $State = $PS_CLOSE;
    if ($PropertiesButton > 0) {
	$State = $PS_OPEN;
    }

    foreach my $Class (@$Classes) {
	my $Table = ClassTable({Class=>$Class});
	foreach my $Path (keys %$Table) {
	    DataTreeToggleProp({Path=>$Path, State=>$State});
	}
    }
}

#
# Display version info
#
sub About {
    if (!$SysInfoPath) {
	Error("The \$SysInfoPath variable is not set.");
	return -1;
    }

    my $FH = GetFH();
    my $Cmd = sprintf("%s -version", $SysInfoPath);
    if (!open($FH, "$Cmd |")) {
	Error("Cannot run <%s> command: %s", $Cmd, $!);
	return -1;
    }

    my @Contents = <$FH>;
    close($FH);
    if ($#Contents < 0) {
	Error("No input found for <%s>.", $Cmd);
	return -1;
    }

    push(@Contents, sprintf("SysInfo User Interface Version %s\n", UIversion));

    WindowDisplay({Contents=>\@Contents, Title=>'About '. $Title,
		   NeedFixed=>1,
		   Image=>$SplashLogo});

    return 0;
}

#
# Display copyright info
#
sub Copyright {
    if (!$SysInfoPath) {
	Error("The \$SysInfoPath variable is not set.");
	return -1;
    }

    my $FH = GetFH();
    my $Cmd = sprintf("%s -copyright", $SysInfoPath);
    if (!open($FH, "$Cmd |")) {
	Error("Cannot run <%s> command: %s", $Cmd, $!);
	return -1;
    }

    my @Contents = <$FH>;
    close($FH);
    if ($#Contents < 0) {
	Error("No input found for <%s>.", $Cmd);
	return -1;
    }
    WindowDisplay({Contents=>\@Contents, Title=>$Title . ' Copyright',
		   Image=>$SplashLogo});

    return 0;
}

#
# Print Input window
#
sub PrintInput {
#    $OpenInputWin = MainWindow->new(-title=>'Print',
#				   );
#    my $ow = $OpenInputWin;
#    $DataFr->postscript(-file=>"/tmp/sysinfo.ps");
}

#
# Quit the application
#
sub Quit {
    my($widget) = @_;

    Debug("Quit called");
    Debug("Quit: args = %s", $widget);
    Debug("Quit: event = %s", $Tk::event);
    Debug("Quit: widget = %s", $Tk::widget);
    Debug("Quit: event->W = %s", $Tk::event->W);

    if ($MainWindow) {
	Debug("Quit: preparing to exit.");
	$MainWindow->bind("<Destroy>", "");
	$MainWindow->destroy;
	$MainWindow = undef;
    }
    exit(0);
}

sub MsgStdErr {
    my $Line;
printf ("MsgStdErr() called\n");
    if ($Line = <STDERR>) {
printf "Line=%s", $Line;
	Msg({tag=>$MSGT_ERROR}, $Line);
    }
}

#
# Signal Handler
#
sub SigHandler {
    my($Sig) = @_;

    Debug("Caught a SIG%s signal.  Exiting cleanly.", $Sig);
    exit(0);
}

#
# Front end to Open Input File
#
sub OpenInput {
    OpenInputFile({MainWindow=>$MainWindow});
}

#
# Support Info
#
sub Support {
    Debug("Support: START");
    my $sw = MainWindow->new(-title=>WinTitle('Support Info'));

    if ($MagniCompLogo300) {
	if (my $image = $sw->Getimage($MagniCompLogo300)) {
	    my $ImageLbl = $sw->Label(-image=>$image)
		->pack(-side=>'top', -expand=>'1', -fill=>'x');
	}
    }

    my $fr = $sw->LabFrame(-label=>'Support Information',
			   -labelside=>'acrosstop')
	->pack(-side=>'top', -fill=>'both', -padx=>1, -pady=>1);

    my $msg = "Support information is available by pointing your web browser at:\n\n\thttp://www.MagniComp.com/support/\n\nWhen reporting a problem, be sure to include diagnostic data which can be obtained by pressing this button:";

    my $MsgLbl = $fr->Label(-font=>$Font,
			    -justify=>'left',
			    -wraplength=>'4i',
			    -textvariable=>\$msg)
	->pack(-fill=>'x', -expand=>1, -side=>'top');

    my $DiagBtn = $fr->Button(-text=>'Diagnostic Info...',
			      -font=>$FontButton,
			      -command=>\&Diagnostic)
	->pack(-side=>'top', -anchor=>'center', -pady=>20, -padx=>20);

    my $CloseBtn = $sw->Button(-text=>'Close',
			       -font=>$FontButton,
			       -command=>sub { $sw->withdraw })
	->pack(-side=>'bottom', -anchor=>'center', -pady=>20, -padx=>20);
}

#
# File Menu Items
#
sub MenuItems_File {
    [
     [ 'command', 'Open File...',
       -accelerator=>"Control+O",
       -font=>$FontMenuItem,
       -command=>\&OpenInput ],
##     [ 'command' => 'Print...',
##       -accelerator=>"Control+P",
##       -font=>$FontMenuItem,
##       -command=>\&PrintInput ],
     '',
     [ 'command' => 'Report...',
       -accelerator=>"Control+R",
       -font=>$FontMenuItem,
       -command=>\&Report ],
     '',
     [ 'command' => 'Exit',
       -accelerator=>"Control+C",
       -font=>$FontMenuItem,
       -command=>\&Quit ],
    ];
}

#
# View Menu Items
#
sub MenuItems_View {
    #
    # Create the Class options
    #
    my @ClassOps;
    foreach my $Class (@$Classes) {
	my $Op = [ 'checkbutton',
		   $Class->{Name},
		   -font=>$FontMenuItem,
		   -variable=>\$Class->{Show},
		   -command=>\&ClassShow,
		 ];
	push(@ClassOps, $Op);
    }
    #
    # Create the Format options
    #
    my @FormatOps;
    foreach my $fmt (FormatList()) {
	my $Op = [ 'radiobutton',
		   $fmt->{Name},
		   -font=>$FontMenuItem,
		   -variable=>\$FormatSelected,
		   -command=>\&FormatChange,
		 ];
	push(@FormatOps, $Op);
    }

    #
    # Return the menu
    #
    [
     [
      'cascade', 'Show Class',
      -font=>$FontMenuItem,
      '-menuitems' => \@ClassOps
     ],
     [
      'cascade', 'Display Format',
      -font=>$FontMenuItem,
      '-menuitems' => \@FormatOps
     ],
     '',
     [
      'checkbutton', 'All Properties',
      -variable=>\$PropertiesButton,
      -font=>$FontMenuItem,
      -command=>\&ToggleProperties,
     ],
     [
      'checkbutton', 'Enable Debug',
      -font=>$FontMenuItem,
      -variable=>\$mcDebug::Debug,
      -command=>\&mcDebug::DebugToggle,
     ],
    ];
}

#
# Help Menu Items
#
sub MenuItems_Help {
    [
     [ 'command', 'About...',
       -font=>$FontMenuItem,
       -command=>\&About ],
     [ 'command', 'License Info...',
       -font=>$FontMenuItem,
       -command=>\&LicenseInfo ],
     [ 'command', 'License Terms...',
       -font=>$FontMenuItem,
       -command=>\&LicenseTermsView ],
     [ 'command', 'Copyright...',
       -font=>$FontMenuItem,
       -command=>\&Copyright ],
     '',
     [ 'command', 'Support...',
       -font=>$FontMenuItem,
       -command=>\&Support ],
     [ 'command', 'Diagnostic Info...',
       -font=>$FontMenuItem,
       -command=>\&Diagnostic ],
    ];
}

#
# Create a spacer between icon sets
#
sub IconSetSpace {
    my $Args = shift;
    my $Widget = $Args->{Widget};

    if (!$Widget) {
	return;
    }

    if (!$IconDivider) {
	# Create Icon Divider image
	$IconDivider = $MainWindow->Getimage('icondivider');
    }

    $Widget->Label(-image=>$IconDivider)
	->pack(-side=>'left');
}

#
# ShowHost is called when the user presses the "Show Host" button.
# We need to show info for the specified host.
#
sub ShowHost {
    Debug("ShowHost called.  ShowHostName=<%s>", $ShowHostName);

    if (!length($ShowHostName)) {
	Error("You must enter a hostname before you press the \"Show Host\" button.");
	return;
    }

    if (!$UIArgs) {
	Error("Internal error - no UIArgs found.");
	return;
    }
    if (!exists($UIArgs->{SysInfo})) {
	Error("Internal error - no UIArgs->SysINfo found.");
	return;
    }

    $UIArgs->{SysInfo}->Host($ShowHostName);
    Retrieve({Reset=>1});

    if ($MainWindow) {
	$MainWindow->configure(-title=>WinTitle({Host=>$ShowHostName}));
    }
}

#
# Start GUI
#
sub GUI {
    my $Args = shift;
    $UIArgs = $Args;
    $SysInfo = $Args->{SysInfo};
    $SysInfoPath = $Args->{SysInfoPath};
    my @TkArgs = @{ $Args->{TkArgs} };
    my @UserClassList = @{ $Args->{ClassList} };

    SetIM($IM_GUI);

    Debug("Starting GUI");

    LoadModules();

    ClassSet($Args);

    #
    # Set global UI Args
    #
    $uiRetrieve::UIArgs = $UIArgs;
    $uiLicense::SysInfo = $SysInfo;

    #
    # Trap signals
    #
    $SIG{'ABRT'} = \$SigHandler;
    $SIG{'BUS'} = \$SigHandler;
    $SIG{'SEGV'} = \$SigHandler;
    $SIG{'ILL'} = \$SigHandler;

    #
    # Some OS have the default background color for apps set to something
    # incompatible (like "white") with us so we set this explicitly.
    #
    Tk::CmdLine::SetResources('*foreground:black');

    #
    # Parse Tk (really X11) command line options
    #
    if ($#TkArgs >= 0) {
	Debug("Parsing Tk (X11) options: %s", join(' ', @TkArgs));
	Tk::CmdLine::SetArguments(@TkArgs);
	# If user specified a geometry, insure we don't set ours later
	for (my $i = 0; $i <= $#TkArgs; ++$i) {
	    my $opt = $TkArgs[$i];
	    if ($opt =~ /geometry/) {
		$MainGeometry = undef;
	    }
	}
    }

    #
    # Tk setup
    #
    $MainWindow = MainWindow->new(-title=>WinTitle($Args));
    my $mw = $MainWindow;

    #
    # Our canonical main window height x width
    #
    if ($MainGeometry) {
	$mw->geometry($MainGeometry);
    }

    #
    # Our canonical application name
    #
    $mw->appname("sysinfo");

    #
    # Set program icon (not yet working)
    #
    if (my $icon = $mw->Getimage($SysInfoIcon)) {
	$mw->Icon(-image=>$icon);
	$mw->iconname("SysInfo");
    }

    #
    # Setup Debug values
    #
    Debug({Debug=>$Debug, Func=>\&Msg});

    #
    # Create Splash screen
    #
    MeterCreate({Splash=>1, Title=>'SysInfo', Message=>'Welcome to SysInfo'});

    #
    # Read in preferences
    #
    $Args->{Prefs} = $Prefs = KVFread({File=>$PrefsFile, QuietError=>1});

    #
    # Key Bindings
    #
    ## Need to disable this since we now use destroy() for widget destructs
    ##$mw->bind("<Destroy>", \&Quit);
    $mw->bind("<Control-c>", \&Quit);
    $mw->bind("<Control-o>", \&OpenInput);

    ###########################################################################
    #
    # Each of the major screen areas are categorized as either 'top' widgets
    # or 'bottom' widgets.  The top level menu bar is a 'top' widget.  Most all
    # others are 'bottom' widgets.  The bottom widgets are listed as such to
    # allow Tk to auto-resize each widget when the user resizes the main
    # application window.  Each bottom widget must be listed in reverse
    # order to insure proper onscreen placement.
    #
    ###########################################################################

    #
    # Menus
    #
    $mw->configure(-menu=>my $MenuBar = $mw->Menu);
    $MenuBar->configure(-font=>$FontMenuTitle);
    my $FileMenu = $MenuBar->cascade(-label=>'~File',
				     -menuitems=>MenuItems_File);
    my $ViewMenu = $MenuBar->cascade(-label=>'~View',
				     -menuitems=>MenuItems_View);
    my $HelpMenu = $MenuBar->cascade(-label=>'~Help',
				     -menuitems=>MenuItems_Help);
    #
    # Message window
    #
    $msgText = $mw->Scrolled("Text",
			     -scrollbars=>'osoe',
			     -width=>$WidthChars,
			     -height=>2,
			     -font=>$FontText,
			     -background=>'#EEEEEE',
			    )
	->pack(-side=>'bottom',
	       -fill=>'both',
	       -expand=>0,	# lock size of widget - ignore window resize
	      );

    #
    # Create an adjuster widget above $MsgText
    #
    my $msgAdj = $mw->Adjuster->
	packAfter($msgText, -side=>'bottom');

    #
    # Icon Row Frame
    #
    my $IconFr = $mw->Frame(-borderwidth=>2,
			    -relief=>'raised')
	->pack(-side=>'top', -anchor=>'w', -fill=>'x');

    #
    # Status message area
    #
## We don't need this anymore since we have Balloon
##    $mw->Label(-textvariable=>\$StatusMessage,
##	       -borderwidth=>2,
##	       -font=>$FontLabel,
##	       -relief=>'groove')
##	->pack(-side=>'top', -fill=>'x');

    #
    # Tool tips / help
    #
    $HelpTips = $mw->Balloon(-background=>$HelpTipsBackground,
			     # Need this for $DataTabs
			     -balloonposition=>'mouse');
    if (my $m = $HelpTips->Subwidget('message')) {
	$m->configure(justify => 'left');
    }

    #
    # Open button
    #
    my $image = $mw->Getimage('fileopen');
    $OpenBt = $IconFr->Button(#-text=>"Open",
			      -image=>$image,
			      -font=>$FontButton,
			      -command=>\&OpenInput)
	->pack(-side=>'left', -anchor=>'center');
    $HelpTips->attach($OpenBt,
		      -balloonmsg=>"Open a \"sysinfo --format report\" file and use as input");

    #
    # Format Buttons
    #
    IconSetSpace({Widget=>$IconFr});
    foreach my $fmt (FormatList) {
	my $n = $fmt->{Name};
	my $image = ($fmt->{Icon}) ? $mw->Getimage($fmt->{Icon}) : undef;
	if (!$image) {
	    Debug("No Icon defined for Format type <%s>", $n);
	    # No icon means don't create a button.
	    next;
	}
	my $b = $IconFr->Radiobutton(#-text=>$a,
				     -anchor=>'center',
				     -image=>$image,
				     -indicatoron=>0,
				     -value=>$n,
				     -font=>$FontButton,
				     -variable=>\$FormatSelected,
				     -command=>\&FormatChange)
	    ->pack(-side=>'left');
	$fmt->{button} = $b;	# Save for later use

	$HelpTips->attach($b, -balloonmsg=>"Set display format to " . lc($fmt->{Desc}));
    }

    #
    # Report button
    #
    IconSetSpace({Widget=>$IconFr});
    my $image = $mw->Getimage('report');
    $ReportBt = $IconFr->Button(#-text=>"Reports",
				-image=>$image,
				-font=>$FontButton,
				-command=>\&Report)
	->pack(-side=>'left', -anchor=>'center');
    $HelpTips->attach($ReportBt,
		      -balloonmsg=>"Create reports and save to files or print");

    #
    # Add Show Host text widget
    #
    IconSetSpace({Widget=>$IconFr});
    $ShowHostEntry = $IconFr->Entry(-width=>12,
				    -textvariable=>\$ShowHostName)
	->pack(-side=>'left', -anchor=>'center');
    $ShowHostBt = $IconFr->Button(-text=>"Show Host",
				  -font=>$FontButton,
				  -command=>\&ShowHost)
	->pack(-side=>'left', -anchor=>'center');
    $HelpTips->attach($ReportBt,
		      -balloonmsg=>"Show info for specified host instead of local system");

    #
    # We only use Class buttons if Debug is enabled
    #
    if ($Debug) {
	IconSetSpace({Widget=>$IconFr});

	#
	# Class Buttons
	#
	foreach my $Class (@$Classes) {
	    my $image = ClassImage({Class=>$Class, Widget=>$mw});
	    $Class->{Button} = $IconFr->Checkbutton(-text=>$Class->{Name},
						    -image=>$image,
						    -indicatoron=>0,
						    -font=>$FontButton,
						    -variable=>\$Class->{Show},
						    -command=>\&ClassShow)
		->pack(-side=>'left', -anchor=>'center');

	    my $msg = 'Toggle display of ' . lc($Class->{Desc});
	    $HelpTips->attach($Class->{Button}, -balloonmsg=>$msg);
	    ##	BindStatus($Class->{Button}, $msg);

	    # Initialize State
	    $Class->{ShowState} = $Class->{Show};
	}
    }

    #
    # Add SysInfo logo to icon bar
    #
    if (my $image = $mw->Getimage($SysInfoLogo18)) {
	my $ImageLbl = $IconFr->Label(-image=>$image)
	    ->pack(-side=>'right');
    }

    #
    # Setup data widgets for any requested classes
    #
    if ($#UserClassList >= 0) {
	WidgetSetup({ClassList=>\@UserClassList});
    }
    #
    # Setup data widgets for all/remaining classes
    #
    WidgetSetup();

    #
    # Send STDERR to the MsgText widget
    #
    ##$MainWindow->fileevent(STDERR, 'readable', \&MsgStdErr);

    #
    # Check status of license
    #
    LicenseCheck($Args);

    #
    # Check to see if user has accepted/declined license terms
    #
    if (LicenseTermsAsk($Args) != 0) {
	Error("License terms were not accepted.");
	exit(1);
    }

    #
    # Automatically call Retrieve 500ms after GUI is started.
    #
    $AfterID = $MainWindow->after(500, \&Retrieve);

    Debug("Starting MainLoop");
    #
    # Tk event loop
    #
    MainLoop();
    Debug("MainLoop complete");

    # Insure we don't try to use object which are now destroyed
    $msgText = undef;
    $MainWindow = undef;

    Debug("Done with MainLoop");

    return 0;
}

1;
