#
# $Id: GuiClass.pm,v 1.2 2003/05/04 00:55:00 mcooper Exp $
#
# GUI Class functions
#

package GuiClass;

require 5.003;

require Exporter;

use mcDebug;
use uiMisc;
use uiGlobals;
use uiFormat;
use uiPath;
use uiRetrieve;

use GuiDataDisplay;
use GuiWidget;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   GuiClassHideWidget
		   GuiClassShowClass
		  );
}

#
# Hide the indicated class widget
#
sub GuiClassHideWidget {
    my $Args = shift;
    my $Class = $Args->{Class};
    my $FormatName = $Args->{FormatName};
    my $ArrangeName = $Args->{ArrangeName};

    if (!$Class) {
	return;
    }

    if (my $WidgetName = GuiWidget::WidgetName({Class=>$Class,
						ArrangeName=>$ArrangeName,
						FormatName=>$FormatName})) {
	if (my $Widget = $Class->{$WidgetName}) {
	    Debug("ClassHideWidget: Hiding widget <%s> for class <%s>",
		  $WidgetName, $Class->{Name});
	    $Widget->packForget if WidgetExists($Widget);
	    $Class->{Widget} = undef;
	}
    }
}

#
# Show Class $Class
#
sub GuiClassShowClass {
    my $Args = shift;
    my $Class = $Args->{Class};
    my $KeepMeter = $Args->{KeepMeter};
    my $FormatChange = $Args->{FormatChange};
    my $FormatName = FormatName();

    my $NoPath = 0;
    if (!uiPath::PathExists($Class->{Name})) {
	$NoPath = 1;
    }

    my $ChangeState = 0;
    if ($NoPath) {
	if ($Class->{Show} && $Class->{Show} != $Class->{ShowState}) {
	    $ChangeState = 1;
	}
    } else {
	if ($Class->{Show} != $Class->{ShowState}) {
	    $ChangeState = 1;
	}
    }

    Debug("ClassShow: Name=<%s> Show=%d State=%s ChangeState=%d FormatChange=%d NoPath=%s",
	  $Class->{Name}, $Class->{Show}, $Class->{ShowState},
	  $ChangeState, $FormatChange, $NoPath);

    if ($ChangeState) {
	Debug("ClassShow(): Changing <%s> to state %s",
	      $Class->{Name}, $Class->{Show});
	
	if ($Class->{Show}) {
	    Debug("ClassShow(): Setting widget for <%s> to SHOW",
		  $Class->{Name});
	} else {
	    Debug("ClassShow(): Hiding widget <%s> for <%s>",
		  FormatWidget(), $Class->{Name});
	    ClassHideWidget({Class=>$Class, FormatName=>$FormatName});
	}

	$Class->{ShowState} = $Class->{Show};
    }

    if ($Class->{Show}) {
	#
	# Insure that the proper widgets are setup for this class
	#
	if ($ChangeState || $FormatChange) {
	    GuiWidget::WidgetSetup({Class=>$Class});
	}
	
	#
	# $NoPath means there is no class data.  If DataDisplayed is FALSE
	# then the class data is present and just needs to be displayed.
	#
	my $WidgetName = WidgetName({Class=>$Class,
				     FormatName=>$FormatName});
	if ($NoPath) {
	    Retrieve({ClassNames=>[ $Class->{Name} ]});
	} elsif (!$Class->{$WidgetName}->{DataDisplayed}) {
	    DataDisplay({Class=>$Class, KeepMeter=>1});
	}

	#
	# Insure that if this was a $ChangeState that the current tab is raised
	#
	if ($ChangeState) {
	    WidgetTabRaise({Tab=>$Class->{Name}});
	}
    }
}

1;
