#
# $Id: GuiDataDisplay.pm,v 1.11 2003/05/04 00:56:10 mcooper Exp $
#
# GUI Data Display functions
#

package GuiDataDisplay;

require 5.003;

require Exporter;

use mcDebug;
use uiMisc;
use uiGlobals;
use uiStyles;
use uiFormat;
use uiData;
use uiDataTree;
use uiDataColumn;
use uiTableAdd;

use GuiMeter;
use GuiIcon;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   DataDisplay
		  );
}

#
# Take all the objects added into $Table by TableAdd and display them
# using the appropriate Tk widget.
#
sub DataDisplayClass {
    my $Args = shift;
    my $Class = $Args->{Class};
    my $Reset = $Args->{Reset};
    my $KeepMeter = $Args->{KeepMeter};
    my $Table = uiClass::ClassTable({Class=>$Class});

    Debug("DataDisplayClass: Class=<%s>", $Class->{Name});

    if (!$Table) {
	Debug("DataDisplayClass: Class <%s> has no data",
	      $Class->{Name});
	return;
    }

    my $WidgetName = GuiWidget::WidgetName({Class=>$Class,
					    FormatName=>FormatName()});

    if (!$Reset && exists($Class->{$WidgetName}) &&
	$Class->{$WidgetName}->{DataDisplayed}) {
	Debug("DataDisplayClass: Widget <%s> for class <%s> already DISPLAYED",
	      $WidgetName, $Class->{Name});
	return;
    }

    my $What = sprintf("Building data display for %s. . .", $Class->{Name});
    Status($What);
    MeterCreate({Message=>$What});

    # Prepare list of paths
    my @List = sort keys %$Table;

    if ($Reset) {
	#
	# Delete all the existing entries in the widget
	#
	foreach my $path (@List) {
	    Debug("DataDisplayClass: Deleting widget entry for <%s>", $path);
	    my $Widget = $Class->{Widget};
	    if (!$Widget) {
		$Widget = DataGetWidget($path);
	    }
	    if (!$Widget || !WidgetExists($Widget)) {
		Debug("DataDisplayClass: Cannot find Widget for <%s>", $path);
		next;
	    }
	    if ($Widget->info('exists', $path)) {
		$Widget->delete('entry', $path);
	    }
	}
    }

    #
    # Now add each path to the Tree in sorted order.  The sorted order
    # is required by $Widget->add()
    #
    my $Count = 0;
    my $Tree;
    for (my $i = 0; $i <= $#List; ++$i) {
	if ($#List > 0) {
	    MeterProgress({Percent=>$i/$#List});
	}

	++$Count;

	my $Path = $List[$i];
	if (!length($Path)) {
	    next;
	}

	my $Widget = $Class->{Widget};
	if (!$Widget) {
	    $Widget = DataGetWidget($Path);
	}
	if (!$Widget || !WidgetExists($Widget)) {
	    Debug("DataDisplayClass: Cannot find Widget for <%s>", $Path);
	    next;
	}

	if ($Widget->info('exists', $Path)) {
	    Debug("DataDisplayClass: Skipping <%s> - Already exists", $Path);
	    next;
	}
	if ($Path =~ /$PathSep$/) {
	    Debug("DataDisplayClass: Deleting <%s>", $Path);
	    # Fix it so we don't have later problems
	    delete $Table->{$Path};
	    next;
	}
	my $Entry = $Table->{$Path}->{Obj};
	my @p = split($PathSep, $Path);
	my $Name = $p[$#p];

	if ($Entry =~ /HASH/) {
	    if (HasMember($Entry, 'Name') && $Entry->Name) {
		$Name = $Entry->Name;
	    }
	} else {
	    $Name = $Entry;
	}
	if (!length($Name) || !length($Path)) {
	    next;
	}

	Debug("DataDisplayClass: Adding name=<%s> path=<%s>", $Name, $Path);

	my $Str = $Name;

	if (FormatIs($FT_TREE)) {
	    #
	    # Tree widget
	    #
	    if ($Entry =~ /HASH/ &&
		(my $Summary = DataTreeSummary({Entry=>$Entry,
						Path=>$Path}))) {
		# If the Summary starts with $Name, use it without modifying
		if ($Summary =~ /^$Name/) {
		    $Str = $Summary;
		} else {
		    $Str = sprintf("%s %s", $Name, $Summary);
		}
	    }

	    if (my $Icon = GetIcon({Entry=>$Entry, Path=>$Path})) {
		my $Image = $Widget->Getimage($Icon);
		$Widget->add($Path, -text=>$Str, -image=>$Image);
	    } else {
		$Widget->add($Path, -text=>$Str);
	    }
	    $Tree = $Widget;
	} elsif (FormatIs($FT_COLUMNS)) {
	    #
	    # HList widget
	    #

	    # Add the main path
	    if (!$Widget->infoExists($Path)) {
		$Widget->add($Path);
	    }

	    if (!$Widget->{Styles}) {
		StylesAdd({Widget=>$Widget});
	    }
	    my $Styles = $Widget->{Styles};

	    if ($Class->{SingleEntry}) {
		#
		# This $Entry has a single top level member.  As such we
		# want to present the data as properties instead of
		# top-level nodes.
		#

		# Add the property values for all of the top level to $Table
		TableAddProps({Entry=>$Entry, Widget=>$Widget, Path=>$Path,
			       Class=>$Class, Table=>$Table});

		# Look for each path which starts with 'General'
		my @Paths = sort keys %$Table;
		my $gc = 0;
		foreach my $path (@Paths) {
		    my @p = split($PathSep, $path);
		    if (!Eq($p[0], $Class->{Name})) {
			next;
		    }

		    if ($Widget->infoExists($path)) {
			next;
		    }

		    # Add the path
		    $Widget->add($path);

		    my $Style =  $Styles->{odd};
		    if ($gc++ % 2) {
			$Style = $Styles->{even};
		    }

		    my @entries = ($Table->{$path}->{Desc},
				   $Table->{$path}->{Value});
		    if (DataColumnAdd({Name=>$Name, Entry=>$Entry,
				       Widget=>$Widget,
				       Class=>$Class,
				       Style=>$Style,
				       Entries=>\@entries,
				       Path=>$path}) < 0) {
			$Widget->delete('entry', $path);
		    }
		}
	    } else {
		my $Style =  $Styles->{odd};
		if ($gc++ % 2) {
		    $Style = $Styles->{even};
		}

		my $Result = DataColumnAdd({Name=>$Name, Entry=>$Entry,
					    Widget=>$Widget,
					    Class=>$Class,
					    Style=>$Style,
					    HeaderStyle=>$Styles->{header},
					    Path=>$Path});
		#
		# If the add failed and $Path is not the class (which is
		# a required path), then delete the entry.
		#
		if ($Result < 0 && !Eq($Class->{Name}, $Path)) {
		    Debug("DataDisplayClass: DataColumnAdd failed - deleting path <%s>",
			  $Path);
		    $Widget->delete('entry', $Path);
		}
	    }

	}

	TableAddProps({Entry=>$Entry, Widget=>$Widget, Path=>$Path,
		       Class=>$Class});
    }

    Debug("DataDisplayClass: Path Table for <%s> has %d entries",
	  $Class->{Name}, $Count);

    if (FormatIs($FT_TREE) && $Tree && WidgetExists($Tree)) {
	#
	# Now that $Tree is populated, add the +/- to allow user to
	# expand the tree as needed.
	#
	$Tree->autosetmode();
    }

    #
    # Mark the data as being displayed
    #
    if (exists($Class->{$WidgetName})) {
	$Class->{$WidgetName}->{DataDisplayed} = 1;
    }

    if (!$KeepMeter) {
	MeterDestroy();
    }

    Status("");
}

#
# Take all the objects in each Class->Table (added by TableAdd) and 
# display them.
#
sub DataDisplay {
    my $Args = shift;
    my $KeepMeter = $Args->{KeepMeter};

    if ($Args->{Class}) {
	DataDisplayClass($Args);
    } else {
	$Args->{KeepMeter} = 1;
	foreach my $Class (@$Classes) {
	    $Args->{Class} = $Class;
	    DataDisplayClass($Args);
	}

	if (!$KeepMeter) {
	    MeterDestroy();
	}
    }
}

1;
