#
# $Id: GuiReport.pm,v 1.11 2003/05/17 22:01:20 mcooper Exp $
#
# GUI Report functions
#

package GuiReport;

require 5.003;

require Exporter;

#
# Make sure to update ui/sysinfoui.in -> $PerlModules to match
# the 'use' commands below.
#
use Tk;

use mcDebug;
use uiMisc;
use uiGlobals;
use uiClass;
use GuiMeter;
use GuiWindow;

my $ReportWin;
my $OutputFile;			# Name of file to output to
my $RepSep	= '|';		# Report character seperator

#
# Report Format definetions.  Names must be valid for "sysinfo -format <name>".
#
my $RF_TREE	= 'tree';
my $RF_COLUMNS	= 'columns';
my $RF_REPORT	= 'report';
#
# Formats Table
#
my $Formats = {
	       $RF_TREE =>
		   {
		    Order=>0,
		    Name=>$RF_TREE,
		    Desc=>'Hierarchical tree suitable for human consumption',
		   },
	       $RF_COLUMNS =>
		   {
		    Order=>1,
		    Name=>$RF_COLUMNS,
		    Desc=>'Columns and rows suitable for human consumption',
		   },
	       $RF_REPORT =>
		   {
		    Order=>2,
		    Name=>$RF_REPORT,
		    Desc=>'Field deliminated for software/program consumption',
		   },
};
my $FormatSelected = $RF_COLUMNS;	# Currently selected $Formats key name

#
# Output Encoding definetions. Must be valid for "sysinfo -encode <name>".
#
my $OE_TEXT	= 'text';
my $OE_HTML	= 'html';
#
# Encoding Table
#
my $Encodings = {
	       $OE_TEXT =>
		   {
		    Order=>0,
		    Name=>$OE_TEXT,
		    Desc=>'ASCII text for viewing in a terminal window or file',
		   },
	       $OE_HTML =>
		   {
		    Order=>1,
		    Name=>$OE_HTML,
		    Desc=>'HTML for viewing in a web browser',
		   },
};
my $EncodeSelected = $OE_TEXT;	# Currently selected $Encodings key name

#
# Output destinations
#
my $RO_FILE	= 'file';
my $RO_PRINTER	= 'printer';
my $Outputs = {
	       $RO_FILE =>
		   {
		    Order=>0,
		    Name=>$RO_FILE,
		    Desc=>'Send to file',
		   },
	       $RO_PRINTER =>
		   {
		    Order=>1,
		    Name=>$RO_PRINTER,
		    Desc=>'Print command',
		   },
};
my $OutputDest  = $RO_FILE;
my $PrintCommand = "lpr";

#
# Message Levels.  Must be a valid "sysinfo -msglevel <level>"
#
my $ML_TERSE	= 'terse';
my $ML_BRIEF	= 'brief';
my $ML_GENERAL	= 'general';
my $ML_DESC	= 'descriptions';
my $ML_CONFIG	= 'config';
my $ML_ALL	= 'all';
my $MessageLevels = {
		     $ML_TERSE =>
			 {
			  Order=>0,
			  Name=>$ML_TERSE,
			  Desc=>'Terse - Display only values',
			 },
		     $ML_BRIEF =>
			 {
			  Order=>1,
			  Name=>$ML_BRIEF,
			  Desc=>'Brief - More info in a condensed format',
			 },
		     $ML_GENERAL =>
			 {
			  Order=>2,
			  Name=>$ML_GENERAL,
			  Desc=>'General - Good for a quick glance',
			 },
		     $ML_DESC =>
			 {
			  Order=>3,
			  Name=>$ML_DESC,
			  Desc=>'Desc - More detailed info',
			 },
		     $ML_CONFIG =>
			 {
			  Order=>4,
			  Name=>$ML_CONFIG,
			  Desc=>'Config - Detailed configuration info',
			 },
		     $ML_ALL =>
			 {
			  Order=>5,
			  Name=>$ML_ALL,
			  Desc=>'All - The maximum amount of info',
			 },
};
my $MessageLevel = $ML_GENERAL;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   Report
		  );
}

#
# Handle press of an Output Destination button
#
sub OutputButtons {
    foreach my $key (keys %$Outputs) {
	my $State = 'disable';
	if (Eq($key, $OutputDest)) {
	    $State = 'normal';
	}

	my @Widgets = @{ $Outputs->{$key}->{InputWidgets} };
	foreach my $widget (@Widgets) {
	    if (WidgetExists($widget)) {
		$widget->configure(-state=>$State);
	    }
	}
    }
}

#
# Set the output file name
#
sub BrowseFile {
    $OutputFile = $ReportWin->getOpenFile(-title=>WinTitle("Send to file"));
    Debug("BrowseFile: File=<%s>", $OutputFile);
}

#
# Sort routine for sorting $EncodeTable
#
sub EncodeSort {
    return $Encodings->{$a}->{Order} <=> $Encodings->{$b}->{Order};
}

#
# Return a list/array of valid encodes in sorted order
#
sub EncodeList {
    my @list;

    my @keys = keys %$Encodings;
    foreach my $key (sort EncodeSort @keys) {
	push(@list, $Encodings->{$key});
    }

    return @list;
}

#
# Sort routine for sorting $FormatTable
#
sub FormatSort {
    return $Formats->{$a}->{Order} <=> $Formats->{$b}->{Order};
}

#
# Return a list/array of valid formats in sorted order
#
sub FormatsList {
    my @list;

    my @keys = keys %$Formats;
    foreach my $key (sort FormatSort @keys) {
	push(@list, $Formats->{$key});
    }

    return @list;
}

#
# Sort routine for sorting $MessageLevelTable
#
sub MessageLevelSort {
    return $MessageLevels->{$a}->{Order} <=> $MessageLevels->{$b}->{Order};
}

#
# Return a list/array of valid MessageLevels in sorted order
#
sub MessageLevelsList {
    my @list;

    my @keys = keys %$MessageLevels;
    foreach my $key (sort MessageLevelSort @keys) {
	push(@list, $MessageLevels->{$key});
    }

    return @list;
}

#
# Get the command to run
#
sub GetCommand {
    my $Args = shift;
    my $DoQuote = $Args->{DoQuote};

    if (Eq($OutputDest, $RO_FILE) && !length($OutputFile)) {
	Error("You must specifiy the file name to write the report to.");
	return;
    }
    if (Eq($OutputDest, $RO_PRINTER) && !length($PrintCommand)) {
	Error("You must specifiy the print command to use.");
	return;
    }

    my @ClassNames;
    foreach my $Class (@$Classes) {
	if ($Class->{ReportOn}) {
	    push(@ClassNames, $Class->{Name});
	}
    }
    if ($#ClassNames < 0) {
	Error("No class is selected which means there is nothing to report.");
	return;
    }

    my @Cmd;
    push(@Cmd, $ENV{'SYSINFO_COMMAND_PATH'} || $0);
    push(@Cmd, '--nw');
    push(@Cmd, '--repsep');
    if ($DoQuote) {
	push(@Cmd, $RepSep);
    } else {
	# We must be showing command to user so be careful
	push(@Cmd, '\'' . $RepSep . '\'');
    }
    push(@Cmd, '--class');
    push(@Cmd, join(',', @ClassNames));
    push(@Cmd, '--format');
    push(@Cmd, $FormatSelected);
    push(@Cmd, '--encode');
    push(@Cmd, $EncodeSelected);
    push(@Cmd, '--msglevel');
    push(@Cmd, $MessageLevel);
    if (Eq($OutputDest, $RO_FILE) && length($OutputFile)) {
	push(@Cmd, '--output');
	push(@Cmd, $OutputFile);
    }

    Debug("ReportRun: Argv command=<%s>",
	  join(' ', @Cmd));

    #
    # Build the Command string quoting all, but the command name itself.
    #
    my @CmdQ;
    for (my $i = 0; $i <= $#Cmd; ++$i) {
	if ($i == 0) {
	    push(@CmdQ, $Cmd[$i]);
	} else {
	    push(@CmdQ, sprintf("'%s'", $Cmd[$i]));
	}
    }

    if ($DoQuote) {
	return @CmdQ;
    } else {
	return @Cmd;
    }
}

#
# Show the command to run to the user
#
sub ReportCommand {
    Debug("ReportCommand: called");

    my @Cmd = GetCommand();
    if ($#Cmd < 0) {
	return -1;
    }

    my $Command = join(' ', @Cmd);
    Debug("ReportRun: Command=<%s>", $Command);

    WindowDisplay({Title=>'Report Command',
		   Contents=>$Command});

    return 0;
}

#
# Run the actual report
#
sub ReportRun {
    Debug("ReportRun: called");

    my @CmdQ = GetCommand({DoQuote=>1});
    if ($#CmdQ < 0) {
	return -1;
    }

    my $Command = join(' ', @CmdQ);
    Debug("ReportRun: Command=<%s>", $Command);

    #
    # Time to run the report
    #
    MeterCreate({Message=>'Probing system to create report ...'});
    my $SuccessMsg;
    my $Status;
    if (Eq($OutputDest, $RO_FILE)) {
	$SuccessMsg = sprintf("The report was written to file %s",
			      $OutputFile);
	$Status = system($Command);
	MeterDestroy();
    } elsif (Eq($OutputDest, $RO_PRINTER)) {
	$SuccessMsg = sprintf("The report was printed via `%s'.",
			      $PrintCommand);
	my $INFH = GetFH();
	if (!open($INFH, "$Command |")) {
	    Error("Run command (%s) failed: %s", $Command, $!);
	    return -1;
	}
	my $PRT = GetFH();
	if (!open($PRT, sprintf("| %s", $PrintCommand))) {
	    Error("Create print command (%s) failed: %s", $PrintCommand, $!);
	    close($INFH);
	    return -1;
	}

	while (my $Line = <$INFH>) {
	    printf $PRT "%s", $Line;
	}

	close($INFH);
	$Status = !close($PRT);	# Close returns TRUE on success
	MeterDestroy();
    }

    if ($Status != 0) {
	Error("The report command did not complete successfully.");
	return -1;
    } else {
	UserMsg("The report command completed successfully.  " . $SuccessMsg);
	$ReportWin->withdraw;
	return 0;
    }
}

#
# Create a Report
#
sub Report {
    Debug("Report: START");
    $ReportWin = MainWindow->new(-title=>WinTitle('Report'));

    # Outer frame
    my $frouter = $ReportWin->LabFrame(-label=>'Create Report',
				       -labelside=>'top')
	->pack(-side=>'top', -fill=>'both');

    # Inner frame which provides spacing
    my $fr = $frouter->Frame
	->pack(-side=>'top', -fill=>'both', -padx=>10);

    #
    # Class selection
    #
    my $ClassFr = $fr->LabFrame(-label=>'Classes of data to include',
				-labelside=>'acrosstop')
	->pack(-fill=>'x',
	       -expand=>1,
	       -side=>'top');
    foreach my $Class (@$Classes) {
	$Class->{ReportOn} = $Class->{Show}; # Default to what's enabled now
	my $image = ClassImage({Class=>$Class, Widget=>$ReportWin});
	my $b = $ClassFr->Checkbutton(#-text=>$Class->{Desc},
				      -image=>$image,
				      -indicatoron=>0,
				      -anchor=>'center',
				      -font=>$FontButton,
				      -variable=>\$Class->{ReportOn})
	    ->pack(-side=>'left', -anchor=>'w');
	$HelpTips->attach($b, -balloonmsg=>$Class->{Desc});
    }

    #
    # Message Level buttons
    #
    my $LevelFr = $fr->LabFrame(-label=>'Level of information',
				-labelside=>'acrosstop')
	->pack(-fill=>'x',
	       -expand=>1,
	       -side=>'top');
    foreach my $Level (MessageLevelsList()) {
	my $b = $LevelFr->Radiobutton(-text=>$Level->{Desc},
				      -anchor=>'center',
				      -font=>$FontButton,
				      -value=>$Level->{Name},
				      -variable=>\$MessageLevel)
	    ->pack(-side=>'top', -anchor=>'w');
    }

    #
    # Format buttons
    #
    my $FormatFr = $fr->LabFrame(-label=>'Output format',
				 -labelside=>'acrosstop')
	->pack(-fill=>'x', -expand=>1, -side=>'top');
    foreach my $Format (FormatsList()) {
	my $Widget = $FormatFr->Frame
	    ->pack(-fill=>'x', -expand=>1, -side=>'top');

	my $b = $Widget->Radiobutton(-text=>$Format->{Desc},
				     -anchor=>'center',
				     -font=>$FontButton,
				     -value=>$Format->{Name},
				     -variable=>\$FormatSelected)
	    ->pack(-side=>'left', -anchor=>'w');

	if (Eq($Format->{Name}, $RF_REPORT)) {
	    my $e = $Widget->Entry(-width=>1,
				   -textvariable=>\$RepSep)
		->pack(-side=>'left', -anchor=>'w');
	}
    }

    #
    # Encoding buttons
    #
    my $EncodeFr = $fr->LabFrame(-label=>'Output encoding',
				 -labelside=>'acrosstop')
	->pack(-fill=>'x',
	       -expand=>1,
	       -side=>'top');
    foreach my $Encode (EncodeList()) {
	my $b = $EncodeFr->Radiobutton(-text=>$Encode->{Desc},
				       -anchor=>'center',
				       -font=>$FontButton,
				       -value=>$Encode->{Name},
				       -variable=>\$EncodeSelected)
	    ->pack(-side=>'top', -anchor=>'w');
    }

    #
    # Select output destination
    #
    my $OutputFr = $fr->LabFrame(-label=>'Output destination',
				 -labelside=>'acrosstop')
	->pack(-fill=>'x',
	       -expand=>1,
	       -side=>'top');
    #
    # File output
    #
    my $FileFr = $OutputFr->Frame
	->pack(-fill=>'x',
	       -expand=>1,
	       -side=>'top');
    my $b = $FileFr->Radiobutton(-text=>$Outputs->{$RO_FILE}->{Desc},
				 -anchor=>'center',
				 -font=>$FontButton,
				 -value=>$RO_FILE,
				 -command=>\&OutputButtons,
				 -variable=>\$OutputDest)
	->pack(-side=>'left', -anchor=>'w');
    $Outputs->{$RO_FILE}->{Button} = $b;
    # File input widget for user typing in file name
    my $FileEntry = $FileFr->Entry(-width=>20,
				   -textvariable=>\$OutputFile)
	->pack(-side=>'left', -anchor=>'w');
    push(@FileInputs, $FileEntry);
    # Browse for file button
    my $b = $FileFr->Button(-text=>'Browse...',
			    -font=>$FontButton,
			    -command=>\&BrowseFile)
	->pack(-side=>'left', -anchor=>'w');
    push(@FileInputs, $b);
    # Save the list of input widgets
    $Outputs->{$RO_FILE}->{InputWidgets} = \@FileInputs;

    #
    # Printer output
    #
    my $PrinterFr = $OutputFr->Frame
	->pack(-fill=>'x',
	       -expand=>1,
	       -side=>'top');
    my $b = $PrinterFr->Radiobutton(-text=>$Outputs->{$RO_PRINTER}->{Desc},
				    -anchor=>'center',
				    -font=>$FontButton,
				    -value=>$RO_PRINTER,
				    -command=>\&OutputButtons,
				    -variable=>\$OutputDest)
	->pack(-side=>'left', -anchor=>'w');
    $Outputs->{$RO_PRINTER}->{Button} = $b;
    #
    # Print input widget for user typing in print command
    # XXX At some point we need to get a list of printers and let the user
    # select the printer, not the print command.
    #
    my $PrintCmdEntry = $PrinterFr->Entry(-width=>18,
					  -textvariable=>\$PrintCommand)
	->pack(-side=>'left', -anchor=>'w');
    push(@PrintInputs, $PrintCmdEntry);
    # Save the list of input widgets
    $Outputs->{$RO_PRINTER}->{InputWidgets} = \@PrintInputs;

    #
    # Set state of output buttons
    #
    OutputButtons();

    #
    # Button row frame
    #
    my $ButtonFr = $fr->Frame
	->pack(-side=>'bottom', -anchor=>'center', -pady=>10);
    #
    # Button row buttons
    #
    my $CancelBtn = $ButtonFr->Button(-text=>'Cancel',
				      -font=>$FontButton,
				      -command=>sub { $ReportWin->withdraw })
	->pack(-side=>'left', -anchor=>'center', -padx=>20);
    my $CreateBtn = $ButtonFr->Button(-text=>'Create',
				      -font=>$FontButton,
				      -command=>\&ReportRun)
	->pack(-side=>'left', -anchor=>'center', -padx=>20);
    my $CmdBtn = $ButtonFr->Button(-text=>'Show Command',
				      -font=>$FontButton,
				      -command=>\&ReportCommand)
	->pack(-side=>'left', -anchor=>'center', -padx=>20);
}

1;
