#
# $Id: GuiWindow.pm,v 1.8 2003/05/04 00:58:49 mcooper Exp $
#
# SysInfo UI Window functions
#

package GuiWindow;

require 5.003;
require Exporter;

use mcDebug;
use uiMisc;
use uiGlobals;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   WindowDisplay ErrorWindow
		  );
}

#
# Display Contents in a window
#
sub WindowDisplay {
    my $Args = shift;
    my $Title = $Args->{Title} || $Title;
    my $Image = $Args->{Image};
    my $NeedFixed = $Args->{NeedFixed};
    my @Contents;

    if (ref($Args->{Contents}) eq 'ARRAY') {
	@Contents = @{ $Args->{Contents} };
    } else {
	# Scalar
	my @lines = split('\n', $Args->{Contents});
	push(@Contents, @lines);
    }

    #
    # Size up window needs
    #
    my $Width = 0;
    my $Lines = 0;
    foreach my $Line (@Contents) {
	my @NewLines = split('\n', $Line);
	if ($#NewLines < 0) {
	    $Lines += 1;
	} else {
	    $Lines += $#NewLines + 1;
	}
	if ((my $l = length($Line)) > $Width) {
	    $Width = $l;
	}
    }

    my $aw = MainWindow->new(-title=>WinTitle());

    if ($Image) {
	if (my $image = $aw->Getimage($Image)) {
	    my $ImageLbl = $aw->Label(-image=>$image)
		->pack(-side=>'top', -expand=>'1', -fill=>'x');
	}
    }

    my $Fr = $aw->LabFrame(-label=>$Title, -labelside=>'acrosstop')
	->pack(-side=>'top', -anchor=>'center');

    my $Msg = $Fr->Text(-relief=>'flat',
			-font=>($NeedFixed) ? $FontFixed : $Font,
			-height=>$Lines+1,
			-width=>$Width+1)
	->pack(-side=>'top', -anchor=>'center');

    $Msg->tagConfigure('center', -justify=>'center');
    foreach my $Line (@Contents) {
	if ($Line !~ /\n/) {
	    $Line = $Line . "\n";
	}
	$Msg->insert('end', $Line);
    }

    my $CloseBtn = $aw->Button(-text=>'Close',
			       -font=>$FontButton,
			       -command=>sub { $aw->withdraw })
	->pack(-side=>'bottom', -anchor=>'center');
}

sub ErrorWindow {
    my $Args = shift;
    my $Text = $Args->{Text};

    push(@c, $Text);
    WindowDisplay({Title=>$Title . ' Error',
		   Contents=>\@c});
}

1;
