#
# $Id: mcl.pm,v 1.7 2003/04/15 18:50:38 mcooper Exp $
#
# MagniComp License functions
#

package mcl;

require 5.003;
require Exporter;

use mcDebug;
use uiMisc;
use mcSysInfo;
use mcMagniCompLic;
use uiGlobals;
use uiFormat;

#
# Keep in sync with $srcdir/include/mcdefs.h
#
my $DemoDays	= 60;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   mclShow mclCheck
		  );
}

#
# Show one line
#
sub Line {
    my $Desc = shift;
    my $Value = shift;

    if (!length($Value)) {
	return;
    }

    return sprintf("%18s: %s", $Desc, $Value);
}

#
# Get an array of mcl Lines
#
sub mclLines {
    my $License = shift;
    my @Lines;

    if (!$License) {
	return;
    }

    push(@Lines, sprintf("LICENSE INFO FOR %s", $License->SoftName));
    if ($License->Serial) {
	push(@Lines, Line("License ID", $License->Serial));
    }
    push(@Lines, Line("License Key", $License->Key));
    push(@Lines, Line("License Type", $License->Type));
    push(@Lines, Line("Number of RTUs", $License->Count));
    push(@Lines, Line("Created", $License->IssueDate));
    push(@Lines, Line("Product Name", $License->SoftName));
    push(@Lines, Line("Product Version", $License->SoftVer));
    if ($License->MinCPU || $License->MaxCPU) {
	push(@Lines, Line("Min-Max # CPUs",
			  $License->MinCPU . '-' . $License->MaxCPU));
    }
    push(@Lines, Line("License Expires", $License->ExpireDate));
    push(@Lines, Line("Licensed to Name", $License->AccountName));
    if ($License->AccountNumber) {
	push(@Lines, Line("Licensed to Acct", $License->AccountNumber));
    }
    push(@Lines, Line("Comment", $License->Comment));
    push(@Lines, Line("License Terms", $License->LicenseURL));
    push(@Lines, Line("Product HomePage", $License->HomePageURL));

    return @Lines;
}

#
# Show license $License
#
sub mclShowLicense {
    my $License = shift;

    if (!$License) {
	return;
    }

    my @Lines = mclLines($License);
    foreach my $l (@Lines) {
	printf "%s\n", $l;
    }
}

#
# Show MC License info in text format
# If ReturnInfo is true, returns info as array.  Otherwise prints to stdout.
#
sub mclShow {
    my $Args = shift;
    my $SysInfo = $Args->{SysInfo};
    my $ReturnInfo = $Args->{ReturnInfo};	# Return @Info
    my @Info;

    if (!$SysInfo) {
	Debug("mclShow: No \$SysInfo object passed to us.");
	return -1;
    }

    # Do the actual retrieve/probe system
    $Args->{Info} = 'MagniCompLic';
    if (!$SysInfo->Retrieve($Args)) {
	delete $SysInfoOpts->{Info};
	return -1;
    }
    delete $Args->{Info};

    if (!$SysInfo->MagniCompLic) {
	Debug("mclShow: No MagniCompLic object found.");
	return -1;
    }

    if (FormatIs($FT_REPORT)) {
	my $Entries = $SysInfo->MagniCompLic->EntriesReport;
	foreach my $key (keys %$Entries) {
	    Debug("mclShow: MCL Entry Key=<%s>", $key);
	    printf("%s\n", $Entries->{$key});
	}
    } else {
	my $Entries = $SysInfo->MagniCompLic->Entries;
	my $TotalRTU = 0;
	foreach my $key (keys %$Entries) {
	    Debug("mclShow: MCL Entry Key=<%s>", $key);
	    if ($ReturnInfo) {
		push(@Info, mclLines($Entries->{$key}));
	    } else {
		mclShowLicense($Entries->{$key});
	    }
	    $TotalRTU += $Entries->{$key}->Count;
	}

	if ($TotalRTU gt 0) {
	    my $t = sprintf("TOTAL Number of RTUs: %d", $TotalRTU);
	    if ($ReturnInfo) {
		push(@Info, '');
		push(@Info, $t);
	    } else {
		printf("\n%s\n", $t);
	    }
	}
    }

    if ($ReturnInfo) {
	return @Info;
    }

    return 0;
}

#
# Check to see if we're running in demo mode.
# If we are, tell the user what's going on.
# Returns hash with:
#	Status = -1	Failed, caller should exit
#              =  0	Success
#	Message = printable message
#
sub mclCheck {
    my $Args = shift;
    my $SysInfo = $Args->{SysInfo};
    my $Result;
    my @Msg;

    $Result->{Status} = 0;

    if (!$SysInfo) {
	$Result->{Message} = "mclCheck: No \$SysInfo object passed to us.";
	$Result->{Status} = -1;
	return $Result;
    }

    $SaveOpts = $Args->{SysInfoOpts};
    delete $Args->{SysInfoOpts};
    # Setup what we want
    $Args->{Info} = 'MagniCompLic';

    #
    # Some options we want to keep for the call to Retrieve
    # XXX These names must exactly match mcSysInfo supported options.
    #
    my @KeepOptions = ( 'Refresh', 'Cache', 'ExpireTime' );
    foreach my $o (@KeepOptions) {
	if (exists($SaveOpts->{$o})) {
	    $Args->{$o} = $SaveOpts->{$o};
	}
    }

    # Retrieve the license
    if (!$SysInfo->Retrieve($Args)) {
	delete $SysInfoOpts->{Info};
	$Args->{SysInfoOpts} = $SaveOpts;
	$Result->{Message} = 'Failed to retrieve MagniComp license.';
	$Result->{Status} = -1;
	return $Result;
    }
    $Args->{SysInfoOpts} = $SaveOpts;
    delete $Args->{Info};

    if (!$SysInfo->MagniCompLic) {
	$Result->{Message} = "MagniComp license object contains no license data.";
	$Result->{Status} = -1;
	return $Result;
    }

    #
    # We should only have 1 license, so pull it out and set to $mcl
    #
    my $mcobj = $SysInfo->MagniCompLic;
    my $entries = $mcobj->Entries;
    @k = keys %$entries;
    my $mcl = $entries->{$k[0]};
    if (!$mcl) {
	$Result->{Message} = "Could not obtain MagniComp license of any type.";
	$Result->{Status} = -1;
	return $Result;
    }

    Debug("mclCheck: Type=<%s> Status=<%s>",
	  $mcl->Type, $mcl->Status);

    if (Eq($mcl->Type, 'demo')) {
	push(@Msg, 'You are running a free ' . $DemoDays . '-day DEMO version of ' . $mcl->SoftName . ' on');
	push(@Msg, $mcl->HostName . '.  Please obtain a permanent license or');
	push(@Msg, 'cease using this program within ' . $DemoDays . ' days. Licenses may be ');
	push(@Msg, 'obtained from http://www.MagniComp.com/sysinfo/#getlicense');
    }

    if (Eq($mcl->Status, 'EXPIRED')) {
	push(@Msg, "");	# Blank line
	if ($mcl->ExpireDate) {
	    push(@Msg, sprintf("Your %s license expired %s.",
			       uc($mcl->Type), $mcl->ExpireDate));
	} else {
	    push(@Msg, sprintf("Your %s license has expired.",
			       uc($mcl->Type)));
	}
	# Say bye-bye
	$Result->{Status} = -1;
    } elsif (Eq($mcl->Type, 'demo') && $mcl->ExpireDate) {
	push(@Msg, "");	# Blank line
	if ($mcl->DaysLeft) {
	    push(@Msg, 'This DEMO license expires in ' . $mcl->DaysLeft . ' days on ' . $mcl->ExpireDate . '.');
	} elsif (length($mcl->DaysLeft) && $mcl->DaysLeft == 0) {
	    push(@Msg, 'This DEMO license expires TODAY (' . $mcl->ExpireDate . ').');
	}
    }

    if ($#Msg >= 0) {
	$Result->{Message} = join("\n", @Msg);
	$Result->{Messages} = \@Msg;
    }
    $Result->{Type} = $mcl->Type;

    return $Result;
}

1;
