#
# $Id: uiCliShowGeneral.pm,v 1.8 2003/04/21 20:19:59 mcooper Exp $
#
# SysInfo CLI Show General functions
#

package uiCliShowGeneral;

require 5.003;
require Exporter;

use POSIX;

use mcDebug;
use uiGlobals;
use uiClass;
use uiMisc;
use uiPresent;
use uiPresentItem;
use mcMember;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   CliShowGeneral
		  );
}

#
# CLI Show General class info
#
sub CliShowGeneral {
    my $Args = shift;
    my $SysInfo = $Args->{SysInfo};
    my $Show = $Args->{Show};
    my @PrItems;

    Debug("CLI: ShowGeneral");

    if (!$SysInfo) {
	Debug("CliShowGeneral: No SysInfo data was found.");
	return -1;
    }

    if ($Show) {
	#
	# Show one specific member info
	#
	my $member = MemberFind($Show, $SysInfo->Members);
	if (!$member) {
	    Error("Cannot locate member \"%s\".", $Show);
	    return -1;
	}
	if (!$member->Name) {
	    Error("Member object for \"%s\" does not have an object name.",
		  $Show);
	    return -1;
	}
	my $value = $SysInfo->{$member->Name};

	#
	# Setup the properties for this item.
	#
	my $PrItem = uiPresentItem->New;
	$PrItem->Desc($SysInfo->Desc($member->Name));
	$PrItem->Name($member->Name);
	$PrItem->Value($value);
	$PrItem->SysInfoObj($SysInfo);

	push(@PrItems, $PrItem);
    } else {
	#
	# Show all members for this object
	#
	my @Items = keys %$SysInfo;

	foreach my $item (sort {$SysInfo->ORDER(lc($a)) <=>
				    $SysInfo->ORDER(lc($b))} @Items) {
	    my $key = lc($item);

	    Debug("CliShowGeneral: Handle item <%s> Order=<%s> Desc=<%s>",
		  $item, $SysInfo->ORDER($key), $SysInfo->Desc($key));

	    #
	    # If Show was given, then skip over the item if it doesn't match
	    #
	    if ($Show && !Eq($Show, $item)) {
		next;
	    }

	    if (!$SysInfo->ORDER($key)) {
		next;
	    }

	    #
	    # Setup the properties for this item.
	    #
	    my $PrItem = uiPresentItem->New;
	    $PrItem->Desc($SysInfo->Desc($key));
	    $PrItem->Name($item);
	    $PrItem->Value($SysInfo->{$item});
	    $PrItem->SysInfoObj($SysInfo);

	    push(@PrItems, $PrItem);
	}
    }

    my $Present = uiPresent->New;
    my $Class = ClassGet($CL_GENERAL);
    $Present->Class($Class);

    $Present->Items(@PrItems);

    $Present->Show({Class=>$Class});
}

1;
