#
# $Id: uiCliShowLicense.pm,v 1.6 2002/07/14 19:09:59 mcooper Exp $
#
# SysInfo CLI Show License functions
#

package uiCliShowLicense;

require 5.003;
require Exporter;

use POSIX;

use mcDebug;
use uiGlobals;
use uiClass;
use uiMisc;
use uiPresent;
use uiPresentItem;
use uiPresentProp;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   CliShowLicense
		  );
}

#
# CLI Show License class info
#
sub CliShowLicense {
    my $Args = shift;
    my $SysInfo = $Args->{SysInfo};
    my $Entries = $SysInfo->Entries;

    Debug("CLI: ShowLicense");

    if (!$SysInfo) {
	Debug("CliShowLicense: No SysInfo data was found.");
	return -1;
    }

    my $Present = uiPresent->New;
    my $Class = ClassGet($CL_LICENSE);
    $Present->Class($Class);

    my @Items = keys %$Entries;
    my @PrItems;

    foreach my $item (@Items) {
	my $key = lc($item);

	my $PrItem = uiPresentItem->New;
	$PrItem->Name($item);

	#
	# Setup the properties for this item.
	#
	my $License = $Entries->{$item};

	$PrItem->SysInfoObj($License);
	if (exists($License->{"SUMMARY"})) {
	    $PrItem->Summary($License->SUMMARY);
	}

	my @PropNames = keys %$License;
	my @PrProps;
	foreach my $prop (sort {$License->ORDER(lc($a)) <=>
				    $License->ORDER(lc($b))} @PropNames) {
	    if (!$License->{$prop}) {
		next;
	    }

	    my $propkey = lc($prop);
	    my $order = $License->ORDER($propkey);
	    my $desc = $License->Desc($propkey);

	    Debug("CliShowLicense: Handle item=<%s> Prop=<%s> Order=<%s> Desc=<%s>",
		  $item, $prop, $order, $desc);

	    if (!$order) {
		next;
	    }

	    my $PrProp = uiPresentProp->New;
	    $PrProp->Order($order);
	    $PrProp->Desc($desc);
	    $PrProp->Key($propkey);
	    $PrProp->Value($License->{$prop});
	    push(@PrProps, $PrProp);
	}

	if (exists($License->{"DescList"}) &&
	    (my @DescProps = GetDescProps($License->DescList))) {
	    push(@PrProps, @DescProps);
	}

	$PrItem->Props(@PrProps);
	push(@PrItems, $PrItem);
    }

    # Sort the items into alphabetical order
    $Present->Items(sort {$a->Summary cmp $b->Summary} @PrItems);

    $Present->Show({Class=>$Class});
}

1;
