#
# $Id: uiCliShowNetIf.pm,v 1.2 2003/03/28 19:40:23 mcooper Exp $
#
# SysInfo CLI Show NetIf functions
#

package uiCliShowNetIf;

require 5.003;
require Exporter;

use POSIX;

use mcDebug;
use uiGlobals;
use uiClass;
use uiMisc;
use uiPresent;
use uiPresentItem;
use uiPresentProp;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   CliShowNetIf
		  );
}

#
# CLI Show NetIf class info
#
sub CliShowNetIf {
    my $Args = shift;
    my $SysInfo = $Args->{SysInfo};
    my $Entries = $SysInfo->Entries;

    Debug("CLI: ShowNetIf");

    if (!$SysInfo) {
	Debug("CliShowNetIf: No SysInfo data was found.");
	return -1;
    }

    my $Present = uiPresent->New;
    my $Class = ClassGet($CL_NETIF);
    $Present->Class($Class);

    my @Items = sort keys %$Entries;
    my @PrItems;

    foreach my $item (@Items) {
	my $key = lc($item);

	my $PrItem = uiPresentItem->New;
	$PrItem->Name($item);

	#
	# Setup the properties for this item.
	#
	my $NetIf = $Entries->{$item};

	$PrItem->SysInfoObj($NetIf);
	if (exists($NetIf->{"SUMMARY"})) {
	    $PrItem->Summary($NetIf->SUMMARY);
	}

	my @PropNames = keys %$NetIf;
	my @PrProps;
	foreach my $prop (sort {$NetIf->ORDER(lc($a)) <=>
				    $NetIf->ORDER(lc($b))} @PropNames) {
	    if (!$NetIf->{$prop}) {
		next;
	    }

	    my $propkey = lc($prop);
	    my $order = $NetIf->ORDER($propkey);
	    my $desc = $NetIf->Desc($propkey);

	    Debug("CliShowNetIf: Handle item=<%s> Prop=<%s> Order=<%s> Desc=<%s>",
		  $item, $prop, $order, $desc);

	    if (!$order) {
		next;
	    }

	    my $PrProp = uiPresentProp->New;
	    $PrProp->Order($order);
	    $PrProp->Desc($desc);
	    $PrProp->Key($propkey);
	    $PrProp->Value($NetIf->{$prop});
	    push(@PrProps, $PrProp);
	}

	if (exists($NetIf->{"Flags"}) &&
	    (my @DescProps = GetDescProps($NetIf->Flags))) {
	    push(@PrProps, @DescProps);
	}

	$PrItem->Props(@PrProps);
	push(@PrItems, $PrItem);
    }

    # Sort the items into alphabetical order
    $Present->Items(sort {$a->Summary cmp $b->Summary} @PrItems);

    $Present->Show({Class=>$Class});
}

1;
