#
# $Id: uiCliShowNetwork.pm,v 1.6 2003/04/29 20:53:46 mcooper Exp $
#
# SysInfo CLI Show Network functions
#

package uiCliShowNetwork;

require 5.003;
require Exporter;

use POSIX;

use mcDebug;
use uiGlobals;
use uiClass;
use uiMisc;
use uiPresent;
use uiPresentItem;
use mcMember;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   CliShowNetwork
		  );
}

#
# CLI Show Network class info
#
sub CliShowNetwork {
    my $Args = shift;
    my $Network = $Args->{SysInfo};
    my $Show = $Args->{Show};
    my @PrItems;

    Debug("CLI: ShowNetwork");

    if (!$Network) {
	Debug("CliShowNetwork: No SysInfo data was found.");
	return -1;
    }

    #
    # There is only one entry in $Network->Entries
    #
    my @Items = keys %{$Network->Entries};
    if ($#Items < 0) {
	Error("CliShowNetwork: No network information object was found.");
	return -1;
    }

    #
    # The first (and only) item should be the sole mcNetworkInfo item.
    #
    my $NetInfo = $Network->Entries->{$Items[0]};

    if ($Show) {
	#
	# Show one specific member info
	#
	my $member = MemberFind($Show, $SysInfo->Network->Members);
	if (!$member) {
	    Error("Cannot locate member \"%s\".", $Show);
	    return -1;
	}
	if (!$member->Name) {
	    Error("Member object for \"%s\" does not have an object name.",
		  $Show);
	    return -1;
	}
	my $value = $NetInfo->{$member->Name};

	#
	# Setup the properties for this item.
	#
	my $PrItem = uiPresentItem->New;
	$PrItem->Desc($NetInfo->Desc($member->Name));
	$PrItem->Name($member->Name);
	$PrItem->Value($value);
	$PrItem->SysInfoObj($NetInfo);

	push(@PrItems, $PrItem);
    } else {
	@Items = keys %$NetInfo;
	foreach my $item (sort {$NetInfo->ORDER(lc($a)) <=>
				    $NetInfo->ORDER(lc($b))} @Items) {
	    my $key = lc($item);

	    Debug("CliShowNetwork: Handle item <%s> Order=<%s> Desc=<%s>",
		  $item, $NetInfo->ORDER($key), $NetInfo->Desc($key));

	    if (!$NetInfo->ORDER($key)) {
		next;
	    }

	    #
	    # Setup the properties for this item.
	    #
	    my $PrItem = uiPresentItem->New;
	    $PrItem->Desc($NetInfo->Desc($key));
	    $PrItem->Name($item);
	    $PrItem->Value($NetInfo->{$item});
	    $PrItem->SysInfoObj($NetInfo);

	    push(@PrItems, $PrItem);
	}
    }

    my $Present = uiPresent->New;
    my $Class = ClassGet($CL_NETWORK);
    $Present->Class($Class);

    $Present->Items(@PrItems);

    $Present->Show({Class=>$Class});
}

1;
