#
# $Id: uiCliShowPartition.pm,v 1.4 2002/07/14 18:49:24 mcooper Exp $
#
# SysInfo CLI Show Partition functions
#

package uiCliShowPartition;

require 5.003;
require Exporter;

use POSIX;

use mcDebug;
use uiGlobals;
use uiClass;
use uiMisc;
use uiPresent;
use uiPresentItem;
use uiPresentProp;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   CliShowPartition
		  );
}

#
# CLI Show Partition class info
#
sub CliShowPartition {
    my $Args = shift;
    my $SysInfo = $Args->{SysInfo};
    my $Entries = $SysInfo->Entries;

    Debug("CLI: ShowPartition");

    if (!$SysInfo) {
	Debug("CliShowPartition: No SysInfo data was found.");
	return -1;
    }

    my $Present = uiPresent->New;
    my $Class = ClassGet($CL_PARTITION);
    $Present->Class($Class);

    my @Items = keys %$Entries;
    my @PrItems;

    foreach my $item (@Items) {
	my $key = lc($item);

	my $PrItem = uiPresentItem->New;
	$PrItem->Name($item);

	#
	# Setup the properties for this item.
	#
	my $Partition = $Entries->{$item};

	$PrItem->SysInfoObj($Partition);

	if (exists($Partition->{"SUMMARY"})) {
	    $PrItem->Summary($Partition->SUMMARY);
	}

	my @PropNames = keys %$Partition;
	my @PrProps;
	foreach my $prop (sort {$Partition->ORDER($a) <=>
				    $Partition->ORDER($b)} @PropNames) {
	    if (!$Partition->{$prop}) {
		next;
	    }

	    my $propkey = lc($prop);
	    my $order = $Partition->ORDER($prop);
	    my $desc = $Partition->Desc($prop);

	    Debug("CliShowPartition: Handle item=<%s> Prop=<%s> Order=<%s> Desc=<%s>",
		  $item, $prop, $order, $desc);

	    if (!$order) {
		next;
	    }

	    my $PrProp = uiPresentProp->New;
	    $PrProp->Order($order);
	    $PrProp->Desc($desc);
	    $PrProp->Key($propkey);
	    $PrProp->Value(GetValue($Partition->{$prop}));
	    push(@PrProps, $PrProp);
	}

	if (exists($Partition->{"DescList"}) &&
	    (my @DescProps = GetDescProps($Partition->DescList))) {
	    push(@PrProps, @DescProps);
	}

	$PrItem->Props(@PrProps);
	push(@PrItems, $PrItem);
    }

    # Sort the items into alphabetical order
    $Present->Items(sort {$a->Summary cmp $b->Summary} @PrItems);

    $Present->Show({Class=>$Class});
}

1;
