#
# $Id: uiCliShowPrinter.pm,v 1.4 2002/07/27 22:29:07 mcooper Exp $
#
# SysInfo CLI Show Printer functions
#

package uiCliShowPrinter;

require 5.003;
require Exporter;

use POSIX;

use mcDebug;
use uiGlobals;
use uiClass;
use uiMisc;
use uiEncode;
use uiPresent;
use uiPresentItem;
use uiPresentProp;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   CliShowPrinter
		  );
}

#
# CLI Show Printer class info
#
sub CliShowPrinter {
    my $Args = shift;
    my $SysInfo = $Args->{SysInfo};
    my $Entries = $SysInfo->Entries;

    Debug("CLI: ShowPrinter");

    if (!$SysInfo) {
	Debug("CliShowPrinter: No SysInfo data was found.");
	return -1;
    }

    my $Present = uiPresent->New;
    my $Class = ClassGet($CL_PRINTER);
    $Present->Class($Class);

    my @Items = keys %$Entries;
    my @PrItems;

    #
    # Calculate max length of Printer Name
    #
    my $MaxNameLen = 0;
    foreach my $item (@Items) {
	if (length($item) > $MaxNameLen) {
	    $MaxNameLen = length($item);
	}
    }

    foreach my $item (@Items) {
	my $key = lc($item);

	my $PrItem = uiPresentItem->New;
	$PrItem->Name($item);

	#
	# Setup the properties for this item.
	#
	my $Printer = $Entries->{$item};
	$Printer->{MaxNameLen} = $MaxNameLen;

	$PrItem->SysInfoObj($Printer);
	if (exists($Printer->{"SUMMARY"})) {
	    $PrItem->Summary($Printer->SUMMARY({Encode=>EncodeName}));
	}

	my @PropNames = keys %$Printer;
	my @PrProps;
	foreach my $prop (sort {$Printer->ORDER(lc($a)) <=>
				    $Printer->ORDER(lc($b))} @PropNames) {
	    if (!$Printer->{$prop}) {
		next;
	    }

	    my $propkey = lc($prop);
	    my $order = $Printer->ORDER($prop);
	    my $desc = $Printer->Desc($prop);

	    Debug("CliShowPrinter: Handle item=<%s> Prop=<%s> Order=<%s> Desc=<%s>",
		  $item, $prop, $order, $desc);

	    if (!$order) {
		next;
	    }

	    my $PrProp = uiPresentProp->New;
	    $PrProp->Order($order);
	    $PrProp->Desc($desc);
	    $PrProp->Key($propkey);
	    $PrProp->Value(GetValue($Printer->{$prop}));
	    push(@PrProps, $PrProp);
	}

	if (exists($Printer->{"DescList"}) &&
	    (my @DescProps = GetDescProps($Printer->DescList))) {
	    push(@PrProps, @DescProps);
	}

	$PrItem->Props(@PrProps);
	push(@PrItems, $PrItem);
    }

    # Sort the items into alphabetical order
    $Present->Items(sort {$a->Summary cmp $b->Summary} @PrItems);

    $Present->Show({Class=>$Class});
}

1;
