#
# $Id: uiCliShowSoftware.pm,v 1.18 2003/03/01 22:48:19 mcooper Exp $
#
# SysInfo CLI Show Software functions
#

package uiCliShowSoftware;

require 5.003;
require Exporter;

use POSIX;
use CGI qw(:all);

use mcDebug;
use mcGetSize;
use uiGlobals;
use uiClass;
use uiMisc;
use uiEncode;
use uiFormat;
use uiPresent;
use uiPresentItem;
use uiPresentProp;
use uiPresentHtml;
use uiText;
use uiTextTree;
use uiTextColumns;

my $AddHistory;
my $ColumnFmt = undef;
my $BorderEnable = 0;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   CliShowSoftware
		  );
}

#
# Get PresentItem properties for $SoftInfo
#
sub GetProps {
    my $SoftInfo = shift;

    #
    # Setup the properties for this item.
    #
    my @PropNames = keys %$SoftInfo;
    my @PrProps;
    foreach my $prop (sort {$SoftInfo->ORDER($a) <=>
				$SoftInfo->ORDER($b)} @PropNames) {
	my $propkey = $prop;
	my $desc = $SoftInfo->Desc($propkey);
	my $value = GetValue($SoftInfo->{$prop});

	if (!length($value)) {
	    next;
	}

	Debug("CliShowSoftware: Handle item=<%s> Prop=<%s> Desc=<%s> Value=<%s>",
	      $item, $prop, $desc, $value);
	
	if (!$desc) {
	    next;
	}

	my $PrProp = uiPresentProp->New;
	$PrProp->Desc($desc);
	$PrProp->Key($propkey);
	#
	# Special handling cases
	#
	if (Eq($prop, 'InstDate') || Eq($prop, 'BuildDate')) {
	    # If there are no / or - chars then assume it needs conversion
	    if ($value !~ /\// && $value !~ /-/) {
		$value = GetDate($value);
	    }
	} elsif (Eq($prop, 'DiskUsage')) {
	    $value = GetSizeStr($value, $BYTES);
	} elsif (EncodeIs($EN_HTML)) {
	    $value =~ s/\\n/<br>/g;
	}
	$PrProp->Value($value);
	push(@PrProps, $PrProp);
    }

    if (my @DescProps = GetDescProps($SoftInfo->DescList)) {
	push(@PrProps, @DescProps);
    }

    #
    # Add in SoftFile entries
    #
    if ($SoftInfo->FileList) {
	my $Prop = uiPresentProp->New;
	$Prop->OpaqueArray($SoftInfo->FileList);
	push(@PrProps, $Prop);
    }

    return @PrProps;
}

#
# Make a PresentItem
#
sub MakeItem {
    my $SoftInfo = shift;
    my $Class = shift;
    my $Master = shift;

    my $PrItem = uiPresentItem->New;
    $PrItem->Name($SoftInfo->Name);
    $PrItem->SysInfoObj($SoftInfo);
    if (exists($SoftInfo->{"SUMMARY"})) {
	$PrItem->Summary($SoftInfo->SUMMARY({Encode=>EncodeName}));
    }
    # Set our master (parent)
    $PrItem->Master($Master);

    if ($SoftInfo->Name) {
	#
	# Add properties
	#
	if (my @PrProps = GetProps($SoftInfo)) {
	    $PrItem->Props(@PrProps);
	}
    }

    return $PrItem;
}

#
# Show a SoftInfo object recursively.
# --RECURSE--
#
sub ShowRecurse {
    my $SoftInfo = shift;
    my $Class = shift;
    my $Level = shift;
    my $Master = shift;

    if (!$SoftInfo) {
	return;
    }

    my $pr = uiPresent->New;
    $pr->Class($Class);

    my $PrItem = $SoftInfo->{"PresentItem"} ||
	MakeItem($SoftInfo, $Class, $Master);

    if (EncodeIs($EN_TEXT)) {
	if (FormatIs($FT_TREE)) {
	    ShowTextTreeItem($pr, $Level, $PrItem, $MaxLen);
	} elsif (FormatIs($FT_COLUMNS)) {
	    if (!$ColumnFmt) {
		$ColumnFmt = ShowTextColumnsHeader({Class=>$Class});
	    }
	    ShowTextColumnsItem({Item=>$PrItem, Class=>$Class,
				 Fmt=>$ColumnFmt});
	} else {
	    Error("Format <%s> is unknown and unsupported [SW1].", FormatName);
	    exit(1);
	}
    }

    #
    # Show child nodes in alpha sorted order
    #
    if ($SoftInfo->SlaveNodes) {
	my @children = $SoftInfo->SlaveNodes;

	if (Debug) {
	    Debug("    ShowRecurse Slaves:");
	    foreach my $di (sort {$a->Name cmp $b->Name} @children) {
		Debug("\tSlave Node <%s>", $di->Name);
	    }
	}

	foreach my $child (sort {$a->Name cmp $b->Name} @children) {
	    ShowRecurse($child, $Class, $Level + 1, $SoftInfo);
	}
    }
}

#
# Show a SoftInfo object recursively as HTML
# --RECURSE--
#
sub ShowHtmlRecurse {
    my $Args = shift;
    my $SoftInfo = $Args->{SoftInfo};
    my $Class = $Args->{Class};
    my $Level = $Args->{Level};
    my $ColCount = $Args->{ColCount};
    my $Master = $Args->{Master};
    my @Rows;

    if (!$SoftInfo) {
	return;
    }

    Debug("uiCliShowSoftware->ShowHtmlRecurse: Node=<%s>",
	  $SoftInfo->Name);

    my $pr = uiPresent->New;
    $pr->Class($Class);

    my $PrItem = $SoftInfo->{"PresentItem"} ||
	MakeItem($SoftInfo, $Class, $Master);

    if (FormatIs($FT_TREE)) {
	my @r = HtmlTreeItem({Present=>$pr, Level=>$Level,
			      Item=>$PrItem});
	if ($#r >= 0) {
	    push(@Rows, @r);
	}
    } elsif (FormatIs($FT_COLUMNS)) {
	my @r = HtmlColumnItem({Item=>$PrItem, Class=>$Class});
	if ($#r >= 0) {
	    push(@Rows, @r);
	}
    } else {
	Error("Format <%s> is unknown and unsupported [SW2].", FormatName);
	exit(1);
    }

    #
    # Show child nodes
    #
    if ($SoftInfo->SlaveNodes) {
	my @children = $SoftInfo->SlaveNodes;

	if (Debug) {
	    Debug("    ShowRecurse Slaves:");
	    foreach my $di (sort {$a->Name cmp $b->Name} @children) {
		Debug("\tSlave Node <%s>", $di->Name);
	    }
	}
	
	my @ChildRows;
	foreach my $child (@children) {
	    my @rows = ShowHtmlRecurse({SoftInfo=>$child, Class=>$Class,
					Level=>$Level + 1, Master=>$SoftInfo});
	    if ($#rows >= 0) {
		push(@ChildRows, @rows);
	    }
	}
	if ($#ChildRows >= 0) {
	    if (FormatIs($FT_COLUMNS)) {
		push(@Rows, @ChildRows);
	    } else {
		push(@Rows, Tr(td({colspan=>$ColCount},
				  div({style=>"margin-left: .5in"},
				      table({border=>$BorderEnable,
					     width=>'100%',
					     cellpadding=>0,
					     cellspacing=>0},
					    @ChildRows)))));
	    }
	}
    }

    return @Rows;
}

#
# Show HTML
#
sub SoftwareShowHtml {
    my $Args = shift;
    my $Class = $Args->{Class};
    my @Nodes = @{ $Args->{Nodes} };
    my @Rows;

    if (FormatIs($FT_COLUMNS)) {
	if (my $r = HtmlColumnHeaders($Class)) {
	    push(@Rows, $r);
	}
    }

    my $ColCnt = HtmlColumnCount;

    #
    # Get the rows for each Node
    #
    foreach my $child (@Nodes) {
	my @rows = ShowHtmlRecurse({SoftInfo=>$child, Class=>$Class,
				    ColCount=>$ColCnt, Level=>0});
	if ($#rows >= 0) {
	    push(@Rows, @rows);
	}
    }

    if ($#Rows >= 0) {
	print table({border=>$BorderEnable,
		     width=>'100%',
		     cellpadding=>0,
		     cellspacing=>(FormatIs($FT_COLUMNS)) ? 3 : 0},
		    @Rows), "\n";
    }
}

#
# CLI Show Software class info
#
sub CliShowSoftware {
    my $Args = shift;
    my $SysInfo = $Args->{SysInfo};

    Debug("CLI: ShowSoftware");

    if (!$SysInfo) {
	Debug("CliShowSoftware: No SysInfo data was found.");
	return -1;
    }

    my $Class = ClassGet($CL_SOFTWARE);
    my $Present = uiPresent->New;
    $Present->Class($Class);

    $Present->StartClass;

    #
    # Set column widths if this is $FT_COLUMNS
    # A side effect of this is that we build and maintain a
    # PresentItem object for each SysInfo object.  This increases
    # memory utilization at the cost of speed.
    #
    if (EncodeIs($EN_TEXT) && FormatIs($FT_COLUMNS)) {
	my $Entries = $SysInfo->{"Entries"};
	foreach my $name (keys %$Entries) {
	    my $node = $Entries->{$name};
	    # Save the PresentItem node for later use
	    $node->{"PresentItem"} = MakeItem($node, $Class,
					      $node->MasterNode);
	    ColumnsSetWidth({Class=>$Class, Item=>$node->{"PresentItem"}});
	}
    }

    if (EncodeIs($EN_TEXT)) {
	#
	# Show each top level slave node and its children
	#
	foreach my $child (sort {$a->Name cmp $b->Name}
			   $SysInfo->Tree->SlaveNodes) {
	    ShowRecurse($child, $Class, 0, undef);
	}
    } elsif (EncodeIs($EN_HTML)) {
	my @Nodes = $SysInfo->Tree->SlaveNodes;
	SoftwareShowHtml({Nodes=>\@Nodes, Class=>$Class});
    }

    $Present->EndClass;
}

1;
