#
# $Id: uiDataTree.pm,v 1.10 2003/04/06 23:23:55 mcooper Exp $
#
# UI Data Tree functions
#

package uiDataTree;

require 5.003;

require Exporter;

use mcDebug;
use mcGetSize;
use uiMisc;
use uiGlobals;
use uiData;
use uiPath;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   DataTreeSummary DataTreeToggleProp
		  );
}

#
# Toggle on/off a Tree entry's properties.
#
sub DataTreeToggleProp {
    my $Args = shift;
    my $Tree = $Args->{Tree};
    my $Path = $Args->{Path};
    my $State = $Args->{State};

    if (!$Tree && $Path) {
	$Tree = DataGetWidget($Path);
    }

    if (!$Tree || !WidgetExists($Tree)) {
	return;
    }
    if (!PathExists($Path)) {
	return;
    }

    my $Table = PathTable($Path);

    if (!$State) {
	$State = $Table->{$Path}->{State};
	if (!$State || Eq($State, $PS_CLOSE)) {
	    $State = $PS_OPEN;
	} else {
	    $State = $PS_CLOSE;
	}
    }

    Debug("DataTreeToggleProp <%s> <%s>-><%s>", 
	  $Path, $Table->{$Path}->{State}, $State);

    foreach my $Kid ($Tree->infoChildren($Path)) {
	my $KidPath = $Kid;
	if ($KidPath !~ /$PathSep/) {
	    $KidPath = $Path . $PathSep . $Kid;
	}
	# If the kid has kids, it's not a property
	if ($Tree->infoChildren($KidPath)) {
	    next;
	}
	if (Eq($State, $PS_OPEN)) {
	    $Tree->show('entry', $KidPath);
	} else {
	    $Tree->hide('entry', $KidPath);
	}
    }

    $Table->{$Path}->{State} = $State;
}

#
# Build a summary description for Tree use
#
sub DataTreeSummary {
    my $Args = shift;
    my $Entry = $Args->{Entry};
    my $Path = $Args->{Path};
    my $Summary;

    if (!$Entry) {
	return;
    }

    my @a;
    if ($Entry =~ /mcDevInfo/) {
	# Special handling of "Total Disk" entry
	if (lc($Path) =~ /total disk/) {
	    if (length($Entry->ModelDesc)) { push(@a, $Entry->ModelDesc); }
	} else {
	    if (length($Entry->Vendor)) { push(@a, $Entry->Vendor); }
	    if (length($Entry->Model)) { push(@a, $Entry->Model); }
	    if (length($Entry->ModelDesc)) { push(@a, $Entry->ModelDesc); }
	    if (length($Entry->Capacity)) { push(@a, $Entry->Capacity); }
	    if (length($Entry->Speed)) { push(@a, $Entry->Speed); }
	    if (length($Entry->ClassType)) { push(@a, $Entry->ClassType); }
	    if (length($Entry->Type)) { push(@a, $Entry->Type); }
	}
    } elsif ($Entry =~ /mcPartInfo/) {
	if (length($Entry->Size)) { push(@a, GetSizeStr($Entry->Size, 
							$MBYTES)); }
	if (length($Entry->Type)) { push(@a, $Entry->Type); }
	if (length($Entry->UsageStatus) && 
	    !Eq($Entry->UsageStatus, $Entry->Type) &&
	    !Eq($Entry->UsageStatus, 'unknown')) {
	    push(@a, lc($Entry->UsageStatus));
	}
	if (length($Entry->MntName) && !Eq($Entry->MntName, $Entry->Type)) {
	    push(@a, $Entry->MntName);
	}
    } elsif ($Entry =~ /mcPrinterInfo/) {
	if (length($Entry->Vendor)) { push(@a, $Entry->Vendor); };
	if (length($Entry->Model)) { push(@a, $Entry->Model); };
	if (length($Entry->Descript)) { push(@a, $Entry->Descript); };
	if (length($Entry->Proto)) { push(@a, $Entry->Proto); }
	my @langs = $Entry->Langs;
	if ($#langs >= 0) {
	    push(@a, $langs[0]);
	}
	if (length($Entry->Device)) {
	    push(@a, "on");
	    push(@a, $Entry->Device);
	}
	if (length($Entry->Server)) {
	    push(@a, "on");
	    push(@a, $Entry->Server);
	}
    } elsif ($Entry =~ /mcCacheInfo/) {
	if (length($Entry->Size)) { push(@a, GetSizeStr($Entry->Size,
							$KBYTES)); }
	if (length($Entry->Type)) { push(@a, $Entry->Type); }
	push(@a, 'cache');
    } elsif ($Entry =~ /mcSoftInfo/) {
	push(@a, $Entry->Description);
    } elsif ($Path =~ /SysConf/ || $Path =~ /Kernel/) {
	if (HasMember($Entry, 'Desc') && $Entry->Desc) {
	    push(@a, $Entry->Desc);
	}
	if (HasMember($Entry, 'Value')) {
	    my $What = "=";
	    if (Eq($Entry->Value, 'TRUE') || Eq($Entry->Value, 'FALSE')) {
		$What = "is";
	    }
	    push(@a, $What);
	    push(@a, $Entry->Value);
	}
    } elsif (exists($Entry->{SUMMARY})) {
	$Summary = $Entry->SUMMARY;
    }

    if ($#a >= 0) {
	$Summary = join(' ', @a);
    }
    $Summary =~ s/  / /g;
    $Summary =~ s/^ //;
    $Summary =~ s/ $//;

    if (!$Summary) {
	if (my $Class = uiClass::ClassGet($Path)) {
	    if (length($Class->{Desc})) {
		$Summary = '- ' . $Class->{Desc};
	    }
	}
    }

    return $Summary;
}

1;
