#
# $Id: uiEncode.pm,v 1.3 2003/05/07 18:08:20 mcooper Exp $
#
# UI Encode functions
#

package uiEncode;

require 5.003;

require Exporter;

use mcDebug;
use uiMisc;
use uiGlobals;

my $Encode;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   EncodeIs EncodeName EncodeNames
		   EncodeSort EncodeSet EncodeList
		  );
}

#
# Set the current encode to be $enc
# If no arg is supplied, returns the current Encode.
#
sub EncodeSet {
    my $enc = shift;
    my $err;

    if ($enc || !$Encode) {
	if (!($Encode = $EncodeTable->{$enc})) {
	    $err = sprintf("The string \"%s\" is not a valid -encode value.",
			   $enc);
	    Debug("EncodeSet: %s\n", $err);
	    # Insure that $Encode points to something valid
	    $Encode = $EncodeTable->{$EN_TEXT};
	}
    }

    $EncodeSelected = $Encode->{Name};

    return $err;
}

#
# Sort routine for sorting $EncodeTable
#
sub EncodeSort {
    return $EncodeTable->{$a}->{Order} <=> $EncodeTable->{$b}->{Order};
}

#
# Return a list/array of valid encodes in sorted order
#
sub EncodeList {
    my @list;

    my @keys = keys %$EncodeTable;
    foreach my $key (sort EncodeSort @keys) {
	push(@list, $EncodeTable->{$key});
    }

    return @list;
}

#
# Return a list/array of valid encode names
#
sub EncodeNames {
    my @list;

    my @keys = keys %$EncodeTable;
    foreach my $key (sort EncodeSort @keys) {
	push(@list, $key);
    }

    return @list;
}

#
# Is the current encode $Name?
#
sub EncodeIs {
    my $enc = shift;
    my $Name = $enc;

    #
    # If $enc isn't a HASH, then it's a SCALAR
    #
    if (ref($enc) eq 'HASH') {
	$Name = $enc->{Name};
    }

    my $n = $Encode->{Name};
    if (Eq($n, $Name)) {
	return 1;
    } else {
	return 0;
    }
}

#
# Return the name of $enc or the current encode if none is specified.
#
sub EncodeName {
    my $enc = shift;
    my $f = $enc;

    if (!$f) {
	$f = $Encode;
    }

    return $f->{Name};
}

1;
