#
# $Id: uiKVF.pm,v 1.5 2003/04/15 20:38:22 mcooper Exp $
#
# KVF functions
#

package uiKVF;
require 5.003;
require Exporter;

use mcDebug;
use uiMisc;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   KVFread KVFsave
		  );
}

#
# Read and parse specified Key=Value file
#
sub KVFread {
    my $Args = shift;
    my $File = $Args->{File};
    my $QuietError = $Args->{QuietError};
    my $KV;

    Debug("Reading KV File <%s>", $File);

    my $FH = GetFH();
    if (!open($FH, "< $File")) {
	if (!$QuietError || $Debug) {
	    Error("%s: Open for read failed: %s", $File, $!);
	}
	return;
    }

    my $LineNo = 0;
    while (my $Line = <$FH>) {
	++$LineNo;
	chop($Line);
	$Line =~ s/\#.*//;	# Remove comments
	$Line =~ s/[ \t]//g;	# Remove whitespace
	if ($Line !~ /=/) {
	    next;
	}
	my @Fields = split('=', $Line);
	if ($#Fields != 1) {
	    Error("%s: Line %d: Wrong number of fields (expecting 2, found %d).",
		  $File, $LineNo, $#Fields+1);
	    next;
	}
	my $Key = $Fields[0];
	my $Value = $Fields[1];
	$Value =~ s/\"//g; 	# Remove quotes
	$KV->{$Key} = $Value;
    }

    close($FH);

    if ($Debug) {
	foreach my $key (sort keys %$KV) {
	    Debug("KV <%s>=<%s>", $key, $KV->{$key});
	}
    }

    return $KV;
}

#
# Save a Key=Value File
#
sub KVFsave {
    my $Args = shift;
    my $File = $Args->{File};
    my $Tmp = $File . ".tmp$$";
    my $KV = $Args->{KV};

    Debug("KVFsave: File=<%s>", $File);

    if (!$File) {
	Error("KVFsave: No file name specified.");
	return -1;
    }

    my $IFH = GetFH();
    if (!open($IFH, '<', $File)) {
	Error("%s: Open for read failed: %s", $File, $!);
	return -1;
    }
    my $TFH = GetFH();
    if (!open($TFH, '>', $Tmp)) {
	close($IFH);
	Error("%s: Open for write failed: %s", $Tmp, $!);
	return -1;
    }

    while (my $Line = <$IFH>) {
	chomp($Line);
	if ($Line =~ /^\#/) {
	    printf $TFH "%s\n", $Line;
	    next;
	}
	my @p = split('=', $Line);
	if ($#p > 0) {
	    my $key = $p[0];
	    my $value = $p[1];
	    # See if $key is one of the ones being set
	    foreach my $k (keys %$KV) {
		if (lc($key) eq lc($k)) {
		    # Set the new value
		    $value = $KV->{$k};
		    # Delete so we don't add it twice
		    delete($KV->{$k});
		    last;
		}
	    }
	    printf $TFH "%s=%s\n", $key, $value;
	} else {
	    printf $TFH "%s\n", $Line;
	}
    }

    # Write out any remaining KV's
    foreach my $k (keys %$KV) {
	printf $TFH "%s=%s\n", $k, $KV->{$k};
    }

    close($IFH);
    close($TFH);

    unlink($File);
    if (!rename($Tmp, $File)) {
	Error("Rename %s -> %s failed: %s", $Tmp, $File, $!);
	return -1;
    }

    return 0;
}

1;
