#
# $Id: uiList.pm,v 1.5 2003/04/19 23:58:02 mcooper Exp $
#
# SysInfo List options
#

package uiList;

require 5.003;
require Exporter;

use mcDebug;
use uiGlobals;
use uiClass;
use uiMisc;
use uiMsgLevel;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   List
		  );
}

my $ListTable = {
		 'show' => { Option=>'--show', Desc=>'Show keys',
			     ListFunc=>\&ListShow },
		 'type' => { Option=>'--type', Desc=>'Device type keys',
			     ListFunc=>\&ListType },
		 'class' => { Option=>'--class', Desc=>'Data class names',
			      ListFunc=>\&ListClass },
		 'encode' => { Option=>'--encode',
			       Desc=>'Output encoding types',
			       ListFunc=>\&ListEncode },
		 'format' => { Option=>'--format',
			       Desc=>'Output formating types',
			       ListFunc=>\&ListFormat },
		 'msgclass' => { Option=>'--msgclass',
				 Desc=>'Message class names',
				 ListFunc=>\&ListMsgClass },
		 'msglevel' => { Option=>'--msglevel',
				 Desc=>'Message level names',
				 ListFunc=>\&ListMsgLevel },
};

#
# Create an item entry and return it
#
sub ListMake {
    my $key = shift;
    my $desc = shift;

    return {Key=>$key, Desc=>$desc};
}

#
# Pring the items in @Listing
#
sub ListPrint {
    my @Listing = @_;

    # Calculate max width for Key
    my $max = 0;
    foreach my $item (@Listing) {
	if (length($item->{Key}) > $max) {
	    $max = length($item->{Key});
	}
    }
    $max += 2;	# Give us some room

    if (!MsgLevelIs($ML_TERSE)) {
	printf "%-*s %s\n", $max, 'VALUE', 'DESCRIPTION';
    }
    foreach my $item (@Listing) {
	printf "%-*s %s\n", $max, $item->{Key}, $item->{Desc};
    }
}

#
# List --msgclass values
#
sub ListMsgClass {
    my $Args = shift;
    my @Listing;

    foreach my $key (sort keys %$MsgClassTable) {
	my $e = $MsgClassTable->{$key};
	my $item = ListMake($e->{Name}, $e->{Desc});
	push(@Listing, $item);
    }

    ListPrint(@Listing);
}

#
# List --msglevel values
#
sub ListMsgLevel {
    my $Args = shift;
    my @Listing;

    foreach my $key (sort keys %$MsgLevelTable) {
	my $e = $MsgLevelTable->{$key};
	my $item = ListMake($e->{Name}, $e->{Desc});
	push(@Listing, $item);
    }

    ListPrint(@Listing);
}

#
# List --format values
#
sub ListFormat {
    my $Args = shift;
    my @Listing;

    foreach my $key (sort keys %$FormatTable) {
	my $e = $FormatTable->{$key};
	my $item = ListMake($e->{Name}, $e->{Desc});
	push(@Listing, $item);
    }

    ListPrint(@Listing);
}

#
# List --encode values
#
sub ListEncode {
    my $Args = shift;
    my @Listing;

    foreach my $key (sort keys %$EncodeTable) {
	my $e = $EncodeTable->{$key};
	my $item = ListMake($e->{Name}, $e->{Desc});
	push(@Listing, $item);
    }

    ListPrint(@Listing);
}

#
# List Class values
#
sub ListClass {
    my $Args = shift;
    my @Listing;

    # Add special 'all' entry
    push(@Listing, ListMake('All', 'All classes of information'));

    foreach my $class (@$Classes) {
	my $item = ListMake($class->{Name}, $class->{Desc});
	push(@Listing, $item);
    }

    ListPrint(@Listing);
}

#
# List Show values
#
sub ListShow {
    my $Args = shift;
    my $SysInfo = $Args->{SysInfo};
    my @Listing;

    foreach my $cname (keys %$SysInfo) {
	my $obj = $SysInfo->{$cname};
	#
	# Look to see if this hash member is a hash itself and if so
	# if it has a MemberNames member.
	#
	if ($obj !~ /HASH/ || !exists($obj->{Members})) {
	    next;
	}
	#
	# Get list of all member names for $obj
	#
	my $Members = $obj->Members($Args);
	if (!$Members) {
	    next;
	}

	foreach my $mname (sort keys %$Members) {
	    my $member = $Members->{$mname};
	    my $item = ListMake($member->Key, $member->Desc);
	    push(@Listing, $item);
	}
    }

    ListPrint(@Listing);
}

#
# List Type values
#
sub ListType {
    my $Args = shift;
    my $SysInfo = $Args->{SysInfo};
    my @Listing;

    my $Types = $SysInfo->Types($Args);
    foreach my $tname (sort keys %$Types) {
	my $type = $Types->{$tname};
	my $item = ListMake($type->Name, $type->Desc);
	push(@Listing, $item);
    }

    ListPrint(@Listing);
}

#
# List what's in ARGARRAY
# Return 0 on success.
# Return -1 on error.
#
sub List {
    my $Args = shift;
    my @List = @{ $Args->{ARGARRAY} };
    my @Listing;

    if ($#List < 0) {
	#
	# List what we can list
	#
	if (!MsgLevelIs($ML_TERSE)) {
	    print "The following values may be specified to the `--list' option:\n";
	}
	foreach my $item (sort keys %$ListTable) {
	    push(@Listing, ListMake($item,
				    $ListTable->{$item}->{Desc} .
				    ' for use with ' .
				    $ListTable->{$item}->{Option} ));
	}
	ListPrint(@Listing);
	return 0;
    } else {
	#
	# List the specific item
	#
	foreach my $lname (@List) {
	    my $ln = lc($lname);
	    if (!exists($ListTable->{$ln})) {
		Error("I do not know how to list anything about \"%s\".",
		      $lname);
		return -1;
	    }
	    my $Entry = $ListTable->{$ln};
	    if (!MsgLevelIs($ML_TERSE) && $Entry->{Option}) {
		printf "\nValues for use with %s:\n", $Entry->{Option};
	    }
	    $Entry->{ListFunc}($Args);
	}
    }

    return 0;
}

1;
