#
# $Id: uiMisc.pm,v 1.26 2003/04/29 20:42:01 mcooper Exp $
#
# Misc user interface functions
#

package uiMisc;

require 5.003;

require Exporter;

use POSIX;
use mcDebug;
use mcGetSize;
use uiGlobals;

my $BEversion;
my $FHnumber = 'fh0000';
$msgText = undef;
$ProgramName = undef;

$MSGT_HINT	= 'Hint';
$MSGT_NOTICE	= 'Notice';
$MSGT_ERROR	= 'Error';

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   UIversion BEversion
		   IsHash GetValue Casify HasMember
		   GetFH Eq Error UserMsg
		   GetItemValue
		   Command Status GetHostName GetDate
		   ReadFile InList ArrayDelete WinTitle
		   GuiMode CliMode SetIM DirName GetCwd
		   WidgetExists
		   Msg $msgText
		   $MSGT_HINT $MSGT_NOTICE $MSGT_ERROR
		  );
}

#
# Front end to Tk::Exists to do extra safety checks
#
sub WidgetExists {
    my $w = shift;

    # Look for an object=Hash
    if ($w && $w !~ /=HASH/) {
	Debug("WidgetExists: Widget <%s> doesn't look right.", $w);
	return undef;
    }

    return Tk::Exists($w);
}

#
# Set the current Interface Mode
#
sub SetIM {
    $CurrentIM = shift;
}

#
# Are we in GUI Mode?
#
sub GuiMode {
    if ($CurrentIM == $IM_GUI) {
	return 1;
    } else {
	return 0;
    }
}

#
# Are we in CLI Mode?
#
sub CliMode {
    if ($CurrentIM == $IM_CLI) {
	return 1;
    } else {
	return 0;
    }
}

#
# Put a message in the message widget
#
sub MsgGUI {
    my $Opts;

    if (!$msgText) {
	return 0;
    }

    if (ref($_[0]) eq 'HASH') {
	$Opts = shift;
	$msgText->tagConfigure($MSGT_HINT, -foreground=>'blue');
	$msgText->tagConfigure($MSGT_NOTICE, -foreground=>'purple');
	$msgText->tagConfigure($MSGT_ERROR, -foreground=>'red');
    }
    my $Fmt = shift;

    if ($msgText) {
	my @Msg;
	push(@Msg, 'end');
	push(@Msg, sprintf($Fmt, @_));
	if ($Opts->{Tag}) {
	    push(@Msg, $Opts->{Tag});
	}
	$msgText->insert(@Msg);
	return 1;
    }
    return 0;
}

#
# Output a message
#
sub Msg {
    if (CliMode) {
	my $fmt = shift @_;
	printf($fmt, @_);
	printf("\n");
    } else {
	MsgGUI(@_);
    }
}

#
# Return a NEW file handle
#
sub GetFH {
    return ++$FHnumber;
}

#
# Compare strings regardless of case.
#
sub Eq {
    if (!length($_[0]) || !length($_[1])) {
	return 0;
    }
    return lc($_[0]) eq lc($_[1]);
}

#
# Tell the user something
#
sub UserMsg {
    if ($Debug) {
	printf STDERR @_;
	printf STDERR "\n";
    }

    if ($MainWindow && WidgetExists($MainWindow)) {
	my $fmt = shift;
	$MainWindow->messageBox(-title=>WinTitle('Information'),
				-font=>$Font,
				-justify=>'left',
				-wraplength=>'6i',
				-message=>sprintf($fmt, @_),
				-type=>'OK',
				-default=>'OK',
				-icon=>'info');
    }
}

#
# Display an error.
#
sub Error {
    printf STDERR "%s: ", $ProgramName;
    printf STDERR @_;
    printf STDERR "\n";

    if ($MainWindow && WidgetExists($MainWindow)) {
	my $fmt = shift;
	$MainWindow->messageBox(-title=>WinTitle('Error'),
				-font=>$FontError,
				-justify=>'left',
				-wraplength=>'6i',
				-message=>sprintf($fmt, @_),
				-type=>'OK',
				-default=>'OK',
				-icon=>'error');
    }
}

#
# Display status message
#
sub Status {
    my $Fmt = shift;

    my $Msg = sprintf($Fmt, @_);

    $StatusMessage = $Msg;
    if (length($Msg)) {
	Debug($Msg);
    }
}

sub Command {
    my @Argv = @_;
    my $Cmd = join(' ', @Argv);

    Debug("Running: <%s>", $Cmd);
    my $Result = `$Cmd`;
    chop($Result);

    return $Result;
}

sub UIversion {
    my @p = split(/[ ]+/, $UIversion);
    return $p[1];
}

sub BEversion {
    if (length($BEversion)) {
	return $BEversion;
    }

    my $Str = Command($SysInfoPath, '-version', '-msglevel', 'terse');
    my @Fields = split(/[ \t]+/, $Str);
    $BEversion = $Fields[0];

    return $BEversion;
}

#
# Read input and return results in an array
#
sub ReadFile {
    my $Args = shift;
    my $FH = $Args->{FH};
    my $File = $Args->{File};

    if (!$FH && !$File) {
	Error("ReadFile: No arguments specified.");
	return;
    }

    if (!$FH) {
	$FH = GetFH();
	if (!open($FH, "< $File")) {
	    Error("%s: Open for read failed: %s", $File, $!);
	    return;
	}
    }

    if ($FH) {
	my @Contents = <$FH>;
	return @Contents;
    }
}

#
# Get our hostname
#
sub GetHostName {
    my $Args = shift;
    my $SysInfo = $Args->{SysInfo};

    if (length($MyHostName)) {
	return($MyHostName);
    }

    if (!$SysInfo) {
	$SysInfo = mcSysInfo->New;
	$SysInfo->Program($SysInfoPath);
	$SysInfo->InputFile($InFile);
    }
    # Set class list
    $SysInfo->Class('General');
    # Retrieve
    Debug("Fetching HostName...");
    $SysInfo->Retrieve();

    if ($SysInfo->General) {
	$MyHostName = $SysInfo->General->{Hostname};
    }

    return $MyHostName;
}

#
# Is $Var a hash?
#
sub IsHash {
    my $Var = shift;

    if (ref($Var) eq 'HASH' || $Var =~ /=HASH/) {
	return 1;
    } else {
	return 0;
    }
}

#
# Return scalar of value
# --RECURSE--
#
sub GetValue {
    my $Value = shift;

    if (ref($Value) eq 'ARRAY') {
	my @a;
	my @array = @{ $Value };
	for (my $i = 0; $i <= $#array; ++$i) {
	    push(@a, GetValue($array[$i]));
	}
	return join(', ', @a);
    } elsif (IsHash($Value)) {
	# Return nothing
	return;
    } else {
	$Value =~ s/^ //;
	$Value =~ s/ $//;
	return $Value;
    }
}

#
# Upper case first letter of each word in String
#
sub Casify {
    my $String = shift;

    my @a = split(/[ ]+/, $String);
    for (my $i = 0; $i <= $#a; ++$i) {
	$a[$i] = ucfirst($a[$i]);
    }

    return join(' ', @a);
}

#
# Does Obj have a member named $Member?
#
sub HasMember {
    my $Obj = shift;
    my $Member = shift;

    if (exists($Obj->{$Member})) {
	return 1;
    } else {
	return 0;
    }

#    my @a = keys %$Obj;
#    foreach my $k (@a) {
#	if ($Member eq $k) {
#	    return 1;
#	}
#    }
#
#    return 0;
}

#
# Take a time value and return a Date
#
sub GetDate {
    my $Time = shift;

    return strftime("%c", localtime($Time));
}

#
# Is $What in @List?
#
sub InList {
    my $What = shift;
    my @List = @_;

    foreach my $l (@List) {
	if (Eq($l, $What)) {
	    return 1;
	}
    }

    return 0;
}

#
# Delete $Name from @Array
#
sub ArrayDelete {
    my $Name = shift;
    my @Array = @_;

    for (my $i = 0; $i <= $#Array; ++$i) {
	if ($Name eq $Array[$i]) {
	    splice @Array, $i, 1;
	}
    }

    return @Array;
}

#
# Return the Window Title
#
sub WinTitle {
    my $Arg = shift;
    my $Args;
    my $Str;
    my $Host = $Arg->{Host};

    if (ref($Arg) eq 'HASH') {
	$Args = $Arg;
	$Str = $Args->{Str};
    } else {
	$Str = $Arg;
    }

    my $title = $Title;
    if (!$Host) {
	$Host = GetHostName($Args);
    }

    if (length($Host)) {
	$title = $title . ' for ' . $Host;
    }
    if (length($Str)) {
	$title = $title . ' - ' . $Str;
    }

    return $title;
}

#
# Get an item value
#
sub GetItemValue {
    my $key = shift;
    my $Item = shift;
    my $Header = shift;
    my $value;

    if (exists($Item->{$key})) {
	$value = GetValue($Item->{$key}) if defined($Item->{$key});
    } elsif (exists($Item->{"SysInfoObj"}) &&
	     exists($Item->{"SysInfoObj"}->{$key})) {
	$value = GetValue($Item->{"SysInfoObj"}->{$key})
	    if defined($Item->{"SysInfoObj"}->{$key});
    }

    #
    # Convert value if needed
    #
    if (defined($value) && $Header->{ConvertFrom}) {
	if (Eq($Header->{ConvertFrom}, 'MB')) {
	    $value = GetSizeStr($value, $MBYTES);
	} elsif (Eq($Header->{ConvertFrom}, 'KB')) {
	    $value = GetSizeStr($value, $KBYTES);
	} elsif (Eq($Header->{ConvertFrom}, 'bytes')) {
	    $value = GetSizeStr($value, 1);
	}
    }

    return $value;
}

#
# Return directory name from $Path
#
sub DirName {
    my $Path = shift;

    my @p = split('/', $Path);
    splice(@p, -1);

    return join('/', @p);
}

#
# Current working directory
#
sub GetCwd {
    my $dir = `pwd`;
    chomp($dir);

    return $dir;
}

1;
