#
# $Id: uiMsgLevel.pm,v 1.6 2003/05/07 18:08:20 mcooper Exp $
#
# MsgLevel functions
#

package uiMsgLevel;

require 5.003;

require Exporter;

use mcDebug;
use uiMisc;
use uiGlobals;

my $MsgLevel;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   MsgLevelIs MsgLevelName MsgLevelNames
		   MsgLevelSort MsgLevelSet MsgLevelList
		  );
}

#
# Set the current encode to be $enc
# If no arg is supplied, returns the current MsgLevel.
#
sub MsgLevelSet {
    my $enc = shift;
    my $err;

    if ($enc || !$MsgLevel) {
	if (!($MsgLevel = $MsgLevelTable->{$enc})) {
	    # Insure that $MsgLevel points to something valid
	    $err = sprintf("The string \"%s\" is not a valid -msglevel value.",
			   $enc);
	    Debug("MsgLevelSet: %s\n", $err);
	    # Default to something in case error is ignored
	    $MsgLevel = $MsgLevelTable->{$ML_GENERAL};
	}
    }

    $MsgLevelSelected = $MsgLevel->{Name};

    return $err;
}

#
# Sort routine for sorting $MsgLevelTable
#
sub MsgLevelSort {
    return $MsgLevelTable->{$a}->{Order} <=> $MsgLevelTable->{$b}->{Order};
}

#
# Return a list/array of valid levels in sorted order
#
sub MsgLevelList {
    my @list;

    my @keys = keys %$MsgLevelTable;
    foreach my $key (sort MsgLevelSort @keys) {
	push(@list, $MsgLevelTable->{$key});
    }

    return @list;
}

#
# Return a list/array of valid names
#
sub MsgLevelNames {
    my @list;

    my @keys = keys %$MsgLevelTable;
    foreach my $key (sort MsgLevelSort @keys) {
	push(@list, $key);
    }

    return @list;
}

#
# Is one of @List currently selected as the msglevel?
#
sub MsgLevelIs {
    my @List = @_;

    foreach my $name (@List) {
	if (Eq($MsgLevelSelected, $name)) {
	    return 1;
	}
    }

    return 0;
}

#
# Return the name of $enc or the current encode if none is specified.
#
sub MsgLevelName {
    my $enc = shift;
    my $f = $enc;

    if (!$f) {
	$f = $MsgLevel;
    }

    return $f->{Name};
}

1;
