#
# $Id: uiPresentHtml.pm,v 1.8 2002/07/28 18:24:19 mcooper Exp $
#
# Present TREE|COLUMNS in HTML
#

package uiPresentHtml;
require 5.003;

require Exporter;

use CGI qw(:all);
use mcGetSize;
use uiMisc;
use uiGlobals;
use uiHtml;
use uiMsgLevel;
use uiFormat;

my $RowCount = 0;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   PresentHtml HtmlTreeItem HtmlColumnHeaders HtmlColumnItem
		   HtmlColumnCount
		  );
}

#
# Printable Property string
#
sub PropPrt {
    my $s = shift;

    if (length($s)) {
	return font({size=>2}, $s);
    } else {
	return '&nbsp;';
    }
}

#
# Get partition info as TREE HTML
#
sub GetPartHtml {
    my $Present = shift;
    my $Level = shift;
    my @Partitions = @_;
    my @Rows;

    push(@Rows, Tr({bgcolor=>$Hdr1Color},
		   td({colspan=>6, align=>'center'},
		      PropPrt(b('PARTITIONS')))));
    push(@Rows, Tr({bgcolor=>$Hdr2Color},
		   th({valign=>'bottom'}, PropPrt('PART')),
		   th({valign=>'bottom'}, PropPrt('START SECTOR')),
		   th({valign=>'bottom'}, PropPrt('NUMBER OF SECTORS')),
		   th({valign=>'bottom'}, PropPrt('SIZE (MB)')),
		   th({valign=>'bottom'}, PropPrt('TYPE')),
		   th({valign=>'bottom'}, PropPrt('USAGE'))));

    my $RowCount = 0;
    foreach my $Part (@Partitions) {
	my $bgcolor = ($RowCount++ % 2 == 0) ? $Row2aColor : $Row2bColor;
	push(@Rows, Tr({bgcolor=>$bgcolor},
		       td({align=>'left'}, PropPrt($Part->Name)),
		       td({align=>'right'}, PropPrt($Part->StartSect)),
		       td({align=>'right'}, PropPrt($Part->NumSect)),
		       td({align=>'right'}, PropPrt($Part->Size)),
		       td({align=>'left'}, PropPrt($Part->Type)),
		       td({align=>'left'}, PropPrt($Part->MntName))));
    }

    return Tr(td({colspan=>3},
		 div({style=>"margin-left: .5in"},
		     table({border=>0,
			    width=>200,
			    cellpadding=>0,
			    cellspacing=>3},
			   @Rows))));
}

#
# Get software file info as TREE HTML
#
sub GetSoftFileHtml {
    my $Present = shift;
    my $Level = shift;
    my @Files = @_;
    my @Rows;

    push(@Rows, Tr({bgcolor=>$Hdr1Color},
		   td({colspan=>6, align=>'center'},
		      PropPrt(b('FILES')))));
    push(@Rows, Tr({bgcolor=>$Hdr2Color},
		   th({valign=>'bottom'}, PropPrt('Path')),
		   th({valign=>'bottom'}, PropPrt('Link To')),
		   th({valign=>'bottom'}, PropPrt('Size')),
		   th({valign=>'bottom'}, PropPrt('MD5')),
		   th({valign=>'bottom'}, PropPrt('Checksum'))));

    my $RowCount = 0;
    foreach my $sf (@Files) {
	my $bgcolor = ($RowCount++ % 2 == 0) ? $Row2aColor : $Row2bColor;
	push(@Rows, Tr({bgcolor=>$bgcolor},
		       td({align=>'left'}, PropPrt($sf->Path)),
		       td({align=>'left'}, PropPrt($sf->LinkTo)),
		       td({align=>'right', nowrap=>1},
			  PropPrt(GetSizeStr($sf->FileSize, 1))),
		       td({align=>'left'}, PropPrt($sf->MD5)),
		       td({align=>'left'}, PropPrt($sf->Checksum))));
    }

    return Tr(td({colspan=>3},
		 div({style=>"margin-left: .5in"},
		     table({border=>0,
			    width=>200,
			    cellpadding=>0,
			    cellspacing=>3},
			   @Rows))));
}

#
# Get the property entries for this $Item
#
sub GetPropRows {
    my $Args = shift;
    my $Item = $Args->{Item};
    my @PrRows;

    #
    # Do scalar properties
    #
    foreach my $prop ($Item->Props) {
	if ($prop->Opaque || $prop->OpaqueArray) {
	    # Skip for the moment
	    next;
	}
	
	my $val = GetValue($prop->Value);
	if (!length($val)) {
	    next;
	}
	
	#
	# Put together the 2 property cells
	#
	my @Cells;
	push(@Cells, td({align=>'left', valign=>'top',
			 width=>'10%', nowrap=>1},
			b(PropPrt($prop->Desc))));
	push(@Cells, td(PropPrt('&nbsp'))); # Spacer
	push(@Cells, td({align=>'left', valign=>'center',
			 width=>'90%'},
			kbd(PropPrt($val))));
	
	#
	# Add the row with alternating colors
	#
	my $bgcolor = ($RowCount++ % 2 == 0) ? $Row2aColor : $Row2bColor;
	push(@PrRows, Tr({valign=>"top", bgcolor=>$bgcolor},
			 @Cells));
    }

    #
    # Do Opaque properties now
    #
    foreach my $prop ($Item->Props) {
	if ($prop->OpaqueArray) {
	    my @a = $prop->OpaqueArray;
	    if ($a[0] =~ /mcPartInfo/) {
		my @r = GetPartHtml($Present, $Level,
				    $prop->OpaqueArray);
		if ($#r >= 0) {
		    push(@PrRows, @r);
		}
	    } elsif ($a[0] =~ /mcSoftFile/) {
		my @r = GetSoftFileHtml($Present, $Level,
					$prop->OpaqueArray);
		if ($#r >= 0) {
		    push(@PrRows, @r);
		}
	    }
	}
    }

    return @PrRows;
}

#
# Return a TREE HTML Row for $Item
#
sub HtmlTreeItem {
    my $Args = shift;
    my $Present = $Args->{Present};
    my $Level = $Args->{Level};
    my $Item = $Args->{Item};
    my $Row;
    my @Rows;

    my $bgcolor = ($RowCount++ % 2 == 0) ? $Row1aColor : $Row1bColor;

    if (my $summary = $Item->Summary) {
	$Row = Tr({valign=>"top", bgcolor=>$bgcolor},
		  td({nowrap=>1}, $summary));
    } else {
	my $value = GetValue($Item->Value);
	if (!length($value)) {
	    return;
	}

	my @Cells;
	my $Spacer = td('&nbsp;');
	push(@Cells, td({nowrap=>1}, b($Item->Desc)));

	if (MsgLevelIs($ML_ALL, $ML_CONFIG, $ML_DESC)) {
	    push(@Cells, $Spacer);
	    push(@Cells, td({nowrap=>1}, kbd($Item->Name)));
	}
	push(@Cells, $Spacer);
	push(@Cells, td({nowrap=>1}, kbd($value)));

	$Row = Tr({valign=>"top", bgcolor=>$bgcolor},
		  @Cells);
    }

    if ($Row) {
	push(@Rows, $Row);
    }

    #
    # Add Property items if requested
    #
    if (MsgLevelIs($ML_ALL, $ML_CONFIG, $ML_DESC) && $Item->Props) {
	my @PrRows = GetPropRows($Args);
	if ($#PrRows >= 0) {
	    push(@Rows, Tr({valign=>"top"},
			   td(
			      div({style=>"margin-left: 1in"},
				  table({border=>0,
					 width=>'100%',
					 cellpadding=>0,
					 cellspacing=>0},
					@PrRows)))));
	}
    }

    return @Rows;
}

#
# Return a row of headers for $Class
#
sub HtmlColumnHeaders {
    my $Class = shift;
    my @Headers = @{ $Class->{Headers} };

    if (!$Class) {
	return;
    }

    my @t;
    foreach my $hdr (@Headers) {
	if (!Eq($hdr->{Type}, 'text')) {
	    next;	# Skip non text headers
	}
	push(@t, th({valign=>'bottom'}, $hdr->{Name}));
    }

    return Tr({bgcolor=>$Hdr1Color}, @t);
}

#
# Return # of columns
#
sub HtmlColumnCount {
    my $Class = shift;
    my @Headers = @{ $Class->{Headers} };

    if (!$Class) {
	return;
    }

    my $c = 0;
    foreach my $hdr (@Headers) {
	if (!Eq($hdr->{Type}, 'text')) {
	    next;	# Skip non text headers
	}
	++$c;
    }

    return $c;
}

#
# Get a row of column item
#
sub HtmlColumnItem {
    my $Args = shift;
    my $Class = $Args->{Class};
    my $Item = $Args->{Item};
    my @Headers = @{ $Class->{Headers} };
    my $ColCnt;
    my @Rows;

    my @Cols;
    foreach my $hdr (@Headers) {
	if (!Eq($hdr->{Type}, 'text')) {
	    next;	# Skip non text headers
	}
	++$ColCnt;

	my $value;
	if (my $key = $hdr->{Key}) {
	    $value = GetItemValue($key, $Item, $hdr);
	}

	my $Opts;
	if ($hdr->{HAlign}) {
	    $Opts->{align} = $hdr->{HAlign};
	}
	push(@Cols, td($Opts, (length($value)) ? $value : '&nbsp;'));
    }

    my $bgcolor = ($RowCount++ % 2 == 0) ? $Row1aColor : $Row1bColor;
    push(@Rows, Tr({bgcolor=>$bgcolor}, @Cols));

    #
    # Add Property items if requested
    #
    if (MsgLevelIs($ML_ALL, $ML_CONFIG, $ML_DESC) && $Item->Props) {
	my @PrRows = GetPropRows($Args);
	if ($#PrRows >= 0) {
	    push(@Rows, Tr({valign=>"top"},
			   td({colspan=>$ColCnt},
			      div({style=>"margin-left: 1in"},
				  table({border=>0,
					 width=>'100%',
					 cellpadding=>0,
					 cellspacing=>0},
					@PrRows)))));
	}
    }

    return @Rows;
}

#
# Show as TREE|COLUMNS in HTML
#
sub PresentHtml {
    my $Args = shift;
    my $Present = $Args->{uiPresent};
    my $Class = $Args->{Class};
    my $Level = 0;
    my @Rows;

    if (FormatIs($FT_COLUMNS)) {
	#
	# Columns
	#
	if (my $r = HtmlColumnHeaders($Class)) {
	    push(@Rows, $r);
	}

	foreach my $item ($Present->ItemList) {
	    if (my @r = HtmlColumnItem({Class=>$Class, Item=>$item})) {
		push(@Rows, @r);
	    }
	}
    } else {
	#
	# Tree
	#
	foreach my $item ($Present->Items) {
	    if (my @r = HtmlTreeItem({Present=>$Present, Level=>$Level,
				      Item=>$item})) {
		push(@Rows, @r);
	    }
	}
    }

    if ($#Rows < 0) {
	return;
    }

    if (FormatIs($FT_COLUMNS)) {
	print table({border=>0, cellpadding=>0, cellspacing=>3},
		    @Rows), "\n";
    } else {
	print table({border=>0, cellpadding=>0, cellspacing=>0},
		    @Rows), "\n";
    }
}

1;
