#
# $Id: uiRetrieve.pm,v 1.11 2003/05/04 00:57:15 mcooper Exp $
#
# UI Retrieve functions
#

package uiRetrieve;

require 5.003;

require Exporter;

use mcDebug;
use uiMisc;
use uiGlobals;
use uiTableAdd;

$SysInfo = undef;
$UIArgs = undef;
my $LoadModulesDone = undef;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   Retrieve
		  );

}

#
# Load modules when called
#
sub LoadModules {
    if (!$LoadModulesDone && GuiMode) {
	Debug("uiRetrieve: Loading GUI modules ...");
	$LoadModulesDone = 1;

	eval 'use Tk::Dialog';

	eval 'use GuiDataDisplay';
	eval 'use GuiButtons';
	eval 'use GuiWidget';
	eval 'use GuiMeter';
    }
}

#
# Set data which we retrieved
#
sub RetrieveSet {
    my $Args = shift;
    my $SysInfo = $Args->{SysInfo};
    my $Reset = $Args->{Reset};
    my @SetList = @{ $Args->{SetList} };

    LoadModules();

    my $What = "Adding information to table . . .";
    MeterCreate({Message=>$What});
    Status($What);

    #
    # Add the retrieved data for each class to the appropriate class definetion
    # and then display the data.
    #
    for (my $i = 0; $i <= $#SetList; ++$i) {
	if ($#SetList > 0) {
	    MeterProgress({Percent=>$i/$#SetList});
	}

	my $Class = uiClass::ClassGet($SetList[$i]);

	#
	# Skip if the Entries and the Class's Table exist.
	#
	if ($Class->{Entries} && uiClass::ClassTable({Class=>$Class})) {
	    next;
	}

	#
	# Set the info retrieved by $SysInfo.
	# XXX Assume all $SysInfo classes match our $Class->{Name} names.
	#
	if (!$Class->{Entries}) {
	    $Class->{Entries} = $SysInfo->{$Class->{Name}}->Entries;
	}

	if ($Class->{Entries}) {
	    #
	    # Add the retrieved data to the Class's Table
	    #
	    TableAdd({Class=>$Class, Obj=>$Class->{Entries}});
	}

	#
	# Check to see if this class is valid or not.  If not disable it.
	#
	if (!$Class->{Entries} && $Class->{Show}) {
	    Msg("Class %s is not supported on this system.\n", $Class->{Name});
	    $Class->{Button}->configure(-state=>'disable');
	    $Class->{Show} = $Class->{ShowState} = 0;
	}

	#
	# Display the (newly added) data
	#
	DataDisplay({Class=>$Class, KeepMeter=>1, Reset=>$Reset});
    }

    ButtonStates();
    MeterDestroy();
}

#
# Retrieve sysinfo data and add it to $SysInfo->*->Entries
#
sub Retrieve {
    my $Args = shift;
    my $Reset = $Args->{Reset};
    my $si = $UIArgs->{SysInfo};
    my @RetrieveList = @{ $Args->{ClassNames} };
    my @SetList = @RetrieveList;

    LoadModules();

    if ($si && !$SysInfo) {
	# Set our private global
	$SysInfo = $si;
    }

    if (!$SysInfo) {
	Error("No \$SysInfo object passed to uiRetrieve()");
	return;
    }

    Status("Probing for system information . . . ");
    Debug("Retrieve: List=<%s>", join(',', @RetrieveList));

    #
    # Cancel any pending ->after requests
    #
    if ($AfterID) {
	Debug("Canceling afterID = <%s>", $AfterID);
	$MainWindow->afterCancel($AfterID);
	$AfterID = undef;
    }

    #
    # SysInfo setup and retrieval
    #
    if ($#RetrieveList < 0) {
	foreach my $Class (@$Classes) {
	    # If we've been told to reset all the data, then delete
	    if ($Reset) {
		if (exists($Class->{Entries})) {
		    delete($Class->{Entries});
		}
		if (exists($Class->{Table})) {
		    delete($Class->{Table});
		}
	    }
	    if ($Class->{Show} && !$Class->{Entries}) {
		push(@RetrieveList, $Class->{Name});
	    }
	}
    }

    if ($#RetrieveList < 0) {
	Msg("Nothing more to retrieve.\n");
	Status("");
	return;
    }

    #
    # See if we need to reset/delete what we know.
    # This is used by the GUI.
    #
    if ($Reset) {
	#
	# We need to reset so we're creating a new SysInfo object
	# and adding it to $UIArgs.
	#
	$UIArgs->{SysInfo} = $SysInfo = mcSysInfo->New;
	# Now we need to save what we want from the old object
	$SysInfo->Host($si->Host);
	$SysInfo->Port($si->Port);
	$SysInfo->Username($si->Username);
	$SysInfo->Password($si->Password);
	$SysInfo->Program($si->Program);
	$SysInfo->ProgramArgs($si->ProgramArgs);
	# Finally complete the deletion
	$si = $SysInfo;
	# Need to set SetList per RetrieveList as it won't be otherwise
	@SetList = @RetrieveList;
    }

    #
    # Validate @RetrieveList.
    #
    # Check to see if the requested classes are marked as being slow
    # in retrieval and if so, ask the user if they really want to proceed.
    #
    for (my $i = 0; $i <= $#RetrieveList; ++$i) {
	my $ClassName = $RetrieveList[$i];
	if (!length($ClassName)) {
	    next;
	}
	my $Class = uiClass::ClassGet($ClassName);
	if (!$Class) {
	    Debug("Retrieve: Cannot find class <%s>", $ClassName);
	    next;
	}

	#
	# If we already have the entries, there's no reason to retrieve
	# them again.
	#
	if ($Class->{Entries}) {
	    Debug("Retrieve: Class <%s> Already have mcSysInfo entries.",
		  $Class->{Name});
	    # Delete from @RetrieveList
	    @RetrieveList = ArrayDelete($ClassName, @RetrieveList);
	    next;
	}

	my $Ans;
	if ($Class->{SlowRetrieve} && !$Class->{SlowRetrieveAsked}) {
	    my $Dialog = $MainWindow->Dialog(-title=>WinTitle("Confirm slow retrieval"),
					     -text=>sprintf("Retrieval of %s information can be very slow.\n\nAre you sure you want to continue?",
							    $Class->{Name}),
					     -bitmap=>'question',
					     -buttons=>['Continue', 'Cancel'],
					     -default_button=>'Continue'
					     );
	    $Ans = $Dialog->Show();
	}

	if (Eq($Ans, 'Cancel') || $Class->{SlowRetrieveAsked}) {
	    Debug("Retrieve: Marking class <%s> as no show per user",
		  $Class->{Name});
	    # Mark it as no Show
	    $Class->{Show} = 0;
	    # Delete from Retrieve and Set lists
	    @RetrieveList = ArrayDelete($ClassName, @RetrieveList);
	    @SetList = ArrayDelete($ClassName, @SetList);
	    # Mark it
	    $Class->{SlowRetrieveAsked} = 1;
	    # Insure the current tab is raised
	    WidgetTabRaiseCurrent();
	}
    }

    #
    # Check again in case we have nothing after SlowRetrieve checks.
    #
    if ($#RetrieveList < 0 && $#SetList < 0) {
	Msg("Nothing more to retrieve.\n");
	Status("");
	return;
    }

    if ($#RetrieveList >= 0) {
	# Set class RetrieveList
	$SysInfo->Class(@RetrieveList);
	#
	# Retrieve it all
	#
	MeterCreate({Message=>'Probing system . . .'});
	$MainWindow->update;	# Update screen before (slow?) Retrieve
	$SysInfo->Retrieve();	# Do the actual retrieve/probe system

	#
	# Check to see if we got any info back.
	# Warn/disable for any classes we failed to get data for.
	#
	for (my $i = 0; $i <= $#RetrieveList; ++$i) {
	    my $ClassName = $RetrieveList[$i];
	    my $Class = uiClass::ClassGet($ClassName);
	    if (!$Class) {
		Debug("Retrieve: Cannot find class <%s>", $ClassName);
		next;
	    }

	    if ($SysInfo->{$Class->{Name}}->Entries) {
		# Looks good
		next;
	    }

	    Debug("Retrieve: Probe didn't find any info for class <%s>",
		  $Class->{Name});

	    $MainWindow->messageBox(-title=>WinTitle('Not supported'),
				    -message=>sprintf("%s information is not available for your system.  This is usually because the OS doesn't supported the required interfaces.",
						      $Class->{Name}),
				    -type=>'OK',
				    -default=>'OK',
				    -icon=>'info');

	    # Mark it as no Show
	    $Class->{Show} = 0;
	    # Delete from Set List
	    @SetList = ArrayDelete($ClassName, @SetList);
	    # Disable Tab
	    WidgetTabDisable($Class->{Name});
	    # Insure the current tab is raised
	    WidgetTabRaiseCurrent();
	}
    }

    #
    # Set what we now know
    #
    RetrieveSet({SysInfo=>$SysInfo, SetList=>\@SetList, Reset=>$Reset});

    Debug("Retrieve DONE");
}

1;
