#
# $Id: uiTableAdd.pm,v 1.19 2003/05/04 00:57:26 mcooper Exp $
#
# UI Table Add functions
#

package uiTableAdd;

require 5.003;

require Exporter;

use mcDebug;
use mcGetSize;
use uiMisc;
use uiGlobals;
use uiPath;
use uiFormat;
use uiArrange;

#
# Definetions used when displaying Key definetions
#
my $KeyDefs = [
		   {Key=>'AltName', 	Value=>'AKA Name'},
		   {Key=>'Name', 	NoShow=>1},
	      ];

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   TableAdd TableAddProps
		  );
}

#
# Get the key definetion from the table
#
sub GetKeyDef {
    my $Key = shift;
    my $Class = shift;

    my @Defs = @$KeyDefs;
    for (my $i = 0; $i <= $#Defs; ++$i) {
	if ($Defs[$i]->{Class} && !Eq($Class, $Defs[$i]->{Class})) {
	    next;
	}
	if (Eq($Key, $Defs[$i]->{Key})) {
	    return $Defs[$i];
	}
    }
}

#
# Add properties entry to the Tk Tree widget
#
sub TreeAddPropsEntry {
    my $Tree = shift;
    my $Path = shift;
    my $Str = shift;
    my $Hide = shift;

    if (!$Tree || !WidgetExists($Tree)) {
	return;
    }

    if ($Tree->infoExists($Path)) {
	Debug("TreeAddPropsEntry: Path exists <%s>", $Path);
	return;
    }

    $Tree->add($Path, -text=>$Str);
    $Tree->setmode($Path, 'none');
    if ($Hide > 0) {
	$Tree->hide('entry', $Path);
    } else {
	$Tree->show('entry', $Path);
    }
}

#
# Add properties of a hash to $NewTable
#
sub TableAddPropsHash {
    my $Args = shift;
    my $Key = $Args->{Key};
    my $Hash = $Args->{Hash};
    my $Hide = $Args->{Hide};
    my $Value = $Args->{Value};
    my $OffSet = $Args->{OffSet};
    my $Table = $Args->{Table};
    my $NewTable = $Args->{NewTable};
    my $ParentPath = $Args->{ParentPath};
    my $Tag = $Args->{Tag};

    if (!length($Value)) {
	next;
    }

    # Use the object's Desc function to lookup its description
    if (!$Tag && $Hash && exists($Hash->{'Desc'})) {
	$Tag = $Hash->Desc($Key);
    }
    # Change display tag 'DescXXXX' to 'Description'
    if (!$Tag && $Key =~ /Desc.[0-9]/) {
	$Tag = 'Description';
    }
    # Default to the key
    if (!$Tag) {
	$Tag = $Key;
    }

    #
    # Create a unique key based on $Tag and it's value
    # $Tag is used to insure alpha sorted order is used
    # later when $Tag is displayed to user.
    #
    # Path to this entry
    my $Path = $ParentPath . $PathSep . PathSafe($Tag) . PathSafe($Value);

    my $Sum = sprintf("%-*s %s",
		      $OffSet, ucfirst($Tag) . ':',
		      $Value);

##    Debug("TableAddProps:     Desc=<%s> Value=<%s> Path=<%s> HASH",
##	  $Tag, $Value, $Path);

    $NewTable->{$Path} = $Table->{$Path} = {
					    Summary=>$Sum,
					    Hide=>$Hide,
					    Desc=>$Tag,
					    Value=>$Value,
					   };
}

#
# Set a scalar value into $Table and $NewTable
#
sub SetScalar {
    my $Args = shift;
    my $Name = $Args->{Name};
    my $Entry = $Args->{Entry};
    my $Class = $Args->{Class};
    my $Hide = $Class->{Hide};
    my $ParentPath = $Args->{ParentPath};
    my $Value = $Args->{Value};
    my $OffSet = $Args->{OffSet};
    my $Table = $Args->{Table};
    my $NewTable = $Args->{NewTable};

    my $KeyDef = $Name;
    if (exists($Entry->{'Desc'})) {
	$KeyDef = $Entry->Desc($Name);
	if (!length($KeyDef)) {
	    return;
	}
    }
    if (!$KeyDef) {
	if (my $Def = GetKeyDef($Name, $Class->{Name})) {
	    if ($Def->{NoShow}) {
		return;
	    }
	    if ($Def->{Value}) {
		$KeyDef = $Def->{Value};
	    }
	}
    }

    if (!$KeyDef) {
	$KeyDef = $Name;
    }

    #
    # Create the $Path value using $KeyDef to insure alpha sorted
    # order when displayed to user.
    #
    if (!length($Value)) {
	if (Eq($KeyDef, 'Disk Usage')) {
	    $Value = GetSizeStr($Value);
	} elsif (lc($KeyDef) =~ /date/ && ($Entry->{$Name} !~ /.[\-\/]/)) {
	    $Value = GetDate($Entry->{$Name});
	} else {
	    $Value = GetValue($Entry->{$Name});
	}
    }
    if (!length($Value)) {
	return;
    }

    #
    # Include $Value in $Path to insure that we eliminate duplicate
    # values which may be added by TableAddPropsHash() as well as
    # insure that we do add entries which have the same key, but
    # different values.
    #
    my $Path = $ParentPath . $PathSep . PathSafe($KeyDef) .
	PathSafe($Value);

    my $Sum = sprintf("%-*s %s", $OffSet, $KeyDef . ':', $Value);

    #
    # $Table is the global table which tracks current valid entries.
    # $NewTable contains just this runs new additions.
    #
    $NewTable->{$Path} = $Table->{$Path} = {
					    Summary=>$Sum,
					    Hide=>$Hide,
					    Desc=>$KeyDef,
					    Value=>$Value
					   };
}

#
# Add properties of Entry to $Table
#
sub TableAddProps {
    my $Args = shift;
    my $Widget = $Args->{Widget};
    my $Entry = $Args->{Entry};
    my $ParentPath = $Args->{Path};
    my $Class = $Args->{Class};
    my $Hide = $Class->{Hide};

    if ($Entry !~ /HASH/) {
	return;
    }

    # There's no useful property info for these
    if ($Class->{NoProps}) {
	return;
    }

    my $Table = uiPath::PathTable($ParentPath);

    my @p = split($PathSep, $ParentPath);

    my $OffSet = 25;
    my $NewTable = {};	# Create empty hash so we have something to pass ref

    foreach my $Name (keys %$Entry) {
	my $Value = $Entry->{$Name};

##	Debug("TableAddProps: Name=<%s> Value=<%s> ref=<%s>",
##	      $Name, $Value, ref($Value));

	if (IsHash($Value)) {
	    #
	    # Decode each value of $Value as a hash
	    #
	    foreach my $key (keys %$Value) {
		TableAddPropsHash({Key=>$key, Hash=>$Entry,
				   Value=>$Value->{$key},
				   OffSet=>$OffSet,
				   Hide=>$Hide,
				   Table=>$Table,
				   NewTable=>$NewTable,
				   ParentPath=>$ParentPath});
	    }
	} elsif (ref($Value) eq 'ARRAY') {
	    #
	    # Decode array
	    #
	    foreach my $entry (@$Value) {
		if (ref($entry) eq 'HASH') {
		    my $key = $entry->{Key};
		    my $value = $entry->{Value};
		    TableAddPropsHash({Key=>$key, Value=>$value, Hash=>$Entry,
				       OffSet=>$OffSet,
				       Hide=>$Hide,
				       Tag=>$entry->{Tag},
				       Table=>$Table,
				       NewTable=>$NewTable,
				       ParentPath=>$ParentPath});
		} else {
		    # Scalar
		    SetScalar({Name=>$Name,
			       Table=>$Table,
			       NewTable=>$NewTable,
			       OffSet=>$OffSet,
			       Entry=>$Entry,
			       Value=>$entry,
			       Class=>$Class,
			       ParentPath=>$ParentPath});
		}
	    }
	} else {
	    #
	    # Decode scalar
	    #
	    SetScalar({Name=>$Name,
		       Table=>$Table,
		       NewTable=>$NewTable,
		       OffSet=>$OffSet,
		       Entry=>$Entry,
		       Value=>$Value,
		       Class=>$Class,
		       ParentPath=>$ParentPath});
	}
    }

    if (FormatIs($FT_TREE) && $Widget && WidgetExists($Widget)) {
	#
	# Add the entries in sorted order
	#
	foreach my $Path (sort keys %$NewTable) {
	    TreeAddPropsEntry($Widget, $Path,
			      $NewTable->{$Path}->{'Summary'},
			      $NewTable->{$Path}->{'Hide'});
	}
    }
}

#
# Build a Table by adding $Class->{Entires} or $Obj to it.
#
sub TableAdd {
    my $Args = shift;
    my $Class = $Args->{Class};
    my $Obj = $Args->{Obj};
    my $Table;

    if (GuiMode) {
	Debug("uiTableAdd: Loading GUI modules ...");
	eval 'use GuiMeter';
    }

    if (!$Class) {
	Debug("TableAdd: No class specified.");
	return;
    }

    if (!$Obj) {
	$Obj = $Class->{Entries};
    }

    #
    # Insure the Class is itself present.
    #
    $Table->{$Class->{Name}}->{Obj} = $Class->{Name};

    #
    # If this is Hardware and Arrange is by Type, then populate $Table
    # with paths by type.  i.e. "Hardware&Disk Drive".  This is required
    # to insure proper lineage for devices.
    #
    if (Eq($Class->{Name}, $CL_HARDWARE) &&
	ArrangeIs({Name=>$AR_TYPE, Class=>$Class})) {
	my $What = sprintf("Generating type paths for %s . . .",
			   $Class->{Name});
	MeterCreate({Message=>$What});
	Status($What);

	my @List = sort keys %{ $Obj };
	for (my $i = 0; $i <= $#List; ++$i) {
	    if ($#List > 0) {
		MeterProgress({Percent=>$i/$#List});
	    }

	    my $type;
	    my $Path = PathGetByType({Name=>$List[$i],
				      Class=>$Class,
				      TypePtr=>\$type,
				      Obj=>$Obj});
	    if (!$Table->{$Path}) {
		Debug("TableAdd: Creating Table Entry for type <%s>=<%s>",
		      $Path, $type);
		$Table->{$Path}->{Obj} = $type;
	    }
	}
    }

    my $What = sprintf("Adding table data for %s . . .", $Class->{Name});
    MeterCreate({Message=>$What});
    Status($What);

    #
    # Build a hash table ($Table) with all the pathnames
    #
    my @List = sort keys %{ $Obj };
    for (my $i = 0; $i <= $#List; ++$i) {
	if ($#List > 0) {
	    MeterProgress({Percent=>$i/$#List});
	}

	my $Name = $List[$i];
	my $Entry = $Obj->{$Name};
	my $Path;
	#
	# If this entry has a single entry (i.e. mcGeneral or mcNetwork)
	# then build $Path based on the class name only.
	#
	if ($Class->{SingleEntry}) {
	    $Path = PathSafe($Class->{Name});
	} else {
	    $Path = PathGet({Entry=>$Entry, Obj=>$Obj, Name=>$Name,
			     Top=>$Class->{Name}});
	}
	Debug("TableAdd: Adding <%s>=<%s>", $Path, $Entry);
	$Table->{$Path}->{Obj} = $Entry;

	if ($Entry =~ /mcDevInfo/ && $Entry->Partitions) {
	    #
	    # Add partitions as child devices
	    #
	    my @List = $Entry->Partitions;
	    foreach my $Part (@List) {
		my $p = join($PathSep, $Path, $Part->Name);
		Debug("TableAdd: Adding <%s>=<%s>", $p, $Part);
		$Table->{$p}->{Obj} = $Part;
	    }
	} elsif ($Entry && $Entry =~ /mcDevInfo/ &&
		 $Entry->DevSpec =~ /mcCpuInfo/ &&
		 $Entry->DevSpec->Caches) {
	    #
	    # Add each cache as a child device
	    #
	    my @List = $Entry->DevSpec->Caches;
	    foreach my $Cache (@List) {
		my $p = join($PathSep, $Path, $Cache->Name);
		Debug("TableAdd: Adding <%s>=<%s>", $p, $Cache);
		$Table->{$p}->{Obj} = $Cache;
	    }
	}
    }

    # Save it
    uiClass::ClassTable({Class=>$Class, Set=>$Table});

    Status("");
}

1;
