#
# $Id: uiTextProps.pm,v 1.5 2003/05/07 18:08:21 mcooper Exp $
#
# Text Properties functions
#

package uiTextProps;
require 5.003;

require Exporter;

use mcDebug;
use mcGetSize;
use uiMisc;
use uiGlobals;
use uiMsgLevel;
use uiText;
use uiCliMsg;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   ShowPropDesc ShowTextProps
		  );
}

#
# Show a property description
#
sub ShowPropDesc {
    my $Desc = shift;
    my $Level = shift;
    my $MaxLen = shift;

    TextIndent($Level);
    CliMsg("        %-*s: ", $MaxLen, $Desc);
}

#
# Show partition info as TREE TEXT
#
sub ShowPartTextTree {
    my $Present = shift;
    my $Level = shift;
    my @Partitions = @_;

    CliMsg("\n");

    TextIndent($Level+2);
    CliMsg("%36s\n", BannerText("PARTITIONS"));

    TextIndent($Level+2);
    CliMsg("%8s %10s %10s %9s\n",
	   "", "START", "NUMBER OF", "SIZE");
    TextIndent($Level+2);
    CliMsg("%8s %10s %10s %9s %8.8s %s\n",
	   "PART", "SECTOR", "SECTORS", "(MB)", "TYPE", "USAGE");

    foreach my $Part (@Partitions) {
	TextIndent($Level+2);
	CliMsg("%8s %10d %10d %9d %8.8s %s\n",
	       $Part->Name, $Part->StartSect, $Part->NumSect,
	       $Part->Size, $Part->Type, $Part->MntName);
    }
}

#
# Show SoftFile info as TREE TEXT
#
sub ShowSoftFileTextTree {
    my $Present = shift;
    my $Level = shift;
    my $MaxLen = shift;
    my @List = @_;

    ShowPropDesc('Files', $Level, $MaxLen);
    CliMsg("\\\n");

    foreach my $sf (@List) {
	TextIndent($Level+3);
	CliMsg("%s\n", $sf->SUMMARY);
    }
}

#
# Show properties for TEXT encoding
#
sub ShowTextProps {
    my $Args = shift;
    my $item = $Args->{Item};
    my $Level = $Args->{Level};
    my $Present = $Args->{Present};
    my $Class = $Args->{Class};

    if (!$Class && $Present) {
	$Class = $Present->Class;
    }

    #
    # Show the details
    #
    my @Props = $item->Props;

    #
    # Add Master name
    #
    if ($item->Master && $item->Master->Name) {
	my $prop = uiPresentProp->New;
	$prop->Desc("Connected to");
	$prop->Value($item->Master->Name);
	push(@Props, $prop);
    }

    #
    # Add children names
    #
    if ($item->Slaves) {
	my @ChildNames;
	my @Children = $item->Slaves;
	for (my $child = @Children[0]; $child; $child = $child->Next) {
	    push(@ChildNames, $child->Name);
	}
	my $prop = uiPresentProp->New;
	$prop->Desc("Attached");
	$prop->Value(join(' ', @ChildNames));
	push(@Props, $prop);
    }

    # Calculate max length of descriptions
    my $MaxLen = 0;
    foreach my $prop (@Props) {
	if (length($prop->Desc) > $MaxLen) {
	    $MaxLen = length($prop->Desc);
	}
    }

    #
    # Do all the generic scalar properties
    #
    foreach my $prop (@Props) {
	if ($prop->Opaque || $prop->OpaqueArray) {
	    # Skip for the moment
	    next;
	}
	if (!length($prop->Value)) {
	    next;
	}
	ShowPropDesc($prop->Desc, $Level, $MaxLen);
	#
	# Allow the Value to contain printf() controls
	# so things like mcSoftInfo->DescVerbose can have \n embedded.
	#
	my $val = $prop->Value;
	if ($val =~ /\\n/) {
	    CliMsg("\\\n");
	}
	$val =~ s/\\n/\n/g;
	CliMsg($val);
	CliMsg("\n");
    }

    #
    # Do Opaque properties now
    #
    foreach my $prop (@Props) {
	if ($prop->OpaqueArray) {
	    my @a = $prop->OpaqueArray;
	    if ($a[0] =~ /mcPartInfo/) {
		ShowPartTextTree($Present, $Level, $prop->OpaqueArray);
	    } elsif ($a[0] =~ /mcSoftFile/) {
		ShowSoftFileTextTree(Present, $Level, $MaxLen,
				     $prop->OpaqueArray);
	    }
	}
    }

    if (!MsgLevelIs($ML_GENERAL) && $Class &&
	Eq($Class->{"Name"}, $CL_HARDWARE)) {
	CliMsg("\n");	# Extra newline for spacing
    }
}

1;
