/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.5 $";
#endif

/*
 * Misc
 */
#if defined(HAVE_AUTOCONFIG_H)
# include "mcsysinfo-autoconfig.h"
#endif	/* HAVE_AUTOCONFIG_H */

#include "mcbase.h"
#include <unistd.h>
#include <sys/types.h>
#include <sys/param.h>
#include <dirent.h>
#if HAVE_STRING_H
# include <string.h>
#endif

/*
 * Directory functions
 */
extern char **DirGetNames(char *	Dir,
			  int		WithPath,
			  char **	ErrPtr)
{
    DIR *		DH;
    struct dirent *	dp;
    static char		Path[MAXPATHLEN];
    static char		File[MAXPATHLEN];
    static char		ErrMsg[8192];
    char *		FileName;
    char **		List = NULL;
    int			ListMax = 0;
    int			ListCnt = 0;

    DH = opendir(Dir);
    if (!DH) {
	if (ErrPtr) {
	    snprintf(ErrMsg, sizeof(ErrMsg),
		     "%s: Open directory failed: %s", Dir, SYSERR);
	    *ErrPtr = ErrMsg;
	}
	return (char **) NULL;
    }

    while (dp = readdir(DH)) {
	/* Skip all files beginning with '.' */
	if (dp->d_name[0] == '.')
	    continue;

	snprintf(File, sizeof(File), "%.*s", dp->d_reclen, dp->d_name);

#define DIR_ALLOC_BLOCK		32
	if (ListMax == 0) {
	    /* Alloc first chunk */
	    ListMax = DIR_ALLOC_BLOCK;
	    List = (char **) xmalloc(ListMax * sizeof(char *));
	} else if (ListCnt+1 >= ListMax) {
	    /* Time to alloc more space */
	    ListMax += DIR_ALLOC_BLOCK;
	    List = (char **) xrealloc(List, ListMax * sizeof(char *));
	}

	if (WithPath) {
	    snprintf(Path, sizeof(Path), "%s/%s", Dir, File);
	    FileName = Path;
	} else {
	    FileName = File;
	}

	List[ListCnt++] = strdup(FileName);
	List[ListCnt] = NULL;
    }

    closedir(DH);

    return (ListCnt > 0) ? List : (char **) NULL;
}

/*
 * Free DirList
 */
extern void DirFreeNames(char **DirList)
{
    char **		cpp;

    for (cpp = DirList; cpp && *cpp; ++cpp)
	(void) free(*cpp);

    if (DirList)
	(void) free(DirList);
}
