/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.3 $";
#endif

/*
 * Description functions
 */

#include <stdio.h>
#include "defs.h"

/*
 * Old -> New function mapping:
 *	OLD		NEW
 *	PrimeDesc	DescPrime
 *	AddDesc		DescAdd
 *	AddDevDesc	AddDevDesc
 *	DevDescFind	DescFind
 *	DevDescAppend	DescAppend
 */

/*
 * Get the prime description of a device.
 */
extern char *DescPrime(List)
     Desc_t		       *List;
{
    register Desc_t	       *Desc;

    if (!List)
	return((char *) NULL);

    for (Desc = List; Desc; Desc = Desc->Next)
	if (FLAGS_ON(Desc->Flags, DA_PRIME))
	    return(Desc->Desc);

    return((char *) NULL);
}

/*
 * Add a description to a Desc_t list
 */
extern int DescAdd(DescList, Label, Desc, Action)
    Desc_t		      **DescList;
    char		       *Label;
    char		       *Desc;
    int				Action;
{
    Desc_t		       *Ptr;
    Desc_t		       *New;

    if (!DescList || !Desc)
	return -1;

    /*
     * Check for duplicates
     */
    for (Ptr = *DescList; Ptr; Ptr = Ptr->Next)
	if (EQ(Ptr->Desc, Desc) && EQ(Ptr->Label, Label))
	    return -1;

    /*
     * Create new dev description
     */
    New = (Desc_t *) xcalloc(1, sizeof(Desc_t));
    New->Desc = strdup(Desc);
    if (Label)
	New->Label = strdup(Label);
    if (Action & DA_PRIME)
	New->Flags |= DA_PRIME;

    /*
     * Add to list
     */
    if (*DescList) {
	if (Action & DA_INSERT) {
	    New->Next = *DescList;
	    *DescList = New;
	} else {
	    for (Ptr = *DescList; Ptr && Ptr->Next; Ptr = Ptr->Next);
	    Ptr->Next = New;
	}
    } else
	*DescList = New;

    return 0;
}

/*
 * Add a device description to a device.
 */
extern int AddDevDesc(DevInfo, Desc, Label, Action)
    DevInfo_t		       *DevInfo;
    char		       *Desc;
    char		       *Label;
    int				Action;
{
    /*
     * This is for backwards compat given the large number of calls
     */
    return AddDesc(&(DevInfo->DescList), Label, Desc, Action);
}

/*
 * Find Key in the Desc of DevInfo and return Value
 */
extern char *DescFind(List, Key)
     Desc_t		       *List;
     char		       *Key;
{
    Desc_t		       *lp;

    if (!List || !Key)
	return (char *) NULL;

    for (lp = List; lp; lp = lp->Next) {
	if (EQ(lp->Label, Key))
	    return lp->Desc;
    }

    return (char *) NULL;
}

/*
 * Append New to List
 */
extern int DescAppend(New, List)
     Desc_t		       *New;
     Desc_t		       *List;
{
    Desc_t		       *dd;

    for (dd = List; dd && dd->Next; dd = dd->Next);

    dd->Next = New;

    return 0;
}
