/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.6 $";
#endif

/*
 * Disk Usage functions
 */

#include "defs.h"
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>

#define DU_MAX_DEPTH		1024

/*
 * --RECURSE--
 * Return disk usage (in bytes) for DirName and all it's contents
 */
extern Large_t DiskUsage(char *DirName, int Depth)
{
    Large_t			Amt = 0;
    DIR *			Dir;
    struct dirent *		dp;
    struct stat			StatBuf;
    char			Name[MAXPATHLEN];
    char			Path[MAXPATHLEN];

    if (!DirName)
	return 0;

    McDebug("DiskUsage: Scanning <%s> Depth=%d", DirName, Depth);

    /* Safety check */
    if (Depth >= DU_MAX_DEPTH) {
	McDebug("DiskUsage: Max Depth (%d) reached", DU_MAX_DEPTH);
	return 0;
    }

    Dir = opendir(DirName);
    if (!Dir) {
	McDebug("DiskUsage: %s: opendir failed: %s", DirName, SYSERR);
	return (Large_t) 0;
    }

    while (dp = readdir(Dir)) {
#if defined(HAVE_STRUCT_DIRENT_NAMLEN)
	snprintf(Name, sizeof(Name), "%.*s", dp->d_namlen, dp->d_name);
#else
	snprintf(Name, sizeof(Name), "%s", dp->d_name);
#endif

	if (eq(Name, ".") || eq(Name, ".."))
	    continue;

	snprintf(Path, sizeof(Path), "%s/%s", DirName, Name);
	if (lstat(Path, &StatBuf) != 0) {
	    McDebug("DiskUsage: %s: stat failed: %s", Path, SYSERR);
	    continue;
	}

	if (S_ISREG(StatBuf.st_mode)) {
	    Amt += (Large_t) StatBuf.st_size;	/* st_size is in bytes */
	} else if (S_ISDIR(StatBuf.st_mode)) {
	    Amt += (Large_t) StatBuf.st_blocks / (Large_t) StatBuf.st_blksize;
	    Amt += DiskUsage(Path, Depth+1);	/* Descend */
	}
    }

    closedir(Dir);

#if 0
    McDebug("DiskUsage: DU <%s> is %llu", DirName, Amt);
#endif

    return Amt;
}
