/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.29 $";
#endif

/*
 * This file contains the primary documented API for SysInfo
 */

#include "defs.h"
#include "netif.h"
#include "part.h"
#include <sys/errno.h>

/*
 * Global options
 */
McsiOpt_t		       *McsiOpts = NULL;
char			       *McProgramPath = NULL;
char			       *McProgramName = NULL;

typedef struct {
    int			Cmd;		/* MCSI_* value */
    int		      (*Func)();	/* Function to call */
} CmdSwitch_t;

CmdSwitch_t CmdSwitch[] = {
    { MCSI_HOSTNAME,		GetHostName },
    { MCSI_HOSTADDRS,		GetHostAddrs },
    { MCSI_HOSTALIASES,		GetHostAliases },
    { MCSI_HOSTID,		GetHostID },
    { MCSI_APPARCH,		GetAppArch },
    { MCSI_KERNARCH,		GetKernArch },
    { MCSI_KERNBITS,		GetKernBits },
    { MCSI_CPUTYPE,		GetCpuType },
    { MCSI_CPUSPEED,		GetCpuSpeed },
    { MCSI_CPUINFO,		GetCpuInfo },
    { MCSI_NUMCPU,		GetNumCpu },
    { MCSI_KERNVER,		GetKernVer },
    { MCSI_MAN,			GetMan },
    { MCSI_MANSHORT,		GetManShort },
    { MCSI_MANLONG,		GetManLong },
    { MCSI_OSDIST,		GetOSDist },
    { MCSI_OSNAME,		GetOSName },
    { MCSI_OSVER,		GetOSVer },
    { MCSI_SERIAL,		GetSerial },
    { MCSI_ROMVER,		GetRomVer },
    { MCSI_MODEL,		GetModel },
    { MCSI_BOOTTIME,		GetBootTime },
    { MCSI_CURRENTTIME,		GetCurrentTime },
    { MCSI_PHYSMEM,		GetMemory },
    { MCSI_VIRTMEM,		GetVirtMem },
    { MCSI_DEVTREE,		DeviceInfoMCSI },
    { MCSI_KERNELVAR,		KernelVarsMCSI },
    { MCSI_LIBCNAME,		GetLibcName },
    { MCSI_LIBCVER,		GetLibcVer },
    { MCSI_SYSCONF,		GetSysConf },
    { MCSI_SOFTINFO,		SoftInfoMCSI },
    { MCSI_LICENSE,		LicenseMCSI },
    { MCSI_PARTITION,		PartInfoMCSI },
    { MCSI_PRINTER,		PrinterMCSI },
    { MCSI_OSDISTNAME,		GetOSDistName },
    { MCSI_OSDISTVER,		GetOSDistVer },
    { MCSI_OSDISTNAMESH,	GetOSDistNameSh },
    { MCSI_NUMCPUVIRT,		GetNumCpuVirt },
    { MCSI_NETIF,		NetIfMCSI },
    { MCSI_NETWORK,		NetworkMCSI },
    { 0 }
};

/*
 * mcSysInfo() is the exclusive exported API call.
 */
extern int mcSysInfo(MCSIquery_t * Query)
{
    CmdSwitch_t       	       *Cmds;
    static int			CFparseCalled;
    char		       *cp;

    if (!Query) {
	errno = ENXIO;
	return -1;
    }

    switch (Query->Op) {
    case MCSIOP_PROGRAM:	/* Obsolete by MCSIOP_SETOPTS */
	if (Query->InSize && Query->In) {
	    McProgramName = (char *) Query->In;
	    return 0;
	}
	return -1;
	break;
    case MCSIOP_SETOPTS:
	if (Query->InSize && Query->In) {
	    if (sizeof(McsiOpt_t) != Query->InSize) {
		SImsg(SIM_GERR, "mcSysInfo(): McsiOpt_t size mismatch.");
		return -1;
	    }
	    McsiOpts = xcalloc(1, Query->InSize);
	    memcpy(McsiOpts, Query->In, Query->InSize);

	    /*
	     * Set McProgram{Name,Path}
	     */
	    if (McsiOpts->ProgramPath)
		McProgramPath = McsiOpts->ProgramPath;
	    if (McsiOpts->ProgramName)
		McProgramName = McsiOpts->ProgramName;
	    if (!McsiOpts->ProgramName && McsiOpts->ProgramPath) {
		McProgramName = McsiOpts->ProgramPath;
		if (cp = strrchr(McProgramName, '/'))
		    McProgramName = cp + 1;
	    }
	    if (McProgramName)
		SImsgSetup(McProgramName);
	    if (FLAGS_ON(McsiOpts->Flags, MCSIOPT_DEBUG))
		MsgClassFlags = SIM_DBG|SIM_ALL;

	    return 0;
	}
	return -1;
	break;
    default:
	/*
	 * Parse config files if we haven't yet
	 */
	if (!CFparseCalled && McsiOpts && McsiOpts->ConfigDir) {
	    CFparseCalled = TRUE;

	    /*
	     * CFparse() is called here and not higher up so that
	     * mcSysInfo C API callers don't need to call this explicitly.
	     * In other words, this makes it transparent.
	     */
	    if (CFparse(McsiOpts->ConfigFile, McsiOpts->ConfigDir) != 0) {
		SImsg(SIM_CERR,
		      "Errors occured while parsing config file(s).");
	    }
	}

	if (!CFparseCalled)
	    SImsg(SIM_DBG,
		  "WARNING: Configuration files have not yet been read.");

	for (Cmds = CmdSwitch; Cmds && Cmds->Cmd; ++Cmds) {
	    if (Cmds->Cmd == Query->Cmd)
		return Cmds->Func(Query);
	}
    }

    SImsg(SIM_GERR, "No such mcSysInfo() command as %d", Query->Cmd);
    errno = ENOENT;

    return -1;
}
