/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.70 $";
#endif

/*
 * Miscellaneous functions
 */

#include "defs.h"
#include "file.h"
#if HAVE_STRING_H
# include <string.h>
#endif
#if HAVE_STRINGS_H
# include <strings.h>
#endif
#if HAVE_STDLIB_H
# include <stdlib.h>
#endif
#include <time.h>
#include <sys/stat.h>

/*
 * Report a fatal out of memory error
 */
extern void MemFatal(Msg)
     char		       *Msg;
{
    SImsg(SIM_GERR, "%s", Msg);
    exit(1);
}

/*
 * Convert integer to ASCII
 */
char *itoa(Num)
    u_long 			Num;
{
    static char 		Buf[64];

    (void) snprintf(Buf, sizeof(Buf),  "%d", Num);

    return(Buf);
}

/*
 * Convert long to ASCII
 */
char *ltoa(Num)
     long			Num;
{
    static char 		Buf[128];

    (void) snprintf(Buf, sizeof(Buf),  "%ld", Num);

    return(Buf);
}

/*
 * Convert Large_t to ASCII
 */
char *Ltoa(Num)
    Large_t 			Num;
{
    static char 		Buf[128];

#if	defined(HAVE_PRINTF_LL)
    (void) snprintf(Buf, sizeof(Buf),  "%lld", Num);
#else
    (void) snprintf(Buf, sizeof(Buf),  "%ld", Num);
#endif	/* HAVE_PRINTF_LL */

    return(Buf);
}

/*
 * Convert ASCII to Large_t
 */
Large_t atoL(String)
     char		       *String;
{
#if	defined(HAVE_STRTOLL)
    /* Only call strtoll() if Large_t is truely equal to long long */
    if (sizeof(Large_t) == sizeof(long long))
	return (Large_t) strtoll(String, NULL, 0);
    else
	return (Large_t) strtol(String, NULL, 0);
#else	/* !HAVE_STRTOLL */
    return (Large_t) strtol(String, NULL, 0);
#endif	/* HAVE_STRTOLL */
}

/*
 * Convert integer to string version in hex
 */
char *itoax(Num)
    u_long 			Num;
{
    static char 		Buff[64];

    (void) snprintf(Buff, sizeof(Buff),  "0x%x", Num);

    return(Buff);
}

/*
 * Clean "String" by removing leading and trailing whitespace.
 * Returns pointer to new (allocated) copy.
 *
 * Flags:
 * 	CSF_BSWAP	Change "BADC" to "ABCD".
 */
extern char *CleanString(String, StrLen, Flags)
     char		       *String;
     int			StrLen;
     int			Flags;
{
    register char	       *cp;
    register char	       *bp;
    register char	       *end;
    register int		Len;
    register int		i;
    char		       *Buffer;
    char			LastChar = CNULL;
    char			c;
    int				Swap = TRUE;

    if (!String || !StrLen)
	return((char *) NULL);

    /* Skip over any leading white space */
    for (cp = String; cp && *cp && isspace(*cp); ++cp);
    Len = StrLen - (cp - String);

    /* Copy to working Buffer */
    Buffer = (char *) xcalloc(1, Len + 1);
    for (bp = Buffer, i = 0; i < Len; ++i, ++cp) {
	if (FLAGS_ON(Flags, CSF_BSWAP)) {
	    /*
	     * Byte Swap every other pass through.
	     */
	    if (Swap) {
		c = *cp;
		*cp = *(cp+1);
		*(cp+1) = c;
		Swap = FALSE;
	    } else 
		Swap = TRUE;
	}

	if (FLAGS_ON(Flags, CSF_ENDNONPR)) {
	    /*
	     * If this is a NonPrintable char, end
	     */
	    if (!isprint(*cp)) {
		break;
	    }
	} else if (FLAGS_ON(Flags, CSF_ALPHANUM)) {
	    /*
	     * If this is a not an Alpha Numeric, end
	     */
	    if (!isalnum(*cp)) {
		break;
	    }
	} else {
	    /*
	     * Skip if this is SPACE and last char was SPACE or this char
	     * is not printable
	     */
	    if ((LastChar && isspace(*cp) && isspace(LastChar)) || 
		!isprint(*cp)) {
		LastChar = *cp;
		continue;
	    }
	}
	*bp++ = *cp;
	*bp = CNULL;
	LastChar = *cp;
    }
    
    /*
     * Remove all trailing spaces.
     * bp points to end of Buffer at this point.
     */
    end = bp;
    for (cp = end - 1; cp > Buffer && isspace(*cp); --cp);
    if (cp != end - 1 && cp && isspace(*++cp))
	*cp = CNULL;

    if (strlen(Buffer) == 0)
	return (char *) NULL;

    /* skip over any remaining leading white space */
    for (cp = Buffer; cp && cp < end && isspace(*cp); ++cp);

    return(cp);
}

/*
 * Expand a key string into something more easily read.
 */
extern char *ExpandKey(String)
    char		       *String;
{
    static char			Buff[BUFSIZ];
    register int		Len;
    register char	       *cp;

    if (!String)
	return((char *)NULL);

    Len = strlen(String);
    (void) strncpy(Buff, String, (Len < sizeof(Buff)) ? Len : sizeof(Buff));
    Buff[Len] = CNULL;

    /* Capitolize first letter */
    Buff[0] = toupper(Buff[0]);

    for (cp = &Buff[0]; cp && *cp; ++cp) {
	if (*cp == '-' || *cp == '_' || *cp == ',') {
	    *cp++ = ' ';
	    if (*cp)
		*cp = toupper(*cp);
	} else if (cp != &Buff[0])
	    /* Lower case all but the first char */
	    *cp = tolower(*cp);
    }

    /* Remove ending '?' */
    Len = strlen(Buff);
    if (Buff[Len-1] == '?')
	Buff[Len-1] = CNULL;

    return(Buff);
}

/*
 * Front end to llseek() since not all platforms have one.
 */
extern Offset_t myllseek(fd, Offset, Flags)
     int			fd;
     Offset_t			Offset;
     int			Flags;
{
#if	defined(OS_LLSEEK)
    /* Using OS_LLSEEK */
    return OS_LLSEEK(fd, Offset, Flags);
#elif	defined(HAVE_LSEEK64)
    return lseek64(fd, Offset, Flags);
#elif	defined(HAVE_LLSEEK)
    return llseek(fd, Offset, Flags);
#else
    /* Using lseek */
    return lseek(fd, Offset, Flags);
#endif
}

/*
 * Convert TimeVal into a time string.
 * Fmt is a format string suitable for use by strftime(3).
 */
extern char *TimeStr(TimeVal, Fmt)
     time_t		        TimeVal;
     char		       *Fmt;
{
    struct tm		       *Tm;
    static char			Buff[128];

    if (!Fmt)
	return (char *) NULL;

    if (Tm = localtime(&TimeVal))
	if (strftime(Buff, sizeof(Buff), Fmt, Tm))
	    return Buff;

    return (char *) NULL;
}

/*
 * Look for the first existing member of Paths.
 */
extern char *FindCommand(Paths)
     char		      **Paths;
{
    char		      **cpp;
    struct stat			Stat;

    for (cpp = Paths; cpp && *cpp; ++cpp)
	if (FileExists(*cpp, 0))
	    return *cpp;

    return (char *) NULL;
}

/*
 * Copy bytes from Dst to Src.
 * Care is taken to not overrun the Dst buffer size (DstLen)
 */
extern int copybytes(Src, SrcLen, Dst, DstLen)
     void		       *Src;
     size_t			SrcLen;
     void		       *Dst;
     size_t			DstLen;
{
    if (SrcLen != DstLen)
	SImsg(SIM_DBG, "copybytes: SrcLen != DstLen: SrcLen=%d DstLen=%d", 
	      SrcLen, DstLen);

    if (memcpy(Dst, Src, (SrcLen < DstLen) ? SrcLen : DstLen))
	return 0;
    else
	return -1;
}

/*
 * Print contents of File to DBG
 */
extern int ShowFileDebug(File)
     char		       *File;
{
    FILE		       *fd;
    static char			Buff[1024];
    char *			cp;

    fd = fopen(File, "r");
    if (!fd) {
	SImsg(SIM_DBG, "CatFile: %s: Open for read failed: %s", File, SYSERR);
	return -1;
    }

    while (fgets(Buff, sizeof(Buff), fd)) {
	if (cp = strchr(Buff, '\n'))
	    *cp = CNULL;
	SImsg(SIM_DBG, "%s", Buff);
    }

    fclose(fd);

    return 0;
}

#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

/*
 * Convert an IP Address string (128.125.1.2) to a hostname.
 */
extern char *AddrStrToHost(char *Address)
{
    struct hostent *		hp;
    static struct in_addr	Addr;

    if (!inet_aton(Address, &Addr)) {
	McDebug("AddrStrToHost: Conversion of <%s> to in_addr failed: %s",
		Address, SYSERR);
	return (char *) NULL;
    }

    hp = gethostbyaddr((char *)&Addr, sizeof(struct in_addr), AF_INET);
    if (!hp) {
	McDebug("AddrStrToHost: Lookup of address <%s> failed: %s",
		Address, SYSERR);
	return (char *) NULL;
    }

    return hp->h_name;
}

/*
 * Convert a hostname to an IP address string
 */
extern char *HostStrToAddr(char *Hostname)
{
    struct hostent *		hp;
    struct in_addr		InAddr;

    if (!Hostname)
	return (char *) NULL;

    hp = gethostbyname(Hostname);
    if (!hp) {
	McDebug("HostStrToAddr: Lookup of host <%s> failed: %s",
		Hostname, SYSERR);
	return (char *) NULL;
    }

    if (hp->h_addrtype == AF_INET)
	/* Convert the value to dot format and return it */
	return inet_ntoa(*(struct in_addr *) hp->h_addr);
    else {
	McDebug("HostStrToAddr: addrtype 0x%X for <%s> is unknown.",
		hp->h_addrtype, Hostname);
	return (char *) NULL;
    }
}

/*
 * Free each member of an array
 */
extern int ArrayFree(char **List)
{
    char **			cpp;

    if (!List)
	return -1;

    for (cpp = List; cpp && *cpp; ++cpp)
	(void) free(*cpp);

    return 0;
}
