/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.17 $";
#endif

/*
 * Miscellaneous functions
 */

#include "defs.h"
#include <sys/stat.h>

/*
 * Output file
 */
char *				OutputFile = NULL;
static FILE *			OutputFH;
static char *			ProgramName;

/*
 * Do setup for SImsg()
 */
extern int SImsgSetup(Program)
     char		       *Program;
{
    uid_t			uid = 0;
    uid_t			euid = 0;

    if (Program)
	ProgramName = Program;
    if (!ProgramName)
	ProgramName = McProgramName;

    if (!OutputFH) {
	if (OutputFile) {
	    /*
	     * If we're setuid, then drop privs temporarily so we don't
	     * allow the user to write files they normally can't
	     */
	    uid = getuid();
	    euid = geteuid();
	    if (uid != euid)
		seteuid(uid);

	    OutputFH = fopen(OutputFile, "w");

	    /*
	     * Raise privs back to previous level
	     */
	    if (uid != euid)
		setuid(euid);

	    if (!OutputFH) {
		fprintf(stderr, "%s%s%s: Open for writing failed: %s\n",
			(ProgramName) ? ProgramName : "", 
			(ProgramName) ? ": " : "", 
			OutputFile, SYSERR);
		return -1;
	    }
	} else {
	    OutputFH = stdout;
	}
    }

    return 0;
}

/*
 * End SImsg calls
 */
extern int SImsgEnd()
{
    if (OutputFH && OutputFH != stdout)
	fclose(OutputFH);

    return 0;
}

/*
 * SysInfo Message functions
 *
 * SIM_INFO messages will include \n (newline) when needed by calling
 * function.  All other SIM_* will have newline provided by SImsg().
 */
#if	ARG_TYPE == ARG_STDARG
#include <stdarg.h>
/*
 * StdArg version of SImsg()
 */
extern void vSImsg(int MsgType, char *fmt, va_list args)
{
    if (MsgType == 0 || MsgClassFlags == 0)
	/* No output */
	return;

    if (SImsgSetup(NULL) < 0)
	return;

    if (FLAGS_ON(MsgType, SIM_INFO) && FLAGS_ON(MsgClassFlags, SIM_INFO)) {
	/* Normal output */
	(void) vfprintf(OutputFH, fmt, args);
    } else if (FLAGS_ON(MsgType, SIM_WARN) &&
	       FLAGS_ON(MsgClassFlags, SIM_WARN)) {
	(void) fflush(OutputFH);
	if (FLAGS_OFF(MsgType, SIM_NOLBL))
	    (void) fprintf(stderr, "%s: WARNING: ", ProgramName);
	(void) vfprintf(stderr, fmt, args);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    } else if (FLAGS_ON(MsgType, SIM_UNKN) &&
	       FLAGS_ON(MsgClassFlags, SIM_UNKN)) {
	(void) fflush(OutputFH);
	if (FLAGS_OFF(MsgType, SIM_NOLBL))
	    (void) fprintf(stderr, "%s: UNKNOWN: ", ProgramName);
	(void) vfprintf(stderr, fmt, args);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    } else if (FLAGS_ON(MsgType, SIM_GERR) &&
	       FLAGS_ON(MsgClassFlags, SIM_GERR)) {
	(void) fflush(OutputFH);
	if (FLAGS_OFF(MsgType, SIM_NOLBL))
	    (void) fprintf(stderr, "%s: GERROR: ", ProgramName);
	(void) vfprintf(stderr, fmt, args);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    } else if (FLAGS_ON(MsgType, SIM_CERR) &&
	       FLAGS_ON(MsgClassFlags, SIM_CERR)) {
	(void) fflush(OutputFH);
	if (FLAGS_OFF(MsgType, SIM_NOLBL))
	    (void) fprintf(stderr, "%s: CERROR: ", ProgramName);
	(void) vfprintf(stderr, fmt, args);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    } else if (FLAGS_ON(MsgType, SIM_DBG) &&
	       FLAGS_ON(MsgClassFlags, SIM_DBG)) {
	(void) fflush(OutputFH);
	(void) vfprintf(stderr, fmt, args);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    } else if (FLAGS_ON(MsgType, SIM_HINT) &&
	       FLAGS_ON(MsgClassFlags, SIM_HINT)) {
	(void) fflush(OutputFH);
	(void) vfprintf(stderr, fmt, args);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    }
}

/*
 * StdArg version of SImsg()
 */
extern void SImsg(int MsgType, char *fmt, ...)
{
    va_list 			args;

    va_start(args, fmt);
    vSImsg(MsgType, fmt, args);
    va_end(args);
}

/*
 * StdArg version of Debug()
 */
extern void McDebug(char *fmt, ...)
{
    va_list 			args;

    va_start(args, fmt);
    vSImsg(SIM_DBG, fmt, args);
    va_end(args);
}
#endif	/* ARG_STDARG */
#if	ARG_TYPE == ARG_VARARG
#include <varargs.h>
/*
 * Varargs version of SImsg()
 */
extern void vSImsg(MsgType, fmt, args)
     int			MsgType;
     char		       *fmt;
     va_list 			args;
{
    if (MsgType == 0 || MsgClassFlags == 0)
	/* No output */
	return;

    if (SImsgSetup(NULL) < 0)
	return;

    if (FLAGS_ON(MsgType, SIM_INFO) && 
	FLAGS_ON(MsgClassFlags, SIM_INFO)) {
	(void) vfprintf(OutputFH, fmt, args);
    } else if (FLAGS_ON(MsgType, SIM_WARN) &&
	       FLAGS_ON(MsgClassFlags, SIM_WARN)) {
	(void) fflush(OutputFH);
	if (FLAGS_OFF(MsgType, SIM_NOLBL))
	    (void) fprintf(stderr, "%s: WARNING: ", ProgramName);
	(void) vfprintf(stderr, fmt, args);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    } else if (FLAGS_ON(MsgType, SIM_UNKN) &&
	       FLAGS_ON(MsgClassFlags, SIM_UNKN)) {
	(void) fflush(OutputFH);
	if (FLAGS_OFF(MsgType, SIM_NOLBL))
	    (void) fprintf(stderr, "%s: UNKNOWN: ", ProgramName);
	(void) vfprintf(stderr, fmt, args);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    } else if (FLAGS_ON(MsgType, SIM_GERR) &&
	       FLAGS_ON(MsgClassFlags, SIM_GERR)) {
	(void) fflush(OutputFH);
	if (FLAGS_OFF(MsgType, SIM_NOLBL))
	    (void) fprintf(stderr, "%s: GERROR: ", ProgramName);
	(void) vfprintf(stderr, fmt, args);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    } else if (FLAGS_ON(MsgType, SIM_CERR) && 
	       FLAGS_ON(MsgClassFlags, SIM_CERR)) {
	(void) fflush(OutputFH);
	if (FLAGS_OFF(MsgType, SIM_NOLBL))
	    (void) fprintf(stderr, "%s: CERROR: ", ProgramName);
	(void) vfprintf(stderr, fmt, args);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    } else if (FLAGS_ON(MsgType, SIM_DBG) &&
	       FLAGS_ON(MsgClassFlags, SIM_DBG)) {
	(void) fflush(OutputFH);
	(void) vfprintf(stderr, fmt, args);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    } else if (FLAGS_ON(MsgType, SIM_HINT) &&
	       FLAGS_ON(MsgClassFlags, SIM_HINT)) {
	(void) fflush(OutputFH);
	(void) vfprintf(stderr, fmt, args);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    }
}

/*
 * Varargs version of SImsg()
 */
extern void SImsg(MsgType, fmt, va_alist)
     int			MsgType;
     char		       *fmt;
     va_dcl
{
    va_list 			args;

    va_start(args);
    vSImsg(MsgType, fmt, args);
    va_end(args);
}

/*
 * Varargs version of Debug()
 */
extern void McDebug(fmt, va_alist)
     char		       *fmt;
     va_dcl
{
    va_list 			args;

    va_start(args);
    vSImsg(SIM_DBG, fmt, args);
    va_end(args);
}
#endif	/* ARG_VARARG */
#if	!defined(ARG_TYPE) || ARG_TYPE == ARG_NONE
/*
 * Non-Varargs version of SImsg()
 */
extern void SImsg(MsgType, fmt, a1, a2, a3, a4, a5, a6)
     int			MsgType;
     char 		       *fmt;
{
    if (MsgType == 0 || MsgClassFlags == 0)
	/* No output */
	return;

    if (SImsgSetup(NULL) < 0)
	return;

    if (FLAGS_ON(MsgType, SIM_INFO) &&
	FLAGS_ON(MsgClassFlags, SIM_INFO)) {
	(void) vfprintf(OutputFH, fmt, a1, a2, a3, a4, a5, a6);
    } else if (FLAGS_ON(MsgType, SIM_WARN) &&
	       FLAGS_ON(MsgClassFlags, SIM_WARN)) {
	(void) fflush(OutputFH);
	if (FLAGS_OFF(MsgType, SIM_NOLBL))
	    (void) fprintf(stderr, "%s: WARNING: ", ProgramName);
	(void) vfprintf(stderr, fmt, a1, a2, a3, a4, a5, a6);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    } else if (FLAGS_ON(MsgType, SIM_UNKN) &&
	       FLAGS_ON(MsgClassFlags, SIM_UNKN)) {
	(void) fflush(OutputFH);
	if (FLAGS_OFF(MsgType, SIM_NOLBL))
	    (void) fprintf(stderr, "%s: UNKNOWN: ", ProgramName);
	(void) vfprintf(stderr, fmt, a1, a2, a3, a4, a5, a6);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    } else if (FLAGS_ON(MsgType, SIM_GERR) &&
	       FLAGS_ON(MsgClassFlags, SIM_GERR)) {
	(void) fflush(OutputFH);
	if (FLAGS_OFF(MsgType, SIM_NOLBL))
	    (void) fprintf(stderr, "%s: GERROR: ", ProgramName);
	(void) vfprintf(stderr, fmt, a1, a2, a3, a4, a5, a6);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    } else if (FLAGS_ON(MsgType, SIM_CERR) &&
	       FLAGS_ON(MsgClassFlags, SIM_CERR)) {
	(void) fflush(OutputFH);
	if (FLAGS_OFF(MsgType, SIM_NOLBL))
	    (void) fprintf(stderr, "%s: CERROR: ", ProgramName);
	(void) vfprintf(stderr, fmt, a1, a2, a3, a4, a5, a6);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    } else if (FLAGS_ON(MsgType, SIM_DBG) &&
	       FLAGS_ON(MsgClassFlags, SIM_DBG)) {
	(void) fflush(OutputFH);
	(void) vfprintf(stderr, fmt, a1, a2, a3, a4, a5, a6);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    } else if (FLAGS_ON(MsgType, SIM_HINT) &&
	       FLAGS_ON(MsgClassFlags, SIM_HINT)) {
	(void) fflush(OutputFH);
	(void) vfprintf(stderr, fmt, a1, a2, a3, a4, a5, a6);
	if (FLAGS_OFF(MsgType, SIM_NONL))
	    (void) fprintf(stderr, "\n");
    }
}

/*
 * Non-Varargs version of Debug()
 */
extern void McDebug(fmt, a1, a2, a3, a4, a5, a6)
     char 		       *fmt;
{
    SImsg(SIM_DBG, fmt, a1, a2, a3, a4, a5, a6);
}
#endif 	/* !ARG_TYPE */
