/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.6 $";
#endif

/*
 * Device Post functions
 */

#include "defs.h"
#include "bjhashtab.h"

static htab_t *			PseudoTab;
static htab_t *			PseudoBarronTab;

/*
 * Change a device (DevInfo) by removing it from it's OldMaster and 
 * add it to NewMaster.
 */
static void ChangeMaster(DevInfo, OldMaster, NewMaster, Why)
     DevInfo_t *		DevInfo;
     DevInfo_t *		OldMaster;
     DevInfo_t *		NewMaster;
     char *			Why;
{
    DevInfo_t *			Last;
    DevInfo_t *			dp;

    if (!DevInfo || !OldMaster || !NewMaster)
	return;

    SImsg(SIM_DBG,
	  "ChangeMaster: Change master of <%s> from <%s> to <%s> %s",
	  PRTS(DevInfo->Name), PRTS(OldMaster->Name), PRTS(NewMaster->Name),
	  (Why) ? Why : "");

    /*
     * Remove DevInfo from its current Master's Slaves list
     */
    for (dp = OldMaster->Slaves, Last = NULL; dp; dp = dp->Next) {
	if (dp == DevInfo) {
	    if (Last)
		Last->Next = dp->Next;		/* Splice cut */
	    else
		OldMaster->Slaves = dp->Next;	/* Splice cut */
	    break;
	}
	Last = dp;
    }
    
    /* Add this device to its new Master's Slaves list */
    for (dp = NewMaster->Slaves; dp && dp->Next; dp = dp->Next);
    dp->Next = DevInfo;

    /* Change the device's forward sibling to NULL */
    DevInfo->Next = NULL;

    /* Update info in DevInfo to point at NewMaster */
    DevInfo->Master = NewMaster;
    DevInfo->MasterName = NewMaster->Name;
}

/*
 * Find all Pseudo intermediate devices and add to PseudoTab for later deletion
 * -RECURSE-
 */
static void PseudoInterFind(DevInfo, TreePtr, SearchNames)
     DevInfo_t *		DevInfo;
     DevInfo_t **		TreePtr;
     char **			SearchNames;
{
    DevInfo_t *			Master;
    DevInfo_t *			NewMaster;
    static char			key[128];

    if (!DevInfo || !TreePtr)
	return;

    SImsg(SIM_DBG, "PseudoInterFind: Check <%s>", PRTS(DevInfo->Name));

    if (DevInfo && (Master = DevInfo->Master)) {
	/*
	 * Eliminate pseudo devices between a device (DevInfo) and it's
	 * real Master.
	 */
	if (Master && Master->Type == DT_PSEUDO) {
	    if (!PseudoTab)
		PseudoTab = bjhcreate(16);
	    /*
	     * Use the addr of DevInfo as the key to insure uniqueness.
	     * This is required since pseudo devices don't have their
	     * unit # as part of the device name so the device name is
	     * not always unique.
	     */
	    snprintf(key, sizeof(key), "0x%X", DevInfo);
	    SImsg(SIM_DBG, "PseudoInterFind: Add <%s> Key=<%s>",
		  PRTS(DevInfo->Name), key);
	    bjhadd(PseudoTab, (ub1 *)strdup(key), strlen(key), DevInfo);
	}
    }

    if (DevInfo->Slaves) {
	SImsg(SIM_DBG, "PseudoInterFind: Descend slave to <%s>",
	      PRTS(DevInfo->Slaves->Name));
	PseudoInterFind(DevInfo->Slaves, TreePtr, SearchNames);
    }
    if (DevInfo->Next) {
	SImsg(SIM_DBG, "PseudoInterFind: Descend next to <%s>",
	      PRTS(DevInfo->Next->Name));
	PseudoInterFind(DevInfo->Next, TreePtr, SearchNames);
    }
}

/*
 * Process deletions of pseudo intermediates in PseudoTab
 * Returns a DevInfo_t for the new master.
 * -RECURSE-
 */
static DevInfo_t *PseudoTreeRemove(DevInfo, Master, TreePtr, SearchNames)
     DevInfo_t *		DevInfo;
     DevInfo_t *		Master;
     DevInfo_t **		TreePtr;
     char **			SearchNames;
{
    DevInfo_t *			NewMaster;
    DevInfo_t *			OldMaster = NULL;

    SImsg(SIM_DBG, "PseudoTreeRemove: Delete <%s>",
	  PRTS(Master->Name));

    if (NewMaster = GetMaster(Master, TreePtr, SearchNames)) {
	if (NewMaster->Type == DT_PSEUDO) {
	    /* 
	     * Remove the new master 
	     * -RECURSE-
	     */
	    OldMaster = NewMaster;	/* Save */
	    NewMaster = PseudoTreeRemove(NewMaster, NewMaster->Master,
					 TreePtr, SearchNames);
	    SImsg(SIM_DBG,
		  "PseudoTreeRemove: New master <%s> is pseudo; using <%s> instead",
		  PRTS(OldMaster->Name), PRTS(NewMaster->Name));

	    if (!NewMaster) {
		SImsg(SIM_DBG,
		      "PseudoTreeRemove: Recurse called returned no master.");
		return (DevInfo_t *) NULL;
	    }
	}
	    
	/* Delete DevInfo from Master and add it to NewMaster */
	ChangeMaster(DevInfo, 			/* Device to change */
		     Master,			/* Old Master */
		     NewMaster,			/* New Master */
		     "eliminate pseudo master"	/* Reason why */
		     );

	return NewMaster;
    } else {
	SImsg(SIM_DBG, "PseudoTreeRemove: Couldn't find master for <%s>",
	      PRTS(Master->Name));
	return (DevInfo_t *) NULL;
    }
}

/*
 * Process deletions of pseudo intermediates in PseudoTab
 */
static void PseudoDelete(TreePtr, SearchNames)
     DevInfo_t **		TreePtr;
     char **			SearchNames;
{
    DevInfo_t *			DevInfo;
    DevInfo_t *			Master;

    if (!TreePtr)
	return;

    if (!bjhfirst(PseudoTab)) {
	SImsg(SIM_DBG, "PseudoDelete: bjhfirst failed");
	return;
    }

    do {
	DevInfo = (DevInfo_t *) bjhstuff(PseudoTab);
	if (!DevInfo)
	    continue;
	Master = DevInfo->Master;
	if (!Master)
	    continue;

	/* Remove Master from TreePtr */
	PseudoTreeRemove(DevInfo, Master, TreePtr, SearchNames);
    } while (bjhnext(PseudoTab));

    /* We're done with the table */
    bjhdestroy(PseudoTab);
    PseudoTab = NULL;
}

/*
 * Find all abandonded/barron devices and add each to PseudoBarronTab
 * -RECURSE-
 */
static void PseudoCleanFind(DevInfo)
     DevInfo_t *		DevInfo;
{
    static char			key[128];

    SImsg(SIM_DBG, "PseudoClean: Check <%s>",
	  PRTS(DevInfo->Name));

    if (DevInfo && !DevInfo->Slaves && DevInfo->Type == DT_PSEUDO) {
	if (!PseudoBarronTab)
	    PseudoBarronTab = bjhcreate(8);

	snprintf(key, sizeof(key), "0x%X", DevInfo);

	SImsg(SIM_DBG, "PseudoCleanFind: Add <%s> Key=<%s>",
	      PRTS(DevInfo->Name), key);

	bjhadd(PseudoBarronTab, (ub1 *)strdup(key), strlen(key), DevInfo);
    }

    if (DevInfo->Slaves)
	PseudoCleanFind(DevInfo->Slaves);
    if (DevInfo->Next)
	PseudoCleanFind(DevInfo->Next);
}

/*
 * Cleanup abandoned pseudo devices
 */
static void PseudoClean(TreePtr)
     DevInfo_t **		TreePtr;
{
    DevInfo_t *			DevInfo;

    SImsg(SIM_DBG,
	  "PseudoClean: Searching for and eliminating barron pseudo devices");

    /* 
     * Traverse all of TreePtr looking for abandonded/barron pseudo devices.
     * Each one is added to PseudoBarronTab.
     */
    PseudoCleanFind(*TreePtr);

    /* 
     * Dispatch with the list 
     */

    /* Position */
    if (!bjhfirst(PseudoBarronTab)) {
	SImsg(SIM_DBG, "PseudoClean: bjhfirst failed");
	return;
    }

    /* Iterate and dispatch */
    do {
	DevInfo = (DevInfo_t *) bjhstuff(PseudoBarronTab);
	if (!DevInfo)
	    continue;

	SImsg(SIM_DBG, "PseudoClean: Remove barron <%s>",
	      PRTS(DevInfo->Name));
	DeviceRemoveLinks(DevInfo, DevInfo->Master);	/* Bye-bye */
	/* XXX We still need to actually delete Master */
    } while (bjhnext(PseudoBarronTab));

    /* We're done with the table */
    bjhdestroy(PseudoBarronTab);
    PseudoBarronTab = NULL;
}

/*
 * Post process TreePtr.
 * This should be called once TreePtr is complete.
 */
extern void DevPostProcess(TreePtr, SearchNames)
     DevInfo_t **		TreePtr;
     char **			SearchNames;
{
    if (!TreePtr)
	return;

#if	defined(ELIMINATE_PSEUDO_INTERMEDIATES)
    SImsg(SIM_DBG, "DevPostProcess: Finding pseudo intermediate devices");
    PseudoInterFind(*TreePtr, TreePtr, SearchNames);
    SImsg(SIM_DBG, "DevPostProcess: Deleting pseudo intermediate devices");
    PseudoDelete(TreePtr, SearchNames);
    SImsg(SIM_DBG, "DevPostProcess: Elimination of pseudo intermediates complete");
    PseudoClean(TreePtr);
    SImsg(SIM_DBG, "DevPostProcess: Cleaning pseudo intermediates complete");
#endif	/* ELIMINATE_PSEUDO_INTERMEDIATES */

}
