/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.7 $";
#endif

/*
 * Common SysInfo Disk functions
 */

#include "defs.h"

/*
 * Return the Type of DiskDrive_t entry from DevInfo
 */
extern DiskDrive_t *DiskDriveGet(DevInfo, Type)
     DevInfo_t		       *DevInfo;
     int			Type;
{
    DiskDriveData_t	       *DiskData;

    if (!DevInfo || !DevInfo->DevSpec)
	return (DiskDrive_t *) NULL;

    DiskData = (DiskDriveData_t *) DevInfo->DevSpec;
    switch (Type) {
    case DISKDRIVE_HW:
	return DiskData->HWdata;
    case DISKDRIVE_OS:
	return DiskData->OSdata;
    default:
	McDebug("DiskDriveGet: Type %d is not defined.", Type);
    }

    return (DiskDrive_t *) NULL;
}

/*
 * Add Disk to DevInfo
 */
extern int DiskDriveAdd(DevInfo, Type, Disk)
     DevInfo_t		       *DevInfo;
     int			Type;
     DiskDrive_t	       *Disk;
{
    DiskDriveData_t	       *DiskData;

    if (!DevInfo || !Disk)
	return -1;

    if (DevInfo->DevSpec)
	DiskData = (DiskDriveData_t *) DevInfo->DevSpec;
    else {
	DiskData = NewDiskDriveData();
	DevInfo->DevSpec = (Opaque_t) DiskData;		/* Add it */
    }

    switch (Type) {
    case DISKDRIVE_HW:
	DiskData->HWdata = Disk;
	break;
    case DISKDRIVE_OS:
	DiskData->OSdata = Disk;
	break;
    default:
	McDebug("DiskDriveAdd: Type %d is not defined.", Type);
    }

    return 0;
}

/*
 * Do setup for *Decode() funcs which need a DiskDrive_t to work on
 */
extern DiskDrive_t *DiskSetup(DevInfo, What)
     DevInfo_t		       *DevInfo;
     char		       *What;
{
    DiskDriveData_t	       *DiskDriveData;

    if (!DevInfo || !What)
	return((DiskDrive_t *) NULL);

    /*
     * We may want to add other DevType's as we discover them, but
     * right now we only know how to deal with DiskDrive's.
     */
    if (DevInfo->Type != DT_DISKDRIVE) {
	SImsg(SIM_DBG, "%s: %s unsupported for DevType %d",
	      DevInfo->Name, What, DevInfo->Type);
	return((DiskDrive_t *) NULL);
    }

    /*
     * Find or create the DiskDriveData and DiskDrive we need
     */
    if (DevInfo->DevSpec)
	DiskDriveData = (DiskDriveData_t *) DevInfo->DevSpec;
    else {
	DiskDriveData = NewDiskDriveData(NULL);
	DevInfo->DevSpec = (void *) DiskDriveData;
    }
    if (!DiskDriveData->HWdata)
	DiskDriveData->HWdata = NewDiskDrive(NULL);

    return(DiskDriveData->HWdata);
}
