/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.45 $";
#endif

/*
 * Type Info
 */
#include "defs.h"

/*
 * Device Type information
 */
DevType_t DevTypes[] = {
    /* Allow "none" for entries which should be ignored */
    { DT_NONE,		DTN_NONE,	NULL },
    { DT_GENERIC,	DTN_GENERIC,	NULL },
    { DT_PSEUDO,	DTN_PSEUDO,	"pseudo device" },
    /* DT_DRIVER is like DT_PSUEDO, but DT_DRIVER devices have unit#'s */
    { DT_DRIVER,	DTN_DRIVER,	"device driver" },
    { DT_DEVICE,	DTN_DEVICE,	"device" },
    { DT_BUS,		DTN_BUS,	"system bus" },
    { DT_BRIDGE,	DTN_BRIDGE,	"bus bridge" },
    { DT_BIOS,		DTN_BIOS,	"system BIOS" },
    { DT_SYSBOARD,	DTN_SYSBOARD,	"system board" },
    { DT_ENCLOSURE,	DTN_ENCLOSURE,	"enclosure" },
    { DT_CONTROLLER,	DTN_CONTROLLER,	"controller" },
    { DT_CONTROLLER,	DTN_ADAPTER,	"controller" },
    { DT_CARD,		DTN_CARD,	"system card" },
    { DT_PROCESSOR,	DTN_PROCESSOR,	"processor" },
    { DT_CPU,		DTN_CPU,	"CPU" },
    { DT_DISKCTLR,	DTN_DISKCTLR,	"disk controller" },
    { DT_DISKCTLR,	DTN_MSD,	"disk controller" },
    { DT_DISKDRIVE,	DTN_DISKDRIVE,	"disk drive" },
    { DT_DISKDRIVE,	"disk",		"disk drive" },
    { DT_DISKPART,	DTN_DISKPART,	"disk partition" },
    { DT_FRAMEBUFFER,	DTN_FRAMEBUFFER,"frame buffer" },
    { DT_FRAMEBUFFER,	DTN_FB,		"frame buffer" },
    { DT_FRAMEBUFFER,	DTN_VID,	"frame buffer" },
    { DT_GFXACCEL,	DTN_GFXACCEL,	"graphics accelerator" },
    { DT_KEYBOARD,	DTN_KEYBOARD,	"Keyboard" },
    { DT_KEYBOARD,	DTN_KEY,	"Keyboard" },
    { DT_POINTER,	DTN_POINTER,	"pointer" },
    { DT_POINTER,	DTN_PTR,	"pointer" },
    { DT_POINTER,	DTN_MOUSE,	"mouse" },
    { DT_NETIF,		DTN_NETIF,	"network interface" },
    { DT_NETIF,		DTN_NET,	"network interface" },
    { DT_TAPECTLR,	DTN_TAPECTLR,	"tape controller" },
    { DT_TAPEDRIVE,	DTN_TAPEDRIVE,	"tape drive" },
    { DT_TAPEDRIVE,	"tape",		"tape drive" },
    { DT_DVD,		DTN_DVD,	"DVD drive" },
    { DT_CD,		DTN_CD,		"CD drive" },
    { DT_CDROM,		DTN_CDROM,	"CD-ROM drive" },
    { DT_CDROM,		"cd",		"CD-ROM drive" },
    { DT_FLOPPY,	DTN_FLOPPY,	"floppy disk drive" },
    { DT_FLOPPY,	"fd",		"floppy disk drive" },
    { DT_FLOPPYCTLR,	DTN_FLOPPYCTLR,	"floppy disk controller"},
    { DT_FLOPPYCTLR,	"fda",		"floppy disk controller"},
    { DT_MEMCTL,	DTN_MEMCTL,	"memory controller" },
    { DT_MEMORY,	DTN_MEMORY,	"memory" },
    { DT_MEMORY,	DTN_MEM,	"memory" },
    { DT_SERIAL,	DTN_SERIAL,	"serial device" },
    { DT_SERIAL,	DTN_COM,	"serial device" },
    { DT_SERIAL,	"tty",		"serial device" },
    { DT_SERIAL,	"rs232",	"serial device" },
    { DT_PARALLEL,	DTN_PARALLEL,	"parallel device" },
    { DT_PARALLEL,	DTN_PRT,	"parallel device" },
    { DT_AUDIO,		DTN_AUDIO,	"audio device" },
    { DT_MFC,		DTN_MFC,	"Multi Function Card" },
    { DT_MONITOR,	DTN_MONITOR,	"video monitor" },
    { DT_PRINTER,	DTN_PRINTER,	"printer" },
    { DT_WORM,		DTN_WORM,	"WORM" },
    { DT_SCANNER,	DTN_SCANNER,	"scanner" },
    { DT_CONSOLE,	DTN_CONSOLE,	"console" },
    { 0 },
};

/*
 * Get a device type and return DevType_t
 */
extern DevType_t *TypeGetByType(DevType)
    int				DevType;
{
    register int 		i;

    if (DevType <= 0)
	return((DevType_t *) NULL);

    for (i = 0; DevTypes[i].Name; ++i)
	if (DevTypes[i].Type == DevType)
	    return(&DevTypes[i]);

    return((DevType_t *) NULL);
}

/*
 * Get a device type and return it's name
 */
extern char *TypeGetType(DevType)
    int				DevType;
{
    register int 		i;

    if (DevType <= 0)
	return((char *) NULL);

    for (i = 0; DevTypes[i].Name; ++i)
	if (DevTypes[i].Type == DevType)
	    return DevTypes[i].Name;

    return((char *) NULL);
}

/*
 * Get a device type by name
 */
extern DevType_t *TypeGetByName(Name)
    char		       *Name;
{
    register DevType_t	       *TypePtr;

    if (!Name)
	return((DevType_t *) NULL);

    for (TypePtr = DevTypes; TypePtr->Name; ++TypePtr)
	if (EQ(TypePtr->Name, Name))
	    return(TypePtr);

    SImsg(SIM_DBG, "TypeGetByName: No type defined called <%s>", Name);

    return((DevType_t *) NULL);
}

/*
 * Set type information
 */
extern void TypeSetInfo(Names)
    char		       *Names;
{
    register DevType_t	       *TypePtr;
    register char	       *cp;

    if (Names) {
	/*
	 * Enable a specific list of typees.
	 * The Names variable is a `,' seperated list
	 * of type names.
	 */
	/* XXX Names param is altered */
	for (cp = strtok(Names, ","); cp; cp = strtok((char *)NULL, ",")) {
	    TypePtr = TypeGetByName(cp);
	    if (!TypePtr) {
		SImsg(SIM_GERR, "The type name `%s' is invalid.", cp);
		exit(1);
	    }
	    TypePtr->Enabled = TRUE;
	}
    } else 
	/*
	 * No specific type names were specified so enabled all of them.
	 */
	for (TypePtr = DevTypes; TypePtr->Name; ++TypePtr)
	    TypePtr->Enabled = TRUE;
}
