/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.30 $";
#endif

/*
 * Boot Time information
 */

#include <sys/types.h>	/* Must be before utmp.h */
#include <utmp.h>	/* Must be before defs.h */
#include "defs.h"
#include "psi.h"
#include <sys/stat.h>
#include <sys/time.h>

#if	defined(BOOT_TIME) && defined(HAVE_GETUTID)
/*
 * Look in utmp for the system boot time using getutid()
 */
extern Opaque_t GetBootTimeGetutid()
{
    static struct utmp		Utmp;
    struct utmp		       *Ptr;
    char		       *DateStr = NULL;

    setutent();
    memset(&Utmp, 0, sizeof(Utmp));
    Utmp.ut_type = BOOT_TIME;

    if (Ptr = getutid(&Utmp))
	if (Ptr->ut_type == BOOT_TIME)
	    DateStr = TimeToStr(Ptr->ut_time, NULL);

    endutent();

    return(DateStr);
}
#endif	/* BOOT_TIME && HAVE_GETUTID */

#if	defined(BOOT_TIME)

#ifndef UTMP_FILE
#define UTMP_FILE		"/etc/utmp"
#endif

/*
 * Look in utmp for the system boot time
 */
extern Opaque_t GetBootTimeUtmp()
{
    struct utmp		       *Ptr;
    struct utmp		       *UtmpBuff;
    struct stat			StatBuff;
    int				fd;
    size_t			Amt;
    char		       *DateStr = NULL;

    if (stat(UTMP_FILE, &StatBuff) < 0) {
	SImsg(SIM_GERR, "stat failed: %s: %s", UTMP_FILE, SYSERR);
	return((char *) NULL);
    }
    Amt = StatBuff.st_size;

    if ((fd = open(UTMP_FILE, O_RDONLY)) < 0) {
	SImsg(SIM_GERR, "open O_RDONLY failed: %s: %s", UTMP_FILE, SYSERR);
	return((char *) NULL);
    }

    UtmpBuff = (struct utmp *) xmalloc(Amt);
    if (read(fd, (char *)UtmpBuff, Amt) != Amt) {
	SImsg(SIM_GERR, "read failed: %s: %s", UTMP_FILE, SYSERR);
	(void) close(fd);
	return((char *) NULL);
    }
    (void) close(fd);

    for (Ptr = UtmpBuff; Ptr < UtmpBuff + Amt; ++Ptr) 
	if (Ptr->ut_type == BOOT_TIME) {
	    DateStr = TimeToStr(Ptr->ut_time, NULL);
	    break;
	}

    (void) free(UtmpBuff);
    return(DateStr);
}
#endif	/* BOOT_TIME */

#if	defined(HAVE_NLIST)
#if !defined(BOOTTIME_SYM)
#	define BOOTTIME_SYM 	"_boottime"
#endif

/*
 * Get kernel BOOTTIME string by reading the
 * symbol "_boottime" from the kernel.
 */
extern Opaque_t GetBootTimeSym()
{
    static struct timeval	BootTime;
    time_t			TimeVal;
    char		       *DateStr = NULL;

    if (KVMgetSymbol((kvm_t *) NULL,
		     BOOTTIME_SYM,
		     &BootTime, sizeof(BootTime),
		     KDT_DATA) < 0) {
	SImsg(SIM_GERR, "Read of \"%s\" from kernel failed.", 
	      BOOTTIME_SYM);
    } else {
	TimeVal = BootTime.tv_sec;
	DateStr = TimeToStr(TimeVal, NULL);
    }

    return (Opaque_t) DateStr;
}
#endif	/* HAVE_NLIST */

/*
 * Get Boot Time
 */
extern int GetBootTime(Query)
     MCSIquery_t	      *Query;
{
    char		       *Str = NULL;

    if (Query->Op == MCSIOP_CREATE) {
	if (Str = PSIquery(PsiSwitch.GetBootTime)) {
	    Query->Out = (Opaque_t) strdup(Str);
	    Query->OutSize = strlen(Str);
	    return 0;
	}
    } else if (Query->Op == MCSIOP_DESTROY) {
	if (Query->Out && Query->OutSize)
	    (void) free(Query->Out);
	return 0;
    }

    return -1;
}
