/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.48 $";
#endif

/*
 * Memory related functions.
 */

#include "defs.h"

/*
 * Divide and Round Up
 */
extern Large_t DivRndUp(Num, Div)
    Large_t	 		Num;
    Large_t	 		Div;
{
    Large_t			i;

    i = Num / Div;

    return((Num % Div) ? i+1 : i);
}

/*
 * Get string of the amount of physical memory
 */
extern char *GetMemoryStr(Amount)
    Large_t			Amount;
{
    static char			Buff[64];

    if (Amount > 0) {
	(void) snprintf(Buff, sizeof(Buff), "%s",
			GetSizeStr(Amount, MBYTES, 0));
	return(Buff);
    }

    return((char *) NULL);
}

#if	defined(PHYSMEM_SYM) && !defined(HAVE_PHYSMEM)
#	define HAVE_PHYSMEM	1
#endif	/* PHYSMEM_SYM */
#if	!defined(PHYSMEM_SYM)
#	define PHYSMEM_SYM	"_physmem"
#endif	/* PHYSMEM_SYM */
#if	!defined(GETPAGESIZE)
#	define GETPAGESIZE() 	getpagesize()
#endif	/* GETPAGESIZE */

/*
 * Common method of determining amount of physical memory in a
 * BSD Unix machine.
 *
 * Get memory by reading the variable "physmem" from the kernel
 * and the system page size.
 */ 
extern Opaque_t GetMemoryPhysmemSym()
{
#if	defined(HAVE_PHYSMEM)
    static Large_t	 	Amount;
    Large_t			Bytes;
    int				physmem;

    if (!Amount) {
	if (KVMgetSymbol((kvm_t *) NULL,
			 PHYSMEM_SYM,
			 &physmem, sizeof(physmem),
			 KDT_DATA) < 0) {
	    SImsg(SIM_DBG, "Read of \"%s\" from kernel failed.", 
		  PHYSMEM_SYM);
	    return (Opaque_t) NULL;
	}

	/*
	 * Could use ctob() instead of "Page Size * Num Pages",
	 * but this is more portable.
	 */
	Bytes = (Large_t)GETPAGESIZE() * (Large_t)physmem;
	SImsg(SIM_DBG, "Physmem: Bytes = %.0f physmem = %d pagesize = %d", 
	      (float) Bytes, physmem, GETPAGESIZE());
	Amount = DivRndUp(Bytes, (Large_t)MBYTES);
    }

    return (Opaque_t) GetMemoryStr(Amount);
#else	/* !HAVE_PHYSMEM */
    return (Opaque_t) NULL;
#endif	/* HAVE_PHYSMEM */
}

/*
 * Get amount of memory on machine.
 */
extern int GetMemory(Query)
     MCSIquery_t	      *Query;
{
    extern PSI_t	       GetMemoryPSI[];
    static char		      *Str;

    if (Query->Op == MCSIOP_CREATE) {
	if (Str = PSIquery(PsiSwitch.GetMemory)) {
	    Query->Out = (Opaque_t) strdup(Str);
	    Query->OutSize = strlen(Str);
	    return 0;
	}
    } else if (Query->Op == MCSIOP_DESTROY) {
	if (Query->Out && Query->OutSize)
	    (void) free(Query->Out);
	return 0;
    }

    return -1;
}
