/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

/*
 * $Revision: 1.10 $
 */

/*
 * Mount related definetions
 */
#ifndef __mcmnt_h__
#define __mcmnt_h__

#include <sys/types.h>
#include <sys/stat.h>

/*
 * This major/minor code comes from GNU origins.
 */
/* Since major is a function on SVR4, we can't use `ifndef major'.  */
#if MAJOR_IN_MKDEV
# include <sys/mkdev.h>
# define HAVE_MAJOR
#endif
#if MAJOR_IN_SYSMACROS
# include <sys/sysmacros.h>
# define HAVE_MAJOR
#endif
#ifdef major			/* Might be defined in sys/types.h.  */
# define HAVE_MAJOR
#endif

#ifndef HAVE_MAJOR
# define major(dev)  (((dev) >> 8) & 0xff)
# define minor(dev)  ((dev) & 0xff)
# define makedev(maj, min)  (((maj) << 8) | (min))
#endif
#undef HAVE_MAJOR

/*
 * List of Local Filesystem types
 */
#define LOCAL_FS_TYPES	{ "ufs", "vxfs", "ext", "ext2", "ext3", "hfs", \
			  "xfs", "jfs", NULL }

/*
 * Mount info
 */
struct _MntInfo {
    char	       *DevPath;	/* Device path; /dev/dsk/c0t0s1 */
    char	       *DevPathRaw;	/* Raw Device name; /dev/rdsk/c0t0s1 */
    char	       *DevName;	/* Name of Device; c0t0 */
    char	       *MntName;	/* Dir mounted on; /usr */
    char	       *Type;		/* fs type */
    int			Usage;		/* Same as PartInfo_t.Usage */
    char	       *Opts;		/* Options string */
    char	      **MntOpts;	/* Array of Opts */
    Large_t		Size;		/* Total size (MB) */
    Large_t		AmtUsed;	/* Amount used (MB) */
    dev_t		rdev;		/* Value from stat.st_rdev */
    struct _MntInfo    *Next;		/* Next entry */
};
typedef struct _MntInfo		MntInfo_t;

/*
 * Declarations
 */
extern MntInfo_t *		MntInfoCreate(void);
extern Opaque_t			BuildPartInfoMnt(PartInfo_t **, char **);
extern int			GetPartUsage(DevInfo_t *, char *, DiskPart_t *);
extern int			PartSetFilesys(DevInfo_t *, char *, PartInfo_t *);
extern int 			MntGetFileSys(MntInfo_t *);
extern int 			MntExists(MntInfo_t *, MntInfo_t *);
extern int 			MntInfoFill(MntInfo_t *);
#endif	/* __mcmnt_h__ */
