/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

/*
 * Frontend functions for kvm_*() functions
 */

#include <stdio.h>
#include "defs.h"
#include <fcntl.h>

/*
 * Perform a kvm_close().
 */
extern void KVMclose(kd)
    kvm_t 		       *kd;
{
    if (kd)
	(void) kvm_close(kd);
}

/*
 * Perform a kvm_open().
 */
extern kvm_t *KVMopen()
{
    kvm_t 		       *kd = NULL;
    extern char 	       *ProgramName;

    if ((kd = kvm_open((char *)NULL, (char *)NULL, (char *)NULL, O_RDONLY,
		       ProgramName)) == NULL) {
	SImsg(SIM_GERR, "kvm_open failed: %s.", SYSERR);
	return((kvm_t *) NULL);
    }

#if MC_KVM_LIB
    SImsg(SIM_DBG, "KVMopen: Open succeeded: namelist=<%s> vmfile=<%s>",
	  PRTS(kd->namelist), PRTS(kd->vmfile));
#endif

    return(kd);
}

#if defined(HAVE_NLIST) || defined(HAVE_KNLIST)

static nlist_t NListTab[] = {
    { 0 },
    { 0 },
};

/*
 * Perform an nlist on "kd" looking for "Symbol" or "NameList".
 * Return the a pointer to the found nlist structure.
 * On failure, return NULL and KVMclos(kd).
 */
extern nlist_t *KVMnlist(kd, Symbol, NameList, NumNameList)
    kvm_t		       *kd;
    char		       *Symbol;
    nlist_t		       *NameList;
    int				NumNameList;
    /*ARGSUSED*/
{
    nlist_t		       *NLPtr;
    nlist_t		       *TabPtr;
    char		       *cp;
    int				Status;
#if defined(HAVE_KNLIST)
    int				NumEle = 0;
#endif

    SImsg(SIM_DBG, "KVMnlist:\t Symbol=<%s> NumNameList=%d",
	  PRTS(Symbol), NumNameList);

    if (NameList) {
	NLPtr = NameList;
	TabPtr = NameList;
#if defined(HAVE_KNLIST)
	NumEle = NumNameList;
#endif
    } else {
	TabPtr = NListTab;
	NLPtr = &NListTab[0];
#if defined(HAVE_KNLIST)
	NumEle = 1;
#endif
	memset((void *) NLPtr, 0, sizeof(nlist_t));
	GetNlNamePtr(NLPtr) = Symbol;
    }

#if defined(COFF) || defined(ELF)
    for (NLPtr = TabPtr; GetNlNamePtr(NLPtr); ++NLPtr) {
	cp = GetNlNamePtr(NLPtr);
	/*
	 * Skip over initial '_'
	 */
	if (*cp == '_')
	    GetNlNamePtr(NLPtr) = cp + 1;
    }
#endif

#if defined(HAVE_KNLIST)
    SImsg(SIM_DBG, "KVMnlist:\t Reading %d names from 0x%x... ",
	  NumEle, TabPtr);
    Status = knlist(TabPtr, NumEle, sizeof(nlist_t));
#else	/* !HAVE_KNLIST */
    SImsg(SIM_DBG, "KVMnlist:\t Reading names from 0x%x ... ", TabPtr);
    Status = kvm_nlist(kd, TabPtr);
#endif	/* HAVE_KNLIST */
    if (Status == -1) {
	cp = GetNlNamePtr(NLPtr);
	SImsg(SIM_GERR, "kvm_nlist name \"%s\" failed (status = %d): %s.", 
			 (cp) ? cp : "(unknown)", Status, SYSERR);
#if defined(MC_KVM_LIB)
	if (kvmerrstr)	/* Our own kvm_nlist() sets this external */
	    SImsg(SIM_DBG, "kvm_nlist failed with: %s", kvmerrstr);
#endif
	return((nlist_t *) NULL);
    }

    return(&TabPtr[0]);
}
#endif	/* HAVE_NLIST || HAVE_KNLIST */

#if HAVE_NLIST64 && HAVE_STRUCT_NLIST64

static nlist64_t NListTab64[] = {
    { 0 },
    { 0 },
};

/*
 * Perform a 64-bit nlist on "kd" looking for "Symbol" or "NameList".
 * Return the a pointer to the found nlist structure.
 * On failure, return NULL and KVMclos(kd).
 */
extern nlist64_t *KVMnlist64(kvm_t *		kd,
			     char *		Symbol,
			     nlist64_t *	NameList,
			     int		NumNameList)
     /*ARGSUSED*/
{
    nlist64_t		       *NLPtr;
    nlist64_t		       *TabPtr;
    char		       *cp;
    int				Status;

    SImsg(SIM_DBG, "KVMnlist64:\t Symbol=<%s> NumNameList=%d",
	  PRTS(Symbol), NumNameList);

    if (NameList) {
	NLPtr = NameList;
	TabPtr = NameList;
    } else {
	TabPtr = NListTab64;
	NLPtr = &NListTab64[0];
	memset((void *) NLPtr, 0, sizeof(nlist64_t));
	GetNlNamePtr(NLPtr) = Symbol;
    }

#if defined(COFF) || defined(ELF)
    for (NLPtr = TabPtr; GetNlNamePtr(NLPtr); ++NLPtr) {
	cp = GetNlNamePtr(NLPtr);
	/*
	 * Skip over initial '_'
	 */
	if (*cp == '_')
	    GetNlNamePtr(NLPtr) = cp + 1;
    }
#endif

    SImsg(SIM_DBG, "KVMnlist64:\t Reading names from 0x%x ... ", TabPtr);
    Status = kvm_nlist64(kd, TabPtr);
    if (Status == -1) {
	cp = GetNlNamePtr(NLPtr);
	SImsg(SIM_GERR, "kvm_nlist64 name \"%s\" failed (status = %d): %s.", 
			 (cp) ? cp : "(unknown)", Status, SYSERR);
#if defined(MC_KVM_LIB)
	if (kvmerrstr)	/* Our own kvm_nlist64() sets this external */
	    SImsg(SIM_DBG, "kvm_nlist64 failed with: %s", kvmerrstr);
#endif
	return((nlist64_t *) NULL);
    }

    return(&TabPtr[0]);
}
#endif	/* HAVE_NLIST64 */

/*
 * Perform a kvm_read().
 *
 * If DataType==KDT_STRING, read 1 byte at a time until '\0' or
 * NumBytes is reached. This is necessary in order to avoid
 * reading invalid memory pages from the kernel which can lead
 * to system crashes.
 *
 * If DataType==KDT_DATA then we read NumBytes of data into
 * Buf all at once.
 */
extern int KVMget(kd, Addr, Buf, NumBytes, DataType)
    kvm_t 		       *kd;
    KVMaddr_t	 		Addr;
    void 		       *Buf;
    size_t 		        NumBytes;
    int				DataType;
{
    char		       *Ptr;
    char		       *End;

    if (!kd || !Buf)
	return(-1);

    switch (DataType) {
    case KDT_STRING:
	Ptr = (char *) Buf;
	End = (char *) &Ptr[NumBytes-1];
	do {
	    if (kvm_read(kd, Addr++, Ptr, 1) != 1) {
		SImsg(SIM_GERR, "kvm_read failed prematurely: %s.", SYSERR);
		return(-1);
	    }
	} while (Ptr < End && *Ptr++);
	*Ptr = C_NULL;
	break;

    case KDT_DATA:
	if (kvm_read(kd, Addr, Buf, NumBytes) != NumBytes) {
	    SImsg(SIM_GERR, "kvm_read failed (amount=%d): %s.", 
			     NumBytes, SYSERR);
	    return(-1);
	}
	break;

    default:
	SImsg(SIM_UNKN, "Unknown Kernel Data Type: %d.", DataType);
	return(-1);
    }

    return(0);
}

/*
 * Get the address of Symbol and return it.
 * Returns KVMaddr_t on success.
 * Returns 0 on error.
 */
extern KVMaddr_t KVMgetAddr(kvm_t *		kd,
			    char *		Symbol)
{
#if  HAVE_NLIST
    static int			Done;
    static MCSIquery_t		Query;
    nlist_t *			nl;
    int				KernBits = 0;
    KVMaddr_t			Addr = 0;
#if     HAVE_STRUCT_NLIST64
    nlist64_t *			nl64;
#endif /* HAVE_STRUCT_NLIST64 */

    if (!kd || !Symbol) {
	SImsg(SIM_DBG, "KVMgetAddr: Invalid parameter passed.");
	return 0;
    }

    /*
     * Determine if we're running 32-bit or 64-bit
     */
    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Cmd = MCSI_KERNBITS;
    if (mcSysInfo(&Query) == 0) {
	KernBits = atoi((char *) Query.Out);
    } else {
	SImsg(SIM_DBG, "KVMgetAddr: Cannot determine kernel bit size.");
	return 0;
    }

    if (!Done) {
	SImsg(SIM_DBG, "KVMgetAddr: Kernel bit size is %d", KernBits);
	Done = TRUE;
    }

    /*
     * Use the correct nlist interface to find the address of Symbol
     */
    switch (KernBits) {
    case 64:
	/*
	 * Use KVMnlist64 if we have one, otherwise fall through to the
	 * standard version.
	 */
#if     HAVE_STRUCT_NLIST64
	if ((nl64 = KVMnlist64(kd, Symbol, (nlist64_t *) NULL, 0)) == NULL)
	    return 0;
	Addr = nl64->n_value;
	break;
#endif	/* HAVE_STRUCT_NLIST64 */
    case 32:
	if ((nl = KVMnlist(kd, Symbol, (nlist_t *) NULL, 0)) == NULL)
	    return 0;
	Addr = nl->n_value;
	break;
    default:
	SImsg(SIM_DBG, "KVMgetAddr: Unknown kernel bit size case (%d).",
	      KernBits);
	return 0;
    }

    SImsg(SIM_DBG, "KVMgetAddr: Address of <%s> is 0x%x", Symbol, Addr);

    return Addr;
#else	/* !HAVE_NLIST */
    return 0;
#endif 	/* HAVE_NLIST */
}

/*
 * Get Symbol and store results in Buffer.
 * This function combines KVMnlist() and KVMget().
 * If kd is NULL, we'll KVMopen() and KVMclose() ourself.
 * Returns 0 on success.
 * Returns -1 on error.
 */
extern int KVMgetSymbol(kvm_t *		kd,
			char *		Symbol,
			void *		Buffer,
			size_t		BufferLen,
			int		DataType)
{
    int				Status = -1;
    static MCSIquery_t		Query;
    KVMaddr_t			Addr = 0;
    kvm_t *			mykd = NULL;

    SImsg(SIM_DBG, "KVMgetSymbol: Get symbol <%s>", PRTS(Symbol));

    if (!kd) {
	/* Do a one-time KVMopen */
	mykd = kd = KVMopen();
	if (!mykd) {
	    SImsg(SIM_DBG, "KVMgetSymbol: Cannot open kvm.");
	    return -1;
	}
    }

    if (!Symbol || !Buffer || BufferLen <= 0) {
	SImsg(SIM_DBG, "KVMgetSymbol: Invalid parameter passed.");
	if (mykd)
	    KVMclose(mykd);
	return -1;
    }

    /*
     * Read the address via nlist from the kernel
     */
    Addr = KVMgetAddr(kd, Symbol);
    if (!Addr) {
	SImsg(SIM_DBG, "KVMgetSymbol: Unable to locate address of <%s>",
	      Symbol);
	if (mykd)
	    KVMclose(mykd);
	return -1;
    }

    /* 
     * Read the data from Addr and store in Buffer
     */
    if (KVMget(kd, Addr, Buffer, BufferLen, DataType) < 0) {
	SImsg(SIM_DBG, "KVMgetSymbol: Read of \"%s\" from kernel failed.", 
	      Symbol);
	Status = -1;
    } else {
	Status = 0;	/* Success */
    }

    if (mykd)
	/* Close down mykd since we created it */
	KVMclose(mykd);

    return Status;
}

/*
 * Check to see if PtrNL is valid.
 */
extern int _CheckNlist(PtrNL)
#if	defined(HAVE_NLIST)
    nlist_t		       *PtrNL;
#else
    void		       *PtrNL;
#endif	/* HAVE_NLIST */
{
#if	defined(HAVE_NLIST)
    char		       *cp;

    if (!PtrNL)
	return(-1);

    /*
     * Should use n_type, but that's not set
     * correctly on some OS's.
     */
    if (!PtrNL->n_value) {
	cp = GetNlNamePtr(PtrNL);
	SImsg(SIM_DBG, "Kernel symbol \"%s\" not found.", 
	      (cp) ? cp : "(unknown)");
	return(-1);
    }

#endif	/* HAVE_NLIST */
    return(0);
}
