/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.14 $";
#endif

/*
 * Flexlm License functions
 */

#include "defs.h"
#include <netdb.h>
#if defined(HAVE_STDLIB_H)
# include <stdlib.h>
#endif
#include <sys/types.h>
#include <sys/socket.h>

/*
 * Default range of ports to scan looking for flexlm servers
 */
#define PORT_START		7100
#define PORT_END		7999

#define LIC_SERVER_SOFT		"FLEXlm"

/*
 * Add Port to PortsPtr.
 * Returns current index value of PortsPtr
 */
static int PortAdd(PortsPtr, PortCntPtr, Port)
     int		      **PortsPtr;
     int		       *PortCntPtr;
     int			Port;
{
    int			       *Ports;
    static int			PortsSize;
    int				PortIdx;

    if (!PortsPtr || !PortCntPtr)
	return -1;

    Ports = *PortsPtr;
    PortIdx = *PortCntPtr;

    /* Alloc/Re-Alloc */
    if (!Ports) {
	PortsSize = 128;		/* Arbitrary */
	Ports = (int *) xcalloc(PortsSize, sizeof(int *));
    } else if (PortIdx >= PortsSize-1) {
	PortsSize = PortsSize + 1024;	/* Add arbitrary amount */
	Ports = (int *) realloc((void *)Ports, PortsSize);
    }

    Ports[PortIdx++] = Port;
    Ports[PortIdx] = -1;	/* Mark end */

    *PortsPtr = Ports;
    *PortCntPtr = PortIdx;

    return PortIdx;
}

/*
 * Scan ports PortStart - PortEnd looking for anything which listens.
 * Returns array of bound ports.
 */
static int ScanPorts(PortsPtr, PortCntPtr, PortStart, PortEnd)
     int		      **PortsPtr;
     int		       *PortCntPtr;
     int			PortStart;
     int			PortEnd;
{
    struct sockaddr_in		sin;
    struct hostent	       *hp;
    int				s = -1;
    int				p;
    int				PortIdx = 0;
    time_t			TimeStart;
    time_t			TimeEnd;

    if (!PortsPtr)
	return -1;

    SImsg(SIM_DBG, "\tFLEXLM: Scanning ports %d - %d",
	  PortStart, PortEnd);

    time(&TimeStart);

    hp = gethostbyname("localhost");
    if (!hp) {
	SImsg(SIM_DBG,
	      "\tFLEXLM: Cannot lookup host \"localhost\": %s",
	      SYSERR);
	return -1;
    }
    
    memset(&sin, 0, sizeof(sin));
    memcpy(&(sin.sin_addr), hp->h_addr, hp->h_length);
    sin.sin_family = AF_INET;

    /*
     * We don't know the flexlm protocol so we just want to see if something
     * is listening on this port.
     */
    for (p = PortStart; p <= PortEnd; ++p) {
	if (s < 0) {
	    /* Create a socket */
	    s = socket(AF_INET, SOCK_STREAM, 0);
	    if (s < 0) {
		SImsg(SIM_DBG, "\tFLEXLM: Create socket failed: %s",
		      SYSERR);
		return -1;
	    }
	}

	/* Set the port to connect to */
	sin.sin_port = p;

	/* Connect to the port to see if anybody is listening */
	if (connect(s, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
#if	defined(FLEXLM_EXTRA_DEBUG)
	    SImsg(SIM_DBG, "\tFLEXLM: Connect to port %d failed: %s",
		  p, SYSERR);
#endif
	    continue;
	}
	SImsg(SIM_DBG, "\tFLEXLM: Found in use port %d", p);

	/* Shut down the socket */
	close(s);
	s = -1;

	PortIdx = PortAdd(PortsPtr, PortCntPtr, p);
    }

    time(&TimeEnd);

    SImsg(SIM_DBG, "\tFLEXLM: Found %d/%d ports in %d seconds",
	  PortIdx, PortEnd - PortStart, TimeEnd - TimeStart);
    
    return 0;
}

/*
 * Return the path to "lmstat"
 */
static char *LmstatPath()
{
    static char		        LmstatPath[MAXPATHLEN];
    char		       *BaseDir;
    char		       *OsName;
    char		       *Arch;
    MCSIquery_t			Query;
    
    if (LmstatPath[0])
	return LmstatPath;

    BaseDir = DirName(McProgramPath);

    Query.Op = MCSIOP_CREATE;
    Query.Cmd = MCSI_OSNAME;
    if (mcSysInfo(&Query) < 0) {
	SImsg(SIM_DBG, "\tFLEXLM: Cannot determine OS name");
	return (char *) NULL;
    }
    OsName = strdup(strlower((char *)Query.Out));

    Query.Op = MCSIOP_CREATE;
    Query.Cmd = MCSI_APPARCH;
    if (mcSysInfo(&Query) < 0) {
	SImsg(SIM_DBG, "\tFLEXLM: Cannot determine App Arch");
	return (char *) NULL;
    }
    Arch = strdup(strlower((char *)Query.Out));
    
    snprintf(LmstatPath, sizeof(LmstatPath),
	     "%s/../libexec/platform/%s-%s/lmstat",
	     BaseDir, OsName, Arch);

    SImsg(SIM_DBG, "\tFLEXLM: lmstat path = <%s>", LmstatPath);

    if (!FileExists(LmstatPath, 0)) {
	LmstatPath[0] = CNULL;
	return (char *) NULL;
    }

    return LmstatPath;
}

/*
 * Given the port flexlm is running on, get the flexlm license file.
 */
static License_t *CheckPort(Port, TmpFile, TmpFp, lmstat)
     int			Port;
     char		       *TmpFile;
     FILE		       *TmpFp;
     char		       *lmstat;
{
    static char		       *Hostid;
    static char		       *Hostname;
    char			Buff[1024];
    char			Cmd[MAXPATHLEN];
    FILE		       *cfp;
    MCSIquery_t			Query;
    License_t		       *License = NULL;
    int				Status;
    char		      **Argv;
    char		       *cp;
    int				Argc;
    int				len;
    
    
    SImsg(SIM_DBG, "\tFLEXLM: Checking port %d to see if its flexlm",
	  Port);

    if (!Hostid) {
	Query.Op = MCSIOP_CREATE;
	Query.Cmd = MCSI_HOSTID;
	if (mcSysInfo(&Query) < 0) {
	    SImsg(SIM_DBG, "\tFLEXLM: Cannot determine hostid");
	    return (License_t *) NULL;
	}
	Hostid = strdup((char *)Query.Out);
    }
    if (!Hostname) {
	Query.Op = MCSIOP_CREATE;
	Query.Cmd = MCSI_HOSTNAME;
	if (mcSysInfo(&Query) < 0) {
	    SImsg(SIM_DBG, "\tFLEXLM: Cannot determine hostname");
	    return (License_t *) NULL;
	}
	Hostname = strdup((char *)Query.Out);
    }

    fseek(TmpFp, SEEK_SET, 0);		/* Rewind to start of file */
    snprintf(Buff, sizeof(Buff), "SERVER %s %s %d", Hostname, Hostid, Port);
    fprintf(TmpFp, "%s\n", Buff);
    fflush(TmpFp);			/* Flush to disk */

    snprintf(Cmd, sizeof(Cmd), "%s -c %s", lmstat, TmpFile);

    SImsg(SIM_DBG, "\tFLEXLM:     Command: <%s>", Cmd);
    SImsg(SIM_DBG, "\tFLEXLM: license.dat: <%s>", Buff);

    cfp = popen(Cmd, "r");
    if (!cfp) {
	SImsg(SIM_DBG, "\tFLEXLM: Command <%s> failed to run: %s",
	      Cmd, SYSERR);
	return (License_t *) NULL;
    }

    License = LicenseCreate(NULL);
    License->Port = Port;
    License->ServerSoft = strdup(LIC_SERVER_SOFT);

    while (fgets(Buff, sizeof(Buff), cfp)) {
	Chop(Buff);
	SImsg(SIM_DBG, "%s", Buff);

	/*
	 * Look for:
	 *     License file(s) on sun0.magnicomp.com: /etc/opt/licenses/licenses_combined:
	 * and grab the last arg as the license.dat file.
	 */
	if (EQN(Buff, "    License file(s) on", 22)) {
	    Argc = StrToArgv(Buff, " ", &Argv, NULL, STRTO_SKIPRP);
	    if (Argc < 5)
		continue;
	    cp = Argv[5];
	    /* Remove ':' at end of file name */
	    len = strlen(cp);
	    if (cp[len-1] == ':')
		cp[len-1] = CNULL;

	    if (EQ(cp, TmpFile)) {
		/*
		 * This typically means that lmstat couldn't talk to the
		 * flexlm server
		 */
		SImsg(SIM_DBG, "\tFLEXLM: lmstat reports license file as our temporary license file");
		LicenseDestroy(License);
		return (License_t *) NULL;
	    }

	    License->File = strdup(cp);
	    SImsg(SIM_DBG, "\tFLEXLM: License file is <%s>",
		  License->File);
	}
    }
    
    Status = pclose(cfp);
    if (Status != 0) {
	SImsg(SIM_DBG, "\tFLEXLM: lmstat exited status %d", Status);
	SImsg(SIM_DBG, "\tFLEXLM: Server on port %d is not flexlm",
	      Port);
	LicenseDestroy(License);
	return (License_t *) NULL;
    }
    
    return License;
}

/*
 * Read a license.dat and create new licenses
 */
static int SetLicenseDat(LicenseList, License)
     License_t		      **LicenseList;
     License_t		       *License;
{
    FILE		       *fp;
    char			Line[8124];
    char			Buff[1024];
    int				Argc;
    char		      **Argv;
    License_t		       *lp;
    License_t		       *New;
    License_t		       *Last = NULL;
    int				len;
    int				ContMarker;
    int				NewContMarker;
    register int		i;
        
    if (!License || !License->File)
	return -1;
    
    SImsg(SIM_DBG, "\tFLEXLM: Reading license file <%s>",
	  License->File);
    
    fp = fopen(License->File, "r");
    if (!fp) {
	SImsg(SIM_DBG, "\tFLEXLM: <%s> license file open failed: %s",
	      License->File, SYSERR);
	return -1;
    }

    while (fgets(Buff, sizeof(Buff), fp)) {
	Chop(Buff);
	SImsg(SIM_DBG, "%s", Buff);

	if (Buff[0] == '#')
	    continue;

	/*
	 * Look for new line continue marker
	 */
	len = strlen(Buff);
	if (Buff[len-1] == '\\') {
	    NewContMarker = TRUE;	/* Set new marker */
	    Buff[len-1] = ' ';		/* Fix Buff to remove mark */
	} else {
	    NewContMarker = FALSE;
	}

	/*
	 * If the continue marker is set, append to Line
	 */
	if (ContMarker) {
	    StrAppend(Line, sizeof(Line), Buff);
	} else {
	    snprintf(Line, sizeof(Line), "%s", Buff);
	}

	ContMarker = NewContMarker;
	if (NewContMarker)
	    continue;

	if (!Line[0])			/* Emtpy line */
	    continue;

	Argc = StrToArgv(Line, " \t", &Argv, "\"\"", STRTO_SKIPRP);

	if (EQ(Argv[0], "DAEMON") || EQ(Argv[0], "VENDOR")) {
	    if (Argc > 2)
		License->VendorServerSoft = strdup(Argv[1]);
	} else if (EQ(Argv[0], "FEATURE") || EQ(Argv[0], "INCREMENT")) {
	    New = LicenseCreate(NULL);
	    /* Copy base info into New license */
	    memcpy(New, License, sizeof(License_t));

	    if (Argc > 2)
		New->SoftName = strdup(Argv[1]);
	    if (Argc > 4)
		New->SoftVer = strdup(Argv[3]);
	    if (Argc > 5)
		New->ExpireDate = strdup(Argv[4]);
	    if (Argc > 6)
		New->Count = atoi(Argv[5]);
	    if (Argc > 7)
		New->Key = strdup(Argv[6]);

	    /* Look for other items of interest which are not fixed fields */
	    for (i = 7; i < Argc; ++i) {
		if (EQN(Argv[i], "ISSUED=", 7))
		    New->IssueDate = strdup(Argv[i]+7);
		else if (EQN(Argv[i], "ISSUER=", 7))
		    New->Issuer = strdup(Argv[i]+7);
		else if (EQN(Argv[i], "SN=", 3))
		    New->Serial = strdup(Argv[i]+3);
		else if (EQN(Argv[i], "START=", 6))
		    New->StartDate = strdup(Argv[i]+6);
		else if (EQN(Argv[i], "HOSTID=", 7))
		    New->Hostid = strdup(Argv[i]+7);
	    }

	    /*
	     * In flexlm terms, anything with a Hostid specified is node
	     * locked, while everything else is floating.
	     */
	    if (New->Hostid)
		New->Type = LT_NODELOCK;
	    else
		New->Type = LT_FLOAT;

	    /* Add license */
	    if (!*LicenseList)
		*LicenseList = Last = New;
	    else {
		if (!Last) {
		    for (lp = *LicenseList; lp && lp->Next; lp = lp->Next);
		    Last = lp;
		}
		Last->Next = New;
		Last = New;
	    }
	}
    }
    
    fclose(fp);

    return 0;
}

/*
 * Build a list of licenses (LicenseList) from flexlm.
 *
 * We first determine what ports have a server (of any type) listening.
 * We then use lmstat to query each port we found *something* on to 
 * determine if the port is in use by a flexlm server.  If so, we get the
 * local name of the license.dat file for that server from lmstat.  We use
 * the local path to license.dat to open and parse the file.  This repeats
 * for each bound port we found.
 */
extern Opaque_t BuildLicenseFlexlm(LicenseList, SearchExp, MCSIopts)
     License_t		      **LicenseList;
     char		      **SearchExp;
     MCSIopts_t		       *MCSIopts;
{
    int			       *Ports = NULL;
    int				PortCnt = 0;
    char		       *lmstat;
    License_t		       *License;
    License_t		       *Last = NULL;
    LicOpts_t		       *LicOpts;
    static char			TmpFile[MAXPATHLEN];
    MCSIopts_t		       *mo;
    static FILE		       *fp = NULL;
    register int		i;
    int				DefaultPorts = TRUE;
    char *			cp;

    SImsg(SIM_DBG, "BuildLicenseFLEXlm: Looking for flexlm licenses");

    if (!LicenseList)
	return (Opaque_t) -1;

    /*
     * If this env var is set, read license data from it instead of
     * performing a port scan.
     */
    if (cp = getenv(SYSINFO_FLEXLM_FILE_VAR)) {
	McDebug("\tFLEXLM: Reading flexlm license.dat from $%s <%s>",
		SYSINFO_FLEXLM_FILE_VAR, cp);
	License = LicenseCreate(NULL);
	License->File = cp;
	SetLicenseDat(LicenseList, License);
	return (Opaque_t) 0;
    }
    
    lmstat = LmstatPath();
    if (!lmstat) {
	SImsg(SIM_DBG, "\tFLEXLM: Cannot find lmstat command.");
	return (Opaque_t) -1;
    }

    /*
     * Look for caller specified port ranges to scan
     */
    for (mo = MCSIopts; mo; mo = mo->Next) {
	if (mo->Op != MCSI_LICENSE)
	    continue;
	LicOpts = (LicOpts_t *) mo->Options;
	if (!LicOpts)
	    continue;
	DefaultPorts = FALSE;
	ScanPorts(&Ports, &PortCnt, LicOpts->StartPort, LicOpts->EndPort);
    }

    if (DefaultPorts)
	ScanPorts(&Ports, &PortCnt, PORT_START, PORT_END);
    
    if (!Ports) {
	SImsg(SIM_DBG,
	      "\tFLEXLM: No running license servers detected");
	return (Opaque_t) -1;
    }
	
    for (i = 0; Ports[i] != -1; ++i) {
	/* Create temporary file */
	if (!fp) {
	    snprintf(TmpFile, sizeof(TmpFile), "/tmp/sysinfo.flexlm.%d",
		     getpid());
	    fp = fopen(TmpFile, "w");
	    if (!fp) {
		SImsg(SIM_DBG,
		      "\tFLEXLM: %s: Create temporary file failed: %s",
		      SYSERR);
		return (License_t *) NULL;
	    }
	}

	if (License = CheckPort(Ports[i], TmpFile, fp, lmstat)) {
	    SetLicenseDat(LicenseList, License);
	}
    }

    /* Cleanup */
    if (fp)
	fclose(fp);
    unlink(TmpFile);
    
    return (Opaque_t) 0;
}
