/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.5 $";
#endif

/*
 * License class information
 */

#include "defs.h"

static char *LicenseTypes[]	= LICENSE_TYPES;
static char *LicenseDescs[]	= LICENSE_DESCS;

/*
 * Given a Range of ports, return an MCSIopts_t version.
 */
static MCSIopts_t *PortOpts(Range)
     char		       *Range;
{
    char		       *cp;
    MCSIopts_t		       *Opts;
    LicOpts_t		       *LicOpts;

    cp = strchr(Range, '-');
    if (!cp) {
	SImsg(SIM_DBG, "PortOpts: Range <%s> is invalid due to no '-' found.",
	      Range);
	return (MCSIopts_t *) NULL;
    }
    
    *cp++ = CNULL;

    LicOpts = (LicOpts_t *) xcalloc(1, sizeof(LicOpts_t));
    LicOpts->StartPort = atoi(Range);
    LicOpts->EndPort = atoi(cp);
    
    Opts = (MCSIopts_t *) xcalloc(1, sizeof(MCSIopts_t));
    Opts->Op = MCSI_LICENSE;
    Opts->Options = (Opaque_t) LicOpts;

    return Opts;
}

/*
 * Set MCSI license options for ports from Ranges
 */
extern int LicenseSetPorts(Ranges, MCSIopts)
     char		       *Ranges;
     MCSIopts_t		       *MCSIopts;
{
    MCSIopts_t		       *Opts;
    MCSIopts_t		       *Last = NULL;
    int				Argc;
    char		      **Argv;
    int				i;

    if (!Ranges || !MCSIopts)
	return -1;

    /*
     * Ranges is of form:
     * 	start-end,start-end,...
     */
    Argc = StrToArgv(Ranges, ",", &Argv, NULL, STRTO_SKIPRP);
    for (i = 0; i < Argc; ++i) {
	if (Opts = PortOpts(Argv[i])) {
	    /* Add to MCSIopts */
	    if (!Last) {
		MCSIopts->Next = Last = Opts;
	    } else {
		Last->Next = Opts;
		Last = Opts;
	    }
	}
    }

    return 0;
}

/*
 * Create a new License_t
 */
extern License_t *LicenseCreate(Old)
     License_t		       *Old;
{
    License_t		       *New;

    New = (License_t *) xcalloc(1, sizeof(License_t));

    return New;
}

/*
 * Destroy a License_t
 */
extern int LicenseDestroy(License)
     License_t		       *License;
{
    if (!License)
	return -1;

    if (License->SoftName)	(void) free(License->SoftName);
    if (License->File)		(void) free(License->File);

    (void) free(License);

    return 0;
}

/*
 * Return string name of Type
 */
extern char *LicenseType(Type)
     int			Type;
{
    if (Type < 0 || Type > sizeof(LicenseTypes)/sizeof(char *))
	return (char *) NULL;

    return LicenseTypes[Type];
}

/*
 * Return string name of Type
 */
extern char *LicenseDesc(Type)
     int			Type;
{
    if (Type < 0 || Type > sizeof(LicenseDescs)/sizeof(char *))
	return (char *) NULL;

    return LicenseDescs[Type];
}

/*
 * Fill in any license values we can.
 */
static int LicenseFill(List)
     License_t		       *List;
{
    License_t		       *Lic;

    for (Lic = List; Lic; Lic = Lic->Next) {
	if (Lic->Type && !Lic->TypeStr)
	    Lic->TypeStr = LicenseType(Lic->Type);
	if (Lic->Type && !Lic->TypeDescStr)
	    Lic->TypeDescStr = LicenseDesc(Lic->Type);
    }

    return 0;
}

/*
 * Interface function between mcSysInfo() and underlying probes.
 */
extern int LicenseMCSI(Query)
     MCSIquery_t	       *Query;
{
    static License_t 	       *Root;
    License_t	 	       *Part;
    PSI_t		       *PSIptr;

    if (!Query) {
	errno = ENXIO;
	return -1;
    }

    switch (Query->Op) {
    case MCSIOP_CREATE:
	if (!Root) {
	    if (!PsiSwitch.BuildLicense || !PsiSwitch.BuildLicense->Get) {
		SImsg(SIM_DBG, 
		      "Support for `License' class information is not available on this platform.");
		return 0;
	    }

	    SImsg(SIM_DBG, "BUILDING License Tree ...");
	    /*
	     * Call each defined Build function
	     */
	    for (PSIptr = PsiSwitch.BuildLicense;
		 PSIptr && PSIptr->Get; ++PSIptr) {
		(void) (PSIptr->Get)(&Root, Query->SearchExp, Query->Options);
	    }

	    /* Fill in what we can */
	    LicenseFill(Root);
	}

	if (!Root) {
	    SImsg(SIM_DBG, "No license information was found.");
	    errno = ENOENT;
	    return -1;
	}

	Query->Out = (Opaque_t) Root;
	Query->OutSize = sizeof(License_t *);
	break;
    case  MCSIOP_DESTROY:
	if (Query->Out && Query->OutSize)
	    return LicenseDestroy((License_t *) Query->Out);
	break;
    }

    return 0;
}
