/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.4 $";
#endif

/*
 * Portions of code found in this file are based on the 4.3BSD 
 * netstat(8) program.
 */

/*
 * Network Interface address routines
 */
#include "netif.h"		/* This includes defs.h properly */

/*
 * Interface Address union
 */
union {
    struct ifaddr 	ifaddr;
#if	defined(HAVE_IN_IFADDR)
    struct in_ifaddr 	in_ifaddr;
#endif	/* HAVE_IN_IFADDR */
} 				ifaddress;

/*
 * Set network address data.
 */
extern int NetIfSetAddrs(char *			ifname,
			 int			sock,
			 struct sockaddr_in *	hostaddr,
			 struct sockaddr_in *	maskaddr,
			 struct sockaddr_in *	broadaddr)
{
    struct ifreq		ifr;
    int				Status = 0;

    (void) snprintf(ifr.ifr_name, sizeof(ifr.ifr_name), "%s", ifname);

    /*
     * Get address info
     */
    if (ioctl(sock, SIOCGIFADDR, (void *) &ifr) < 0) {
	SImsg(SIM_GERR, "%s: ioctl SIOCGIFADDR failed: %s.", ifname, SYSERR);
	--Status;
    } else {
	memcpy((void *)hostaddr, (void *)&ifr.ifr_addr, 
	       sizeof(struct sockaddr_in));
    }

    /*
     * Get the network mask
     */
    if (ioctl(sock, SIOCGIFNETMASK, (void *) &ifr) < 0) {
	SImsg(SIM_GERR, "%s: ioctl SIOCGIFNETMASK failed: %s.", 
			 ifname, SYSERR);
	--Status;
    } else {
	memcpy((void *)maskaddr, (void *)&ifr.ifr_addr, 
	       sizeof(struct sockaddr_in));
    }

    /*
     * Get the broadcast addr
     */
    if (ioctl(sock, SIOCGIFBRDADDR, (void *) &ifr) < 0) {
	SImsg(SIM_GERR, "%s: ioctl SIOCGIFBRDADDR failed: %s.", 
			 ifname, SYSERR);
	--Status;
    } else {
	memcpy((void *)broadaddr, (void *)&ifr.ifr_addr, 
	       sizeof(struct sockaddr_in));
    }

    /* If everything failed, return a failed status */
    return (Status == -3) ? -1 : 0;
}

/*
 * Return the netent of the network whose address is given.
 * The address is assumed to be that of a net or subnet, not a host.
 */
static struct netent *GetNet(u_long inaddr, u_long mask)
{
    u_long 			net;
    register u_long 		i, netaddr;
    int 			subnetshift;
    static struct in_addr 	in_addr;

    if (in_addr.s_addr = inaddr) {
	i = in_addr.s_addr;
	if (mask == 0) {
	    if (IN_CLASSA(i)) {
		mask = IN_CLASSA_NET;
		subnetshift = 8;
	    } else if (IN_CLASSB(i)) {
		mask = IN_CLASSB_NET;
		subnetshift = 8;
	    } else {
		mask = IN_CLASSC_NET;
		subnetshift = 4;
	    }
	    /*
	     * If there are more bits than the standard mask
	     * would suggest, subnets must be in use.
	     * Guess at the subnet mask, assuming reasonable
	     * width subnet fields.
	     */
	    while (in_addr.s_addr &~ mask)
		mask = (long)mask >> subnetshift;
	}
	net = in_addr.s_addr & mask;
	while ((mask & 1) == 0)
	    mask >>= 1, net >>= 1;
	netaddr = net;
    } else {
	netaddr = inaddr;
    }

    return(getnetbyaddr(netaddr, AF_INET));
}

/*
 * Get NetIF_t for an Internet address
 */
extern NetIF_t * GetNetifINET(AddrFamily_t * 		AddrFam,
			      struct sockaddr_in * 	hostaddr,
			      struct sockaddr_in * 	maskaddr,
			      struct sockaddr_in * 	broadaddr)
{
    NetIF_t *			NetIf;
    struct in_addr		in_addr;
    struct netent 	       *np;
    struct hostent 	       *hp;
    struct sockaddr_in 	       *sin;
    char		       *inet_ntoa();
    char *			cp;

    NetIf = NetIfNew();

    if (hostaddr && maskaddr) {
	np = GetNet((u_long) hostaddr->sin_addr.s_addr, 
		    (u_long) maskaddr->sin_addr.s_addr);
	in_addr.s_addr = hostaddr->sin_addr.s_addr & 
	  maskaddr->sin_addr.s_addr;
	if ((cp = inet_ntoa(in_addr)) && !EQ(cp, "0.0.0.0"))
	    NetIf->NetAddr = strdup(cp);
	sin = hostaddr;
    } else {
#if	defined(HAVE_IN_IFADDR)
	np = GetNet((u_long) ifaddress.in_ifaddr.ia_subnet, 
		    (u_long) ifaddress.in_ifaddr.ia_subnetmask);
	in_addr.s_addr = ifaddress.in_ifaddr.ia_subnet;
	if ((cp = inet_ntoa(in_addr)) && !EQ(cp, "0.0.0.0"))
	    NetIf->NetAddr = strdup(cp);
	sin = (struct sockaddr_in *) &ifaddress.in_ifaddr.ia_addr;
#endif	/* HAVE_IN_IFADDR */
    }

    if ((cp = inet_ntoa(sin->sin_addr)) && !EQ(cp, "0.0.0.0"))
	NetIf->HostAddr = strdup(cp);

    hp = gethostbyaddr((char *) &(sin->sin_addr),
		       sizeof(struct in_addr), AF_INET);
    if (hp)
	NetIf->HostName = strdup(hp->h_name);

    if (np)
	NetIf->NetName = strdup(np->n_name);
	    
    if (AddrFam)
	NetIf->AddrType = AddrFam->Name;

    if (maskaddr && (cp = inet_ntoa(maskaddr->sin_addr)) && !EQ(cp, "0.0.0.0"))
	NetIf->NetMask = strdup(cp);

    if (broadaddr && (cp = inet_ntoa(broadaddr->sin_addr)) &&
	!EQ(cp, "0.0.0.0"))
	NetIf->BroadAddr = strdup(cp);

    return NetIf;
}

/*
 * Get NetIF_t for an unknown address type
 */
extern NetIF_t * GetNetifUnknown(AddrFamily_t *		AddrFam,
				 struct sockaddr_in * 	hostaddr,
				 struct sockaddr_in * 	maskaddr,
				 struct sockaddr_in * 	broadaddr)
{
    NetIF_t *			NetIf;

    NetIf = NetIfNew();

    if (AddrFam)
	NetIf->AddrType = AddrFam->Name;

    return NetIf;
}
