/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.6 $";
#endif

/*
 * Portions of code found in this file are based on the 4.3BSD 
 * netstat(8) program.
 */

/*
 * Network Interface functions
 */

#include <stdio.h>
#if	defined(HAVE_AUTOCONFIG_H)
#include "mcsysinfo-autoconfig.h"
#endif	/* HAVE_AUTOCONFIG_H */
#include "os.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <net/if.h>

#include "defs.h"

#if HAVE_STRING_H
# include <string.h>
#endif

/*
 * These values are from <net/if.h>
 */
static NetIfFlag_t FlagTable[] = {
    { IFF_UP,		"UP",		"Interface is connected" },
    { IFF_RUNNING,	"RUNNING",	"Interface is running" },
    { IFF_LOOPBACK,	"LOOPBACK",	"This is a loopback interface" },
    { IFF_POINTOPOINT,	"POINT-TO-POINT",	"Point to Point Link" },
#if defined(IFF_NOTRAILERS)
    { IFF_NOTRAILERS,	"NOTRAILERS",	"No trailers" },
#endif
#if defined(IFF_PROMISC)
    { IFF_PROMISC,	"PROMISC",	"Promiscuous mode enabled" },
#endif
#if defined(IFF_MULTICAST)
    { IFF_MULTICAST,	"MULTICAST",	"Multicast enabled" },
#endif
#if defined(IFF_AUTOMEDIA)
    { IFF_AUTOMEDIA,	"AUTOMEDIA",	"Automatically select media" },
#endif
#if defined(IFF_NOARP)
    { IFF_NOARP,	"NOARP",	"Do not arp" },
#endif
#if defined(IFF_ALLMULTI)
    { IFF_ALLMULTI,	"ALLMULTI",	"Receive all multicast packets" },
#endif
#if defined(IFF_FILTMULTI)
    { IFF_FILTMULTI,	"FILTMULTI",	"Filter multicast packets" },
#endif
#if defined(IFF_INTELLIGENT)
    { IFF_INTELLIGENT,	"INTELLIGENT",	"Protocol code on board" },
#endif
#if defined(IFF_BROADCAST)
    { IFF_BROADCAST,	"BROADCAST",	"Broadcast address valid" },
#endif
#if defined(IFF_MULTI_BCAST)
    { IFF_MULTI_BCAST,	"MULTI-BCAST",	"Multicast using broadcast address" },
#endif
#if defined(IFF_UNNUMBERED)
    { IFF_UNNUMBERED,	"UNNUMBERED",	"Non-unique address" },
#endif
#if defined(IFF_DHCPRUNNING)
    { IFF_DHCPRUNNING,	"DHCPRUNNING",	"DHCP controls this interface" },
#endif
#if defined(IFF_PRIVATE)
    { IFF_PRIVATE,	"PRIVATE",	"Do not advertise" },
#endif
#if defined(IFF_IPV4)
    { IFF_IPV4,		"IPv4",		"IP Version 4" },
#endif
#if defined(IFF_IPV6)
    { IFF_IPV6,		"IPv6",		"IP Version 6" },
#endif
#if defined(IFF_FAILED)
    { IFF_FAILED,	"FAILED",	"NIC has failed" },
#endif
#if defined(IFF_STANDBY)
    { IFF_STANDBY,	"STANDBY",	"Standby NIC used on failures" },
#endif
#if defined(IFF_INACTIVE)
    { IFF_INACTIVE,	"INACTIVE",	"Standby active or not?" },
#endif
#if defined(IFF_OFFLINE)
    { IFF_OFFLINE,	"OFFLINE",	"NIC has been offlined" },
#endif
#if defined(IFF_SIMPLEX)
    { IFF_SIMPLEX,	"SIMPLEX",	"Cannot hear own transmissions" },
#endif
#if defined(IFF_VIPA)
    { IFF_VIPA,		"VIRTUAL",	"Virtual Interface" },
#endif
#if defined(IFF_64BIT)
    { IFF_64BIT,	"64BIT",	"64-bit safe" },
#endif
#if defined(IFF_CKSUM)
    { IFF_CKSUM,	"CKSUM",	"Checksumming" },
#endif
#if defined(IFF_ALLCAST)
    { IFF_ALLCAST,	"ALLCAST",	"SRCROUTE Broadcast" },
#endif
#if defined(IFF_DRVLOCK)
    { IFF_DRVLOCK,	"DRVLOCK",	"Driver does its own MP locking" },
#endif
#if defined(IFF_IPALIAS)
    { IFF_IPALIAS,	"IPALIAS",	"Supports IP alias ioctl SIOCAIFADDR"},
#endif
    { 0 },
};

/*
 * Make a copy of Old and return the new ptr.
 */
extern NetIfFlag_t *NetIfFlagCopy(NetIfFlag_t *Old)
{
    NetIfFlag_t *		New;

    New = (NetIfFlag_t *) xcalloc(1, sizeof(NetIfFlag_t));
    New->Flag = Old->Flag;
    if (Old->Name)
	New->Name = strdup(Old->Name);
    if (Old->Desc)
	New->Desc = strdup(Old->Desc);

    return New;
}

/*
 * Set Flags from FlagTable in NetIf
 */
extern int NetIfFlags(NetIF_t *		NetIf,
		      unsigned long	Flags)
{
    NetIfFlag_t *		f;
    NetIfFlag_t *		New;
    NetIfFlag_t *		Last = NULL;

    McDebug("NetIfFlags:\t if=<%s> Flags=0x%X", NetIf->Name, Flags);

    /*
     * Check each flag in the FlagTable to see if its set
     */
    for (f = FlagTable; f && f->Name; ++f) {
	if (FLAGS_ON(Flags, f->Flag)) {
	    McDebug("NetIfFlags:\t if=<%s> has flag=<%s>",
		    NetIf->Name, f->Name);
	    New = NetIfFlagCopy(f);
	    if (NetIf->Flags) {
		/*
		 * Find the end of Flags.
		 * We do this each time since we're called by more than
		 * one NetIf.
		 */
		for (Last = NetIf->Flags; Last && Last->Next;
		     Last = Last->Next);
		Last->Next = New;
	    } else {
		NetIf->Flags = New;
	    }
	}
    }

    return 0;
}
