/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.5 $";
#endif

/*
 * Portions of code found in this file are based on the 4.3BSD 
 * netstat(8) program.
 */

/*
 * Network Interface functions via sysctl(3)
 */

#include "netif.h"
#if HAVE_NET_IF_MEDIA_H
# include <net/if_media.h>
#endif

/*
 * Set Media related info in NetIf using SIOCGIFMEDIA
 */
extern int NetIfSetMedia(NetIF_t *NetIf, int Sock)
{
#if defined(SIOCGIFMEDIA)
    struct ifmediareq		ifmr;
    int *			MediaList;
    char *			mt = NULL;
    char *			st = NULL;
    char *			sp = NULL;

    if (!NetIf)
	return -1;

    McDebug("NetIfSetMedia: IF <%s> checking media properties ...",
		PRTS(NetIf->Name));
    
    (void) memset(&ifmr, 0, sizeof(ifmr));
    (void) strncpy(ifmr.ifm_name, NetIf->Name, sizeof(ifmr.ifm_name));

    /* Query to get basic info about what's available */
    if (ioctl(Sock, SIOCGIFMEDIA, (caddr_t) &ifmr) < 0) {
	McDebug("NetIfSetMedia: IF <%s> ioctl SIOCGIFMEDIA get info failed: %s",
		PRTS(NetIf->Name), SYSERR);
	return -1;
    }

    if (ifmr.ifm_count == 0) {
	McDebug("NetIfSetMedia: IF <%s> Count is 0 -> no media types",
		PRTS(NetIf->Name));
	return -1;
    }

    /* Alloc memory for the media list */
    MediaList = (int *) xmalloc(ifmr.ifm_count * sizeof(int));
    ifmr.ifm_ulist = MediaList;

    /* Fetch the actual data */
    if (ioctl(Sock, SIOCGIFMEDIA, (caddr_t) &ifmr) < 0) {
	McDebug("NetIfSetMedia: IF <%s> ioctl SIOCGIFMEDIA get data failed: %s",
		PRTS(NetIf->Name), SYSERR);
	return -1;
    }

    if (FLAGS_ON(ifmr.ifm_status, IFM_AVALID))
	McDebug("NetIfSetMedia:\t Is %s",
		(FLAGS_ON(ifmr.ifm_status, IFM_ACTIVE)) ? "ACTIVE" : "INACTIVE");

    if (FLAGS_ON(ifmr.ifm_current, IFM_AUTO))
	McDebug("NetIfSetMedia:\t Is auto-select");
    else if (FLAGS_ON(ifmr.ifm_current, IFM_MANUAL))
	McDebug("NetIfSetMedia:\t Is manual-select");

    /*
     * Identify media type and sub type
     * Values are from <sys/if_media.h>
     */
    switch (IFM_TYPE(ifmr.ifm_active)) {
    case IFM_ETHER:
	mt = IFMT_ETHERNET;
	switch (IFM_SUBTYPE(ifmr.ifm_active)) {
	case IFM_10_T:	  st = "10-base-T/UTP";	sp = "10 Mb/s"; break;
	case IFM_10_2:	  st = "10-base-2/BNC";	sp = "10 Mb/s"; break;
	case IFM_10_5:	  st = "10-base-5/AUI";	sp = "10 Mb/s"; break;
	case IFM_100_TX:  st = "100-base-TX"; 	sp = "100 Mb/s"; break;
	case IFM_100_FX:  st = "100-base-FX"; 	sp = "100 Mb/s"; break;
	case IFM_100_T4:  st = "100-base-T4"; 	sp = "100 Mb/s"; break;
	case IFM_100_VG:  st = "100-base-VG"; 	sp = "100 Mb/s"; break;
	case IFM_100_T2:  st = "100-base-T2"; 	sp = "100 Mb/s"; break;
	case IFM_1000_SX: st = "1000-base-SX"; 	sp = "1000 Mb/s"; break;
	case IFM_10_STP:  st = "10-base-STP"; 	sp = "10 Mb/s"; break;
	case IFM_10_FL:	  st = "10-base-FL"; 	sp = "10 Mb/s"; break;
	case IFM_1000_LX: st = "1000-base-LX"; 	sp = "1000 Mb/s"; break;
	case IFM_1000_CX: st = "1000-base-CX"; 	sp = "1000 Mb/s"; break;
#if IFM_1000_TX
	case IFM_1000_TX: st = "1000-base-TX"; 	sp = "1000 Mb/s"; break;
#endif
#if IFM_1000_T
	case IFM_1000_T:  st = "1000-base-TX"; 	sp = "1000 Mb/s"; break;
#endif
#if IFM_HPNA_1
	case IFM_HPNA_1:  st = "Home-PNA-1"; 	sp = "10 Mb/s"; break;
#endif
	default:
	    McDebug("NetIfSetMedia:\t Unknown ether subtype 0x%X",
		    IFM_SUBTYPE(ifmr.ifm_active));
	}
	break;
    case IFM_TOKEN:
	mt = IFMT_TOKEN;
	switch (IFM_SUBTYPE(ifmr.ifm_active)) {
	case IFM_TOK_STP4:  st = "STP4/DB9";	sp = "4 Mb/s"; break;
	case IFM_TOK_STP16: st = "STP16/DB9";	sp = "16 Mb/s"; break;
	case IFM_TOK_UTP4:  st = "UTP4/DB9";	sp = "4 Mb/s"; break;
	case IFM_TOK_UTP16: st = "UTP16/DB9";	sp = "16 Mb/s"; break;
	default:
	    McDebug("NetIfSetMedia:\t Unknown token subtype 0x%X",
		    IFM_SUBTYPE(ifmr.ifm_active));
	}
	break;
    case IFM_FDDI:
	mt = IFMT_FDDI;
	switch (IFM_SUBTYPE(ifmr.ifm_active)) {
	case IFM_FDDI_SMF:  st = "SMF";	sp = "100 Mb/s"; break;
	case IFM_FDDI_MMF:  st = "SMF";	sp = "100 Mb/s"; break;
	case IFM_FDDI_UTP:  st = "CDDI"; sp = "100 Mb/s"; break;
	default:
	    McDebug("NetIfSetMedia:\t Unknown fddi subtype 0x%X",
		    IFM_SUBTYPE(ifmr.ifm_active));
	}
	break;
#if IFM_IEEE80211
    case IFM_IEEE80211:
	mt = IFMT_80211;
	break;
#endif
    default:
	McDebug("NetIfSetMedia:\t Unknown media type 0x%X",
		IFM_TYPE(ifmr.ifm_active));
    }

    /* Set what we just learned */
    McDebug("NetIfSetMedia:\t Media Type=<%s> SubType=<%s> Speed=<%s>",
	    PRTS(mt), PRTS(st), PRTS(sp));
    if (mt)
	NetIf->MediaType = strdup(mt);
    if (st)
	NetIf->MediaSubType = strdup(st);
    if (sp)
	NetIf->Speed = strdup(sp);

    /* Set duplex */
    if (FLAGS_ON(ifmr.ifm_active, IFM_FDX))
	NetIf->Duplex = strdup(IFD_FULL);
    else if (FLAGS_ON(ifmr.ifm_active, IFM_HDX))
	NetIf->Duplex = strdup(IFD_HALF);

    (void) free(MediaList);
#endif	/* SIOCGIFMEDIA */
    return 0;
}
