/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.19 $";
#endif

/*
 * Portions of code found in this file are based on the 4.3BSD 
 * netstat(8) program.
 */

/*
 * Network Interface functions
 */
#include "netif.h"

#if	GETNETIF_TYPE == GETNETIF_IFCONF

/*
 * Set info from flags 
 */
static int NetIfSetFlags(NetIF_t *	NetIf,
			 int		Sock)
{
#if HAVE_STRUCT_LIFREQ
    /* Use the "Large" size ifreq */
    struct lifreq		ifr;
    uint64_t			Flags;
#else
    /* Use the "regular" size ifreq */
    struct ifreq		ifr;
    IFRU_FLAGS_TYPE		Flags;
#endif

#if HAVE_STRUCT_LIFREQ
    (void) strcpy(ifr.lifr_name, NetIf->Name);
#else
    (void) strcpy(ifr.ifr_name, NetIf->Name);
#endif

    /*
     * Get flags
     */
#if HAVE_STRUCT_LIFREQ
    if (ioctl(Sock, SIOCGLIFFLAGS, (void *) &ifr) < 0) {
#else
    if (ioctl(Sock, SIOCGIFFLAGS, (void *) &ifr) < 0) {
#endif
	SImsg(SIM_GERR, "%s: ioctl SIOCGIFFLAGS failed: %s.",
	      NetIf->Name, SYSERR);
	return(-1);
    }

#if HAVE_STRUCT_LIFREQ
    memcpy((void *)&Flags, (void *)&ifr.lifr_flags, sizeof(Flags));
#else
    memcpy((void *)&Flags, (void *)&ifr.ifr_flags, sizeof(Flags));
#endif

    McDebug("NetIfSetFlags:\t if=<%s> Flags=0x%X", NetIf->Name, Flags);

    /* Actually set Flags into NetIf */
    NetIfFlags(NetIf, (unsigned long) Flags);

    return 0;
}

/*
 * Return a cleaned IfName for Name
 */
static char *IfName(char *Name)
{
    static char			Buff[128];
    char *			cp;

    snprintf(Buff, sizeof(Buff), "%s", Name);
    for (cp = Name; cp && *cp; ++cp)
	/* The first non printable we find, set NULL and stop */
	if (!isprint(*cp)) {
	    *cp = CNULL;
	    break;
	}

    return Buff;
}

/*
 * Use the SIOCGIFCONF interface to retrieve network interface info
 */
extern Opaque_t NetIfBuild(NetIF_t **		ListPtr,
			   char **		SearchExpr)
{
    NetIF_t *			Last = NULL;
    NetIF_t *	 	        NetIf = NULL;
    NetIF_t *	 	        ni;
    struct ifreq	       *ifreq;
    static struct ifconf	ifconf;
    static struct sockaddr_in	HostAddr;
    static struct sockaddr_in	MaskAddr;
    static struct sockaddr_in	BroadAddr;
    static char			ReqBuff[BUFSIZ];
    static int			SockDesc = -1;
    char *			Name;
    AddrFamily_t	       *AddrFamPtr;
    int				Found = 0;
    int				Dup;
    int				n;

    if (!ListPtr) {
	McDebug("NetIfBuild: Missing params.");
	return (Opaque_t) -1;
    }

    McDebug("NetIfBuild: Building list of network interfaces using ifconf . . .");

    if (SockDesc < 0) {
	/*
	 * Get list of all interfaces
	 */
	ifconf.ifc_len = sizeof(ReqBuff);
	ifconf.ifc_buf = ReqBuff;

	if ((SockDesc = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
	    SImsg(SIM_GERR, "NetIfBuild: Cannot create socket: %s.", SYSERR);
	    return (Opaque_t) -1;
	}

	if (ioctl(SockDesc, SIOCGIFCONF, (void *) &ifconf) < 0) {
	    SImsg(SIM_GERR, "NetIfBuild: ioctl SIOCGIFCONF failed: %s.", 
			     SYSERR);
	    return (Opaque_t) -1;
	}
    }

    /*
     * Iterate over all known interfaces
     */
    for (ifreq = ifconf.ifc_req, n = ifconf.ifc_len/sizeof(struct ifreq);
	 --n >= 0; ifreq++) {

	NetIf = NULL;
	Name = IfName(ifreq->ifr_name);

	McDebug("NetIfBuild: ifconf found IF=<%s>", Name);

	/* Validity checks */
	if (strlen(Name) < 2) {
	    McDebug("NetIfBuild:\t skipping short length IF name.");
	    continue;
	}
	if (!isalnum(ifreq->ifr_name[0])) {
	    McDebug("NetIfBuild:\t skipping bogus IF name.");
	    continue;
	}
	/* Check to see if we already have this IF */
	for (Dup = FALSE, ni = *ListPtr; ni; ni = ni->Next)
	    if (EQ(ni->Name, Name)) {
		McDebug("NetIfBuild:\t Skipping duplicate IF name");
		Dup = TRUE;
		break;
	    }
	if (Dup)
	    continue;

	++Found;

	/*
	 * Set the Host and Mask address info
	 */
	if (NetIfSetAddrs(Name, SockDesc,
			  &HostAddr, &MaskAddr, &BroadAddr) == 0) {
	    /*
	     * Now get and call the Address Family specific routine
	     * to extract a NetIF_t.
	     */
	    if (AddrFamPtr = GetAddrFamily(HostAddr.sin_family)) {
		if (NetIf = (*AddrFamPtr->GetNetIF)(AddrFamPtr,
						    &HostAddr, 
						    &MaskAddr,
						    &BroadAddr)) {
		    NetIf->Name = strdup(Name);
		    SetMacInfo(NetIf);
		}
	    } else {
		McDebug("NetIfBuild:\t Address family %d is not defined.", 
			HostAddr.sin_family);
	    }
	}

	if (!NetIf)
	    NetIf = NetIfNew();
	if (!NetIf->Name)
	    NetIf->Name = strdup(Name);
	NetIfTypeSet(NetIf);

	/* Add info on what flags are set for this IF */
	NetIfSetFlags(NetIf, SockDesc);

	/* Add info on MTU */
	NetIfSetMtu(NetIf, SockDesc);
	
	McDebug("NetIfBuild:\t if=<%s> ip=<%s> host=<%s> MAC=<%s> Net=<%s> Mask=<%s>",
		PRTS(NetIf->Name),
		PRTS(NetIf->HostAddr),
		PRTS(NetIf->HostName),
		PRTS(NetIf->MACaddr),
		PRTS(NetIf->NetAddr),
		PRTS(NetIf->NetMask));

#if defined(NETIF_BUILD_OS)
	/* Call the OS specific function to set additional netif info */
	NETIF_BUILD_OS(NetIf);
#endif

	/*
	 * Add to the ListPtr
	 */
	if (Last) {
	    Last->Next = NetIf;
	    Last = NetIf;
	} else {
	    *ListPtr = Last = NetIf;
	}
    }

    McDebug("NetIfBuild: Found %d interfaces", Found);

    return 0;
}
#endif	/* GETNETIF_TYPE == GETNETIF_IFCONF */
