/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.6 $";
#endif

/*
 * Portions of code found in this file are based on the 4.3BSD 
 * netstat(8) program.
 */

/*
 * Network Interface functions
 */

#include <stdio.h>
#if	defined(HAVE_AUTOCONFIG_H)
#include "mcsysinfo-autoconfig.h"
#endif	/* HAVE_AUTOCONFIG_H */
#include "os.h"

#include <sys/types.h>
#include <sys/socket.h>
#if	defined(HAVE_SYS_SOCKIO_H)
#include <sys/sockio.h>
#endif	/* HAVE_SYS_SOCKIO_H */
#include <sys/param.h>
#include <sys/errno.h>
#include <netdb.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#if	defined(HAVE_IN_IFADDR)
#include <netinet/in_var.h>
#endif	/* HAVE_IN_IFADDR */
#if	defined(HAVE_ARPA_INET_H)
#include <arpa/inet.h>
#endif	/* HAVE_ARPA_INET_H */

#include "defs.h"

#if	GETNETIF_TYPE == GETNETIF_IFNET
/*
 * Interface Address union
 */
union {
    struct ifaddr 	ifaddr;
#if	defined(HAVE_IN_IFADDR)
    struct in_ifaddr 	in_ifaddr;
#endif	/* HAVE_IN_IFADDR */
} 				ifaddress;

/*
 * Network Interface symbol
 */
char 				NetifSYM[] = "_ifnet";

/*
 * Set a linked list of NetIF_t's for each address starting at 'startaddr'.
 */
static NetIF_t * GetAddrs(kvm_t *	kd,
			  off_t		startaddr,
			  char *	FullName)
{
    u_long			addr;
    NetIF_t *		        base = NULL;
    NetIF_t *			ni;
    NetIF_t *			pni;
    AddrFamily_t *	        AddrFamPtr;

    for (addr = startaddr; addr; addr = (u_long) ifaddress.ifaddr.ifa_next) {
	/*
	 * Read the ifaddr structure from kernel space
	 */
	if (KVMget(kd, addr, (void *) &ifaddress, 
		   sizeof(ifaddress), KDT_DATA)) {
	    SImsg(SIM_GERR, "SetAddrs: cannot read if address");
	    continue;
	}

	/*
	 * Now get and call the Address Family specific routine
	 * to extract a NetIF_t.
	 */
	if (AddrFamPtr = GetAddrFamily(ifaddress.ifaddr.ifa_addr.sa_family)) {
	    if (ni = (*AddrFamPtr->GetNetIF)(AddrFamPtr,
					     NULL, NULL, NULL)) {
		ni->Name = strdup(FullName);
		SetMacInfo(ni);
	    }
	} else {
	    McDebug("SetAddrs: Address family %d is not defined.", 
		    ifaddress.ifaddr.ifa_addr.sa_family);
	    continue;
	}

	if (!ni)
	    continue;

	/*
	 * Add the new NetIF_t to the base of the linked list.
	 */
	if (base) {
	    for (pni = base; pni && pni->Next; pni = pni->NextAddr);
	    pni->NextAddr = ni;
	} else {
	    base = ni;
	}
    }

    return(base);
}

/*
 * Build list of NetIf's by looking up "ifnet" table in kernel.
 */
extern Opaque_t NetIfBuild(NetIF_t **		ListPtr,
			   char **		SearchExpr)
{
    NetIF_t *			Last = NULL;
    NetIF_t *	 	        NetIf;
    u_long 		        ifnetaddr;
    kvm_t * 		        kd;
    static char 		ifname[16], FullName[17];
    static struct ifnet         ifnet;
    int				Found = 0;

    McDebug("NetIfBuild: Building list of network interfaces using ifnet . . .");

    if (!(kd = KVMopen()))
	return((DevInfo_t *) NULL);

    /*
     * Get the address of the ifnetaddr structure
     */
    if (KVMgetSymbol(kd,
		     NetifSYM,
		     &ifnetaddr, sizeof(ifnetaddr),
		     KDT_DATA) < 0) {
	SImsg(SIM_DBG, "NetIfBuild: Get kvm address for <%s> failed.",
	      NetifSYM);
	KVMclose(kd);
	return((DevInfo_t *) NULL);
    }

    /*
     * Read and then check each ifnet entry we found.
     */
    for (; ifnetaddr; ifnetaddr = (off_t) ifnet.if_next) {
	/*
	 * Read the ifnet structure
	 */
	if (KVMget(kd, ifnetaddr, (void *)&ifnet, sizeof(ifnet), KDT_DATA)) {
	    SImsg(SIM_GERR, "kvm_read ifnetaddr ifnet failed");
	    continue;
	}

	/*
	 * Read if_name from kernel space
	 */
	if (KVMget(kd, (u_long)ifnet.if_name, ifname, 
		   sizeof(ifname), KDT_STRING)) {
	    SImsg(SIM_GERR, "kvm_read ifname failed");
	    continue;
	}

	++Found;

	/*
	 * Copy if_name to 'FullName' and add unit number
	 */
	(void) snprintf(FullName, sizeof(FullName), "%s%d", 
			ifname, ifnet.if_unit);

	if (ifnet.if_addrlist)
	    NetIf = GetAddrs(kd, (off_t) ifnet.if_addrlist, FullName);

	if (!NetIf)
	    NetIf = NetIfNew();
	if (!NetIf->Name)
	    NetIf->Name = strdup(FullName);
	NetIfTypeSet(NetIf);
	NetIf->Unit = ifnet.if_unit;
#if	defined(HAVE_IF_VERSION)
	if (ifnet.if_version && ifnet.if_version[0])
	    NetIf->Model 	= strdup(ifnet.if_version);
#endif	/* HAVE_IF_VERSION */

	McDebug("NetIfBuild:\t if=<%s> ip=<%s> host=<%s> MAC=<%s> Net=<%s> Mask=<%s>",
		PRTS(NetIf->Name),
		PRTS(NetIf->HostAddr),
		PRTS(NetIf->HostName),
		PRTS(NetIf->MACaddr),
		PRTS(NetIf->NetAddr),
		PRTS(NetIf->NetMask));

	/*
	 * Add to the ListPtr
	 */
	if (Last) {
	    Last->Next = NetIf;
	    Last = NetIf;
	} else {
	    *ListPtr = Last = NetIf;
	}
    }

    McDebug("NetIfBuild: Found %d interfaces", Found);

    return 0;
}

#endif	/* GETNETIF_TYPE == GETNETIF_IFNET */

