/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.4 $";
#endif

/*
 * Portions of code found in this file are based on the 4.3BSD 
 * netstat(8) program.
 */

/*
 * Network Interface probe functions
 */

#include "defs.h"

/*
 * Set misc netif info
 */
extern int NetifSet(DevInfo)
     DevInfo_t		       *DevInfo;
{
    char		       *cp;

    /*
     * Use class to set speed
     */
    cp = NULL;
    switch (DevInfo->ClassType) {
    case CT_ETHER10M:		cp = "10 Mb/s";		break;
    case CT_ETHER100M:		cp = "100 Mb/s";	break;
    case CT_ETHER1G:		cp = "1 Gb/s";		break;
    default:
	if (DevInfo->ClassType > 0)
	    SImsg(SIM_DBG, "NetifSet: Unknown classtype=%d",
		  DevInfo->ClassType);
    }
    if (cp)
	DevInfo->Speed = strdup(cp);

    return 0;
}

/*
 * See if IfName matches any names in ProbeData and return name
 * found.
 */
static int IFmatch(ProbeData, IfName)
     ProbeData_t	       *ProbeData;
     char		       *IfName;
{
    register char	      **cpp;

    if (!ProbeData)
	return(FALSE);

    if (EQ(ProbeData->DevName, IfName))
	return(TRUE);

    for (cpp = ProbeData->AliasNames; cpp && *cpp; ++cpp)
	if (EQ(*cpp, IfName))
	    return(TRUE);

    return(FALSE);
}

/*
 * Query/find network interface devices and add them to devicelist
 */
extern DevInfo_t *ProbeNetif(ProbeData_t *ProbeData)
{
    DevInfo_t 		       *DevInfo = NULL;
    MCSIquery_t			Query;
    NetIF_t *			List;
    NetIF_t *			NetIf;
    char *			DevName;
    register char	      **cpp;
    register char 	       *Alias = NULL;
    DevData_t 		       *DevData;
    DevDefine_t	 	       *DevDefine;
    int				NameMatch = FALSE;

    DevName = ProbeData->DevName;
    DevData = ProbeData->DevData;
    DevDefine = ProbeData->DevDefine;
    DevInfo = ProbeData->UseDevInfo;

    McDebug("ProbeNetif(%s)", DevName);

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Op = MCSIOP_CREATE;
    Query.Cmd = MCSI_NETIF;
    if (mcSysInfo(&Query) != 0) {
	McDebug("ProbeNetif: Cannot retrieve NetIf list.");
	return (DevInfo_t *) NULL;
    }

    List = (NetIF_t *) Query.Out;

    /*
     * Scan the list of all NetIf's looking for a match
     */
    for (NetIf = List; NetIf; NetIf = NetIf->Next) {
	/*
	 * Compare the hardware DevName as passed to us, the config file name
	 * and all config file aliases.
	 * Check to see if this is the interface we want.
	 */
	if (IFmatch(ProbeData, NetIf->Name)) {
	    NameMatch = TRUE;
	    break;
	}

	if (DevDefine && DevData && !NameMatch) {
	    /* Check config file name */
	    Alias = MkDevName(DevDefine->Name, DevData->DevUnit,
			      DevDefine->Type, DevDefine->Flags);
	    if (EQ(Alias, NetIf->Name)) {
		NameMatch = TRUE;
		break;
	    }
	  
	    /* Check all name aliases */
	    for (cpp = DevDefine->Aliases; !NameMatch && cpp && *cpp; ++cpp) {
		Alias = MkDevName(*cpp, DevData->DevUnit,
				  DevDefine->Type, DevDefine->Flags);
		if (EQ(Alias, NetIf->Name)) {
		    NameMatch = TRUE;
		    break;
		}
	    }
	}
    }

    if (NameMatch) {
	/* Use the system's canonical name */
	ProbeData->DevName = NetIf->Name;
	if (!(DevInfo = DeviceCreate(ProbeData)))
	    return((DevInfo_t *) NULL);

	DevInfo->Type = DT_NETIF;
	if (NetIf->Unit >= 0)
	    DevInfo->Unit = NetIf->Unit;
	if (!DevInfo->Model && NetIf->Model)
	    DevInfo->Model = strdup(NetIf->Model);
	if (!DevInfo->DevSpec)
	    DevInfo->DevSpec = (void *) NetIf;
    } else {
	/*
	 * Device not found.  This can occur if the device exists, but
	 * is not "plumbed" by the OS which is often the case on Solaris.
	 */
	SImsg(SIM_DBG, "%s: Could not find netif in ifconf table.", DevName);

	if (!(DevInfo = DeviceCreate(ProbeData)))
	    return((DevInfo_t *) NULL);

	DevInfo->Type = DT_NETIF;
    }

    NetifSet(DevInfo);

    return(ProbeData->RetDevInfo = DevInfo);
}
