/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.3 $";
#endif

/*
 * Portions of code found in this file are based on the 4.3BSD 
 * netstat(8) program.
 */

/*
 * Network Interface functions via sysctl(3)
 */

#include "netif.h"

#if	GETNETIF_TYPE == GETNETIF_SYSCTL

#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/sysctl.h>

#include <net/ethernet.h>
#include <net/if.h>
#include <net/if_dl.h>
#include <net/if_types.h>
#include <net/route.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

/*
 * Set what we know for Name
 */
static NetIF_t * GetIf(char *			Name,
		       struct if_msghdr *	ifm)
{
    NetIF_t *			NetIf = NULL;
    int				Sock;
    struct ifreq		ifr;
    static struct sockaddr_in	HostAddr;
    static struct sockaddr_in	MaskAddr;
    static struct sockaddr_in	BroadAddr;
    AddrFamily_t	       *AddrFamPtr;

    /*
     * Setup ifr
     */
    ifr.ifr_addr.sa_family = AF_INET;	/* XXX Only fetch IPv4 */
    snprintf(ifr.ifr_name, sizeof(ifr.ifr_name), "%s", Name);

    /* Create a socket for our queries */
    Sock = socket(ifr.ifr_addr.sa_family, SOCK_DGRAM, 0);
    if (Sock < 0) {
	McDebug("GetIf: Create socket failed: %s", SYSERR);
	return (NetIF_t *) NULL;
    }

    /*
     * Set address info
     */
    if (NetIfSetAddrs(Name, Sock,
		      &HostAddr, &MaskAddr, &BroadAddr) == 0) {
	/*
	 * Now get and call the Address Family specific routine
	 * to extract a NetIF_t.
	 */
	if (AddrFamPtr = GetAddrFamily(HostAddr.sin_family)) {
	    if (NetIf = (*AddrFamPtr->GetNetIF)(AddrFamPtr,
						&HostAddr, 
						&MaskAddr,
						&BroadAddr)) {
		NetIf->Name = strdup(Name);
		SetMacInfo(NetIf);
	    }
	} else {
	    McDebug("NetIfBuild:\t Address family %d is not defined.", 
		    HostAddr.sin_family);
	}
    }

    if (!NetIf)
	NetIf = NetIfNew();
    if (!NetIf->Name)
	NetIf->Name = strdup(Name);
    NetIfTypeSet(NetIf);

    /* Add info on what flags are set for this IF */
    NetIfFlags(NetIf, ifm->ifm_flags);

    /* Add info on MTU */
    NetIfSetMtu(NetIf, Sock);

    /* Add media info */
    NetIfSetMedia(NetIf, Sock);

    /* We're done with the socket now */
    (void) close(Sock);

    McDebug("NetIfBuild:\t if=<%s> ip=<%s> host=<%s> MAC=<%s> Net=<%s> Mask=<%s>",
	    PRTS(NetIf->Name),
	    PRTS(NetIf->HostAddr),
	    PRTS(NetIf->HostName),
	    PRTS(NetIf->MACaddr),
	    PRTS(NetIf->NetAddr),
	    PRTS(NetIf->NetMask));

#if defined(NETIF_BUILD_OS)
    /* Call the OS specific function to set additional netif info */
    NETIF_BUILD_OS(NetIf);
#endif

    return NetIf;
}

/*
 * Use the SIOCGIFCONF interface to retrieve network interface info
 */
extern Opaque_t NetIfBuild(NetIF_t **		ListPtr,
			   char **		SearchExpr)
{
    NetIF_t *			Last = NULL;
    NetIF_t *			ni = NULL;
    struct if_msghdr *		ifm;
    struct if_msghdr *		ifmNext;
    struct sockaddr_dl *	sdl;
    int				mib[6];
    size_t			Needed;
    char *			Buffer;
    char *			Next;
    char *			Limit;
    static char			Name[64];

    mib[0] = CTL_NET;
    mib[1] = PF_ROUTE;
    mib[2] = 0;
    mib[3] = 0;	/* address family */
    mib[4] = NET_RT_IFLIST;
    mib[5] = 0;

    /* Get size needed for actual data */
    if (sysctl(mib, sizeof(mib)/sizeof(int), NULL, &Needed, NULL, 0) < 0) {
	McDebug("NetIfBuild: Get size estimate from sysctl failed: %s",
		SYSERR);
	return (Opaque_t) -1;
    }

    /* Alloc buffer according to what's Needed */
    Buffer = (char *) xmalloc(Needed);

    /* Grab the actual data */
    if (sysctl(mib, sizeof(mib)/sizeof(int), Buffer, &Needed, NULL, 0) < 0) {
	McDebug("NetIfBuild: Get NetIf data from sysctl failed: %s",
		SYSERR);
	return (Opaque_t) -1;
    }

    /*
     * Scan through Buffer looking for interfaces.
     */
    Limit = Buffer + Needed;
    for (Next = Buffer; Next < Limit; ) {
	ifm = (struct if_msghdr *) Next;

	/* Verify the type */
	if (ifm->ifm_type != RTM_IFINFO) {
	    McDebug("NetIfBuild: Out of sync parsing NET_RT_IFLIST.");
	    return (Opaque_t) -1;
	}

	/* Get the interface name */
	sdl = (struct sockaddr_dl *) (ifm + 1);
	snprintf(Name, sizeof(Name), "%.*s", sdl->sdl_nlen, sdl->sdl_data);

	McDebug("NetIfBuild: Found IF <%s>", Name);

	/*
	 * Advance to next entry
	 */
	Next += ifm->ifm_msglen;
	while (Next < Limit) {
	    ifmNext = (struct if_msghdr *) Next;
	    if (ifmNext->ifm_type != RTM_NEWADDR)
		/* Found next entry */
		break;

	    Next += ifmNext->ifm_msglen;
	}

	/*
	 * Get NetIf info for this IF, then add it to the list
	 */
	if (ni = GetIf(Name, ifm)) {
	    if (Last) {
		Last->Next = ni;
		Last = ni;
	    } else {
		*ListPtr = Last = ni;
	    }
	}
    }

    if (Buffer)
	(void) free(Buffer);

    return (Opaque_t) 0;
}
#endif	/* GETNETIF_TYPE == GETNETIF_SYSCTL */
