/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.16 $";
#endif

/*
 * Portions of code found in this file are based on the 4.3BSD 
 * netstat(8) program.
 */

/*
 * Network Interface class routines
 */
#include "netif.h"

/*
 * Set MTU
 */
extern int NetIfSetMtu(NetIF_t *	NetIf,
		       int		Sock)
{
#if HAVE_IFREQ_MTU
#    if HAVE_STRUCT_LIFREQ
    struct lifreq		ifr;
#    else
    struct ifreq		ifr;
#    endif
    int				Mtu;

#    if HAVE_STRUCT_LIFREQ
    (void) strcpy(ifr.lifr_name, NetIf->Name);
#    else
    (void) strcpy(ifr.ifr_name, NetIf->Name);
#    endif

    /*
     * Get MTU
     */
#    if HAVE_STRUCT_LIFREQ
    if (ioctl(Sock, SIOCGLIFMTU, (void *) &ifr) < 0) {
#    else
    if (ioctl(Sock, SIOCGIFMTU, (void *) &ifr) < 0) {
#    endif
	SImsg(SIM_GERR, "%s: ioctl SIOCGIFMTU failed: %s.",
	      NetIf->Name, SYSERR);
	return(-1);
    }

#    if HAVE_STRUCT_LIFREQ
    memcpy((void *)&Mtu, (void *)&ifr.lifr_mtu, sizeof(Mtu));
#    else
    memcpy((void *)&Mtu, (void *)&ifr.ifr_mtu, sizeof(Mtu));
#    endif

    McDebug("NetIfSetMtu:\t if=<%s> MTU=%d", NetIf->Name, Mtu);

    NetIf->MtuSize = Mtu;

    return 0;
#else	/* !HAVE_IFREQ_MTU */
    return -1;
#endif	/* HAVE_IFREQ_MTU */
}

/*
 * Set the interface type
 */
extern int NetIfTypeSet(NetIF_t *NetIf)
{
    if (!NetIf || NetIf->IfType)
	return -1;

    /*
     * Any loopback interface or interface with a ":" in it is a virtual.
     */
    if (strchr(NetIf->Name, ':') ||
	EQ(NetIf->Name, "lo") || EQ(NetIf->Name, "lo0"))
	NetIf->IfType = strdup(IFT_VIRTUAL);
    else
	NetIf->IfType = strdup(IFT_PHYSICAL);

    return 0;
}

/*
 * Create and initialize new NetIF_t
 */
extern NetIF_t *NetIfNew(void)
{
    NetIF_t *		New;

    New = (NetIF_t *) xcalloc(1, sizeof(NetIF_t));
    New->Unit = -1;
    New->MtuSize = -1;

    return New;
}

/*
 * Destroy a NetIF
 */
extern int NetIfDestroy(NetIF_t *List)
{
    NetIF_t *			Net;
    NetIF_t *			Old;
    NetIfFlag_t *		f;

    for (Net = List; Net; ) {
#define FR(a)	if (a) (void) free(a)
	FR(Net->Name);
	FR(Net->IfType);
	FR(Net->AddrType);
	FR(Net->HostAddr);
	FR(Net->HostName);
	FR(Net->MACaddr);
	FR(Net->MACname);
	FR(Net->FacMACaddr);
	FR(Net->FacMACname);
	FR(Net->NetAddr);
	FR(Net->NetName);
	FR(Net->NetMask);
	FR(Net->MediaType);
	FR(Net->MediaSubType);
	FR(Net->Speed);
	FR(Net->Model);
	FR(Net->Duplex);
	for (f = Net->Flags; f; f = f->Next) {
	    if (f->Name)
		(void) free(f->Name);
	    if (f->Desc)
		(void) free(f->Desc);
	    (void) free(f);
	}
	FR(Net->Flags);
#undef  FR
	Old = Net;
	(void) free(Old);
	Net = Net->Next;
    }

    return 0;
}

/*
 * Interface function between mcSysInfo() and underlying probes.
 */
extern int NetIfMCSI(MCSIquery_t *Query)
{
    static NetIF_t 	       *Root;
    PSI_t		       *PSIptr;

    if (!Query) {
	errno = ENXIO;
	return -1;
    }

    switch (Query->Op) {
    case MCSIOP_CREATE:
	if (!Root) {
	    if (!PsiSwitch.BuildNetIf || !PsiSwitch.BuildNetIf->Get) {
		SImsg(SIM_DBG, 
		      "Support for `NetIf' class information is not available on this platform.");
		return 0;
	    }

	    SImsg(SIM_DBG, "BUILDING NetIf Tree ...");
	    /*
	     * Call each defined Build function
	     */
	    for (PSIptr = PsiSwitch.BuildNetIf;
		 PSIptr && PSIptr->Get; ++PSIptr) {
		(void) (PSIptr->Get)(&Root, Query->SearchExp);
	    }
	}

	if (!Root) {
	    SImsg(SIM_DBG, "No NetIf information was found.");
	    errno = ENOENT;
	    return -1;
	}

	Query->Out = (Opaque_t) Root;
	Query->OutSize = sizeof(NetIF_t *);
	break;
    case  MCSIOP_DESTROY:
	if (Query->Out && Query->OutSize)
	    return NetIfDestroy((NetIF_t *) Query->Out);
	break;
    }

    return 0;
}
