/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.4 $";
#endif

/*
 * Network Information - gateway
 */

#include "defs.h"
#include "cmd.h"

/*
 * Paths to possible locations for netstat(1)
 */
static char *NetstatPaths[] = {
    "/bin/netstat", "/usr/bin/netstat", "/sbin/netstat", "/usr/sbin/netstat",
    "/etc/netstat", "/usr/etc/netstat", NULL
};

/*
 * Determine the default gateway.
 * We parse the output from netstat(1) as that's more portable and easy
 * than grabbing the routing table directly.
 */
extern int NetworkGateway(Network_t *Net)
{
    static char		Line[1024];
    static Cmd_t	Cmd;
    char *		Argv[4];
    char *		cp;
    char **		Fields;
    int			FieldsCnt;

    if (!Net)
	return -1;

    McDebug("NetworkGateway: Searching for default gateway ...");

    CmdInit(&Cmd);
    Cmd.CmdPath = NetstatPaths;
    Cmd.TimeOut = 30;			/* XXX Timeout */
    Cmd.Flags = CMF_READ;
    Argv[1] = "-r";			/* Show routing table */
    Argv[2] = "-n";			/* Output IP addrs, not hostnames */
    Argv[3] = NULL;
    Cmd.Argv = Argv;
    Cmd.Argc = sizeof(Argv)/sizeof(char *);;

    if (CmdOpen(&Cmd) < 0) {
	McDebug("NetworkGateway: Pipe open to %s failed: %s",
		(Cmd.Program) ? Cmd.Program : "netstat", SYSERR);
	return -1;
    }

    /*
     * Look for a line which looks like:
     *		0.0.0.0	10.1.2.3 ...
     *  	default 10.1.2.3 ...
     */
    while (fgets(Line, sizeof(Line), Cmd.FH)) {
	if (cp = strchr(Line, '\n'))
	    *cp = CNULL;

	McDebug("NetworkGateway: Line: %s", Line);

	FieldsCnt = StrToArgv(Line, " ", &Fields, NULL, STRTO_SKIPRP);
	if (FieldsCnt > 2 && (EQ(Fields[0], "0.0.0.0") ||
			      EQ(Fields[0], "default"))) {
	    McDebug("NetworkGateway:\t Found Dest=<%s> Gateway=<%s>",
		    Fields[0], Fields[1]);
	    Net->DefGatewayIP = strdup(Fields[1]);
	    break;
	}
    }

    CmdClose(&Cmd);

    if (Net->DefGatewayIP) {
	/* Lookup the name */
	if (cp = AddrStrToHost(Net->DefGatewayIP))
	    Net->DefGatewayName = strdup(cp);
	/* Success */
	return 0;
    }

    McDebug("NetworkGateway: Could not find default gateway.");

    /* Failure */
    return -1;
}
