/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.8 $";
#endif

/*
 * Network Information
 */

#include "defs.h"

/*
 * Create a new Network_t
 */
extern Network_t *NetworkNew(void)
{
    Network_t *			New;

    New = (Network_t *) xcalloc(1, sizeof(Network_t));

    return New;
}

/*
 * Destroy Net
 */
static int NetworkDestroy(Network_t *Net)
{
    if (!Net)
	return 0;

    if (Net->DefGatewayIP)
	(void) free(Net->DefGatewayIP);
    if (Net->DefGatewayName)
	(void) free(Net->DefGatewayName);
    if (Net->DnsDomain)
	(void) free(Net->DnsDomain);
    if (Net->DnsServerIPs) {
	ArrayFree(Net->DnsServerIPs);
	(void) free(Net->DnsServerIPs);
    }
    if (Net->DnsServerNames) {
	ArrayFree(Net->DnsServerNames);
	(void) free(Net->DnsServerNames);
    }
    if (Net->NisDomain)
	(void) free(Net->NisDomain);
    if (Net->NisServerIPs) {
	ArrayFree(Net->NisServerIPs);
	(void) free(Net->NisServerIPs);
    }
    if (Net->NisServerNames) {
	ArrayFree(Net->NisServerNames);
	(void) free(Net->NisServerNames);
    }
    if (Net->NisServerCurrent)
	(void) free(Net->NisServerCurrent);

    return 0;
}

/*
 * Build up Network
 */
extern Opaque_t NetworkBuild(Network_t **	NetworkPtr,
			     MCSIquery_t *	Query)
{
    Network_t *			Net;
    int				Found = 0;
    long			Types;

    if (!NetworkPtr)
	return (Opaque_t) -1;

    Types = Query->SubClasses;		/* What info we should fetch */
    if (!Types)
	Types = MCNI_ALL;		/* Default to all info */

    Net = NetworkNew();

    if (FLAGS_ON(Types, MCNI_DNS) && (NetworkDns(Net) == 0))
	++Found;

    if (FLAGS_ON(Types, MCNI_DEFGWIP|MCNI_DEFGWNAME) &&
	(NetworkGateway(Net) == 0))
	++Found;

    if (FLAGS_ON(Types, MCNI_NIS) && NetworkNis(Net) == 0)
	++Found;

    /* Return success if anything succeeded */
    if (!Found) {
	NetworkDestroy(Net);
	return (Opaque_t) -1;
    } else {
	*NetworkPtr = Net;
	return (Opaque_t) 0;
    }
}

/*
 * Interface function between mcSysInfo() and underlying probes.
 */
extern int NetworkMCSI(MCSIquery_t *Query)
{
    static Network_t 	       *Root;
    PSI_t		       *PSIptr;

    if (!Query) {
	errno = ENXIO;
	return -1;
    }

    switch (Query->Op) {
    case MCSIOP_CREATE:
	if (!Root) {
	    if (!PsiSwitch.BuildNetwork || !PsiSwitch.BuildNetwork->Get) {
		SImsg(SIM_DBG, 
		      "Support for `Network' class information is not available on this platform.");
		return 0;
	    }

	    SImsg(SIM_DBG, "BUILDING Network Info ...");
	    /*
	     * Call each defined Build function
	     */
	    for (PSIptr = PsiSwitch.BuildNetwork;
		 PSIptr && PSIptr->Get; ++PSIptr) {
		(void) (PSIptr->Get)(&Root, Query);
	    }
	}

	if (!Root) {
	    SImsg(SIM_DBG, "No Network information was found.");
	    errno = ENOENT;
	    return -1;
	}

	Query->Out = (Opaque_t) Root;
	Query->OutSize = sizeof(Network_t *);
	break;
    case  MCSIOP_DESTROY:
	if (Query->Out && Query->OutSize)
	    return NetworkDestroy((Network_t *) Query->Out);
	break;
    }

    return 0;
}
