/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.6 $";
#endif

/*
 * Network NIS functions
 */

#include "defs.h"
#include "cmd.h"

#ifndef PATH_YPCONF
#define PATH_YPCONF	"/etc/yp.conf"
#endif

/*
 * Paths to possible locations for ypwhich(1)
 */
static char *YpwhichPaths[] = {
    "/usr/bin/ypwhich", "/bin/ypwhich", "/usr/sbin/ypwhich", NULL
};

/*
 * Set the NIS master
 */
static int MasterServer(Network_t *Network, char *Ypwhich)
{
    static char		Line[1024];
    static Cmd_t	Cmd;
    char *		Argv[3];
    char *		cp;

    if (!Network || !Ypwhich)
	return -1;

    CmdInit(&Cmd);
    Cmd.CmdPath = YpwhichPaths;
    Cmd.TimeOut = 30;			/* XXX Timeout */
    Cmd.Flags = CMF_READ;
    Argv[1] = "-m";
    Argv[2] = NULL;
    Cmd.Argv = Argv;
    Cmd.Argc = 2;

    if (CmdOpen(&Cmd) < 0) {
	McDebug("MasterServer: Pipe open to %s failed: %s",
		Cmd.Program, SYSERR);
	return -1;
    }

    /* We just want the first line */
    fgets(Line, sizeof(Line), Cmd.FH);

    CmdClose(&Cmd);

    /*
     * Cleanup Line and advance to second word
     */
    if (cp = strchr(Line, '\n'))
	*cp = CNULL;
    if (cp = strrchr(Line, ' '))
	++cp;
    else if (cp = strrchr(Line, '\t'))
	++cp;
    else {
	McDebug("MasterServer: Cannot parse this line from %s <%s>",
		Cmd, Line);
	return -1;
    }

    McDebug("MasterServer: ypwhich says NIS master is <%s>", cp);

    if (strlen(cp)) {
	Network->NisMaster = strdup(cp);
	return 0;
    }

    return -1;
}

/*
 * Set the current NIS server
 */
static int CurrentServer(Network_t *Network, char *Ypwhich)
{
    static char		Line[1024];
    static Cmd_t	Cmd;
    char *		cp;

    if (!Network || !Ypwhich)
	return -1;

    CmdInit(&Cmd);
    Cmd.CmdPath = YpwhichPaths;
    Cmd.TimeOut = 30;			/* XXX Timeout */
    Cmd.Flags = CMF_READ;

    if (CmdOpen(&Cmd) < 0) {
	McDebug("CurrentServer: Pipe open to %s failed: %s",
		Cmd.Program, SYSERR);
	return -1;
    }

    /* Read the first and only line */
    fgets(Line, sizeof(Line), Cmd.FH);

    CmdClose(&Cmd);

    if (cp = strchr(Line, '\n'))
	*cp = CNULL;

    McDebug("CurrentServer: ypwhich says NIS server is <%s>", Line);

    if (strlen(Line)) {
	Network->NisServerCurrent = strdup(Line);
	return 0;
    }

    return -1;
}

/*
 * Set info we find in /etc/yp.conf.
 * This file is typically found only on Linux.
 */
static int NisInfoYpconf(Network_t *Network)
{
    FILE *		fh;
    static char		Line[1024];
    char *		cp;
    char **		Argv;
    int			Argc;
    int			Found = 0;

    if (!FileExists(PATH_YPCONF, 0))
	return -1;

    McDebug("NisInfoYpconf: Reading %s for Network info ...", PATH_YPCONF);

    fh = fopen(PATH_YPCONF, "r");
    if (!fh) {
	McDebug("NisInfoYpconf: %s: Open for read failed:% s",
		PATH_YPCONF, SYSERR);
	return -1;
    }

    /* Broadcast is the default if the file exists and doesn't override */
    ++Found;
    Network->NisServerMethod = MC_NISMETHOD_BROADCAST;

    while (fgets(Line, sizeof(Line), fh)) {
	if (Line[0] == '#')
	    continue;
	if (cp = strchr(Line, '\n'))
	    *cp = CNULL;

	McDebug("NisInfoYpconf: LINE=%s", Line);

	Argc = StrToArgv(Line, " \t", &Argv, NULL, STRTO_SKIPRP);
	if (Argc <= 1)
	    continue;

	/*
	 * Check for "domain $nisdomain server $hostname" and make sure
	 * $nisdomain matches our NIS domain.
	 */
	if (Argc == 4 && EQ(Argv[0], "domain") && 
	    EQ(Argv[1], Network->NisDomain) &&
	    EQ(Argv[2], "server")) {

	    ++Found;
	    McDebug("NisInfoYpconf:\t Setting Server=<%s>", Argv[3]);
	    StrArrayAdd(&(Network->NisServerIPs), strdup(Argv[3]));
	    if (cp = AddrStrToHost(Argv[3]))
		StrArrayAdd(&(Network->NisServerNames), strdup(cp));
	    Network->NisServerMethod = MC_NISMETHOD_STATIC;
	/*
	 * Check for "ypserver $hostname"
	 */
	} else if (Argc == 2 && EQ(Argv[0], "ypserver")) {
	    ++Found;
	    McDebug("NisInfoYpconf:\t Setting server=<%s>", Argv[1]);
	    StrArrayAdd(&(Network->NisServerIPs), strdup(Argv[1]));
	    if (cp = AddrStrToHost(Argv[1]))
		StrArrayAdd(&(Network->NisServerNames), strdup(cp));
	    Network->NisServerMethod = MC_NISMETHOD_STATIC;
	/*
	 * Look for "domain $nisdomain broadcast" and make sure
	 * $nisdomain matches ours.
	 */
	} else if (Argc == 3 && EQ(Argv[0], "domain") && 
	    EQ(Argv[1], Network->NisDomain) &&
	    EQ(Argv[2], "broadcast")) {

	    ++Found;
	    McDebug("NisInfoYpconf:\t Setting Method=<%s>",
		    MC_NISMETHOD_BROADCAST);
	    Network->NisServerMethod = MC_NISMETHOD_BROADCAST;
	}
    }

    fclose(fh);

    return (Found) ? 0 : -1;
}

/*
 * Look for /var/yp/binding/$domain/ypservers
 */
static int NisInfoYpservers(Network_t *Network)
{
    static char		File[MAXPATHLEN];
    FILE *		fh;
    static char		Line[1024];
    char *		cp;
    char *		Host;
    struct in_addr	InAddr;

    if (!Network || !Network->NisDomain)
	return -1;

    snprintf(File, sizeof(File), "/var/yp/binding/%s/ypservers",
	     Network->NisDomain);

    McDebug("NisInfoYpservers: Checking <%s>", File);

    if (!FileExists(File, 0)) {
#if defined(HAVE_VAR_YP_BINDING_YPSERVERS)
	/* On this platform, the lack of this file implies BROADCAST */
	McDebug("NisInfoYpservers: NIS method is broadcast since %s doesn't exist.",
		File);
	Network->NisServerMethod = MC_NISMETHOD_BROADCAST;
	return 0;
#else
	return -1;
#endif
    }

    fh = fopen(File, "r");
    if (!fh) {
	McDebug("NisInfoYpservers: %s: Open for read failed: %s",
		File, SYSERR);
	return -1;
    }

    /*
     * Each line in ypservers is the name of a ypserver.
     * The line contains either an IP address or hostname.
     */
    while (fgets(Line, sizeof(Line), fh)) {
	if (Line[0] == '#')	/* Not normally, but better to be safe */
	    continue;
	if (cp = strchr(Line, '\n'))
	    *cp = CNULL;

	if (inet_aton(Line, &InAddr)) {
	    /* Must be an IP address */
	    StrArrayAdd(&(Network->NisServerIPs), strdup(Line));
	    if (cp = AddrStrToHost(Line))
		StrArrayAdd(&(Network->NisServerNames), strdup(cp));
	} else {
	    /* Must be a hostname */
	    StrArrayAdd(&(Network->NisServerNames), strdup(Line));
	    if (cp = HostStrToAddr(Line))
		StrArrayAdd(&(Network->NisServerIPs), strdup(cp));
	}

	Network->NisServerMethod = MC_NISMETHOD_STATIC;
    }

    fclose(fh);

    return 0;
}

/*
 * Set Network_t NIS info in Network
 */
extern int NetworkNis(Network_t *Network)
{
    static char		Buff[1024];
    int			Found = 0;
    char *		Ypwhich;

    /* Set NIS domain first */
    if (getdomainname(Buff, sizeof(Buff)) == 0) {
	Network->NisDomain = strdup(Buff);
	++Found;
    }

    /*
     * If we don't have a NIS domain, NIS cannot be running so skip further
     * tests.
     */
    if (!Network->NisDomain || !*Network->NisDomain) {
	McDebug("NetworkNis: No NIS domain was set so NIS cannot be running.");
	return -1;
    }

    if (NisInfoYpconf(Network) == 0)
	++Found;

    if (NisInfoYpservers(Network) == 0)
	++Found;

    Ypwhich = FindCommand(YpwhichPaths);
    if (Ypwhich) {
	if (CurrentServer(Network, Ypwhich) == 0)
	    ++Found;
	if (MasterServer(Network, Ypwhich) == 0)
	    ++Found;
    }

    if (Found)
	return 0;
    else
	return -1;
}
