/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.4 $";
#endif

/*
 * AIX network interface functions
 */

#include "defs.h"
#include "netif.h"
#include "aix-declare.h"

#define PATH_ENTSTAT		"/bin/entstat"

/*
 * Set the MAC address for DevInfo by running a command and scanning
 * the output.
 *
 * Returns 0 on success.
 * Returns -1 on error.
 */
static int AixSetMac(DevInfo_t *DevInfo)
{
    static char			Line[512];
    FILE *			fh;
    static char			Cmd[256];
    char *			cp;
    NetIF_t *			NetIf = NULL;

    if (!DevInfo)
	return -1;

    McDebug("ProbeNetif:\t looking for mac address for <%s> using <%s>",
	    PRTS(DevInfo->Name), PATH_ENTSTAT);

    if (!FileExists(PATH_ENTSTAT, FALSE))
	return -1;

    snprintf(Cmd, sizeof(Cmd), "%s %s", PATH_ENTSTAT, DevInfo->Name);
    McDebug("ProbeNetif:\t Running <%s>", Cmd);
    fh = popen(Cmd, "r");
    if (!fh) {
	SImsg(SIM_DBG, "ProbeNetif: %s: Command open for read failed: %s",
	      Cmd, SYSERR);
	return -1;
    }

    /*
     * Scan output looking for the first line:
     *	Hardware Address: 00:04:ac:17:4a:e4
     */
    while (fgets(Line, sizeof(Line), fh)) {
	if (cp = strchr(Line, '\n'))
	    *cp = CNULL;
	if (!EQN(Line, "Hardware Address:", 17))
	    continue;
	McDebug("ProbeNetif:\t Got <%s>", Line);
	
	/* Skip to start of address */
	cp = strchr(Line, ':');
	for (++cp; isspace(*cp); ++cp);

	McDebug("ProbeNetif:\t MAC Addr=<%s>", cp);

	NetIf = (NetIF_t *) DevInfo->DevSpec;
	if (!NetIf) {
	    NetIf = NetIfNew();
	    DevInfo->DevSpec = (void *) NetIf;
	}

	/* Set the address */
	NetIf->MACaddr = strdup(cp);

	break;
    }

    (void) fclose(fh);

    if (NetIf) {
	return 0;
    } else {
	return -1;
    }
}

/*
 * Probe a network interface
 */
extern DevInfo_t *ProbeNetifAix(ProbeData_t *	ProbeData,
				struct CuDv *	CuDvPtr,
				struct PdDv *	PdDvPtr)
{
    DevInfo_t *			DevInfo;
    char *			OdmName;
    char *			Name;
    static char			NameBuf[36];

    if (!ProbeData)
	return (DevInfo_t *) NULL;

    OdmName = Name = ProbeData->DevName;
    McDebug("ProbeNetifAix: Probe <%s>", PRTS(Name));

    /*
     * ODM typically provides with "entX" names, while the
     * network table uses "enX" names so we need to translate
     * the IF name.
     */
    if (EQN(Name, "ent", 3)) {
	snprintf(NameBuf, sizeof(NameBuf), "en%s", Name + 3);
	McDebug("ProbeNetifAix:\t Translating <%s> -> <%s>", Name, NameBuf);
	ProbeData->DevName = Name = strdup(NameBuf);
    }

    /*
     * Do generic probe
     */
    DevInfo = ProbeNetif(ProbeData);

    /*
     * Do AIX specific probe
     */
    if (DevInfo) {
	AixSetMac(DevInfo);
	/*
	 * Use original ODM name for ODM lookup
	 */
	DevInfo->Name = OdmName;
	SetDescript(DevInfo, CuDvPtr);
	DevInfo->Name = Name;
    }

    return DevInfo;
}
