/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.4 $";
#endif

/*
 * MntInfoSet functions
 */

#include "defs.h"
#include "mcmnt.h"
#include "part.h"

#include <fstab.h>
#include <sys/statvfs.h>

/*
 * Use getfsent() API to set mnt info
 */
static int MntInfoProbeFs(MntInfo_t **MntInfoListPtr)
{
    FILE		       *FilePtr;
    struct fstab	       *Fstab;
    struct statvfs		Statvfs;
    PartInfo_t		       *PartInfo;
    char		      **Argv;
    Large_t			u;
    MntInfo_t		       *Last = NULL;
    MntInfo_t		       *New;
    MntInfo_t		       *MntInfo = NULL;

    SImsg(SIM_DBG, "MntInfoProbeFs: Fetching mnt info via getfsent()");

    /*
     * Find the end of MntInfoPtr and set Last
     */
    for (MntInfo = *MntInfoListPtr; MntInfo; ) {
	if (!MntInfo->Next)
	    Last = MntInfo;
	MntInfo = MntInfo->Next;
    }
	
    setfsent();

    while (Fstab = getfsent()) {
	if (EQ(Fstab->fs_type, FSTAB_XX) || !EQN(Fstab->fs_spec, "/dev/", 5))
	    continue;

	New = MntInfoCreate();
	New->DevPath = strdup(Fstab->fs_spec);
	New->MntName = strdup(Fstab->fs_file);
	New->Type = strdup(Fstab->fs_type);

	if (EQ(Fstab->fs_type, FSTAB_SW))
	    New->Usage = PIU_SWAP;
	else 
	    New->Usage = PIU_FILESYS;

	(void) MntGetFileSys(New);

	SImsg(SIM_DBG,
"MntInfoProbe:    Dev=<%s> Dir=<%s> Type=<%s> Usage=%d Size=%dMB Used=%dMB",
	      PRTS(New->DevPath), PRTS(New->MntName), PRTS(New->Type),
	      New->Usage, (int)New->Size, (int)New->AmtUsed);

	if (MntExists(New, *MntInfoListPtr)) {
	    SImsg(SIM_DBG,
		  "MntInfoProbe: Entry for Dev=<%s> Dir=<%s> already exists - skipped",
		  PRTS(New->DevPath), PRTS(New->MntName));
	    continue;
	}

	if (!Last) 
	    MntInfo = Last = New;
	else {
	    Last->Next = New;
	    Last = New;
	}
    }

    endfsent();

    if (!*MntInfoListPtr)
	*MntInfoListPtr = MntInfo;

    return 0;
}

/*
 * Probe and set MntInfo
 */
extern int MntInfoSet(MntInfo_t **MntInfoListPtr)
{
    /*
     * Build Mount Info list
     */
    (void) MntInfoProbeFs(MntInfoListPtr);

    return 0;
}
