/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.5 $";
#endif

/*
 * MntInfoSet functions
 */

#include "defs.h"
#include "mcmnt.h"
#include <fstab.h>

/*
 * Set mount/filesystem info from fstab and save in MntInfoList.
 * Use getfsent() system API.
 */
extern int MntInfoFstab(MntInfo_t **	MntInfoListPtr)
{
    MntInfo_t		       *Last = NULL;
    MntInfo_t		       *New;
    MntInfo_t		       *MntInfo = NULL;
    struct fstab	       *Ent;

    if (!MntInfoListPtr)
	return -1;

    SImsg(SIM_DBG, "MntInfoFstab: Reading fstab");

    if (!setfsent()) {
	SImsg(SIM_GERR, "setfsent failed: %s", SYSERR);
	return -1;
    }

    /*
     * Find the end of MntInfoPtr and set Last
     */
    for (MntInfo = *MntInfoListPtr; MntInfo; ) {
	if (!MntInfo->Next)
	    Last = MntInfo;
	MntInfo = MntInfo->Next;
    }
	
    while (Ent = getfsent()) {
	New = MntInfoCreate();
	New->DevPath = strdup(Ent->fs_spec);
	if (Ent->fs_file && Ent->fs_file[0] == '/')
	    New->MntName = strdup(Ent->fs_file);
	New->Type = strdup(Ent->fs_vfstype);
	New->Opts = strdup(Ent->fs_mntops);

	/* Fill in generic info */
	MntInfoFill(New);

	/*
	 * Getting filesystem info is only valid if we think the entry is
	 * mounted.
	 */
	(void) MntGetFileSys(New);

	SImsg(SIM_DBG,
"MntInfoFstab:    Dev=<%s> Dir=<%s> Type=<%s> Usage=%d Size=%dMB Used=%dMB",
	      PRTS(New->DevPath), PRTS(New->MntName), PRTS(New->Type),
	      New->Usage, (int)New->Size, (int)New->AmtUsed);

	if (MntExists(New, *MntInfoListPtr)) {
	    SImsg(SIM_DBG,
		  "MntInfoFstab: Entry for Dev=<%s> Dir=<%s> already exists - skipped",
		  PRTS(New->DevPath), PRTS(New->MntName));
	    continue;
	}

	if (!Last) 
	    MntInfo = Last = New;
	else {
	    Last->Next = New;
	    Last = New;
	}
    }

    (void) endfsent();

    if (!*MntInfoListPtr)
	*MntInfoListPtr = MntInfo;

    return 0;
}

/*
 * Probe and set MntInfo
 */
extern int MntInfoSet(MntInfo_t **MntInfoListPtr)
{
    /*
     * Build Mount Info list
     */
    (void) MntInfoFstab(MntInfoListPtr);

    return 0;
}
