/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.14 $";
#endif

/*
 * sysctl(3) functions common to BSD implimentations (BSD/OS, FreeBSD, etc.)
 */

#include "defs.h"
#include <sys/types.h>
#include <sys/sysctl.h>
#include <sys/time.h>
#include <sys/socket.h>		/* BSD/OS 3.1 */
#include <net/if.h>
#include <net/if_dl.h>
#include <net/route.h>
#if defined(HAVE_NET_ETHERNET_H)
# include <net/ethernet.h>
#endif
#if defined(VM_SWAPSTATS)
# include <vm/swap_pager.h>
#endif	/* VM_SWAPSTATS */

/*
 * Use sysctl() to get MODEL.
 */
extern Opaque_t GetModelSysCtl()
{
    int				Query[2];
    static char			Model[128];
    size_t			Length = sizeof(Model);

    Query[0] = CTL_HW;
    Query[1] = HW_MODEL;
    if (sysctl(Query, 2, Model, &Length, NULL, 0) == -1) {
	SImsg(SIM_GERR, "sysctl CTL_HW.HW_MODEL failed: %s", SYSERR);
	return (Opaque_t) NULL;
    }

    return (Opaque_t) Model;
}

/*
 * Use sysctl() to get Number of CPUs
 */
extern char *GetNcpuSysCtl()
{
    int				Query[2];
    int				NumCpu = 0;
    size_t			Length = sizeof(NumCpu);

    Query[0] = CTL_HW;
    Query[1] = HW_NCPU;
    if (sysctl(Query, 2, &NumCpu, &Length, NULL, 0) == -1) {
	SImsg(SIM_GERR, "sysctl CTL_HW.HW_NCPU failed: %s", SYSERR);
	return((char *) NULL);
    }

    return(itoa(NumCpu));
}

/*
 * Use sysctl() to get the Kernel Version
 */
extern char *GetKernVerSysCtl()
{
    int				Query[2];
    static char			Version[256];
    size_t			Length = sizeof(Version);

    Query[0] = CTL_KERN;
    Query[1] = KERN_VERSION;
    if (sysctl(Query, 2, Version, &Length, NULL, 0) == -1) {
	SImsg(SIM_GERR, "sysctl CTL_KERN.KERN_MODEL failed: %s", SYSERR);
	return((char *) NULL);
    }

    return(Version);
}

/*
 * Use sysctl() to get the OS Distribution
 */
extern char *GetOSDistSysCtl()
{
    int				Query[2];
    int				Dist = 0;
    size_t			Length = sizeof(Dist);

    Query[0] = CTL_KERN;
    Query[1] = KERN_OSREV;
    if (sysctl(Query, 2, &Dist, &Length, NULL, 0) == -1) {
	SImsg(SIM_GERR, "sysctl CTL_KERN.KERN_OSREV failed: %s", SYSERR);
	return((char *) NULL);
    }

    return(itoa(Dist));
}

/*
 * Use sysctl() to get the Memory
 */
extern char *GetMemorySysCtl()
{
    int				Query[2];
    int				Memory = 0;
    Large_t			Amount;
    size_t			Length = sizeof(Memory);

    Query[0] = CTL_HW;
    Query[1] = HW_PHYSMEM;
    if (sysctl(Query, 2, &Memory, &Length, NULL, 0) == -1) {
	SImsg(SIM_GERR, "sysctl CTL_HW.HW_PHYSMEM failed: %s", SYSERR);
	return((char *) NULL);
    }

    Amount = DivRndUp((Large_t)Memory, (Large_t)MBYTES);

    return(GetMemoryStr(Amount));
}

/*
 * Use sysctl() to get the Virtual Memory
 */
extern char *GetVirtMemSysCtl()
{
#if	defined(VM_SWAPSTATS)
    int				Query[2];
    struct swapstats		Swap;
    size_t			Length = sizeof(Swap);
    Large_t			Amount;

    Query[0] = CTL_VM;
    Query[1] = VM_SWAPSTATS;
    if (sysctl(Query, 2, &Swap, &Length, NULL, 0) == -1) {
	SImsg(SIM_GERR, "sysctl CTL_Vm.VM_SWAPSTATS failed: %s", SYSERR);
	return((char *) NULL);
    }

    Amount = DivRndUp((Large_t)(Swap.swap_total * 512), (Large_t)MBYTES);

    return(GetMemoryStr(Amount));
#else
    return((char *) NULL);
#endif	/* VM_SWAPSTATS */
}

/*
 * Use sysctl() to get the Boot Time
 */
extern char *GetBootTimeSysCtl()
{
    int				Query[2];
    struct timeval		TimeVal;
    time_t			Time;
    size_t			Length = sizeof(TimeVal);

    Query[0] = CTL_KERN;
    Query[1] = KERN_BOOTTIME;
    if (sysctl(Query, 2, &TimeVal, &Length, NULL, 0) == -1) {
	SImsg(SIM_GERR, "sysctl CTL_KERN.KERN_BOOTTIME failed: %s", SYSERR);
	return((char *) NULL);
    }

    Time = (time_t) TimeVal.tv_sec;
    return TimeToStr(Time, NULL);
}

/*
 * Use sysctl(3) to retrieve the MAC Info for a network interface.
 * This is based on code from FreeBSD 2.2.6 ifconfig(8).
 * This is totally convoluted!  Yuck!
 */
extern void SetMacInfoSysCtl(DevInfo, NetIf)
     DevInfo_t		       *DevInfo;
     NetIF_t		       *NetIf;
{
    static char			Ether[128];
    static char			EtherName[128];
#if	defined(HAVE_ETHER_NTOHOST)
    struct ether_addr		EtherAddr;
#endif	/* HAVE_ETHER_NTOHOST */
    register char	       *cp;
    register int		n;
    register int		Len;
    struct if_msghdr 	       *ifm, *nextifm;
    struct ifa_msghdr 	       *ifam;
    struct sockaddr_dl 	       *sdl;
    char		       *buf, *lim, *next;
    size_t 			needed;
    int 			mib[6];

    mib[0] = CTL_NET;
    mib[1] = PF_ROUTE;
    mib[2] = 0;
    mib[3] = AF_INET;	/* address family */
    mib[4] = NET_RT_IFLIST;
    mib[5] = 0;
    if (sysctl(mib, 6, NULL, &needed, NULL, 0) < 0) {
	SImsg(SIM_GERR, "%s: sysctl get iflist estimate failed: %s",
	      DevInfo->Name, SYSERR);
	return;
    }
    buf = (char *) xmalloc(needed);
    if (sysctl(mib, 6, buf, &needed, NULL, 0) < 0) {
	SImsg(SIM_GERR, "%s: sysctl retrieve iflist table failed: %s",
	      DevInfo->Name, SYSERR);
	return;
    }
    lim = buf + needed;
    
    next = buf;
    while (next < lim) {
	
	ifm = (struct if_msghdr *)next;
	
	if (ifm->ifm_type == RTM_IFINFO) {
	    sdl = (struct sockaddr_dl *)(ifm + 1);
	} else {
	    SImsg(SIM_GERR, 
	    "%s: Cannot get MAC info: out of sync parsing NET_RT_IFLIST.", 
		  DevInfo->Name);
	    return;
	}
	
	next += ifm->ifm_msglen;
	ifam = NULL;
	while (next < lim) {
	    
	    nextifm = (struct if_msghdr *)next;
	    
	    if (nextifm->ifm_type != RTM_NEWADDR)
		break;
	    
	    if (ifam == NULL)
		ifam = (struct ifa_msghdr *)nextifm;
	    
	    next += nextifm->ifm_msglen;
	}

	if (EQN(DevInfo->Name, sdl->sdl_data, sdl->sdl_nlen)) {
	    /* Found it */
	    cp = (char *)LLADDR(sdl);
	    if ((n = sdl->sdl_alen) > 0) {
		Ether[0] = CNULL;
             	while (--n >= 0) {
		    Len = strlen(Ether);
		    (void) snprintf(&Ether[Len], sizeof(Ether)-Len,
				    "%02x%s", *cp++ & 0xff, n > 0 ? ":" : "");
		}

		NetIf->MACaddr = strdup(Ether);
#if	defined(HAVE_ETHER_NTOHOST)
		memcpy(&EtherAddr.octet, LLADDR(sdl), ETHER_ADDR_LEN);
		if (ether_ntohost(EtherName, &EtherAddr) == 0)
		    NetIf->MACname = strdup(EtherName);
#endif	/* HAVE_ETHER_NTOHOST */
	    }
	    break;
	}
    }
    (void) free(buf);
}

/*
 * Return CPU frequency by looking for Name
 */
extern Large_t GetCpuFreqSysCtl(QueryName, QueryInt)
     char		       *QueryName;
     int			QueryInt;
{
    int				Query[2];
    long			CpuFreq = 0;
    size_t			CpuFreqLen = sizeof(CpuFreq);

    if (QueryName) {
	if (sysctlbyname(QueryName, &CpuFreq, &CpuFreqLen, NULL, 0) < 0) {
	    CpuFreq = 0;
	    SImsg(SIM_GERR, "GetCpuFreqSysCtl: sysctl lookup <%s> failed: %s",
		  QueryName, SYSERR);
	    return (Large_t) 0;
	}
    } else {
	Query[0] = CTL_HW;
	Query[1] = QueryInt;
	if (sysctl(Query, 2, &CpuFreq, &CpuFreqLen, NULL, 0) < 0) {
	    CpuFreq = 0;
	    SImsg(SIM_GERR, "GetCpuFreqSysCtl: sysctl lookup %d failed: %s",
		  QueryInt, SYSERR);
	    return (Large_t) 0;
	}
    }

    if (CpuFreq)
	return (Large_t) hz_to_mhz(CpuFreq);
    else
	return (Large_t) 0;
}

/*
 * Get CPU Info from sysctl
 */
extern CpuInfo_t *GetCpuInfoSysCtl()
{
    CpuInfo_t		       *CpuInfo = NULL;
    int				Query[2];
    char			QueryStr[128];
    char			Buff[512];
    size_t			BuffLen = sizeof(Buff);
    long			ValLong;
    size_t			ValLongLen = sizeof(ValLong);
    CacheInfo_t		       *L1i = NULL;
    CacheInfo_t		       *L1d = NULL;
    CacheInfo_t		       *L2 = NULL;

    CpuInfo = CpuInfoNew(NULL);

#if	defined(HW_MACHINE)
    if (!CpuInfo->Arch) {
	Query[0] = CTL_HW;
	Query[1] = HW_MACHINE;
	if (sysctl(Query, 2, &Buff, &BuffLen, NULL, 0) == 0) {
	    if (EQ(Buff, "Power Macintosh"))
		CpuInfo->Arch = strdup("PowerPC");
	    else
		CpuInfo->Arch = strdup(Buff);
	}
    }
#endif

    /*
     * Look up the Clock Frequency
     * In FreeBSD 2.x look for "machdep.CLASS_freq" (i.e. machdep.i586_freq).
     * In FreeBSD 3.0 and later it's "machdep.tsc_freq".
     */
    if (!CpuInfo->Speed && CpuInfo->Arch) {
	(void) snprintf(QueryStr, sizeof(QueryStr), "machdep.%s_freq",
			CpuInfo->Arch);
	CpuInfo->Speed = GetCpuFreqSysCtl(QueryStr, 0);
    }
    if (!CpuInfo->Speed) {
	/*
	 * The Class specific entry wasn't found, so try the new "tsc"
	 * entry as found on FreeBSD 3.0 and later.
	 */
	(void) snprintf(QueryStr, sizeof(QueryStr), "machdep.tsc_freq");
	CpuInfo->Speed = GetCpuFreqSysCtl(QueryStr, 0);
    }
#if	defined(HW_CPU_FREQ)
    if (!CpuInfo->Speed) {
	/*
	 * MacOS X (Darwin)
	 */
	CpuInfo->Speed = GetCpuFreqSysCtl(NULL, HW_CPU_FREQ);
    }
#endif

    Query[0] = CTL_HW;
#if	defined(HW_L1ICACHESIZE)
    Query[1] = HW_L1ICACHESIZE;
    if (sysctl(Query, 2, &ValLong, &ValLongLen, NULL, 0) == 0) {
	if (!L1i)
	    L1i = CacheInfoNew(NULL);
	L1i->Size = (Large_t) ValLong;
    }
#endif
#if	defined(HW_L1DCACHESIZE)
    Query[1] = HW_L1DCACHESIZE;
    if (sysctl(Query, 2, &ValLong, &ValLongLen, NULL, 0) == 0) {
	if (!L1d)
	    L1d = CacheInfoNew(NULL);
	L1d->Size = (Large_t) ValLong;
    }
#endif
#if	defined(HW_L2CACHESIZE)
    Query[1] = HW_L2CACHESIZE;
    if (sysctl(Query, 2, &ValLong, &ValLongLen, NULL, 0) == 0) {
	if (!L2)
	    L2 = CacheInfoNew(NULL);
	L2->Size = (Large_t) ValLong;
    }
#endif

    CpuInfo->L1iCache = L1i;
    CpuInfo->L1iCache = L1d;
    CpuInfo->L1iCache = L2;

    return CpuInfo;
}

