/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.3 $";
#endif

/*
 * FreeBSD specific functions
 */

#include "defs.h"
#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#if	OSMVER <= 2
#include <scsi.h>
#endif	/* OSMVER <= 2 */
#if	OSMVER >= 4
#include <cam/scsi/scsi_all.h>
#else	/* OSMVER < 4 */
#include <sys/device.h>
#include <sys/scsiio.h>
#endif	/* OSMVER */
#include "myscsi.h"

#ifndef MAX_SCSI_UNITS
#define MAX_SCSI_UNITS		7		/* Max # of devs / bus */
#endif

#if	HAVE_LIBCAM
/*
 * Issue a SCSI command and return the results.
 *
 * FreeBSD 3.x ScsiCmd using camlib.
 */
#include <camlib.h>
#include <cam/scsi/scsi_message.h>
extern int ScsiCmd(ScsiCmd_t *ScsiCmd)
{
    static char			Buff[SCSI_BUF_LEN];
    struct cam_device	       *CamDevice;
    union ccb		       *CamCcb;
    extern char		        cam_errbuf[];

    if (!ScsiCmd || !ScsiCmd->Cdb || !ScsiCmd->CdbLen || 
	ScsiCmd->DevFD < 0 || !ScsiCmd->DevFile) {
	SImsg(SIM_DBG, "ScsiCmd: Bad parameters.");
	return(-1);
    }

    if (!(CamDevice = cam_open_device(ScsiCmd->DevFile, O_RDWR))) {
	SImsg(SIM_GERR, "%s: cam_open_spec_device failed: %s",
	      ScsiCmd->DevFile, cam_errbuf);
	return(-1);
    }

    if (!(CamCcb = cam_getccb(CamDevice))) {
	SImsg(SIM_GERR, "%s: cam_getccb failed: %s",
	      ScsiCmd->DevFile, cam_errbuf);
	(void) cam_close_device(CamDevice);
	return(-1);
    }

    /*
     * Initialize
     */
    (void) memset(&(&CamCcb->ccb_h)[1], 0, sizeof(struct ccb_scsiio));
    (void) memset(Buff, 0, sizeof(Buff));

    cam_fill_csio(&CamCcb->csio,
		  1,	    			/* retries */ 
		  NULL,    			/* cbfcnp */ 
		  CAM_DIR_IN,   		/* flags */ 
		  MSG_SIMPLE_Q_TAG,    		/* tag_action */ 
		  (u_int8_t *) Buff,    	/* buffer */ 
		  sizeof (Buff),    		/* buffer len */ 
		  SSD_FULL_SIZE,    		/* sense_len */ 
		  ScsiCmd->CdbLen,		/* cdb_len */
		  MySCSI_CMD_TIMEOUT * 1000);   /* timeout */ 
    /*
     * Setup CAM CCB using our CDB
     */
    (void) memcpy(&CamCcb->csio.cdb_io.cdb_bytes, ScsiCmd->Cdb, 
		  sizeof(CamCcb->csio.cdb_io.cdb_bytes));
    /* 
     * Disable freezing the device queue 
     */
    CamCcb->ccb_h.flags |= CAM_DEV_QFRZDIS;

    if (cam_send_ccb(CamDevice, CamCcb) < 0) {
	SImsg(SIM_GERR, "%s: cam_send_ccb failed: %s",
	      ScsiCmd->DevFile, cam_errbuf);
	(void) cam_close_device(CamDevice);
	(void) cam_freeccb(CamCcb);
	return(-1);
    }

    (void) cam_close_device(CamDevice);
    (void) cam_freeccb(CamCcb);

    ScsiCmd->Data = (void *) Buff;

    return(0);
}
#else	/* !HAVE_LIBCAM */
/*
 * Issue a SCSI command and return the results.
 *
 * FreeBSD 2.x ScsiCmd is a straight SCIOCCOMMAND
 */
extern int ScsiCmd(ScsiCmd_t *ScsiCmd)
{
    static char			Buff[SCSI_BUF_LEN];
    ScsiCdbG0_t		       *CdbPtr = NULL;
    static scsireq_t		Req;
    scsireq_t		       *ReqPtr;

    if (!ScsiCmd || !ScsiCmd->Cdb || !ScsiCmd->CdbLen || 
	ScsiCmd->DevFD < 0 || !ScsiCmd->DevFile) {
	SImsg(SIM_DBG, "ScsiCmd: Bad parameters.");
	return(-1);
    }

    (void) memset(Buff, 0, sizeof(Buff));
    (void) memset(&Req, 0, sizeof(Req));

    (void) memcpy(Req.cmd, ScsiCmd->Cdb, ScsiCmd->CdbLen);
    Req.cmdlen = ScsiCmd->CdbLen;
    Req.databuf = (caddr_t) Buff;
    Req.datalen = sizeof(Buff);
    Req.timeout = MySCSI_CMD_TIMEOUT * 1000;	/* timeout == milliseconds */
    Req.flags = SCCMD_READ;

    /* 
     * We just need Cdb.cmd so it's ok to assume ScsiCdbG0_t here
     */
    CdbPtr = (ScsiCdbG0_t *) ScsiCmd->Cdb;
    errno = 0;
    if (ioctl(ScsiCmd->DevFD, SCIOCCOMMAND, (void *) &Req) == -1) {
	SImsg(SIM_GERR, "%s: ioctl SCIOCCOMMAND SCSI cmd 0x%x failed: %s",
	      ScsiCmd->DevFile, CdbPtr->cmd, SYSERR);
	return(-1);
    }

#if	!defined(SCSIREQ_ERROR)
#define SCSIREQ_ERROR(r)	(r->retsts != SCCMD_OK)
#endif

    ReqPtr = &Req;
    if (SCSIREQ_ERROR(ReqPtr)) {
	SImsg(SIM_GERR, "%s: SCSI command 0x%x failed.", 
	      ScsiCmd->DevFile, CdbPtr->cmd);
	return(-1);
    }

    ScsiCmd->Data = (void *) Buff;
    return(0);
}
#endif	/* HAVE_LIBCAM */

/*
 * Query a SCSI device with base (driver) named Name and Unit.
 * 
 * We used to use ioctl SCIOCIDENTIFY to get SCSI Addr info, but that
 * doesn't work with the da/sa drivers in FreeBSD 3.0
 */
static DevList_t *ScsiProbe(char *		Name,
			    int			Unit,
			    int			DevType,
			    DevInfo_t *		CtlrDevInfo)
{
    static ProbeData_t		ProbeData;
    DevInfo_t		       *DevInfo = NULL;
    DevList_t		       *DevList;
    static char			DevName[64];
    static char			File[128];
    int				fd;

    (void) snprintf(DevName, sizeof(DevName), "%s%d", Name, Unit);

#if OSMVER >= 5
    (void) snprintf(File, sizeof(File), "%s/%s", _PATH_DEV, DevName);
    if (!FileExists(File, FALSE)) {
	McDebug("%s: SCSI device file does not exist.", File);
	return (DevList_t *) NULL;
    }
#else	/* OSMVER < 5 */
# if OSMVER >= 3
    (void) snprintf(File, sizeof(File), "%s/r%s", _PATH_DEV, DevName);
# else	/* OSMVER < 3 */
    (void) snprintf(File, sizeof(File), "%s/r%s.ctl", _PATH_DEV, DevName);
# endif

    fd = open(File, O_RDONLY|O_NDELAY);
    if (fd < 0) {
	SImsg(SIM_DBG, "%s: open %s failed: %s", DevName, File, SYSERR);
	if (errno != EBUSY)
	    return((DevList_t *) NULL);
    } else {
	(void) close(fd);
    }
#endif

    SImsg(SIM_DBG, "%s:\tSCSI Probe: Device Exists", File);

    /*
     * Create the DevInfo
     */
    DevInfo = NewDevInfo(NULL);
    DevInfo->Name = strdup(DevName);
    DevInfo->Driver = strdup(Name);
    DevInfo->ClassType = CT_SCSI;
    DevInfo->Unit = Unit;
    DevInfo->Type = DevType;
    DevAddFile(DevInfo, strdup(File));

    /*
     * Crate DevList and return it.
     */
    DevList = (DevList_t *) xcalloc(1, sizeof(DevList_t));
    DevList->Name = DevInfo->Name;
    DevList->File = DevInfo->Files[0];
    DevList->Unit = Unit;
    DevList->DevInfo = DevInfo;
    DevList->ClassType = DevInfo->ClassType;

    /*
     * Do detailed probe
     */
    memset(&ProbeData, CNULL, sizeof(ProbeData));
    ProbeData.DevName = DevInfo->Name;
    ProbeData.DevFile = File;
    ProbeData.UseDevInfo = DevInfo;
    ProbeData.CtlrDevInfo = CtlrDevInfo;
    (void) ProbeDiskDrive(&ProbeData);

    return(DevList);
}

/*
 * Get and return a list of all SCSI devices we can find.
 * We only build the list once to save time.
 */
extern DevList_t *ScsiDetect(DevInfo_t *CtlrDevInfo)
{
    register int		u;
    static DevList_t	       *ScsiDevices;
    register DevList_t	       *New = NULL;
    register DevList_t	       *Last = NULL;

    if (ScsiDevices)
	return(ScsiDevices);

    /*
     * Look for SCSI Disks
     */
#if	OSMVER >= 3
#define MySCSI_DISK_NAME "da"
#else
#define MySCSI_DISK_NAME "sd"
#endif	/* OSMVER */
    for (u = 0; u < MAX_SCSI_UNITS; ++u) {
	if (New = ScsiProbe(MySCSI_DISK_NAME, u, DT_DISKDRIVE, CtlrDevInfo)) {
	    if (!ScsiDevices)
		ScsiDevices = Last = New;
	    else {
		Last->Next = New;
		Last = New;
	    }
	}
    }

    /*
     * Look for SCSI Tapes
     */
#if	OSMVER >= 3
#define MySCSI_TAPE_NAME "sa"
#else
#define MySCSI_TAPE_NAME "st"
#endif	/* OSMVER */
    for (u = 0; u < MAX_SCSI_UNITS; ++u) {
	if (New = ScsiProbe(MySCSI_TAPE_NAME, u, DT_TAPEDRIVE, CtlrDevInfo)) {
	    if (!ScsiDevices)
		ScsiDevices = Last = New;
	    else {
		Last->Next = New;
		Last = New;
	    }
	}
    }

    return(ScsiDevices);
}
