/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.3 $";
#endif

/*
 * MntInfoSet functions
 */

#include "defs.h"
#include "mcmnt.h"
#include "part.h"

#include <mntent.h>
#include <sys/statvfs.h>

/*
 * Set mount/filesystem info and save in MntInfoList.
 * Use getmntent() system API.
 */
extern int MntInfoProbe(char *		File,		/* File to open */
			MntInfo_t **	MntInfoListPtr,
			int 		IsMountTable)	/* File in current mounts */
{
    FILE		       *fp;
    MntInfo_t		       *Last = NULL;
    MntInfo_t		       *New;
    MntInfo_t		       *MntInfo = NULL;
    struct mntent	       *Ent;
    int				Ignore;
    int				i;
    char **			Argv;
    int				Argc;

    if (!File || !MntInfoListPtr)
	return -1;

    SImsg(SIM_DBG, "MntInfoProbe: Reading File=<%s>", File);

    fp = setmntent(File, "r");
    if (!fp) {
	SImsg(SIM_GERR, "%s: Cannot open: %s", File, SYSERR);
	return -1;
    }

    /*
     * Find the end of MntInfoPtr and set Last
     */
    for (MntInfo = *MntInfoListPtr; MntInfo; ) {
	if (!MntInfo->Next)
	    Last = MntInfo;
	MntInfo = MntInfo->Next;
    }
	
    while (Ent = getmntent(fp)) {
	/* Check for filesys types to ignore */
	if (EQ(Ent->mnt_type, MNTTYPE_IGNORE) ||
	    EQ(Ent->mnt_type, MNTTYPE_NFS) ||
	    EQ(Ent->mnt_type, MNTTYPE_NFS3) ||
	    EQ(Ent->mnt_type, MNTTYPE_DFS) ||
	    EQ(Ent->mnt_type, MNTTYPE_LOFS))
	    continue;

	/* Check mount options for things we want to ignore */
	if (Ent->mnt_opts && !EQ(Ent->mnt_opts, "-") &&
	    !EQ(Ent->mnt_opts, "defaults"))
	    Argc = StrToArgv(Ent->mnt_opts, ",", &Argv, NULL, 0);
	else
	    Argc = 0;
	for (i = 0, Ignore = FALSE; i < Argc && Ignore == FALSE; ++i) {
	    if (EQ(Argv[i], MNTTYPE_IGNORE))
		Ignore = TRUE;
	}
	if (Ignore)
	    continue;

	New = MntInfoCreate();
	New->DevPath = strdup(Ent->mnt_fsname);
	New->MntName = strdup(Ent->mnt_dir);
	New->Type = strdup(Ent->mnt_type);
	if (!EQ(Ent->mnt_opts, "defaults"))
	    New->Opts = strdup(Ent->mnt_opts);

	/* Fill in generic info */
	MntInfoFill(New);

	/*
	 * Getting filesystem info is only valid if we think the entry is
	 * mounted.
	 */
	if (IsMountTable)
	    (void) MntGetFileSys(New);

	SImsg(SIM_DBG,
"MntInfoProbe:    Dev=<%s> Dir=<%s> Type=<%s> Usage=%d Size=%dMB Used=%dMB",
	      PRTS(New->DevPath), PRTS(New->MntName), PRTS(New->Type),
	      New->Usage, (int)New->Size, (int)New->AmtUsed);

	if (MntExists(New, *MntInfoListPtr)) {
	    SImsg(SIM_DBG,
		  "MntInfoProbe: Entry for Dev=<%s> Dir=<%s> already exists - skipped",
		  PRTS(New->DevPath), PRTS(New->MntName));
	    continue;
	}

	if (!Last) 
	    MntInfo = Last = New;
	else {
	    Last->Next = New;
	    Last = New;
	}
    }

    (void) endmntent(fp);

    if (!*MntInfoListPtr)
	*MntInfoListPtr = MntInfo;

    return 0;
}

/*
 * Probe and set MntInfo
 */
extern int MntInfoSet(MntInfo_t **MntInfoListPtr)
{
    /*
     * Build Mount Info list
     *
     * We first look in MNT_MNTTAB (/etc/mtab) to see what's
     * currently mounted.  Then we look in MNT_CHECKLIST (/etc/fstab)
     * to see what may be unmounted.
     */
    (void) MntInfoProbe(MNT_MNTTAB, MntInfoListPtr, TRUE);
    (void) MntInfoProbe(MNT_CHECKLIST, MntInfoListPtr, FALSE);

    return 0;
}
