/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.3 $";
#endif

/*
 * MntInfoSet functions
 */

#include "defs.h"
#include "mcmnt.h"
#include <mntent.h>
#include <sys/stat.h>

/*
 * Check to see if MntDevice is one of the "special" ones
 * that IRIX uses.
 */
static int IsSpecialUse(MntDevice, DevPath)
    char		       *MntDevice;
    char		       *DevPath;
{
    static struct stat		MntStat;
    static struct stat		DevStat;

    if (!EQ(MntDevice, "/dev/root") &&
	!EQ(MntDevice, "/dev/usr") &&
	!EQ(MntDevice, "/dev/swap"))
	return(0);

    if (stat(MntDevice, &MntStat) != 0) {
	SImsg(SIM_GERR, "stat failed: %s: %s", MntDevice, SYSERR);
	return(0);
    }
    if (stat(DevPath, &DevStat) != 0) {
	SImsg(SIM_GERR, "stat failed: %s: %s", DevPath, SYSERR);
	return(0);
    }

    if (MntStat.st_rdev == DevStat.st_rdev)
	return(1);
    else
	return(0);
}

/*
 * Scan File for mnt entries and add to list at MntInfoListPtr.
 */
static int MntInfoProbe(char *		File,
			MntInfo_t **	MntInfoListPtr,
			int		IsMountTable)
{
    char		      **Argv;
    FILE		       *fp;
    MntInfo_t		       *Last = NULL;
    MntInfo_t		       *New;
    MntInfo_t		       *MntInfo = NULL;
    struct mntent	       *Ent;

    if (!File || !MntInfoListPtr)
	return -1;

    SImsg(SIM_DBG, "MntInfoProbe: Reading File=<%s>", File);

    fp = setmntent(File, "r");
    if (!fp) {
	McDebug("MntInfoProbe: %s: Open for read failed: %s", File, SYSERR);
	return -1;
    }

    /*
     * Find the end of MntInfoPtr and set Last
     */
    for (MntInfo = *MntInfoListPtr; MntInfo; ) {
	if (!MntInfo->Next)
	    Last = MntInfo;
	MntInfo = MntInfo->Next;
    }
	
    while (Ent = getmntent(fp)) {
	New = MntInfoCreate();
	New->DevPath = strdup(Ent->mnt_fsname);
	New->MntName = strdup(Ent->mnt_dir);
	New->Opts = strdup(Ent->mnt_opts);
	New->Type = strdup(Ent->mnt_type);

	/* Fill in generic info */
	MntInfoFill(New);

	/*
	 * Getting filesystem info is only valid if we think the entry is
	 * mounted.
	 */
	if (IsMountTable)
	    (void) MntGetFileSys(New);

	SImsg(SIM_DBG,
"MntInfoProbe:    Dev=<%s> Dir=<%s> Type=<%s> Usage=%d Size=%dMB Used=%dMB",
	      PRTS(New->DevPath), PRTS(New->MntName), PRTS(New->Type),
	      New->Usage, (int)New->Size, (int)New->AmtUsed);

	if (MntExists(New, *MntInfoListPtr)) {
	    SImsg(SIM_DBG,
"MntInfoProbe: Entry for Dev=<%s> Dir=<%s> already exists - skipped",
		  PRTS(New->DevPath), PRTS(New->MntName));
	    continue;
	}

	if (!Last) 
	    MntInfo = Last = New;
	else {
	    Last->Next = New;
	    Last = New;
	}
    }

    (void) endmntent(fp);

    if (!*MntInfoListPtr)
	*MntInfoListPtr = MntInfo;

    return 0;
}

/*
 * Probe and set MntInfo
 */
extern int MntInfoSet(MntInfo_t **MntInfoListPtr)
{
    /*
     * Build Mount Info list
     *
     * We first look in _PATH_MOUNTED (/etc/mtab) to see what's
     * currently mounted.  Then we look in _PATH_MNTTAB (/etc/fstab)
     * to see what may be unmounted.
     */
    (void) MntInfoProbe(MOUNTED, MntInfoListPtr, TRUE);
    (void) MntInfoProbe(MNTTAB, MntInfoListPtr, FALSE);

    return 0;
}
