/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.9 $";
#endif

/*
 * Linux ATA specific functions
 */

#include "defs.h"
#include "linux-declare.h"
#include "myata.h"
#include <dirent.h>
#include <linux/hdreg.h>

#define MAX_ATA_DRIVES		32		/* Max drives per system */

/*
 * Issue an ATA command and return the results.
 * This function uses the HDIO_DRIVE_CMD ioctl to issue raw ATA
 * commands to the device.
 */
extern int AtaCmd(AtaCmd)
     AtaCmd_t		       *AtaCmd;
{
#if	defined(HDIO_DRIVE_CMD)
    static char			Buff[HDCMD_OFFSET + ATA_BUF_LEN];
    static HDcmd_t		HDcmd;

    if (!AtaCmd || !AtaCmd->Cdb || AtaCmd->DevFD < 0 || !AtaCmd->DevFile) {
	SImsg(SIM_DBG, "AtaCmd: Bad parameters.");
	return -1;
    }

    (void) memset(&HDcmd, 0, sizeof(HDcmd));
    HDcmd.Cmd = AtaCmd->Cdb->Cmd;
    /*
     * Add the size of this command + the expected data length, divide by
     * 512 (1 sector) and round up the result.
     */
    HDcmd.SectLen = rint( (HDCMD_OFFSET + AtaCmd->DataLen) / 512);

    (void) memcpy(Buff, &HDcmd, HDCMD_OFFSET);

    if (ioctl(AtaCmd->DevFD, HDIO_DRIVE_CMD, Buff) < 0) {
	SImsg(SIM_DBG, "%s: AtaCmd command %s failed: %s",
	      AtaCmd->DevFile, AtaCmd->CmdName, SYSERR);
	return -1;
    }

    AtaCmd->Data = (void *) (Buff + HDCMD_OFFSET);

    return 0;
#else	/* !HDIO_DRIVE_CMD */
    return -1;
#endif	/* HDIO_DRIVE_CMD */
}

/*
 * Query an ATA/IDE/EIDE device with base (driver) named Name and Unit.
 */
static DevList_t *AtaProbe(Name, Unit, ProcPath, DevType)
     char		       *Name;
     int			Unit;
     char		       *ProcPath;
     int			DevType;
{
    DevInfo_t		       *DevInfo = NULL;
    DevList_t		       *DevList = NULL;
    DevInfo_t		     *(*ProbeFunc)() = NULL;
    static ProbeData_t		ProbeData;
    static char			DevName[64];
    static char			Dir[128];
    static char			File[128];
    static Query_t		Query;
    static AtaIdent_t		AtaIdent;
    static struct hd_driveid	DriveID;
    int				fd = -1;
    char		       *cp;

    (void) snprintf(DevName, sizeof(DevName), "%s%c", Name, 'a' + Unit);
    (void) snprintf(File, sizeof(File), "%s%s", _PATH_DEV, DevName);

    /*
     * All we can do right now is see if open() succeeds.
     * We really need a means to issue ATA READ PARAMETERS (AtaParam_t)
     */
    fd = open(File, O_RDONLY|O_NONBLOCK);
    if (fd < 0)
	SImsg(SIM_DBG, "%s: open %s failed: %s", DevName, File, SYSERR);

    if (fd >= 0 && DevType == DT_DISKDRIVE) {
#if	defined(HAVE_ATA)
	SImsg(SIM_DBG, "%s:\tATA PROBE: Found <%s>", DevName, File);

	/*
	 * Create the DevInfo
	 */
	DevInfo = NewDevInfo(NULL);
	DevInfo->Name = strdup(DevName);
	DevInfo->Driver = strdup(Name);
	DevInfo->ClassType = CT_ATA;
	DevInfo->Unit = Unit;
	DevInfo->Type = DevType;
	DevAddFile(DevInfo, strdup(File));

	/*
	 * Perform ATA Queries
	 */
	(void) AtaQuery(DevInfo, File, fd, TRUE);

#if	defined(USE_HDIO_GET_IDENTITY)
	/* 
	 * We no longer use this, but use the direct HDIO_DRIVE_CMD command
	 * in place of HDIO_GET_IDENTIFY.
	 */
	(void) memset(&DriveID, CNULL, sizeof(DriveID));
 
	if (ioctl(fd, HDIO_GET_IDENTITY, &DriveID) < 0) {
	    SImsg(SIM_DBG, "%s: ioctl HDIO_GET_IDENTITY failed: %s",
		  File, SYSERR);
	} else {
	    /* AtaIdent and DriveID should contain same data */
	    (void) memset(&AtaIdent, CNULL, sizeof(AtaIdent));
	    (void) copybytes(&AtaIdent, sizeof(AtaIdent),
			     &DriveID, sizeof(DriveID));
	    (void) memset(&Query, CNULL, sizeof(Query));
	    Query.DevInfo = DevInfo;
	    Query.DevFile = File;
	    Query.DevFD = fd;
	    Query.Data = (void *) &AtaIdent;
	    Query.DataLen = sizeof(AtaIdent);
	    (void) AtaIdentDecode(&Query);
	}
#endif	/* HDIO_GET_IDENTITY */
#endif	/* HAVE_ATA */
    } else {
	(void) snprintf(File, sizeof(File), "%s/model", ProcPath);
	cp = GetOneLine(File);
	if (!cp)
	    /* Don't bother checking the rest of the files */
	    return (DevList_t *) NULL;

	SImsg(SIM_DBG, "%s:\tATA /proc/ide: Found <%s>", DevName, cp);

	DevInfo = NewDevInfo(NULL);
	DevInfo->Name = strdup(DevName);
	DevInfo->Driver = strdup(Name);
	DevInfo->ClassType = CT_ATA;
	DevInfo->Unit = Unit;
	DevInfo->Type = DevType;
	DevInfo->Model = strdup(cp);

	if (!DevInfo->Type) {
	    (void) snprintf(File, sizeof(File), "%s/media", Dir);
	    if (cp = GetOneLine(File)) {
		DevType_t		*Type;

		if (Type = TypeGetByName(cp))
		    DevInfo->Type = Type->Type;
	    }
	}
    }

    /*
     * Since ATA does not (yet) allow us to get the Vendor, but only
     * the Model of device, it's common for the Vendor name to be the
     * first word in the Model field.  If that appears to be the case,
     * we set Vendor to the first word and change Model to be everything
     * starting at the second word.  We look for ' ' and '-' as word
     * seperators.
     */
    if (!DevInfo->Vendor && DevInfo->Model) {
	cp = strchr(DevInfo->Model, ' ');
	if (!cp)
	    cp = strchr(DevInfo->Model, '-');
	/*
	 * Only proceed if we pass sanity tests including the new model
	 * being printable and at least N bytes long.
	 */
	if (cp && *(cp+1) && isprint(*(cp+1)) && strlen(cp+1) >= 4) {
	    SImsg(SIM_DBG, "%s:\tGrabbing Vendor from <%s>",
		  PRTS(File), DevInfo->Model);
	    DevInfo->Vendor = DevInfo->Model;
	    *cp = CNULL;
	    DevInfo->Model = cp + 1;
	}
    }

    /*
     * Choose the Device type specific probe function
     */
    switch (DevInfo->Type) {
    	case DT_FLOPPY:
    	case DT_DISKDRIVE:	ProbeFunc = ProbeDiskDrive; break;
    }

    if (ProbeFunc) {
        (void) memset(&ProbeData, CNULL, sizeof(ProbeData));
	ProbeData.DevName = DevInfo->Name;
	ProbeData.DevFile = File;
	ProbeData.FileDesc = fd;
	ProbeData.UseDevInfo = DevInfo;
        (void) (*ProbeFunc)(&ProbeData);
    }

    (void) close(fd);

    if (!DevInfo)
	return (DevList_t *) NULL;

    /*
     * Create DevList
     */
    DevList = (DevList_t *) xcalloc(1, sizeof(DevList_t));
    DevList->Name = DevInfo->Name;
    if (DevInfo->Files)
	DevList->File = DevInfo->Files[0];
    DevList->Bus = 0;		/* XXX Assume 0 */
    DevList->Unit = Unit;
    DevList->DevInfo = DevInfo;
    DevList->ClassType = CT_ATA;

    return(DevList);
}

/*
 * Sort comparision function for alpha sorts
 */
static int AlphaCompare(a1, a2)
     void		       *a1;
     void		       *a2;
{
    char		       *s1 = (char *) a1;
    char		       *s2 = (char *) a2;

    return strcmp(s2, s1);	/* increasing alpha order */
}

/*
 * Return a list of disk drives in sorted order
 */
static char **AtaDriveList()
{
    DIR			       *dp;
    struct dirent	       *Dir;
    char		       *Entry;
    static char		       *List[MAX_ATA_DRIVES];
    int				i = 0;
    char		      **cpp;

    memset(List, CNULL, sizeof(List));

    dp = opendir(PROC_PATH_BUS_IDE);
    if (!dp) {
	SImsg(SIM_DBG, "%s: Open directory failed: %s",
	      PROC_PATH_BUS_IDE, SYSERR);
	return (char **) NULL;
    }

    while (Dir = readdir(dp)) {
	Entry = Dir->d_name;

	if (!EQN(Entry, "hd", 2))
	    continue;

	SImsg(SIM_DBG, "AtaDriveList: Found <%s>", Entry);

	if (i >= sizeof(List)) {
	    SImsg(SIM_DBG,
		  "AtaDriveList: Reached max # of ATA drives (%d)",
		  sizeof(List));
	    break;
	}

	List[i++] = strdup(Entry);
    }

    closedir(dp);

    qsort((void *)List, i, sizeof(char *), AlphaCompare);

    /* List out order post sort */
    for (cpp = List; cpp && *cpp; ++cpp)
	SImsg(SIM_DBG, "AtaDriveList: Order List <%s>", *cpp);

    return List;
}

/*
 * Detect and return a list of all ATA devices we can find.
 * We only build the list once to save time.
 */
extern DevList_t *AtaDetect(DevInfo_t *CtlrDevInfo)
{
    static DevList_t	       *AtaDevices;
    static DevList_t	       *Last;
    register DevList_t	       *New = NULL;
    DevType_t		       *dt;
    FILE		       *fp;
    static char			Base[MAXPATHLEN];
    static char			Path[MAXPATHLEN];
    static char			Driver[128];
    char		      **cpp;
    char		       *Drive;
    char		       *cp;
    int				Type;
    int				Unit;

    if (AtaDevices)
	return(AtaDevices);

    for (cpp = AtaDriveList(); cpp && *cpp; ++cpp) {
	Drive = *cpp;
	Type = 0;
	Unit = -1;
	New = NULL;
	Driver[0] = CNULL;

	SImsg(SIM_DBG, "AtaDetect: Checking <%s>", Drive);

	Unit = Drive[2] - 'a';
	snprintf(Driver, sizeof(Driver), "hd");
	snprintf(Base, sizeof(Base), "%s/%s", PROC_PATH_BUS_IDE, Drive);

	/* Determine type of device */
	snprintf(Path, sizeof(Path), "%s/media", Base);
	cp = GetOneLine(Path);
	if (!cp) {
	    SImsg(SIM_DBG, "%s: Cannot read %s", Path);
	    continue;
	}

	/* Set type */
	if (dt = TypeGetByName(cp))
	    Type = dt->Type;

	/* Probe ATA device now */
	if (Type)
	    New = AtaProbe(Driver, Unit, Base, Type);

	/* Add device to list */
	if (New) {
	    if (!AtaDevices)
		AtaDevices = Last = New;
	    else {
		Last->Next = New;
		Last = New;
	    }
	}
    }

    return(AtaDevices);
}
