/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.89 $";
#endif

/*
 * Linux specific functions
 */

#include "defs.h"
#include "psi.h"
#include "dmideclare.h"
#include "linux-declare.h"
#include "dospart.h"
#include <sys/stat.h>
#include <linux/kernel.h>
#include <linux/sys.h>
#include <unistd.h>
#include <math.h>
#include <linux/pci.h>	/* For PCI_* */
#include "netif.h"
#include "part.h"
#include "device.h"

/*
 * Platform Specific Interfaces
 */
extern Opaque_t			        GetCpuInfoLinux();
extern Opaque_t			        GetCpuTypeProc();
extern Opaque_t			        GetKernVerProc();
extern Opaque_t			        GetKernVerLinux();
extern Opaque_t			        GetOSDistLinux();
extern Opaque_t			        GetOSDistNameLinux();
extern Opaque_t			        GetOSDistNameShLinux();
extern Opaque_t			        GetOSDistVerLinux();
extern Opaque_t			        GetMemorySysinfo();
extern Opaque_t			        GetMemoryKcore();
extern Opaque_t			        GetVirtMemLinux();
extern Opaque_t			        GetBootTimeProc();
extern Opaque_t			        BuildDevices();
extern Opaque_t			        BuildSoftInfoRPM();

PsiSwitch_t PsiSwitch = {
    /* GetBootTime */	{ GetBootTimeProc, NULL },
    /* GetCpuInfo */	{ GetCpuInfoLinux, NULL },
    /* GetCpuType */ 	{ GetCpuTypeCpuInfo, GetCpuTypeProc, NULL },
    /* GetAppArch */ 	{ GetKernArchUname, NULL },
    /* GetKernArch */ 	{ GetKernArchUname, NULL },
    /* GetKernBits */ 	{ GetKernBitsSysConf, GetKernBitsKernArch, NULL },
    /* GetKernVer */ 	{ GetKernVerProc, GetKernVerLinux, NULL },
    /* GetLibcName */ 	{ GetLibcNameShared, NULL },
    /* GetLibcVer */ 	{ GetLibcVersionShared, NULL },
    /* GetManLong */ 	{ GetManLongDmi, GetManLongSysinfo, 
			  GetManLongDef, NULL },
    /* GetManShort */ 	{ GetManShortDmi, GetManShortSysinfo, 
			  GetManShortDef, NULL },
    /* GetMemory */ 	{ GetMemoryDmi, GetMemoryKcore, GetMemorySysinfo, 
			  NULL },
    /* GetVirtMem */ 	{ GetVirtMemLinux, NULL },
    /* GetModel */ 	{ GetModelDmi, GetModelCpuInfo, GetModelFile, NULL },
    /* GetNumCpu */ 	{ DmiGetNumCpu, GetNumCpuSysconf, NULL },
    /* GetOsDist */ 	{ GetOSDistLinux, NULL },
    /* GetOsName */ 	{ GetOSNameUname, NULL },
    /* GetOsVer */ 	{ GetOSVerUname, NULL },
    /* GetRomVer */  	{ NULL },
    /* GetSerial */ 	{ GetSerialDmi, NULL },
    /* BuildDevices */  { BuildDevices, NULL },
    /* BuildSoftInfo */ { BuildSoftInfoRPM, NULL },
    /* BuildPartInfo */ { BuildPartInfoStd, NULL },
    /* BuildPrinter */	{ BuildPrinterLpr, NULL },
    /* BuildLicense */	{ BuildLicenseFlexlm, NULL },
    /* GetOsDistName */	{ GetOSDistNameLinux, NULL },
    /* GetOsDistVer */	{ GetOSDistVerLinux, NULL },
    /* GetOsDistNameSh */{ GetOSDistNameShLinux, NULL },
    /* GetNumCpuVirt */	{ GetNumCpuSysconf, NULL },
    /* BuildNetIf */ 	{ NetIfBuild },
    /* BuildNetwork */ 	{ NetworkBuild },
};

struct stat 			Stat;

/*
 * Get Boot Time by using the /proc/uptime file.
 */
extern Opaque_t GetBootTimeProc()
{
    FILE		       *fp;
    static char			Buff[64];
    char		       *cp;
    char		       *DateStr;
    time_t			Uptime;
    time_t			BootTime;

    fp = fopen(PROC_FILE_UPTIME, "r");
    if (!fp) {
	SImsg(SIM_GERR, "%s: open readonly failed: %s", 
	      PROC_FILE_UPTIME, SYSERR);
	return((char *) NULL);
    }

    if (!fgets(Buff, sizeof(Buff), fp)) {
	SImsg(SIM_GERR, "%s: Read uptime failed: %s",
	      PROC_FILE_UPTIME, SYSERR);
	(void) fclose(fp);
	return((char *) NULL);
    }

    if (cp = strchr(Buff, ' '))
	*cp = CNULL;

    Uptime = (time_t) strtol(Buff, NULL, 0);
    if (Uptime <= 0) {
	SImsg(SIM_GERR, "Convert `%s' to long failed", Buff);
	(void) fclose(fp);
	return((char *) NULL);
    }

    BootTime = time(NULL);
    BootTime -= Uptime;

    DateStr = TimeToStr(BootTime, NULL);

    (void) fclose(fp);

    return(DateStr);
}

/*
 * Get CPU Info using /proc/cpuinfo
 * Assumes there is only 1 CPU or all CPU's are the same.
 */
extern CpuInfo_t *GetCpuInfoProc()
{
    static CpuInfo_t		CpuInfo;
    FILE		       *fp;
    static char			Buff[256];
    float			Speed = 0;
    char		      **Argv;
    int				Argc;
    int				Cleanup;

    SImsg(SIM_DBG, "GetCpuInfoProc: Start");

    if (CpuInfo.Model || CpuInfo.Man) 
	return &CpuInfo;

    fp = fopen(PROC_FILE_CPUINFO, "r");
    if (!fp) {
	SImsg(SIM_GERR, "%s: open readonly failed: %s", 
	      PROC_FILE_CPUINFO, SYSERR);
	return (CpuInfo_t *) NULL;
    }

    while (fgets(Buff, sizeof(Buff), fp)) {
	Cleanup = TRUE;
	Argc = StrToArgv(Buff, ":", &Argv, NULL, 0);
	if (Argc < 2)
	    continue;
	if (EQ(Argv[0], "vendor_id") && Argv[1][0] && !CpuInfo.Man)
	    CpuInfo.Man = CpuManEdit(strdup(Argv[1]));
	else if (EQ(Argv[0], "model name") && Argv[1][0] && !CpuInfo.Model)
	    CpuInfo.Model = CpuModelEdit(strdup(Argv[1]));
	else if (EQ(Argv[0], "fpu") && EQ(Argv[1], "yes") && !CpuInfo.HasFPU)
	    CpuInfo.HasFPU = TRUE;
	else if (EQ(Argv[0], "cpu MHz") && Argv[1][0] && !CpuInfo.Speed) {
	    /* 
	     * This value is not always perfectly accurate as Linux estimates
	     * the actual Mhz by doing a loop test at boot.
	     */
	    if (sscanf(Argv[1], "%f", &Speed))	
		CpuInfo.Speed = rint((double)Speed);
		/*	(void) sprintf(Speed, "%.0f Mhz", rint((double)sp));*/
	} else {
	    DestroyArgv(&Argv, Argc);
	    Cleanup = FALSE;
	}

	if (Cleanup)
	    (void) free(Argv[0]);
    }

    (void) fclose(fp);

    SImsg(SIM_DBG, "GetCpuInfoProc: Man=<%s> Model=<%s> Speed=%d",
	  CpuInfo.Man, CpuInfo.Model, CpuInfo.Speed);

    return &CpuInfo;
}

/*
 * Get CPU information.
 */
extern Opaque_t GetCpuInfoLinux()
{
    CpuInfo_t		       *CpuInfo = NULL;
    CpuInfo_t		       *CpuInfoProc;
    int				Speed;

#if	defined(IS_X86)
    if (!CpuInfo)
	if (CpuInfo = GetCpuInfoX86()) {
	    /* Try to grab the speed from other sources */
	    if (!CpuInfo->Speed && (CpuInfoProc = GetCpuInfoProc()))
		CpuInfo->Speed = CpuInfoProc->Speed;
	}
#endif	/* IS_X86 */

    if (!CpuInfo)
	CpuInfo = GetCpuInfoProc();

    /* DMI version of Speed is the most accurate */
    if (CpuInfo && (Speed = DmiGetCpuSpeed()))
	CpuInfo->Speed = (Large_t) Speed;

    return CpuInfo;
}

/*
 * Get System Memory using size of /proc/kcore
 */
extern Opaque_t GetMemoryKcore()
{
    static char		       *MemStr = NULL;
    Large_t			MemBytes = 0;
    Large_t			Amount = 0;
    char			Kcore[] = "/proc/kcore";
    struct stat			StatBuf;

    if (MemStr)
	return(MemStr);

    if (stat(Kcore, &StatBuf) != 0) {
	SImsg(SIM_GERR, "%s: stat failed: %s", Kcore, SYSERR);
	return((char *) NULL);
    }

    MemBytes = (Large_t) (StatBuf.st_size - 4096);
    Amount = DivRndUp(MemBytes, (Large_t)MBYTES);
    MemStr = GetSizeStr(Amount, MBYTES, 0);
    SImsg(SIM_DBG, "GetMemoryKcore: Amount=%ld", Amount);
    
    return(MemStr);
}

/*
 * Get System Memory using sysinfo() system call
 */
extern Opaque_t GetMemorySysinfo()
{
    struct sysinfo		si;
    static char		       *MemStr = NULL;
    char		       *cp;
    Large_t			MemBytes = 0;

    if (MemStr)
	return(MemStr);

    if (sysinfo(&si) != 0) {
	SImsg(SIM_GERR, "sysinfo() system call failed: %s", SYSERR);
	return((char *) NULL);
    }

#if	defined(HAVE_SYSINFO_MEM_UNIT)
    SImsg(SIM_DBG, "GetMemorySysinfo: totalram=%d bufferram=%d mem_unit=%d",
	  si.totalram, si.bufferram, si.mem_unit);
#else
    SImsg(SIM_DBG, "GetMemorySysinfo: totalram=%d bufferram=%d",
	  si.totalram, si.bufferram);
#endif

    /*
     * sysinfo.totalram represents total USABLE physical memory.  Memory
     * reserved by the kernel is not included.  So this is as close as we
     * can get for now.  Sigh!
     */
    MemBytes = (Large_t) si.totalram;

#if	defined(HAVE_SYSINFO_MEM_UNIT)
    if (si.mem_unit)
	MemBytes *= (Large_t) si.mem_unit;
#endif
    if (cp = GetSizeStr(MemBytes, BYTES, 0))
	MemStr = strdup(cp);
    SImsg(SIM_DBG, "GetMemorySysinfo: Memory=%s", PRTS(MemStr));
    
    return(MemStr);
}

/*
 * Get Virtual Memory using sysinfo() system call
 */
extern Opaque_t GetVirtMemLinux()
{
    struct sysinfo		si;
    static char		       *MemStr = NULL;
    char		       *cp;
    Large_t			MemBytes = 0;
    Large_t			Amount = 0;

    if (MemStr)
	return(MemStr);

    if (sysinfo(&si) != 0) {
	SImsg(SIM_GERR, "sysinfo() system call failed: %s", SYSERR);
	return((char *) NULL);
    }

#if 	OSVER >= 24
    MemBytes = (Large_t) si.totalram + (Large_t) si.bufferram + 
	(Large_t) si.totalswap;
#else
    MemBytes = (Large_t) si.totalram + (Large_t) si.totalswap;
#endif
#if	defined(HAVE_SYSINFO_MEM_UNIT)
    if (si.mem_unit)
	MemBytes *= (Large_t) si.mem_unit;
#endif
    Amount = DivRndUp(MemBytes, (Large_t)MBYTES);
    if (cp = GetSizeStr(Amount, MBYTES, 0))
	MemStr = strdup(cp);
    
    return(MemStr);
}

/*
 * Get CPU Type from /proc/cpuinfo
 */
extern Opaque_t GetCpuTypeProc()
{
    FILE		       *fp;
    static char			Buff[256];
    static char		       *Cpu = NULL;
    char		      **Argv;
    char		       *cp;
    int				Argc;

    if (Cpu)
	return(Cpu);

    fp = fopen(PROC_FILE_CPUINFO, "r");
    if (!fp) {
	SImsg(SIM_GERR, "%s: open readonly failed: %s", 
	      PROC_FILE_CPUINFO, SYSERR);
	return((char *) NULL);
    }

    while (fgets(Buff, sizeof(Buff), fp)) {
	Argc = StrToArgv(Buff, ":", &Argv, NULL, 0);
	if (Argc < 2)
	    continue;
	if (EQ(Argv[0], "cpu")) {
	    /* Linux 2.0 */
	    Cpu = Argv[1];
	    break;
	} else if (EQ(Argv[0], "model name")) {
	    /* Linux 2.2 */
	    Cpu = Argv[1];
	    if (cp = strchr(Cpu, ' '))
		*cp = CNULL;
	    break;
	}
    }

    (void) fclose(fp);

    return(Cpu);

}

/*
 * Get Kernel Version string using /proc/version
 */
extern Opaque_t GetKernVerProc()
{
    FILE		       *fp;
    static char			Buff[512];
    char		      **Argv;
    int				Argc;

    if (Buff[0])
	return(Buff);

    fp = fopen(PROC_FILE_VERSION, "r");
    if (!fp) {
	SImsg(SIM_GERR, "%s: open readonly failed: %s", 
	      PROC_FILE_VERSION, SYSERR);
	return((char *) NULL);
    }

    if (!fgets(Buff, sizeof(Buff), fp)) {
	SImsg(SIM_GERR, "%s: read failed: %s", PROC_FILE_VERSION, SYSERR);
	return((char *) NULL);
    }

    (void) fclose(fp);

    return(Buff);

}

/*
 * Get Kernel Version string using uname()
 */
extern Opaque_t GetKernVerLinux()
{
    static struct utsname	uts;
    static char		       *VerStr = NULL;

    if (uname(&uts) != 0) {
	SImsg(SIM_GERR, "uname() system call failed: %s", SYSERR);
	return((char *) NULL);
    }

    VerStr = uts.version;

    return(VerStr);
}

/*
 * What distribution (vendor) of Linux are we?
 * This code works on S.u.S.E. Linux.  Don't know about others.
 */
extern Opaque_t GetOSDistLinux()
{
    static char			Buff[256];
    static char		       *Dist;
    register char	       *cp;
    register char	       *End;
    char			IssueFile[] = "/etc/issue";
    char			Welcome[] = "Welcome to ";
    FILE		       *fp;
    int				Found = FALSE;

    if (Dist)
	return Dist;

    if (!(fp = fopen(IssueFile, "r"))) {
	SImsg(SIM_GERR, "%s: Cannot open to get OS Dist: %s",
	      IssueFile, SYSERR);
	return((char *) NULL);
    }

    while (fgets(Buff, sizeof(Buff), fp)) {
	/* Some distributions have VGA control chars in them */
	if (!isalpha(Buff[0]))
	    continue;
	for (cp = Buff; cp && *cp && *cp != '\n' && !isalpha(*cp); ++cp);
	if (*cp == '\n' || !strlen(cp))
	    continue;
	/* Found first nonblank line */
	Found = TRUE;
	break;
    }

    (void) fclose(fp);

    if (!Found)
	return((char *) NULL);

    if (EQN(cp, Welcome, sizeof(Welcome)-1))
	cp += sizeof(Welcome)-1;
    else if (EQN(cp, "Linux ", 6))
	cp += 6;

    if (End = strchr(cp, '-')) {
	--End;
	while (*End && isspace(*End))
	    --End;
	*++End = CNULL;
    }

    return Dist = cp;
}

/*
 * Get OS Distribution Name
 */
extern Opaque_t GetOSDistNameLinux()
{
    static char		       *DistName;
    char		       *cp;
    char		       *Dist;
    int				Argc;
    char		      **Argv;
    register int		i;

    if (DistName)
	return DistName;

    if (cp = (char *) GetOSDistLinux())
	Dist = strdup(cp);
    else
	return ((char *) NULL);

    Argc = StrToArgv(Dist, " ", &Argv, NULL, 0);
    /*
     * Find the first arg which looks like the start of something other than
     * the dist name.
     */
    for (i = 0; i < Argc; ++i) {
	if (isdigit(Argv[i][0]) || EQ(Argv[i], "release"))
	    break;
    }

    if (ArgvToStr(&DistName, Argv, Argc - i, " ", 0) < 0)
	return (char *) NULL;

    return DistName;
}

/*
 * Get OS Distribution Name (Short)
 */
extern Opaque_t GetOSDistNameShLinux()
{
    static char		       *DistName;
    char		       *cp;
    char		       *Dist;
    int				Argc;
    char		      **Argv;
    register int		i;

    if (DistName)
	return DistName;

    Dist = (char *) GetOSDistNameLinux();
    if (!Dist)
	return (Opaque_t) NULL;

    if (EQN(Dist, "Red Hat", 7))
	DistName = "redhat";

    return DistName;
}

/*
 * Get OS Distribution Version
 */
extern Opaque_t GetOSDistVerLinux()
{
    static char		       *DistVer;
    char		       *cp;
    char		       *Dist;
    int				Argc;
    char		      **Argv;
    register int		i;

    if (DistVer)
	return DistVer;

    if (cp = (char *) GetOSDistLinux())
	Dist = strdup(cp);
    else
	return ((char *) NULL);

    Argc = StrToArgv(Dist, " ", &Argv, NULL, 0);
    /*
     * Find the first arg which looks like the start of the version
     */
    for (i = 0; i < Argc; ++i) {
	if (isdigit(Argv[i][0])) {
	    DistVer = Argv[i];
	    break;
	}
    }

    return DistVer;
}

/*
 * Get Partition information
 */
static int GetPart(DevInfo, DevFile, DiskDrive)
     DevInfo_t		       *DevInfo;
     char		       *DevFile;
     DiskDrive_t	       *DiskDrive;
{
    DiskPart_t		       *Part;

    if (!DevInfo || !DiskDrive)
	return -1;

    for (Part = DiskDrive->DiskPart; Part; Part = Part->Next)
	GetPartUsage(DevInfo, DevFile, Part);

    return 0;
}

/*
 * Read one line of a file
 */
extern char *GetOneLine(File)
     char		       *File;
{
    FILE		       *Fp;
    static char			Buff[1024];
    char		       *cp;

    if (Fp = fopen(File, "r")) {
	if (fgets(Buff, sizeof(Buff), Fp)) {
	    if (cp = strchr(Buff, '\n'))
		*cp = CNULL;
	    return Buff;
	}
	(void) fclose(Fp);
    }

    return (char *) NULL;
}

/*
 * Probe a DiskDrive device
 */
extern DevInfo_t *ProbeDiskDrive(ProbeData)
     ProbeData_t	       *ProbeData;
{
    DevInfo_t		       *DevInfo = NULL;
    DiskDrive_t		       *Disk = NULL;
    DiskDriveData_t	       *DiskDriveData = NULL;
    FILE		       *Fp;
    static DosPartQuery_t	Query;
    static char		      **Argv;
    int				Argc;
    char		       *DevFile;
    static char			Buff[512];
    static char			Dir[128];
    static char			File[128];
    char		       *cp;

    if (!ProbeData || !ProbeData->DevName || 
	!ProbeData->DevFile)
	return((DevInfo_t *)NULL);

    if (ProbeData->UseDevInfo)
	DevInfo = ProbeData->UseDevInfo;
    else
	DevInfo = DeviceCreate(ProbeData);
    DevFile = ProbeData->DevFile;

    SImsg(SIM_DBG, "ProbeDiskDrive: DevName=<%s>",
	  PRTS(ProbeData->DevName));

    if (DevInfo->DevSpec)
	DiskDriveData = (DiskDriveData_t *) DevInfo->DevSpec;
    else {
	DiskDriveData = NewDiskDriveData(NULL);
	DevInfo->DevSpec = (void *) DiskDriveData;
    }

    /*
     * Linux "partitions" are really just DOS partitions.
     * First we do our standard get DOS partitions, then we
     * add usage info to it.
     */
    (void) memset(&Query, 0, sizeof(Query));
    Query.DevInfo = DevInfo;
    Query.CtlDev = DevFile;
    (void) DosPartGet(&Query);
    (void) GetPart(DevInfo, DevFile,
		   (DiskDriveData->HWdata) ? DiskDriveData->HWdata :
		   DiskDriveData->OSdata);

    /*
     * This stuff obviously only works on ATA (IDE) drives
     */
    (void) snprintf(Dir, sizeof(Dir), "/proc/ide/%s", ProbeData->DevName);
    (void) snprintf(File, sizeof(File), "%s/capacity", Dir);

    SImsg(SIM_DBG, "ProbeDiskDrive: \tProcDir=<%s>", Dir);

    if (cp = GetOneLine(File)) {
	Large_t			Size;

	if (!DiskDriveData->HWdata)
	    DiskDriveData->HWdata = NewDiskDrive(NULL);
	if (!DiskDriveData->OSdata)
	    DiskDriveData->OSdata = NewDiskDrive(NULL);

	if (!DiskDriveData->OSdata->Size) {
	    Size = atoL(cp);
	    DiskDriveData->OSdata->Size = (float) (((float)Size / (float)2) / 
						   (float)1024);
	}

	/*
	 * Read the geometry
	 */
	(void) snprintf(File, sizeof(File), "%s/geometry", Dir);
	if (Fp = fopen(File, "r")) {
	    while (fgets(Buff, sizeof(Buff), Fp)) {
		if (cp = strchr(Buff, '\n'))
		    *cp = CNULL;
		if ((cp = strchr(Buff, ' ')) || (cp = strchr(Buff, '\t'))) {
		    *cp = CNULL;
		    ++cp;
		}
		if (EQ(Buff, "physical"))
		    Disk = DiskDriveData->HWdata;
		else if (EQ(Buff, "logical"))
		    Disk = DiskDriveData->OSdata;
		else {
		    SImsg(SIM_DBG, "The word <%s> from %s is unknown.",
			  Buff, File);
		    continue;
		}

		if ((Argc = StrToArgv(cp, "/", &Argv, NULL, 0)) &&
		    Argc == 3) {
		    Disk->DataCyl = (u_long) atol(Argv[0]);
		    Disk->Tracks = (u_long) atol(Argv[1]);
		    Disk->Sect = (u_long) atol(Argv[2]);
		}

	    }
	    (void) fclose(Fp);
	}
    }

    SImsg(SIM_DBG, "ProbeDiskDrive: \tReturning DevInfo->Name=<%s>",
	  (DevInfo) ? PRTS(DevInfo->Name) : "");

    return (ProbeData->RetDevInfo = DevInfo);
}

/*
 * Call Device Specific probe function
 */
static int ProbeDeviceSpec(DevInfo, TreePtr, SearchNames)
     DevInfo_t		       *DevInfo;
     DevInfo_t 		      **TreePtr;
     char		      **SearchNames;
{
    char		       *Name;
    static char			DevName[64];
    static int			NetIfs;
    static ProbeData_t		ProbeData;

    if (!DevInfo)
	return -1;

    Name = DevInfo->Name;

    switch (DevInfo->Type) {
    case DT_DISKCTLR:
    case DT_CONTROLLER:
	switch (DevInfo->ClassType) {
	case CT_SCSI:
	case CT_ATA:
	case CT_RAID:
	    ProbeCtlrDetect(DevInfo, TreePtr, SearchNames);	break;
	default:
	    SImsg(SIM_DBG, 
		  "%s: No probe func for DevType=%d ClassType=%d",
		  Name, DevInfo->Type, DevInfo->ClassType);
	}
	break;
    case DT_NETIF:
        /* 
	 * XXX We assume we get the NetIf's (ethernets) in order they are
	 * dynamically numbered by the kernel (ethX).
	 */
	(void) snprintf(DevName, sizeof(DevName), "eth%d", NetIfs++);
	DevInfo->AltName = DevInfo->Name;
	DevInfo->Name = strdup(DevName);
	(void) memset(&ProbeData, 0, sizeof(ProbeData));
	ProbeData.DevName = DevInfo->Name;
	ProbeData.UseDevInfo = DevInfo;
	(void) ProbeNetif(&ProbeData);
	break;
    case DT_BRIDGE:
	/* No special probing for these device types */
	break;
    default:
	if (DevInfo->Type >= 0)
	    SImsg(SIM_DBG, 
		  "%s: No probe func for DevType=%d",
		  Name, DevInfo->Type);
    }

    return 0;
}

/*
 * Set the PCI configuration info for this device (PCIinfo)
 */
static int PCISetConfig(PCIinfo)
     PCIinfo_t		       *PCIinfo;
{
    static char			File[128];
    static char			Config[PCI_CONFIG_SIZE];
    int				fd;
    int				Status;

    /*
     * Open the PCI configuration device file for this specific device
     */
    (void) snprintf(File, sizeof(File), "%s/%02x/%02x.%d",
		    PROC_PATH_BUS_PCI,
		    PCIinfo->Bus, PCIinfo->Unit, PCIinfo->SubDeviceID);

    fd = open(File, O_RDONLY);
    if (fd < 0) {
        SImsg(SIM_DBG, "%s: Cannot open PCI device file to get CLASS info: %s",
	      File, SYSERR);
	return -1;
    }

    Status = read(fd, Config, sizeof(Config));
    (void) close(fd);

    if (Status != sizeof(Config)) {
        SImsg(SIM_DBG, "%s: Read PCI device failed (result %d): %s",
	      File, Status, SYSERR);
	return -1;
    }

    PCIinfo->Class = PCI_CONFIG_WORD(Config, PCI_CLASS_DEVICE);
    PCIinfo->Revision = PCI_CONFIG_BYTE(Config, PCI_REVISION_ID);
    PCIinfo->Header = PCI_CONFIG_BYTE(Config, PCI_HEADER_TYPE);

    return 0;
}

/*
 * Build the device tree by scanning the PCI bus
 */
extern int BuildPCI(DevInfo_t **TreePtr, char **SearchNames)
{
    DevInfo_t                  *DevInfo;
    FILE		       *fp;
    static char                 Buff[512];
    char                        NumStr[16];
    char                      **Argv;
    int                         Argc;
    static PCIinfo_t            Info;
    static DevData_t		DevData;
    char                        DevName[64];
    PCIid_t			DefNum;
    PCIid_t			Vendor;
    PCIid_t			IRQ;
    ProbeData_t			ProbeData;

    if (!(fp = fopen(PROC_FILE_PCI_DEV, "r"))) {
	SImsg(SIM_GERR, "%s: Open failed: %s", PROC_FILE_PCI_DEV, SYSERR);
	return -1;
    }

    while (fgets(Buff, sizeof(Buff)-1, fp)) {
	/*
	 * Format of input is:
	 *    DEFNUM VVVVMMMM IRQ ...
	 *
	 * DEFNUM = Bus, Unit, Func
	 * DD = Device Number
	 * VVVV = Vendor PCI ID
	 * MMMM = Vendor's Model ID
	 * IRQ = IRQ
	 * ... = The rest we ignore
	 */
        Argc = sscanf(Buff, "%x %x %x", 
		      &DefNum, &Vendor, &IRQ);
	if (Argc != 3) {
	    SImsg(SIM_GERR, "%s: Wrong number of arguments found (got %d)",
		  PROC_FILE_PCI_DEV, Argc);
	    continue;
	}
	(void) PCInewInfo(&Info);

	Info.Bus = (PCIid_t) (DefNum >> 8U);
	Info.Unit = (PCIid_t) PCI_SLOT(DefNum & 0xFF);
	Info.SubDeviceID = (PCIid_t) PCI_FUNC(DefNum & 0xFF);

	Info.VendorID = Vendor >> 16U;
	Info.DeviceID = Vendor & 0xFFFF;

	(void) PCISetConfig(&Info);

	SImsg(SIM_DBG, 
	      "PCI: Bus=0x%02x Unit=0x%02x SubID=0x%x Vendor=0x%04x Device=0x%04x Class=0x%04x",
	      Info.Bus, Info.Unit, Info.SubDeviceID,
	      Info.VendorID, Info.DeviceID, Info.Class);

	PCIsetDeviceInfo(&Info);

	AddDesc(&(Info.DevInfo->DescList), "IRQ", itoa(IRQ), DA_INSERT);

	/* Build DevData for proper probing */
	(void) memset(&DevData, 0, sizeof(DevData));
	DevData.DevName = Info.DevInfo->Name;
	DevData.DevUnit = Info.Unit;
	DevData.Slave = -1;
	DevData.DevNum = -1;
	DevData.Flags |= DD_IS_ALIVE;
	DevData.CtlrName = "pci";
	DevData.CtlrUnit = Info.Bus;
	DevData.OSDevInfo = Info.DevInfo;

	/* Probe and add device */
	if (TreePtr && (DevInfo = (DevInfo_t *) 
			ProbeDevice(&DevData, TreePtr, SearchNames, NULL))) {
	    AddDevice(DevInfo, TreePtr, SearchNames);
	    if (DevInfo->Master) {
		DevInfo->Master->Type = DT_BUS;
		DevInfo->Master->ClassType = CT_PCI;
	    }
	}

	/*
	 * Do followup work such as looking for children on this device
	 */
	DevInfo->Vec = Info.SubDeviceID;	/* Hide the bus # */
	ProbeDeviceSpec(DevInfo, TreePtr, SearchNames);
	DevInfo->Vec = -1;			/* Zap it */
    }

    (void) fclose(fp);

    CpuInfoBuild(TreePtr, SearchNames);

    return 0;
}

/*
 * Build Misc device tree
 */
extern int BuildMisc(DevInfo_t **TreePtr, char **SearchNames)
{
    int				Status = 0;

    (void) DetectFloppy(TreePtr, SearchNames);
    (void) DetectMouse(TreePtr, SearchNames);
    (void) DetectParPort(TreePtr, SearchNames);
    (void) DetectUsb(TreePtr, SearchNames);
    (void) BuildDDC(TreePtr, SearchNames);

    return Status;
}

/*
 * Initialize the OS specific parts of the Device Types table
 */
void DevTypesInit()
{
    register int		i;
    static int 			First = TRUE;

    if (!First)
	return;

    First = FALSE;

    for (i = 0; DevTypes[i].Name; ++i)
	switch (DevTypes[i].Type) {
	case DT_DISKDRIVE:	DevTypes[i].Probe = ProbeDiskDrive;	break;
	}
}

/*
 * Build the device tree
 */
extern Opaque_t BuildDevices(DevInfo_t **TreePtr, char **SearchNames)
{
    int				Status = 0;

    /*
     * We need to call this specially since we don't normal have devices
     * defined in our .cf file which normally causes this to happen.
     */
    DevTypesInit();

    /* Scan DMI */
    (void) BuildDmi(TreePtr, SearchNames);

    /* Scan the PCI bus */
    if (BuildPCI(TreePtr, SearchNames) != 0)
	--Status;

    /* Scan misc */
    if (BuildMisc(TreePtr, SearchNames) != 0)
	--Status;

    /* Check for orphaned disks */
    DiskOrphanCheck(TreePtr, SearchNames);

    return (Opaque_t) ((Status == -2) ? -1 : 0);
}

#if	defined(HAVE__LLSEEK)
/*
 * Use the Linux specific _llseek() for now.
 */
#include <sys/types.h>
#include <unistd.h>
#include <linux/unistd.h>
extern Offset_t Linuxllseek(fd, Offset, Flags)
     int			fd;
     Offset_t			Offset;
     int			Flags;
{
    int				Ret;
    Offset_t			Result;

    Ret = _llseek(fd, ((unsigned long long) Offset) >> 32,
		  ((unsigned long long) Offset) & 0xffffffff,
		  &Result, Flags);

    return (Ret == -1) ? (Offset_t) Ret : Result;
}
#endif	/* HAVE__LLSEEK */
