/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.6 $";
#endif

/*
 * Linux parallel port functions
 */

#include "defs.h"
#include <dirent.h>

#define PATH_PROC_PARPORT_DIR 	"/proc/sys/dev/parport"	/* Where devs are */
#define PARPORT_BASENAME	"parport"		/* Base name in _DIR */

/*
 * Read the autoprobe data and set it in DevInfo
 */
static int ParPortAutoProbe(DevInfo, Dir)
     DevInfo_t		       *DevInfo;
     char		       *Dir;
{
    DevType_t		       *DevType;
    static char			Path[MAXPATHLEN];
    static char			Buff[1024];
    char		      **Argv;
    int				Argc;
    char		      **Argv2;
    int				Argc2;
    FILE		       *FH;
    char	       	       *cp;
    register int		i;

    if (!DevInfo || !Dir)
	return -1;

    snprintf(Path, sizeof(Path), "%s/autoprobe", Dir);
    FH = fopen(Path, "r");
    if (!FH) {
	SImsg(SIM_DBG, "ParPort: %s: Open for read failed: %s", Path, SYSERR);
	return -1;
    }

    /*
     * File looks like:
     * CLASS:PRINTER;
     * MODEL:Lexmark Optra E310;
     * MANUFACTURER:Lexmark International;
     * DESCRIPTION:Lexmark Optra E310;
     */
    while (fgets(Buff, sizeof(Buff), FH)) {
	if (cp = strchr(Buff, '\n'))
	    *cp = CNULL;

	Argc = StrToArgv(Buff, ":", &Argv, NULL, 0);
	if (Argc != 2) {
	    SImsg(SIM_DBG, "ParPort: %s: Unknown line: %s",
		  Path, Buff);
	    continue;
	}

	if (cp = strchr(Argv[1], ';'))
	    *cp = CNULL;

	if (EQ(Argv[0], "class")) {
	    if (DevType = TypeGetByName(Argv[1]))
		DevInfo->Type = DevType->Type;
	} else if (EQ(Argv[0], "model")) {
	    DevInfo->Model = Argv[1];
	} else if (EQ(Argv[0], "description")) {
	    /* Use Description if Model is not already set */
	    if (!DevInfo->Model)
		DevInfo->Model = Argv[1];
	} else if (EQ(Argv[0], "manufacturer")) {
	    DevInfo->Vendor = Argv[1];
	} else if (EQ(Argv[0], "command set")) {
	    Argc2 = StrToArgv(Argv[1], ",", &Argv2, NULL, 0);
	    for (i = 0; i < Argc2; ++i)
		if (Argv2[i])
		    AddDesc(&(DevInfo->DescList), "Command Set", Argv2[i], 0);
	} else {
	    SImsg(SIM_DBG, "%s: Key <%s> unknown: %s",
		  Path, Argv[0], Buff);
	}
    }

    (void) fclose(FH);

    /*
     * If the Model and Vendor both start with the same word, then that
     * means the Model includes part of the Vendor's name so skip over that
     * part so that DevInfo->Model doesn't contain the Vendor name.
     */
    if (DevInfo->Model && DevInfo->Vendor) {
	if (cp = strchr(DevInfo->Model, ' ')) {
	    if (EQN(DevInfo->Model, DevInfo->Vendor, cp - DevInfo->Model))
		DevInfo->Model = cp + 1;
	}
    }

    return 0;
}

/*
 * Probe a parellel port to see what's attached
 */
extern DevInfo_t *ProbeParPort(ProbeData)
     ProbeData_t	       *ProbeData;
{
    DevInfo_t		       *DevInfo = NULL;
    DevData_t		       *DevData;
    static char			Dir[MAXPATHLEN];
    static char			DevName[512];
    static char			Buff[512];
    char		       *NewName = NULL;
    int				Unit = -1;

    if (!ProbeData || !ProbeData->DevData)
	return (DevInfo_t *) NULL;

    /*
     * Create temporary credentials
     */
    DevData = ProbeData->DevData;
    Unit = DevData->DevUnit;
    if (Unit < 0)
	Unit = UnitAssign(DT_PARALLEL, -1, NULL);
    snprintf(DevName, sizeof(DevName), "%s%d", PARPORT_BASENAME, Unit);
    snprintf(Dir, sizeof(Dir), "%s/%s", PATH_PROC_PARPORT_DIR, DevName);

    SImsg(SIM_DBG, "ParPort: Probing <%s>", Dir);

    if (!IsDir(Dir, 0)) {
	SImsg(SIM_DBG, "ParPort: %s: No such directory", Dir);
	return (DevInfo_t *) NULL;
    }

    DevInfo = DevInfoNew(NULL);

    ParPortAutoProbe(DevInfo, Dir);

    /*
     * Now that we have the true Type, set things up properly
     */
    if (DevInfo->Type < 0)
	DevInfo->Type = DT_PARALLEL;
    else
	DevInfo->Unit = UnitAssign(DevInfo->Type, -1, NULL);
    snprintf(DevName, sizeof(DevName), "%s%d", PARPORT_BASENAME, Unit);
    DevInfo->Name = strdup(DevName);

    /*
     * Now that we know the specific type of device, choose a better name.
     */
    switch (DevInfo->Type) {
    case DT_PRINTER:		NewName = "lp";		break;
    case DT_FLOPPYCTLR:		NewName = "fdc";	break;
    case DT_SCANNER:		NewName = "scanner";	break;
    }
    if (NewName) {
	(void) free(DevInfo->Name);
	snprintf(DevName, sizeof(DevName), "%s%d", NewName, DevInfo->Unit);
	DevInfo->Name = strdup(DevName);
    }

    return DevInfo;
}

/*
 * Detect devices attached to parallel ports
 */
extern int DetectParPort(DevInfo_t **TreePtr, char **SearchNames)
{
    DevInfo_t		       *DevInfo;
    static ProbeData_t		ProbeData;
    static DevData_t		DevData;
    struct dirent	       *Dir;
    DIR			       *dp;
    char		       *Entry;
    char		       *cp;
    int				Unit;

    ProbeDataInit(&ProbeData);
    DevDataInit(&DevData);

    DevData.DevName = PARPORT_BASENAME;
    ProbeData.DevData = &DevData;

    dp = opendir(PATH_PROC_PARPORT_DIR);
    if (!dp) {
	SImsg(SIM_DBG, "%s: Open directory failed: %s",
	      PATH_PROC_PARPORT_DIR, SYSERR);
	return -1;
    }

    while (Dir = readdir(dp)) {
	Entry = Dir->d_name;
	Unit = -1;

	if (EQN(Entry, PARPORT_BASENAME, sizeof(PARPORT_BASENAME))) {
	    Unit = Entry[sizeof(PARPORT_BASENAME)];
	    DevData.DevUnit = Unit;
	    if (DevInfo = ProbeParPort(&ProbeData))
		AddDevice(DevInfo, TreePtr, SearchNames);
	}
    }

    closedir(dp);

    return 0;
}
