/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.12 $";
#endif

/*
 * Linux SCSI specific functions
 */

#include "defs.h"
#include "strutil.h"
#include "linux-declare.h"
#include "myscsi.h"
#include "sg_err.h"

#if	defined(HAVE_SCSI_SCSI_H)
#	include <scsi/scsi.h>
#else	/* ! HAVE_SCSI_SCSI_H */
# 	if 	defined(HAVE__USR_SRC_LINUX_INCLUDE_SCSI_SCSI_H)
#		include "/usr/src/linux/include/scsi/scsi.h"
#	endif	/* HAVE__USR_SRC_LINUX_INCLUDE_SCSI_SCSI_H */
#endif	/* HAVE_SCSI_SCSI_H */

#if	defined(HAVE_SCSI_SG_H)
#	include <scsi/sg.h>
#	define HAVE_SCSI_SUPPORT
#else	/* !HAVE_SCSI_SCSI_H */
# 	if 	defined(HAVE__USR_SRC_LINUX_INCLUDE_SCSI_SG_H)
#		include "/usr/src/linux/include/scsi/sg.h"
#		define HAVE_SCSI_SUPPORT
#	endif	/* HAVE__USR_SRC_LINUX_INCLUDE_SCSI_SG_H */
#endif	/* HAVE_SCSI_SCSI_H */

/*
 * Documented in sg v3, but not defined by default for some reason
 */
#ifndef SCSI_IOCTL_GET_PCI
#define SCSI_IOCTL_GET_PCI 0x5387
#endif

/*
 * SG V3 or later SCSI interface.
 * Issue a SCSI command and return the results.  This function uses
 * the SCSI Generic (sg) interface.  See 
 *	http://www.torque.net/sg/p/sg_v3_ho.html
 */
extern int ScsiCmdSGv3(ScsiCmd, Query)
     ScsiCmd_t		       *ScsiCmd;
     ScsiQuery_t	       *Query;
{
#if defined(SG_IO)
    static char			Buff[SCSI_BUF_LEN];
    unsigned char		SenseBuff[32];	/* Arbitrary */
    int				Status;
    sg_io_hdr_t			Hdr;
    time_t			Start;
    time_t			Finish;

    SImsg(SIM_DBG, "ScsiCmd:\t%s Using SG v3 interface",
	  PRTS(ScsiCmd->DevFile));

    if (!ScsiCmd || !ScsiCmd->Cdb || !ScsiCmd->CdbLen || 
	ScsiCmd->DevFD < 0 || !ScsiCmd->DevFile) {
	SImsg(SIM_DBG, "ScsiCmd:\tBad parameters.");
	return -1;
    }

    /* Initialize */
    (void) memset(Buff, 0, sizeof(Buff));
    (void) memset(&Hdr, 0, sizeof(Hdr));
    Hdr.interface_id = 'S';			/* Instruct driver->interface*/
    Hdr.cmdp = (unsigned char *) ScsiCmd->Cdb;	/* Command Data Block to run */
    Hdr.cmd_len = ScsiCmd->CdbLen;		/* Length of CDB */
    Hdr.dxfer_direction = SG_DXFER_FROM_DEV;	/* Read from device */
    Hdr.dxferp = (void *) Buff;			/* Returned data buffer */
    Hdr.dxfer_len = sizeof(Buff);		/* Max Resp Len */
    Hdr.sbp = (unsigned char *) SenseBuff;	/* Sense buffer */
    Hdr.mx_sb_len = sizeof(SenseBuff);		/* Length of Sense buffer */
    Hdr.timeout = MySCSI_CMD_TIMEOUT * 1000;	/* timeout == milliseconds */

    time(&Start);

    if (ioctl(ScsiCmd->DevFD, SG_IO, &Hdr) < 0) {
	SImsg(SIM_DBG, 
	      "ScsiCmd:\t%s %s ioctl SG_IO failed: %s",
	      ScsiCmd->DevFile,
	      ScsiGetCommandName(Query->Cmd, Query->PageCode), SYSERR);
	return -1;
    }

    time(&Finish);
    SImsg(SIM_DBG, "ScsiCmd:\t%s ioctl(SG_IO) time=%d secs",
	  PRTS(ScsiCmd->DevFile), Finish-Start);

    Status = sg_err_category3(&Hdr);
    switch (Status) {
    case SG_ERR_CAT_CLEAN:
    case SG_ERR_CAT_RECOVERED:
	/* Success */
	ScsiCmd->Data = (void *) Buff;
	Status = 0;
	break;
    default:
	/* Error */
	SImsg(SIM_DBG, 
	      "ScsiCmd:\t%s %s ioctl SG_IO failed: %s",
	      ScsiCmd->DevFile,
	      ScsiGetCommandName(Query->Cmd, Query->PageCode), SYSERR);
	Status = -1;
    }	

    return Status;
#else
    return -2;	/* Unsupported */
#endif	/* SG_IO */
}

/*
 * SG V2 or earlier SCSI interface.
 * Issue a SCSI command and return the results.  This function uses
 * the SCSI Generic (sg) interface.  See 
 * 	http://www.linuxdoc.org/HOWTO/SCSI-Programming-HOWTO.html
 */
extern int ScsiCmdSGv2(ScsiCmd, Query)
     ScsiCmd_t		       *ScsiCmd;
     ScsiQuery_t	       *Query;
{
#if defined(HAVE_SCSI_SUPPORT)		/* SG v2 API */
#define SG_OFFSET		sizeof(struct sg_header)
    static char			Buff[SG_OFFSET + SCSI_BUF_LEN];
    int				Status;
    int				Timeout;
    struct sg_header		ScsiHeader;

    SImsg(SIM_DBG, "ScsiCmd:\t%s Using SG v2 interface",
	  PRTS(ScsiCmd->DevFile));

    if (!ScsiCmd || !ScsiCmd->Cdb || !ScsiCmd->CdbLen || 
	ScsiCmd->DevFD < 0 || !ScsiCmd->DevFile) {
	SImsg(SIM_DBG, "ScsiCmd: Bad parameters.");
	return -1;
    }

    /*
     * Set the timeout
     */
    Timeout = MySCSI_CMD_TIMEOUT * 1000;	/* timeout == milliseconds */
    if (ioctl(ScsiCmd->DevFD, SG_SET_TIMEOUT, &Timeout) < 0) {
	SImsg(SIM_DBG, "ScsiCmd:\t%s Set SG Timeout to %d msecs failed: %s",
	      ScsiCmd->DevFile, Timeout, SYSERR);
    }

    (void) memset(Buff, 0, sizeof(Buff));

    /*
     * SCSI Header for sg
     */
    (void) memset(&ScsiHeader, 0, sizeof(ScsiHeader));
    ScsiHeader.reply_len = sizeof(Buff);
    ScsiHeader.twelve_byte = ScsiCmd->CdbLen == 12;
    (void) memcpy(Buff, &ScsiHeader, sizeof(ScsiHeader));

    /*
     * Add Command Data Block
     */
    (void) memcpy(Buff + SG_OFFSET, ScsiCmd->Cdb, ScsiCmd->CdbLen);

    /*
     * Issue SCSI Command
     */
    Status = write(ScsiCmd->DevFD, Buff, SG_OFFSET + ScsiCmd->CdbLen);
    if (Status < 0 || Status != SG_OFFSET + ScsiCmd->CdbLen || 
	ScsiHeader.result) {
	SImsg(SIM_DBG, 
	      "ScsiCmd:\t%s %s command issue failed: %s",
	      ScsiCmd->DevFile,
	      ScsiGetCommandName(Query->Cmd, Query->PageCode), SYSERR);
	return -1;
    }

    /*
     * Read results of command
     */
    Status = Read(ScsiCmd->DevFD, Buff, sizeof(Buff));
    if (Status != sizeof(Buff)) {
	SImsg(SIM_DBG, 
	      "ScsiCmd:\t%s %s command read result failed: %s",
	      ScsiCmd->DevFile,
	      ScsiGetCommandName(Query->Cmd, Query->PageCode), SYSERR);
	return -1;
    }
	
    ScsiCmd->Data = (void *) Buff + SG_OFFSET;

    return 0;
#else
    return -2;	/* Unsupported */
#endif	/* HAVE_SCSI_SUPPORT */
}

/*
 * Frontend to ScsiCmdSG*()
 */
extern int ScsiCmd(ScsiCmd, Query)
     ScsiCmd_t		       *ScsiCmd;
     ScsiQuery_t	       *Query;
{
    static int			sgv;
    int				Status = 0;

#if defined(SG_GET_VERSION_NUM)
    /*
     * Use the SG v3 interface if its available.
     */
    if (!sgv)
	Status = ioctl(ScsiCmd->DevFD, SG_GET_VERSION_NUM, &sgv);
    if (Status < 0 || sgv < 30000) {
	SImsg(SIM_DBG, "%s: SG v3 interface is not available.", 
	      PRTS(ScsiCmd->DevFile));
    } else {
	Status = ScsiCmdSGv3(ScsiCmd, Query);
	if (Status >= -1) /* Fall through if Status is -2 (Unsupported) */
	    return Status;
    }
#endif

    return ScsiCmdSGv2(ScsiCmd, Query);
}

/*
 * Convert a name from SCSI_IOCTL_GET_PCI to our standard PCI name
 * format.
 * PciName is of format "BB:UU.N"
 */
static char *ScsiPciName(PciName)
     char		       *PciName;
{
    static char			Buff[32];
    char		       *sp;
    char		       *cp;
    int				Bus = -1;
    int				Unit = -1;
    int				DevID = -1;

    if (!PciName)
	return (char *) NULL;

    /* Don't harm PciName */
    snprintf(Buff, sizeof(Buff), "%s", PciName);

    sp = cp = Buff;
    /*
     * Get Bus
     */
    cp = strchr(sp, ':');
    if (!cp)
	cp = strchr(sp, '/');	/* Just in case */
    if (!cp) {
	SImsg(SIM_DBG, "ScsiPciName: Unknown format <%s>", PciName);
	return (char *) NULL;
    }
    *cp = CNULL;
    Bus = atoi(sp);
    sp = cp + 1;

    /*
     * Get Unit
     */
    cp = strchr(sp, '.');
    if (!cp)
	cp = strchr(sp, '/');	/* Just in case */
    if (!cp) {
	SImsg(SIM_DBG, "ScsiPciName: Unknown format <%s>", PciName);
	return (char *) NULL;
    }
    *cp = CNULL;
    Unit = atoi(sp);
    sp = cp + 1;

    /*
     * Device ID
     */
    DevID = atoi(sp);

    if (Bus < 0 || Unit < 0 || DevID < 0) {
	SImsg(SIM_DBG, "ScsiPciName: Could not parse <%s>", PciName);
	return (char *) NULL;
    }

    snprintf(Buff, sizeof(Buff), "pci%d/%d/%d", Bus, Unit, DevID);

    SImsg(SIM_DBG, "ScsiPciName: Converted <%s> -> <%s>", PciName, Buff);

    return Buff;
}

/*
 * Return the name of a SCSI disk device.
 * Normally this is sdX where X is a-z.
 */
static char *ScsiDiskName(Unit)
     int			Unit;		/* Unit per /dev/sgN */
{
    static char			Name[16];

    if (Unit <= 26)
	(void) snprintf(Name, sizeof(Name), "sd%c", Unit + 'a');
    else {
	/*
	 * For Units > 26 the names look like:
	 *	sdaa - sdaz
	 *	sdba - sdbz
	 *	sdca - sdcz
	 *	...
	 */
	(void) snprintf(Name, sizeof(Name), "sd%c%c",
			((Unit / 26) - 1) + 'a',
			Unit + 'a');
    }

    return Name;
}

/*
 * Query a SCSI device with base (driver)
 */
static DevList_t *ScsiProbe(File, Driver, Unit, DevType, DevInfo)
     char		       *File;
     char		       *Driver;
     int			Unit;
     int			DevType;
     DevInfo_t		       *DevInfo;
{
    DevInfo_t		     *(*ProbeFunc)() = NULL;
    DevList_t		       *DevList;
    static ProbeData_t		ProbeData;
    static char			PciName[32];
    static char			DevName[64];
    static char			Path[MAXPATHLEN];
    static int			CDUnits;	/* # CDs found */
    static int			TapeUnits;	/* # Tapes found */
    int				fd;
    char		       *cp;
    struct Scsi_Id {
	long	l1; /* target | lun << 8 | channel << 16 | low_ino << 24 */
	long	l2; /* Unique id */
    } 				Scsi_Id;

    (void) snprintf(DevName, sizeof(DevName), "%s", BaseName(File));

    SImsg(SIM_DBG, "SCSI Probe:\t%s Probing <%s>", PRTS(File), DevName);

    /*
     * Only use O_RDWR as other flags affect sg driver
     */
    fd = open(File, O_RDWR);
    if (fd < 0) {
	SImsg(SIM_DBG, "%s: ScsiProbe: open failed: %s",
	      File, SYSERR);
	if (fd < 0 && errno != EBUSY)
	    return((DevList_t *) NULL);
    }

    SImsg(SIM_DBG, "SCSI Probe:\t%s Device valid/exists", PRTS(File));

    /*
     * Create the DevInfo
     */
    if (!DevInfo)
	DevInfo = NewDevInfo(NULL);
    if (!DevInfo->Name)
	DevInfo->Name = strdup(DevName);
    if (!DevInfo->Driver)
	DevInfo->Driver = strdup(Driver);
    if (DevInfo->ClassType <= 0)
	DevInfo->ClassType = CT_SCSI;
    if (DevInfo->Type <= 0)
	DevInfo->Type = DevType;
    DevAddFile(DevInfo, strdup(File));

    /*
     * Create DevList and return it.
     */
    DevList = (DevList_t *) xcalloc(1, sizeof(DevList_t));
    DevList->Name = DevInfo->Name;
    DevList->File = DevInfo->Files[0];
    DevList->DevInfo = DevInfo;
    DevList->ClassType = CT_SCSI;

#if defined(SCSI_IOCTL_GET_PCI)
    /*
     * Get the name of the PCI controller we're attached to.
     * Format returned looks like "00:09.0"
     */
    if (ioctl(fd, SCSI_IOCTL_GET_PCI, PciName) < 0) {
	SImsg(SIM_DBG, "SCSI Probe:\t%s ioctl(SCSI_IOCTL_GET_PCI) failed: %s",
	      PRTS(File), SYSERR);
    } else {
	if (cp = ScsiPciName(PciName))
	    DevInfo->MasterName = strdup(cp);
	SImsg(SIM_DBG, "SCSI Probe:\t%s MasterName=<%s>",
	      PRTS(File), PRTS(DevInfo->MasterName));
    }
#endif	/* SCSI_IOCTL_GET_PCI */

    /*
     * Identify SCSI specific ID of this device now
     */
    if (ioctl(fd, SCSI_IOCTL_GET_BUS_NUMBER, &DevList->Bus) < 0) {
	SImsg(SIM_DBG, "%s: ioctl to GET_BUS_NUMER failed: %s",
	      File, SYSERR);
    }
    if (ioctl(fd, SCSI_IOCTL_GET_IDLUN, &Scsi_Id) < 0) {
	SImsg(SIM_DBG, "%s: ioctl to GET_IDLUN failed: %s",
	      File, SYSERR);
    }
    DevInfo->Unit = DevList->Unit = Scsi_Id.l1 & 0xFF;
    DevList->Lun = (Scsi_Id.l1 >> 8L) & 0xFF;
    SImsg(SIM_DBG, "SCSI Probe:\t%s SCSI Bus=%d Unit=%d LUN=%d",
	  File, DevList->Bus, DevList->Unit, DevList->Lun);

    /*
     * Do full SCSI query of this device
     */
    SImsg(SIM_DBG, "SCSI Probe:\t%s SCSI Query", PRTS(File));
    (void) ScsiQuery(DevInfo, File, fd, TRUE);

    /*
     * Now that Type is known, set the canonical device name properly.
     */
    DevName[0] = CNULL;
    switch (DevInfo->Type) {
    case DT_DISKDRIVE:
	/* Disks use the Unit number (X) from /dev/sgX */
	(void) snprintf(DevName, sizeof(DevName), "%s", ScsiDiskName(Unit));
	ProbeFunc = ProbeDiskDrive;
	break;
    case DT_TAPEDRIVE:
	(void) snprintf(DevName, sizeof(DevName), "st%d", TapeUnits++);
	break;
    case DT_CD:
	(void) snprintf(DevName, sizeof(DevName), "scd%d", CDUnits++);
	break;
    default:
	SImsg(SIM_DBG, "SCSI Probe:\t%s Type=%d is unknown/undefined",
	      PRTS(File), DevInfo->Type);
    }
    SImsg(SIM_DBG, "SCSI Probe:\t%s DevName now <%s>",
	  PRTS(File), PRTS(DevName));
    if (DevName[0]) {
	DevInfo->AltName = DevInfo->Name;
	DevInfo->Name = strdup(DevName);
	(void) snprintf(Path, sizeof(Path), "%s%s", _PATH_DEV, DevName);
	File = Path;
	SImsg(SIM_DBG, "SCSI Probe:\t%s File name changed", PRTS(File));
	DevAddFile(DevInfo, strdup(Path));
    }
    if (ProbeFunc) {
        (void) memset(&ProbeData, CNULL, sizeof(ProbeData));
	ProbeData.DevName = DevInfo->Name;
	ProbeData.DevFile = File;
	ProbeData.FileDesc = fd;
	ProbeData.UseDevInfo = DevInfo;
	SImsg(SIM_DBG, "SCSI Probe:\t%s Calling ProbeFunc", PRTS(File));
        (void) (*ProbeFunc)(&ProbeData);
    }

    (void) close(fd);

    SImsg(SIM_DBG, "SCSI Probe:\t%s Done; MasterName=<%s>",
	  PRTS(File), PRTS(DevInfo->MasterName));

    return(DevList);
}

/*
 * Detect SCSI devices by checking every /dev/sg? device.
 * This method causes unnecessary probes which can slow us down.
 * However, this is much more desirable than using /proc/scsi/scsi
 * since this method insures we get the proper mapping of OS Device
 * Name -> /dev/sgX.  This is especially true on systems with more than
 * 1 SCSI controller.
 */
extern DevList_t *ScsiDetect(DevInfo_t *CtlrDevInfo)
{
    static int			u;
    static DevList_t	       *ScsiDevices;
    static DevList_t	       *Last;
    register DevList_t	       *New = NULL;
    static char			File[MAXPATHLEN];
    int				MissCount = 0;

    SImsg(SIM_DBG, "ScsiDetect: Checking for SCSI devices ...");

    if (ScsiDevices)
	return(ScsiDevices);

    if (DebugOn) {
	/* Debugging aid */
	SImsg(SIM_DBG, "START of %s", PROC_FILE_SCSI);
	ShowFileDebug(PROC_FILE_SCSI);
	SImsg(SIM_DBG, "END of %s", PROC_FILE_SCSI);
    }

    /*
     * Look for SCSI devices via generic SG driver
     * MissCount is incremented each time we miss an sg dev file
     * because the file doesn't exist.  When we miss too many
     * we break out of the loop.
     */
    for (u = 0; MissCount < 6; ++u) {
	SImsg(SIM_DBG, "ScsiDetect: Trying sg%d (sg%c)", u, u + 'a');

	/*
	 * Most Linux have /dev/sg[a-z] devices which allow us access to SCSI
	 * queries.  Newer Linux seem to symlink /dev/sg[a-z] -> /dev/sg[0-X].
	 * Not all /dev/sg[a-z] links are always created.  Hence, we try both
	 * approaches.  We favor /dev/sg[0-X] as that's more scalable.
	 */
	snprintf(File, sizeof(File), "/dev/sg%d", u);
	if (!FileExists(File, 0)) {
	    snprintf(File, sizeof(File), "/dev/sg%c", u + 'a');
	    if (!FileExists(File, 0)) {
		MissCount++;
		continue;
	    }
	}

	if (New = ScsiProbe(File, "sg", u, 0, NULL)) {
	    SImsg(SIM_DBG, "ScsiDetect: Adding <%s> to ScsiDevices list",
		  (New->DevInfo) ? PRTS(New->DevInfo->Name) : "");
	    if (!ScsiDevices)
		ScsiDevices = Last = New;
	    else {
		Last->Next = New;
		Last = New;
	    }
	}
    }

    return(ScsiDevices);
}

#if notdef
/*
 * XXX We don't use this version of ScsiDetect() because it's nearly
 * impossible to map the SCSI Unit # -> OS Device Name (i.e. "sdb") ->
 * SCSI Generic driver device /dev/sgX (i.e. "sgb"), especially on
 * systems with more than 1 SCSI controller.
 */
/*
 * Detect SCSI devices by reading /proc/scsi/scsi.  Most Linux kernels
 * (including 2.2 - 2.4) support this file.
 *
 * This method is good in that we only have to check the system for the devices
 * which the OS has already discovered.  It's bad in that this interface
 * was never intended to be easily parsed by software.
 */
extern DevList_t *ScsiDetectProc()
{
    register int		u;
    static DevList_t	       *ScsiDevices;
    static DevList_t	       *Last;
    register DevList_t	       *New = NULL;
    DevInfo_t		       *DevInfo;
    DevType_t		       *dt;
    FILE		       *fp;
    static char			Line1[512];
    static char			Line2[512];
    static char			Line3[512];
    char		      **Argv;
    int				Argc;
    char		       *Type;
    char		       *cp;

    if (ScsiDevices)
	return(ScsiDevices);

    SImsg(SIM_DBG, "ScsiDetect: Reading <%s>", PROC_FILE_SCSI);

    fp = fopen(PROC_FILE_SCSI, "r");
    if (!fp) {
	SImsg(SIM_DBG, "%s: Open for reading failed: %s",
	      PROC_FILE_SCSI, SYSERR);
	return (DevList_t *) NULL;
    }

    /* 
     * Read and disgard first line which should be:
     * 	Attached devices:
     */
    (void) fgets(Line1, sizeof(Line1), fp);
    if (cp = strchr(Line1, '\n'))
	*cp = CNULL;
    SImsg(SIM_DBG, "ScsiDetect: First line <%s>", Line1);

    /*
     * There are 3 sequential lines per device so we read 1 device
     * set at a go.
     */
    while (fgets(Line1, sizeof(Line1), fp) &&
	   fgets(Line2, sizeof(Line2), fp) &&
	   fgets(Line3, sizeof(Line3), fp)) {
	New = NULL;
	DevInfo = NewDevInfo(NULL);

	if (cp = strchr(Line1, '\n'))
	    *cp = CNULL;
	if (cp = strchr(Line2, '\n'))
	    *cp = CNULL;
	if (cp = strchr(Line3, '\n'))
	    *cp = CNULL;
	SImsg(SIM_DBG, "ScsiDetect: Line1=<%s>", Line1);
	SImsg(SIM_DBG, "ScsiDetect: Line2=<%s>", Line2);
	SImsg(SIM_DBG, "ScsiDetect: Line3=<%s>", Line3);

	/*
	 * Parse first line which looks like:
	 *	Host: scsi0 Channel: 00 Id: 04 Lun: 00
	 */
	Argc = StrToArgv(Line1, " ", &Argv, NULL, STRTO_SKIPRP);
	if (Argc < 8) {
	    SImsg(SIM_DBG, "ScsiDetect: Line1=<%s> Arg count wrong (%d)",
		  Line1, Argc);
	    continue;
	}
	if (Argv[5])
	    DevInfo->Unit = atoi(Argv[5]);	/* SCSI HW Unit # */

	/*
	 * Parse the second line which looks like:
	 *  Vendor: IBM      Model: DNES-318350W     Rev: SA30
	 */
	/* Skip leading whitespace */
	for (cp = Line2; isspace(*cp); ++cp);
	Argc = StrToArgv(cp, " ", &Argv, NULL, STRTO_SKIPRP);
	if (Argc < 6) {
	    SImsg(SIM_DBG, "ScsiDetect: Line2=<%s> Arg count wrong (%d)",
		  cp, Argc);
	    continue;
	}
	if (Argv[1])
	    DevInfo->Vendor = strdup(Argv[1]);
	if (Argv[3])
	    DevInfo->Model = strdup(Argv[3]);
	if (Argv[5])
	    DevInfo->Revision = strdup(Argv[5]);

	/*
	 * Parse the third line which looks like:
	 *  Type:   Direct-Access                    ANSI SCSI revision: 03
	 */
	/* Skip leading whitespace */
	for (cp = Line3; isspace(*cp); ++cp);
	Argc = StrToArgv(cp, " ", &Argv, NULL, STRTO_SKIPRP);
	if (Argc < 6) {
	    SImsg(SIM_DBG, "ScsiDetect: Line3=<%s> Arg count wrong (%d)",
		  cp, Argc);
	    continue;
	}
	Type = Argv[1];

	/*
	 * Determine the device type
	 */	
	if (EQ(Type, "Direct-Access"))
	    DevInfo->Type = DT_DISKDRIVE;
	else if (EQ(Type, "Sequential-Access"))
	    DevInfo->Type = DT_TAPEDRIVE;
	else if (EQ(Type, "CD-ROM"))
	    DevInfo->Type = DT_CDROM;
	else if (EQ(Type, "Printer"))
	    DevInfo->Type = DT_PRINTER;
	else if (EQ(Type, "Scanner"))
	    DevInfo->Type = DT_SCANNER;
	else if (dt = TypeGetByName(Type))	/* Look it up in our table */
	    DevInfo->Type = dt->Type;
	else
	    SImsg(SIM_DBG, "%s: Unit %d (%s/%s) is an unknown type = <%s>",
		  PROC_FILE_SCSI, DevInfo->Unit,
		  PRTS(DevInfo->Vendor), PRTS(DevInfo->Model), PRTS(Type));

	SImsg(SIM_DBG, 
	      "ScsiDetect: FOUND Unit=%d Vendor=<%s> Model=<%s> Type=<%s>",
	      DevInfo->Unit, PRTS(DevInfo->Vendor), PRTS(DevInfo->Model),
	      Type);

	/* Probe device now */
	if (DevInfo->Type)
	    New = ScsiProbe("sg", DevInfo->Unit, DevInfo->Type, DevInfo);

	/* Add device to list */
	if (New) {
	    if (!ScsiDevices)
		ScsiDevices = Last = New;
	    else {
		Last->Next = New;
		Last = New;
	    }
	}
    }

    return(ScsiDevices);
}
#endif	/* notdef */
