/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.11 $";
#endif

/*
 * MacOS X disk functions
 */

#include "defs.h"

/*
 * Take the best parts of Target and add them to Master, then
 * destroy Target.
 */
static int DiskAssimilate(DevInfo_t *Master, DevInfo_t *Target)
{
    DevDesc_t *			lp;
    DevDesc_t *			Last = NULL;
    DevDesc_t *			Next = NULL;
    DiskDrive_t	*	        Disk = NULL;
    DevInfo_t *			dp;
    char *			cp;

    if (!Master || !Target)
	return -1;

    SImsg(SIM_DBG, "DiskAssimulate: Assimulating <%s> into <%s>",
	  PRTS(Master->Name), PRTS(Target->Name));

    Disk = DiskDriveGet(Master, DISKDRIVE_HW);
    if (!Disk)
	Disk = NewDiskDrive(NULL);

    /* Target->Name is usually better */
    Master->AltName = Master->Name;
    Master->Name = Target->Name;    /* IOxxxBlockStorageDevice0 -> diskX */

    for (lp = Target->DescList; lp; lp = Next) {
	Next = lp->Next;	/* Set and Save */
	/*
	 * Grab special info
	 */
	if (EQ(lp->Label, "BSD Unit"))
	    Master->Unit = atoi(lp->Desc);
	else if (EQ(lp->Label, "Preferred Block Size"))
	    Disk->SecSize = atoi(lp->Desc);
	else if (EQ(lp->Label, "Size")) {
	    Large_t		s;

	    if (s = atoL(lp->Desc))
		if (cp = GetSizeStr(s, BYTES, 0))
		    Master->Capacity = strdup(cp);
	} else if (strlen(lp->Desc)) {
	    /*
	     * Add to Master as generic DevDesc info
	     */
	    lp->Next = NULL;
	    if (!Last)
		/* Position */
		for (Last = Master->DescList; Last && Last->Next;
		     Last = Last->Next);
	    Last->Next = lp;
	    Last = lp;
	}
    }

    DiskDriveAdd(Master, DISKDRIVE_HW, Disk);	/* Save Disk data */

    /* XXX Abandon/Orphan Target in never, never land */

    /* Take Target's slaves and reset slaves parent info */
    Master->Slaves = Target->Slaves;
    for (dp = Master->Slaves; dp; dp = dp->Next) {
	dp->Master = Master;
	dp->MasterName = Master->Name;
    }

    return 0;
}

/*
 * Probe a disk drive
 * This function must be called after the entire device tree is in
 * place so that we can check for children (Slaves) devices.
 */
extern int DiskProbe(DevInfo_t *DevInfo)
{
    char *			cp;

    if (!DevInfo)
	return -1;

    SImsg(SIM_DBG, "DiskProbe: Device=<%s>",
	  PRTS(DevInfo->Name));

    /*
     * If the slave is of type "IOMedia" then it contains more info
     * about this disk and we need to assimulate it.
     */
    if (DevInfo->Slaves) {
	if (cp = DescFind(DevInfo->Slaves->DescList, "I/O Kit Class")) {
	    SImsg(SIM_DBG, "DiskProbe:\t1st level KitClass=<%s>", cp);
	    /* Typical ATA disk */
	    if (EQ(cp, "IOMedia"))
		DiskAssimilate(DevInfo, DevInfo->Slaves);
	    else if (EQ(cp, "IOBlockStorageServices") &&
		     DevInfo->Slaves->Slaves &&
		     (cp = DescFind(DevInfo->Slaves->Slaves->DescList,
				       "I/O Kit Class"))) {
		/*
		 * Reach down another level for the "Media" device.
		 * This appears typical of SCSI disks.
		 */
		SImsg(SIM_DBG, "DiskProbe:\t2nd level KitClass=<%s>", cp);
		if (EQ(cp, "IOMedia"))
		    DiskAssimilate(DevInfo, DevInfo->Slaves->Slaves);
	    }
	}
    }
		     
    DiskLabel(DevInfo);

    return 0;
}
