/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.5 $";
#endif

/*
 * MacOS X disk label functions
 */

#include "defs.h"
#include "dl_partition_map.h"
#include "part.h"

/*
 * Set partition info using PartMap
 */
static int DiskLabelPart(DevInfo_t *		DevInfo,
			 partition_map *	PartMap)
{
    DPME *			p = NULL;
    PartInfo_t *		Part;
    int				PartNum;
    static char			PartName[64];
    static char			Buff[128];
    DiskDrive_t *		Disk;
    char *			cp;

    if (!DevInfo || !PartMap)
	return -1;

    SImsg(SIM_DBG, "DiskLabelPart:\tStart Dev=<%s>", PRTS(DevInfo->Name));

    p = PartMap->data;
    PartNum = PartMap->disk_address;
    Disk = DiskDriveGet(DevInfo, DISKDRIVE_HW);
    if (!Disk) {
	Disk = NewDiskDrive(NULL);
	DiskDriveAdd(DevInfo, DISKDRIVE_HW, Disk);
    }

    snprintf(PartName, sizeof(PartName), "%ss%d", DevInfo->Name, PartNum);

    SImsg(SIM_DBG,
	  "DiskLabelPart:\tPartName=<%s> Disk=<%s> Part=<%s> Type=<%s> PartNum=%d",
	  PRTS(PartName),
	  PRTS(DevInfo->Name), PRTS(p->dpme_name), PRTS(p->dpme_type),
	  PartNum);
    SImsg(SIM_DBG,
	  "DiskLabelPart:\tStart=%lu #Blocks=%lu lStart=%lu lblocks=%lu",
	  p->dpme_pblock_start, p->dpme_pblocks,
	  p->dpme_lblock_start, p->dpme_lblocks);

    Part = PartInfoCreate(NULL);
    Part->Num = PartNum;
    Part->BaseName = strdup(PartName);
    snprintf(Buff, sizeof(Buff), "s%d", PartNum);
    Part->Name = strdup(Buff);
    snprintf(Buff, sizeof(Buff), "/dev/%s", PartName);
    Part->DevPath = strdup(Buff);
    snprintf(Buff, sizeof(Buff), "/dev/r%s", PartName);
    Part->DevPathRaw = strdup(Buff);
    if (p->dpme_name && isprint(p->dpme_name[0]))
	Part->Title = strdup(p->dpme_name);

    Part->StartSect = (Large_t) p->dpme_pblock_start;
    Part->NumSect = (Large_t) p->dpme_pblocks;
    Part->EndSect = (Large_t) p->dpme_pblock_start + (Large_t) p->dpme_pblocks;
    Part->Size = ((Large_t) p->dpme_pblocks / (Large_t) Disk->SecSize) /
	(Large_t) MBYTES;

    /* Get and set filesys info */
    PartSetFilesys(DevInfo, Part->DevPath, Part);

    /* Fill in values that PartSetFilesys() didn't set */
    if (!Part->MntName && isprint(p->dpme_name[0]))
	Part->MntName = strdup(p->dpme_name);
    if (!Part->Type && isprint(p->dpme_type[0])) {
	cp = p->dpme_type;
	if (EQN(cp, "Apple_", 6))
	    cp += 6;
	Part->Type = strdup(cp);
    }
    if (!Part->TypeDesc && isprint(p->dpme_type[0]))
	Part->TypeDesc = strdup(p->dpme_name);

    /*
     * Add this Part to the Disk
     */
    SImsg(SIM_DBG, "DiskLabelPart:\tPartName=<%s> Adding partition",
	  PRTS(PartName));
    DiskPartAdd(&(Disk->DiskPart), Part);
    SImsg(SIM_DBG, "DiskLabelPart:\tPartName=<%s> DONE", PRTS(PartName));

    return 0;
}

/*
 * Read disk label and set info
 */
extern int DiskLabel(DevInfo_t *DevInfo)
{
    static char			Path[MAXPATHLEN];
    DiskDrive_t *		Disk;
    int				Status = 0;
    partition_map_header *	MapHdr;
    partition_map *		mp;
    Block0 *			b0;
    int				junk;

    if (!DevInfo)
	return -1;

    SImsg(SIM_DBG, "DiskLabel: <%s> Read disk label",
	  PRTS(DevInfo->Name));

    snprintf(Path, sizeof(Path), "/dev/r%s", DevInfo->Name);
    if (!FileExists(Path, 0))
	return -1;

    SImsg(SIM_DBG, "DiskLabel:\tReading label from <%s>", Path);

    /*
     * Retrieve the map of partitions for this disk.
     */
    MapHdr = open_partition_map(Path, &junk, 0);
    if (!MapHdr) {
	SImsg(SIM_DBG, "DiskLabel:\tRead partition map failed");
	return -1;
    }

    /* Setup Disk */
    Disk = DiskDriveGet(DevInfo, DISKDRIVE_HW);
    if (!Disk) {
	Disk = NewDiskDrive(NULL);
	DiskDriveAdd(DevInfo, DISKDRIVE_HW, Disk);
    }

    b0 = MapHdr->misc;
    if (b0->sbSig != BLOCK0_SIGNATURE) {
	SImsg(SIM_DBG, "DiskLabel:\tInvalid block0 signature");
    } else {
	SImsg(SIM_DBG, "DiskLabel:\tBlockSize=%u #Blocks=%lu",
	      b0->sbBlkSize, b0->sbBlkCount);
	Disk->SecSize = b0->sbBlkSize;
	Disk->Size = (float) ((b0->sbBlkCount / b0->sbBlkSize) / MBYTES);
    }

    /* Set Partition info for each map found */
    for (mp = MapHdr->disk_order; mp; mp = mp->next_on_disk)
	DiskLabelPart(DevInfo, mp);

    close_partition_map(MapHdr);

    return Status;
}
