/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.7 $";
#endif

#include "defs.h"
#include "iok.h"

#include <IOKit/IOKitLib.h>

/*
 * MacOS X IO Kit functions
 */

extern char *IOKerr(IOReturn Val)
{
    char *			Msg = NULL;
    static char			Buf[128];

    /*
     * Values are from:
     * System/Library/Frameworks/IOKit.framework/Versions/A/Headers/IOReturn.h
     */
    switch (Val) {
    case kIOReturnError:
	Msg = "General error";					break;
    case kIOReturnNoDevice:
	Msg = "No such device";					break;
    case kIOReturnBusy:
	Msg = "Device busy";					break;
    case kIOReturnUnsupported:
	Msg = "Unsupported function";				break;
    case kIOReturnExclusiveAccess:
	Msg = "Exclusive access / device already open";		break;
    default:
	snprintf(Buf, sizeof(Buf), "0x%x", Val);
	Msg = Buf;
    }

    return Msg;
}

/*
 * Clean a string
 * Return value is NOT alloc'ed
 */
static char *IOKclean(char *String)
{
    char		       *cp;
    char		       *bp;
    static char		       *Buf;

    if (Buf)
	free(Buf);

    bp = Buf = xcalloc(1, strlen(String)+1);
    for (cp = String; cp && *cp; ++cp) {
	if (isprint(*cp)) {
	    *bp = *cp;
	    ++bp;
	} else
	    SImsg(SIM_DBG, "IOKclean: Unprintable char '%c' found in: <%s>",
		  *cp, String);
    }

    return Buf;
}

/*
 * Decode and return string for Object
 * Return value is not alloc'ed.
 */
extern char *IOKstring(CFStringRef Object)
{
    const char		       *Str;
    static char		       *Buf;
    CFIndex			BufSize;

    if (Str = CFStringGetCStringPtr(Object, kCFStringEncodingMacRoman))
	return (char *) Str;
    else {
	BufSize = CFStringGetLength(Object) + 1;
	if (!BufSize)
	    return (char *) NULL;

	if (Buf)
	    free(Buf);

	Buf = xmalloc(BufSize);
	if (CFStringGetCString(Object, Buf, BufSize,
			       kCFStringEncodingMacRoman))
	    return IOKclean(Buf);
    }

    return (char *) NULL;
}

/*
 * Convert String Object into string and/or number
 */
static int CFStringGet(CFTypeRef Object, IOKparam_t *Params)
{
    char		       *cp;

    cp = IOKstring(Object);
    if (!cp)
	return -1;

#if 0
    SImsg(SIM_DBG, "\fCFStringGet: Got <%s>", cp);
#endif

    Params->ValStr = cp;
    Params->ValNum = atoL(cp);

    return 0;
}

/*
 * Convert a Number Object into string and/or number
 */
static int CFNumberGet(CFTypeRef Object, IOKparam_t *Params)
{
    long long			Num;

    if (!CFNumberGetValue(Object, kCFNumberLongLongType, &Num))
	return -1;

#if 0
    SImsg(SIM_DBG, "\tCFNumberGet: Got %lld", Num);
#endif

    Params->ValStr = Ltoa((Large_t)Num);
    Params->ValNum = (Large_t) Num;

    return 0;
}

/*
 * Convert a Boolean Object into string and/or number
 */
static int CFBoolGet(CFTypeRef Object, IOKparam_t *Params)
{
    if (CFBooleanGetValue(Object)) {
	Params->ValStr = "TRUE";
	Params->ValNum = 1;
    } else {
	Params->ValStr = "FALSE";
	Params->ValNum = 0;
    }

    return 0;
}

/*
 * Convert a Data Object into string and/or number
 */
static int CFDataGet(CFTypeRef Object, IOKparam_t *Params)
{
    char		       *Buffer = NULL;
    size_t			BufSize = 0;
    size_t			b = 0;
    const UInt8		       *ObjBuf = NULL;
    CFIndex			Length;
    CFIndex			i;
    int				NormCnt = 0;
    int				SymbolCnt = 0;
    int				IsNum = 0;

    Length = CFDataGetLength(Object);
    ObjBuf = CFDataGetBytePtr(Object);

    /* Limit size of what we get */
#define MAX_DATAGET_LEN 1024
    if (Length > MAX_DATAGET_LEN) {
	SImsg(SIM_DBG, "\tCFDataGet: Truncating Length from %d to %d",
	      Length, MAX_DATAGET_LEN);
	Length = MAX_DATAGET_LEN;
    }

    for (i = 0; i < Length; ++i) {
	if (!ObjBuf[i]) {
	    for ( ; i < Length && !ObjBuf[i]; ++i);
	    break;
	} else {
	    /* Scan looking for printable ASCII */
	    for ( ; i < Length; i++) {
		if (isprint(ObjBuf[i]))
		    NormCnt++;
		else if (ObjBuf[i] >= 128 && ObjBuf[i] <= 254)
		    SymbolCnt++;
		else
		    break;
	    }

	    if (i < Length && !ObjBuf[i])
		/* End of string */
		continue;
	    else
		/* End of buffer or end of printable ASCII */
		break;
	}
    }

    if ((NormCnt >> 2) < SymbolCnt)	/* 80% of it numeric? */
	i = 0;	/* Numeric */
    else if (Length == 1)
	i = 0;	/* Numeric */

    if (i == 0)
	IsNum = TRUE;

    if (i >= Length && NormCnt) {
	/* Decode String value */
	int			Insert = FALSE;

	BufSize = Length+3;
	Buffer = (char *) xcalloc(BufSize, sizeof(char *));

	/* Skip leading NULL's */
	for (i = 0; i < Length && !ObjBuf[i]; ++i);

	/* Add each char */
	for ( ; i < Length; ++i) {
	    if (ObjBuf[i] && isprint(ObjBuf[i])) {
		if (Insert) {
		    Buffer[b++] = ',';
		    Buffer[b++] = ' ';
		    Insert = FALSE;
		}
		Buffer[b++] = ObjBuf[i];
	    } else if (!ObjBuf[i]) {
		/* Insert seperate before next value */
		Insert = TRUE;
	    }
	}
    } else {
	/* Numeric value */
	static char		NumBuf[4];

	BufSize = (Length * 2) + 3;
	Buffer = (char *) xcalloc(BufSize, sizeof(char *));

	snprintf(Buffer, sizeof(Buffer), "0x");
	for (i = 0; i < Length; i++) {
	    snprintf(NumBuf, sizeof(NumBuf), "%02x", ObjBuf[i]);
	    StrAppend(Buffer, BufSize, NumBuf);
	}
    }
		
    if (IsNum) {
	Params->ValNum = atoL(Buffer);
#if 0
	SImsg(SIM_DBG, "\tCFDataGet: Got 0x%llX", Params->ValNum);
#endif
    } else {
	Params->ValStr = Buffer;
#if 0
	SImsg(SIM_DBG, "\tCFDataGet: Got <%s>", PRTS(Params->ValStr));
#endif
    }

    return 0;
}

/*
 * Get a member from an array.
 */
static void CFArrayGetMember(CFTypeRef Object, void *ParamOp)
{
    IOKparam_t		       *Params;

    Params = (IOKparam_t *) ParamOp;

    IOKvalue(Object, Params);
}

/*
 * Convert an Array Object into string and/or number
 */
static int CFArrayGet(CFTypeRef Object, IOKparam_t *Params)
{
    CFRange			Range = { 0, CFArrayGetCount(Object) };

#if 0
    SImsg(SIM_DBG, "\tCFArrayGet: called");
#endif
    Params->First = TRUE;
    CFArrayApplyFunction(Object, Range, CFArrayGetMember, Params);

    return 0;
}

/*
 * Get a member from a set
 */
static void CFSetGetMember(CFTypeRef Object, void *ParamOp)
{
    IOKparam_t		       *Params;

    Params = (IOKparam_t *) ParamOp;
    if (Params->First)
	Params->First = FALSE;

    IOKvalue(Object, Params);
}

/*
 * Convert a Set Object into string and/or number
 */
static int CFSetGet(CFTypeRef Object, IOKparam_t *Params)
{
#if 0
    SImsg(SIM_DBG, "\tCFSetGet: called");
#endif
    Params->First = TRUE;
    CFSetApplyFunction(Object, CFSetGetMember, Params);

    return 0;
}

/*
 * See if there is an expansion for Key
 */
static char *ExpandKey(char *Key)
{
    Define_t		       *Def;

    if (!Key)
	return (char *) NULL;

    if (Def = DefGet("ioprop", Key, 0, DO_NOSLOW)) {
	SImsg(SIM_DBG, "ExpandKey: Key=<%s> Found Key=<%s> Val=<%s>",
	      Key, Def->KeyStr, Def->ValStr1);
	return (Def->ValStr1) ? Def->ValStr1 : Key;
    }

    return Key;
}

/*
 * Set individual properties
 */
extern void IOKSetProp(const void *KeyRef, const void *ValueRef, void *ParamOp)
{
    IOKparam_t		       *Params;
    static IOKparam_t		IOKparams;
    char		       *cp;

    if (!ParamOp)
	return;

    Params = (IOKparam_t *) ParamOp;
    Params->ValStr = NULL;
    Params->ValNum = 0;

    cp = ExpandKey(IOKstring(KeyRef));
    if (!cp)
	return;

    /*
     * Ignore certain specific keys
     * - Ignore driver entries which contain the driver binary code
     * - Ignore PM data which always changes
     */
    if (EQN(cp, "driver,", 7) ||
	EQ(cp, "Scripts") ||
	EQ(cp, "ElementCookie") ||	/* Too many and useless? */
	EQ(cp, "ValueLocation") ||	/* Too many and useless? */
	EQ(cp, "Usage") ||		/* Too many and useless? */
	EQ(cp, "HIDPointerAccelerationTable") ||
	EQ(cp, "Power Management private data") ||
	EQ(cp, "Power Management protected data"))
	return;

    /*
     * Skip entries like:
     * 	ACCF0000-0000-0000-0000-000a2789904e
     *  2d9786c6-9ef3-11d4-ad51-000a27052861
     */
    if (REmatch(cp, ".*[0-9].*-.*[0-9].*-.*[0-9].*-.*[0-9].*",
		NULL)) {
	SImsg(SIM_DBG, "\tIOKSetProp: Skip <%s> matches REGEX1", cp);
	return;
    }

    Params->Key = Capitalize(cp);
    IOKvalue(ValueRef, Params);

    SImsg(SIM_DBG, "\tIOKSetProp: <%s>=<%s> (%lld)",
	  PRTS(Params->Key), PRTS(Params->ValStr), Params->ValNum);
}

/*
 * Convert a Dictionary Object into string and/or number
 */
static int CFDictGet(CFTypeRef Object, IOKparam_t *Params)
{
    Params->First = TRUE;
    CFDictionaryApplyFunction(Object, IOKSetProp, Params);

    return 0;
}

/*
 * Decode something which looks like:
 *	pci106b,20, pciclass,060000
 */
static void DecodeCompat(DevInfo_t *DevInfo, char *Compat)
{
    static char	**		Argv;
    int				Argc;
    register int		i;
    static PCIinfo_t		PciInfo;
    char *			Start;
    char *			cp;

    SImsg(SIM_DBG, "DecodeCompat: <%s>", Compat);

    /* If Compat doesn't start with "pci" abort */
    if (!EQN(Compat, "pci", 3))
	return;

    PCIinit(&PciInfo);
    PciInfo.DevInfo = DevInfo;

    Argc = StrToArgv(Compat, " ", &Argv, NULL, 0);
    for (i = 0; i < Argc; ++i) {
	if (EQN(Argv[i], "pciclass,", 8)) {
	    PciInfo.Class = atoi(Argv[i] + 8);
	} else if (EQN(Argv[i], "pci", 3) && isdigit(Argv[i][4])) {
	    if (cp = strchr(Argv[i], ',')) {	/* Search for vendor,dev */
		*cp++ = CNULL;
		Start = cp;
		if (cp = strchr(Start, ','))	/* Remove trailing ',' */
		    *cp = CNULL;
		PciInfo.DeviceID = strtol(Start, NULL, 16);
	    }
	    PciInfo.VendorID = strtol(Argv[i] + 3, NULL, 16);
	}
    }

    PCIsetDeviceInfo(&PciInfo);
}

/*
 * Decode values from Object and store in Params
 */
extern int IOKvalue(CFTypeRef Object, IOKparam_t *Params)
{
    CFTypeID			Type;
    DevInfo_t		       *DevInfo;
    SoftInfo_t		       *SoftInfo;
    char		       *What = NULL;
    char		       *cp;

    DevInfo = Params->DevInfo;
    SoftInfo = Params->SoftInfo;
    Type = CFGetTypeID(Object);

    /*
     * Call appropriate decoder to get data and attach to Params
     */
    if (Type == CFStringGetTypeID())
	CFStringGet(Object, Params);
    else if (Type == CFNumberGetTypeID())
	CFNumberGet(Object, Params);
    else if (Type == CFBooleanGetTypeID())
	CFBoolGet(Object, Params);
    else if (Type == CFArrayGetTypeID())
	CFArrayGet(Object, Params);
    else if (Type == CFDataGetTypeID())
	CFDataGet(Object, Params);
    else if (Type == CFSetGetTypeID())
	CFSetGet(Object, Params);
    else if (Type == CFDictionaryGetTypeID())
	CFDictGet(Object, Params);
    else
	SImsg(SIM_DBG, "IOKvalue: Object 0x%X is unknown object type", Object);

    if (What)
	SImsg(SIM_DBG, "\tObject Type is <%s>", What);

    if (!Params->Key || !*(Params->Key)) {
	SImsg(SIM_DBG, "\tIOKvalue: Failed to decode key.");
	return -1;
    }

    if (DevInfo) {
	/*
	 * Add the data we've just acquired
	 */
	if (EQ(Params->Key, "model") ||
	    EQ(Params->Key, "Product Name") ||
	    EQ(Params->Key, "PHY type") ||
	    EQ(Params->Key, "Product Identification")) {
	    DevInfo->Model = CleanString(Params->ValStr,
					 strlen(Params->ValStr));
	} else if (EQ(Params->Key, "device_type") ||
		   EQ(Params->Key, "Device Type")) {
	    IOKdevType(Params);
	} else if (EQ(Params->Key, "Vendor Name") ||
		   EQ(Params->Key, "Vendor Identification")) {
	    DevInfo->Vendor = CleanString(Params->ValStr,
					  strlen(Params->ValStr));
	} else if (EQ(Params->Key, "BSD Name") ||
		   EQ(Params->Key, "BSD_Name")) {
	    if (DevInfo->Name && !DevInfo->Model)
		/* Move the name to the model field */
		DevInfo->Model = DevInfo->Name;
	    DevInfo->Name = strdup(Params->ValStr);
	} else if (EQ(Params->Key, "Unit Number") ||
		   EQ(Params->Key, "Unit")) {
	    DevInfo->Unit = (int) Params->ValNum;
	} else if (EQ(Params->Key, "vendor-id") || EQ(Params->Key, "vendor id")) {
	    if (Params->ValStr && !Params->ValNum)
		Params->ValNum = atoL(Params->ValStr);
	    Params->VendorID = Params->ValNum;
	} else if (EQ(Params->Key, "device-id") || EQ(Params->Key, "device id")) {
	    if (Params->ValStr && !Params->ValNum)
		Params->ValNum = atoL(Params->ValStr);
	    Params->DeviceID = Params->ValNum;
	} else if (EQ(Params->Key, "SCSI Vendor Name")) {
	    DevInfo->ClassType = CT_SCSI;
	    DevInfo->Vendor = strdup(Params->ValStr);
	} else if (EQ(Params->Key, "SCSI Product Name")) {
	    DevInfo->ClassType = CT_SCSI;
	    DevInfo->Model = strdup(Params->ValStr);
	} else if (EQ(Params->Key, "SCSI Device Type")) {
	    DevInfo->ClassType = CT_SCSI;
	    ScsiSetDevType(DevInfo, (long) Params->ValNum);
	} else if (EQ(Params->Key, "Serial Number")) {
	    DevInfo->Serial = strdup(Params->ValStr);
	} else if (EQ(Params->Key, "Revision Number") ||
		   EQ(Params->Key, "Product Revision Level")) {
	    DevInfo->Revision = strdup(Params->ValStr);
	} else if (EQ(Params->Key, "IOFBMemorySize") ||
		   EQ(Params->Key, "ATY,memsize")) {
	    if (Params->ValNum > 0 &&
		(cp = GetSizeStr(Params->ValNum, BYTES, 0)))
		DevInfo->Capacity = strdup(cp);
	} else if (EQ(Params->Key, "ATY,Card#")) {
	    DevInfo->Part = strdup(Params->ValStr);
	} else if (EQ(Params->Key, "Partition ID")) {
	    DevInfo->Type = DT_DISKPART;
	} else if (EQ(Params->Key, "compatible")) {
	    DecodeCompat(DevInfo, Params->ValStr);
	    DescAdd(&(DevInfo->DescList), ExpandKey(Params->Key),
		    Params->ValStr,0);
	} else {
	    DescAdd(&(DevInfo->DescList), ExpandKey(Params->Key),
		    Params->ValStr, 0);
	}
    }

    if (SoftInfo) {
	/*
	 * Set Version if Version is not set or if it is and doesn't 
	 * contain a '.' then set it now.
	 */
	if (Params->ValStr &&
	    (!SoftInfo->Version || !strchr(SoftInfo->Version, '.')) &&
	    (EQ(Params->Key, "CFBundleShortVersionString") ||
	     EQ(Params->Key, "CFBundleVersion"))) {
	    SoftInfo->Version = strdup(Params->ValStr);
	} else if (!SoftInfo->Desc &&
		   EQ(Params->Key, "CFBundleGetInfoString") &&
		   Params->ValStr) {
	    SoftInfo->Desc = strdup(Params->ValStr);
	} else if (EQ(Params->Key, "CFBundleName") && Params->ValStr &&
		   (!SoftInfo->Desc ||
		    (strlen(Params->ValStr) > strlen(SoftInfo->Desc)))) {
	    SoftInfo->Desc = strdup(Params->ValStr);
	} else if (EQ(Params->Key, "CFBundleExecutablePath") &&
		   Params->ValStr) {
	    SoftInfo->BaseDir = strdup(Params->ValStr);
	} else if (EQ(Params->Key, "CFBundleSignature") &&
		   Params->ValStr) {
	    if (Params->ValStr[0] != '?')
		SoftInfo->ProdStamp = strdup(Params->ValStr);
	} else {
	    /* Skip common Keys */
	    if (!EQ(Params->Key, "CFBundleIdentifier")) {
		/*
		 * Time to add this as a Desc entry
		 */
		cp = ExpandKey(Params->Key);
		if (EQN(cp, "CFBundle", 8))	/* Remove common prefix */
		    cp += 8;
		DescAdd(&(SoftInfo->DescList), cp, Params->ValStr, 0);
	    }
	}
    }

    return 0;
}
