/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.24 $";
#endif

/*
 * MacOS X IO Kit device functions
 */

#include "defs.h"
#include "iok.h"

/*
 * Should we ignore this Class?
 */
static int IgnoreClass(char *Class)
{
    /*
     * Lookup exact match in config file.
     */
    if (DefGet("IgnoreIOClass", Class, 0, 0))
	return TRUE;

    return FALSE;
}

/*
 * Should we ignore this I/O Name
 */
static int IgnoreName(char *Name)
{
    /*
     * Lookup exact match in config file.
     */
    if (DefGet("IgnoreIOName", Name, 0, 0))
	return TRUE;

    return FALSE;
}

/*
 * Set device type
 */
extern void IOKdevType(IOKparam_t *Params)
{
    char		       *Type;
    DevInfo_t		       *DevInfo;

    DevInfo = Params->DevInfo;
    if (!DevInfo)
	return;
    Type = Params->ValStr;
    if (EQ(Type, "cpu")) {
	DevInfo->Type = DT_PROCESSOR;
	DevInfo->ClassType = CT_CPU;
    } else if (EQ(Type, "bootrom")) {
	DevInfo->Type = DT_BIOS;
    } else if (EQ(Type, "nvram")) {
	DevInfo->Type = DT_BIOS;
    } else if (EQ(Type, "i2c")) {
	DevInfo->Type = DT_BRIDGE;
	DevInfo->ClassType = CT_I2C;
    } else if (EQ(Type, "memory-controller")) {
	DevInfo->Type = DT_MEMCTL;
    } else if (EQ(Type, "pci")) {
	DevInfo->Type = DT_BUS;
	DevInfo->ClassType = CT_PCI;
    } else if (EQ(Type, "uni-north-agp")) {
	DevInfo->Type = DT_BUS;
	DevInfo->ClassType = CT_AGP;
    } else if (EQ(Type, "mac-io")) {
	DevInfo->Type = DT_BRIDGE;
    } else if (EQ(Type, "serial")) {
	DevInfo->Type = DT_SERIAL;
    } else if (EQ(Type, "soundbus")) {
	DevInfo->Type = DT_BUS;
    } else if (EQ(Type, "soundchip")) {
	DevInfo->Type = DT_AUDIO;
    } else if (EQ(Type, "display")) {	/* Poor naming from the IOReg */
	DevInfo->Type = DT_FB;		/* It's an FB, not a monitor */
    } else if (EQ(Type, "ata")) {
	DevInfo->Type = DT_CONTROLLER;	/* XXX Guess */
	DevInfo->ClassType = CT_ATA;
    } else if (EQ(Type, "scsi")) {
	DevInfo->Type = DT_CONTROLLER;	/* XXX Guess */
	DevInfo->ClassType = CT_SCSI;
    } else if (EQ(Type, "usb")) {
	DevInfo->Type = DT_SERIAL;
	DevInfo->ClassType = CT_USB;
    } else if (EQ(Type, "ieee1394")) {
	DevInfo->Type = DT_SERIAL;
	DevInfo->ClassType = CT_FIREWIRE;
    } else if (EQ(Type, "escc") || EQ(Type, "escc-legacy")) {
	DevInfo->Type = DT_PARALLEL;
    } else if (EQ(Type, "i2c-modem")) {
	DevInfo->Type = DT_BUS;
	DevInfo->ClassType = CT_I2C;
    } else if (EQ(Type, "DVD")) {
	DevInfo->Type = DT_DVD;
    } else if (EQ(Type, "CD") || EQ(Type, "CDROM")) {
	DevInfo->Type = DT_CD;
    } else if (EQ(Type, "network")) {
	DevInfo->Type = DT_NETIF;
    } else {
	SImsg(SIM_DBG, "\tIOKdevType: Unknown device type <%s>", Type);
	AddDevDesc(DevInfo, Type, "device_type", 0);
    }
}

/*
 * Probe an IOK object and extract the data we want
 */
static DevInfo_t *IOKprobe(io_registry_entry_t Service,
			   DevInfo_t *Master)
{
    static IOKparam_t	        Params;
    io_name_t			Name;
    io_name_t			Location;
    io_name_t			Class;
#if	0	/* This worked on MacOS 10.1 */
    CFDictionaryRef		Props = 0;
#endif
    CFMutableDictionaryRef	Props = 0;
    DevInfo_t		       *DevInfo = NULL;
    DevDefine_t		       *DevDef;
    static PCIinfo_t		PciInfo;

    Location[0] = CNULL;

    if (IORegistryEntryGetNameInPlane(Service, kIOServicePlane, Name)
	!= KERN_SUCCESS) {
	SImsg(SIM_DBG, "IORegistryEntryGetNameInPlane: Failed: %s",
	      SYSERR);
	return (DevInfo_t *) NULL;
    }
#if HAVE_IOREGISTRYENTRYGETLOCATIONINPLANE
    if (IORegistryEntryGetLocationInPlane(Service, kIOServicePlane, Location)
	!= KERN_SUCCESS)
	SImsg(SIM_DBG, "IORegistryEntryGetLocationInPlane: %s: Failed: %s",
	      Name, SYSERR);
#endif
    if (IOObjectGetClass(Service, Class) != KERN_SUCCESS) {
	SImsg(SIM_DBG, "IOKprobe: %s: Cannot get class name: Failed: %s",
	      Name, SYSERR);
	Class[0] = CNULL;
    }
	
    SImsg(SIM_DBG, "IOKprobe: Object <%s> Loc=<%s> Class=<%s>",
	  Name, PRTS(Location), PRTS(Class));
    
    /*
     * Don't probe certain class types or names
     */
    if (IgnoreClass(Class)) {
	SImsg(SIM_DBG, "\tSkipping due to <%s> being in Ignore Class List",
	      Class);
	return (DevInfo_t *) NULL;
    }
    if (IgnoreName(Name)) {
	SImsg(SIM_DBG, "\tSkipping due to <%s> being in Ignore Name List",
	      Name);
	return (DevInfo_t *) NULL;
    }

    /*
     * Pull the properties of the node
     */
    if (IORegistryEntryCreateCFProperties(Service,
					  &Props,
					  kCFAllocatorDefault,
					  kNilOptions) != KERN_SUCCESS) {
	SImsg(SIM_DBG, "IOKprobe: %s: Cannot retrieve properties: %s",
	      Name, SYSERR);
	return (DevInfo_t *) NULL;
    }

    DevInfo = DevInfoNew(NULL);
    DevInfo->Name = strdup((char *) Name);
    if (*(DevInfo->Name) == '.')	/* Skip over leading '.' if any */
	++DevInfo->Name;
    if (DevInfo->Master = Master)
	DevInfo->MasterName = Master->Name;
    if (Class && strlen(Class)) {
	/*
	 * Lookup a device definetion by the Class name or the IOK Name
	 */
	if ((DevDef = DevDefGet(Class, 0, 0)) ||
	    (DevDef = DevDefGet(Name, 0, 0))) {
	    DevInfo->Vendor = DevDef->Vendor;
	    DevInfo->Model = DevDef->Model;
	    DevInfo->ModelDesc = DevDef->Desc;
	    DevInfo->Type = DevDef->Type;
	    DevInfo->ClassType = DevDef->ClassType;
	}

	AddDevDesc(DevInfo, (char *) Class, "I/O Kit Class", 0);
	if (EQ(Class, "IODTNVRAM")) 	/* Special case */
	    DevInfo->Type = DT_BIOS;
    }
    if (Location && strlen(Location))
	AddDevDesc(DevInfo, (char *) Location, "I/O Kit Location", 0);

    memset(&Params, CNULL, sizeof(Params));
    Params.DevInfo = DevInfo;

    CFDictionaryApplyFunction(Props, IOKSetProp, &Params);

    if (EQ(Class, "IOSCSIParallelDevice"))
	/* Perform SCSI probe */
	IOKprobeSCSI(DevInfo, Service);

    if (DevInfo->Unit < 0)
	DevInfo->Unit = UnitAssign(DevInfo->Type, DevInfo->ClassType,
				   (Class) ? Class : DevInfo->Name);

    if (EQ(DevInfo->Name, Name)) {
	/* Build a new DevInfo Name with our new info */
	(void) free(DevInfo->Name);
	DevInfo->Name = MkDevName(Name, DevInfo->Unit, DevInfo->Type, 0);
    }
    
    CFRelease(Props);

    /*
     * Set PCI info if we have any
     */
    if (Params.VendorID && Params.DeviceID) {
	PCIinit(&PciInfo);
	PciInfo.DevInfo = DevInfo;
	PciInfo.VendorID = Params.VendorID;
	PciInfo.DeviceID = Params.DeviceID;
	SImsg(SIM_DBG,
	      "IOKprobe: PCI <%s> Lookup VendorID=0x%04x DeviceID=0x%04x",
	      PRTS(DevInfo->Name), PciInfo.VendorID, PciInfo.DeviceID);
	PCIsetDeviceInfo(&PciInfo);
    }

    return DevInfo;
}

/*
 * Traverse the IO Kit's tree
 */
static int IOKtraverse(io_registry_entry_t Service,
		       DevInfo_t *Master,
		       DevInfo_t **TreePtr,
		       char **Search)
{
    io_iterator_t		Children = 0;
    io_registry_entry_t		Child = 0;
    io_registry_entry_t		ChildNext = 0;
    DevInfo_t		       *DevInfo;

    /* Fetch the children of this object */
    if (IORegistryEntryGetChildIterator(Service, kIOServicePlane,
					&Children) != KERN_SUCCESS) {
	SImsg(SIM_DBG,
	      "IORegistryEntryGetChildIterator: Cannot get list of children: %s",
	      SYSERR);
	return -1;
    }

    /* Queue up our next child */
    ChildNext = IOIteratorNext(Children);

    /* Probe current object/node */
    if (DevInfo = IOKprobe(Service, Master))
	DevAdd(DevInfo, TreePtr, Search);

    /* Traverse each child object/node */
    while (ChildNext) {
	Child = ChildNext;
	ChildNext = IOIteratorNext(Children);

	IOKtraverse(Child, DevInfo, TreePtr, Search);

	IOObjectRelease(Child);
	Child = 0;
    }

    IOObjectRelease(Children);
    Children = 0;

    return 0;
}

/*
 * Build device tree and place in TreePtr.
 */
extern Opaque_t BuildDevices(DevInfo_t **TreePtr, char **Search)
{
    int				Status = 0;
    io_registry_entry_t 	Service = 0;
    mach_port_t         	iokPort = 0;

    Status = IOMasterPort(bootstrap_port, &iokPort);
    if (Status != KERN_SUCCESS) {
	SImsg(SIM_DBG, "BuildDevices: Cannot obtain I/O Kit's master port");
	return (Opaque_t) NULL;
    }

    Service = IORegistryGetRootEntry(iokPort);
    if (!Service) {
	SImsg(SIM_DBG, "BuildDevices: Cannot obtain I/O Kit's root service");
	return (Opaque_t) NULL;
    }

    IOKtraverse(Service, NULL, TreePtr, Search);

    IOObjectRelease(Service);	/* Free */

    DevPostProcess(TreePtr, Search);

    IOKpost(TreePtr, Search);	/* IOK post processing */

    return (Opaque_t) 0;
}

