/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.4 $";
#endif

/*
 * MacOS X IO Kit post functions
 */

#include "defs.h"

/*
 * Post process an IOK found DevInfo device
 * -RECURSE-
 */
static int IOKpostDevice(DevInfo_t *DevInfo)
{
    char *			cp;

    if (!DevInfo)
	return 0;

    SImsg(SIM_DBG, "IOKpostDevice: Device <%s>", PRTS(DevInfo->Name));

    /*
     * If Model and Vendor is set and Model has a ',' in it, we
     * assume that's somehow indicating the Vendor name and we remove it.
     */
    if (DevInfo->Model && DevInfo->Vendor)
	if (cp = strchr(DevInfo->Model, ',')) {
	    SImsg(SIM_DBG, "IOKpostDevice: Changing <%s> to be <%s>",
		  DevInfo->Model, cp + 1);
	    DevInfo->Model = cp + 1;
	}

    /*
     * The following device types need post processing.
     * Usually they need to wait for the entire tree to be in place
     * as they have a dependency such as they need to know their children
     * or parent info.
     */
    switch (DevInfo->Type) {
    case DT_CPU:		CpuProbe(DevInfo);	break;
    case DT_DISKDRIVE:		DiskProbe(DevInfo);	break;
    case DT_GFXACCEL:
    case DT_FB:			FbProbe(DevInfo);	break;
    case DT_NETIF:		NetifProbe(DevInfo);	break;
    }

    if (DevInfo->Slaves)
	/* Descend */
	IOKpostDevice(DevInfo->Slaves);
    if (DevInfo->Next)
	/* Traverse */
	IOKpostDevice(DevInfo->Next);
}

/*
 * Post process the device tree TreePtr
 */
extern int IOKpost(DevInfo_t **TreePtr)
{
    if (!TreePtr)
	return -1;

    SImsg(SIM_DBG, "IOKpost: Post Process DevInfo tree");

    return IOKpostDevice(*TreePtr);
}

