/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.17 $";
#endif

/*
 * MacOS X
 */

#include "defs.h"
#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/sysctl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "myscsi.h"
#include "netif.h"
#include "part.h"
#include "device.h"

/*
 * Platform Specific Interfaces
 */
extern Opaque_t			        GetCpuInfoMacOSX();
extern Opaque_t			        GetModelMacOSX();
extern Opaque_t			        GetNcpuSysCtl();
extern Opaque_t			        GetKernVerSysCtl();
extern Opaque_t			        GetOSDistSysCtl();
extern Opaque_t			        GetMemoryMaxmem();
extern Opaque_t			        GetMemorySysCtl();
extern Opaque_t			        GetVirtMemSysCtl();
extern Opaque_t			        GetBootTimeSysCtl();
extern Opaque_t			        BuildDevices();
extern Opaque_t			        BuildSoftInfoBundle();
extern Opaque_t			        OSNameMacOSX();
extern Opaque_t			        OSDistNameMacOSX();
extern Opaque_t			        OSDistNameShMacOSX();
extern Opaque_t			        OSDistVerMacOSX();
extern Opaque_t			        GetArchMacOSX();

PsiSwitch_t PsiSwitch = {
    /* GetBootTime */	{ GetBootTimeSysCtl, NULL },
    /* GetCpuInfo */	{ GetCpuInfoMacOSX },
    /* GetCpuType */ 	{ GetCpuTypeCpuInfo, NULL },
    /* GetAppArch */ 	{ GetArchMacOSX, NULL },
    /* GetKernArch */ 	{ GetArchMacOSX, NULL },
    /* GetKernBits */ 	{ GetKernBitsKernArch, NULL },
    /* GetKernVer */ 	{ GetKernVerSysCtl, NULL },
    /* GetLibcName */ 	{ NULL },
    /* GetLibcVer */ 	{ NULL },
    /* GetManLong */ 	{ GetManLongSysinfo, GetManLongDef,
			  NULL },
    /* GetManShort */ 	{ GetManShortSysinfo, GetManShortDef,
			  NULL },
    /* GetMemory */ 	{ GetMemorySysCtl, NULL },
    /* GetVirtMem */ 	{ GetVirtMemSysCtl, NULL },
    /* GetModel */ 	{ GetModelMacOSX, GetModelCpuInfo, GetModelSysCtl,
			  NULL },
    /* GetNumCpu */ 	{ GetNcpuSysCtl, NULL },
    /* GetOsDist */ 	{ GetOSDistSysCtl, NULL },
    /* GetOsName */ 	{ OSNameMacOSX, NULL },
    /* GetOsVer */ 	{ OSDistVerMacOSX, NULL },
    /* GetRomVer */  	{ NULL },
    /* GetSerial */ 	{ NULL },
    /* BuildDevices */  { BuildDevices, NULL },
    /* BuildSoftInfo */ { BuildSoftInfoBundle, NULL },
    /* BuildPartInfo */ { BuildPartInfoStd, NULL },
    /* BuildPrinter */	{ BuildPrinterLp, NULL },
    /* BuildLicense */  { BuildLicenseFlexlm },
    /* GetOsDistName */	{ OSDistNameMacOSX, NULL },
    /* GetOsDistVer */	{ OSDistVerMacOSX, NULL },
    /* GetOsDistNameSh */{ OSDistNameShMacOSX, NULL },
    /* GetNumCpuVirt */	{ GetNcpuSysCtl, NULL },
    /* BuildNetIf */ 	{ NetIfBuild },
    /* BuildNetwork */ 	{ NetworkBuild },
};

/*
 * Machine path
 */
static char *MachinePath[] = { _PATH_MACHINE, NULL };

/*
 * Get value of Key from sw_vers program
 */
static char *GetSwVers(Key)
     char		       *Key;
{
    FILE		       *pf;
    static char			Buff[1024];
    char		       *cp;
    char		       *Value = NULL;

    pf = popen(_PATH_SW_VERS, "r");
    if (!pf) {
	SImsg(SIM_DBG, "%s: Run via popen() failed: %s",
	      _PATH_SW_VERS, SYSERR);
	return (Opaque_t) NULL;
    }

    while (fgets(Buff, sizeof(Buff), pf)) {
	Chop(Buff);
	SImsg(SIM_DBG, "GetSwVers: GOT: %s", Buff);
	cp = strchr(Buff, ':');
	if (!cp)
	    cp = strchr(Buff, ' ');
	if (!cp) {
	    SImsg(SIM_DBG, "GetSwVers: Cannot find end of word: %s", Buff);
	    continue;
	}
	*cp = CNULL;

	if (!EQ(Buff, Key))
	    continue;

	/* Skip white space to start of value */
	for (++cp; cp && *cp && isspace(*cp); ++cp);

	Value = cp;
	break;
    }

    pclose(pf);

    SImsg(SIM_DBG, "GetSwVers: Key=<%s> Found <%s>", Key, Value);

    return Value;
}

/*
 * Get OS Dist Name
 */
extern Opaque_t OSDistNameMacOSX()
{
    static char		       *Value;
    char		       *cp;

    if (Value)
	return Value;

    if (cp = GetSwVers("ProductName"))
	Value = strdup(cp);

    return Value;
}

/*
 * Get OS Name
 */
extern Opaque_t OSNameMacOSX()
{
    static char		       *Value;
    char		       *cp;
    char		       *Full;

    if (Value)
	return Value;

    Full = (char *) OSDistNameMacOSX();
    if (!Full)
	return (Opaque_t) NULL;

    Value = (char *) xcalloc(1, strlen(Full)+1);
    /*
     * Strip out whitespace
     */
    for (cp = Value; Full && *Full; ++Full)
	if (*Full != ' ' && *Full != '\t')
	    *cp++ = *Full;

    return Value;
}

/*
 * Get OS Dist Name (Short)
 */
extern Opaque_t OSDistNameShMacOSX()
{
    static char		       *Value;
    char		       *cp;
    char		       *Full;

    if (Value)
	return Value;

    Full = (char *) OSDistNameMacOSX();
    if (!Full)
	return (Opaque_t) NULL;

    Value = (char *) xcalloc(1, strlen(Full)+1);
    /*
     * Strip out whitespace and convert to lower case
     */
    for (cp = Value; Full && *Full; ++Full)
	if (*Full != ' ' && *Full != '\t')
	    *cp++ = tolower(*Full);

    return Value;
}

/*
 * Get OS Dist Version
 */
extern Opaque_t OSDistVerMacOSX()
{
    static char		       *Value;
    char		       *cp;

    if (Value)
	return Value;

    if (cp = GetSwVers("ProductVersion"))
	Value = strdup(cp);

    return Value;
}

/*
 * Get CPU Info
 */
extern Opaque_t GetCpuInfoMacOSX()
{
    static CpuInfo_t	       *CpuInfo;
    char		       *cp;
    char			Buff[512];

    if (CpuInfo)
	return CpuInfo;

    CpuInfo = GetCpuInfoSysCtl();

    /*
     * Return CPU Type
     */
    if (cp = RunCmds(MachinePath, FALSE)) {
	SImsg(SIM_DBG, "machine command returned <%s>", cp);
	/*
	 * Expand "ppc" to "PowerPC"
	 */
	if (EQN(cp, "ppc", 3)) {
	    snprintf(Buff, sizeof(Buff), "PowerPC %s", cp + 3);
	    cp = Buff;
	}
	CpuInfo->Model = strdup(cp);
    }

    return CpuInfo;
}

/*
 * Get App and Kern architecture
 */
extern Opaque_t GetArchMacOSX()
{
    char		       *cp;

    if (cp = GetKernArchUname()) {
	if (EQ(cp, "Power Macintosh"))
	    /* Translate to something shorter without white space */
	    return (Opaque_t) "powermac";
	else
	    return (Opaque_t) cp;
    }

    return (Opaque_t) NULL;
}

/*
 * Get system model
 */
extern Opaque_t GetModelMacOSX()
{
    char		       *ModelKey;
    static char		       *Model;
    Define_t		       *Def;

    if (Model)
	return Model;

    /*
     * sysctl will return the model key.
     * i.e. "PowerMac3,1" which we then lookup in our table.
     */
    ModelKey = (char *) GetModelSysCtl();
    if (!ModelKey)
	return (Opaque_t) NULL;

    if (Def = DefGet(DL_SYSMODEL, ModelKey, 0, 0))
	if (Def->ValStr1)
	    Model = strdup(Def->ValStr1);

    if (!Model)
	return Model = ModelKey;

    SImsg(SIM_DBG, "GetModelMacOSX: SysCtl Model=<%s> Full Model=<%s>",
	  ModelKey, PRTS(Model));

    return Model;
}

/*
 * Initialize the OS specific parts of the Device Types table
 */
void DevTypesInit()
{
    register int		i;

    for (i = 0; DevTypes[i].Name; ++i)
	switch (DevTypes[i].Type) {
#if 0
	case DT_DISKDRIVE:	DevTypes[i].Probe = ProbeDiskDrive;	break;
	case DT_CD:		DevTypes[i].Probe = ProbeDiskDrive;	break;
	case DT_DVD:		DevTypes[i].Probe = ProbeDiskDrive;	break;
	case DT_TAPEDRIVE:	DevTypes[i].Probe = ProbeTapeDrive;	break;
	case DT_NETIF:		DevTypes[i].Probe = ProbeNetif;		break;
	case DT_CONSOLE:	DevTypes[i].Probe = ProbeConsole;	break;
	case DT_POINTER:	DevTypes[i].Probe = ProbeMouse;		break;
	case DT_FLOPPYCTLR:	DevTypes[i].Probe = ProbeFloppyCtlr;	break;
#endif
	}
}
