/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.3 $";
#endif

/*
 * MntInfoSet functions
 */

#include "defs.h"
#include "mcmnt.h"

#include <sys/param.h>
#include <sys/ucred.h>
#include <sys/mount.h>

/*
 * Set MntInfo using getmntinfo()
 */
static int MntInfoSetMnt(MntInfo_t **MntInfoListPtr)
{  
    MntInfo_t		       *Last = NULL;
    MntInfo_t		       *New;
    MntInfo_t		       *MntInfo = NULL;   
    struct statfs	       *Statfs;
    struct statfs	       *sp;
    register char	       *cp;
    int				StatfsCount;
    register int		i;

    SImsg(SIM_DBG, "MntInfoSetMnt: Building MntInfo from getmntinfo(3)");

    StatfsCount = getmntinfo(&Statfs, 0);
    if (StatfsCount < 0) {
	SImsg(SIM_DBG, "MntInfoSetMnt: getmntinfo failed: %s", SYSERR);
	return -1;
    }

    for (i = 0, sp = Statfs; i < StatfsCount; ++i, ++sp) {
	SImsg(SIM_DBG, "MntInfoSetMnt: Special=<%s> Mnt=<%s> Type=<%s>",
	      PRTS(sp->f_mntfromname), PRTS(sp->f_mntonname),
	      PRTS(sp->f_fstypename));
	if (!sp->f_mntfromname[0])
	    continue;

	New = MntInfoCreate();

	New->DevPath = strdup(sp->f_mntfromname);
	if (cp = strrchr(New->DevPath, '/'))
	    ++cp;
	else
	    cp = New->DevPath;
	New->DevName = strdup(cp);
	if (sp->f_mntonname[0])
	    New->MntName = strdup(sp->f_mntonname);
	if (sp->f_fstypename[0])
	    New->Type = strdup(sp->f_fstypename);

	/* Fill in calculated generic info */
	(void) MntInfoFill(New);

	/* Get filesystem space info */
	(void) MntGetFileSys(New);
	SImsg(SIM_DBG,
	      "MntInfoSetMnt:    Dev=<%s> Dir=<%s> Type=<%s> Usage=%d Size=%dMB Used=%dMB",
	      PRTS(New->DevPath), PRTS(New->MntName), PRTS(New->Type),
	      New->Usage, (int)New->Size, (int)New->AmtUsed);

	if (MntExists(New, *MntInfoListPtr)) {
	    SImsg(SIM_DBG,
		  "MntInfoSetMnt: Entry for Dev=<%s> Dir=<%s> already exists - skipped",
		  New->DevPath, New->MntName);
	    continue;
	}

	if (!Last) 
	    MntInfo = Last = New;
	else {
	    Last->Next = New;
	    Last = New;
	}
    }

    if (!*MntInfoListPtr)
	*MntInfoListPtr = MntInfo;

    return 0;
}

/*
 * Probe and set MntInfo
 */
extern int MntInfoSet(MntInfo_t **MntInfoListPtr)
{
    /*
     * Build Mount Info list
     */
    (void) MntInfoSetMnt(MntInfoListPtr);

    return 0;
}
