/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.6 $";
#endif

/*
 * MacOS X NetIf functions
 */

#include "defs.h"
#include <sys/socket.h>
#include <net/ethernet.h>
#include "netif.h"

/*
 * Convert a MAC string of form "0x001122FFDDEE" to "00:11:22:FF:DD:EE"
 */
static char *MacStr(char *String)
{
    char			Buff[18];
    char *			sp;
    char *			bp;

    if (!String)
	return (char *) NULL;

    for (sp = String + 2, bp = Buff;
	 sp && *sp && (bp < &Buff[sizeof(Buff)]); ) {
	if (bp != Buff)
	    *bp++ = ':';
	*bp++ = *sp++;
	*bp++ = *sp++;
	*bp = CNULL;
    }

    if (Buff[0])
	return strdup(Buff);
    else
	return (char *) NULL;
}

/*
 * Probe a Network Interface (netif)
 */
extern int NetifProbe(DevInfo_t *DevInfo)
{
    char			Buff[512];
    char *			cp;
    NetIF_t *			If;
    Large_t			l;
    ProbeData_t			Probe;

    if (!DevInfo)
	return -1;

    SImsg(SIM_DBG, "NetifProbe: Device=<%s>",
	  PRTS(DevInfo->Name));

    if (DevInfo->DevSpec) {
	If = (NetIF_t *) DevInfo->DevSpec;
    } else {
	If = NetIfNew();
	DevInfo->DevSpec = (Opaque_t) If;
    }

    if (cp = DescFind(DevInfo->DescList, "Local Mac Address")) {
	struct ether_addr      *ea;

	If->MACaddr = MacStr(cp);
	SImsg(SIM_DBG, "NetifProbe:\tMAC=<%s> -> <%s>", cp, If->MACaddr);

	/* Get the hostname of this MAC */
	if (ea = (struct ether_addr *) ether_aton(If->MACaddr)) {
	    if (ether_ntohost(Buff, ea) == 0)
		If->MACname = strdup(Buff);
	}
    }

    /*
     * Look down 2 levels for the OS (ifconfig) name and grab it.
     */
    if (DevInfo->Slaves && DevInfo->Slaves->Slaves) {
	if (cp = DescFind(DevInfo->Slaves->Slaves->DescList,
			  "I/O Kit Class")) {
	    if (EQ(cp, "IOEthernetInterface")) {
		/* This is the OS (ifconfig) name of the device */
		DevInfo->AltName = DevInfo->Name;
		DevInfo->Name = DevInfo->Slaves->Slaves->Name;
	    }
	}
    }

    /*
     * Look down 1 level for the OS interface node
     */
    if (DevInfo->Slaves) {
	if (cp = DescFind(DevInfo->Slaves->DescList, "I/O Kit Class")) {
	    if (EQ(cp, "UniNEnet")) {
		/*
		 * Take interesting bits from this device, give them
		 * to our parent, then destory this node.
		 */
		/* Get speed of NIC */
		if (cp = DescFind(DevInfo->Slaves->DescList, "IOLinkSpeed")) {
		    l = atoL(cp);
		    l = l / 1000000;
		    switch (l) {
		    case 10:	DevInfo->ClassType = CT_ETHER10M;	break;
		    case 100:	DevInfo->ClassType = CT_ETHER100M;	break;
		    case 1000:	DevInfo->ClassType = CT_ETHER1G;	break;
		    default:
			SImsg(SIM_DBG,
			      "NetifProbe:\tNo net speed defined for %d", l);
		    }
		}

		/* Use child's model/vendor if we don't have one yet */
		if (!DevInfo->Model && DevInfo->Slaves->Model)
		    DevInfo->Model = DevInfo->Slaves->Model;
		if (!DevInfo->Vendor && DevInfo->Slaves->Vendor)
		    DevInfo->Vendor = DevInfo->Slaves->Vendor;

		/* Append slaves description list to parents */
		DescAppend(DevInfo->Slaves->DescList, DevInfo->DescList);
	    }
	}
	/* 
	 * XXX Abandon this netif's slave devices
	 */
	SImsg(SIM_DBG, "NetifProbe:\tAbondoning Slaves");
	DevInfo->Slaves = NULL;
    }

    memset(&Probe, 0, sizeof(Probe));
    Probe.DevName = DevInfo->Name;
    Probe.UseDevInfo = DevInfo;
    ProbeNetif(&Probe);

    return 0;
}
