/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.8 $";
#endif

/*
 * Get Device File functions
 */

#include "defs.h"
#include "bjhashtab.h"
#include <dirent.h>
#include <sys/stat.h>

/*
 * Stat entry for a specific directory
 */
struct _StatEntry {
    char		       *Path;		/* Path name */
    dev_t			Rdev;		/* rdev number */
    struct _StatEntry	       *Next;
};
typedef struct _StatEntry       StatEntry_t;

/*
 * Linked list of directories with their cooresponding StatEntry_t lists.
 */
struct _StatData {
    char		       *Dir;		/* Name of directory */
    StatEntry_t		       *StatEntryList;	/* List of StatEntry */
    struct _StatData	       *Next;
};
typedef struct _StatData	StatData_t;

/*
 * Find entry for Dir and return it
 */
static StatData_t *StatFindDir(StatData, Dir)
     StatData_t		       *StatData;
     char		       *Dir;
{
    register StatData_t	       *sdp;

    for (sdp = StatData; sdp; sdp = sdp->Next)
	if (strcmp(Dir, sdp->Dir) == 0)
	    return sdp;

    return (StatData_t *) NULL;
}

/*
 * Build StatData for Dir and store/add it to StatDataPtr
 */
static int StatBuild(StatDataPtr, Dir)
     StatData_t		      **StatDataPtr;
     char		       *Dir;
{
    StatData_t		       *StatData;
    StatEntry_t		       *NewEntry;
    StatEntry_t		       *EntryList = NULL;
    static DIR		       *DirPtr;
    struct dirent	       *DirEnt;
    struct stat			StatBuf;
    static char			PathBuff[MAXPATHLEN];
    char		       *Path = PathBuff;

    SImsg(SIM_DBG, "StatBuild: Building stat cache of <%s>", Dir);

    /*
     * See if we already have a list for this directory
     */
    if (StatData = StatFindDir(*StatDataPtr, Dir))
	EntryList = StatData->StatEntryList;

    if (!(DirPtr = opendir(Dir))) {
	SImsg(SIM_GERR, "%s: Open directory failed: %s", 
	      Dir, SYSERR);
	return(-1);
    }

    /*
     * Add all directory entries we find to EntryList
     */
    while (DirEnt = readdir(DirPtr)) {
	if (EQ(DirEnt->d_name, ".") || EQ(DirEnt->d_name, ".."))
	    continue;
	
	(void) snprintf(Path, sizeof(PathBuff), "%s/%s", 
			Dir, DirEnt->d_name);

	if (stat(Path, &StatBuf) < 0) {
	    SImsg(SIM_GERR, "Stat of %s failed: %s.", Path, SYSERR);
	    continue;
	}

#if	defined(DEBUG_VERBOSE)
	SImsg(SIM_DBG, "StatBuild: adding '%s' rdev=%d",
	      Path, StatBuf.st_rdev);
#endif	/* DEBUG_VERBOSE */

	NewEntry = (StatEntry_t *) xcalloc(1, sizeof(StatEntry_t));
	NewEntry->Path = strdup(Path);
	NewEntry->Rdev = StatBuf.st_rdev;

	/* Add to the list */
	if (EntryList)
	    NewEntry->Next = EntryList;
	EntryList = NewEntry;
    }
    closedir(DirPtr);

    if (!StatData) {
	StatData = (StatData_t *) xcalloc(1, sizeof(StatData_t));
	StatData->Dir = strdup(Dir);
    }
    StatData->StatEntryList = EntryList;

    if (*StatDataPtr) {
	/* Insert it into the existing linked list */
	StatData->Next = *StatDataPtr;
	*StatDataPtr = StatData;
    } else {
	/* Create first member of linked list */
	*StatDataPtr = StatData;
    }

    return 0;
}

/*
 * Look for a device with a raw device number of DevName and return
 * the pathanem of the file.
 */
static char *StatFindByRdev(StatDataPtr, Dir, DevNum, DevName)
     StatData_t		      **StatDataPtr;
     char		       *Dir;
     dev_t			DevNum;
     char		       *DevName;
{
    StatData_t		       *StatData;
    StatEntry_t		       *sep;
    char		       *Path;
    int				Len;

    if (!StatDataPtr || !Dir) {
	SImsg(SIM_DBG, "StatFindByRdev: Missing parameter(s).");
	return (char *) NULL;
    }

    SImsg(SIM_DBG, "StatFindByRdev: Lookup <%s> in <%s>", PRTS(DevName), Dir);

    if (!*StatDataPtr)
	StatBuild(StatDataPtr, Dir);

    /*
     * Lookup the stat data for Dir.  If it doesn't exist, build it.
     */
    StatData = StatFindDir(*StatDataPtr, Dir);
    if (!StatData) {
	StatBuild(StatDataPtr, Dir);
	StatData = StatFindDir(*StatDataPtr, Dir);
	if (!StatData) {
	    SImsg(SIM_DBG, 
		  "StatFindByRdev: Cannot retrieve stat cache of <%s>",
		  Dir);
	    return (char *) NULL;
	}
    }

    /*
     * Search through the stat entry list for a matching rdev entry.
     * Look for the entry with the shortest pathname which we assume
     * to be the best canonical pathname.
     */
    for (sep = StatData->StatEntryList, Path = NULL, Len = 0; 
	 sep; sep = sep->Next)
	if (sep->Rdev == DevNum && (Len == 0 || strlen(sep->Path) < Len)) {
	    Path = sep->Path;
	    Len = strlen(Path);
	}

    if (Path) {
	SImsg(SIM_DBG, "StatFindByRdev: Found <%s> devnum=%d path=<%s>",
	      PRTS(DevName), DevNum, Path);
	return Path;
    } else {
	SImsg(SIM_DBG, 
	      "StatFindByRdev: Cannot find rdev (%d) of <%s> in <%s>", 
	      DevNum, PRTS(DevName), Dir);
	return (char *) NULL;
    }
}

/* 
 * Get device file list.
 * Finds a list of match device files.
 * File list is set to ArgvPtr.
 * Returns number of files found (in ArgvPtr).
 *
 * If DevDefine->File is set, use that to find file name.
 * Otherwise, scan DevDir looking for a device file that matches 
 * DevData->DevNum.
 */
extern int GetDeviceFile(ProbeData, DevDir, ArgvPtr)
     ProbeData_t	       *ProbeData;
     char		       *DevDir;
     char		     ***ArgvPtr;
{
    struct stat			StatBuf;
    static char			PathBuff[MAXPATHLEN];
    char		       *Path = PathBuff;
    static StatData_t	       *StatData;
    register int		i;
    int				Len;
    int				Argc;
    char		      **Argv = NULL;
    static char		      **FileArgv = NULL;
    static int			FileArgc = 0;
    char		       *File;
    char		       *DevName = NULL;
    DevData_t 		       *DevData;
    DevDefine_t		       *DevDefine;

    if (!ProbeData) {
	SImsg(SIM_DBG, "GetDeviceFile: No ProbeData was passed.");
	return(0);
    }

    DevData = ProbeData->DevData;
    DevDefine = ProbeData->DevDefine;

    /* Don't bother trying if the devnum is 0 */
    if (DevData->DevNum == 0 || DevData->DevNum == -1) {
	SImsg(SIM_DBG, "GetDeviceFile: <%s> No DevNum", ProbeData->DevName);
	return(0);
    }

    if (FileArgv && FileArgc > 0) {
	DestroyArgv(&FileArgv, FileArgc);
	FileArgv = NULL;
	FileArgc = 0;
    }

    SImsg(SIM_DBG, "GetDeviceFile: File=<%s> DevDir=<%s>", 
	  (DevDefine && DevDefine->File) ? DevDefine->File : "", 
	  PRTS(DevDir));

    if (DevDefine && DevDefine->File) {
	/*
	 * DevDefine->File is a space seperated list of file names.
	 * If a '%' is found, then the unit number is inserted at
	 * that point.
	 */
	Argc = StrToArgv(DevDefine->File, " ", &Argv, NULL, 0);
	if (Argc <= 0) {
	    SImsg(SIM_DBG, "GetDeviceFile: <%s> No device filenames present",
		  ProbeData->DevName);
	    return(0);
	}
	FileArgv = (char **) xcalloc(Argc, sizeof(char **));
	for (i = 0; i < Argc; ++i) {
	    File = Argv[i];
	    /* Allow NULL members of Argv */
	    if (!File || !File[0])
		continue;
	    /*
	     * If File doesn't start with '/', then insert DevDir.
	     * If File contains '%', then insert the device number.
	     */
	    PathBuff[0] = CNULL;
	    if (File[0] != '/')
		(void) snprintf(PathBuff, sizeof(PathBuff),  "%s/", DevDir);
	    Path = PathBuff + (Len = strlen(PathBuff));

	    if (strchr(File, '%') && DevData->DevUnit >= 0)
		(void) snprintf(Path, sizeof(PathBuff)-Len,
				File, DevData->DevUnit);
	    else
		(void) strcat(Path, File);

	    if (Path[0]) {
		if (stat(Path, &StatBuf) == 0)
		    FileArgv[FileArgc++] = strdup(Path);
		else {
		    SImsg(SIM_GERR, "Stat of %s failed: %s.", Path, SYSERR);
		    Argv[i] = NULL;
		}
	    } else
		SImsg(SIM_DBG, "Convert pathname failed: %s", File);
	}
	if (Argc)
	    DestroyArgv(&Argv, Argc);
    } else {
	/*
	 * Search for a device with the same DevNum in DevDir
	 */
	if (DebugOn && DevData && DevDefine)
	    DevName = MkDevName(DevData->DevName, DevData->DevUnit,
				DevDefine->Type, DevDefine->Flags);
	if (Path = StatFindByRdev(&StatData, DevDir, 
				  DevData->DevNum, DevName)) {
	    FileArgv = (char **) xcalloc(2, sizeof(char *));
	    FileArgv[FileArgc++] = strdup(Path);
	}

    }

    if (FileArgc)
	*ArgvPtr = FileArgv;

    return(FileArgc);
}
