/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.3 $";
#endif

/*
 * MntInfoSet functions
 */

#include "defs.h"
#include "mcmnt.h"

/*
 * Set MntInfo using getmntent()
 */
static int MntInfoSetMnt(MntInfo_t **MntInfoListPtr)
{  
    FILE 		       *fp;
    MntInfo_t		       *Last = NULL;
    MntInfo_t		       *New;
    MntInfo_t		       *MntInfo = NULL;   
    struct vfstab		vfstab;
    register char	       *cp;
    char		      **Argv;   
    struct mnttab 	        MntTab;

    SImsg(SIM_DBG, "MntInfoSetMnt: Building MntInfo from %s", MNTTAB);

    /*
     * Try the current mount table
     */
    if ((fp = fopen(MNTTAB, "r")) == NULL) {
	SImsg(SIM_GERR, "%s: Cannot open for reading: %s.", 
	      MNTTAB, SYSERR);
	return -1;
    }

    while (getmntent(fp, &MntTab) == 0) {
	SImsg(SIM_DBG, "MntInfoSetMnt: Special=<%s> Mnt=<%s> Type=<%s>",
	      PRTS(MntTab.mnt_special), PRTS(MntTab.mnt_mountp), 
	      PRTS(MntTab.mnt_fstype));
	if (!MntTab.mnt_special)
	    continue;

	New = MntInfoCreate();

	New->DevPath = strdup(MntTab.mnt_special);
	if (cp = strrchr(MntTab.mnt_special, '/'))
	    ++cp;
	else
	    cp = MntTab.mnt_special;
	New->DevName = strdup(cp);
	if (MntTab.mnt_mountp)
	    New->MntName = strdup(MntTab.mnt_mountp);
	if (MntTab.mnt_fstype)
	    New->Type = strdup(MntTab.mnt_fstype);
	if (MntTab.mnt_mntopts && !EQ(MntTab.mnt_mntopts, "-"))
	    if (StrToArgv(MntTab.mnt_mntopts, ",", &Argv, NULL, 0) > 0)
		New->MntOpts = Argv;

	/* Fill in calculated generic info */
	(void) MntInfoFill(New);

	/* Get filesystem space info */
	(void) MntGetFileSys(New);
	SImsg(SIM_DBG,
	      "MntInfoSetMnt:    Dev=<%s> Dir=<%s> Type=<%s> Usage=%d Size=%dMB Used=%dMB",
	      PRTS(New->DevPath), PRTS(New->MntName), PRTS(New->Type),
	      New->Usage, (int)New->Size, (int)New->AmtUsed);

	if (MntExists(New, *MntInfoListPtr)) {
	    SImsg(SIM_DBG,
		  "MntInfoSetMnt: Entry for Dev=<%s> Dir=<%s> already exists - skipped",
		  New->DevPath, New->MntName);
	    continue;
	}

	if (!Last) 
	    MntInfo = Last = New;
	else {
	    Last->Next = New;
	    Last = New;
	}
    }

    (void) fclose(fp);

    if (!*MntInfoListPtr)
	*MntInfoListPtr = MntInfo;

    return 0;
}

/*
 * Set MntInfo using getvfsent()
 */
static int MntInfoSetVfstab(MntInfo_t **MntInfoListPtr)
{    
    FILE 		       *fp;
    MntInfo_t		       *Last = NULL;
    MntInfo_t		       *New;
    MntInfo_t		       *MntInfo = NULL;   
    struct vfstab		vfstab;
    register char	       *cp;
    char		      **Argv;

    SImsg(SIM_DBG, "MntInfoSetVfstab: Building MntInfo from %s", VFSTAB);

    /*
     * Try the static mount table, which may not reflect current reality.
     */
    if ((fp = fopen(VFSTAB, "r")) == NULL) {
	SImsg(SIM_GERR, "%s: Cannot open for reading: %s.", 
	      VFSTAB, SYSERR);
	return -1;
    }

    while (getvfsent(fp, &vfstab) == 0) {
	SImsg(SIM_DBG, "MntInfoSetVfstab: Special=<%s> Raw=<%s> Mnt=<%s> Type=<%s>",
	      PRTS(vfstab.vfs_special), PRTS(vfstab.vfs_fsckdev),
	      PRTS(vfstab.vfs_mountp), PRTS(vfstab.vfs_fstype));

	if (!vfstab.vfs_special)
	    continue;

	New = MntInfoCreate();

	if (cp = strrchr(vfstab.vfs_special, '/'))
	    ++cp;
	else
	    cp = vfstab.vfs_special;
	New->DevName = strdup(cp);
	New->DevPath = strdup(vfstab.vfs_special);
	if (vfstab.vfs_fsckdev)
	    New->DevPathRaw = strdup(vfstab.vfs_fsckdev);
	if (vfstab.vfs_fstype && !EQ(vfstab.vfs_fstype, "-"))
	    New->Type = strdup(vfstab.vfs_fstype);
	if (vfstab.vfs_mountp && !EQ(vfstab.vfs_mountp, "-"))
	    New->MntName = strdup(vfstab.vfs_mountp);
	if (vfstab.vfs_mntopts && !EQ(vfstab.vfs_mntopts, "-"))
	    if (StrToArgv(vfstab.vfs_mntopts, ",", &Argv, NULL, 0) > 0)
		New->MntOpts = Argv;

	/* Fill in generic info */
	MntInfoFill(New);

	/*
	 * XXX No attempt is made to get filesys usage since an entry in
	 * vfstab doesn't mean its mounted.
	 */

	SImsg(SIM_DBG,
	      "MntInfoSetVfstab:    Dev=<%s> Dir=<%s> Type=<%s> Usage=%d Size=%dMB Used=%dMB",
	      PRTS(New->DevPath), PRTS(New->MntName), PRTS(New->Type),
	      New->Usage, (int)New->Size, (int)New->AmtUsed);

	if (MntExists(New, *MntInfoListPtr)) {
	    SImsg(SIM_DBG,
		  "MntInfoSetVfstab: Entry for Dev=<%s> Dir=<%s> already exists - skipped",
		  New->DevPath, New->MntName);
	    continue;
	}

	if (!Last) 
	    MntInfo = Last = New;
	else {
	    Last->Next = New;
	    Last = New;
	}
    }

    (void) fclose(fp);

    if (!*MntInfoListPtr)
	*MntInfoListPtr = MntInfo;

    return 0;
}

/*
 * Probe and set MntInfo
 */
extern int MntInfoSet(MntInfo_t **MntInfoListPtr)
{
    /*
     * Build Mount Info list
     *
     * We first look in _PATH_MOUNTED (/etc/mtab) to see what's
     * currently mounted.  Then we look in _PATH_MNTTAB (/etc/fstab)
     * to see what may be unmounted.
     */
    (void) MntInfoSetMnt(MntInfoListPtr);
    (void) MntInfoSetVfstab(MntInfoListPtr);

    return 0;
}

