/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.13 $";
#endif

/*
 * Probe Memory functions
 * 
 * We currently parse the output of prtdiag(1m) to obtain memory bank
 * information because the system interfaces to obtain this info are
 * completely undocumented and the source code to prtdiag(1m) is
 * copyrighted so we cannot use that info.
 */

#include "defs.h"
#include "strutil.h"

/*
 * Macros
 */
#define ICV(a) ((a && (a)[0]) ? atoi(a) : 0)

/*
 * Pathnames
 */
#define _PATH_PRTDIAG_SUN4U	"/usr/platform/sun4u/sbin/prtdiag"

/*
 * These values indicate the # of DIMM's required by the hardware per Bank.
 * This allows us to calculate the size of each DIMM.
 */
#define DIMMS_PER_BANK_EXX00		8	/* E3x00 - E6x00 */

/*
 * Get our system model
 */
static char *GetSysModel()
{
    static MCSIquery_t 		Query;
    int				s;
    char		       *Model = NULL;

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Cmd = MCSI_MODEL;
    s = mcSysInfo(&Query);
    if (s == 0) {
	Model = strdup((char *) Query.Out);
	SImsg(SIM_DBG, "GetSysModel: <%s>", Model);
    }

    return Model;
}

/*
 * Parse a size string of form "N" or "NXX" where XX is a qualifier such
 * as "MB" and return the numeric value in MB.
 */
static int GetSizeVal(SizeStr)
     char		       *SizeStr;
{
    static char			Buff[128];
    char		       *cp;
    Large_t			Qual = 0;
    Large_t			Num = 0;
    Large_t			Value = 0;

    /*
     * Try to detect where something like "MB" or "mb" is present
     */
    if (strchr(SizeStr, 'B') || strchr(SizeStr, 'b')) {
	copybytes(SizeStr, strlen(SizeStr), Buff, sizeof(Buff));
	/* Skip to start of qualifier string */
	for (cp = Buff; cp && *cp && !isalpha(*cp); ++cp);
	if (EQN(cp, "KB", 2))
	    Qual = KBYTES;
	else if (EQN(cp, "MB", 2))
	    Qual = MBYTES;
	else if (EQN(cp, "GB", 2))
	    Qual = GBYTES;
	else {
	    SImsg(SIM_DBG, "GetSizeVal: Unknown qualifier <%s> in <%s>",
		  cp, SizeStr);
	    return 0;
	}
	/* Terminate the string */
	*cp = CNULL;
    }

    Num = atoL(SizeStr);

    /*
     * Convert Num into MB
     */
    switch (Qual) {
    case KBYTES:
	Value = kbytes_to_mbytes(Num);
	break;
    case GBYTES:
	Value = gbytes_to_mbytes(Num);
	break;
    case MBYTES:
    default:	/* Default assume MB */
	Value = Num;
    }

    SImsg(SIM_DBG, "GetSizeVal(%s) Num=%ld Qualifier=%ld Value=%d",
	  SizeStr, Num, Qual, Value);

    return (int) Value;
}

/*
 * Create and return a memory device
 */
static int CreateMemory(TreePtr, SearchNames, DevName, DimmsPerBank, 
			Board, Port, Bank, Size, DimmSize,
			Speed, Interlv, Group)
     DevInfo_t		      **TreePtr;
     char		      **SearchNames;
     char		       *DevName;
     int			DimmsPerBank;
     int		        Board;
     int		        Port;
     int		        Bank;
     int		        Size;
     int		        DimmSize;
     char		       *Speed;
     char		       *Interlv;
     char		       *Group;
{
    DevInfo_t		       *DevInfo = NULL;
    DevInfo_t		       *SysBoard;
    DevInfo_t		       *MemCtl;
    static DevFind_t		Find;
    static char			Buff[1024];
    DevDesc_t		      **lp;
    char		       *cp;

    if (!Size) {
	SImsg(SIM_DBG, "CreateMemory: Bad args");
	return -1;
    }

    DevInfo = NewDevInfo(NULL);
    lp = &(DevInfo->DescList);
    DevInfo->Type = DT_MEMORY;
    DevInfo->Unit = UnitAssign(DT_MEMORY, 0, NULL);
    snprintf(Buff, sizeof(Buff), "memory%d", DevInfo->Unit);
    DevInfo->Name = strdup(Buff);
    if (DevName)
	DevInfo->AltName = strdup(DevName);

    cp = GetSizeStr(Size, (Large_t)MBYTES, 0);
    if (cp)
	DevInfo->Capacity = strdup(cp);
    if (Speed)
	DevInfo->Speed = strdup(Speed);

    if (Bank >= 0) {
	snprintf(Buff, sizeof(Buff), "Bank %d", Bank);
	DevInfo->ModelDesc = strdup(Buff);
    }

    SImsg(SIM_DBG, "CreateMemory: <%s> Board=%d Bank=%d Size=<%d MB>",
	  DevInfo->Name, Board, Bank, Size);

    if (!DimmsPerBank && Size && DimmSize)
	DimmsPerBank = Size / DimmSize;
    if (!DimmSize && DimmsPerBank && Size)
	DimmSize = Size / DimmsPerBank;

    if (Size && DimmsPerBank) {
	AddDesc(lp, "# DIMMs", itoa(DimmsPerBank), 0);
	AddDesc(lp, "DIMM Size", itoa(DimmSize), 0);
	snprintf(Buff, sizeof(Buff), "%d x %d MB",
		 DimmsPerBank, DimmSize);
	DevInfo->Model = strdup(Buff);
    }

    if (Board >= 0)	AddDesc(lp, "System Board", strdup(itoa(Board)), 0);
    if (Bank >= 0)	AddDesc(lp, "Bank", strdup(itoa(Bank)), 0);
    if (Port >= 0)	AddDesc(lp, "PortID", strdup(itoa(Port)), 0);
    if (Interlv)	AddDesc(lp, "Interleave Factor", strdup(Interlv), 0);
    if (Group)		AddDesc(lp, "Interleave Group", strdup(Group), 0);

    /*
     * Try and find the sysboard this belongs to and add it.
     */
    DevFindInit(&Find);
    Find.Expr = DFE_OR;
    Find.DevType = DT_SYSBOARD;
    Find.Tree = *TreePtr;
    Find.Unit = Board;
    if (SysBoard = DevFind(&Find)) {
	/*
	 * Try to do even better by finding a memory controller attached 
	 * to this board.
	 */
	DevFindInit(&Find);
	Find.Tree = SysBoard;
	Find.DevType = DT_MEMCTL;
	if (MemCtl = DevFind(&Find)) {
	    DevInfo->Master = MemCtl;
	    DevInfo->MasterName = MemCtl->Name;
	} else {
	    /*
	     * Settle for adding the memory dev to the board device
	     */
	    DevInfo->Master = SysBoard;
	    DevInfo->MasterName = SysBoard->Name;
	}

    }

    /*
     * We still haven't found our master so now look for a memory controller
     */
    if (!DevInfo->Master) {
	DevFindInit(&Find);
	Find.Expr = DFE_OR;
	Find.DevType = DT_MEMCTL;
	Find.Tree = *TreePtr;
	Find.Unit = Board;
	/* Look for a MEMCTL with the same PortID */
	if (Port >= 0) {
	    Find.Port = Port;
	    Find.Expr = DFE_AND;
	}
	if (SysBoard = DevFind(&Find)) {
	    DevInfo->Master = SysBoard;
	    DevInfo->MasterName = SysBoard->Name;
	}
    }

    if (DevInfo->MasterName)
	SImsg(SIM_DBG, "Attaching <%s> to <%s>",
	      DevInfo->Name, DevInfo->MasterName);

    AddDevice(DevInfo, TreePtr, SearchNames);

    return 0;
}

/*
 * Probe Memory on an Enterprise E[3456]x00 system
 */
static int ProbeMemoryExx00(TreePtr, SearchNames)
    DevInfo_t		      **TreePtr;
    char		      **SearchNames;
{
    FILE		       *pd;
    static char			Buff[1024];
    register char	       *cp;
    int				FoundStart = 0;
    char		      **Argv;
    int				Argc;

    SImsg(SIM_DBG, "Probing memory for Exx00 system type using %s",
	  _PATH_PRTDIAG_SUN4U);

    (void) signal(SIGPIPE, SIG_IGN);

#if	defined(PRTDIAGDEBUG)
    pd = fopen("./prtdiag.input", "r");
#else
    pd = popen(_PATH_PRTDIAG_SUN4U, "r");
#endif

    /*
     * Here is what the memory section looks like:
========================= Memory =========================

                                              Intrlv.  Intrlv.
Brd   Bank   MB    Status   Condition  Speed   Factor   With
---  -----  ----  -------  ----------  -----  -------  -------
 0     0    1024   Active      OK       60ns    8-way     A
 0     1    1024   Active      OK       60ns    8-way     A
 2     0    1024   Active      OK       60ns    8-way     A
 2     1    1024   Active      OK       60ns    4-way     B
 3     0    1024   Active      OK       60ns    8-way     A
 3     1    1024   Active      OK       50ns    4-way     B
 4     0    1024   Active      OK       60ns    8-way     A
 4     1    1024   Active      OK       60ns    4-way     B
 5     0    1024   Active      OK       60ns    8-way     A
 5     1    1024   Active      OK       60ns    4-way     B
 6     0    1024   Active      OK       60ns    8-way     A
 6     1    1024   Active      OK       60ns    2-way     C
 7     0    1024   Active      OK       50ns    8-way     A
 7     1    1024   Active      OK       50ns    2-way     C

*****************************************************************************/

    while (fgets(Buff, sizeof(Buff), pd)) {
	SImsg(SIM_DBG, "PRTDIAG: %s", Buff);

	/*
	 * Look for line containing banner indicating start of memory info
	 */
	if (REmatch(Buff, "== Memory ==", NULL)) {
	    /*
	     * Skip over column titles
	     */
	    while (!FoundStart && fgets(Buff, sizeof(Buff), pd)) {
		SImsg(SIM_DBG, "PRTDIAG: %s", Buff);
		if (EQN(Buff, "---", 3))
		    FoundStart = 1;
	    }

	    if (FoundStart) {
		/* The next line is the start of the data we want */
		continue;
	    }
	}

	if (!FoundStart)
	    continue;

	if (cp = strchr(Buff, '\n'))
	    *cp = CNULL;

	/*
	 * If the line is blank or begins with '=' then we've reached the
	 * end of the memory section
	 */
	if (!Buff[0] || Buff[0] == '=')
	    break;

	/* Skip over leading whitespace */
	for (cp = Buff; cp && *cp && isspace(*cp); ++cp);

	Argc = StrToArgv(cp, " \t", &Argv, NULL, STRTO_SKIPRP);
	if (Argc != 8) {
	    SImsg(SIM_DBG, "ProbeMemoryExx00: Arg count (%d) wrong: %s",
		  Argc, cp);
	    continue;
	}

	(void) CreateMemory(TreePtr, SearchNames, 
			    NULL, 		/* DevName */
			    DIMMS_PER_BANK_EXX00,
			    ICV(Argv[0]), 	/* Board # */
			    -1,			/* Port */
			    ICV(Argv[1]), 	/* Bank # */
			    GetSizeVal(Argv[2]),/* Size */
			    0,			/* DimmSize */
			    Argv[5], 		/* Speed */
			    Argv[6], 		/* Interlv */
			    Argv[7]);		/* Interlv Group */
    }

#if	defined(PRTDIAGDEBUG)
    fclose(pd);
#else
    pclose(pd);
#endif

    if (!FoundStart) {
	SImsg(SIM_DBG, "ProbeMemoryExx00: No memory info found from %s",
	      _PATH_PRTDIAG_SUN4U);
	return -1;
    }

    return 0;
}

/*
 * Probe Memory on an Fire 3x00-6x00 MidFrame and the Fire 15000
 */
static int ProbeMemoryFireMidFrame(TreePtr, SearchNames)
    DevInfo_t		      **TreePtr;
    char		      **SearchNames;
{
    FILE		       *pd;
    static char			Buff[1024];
    register char	       *cp;
    int				FoundStart = 0;
    char		      **Argv;
    int				Argc;

    SImsg(SIM_DBG, "Probing memory for Fire MidFrame system type using %s",
	  _PATH_PRTDIAG_SUN4U);

    (void) signal(SIGPIPE, SIG_IGN);

#if	defined(PRTDIAGDEBUG)
    pd = fopen("./prtdiag.input", "r");
#else
    pd = popen(_PATH_PRTDIAG_SUN4U, "r");
#endif

    /*
     * Here is what the memory section looks like:
========================= Memory Configuration ===============================

                     Logical  Logical  Logical 
               Port  Bank     Bank     Bank         DIMM    Interleave  Interleave
FRU Name        ID   Num      Size     Status       Size    Factor      Segment
-------------  ----  ----     ------   -----------  ------  ----------  ----------
/N0/SB0/P0/B0    0    0       512MB    pass          256MB     8-way       0
/N0/SB0/P0/B0    0    2       512MB    pass          256MB     8-way       0
/N0/SB0/P1/B0    1    0       512MB    pass          256MB     8-way       0
/N0/SB0/P1/B0    1    2       512MB    pass          256MB     8-way       0
/N0/SB0/P2/B0    2    0       512MB    pass          256MB     8-way       0
/N0/SB0/P2/B1    2    1      2048MB    pass         1024MB     4-way       1
/N0/SB0/P2/B0    2    2       512MB    pass          256MB     8-way       0
/N0/SB0/P2/B1    2    3      2048MB    pass         1024MB     4-way       1
/N0/SB0/P3/B0    3    0       512MB    pass          256MB     8-way       0
/N0/SB0/P3/B1    3    1      2048MB    pass         1024MB     4-way       1
/N0/SB0/P3/B0    3    2       512MB    pass          256MB     8-way       0
/N0/SB0/P3/B1    3    3      2048MB    pass         1024MB     4-way       1
/N0/SB2/P0/B0    8    0       512MB    pass          256MB     8-way       2
/N0/SB2/P0/B0    8    2       512MB    pass          256MB     8-way       2
/N0/SB2/P1/B0    9    0       512MB    pass          256MB     8-way       2
/N0/SB2/P1/B0    9    2       512MB    pass          256MB     8-way       2
/N0/SB2/P2/B0   10    0       512MB    pass          256MB     8-way       2
/N0/SB2/P2/B0   10    2       512MB    pass          256MB     8-way       2
/N0/SB2/P3/B0   11    0       512MB    pass          256MB     8-way       2
/N0/SB2/P3/B0   11    2       512MB    pass          256MB     8-way       2

*****************************************************************************/

    while (fgets(Buff, sizeof(Buff), pd)) {
	SImsg(SIM_DBG, "PRTDIAG: %s", Buff);

	/*
	 * Look for line containing banner indicating start of memory info
	 */
	if (REmatch(Buff, "== Memory Configuration ==", NULL)) {
	    /*
	     * Skip over column titles
	     */
	    while (!FoundStart && fgets(Buff, sizeof(Buff), pd)) {
		SImsg(SIM_DBG, "PRTDIAG: %s", Buff);
		if (EQN(Buff, "---", 3))
		    FoundStart = 1;
	    }

	    if (FoundStart) {
		/* The next line is the start of the data we want */
		continue;
	    }
	}

	if (!FoundStart)
	    continue;

	if (cp = strchr(Buff, '\n'))
	    *cp = CNULL;

	/*
	 * If the line is blank or begins with '=' then we've reached the
	 * end of the memory section
	 */
	if (!Buff[0] || Buff[0] == '=')
	    break;

	/* Skip over leading whitespace */
	for (cp = Buff; cp && *cp && isspace(*cp); ++cp);

	Argc = StrToArgv(cp, " \t", &Argv, NULL, STRTO_SKIPRP);
	if (Argc != 8) {
	    SImsg(SIM_DBG, "ProbeMemoryFireMidFrame: Arg count (%d) wrong: %s",
		  Argc, cp);
	    continue;
	}

	(void) CreateMemory(TreePtr, SearchNames, 
			    Argv[0],		/* DevName */
			    0,			/* DimmsPerBank */
			    -1,	 		/* Board # */
			    ICV(Argv[1]),	/* Port */
			    ICV(Argv[2]),	/* Bank */
			    GetSizeVal(Argv[3]),/* Size */
			    ICV(Argv[5]),	/* DimmSize */
			    NULL,		/* Speed */
			    Argv[6],		/* Interlv */
			    Argv[7]);		/* Interlv Group */
    }

#if	defined(PRTDIAGDEBUG)
    fclose(pd);
#else
    pclose(pd);
#endif

    if (!FoundStart) {
	SImsg(SIM_DBG,
	      "ProbeMemoryFireMidFrame: No memory info found from %s");
	return -1;
    }

    return 0;
}

/*
 * Look for and add memory devices.
 */
extern int ProbeMemory(TreePtr, SearchNames)
    DevInfo_t		      **TreePtr;
    char		      **SearchNames;
{
    int				Status = 0;
    char		       *Model;

    Model = GetSysModel();
    if (!Model) {
	SImsg(SIM_DBG, "ProbeMemory: Cannot get system model.");
	return -1;
    }

    /*
     * This code is highly model specific
     */
    if (REmatch(Model, "E.[3456]?00", NULL)) {
	Status = ProbeMemoryExx00(TreePtr, SearchNames);
    } else if (REmatch(Model, "Fire*.[3456].?00", NULL) ||
	       REmatch(Model, "Fire*15.?00", NULL)) {
	Status = ProbeMemoryFireMidFrame(TreePtr, SearchNames);
    } else {
	SImsg(SIM_DBG, 
	      "ProbeMemory: No supported method for probing <%s> systems",
	      Model);
    }

    return Status;
}
