/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.22 $";
#endif

/*
 * Mnt (mount) functions
 */

#include "defs.h"
#include "mcmnt.h"

#if HAVE_SYS_VFS_H
# include <sys/vfs.h>
#endif
#if HAVE_SYS_STATFS_H
# include <sys/statfs.h>
#endif
#if HAVE_SYS_STATVFS_H
# include <sys/statvfs.h>
#endif
#if HAVE_SYS_MOUNT_H
# include <sys/mount.h>
#endif
#if HAVE_MNTENT_H
# include <mntent.h>
#endif
#include "part.h"

/*
 * Force statvfs() on Solaris to avoid incompatable statfs().
 * Force statvfs() on IRIX to avoid 4 parameter statfs().
 */
#if defined(sunos) || defined(irix)
#undef HAVE_STATFS
#endif

static char *LocalFStypes[] = LOCAL_FS_TYPES;
static MntInfo_t *MntInfoList;

/*
 * Create a new MntInfo_t
 */
extern MntInfo_t *MntInfoCreate(void)
{
    MntInfo_t *			New;

    New = (MntInfo_t *) xcalloc(1, sizeof(MntInfo_t));

    return New;
}

/*
 * Get filesystem info and set it in MntInfo
 */
extern int MntGetFileSys(MntInfo_t *MntInfo)
{
    /*
     * The statvfs(2) in FreeBSD 5.0 is broken, but it does have a working
     * statfs(2) so we prefer statfs(2) over statvfs(2).
     */
#if HAVE_STATFS
    struct statfs		Statfs;
#else
# if HAVE_STATVFS
    struct statvfs		Statfs;
# endif	/* HAVE_STATVFS */
#endif	/* HAVE_STATFS */

    if (!MntInfo || !MntInfo->MntName)
	return -1;

    if (*MntInfo->MntName != '/')
	return -1;

#if HAVE_STATFS
    if (statfs(MntInfo->MntName, &Statfs) == 0) {
#else
# if HAVE_STATVFS
    if (statvfs(MntInfo->MntName, &Statfs) == 0) {
# endif	/* HAVE_STATVFS */
#endif	/* HAVE_STATFS */
#define fssize(num, fsbs, bs) (((fsbs) != 0 && (fsbs) < (bs)) ? \
		(num) / ((bs) / (fsbs)) : (num) * ((fsbs) / (bs)))
	MntInfo->Size = (Large_t) fssize((Large_t)Statfs.f_blocks,
					 (Large_t)Statfs.f_bsize, MBYTES);
	MntInfo->AmtUsed = (Large_t) fssize((Large_t)(Statfs.f_blocks -
						      Statfs.f_bfree),
					    (Large_t)Statfs.f_bsize, MBYTES);
#undef fssize
    }

#if HAVE_STATFS || HAVE_STATVFS
#if HAVE_PRINTF_LL
    McDebug("MntGetFileSys:\t fs=<%s> blocks=%lld bsize=%lld",
#else
    McDebug("MntGetFileSys:\t fs=<%s> blocks=%ld bsize=%ld",
#endif
	    PRTS(MntInfo->MntName),
	    (Large_t)Statfs.f_blocks, (Large_t)Statfs.f_bsize);
#endif	/* HAVE_STATFS || HAVE_STATVFS */

    return 0;
}

/*
 * Does Entry exist in List?
 */
extern int MntExists(MntInfo_t *	Entry,
		     MntInfo_t *	List)
{
    register MntInfo_t	       *mp;

    for (mp = List; mp; mp = mp->Next) {
	if (eq(mp->DevPath, Entry->DevPath) || eq(mp->MntName, Entry->MntName))
	    return TRUE;
    }

    return FALSE;
}

/*
 * Fill in standard/generic Mnt Info
 */
extern int MntInfoFill(MntInfo_t *Mnt)
{
    int			Usage = 0;

    McDebug("MntInfoFill: Dev=<%s> Dir=<%s> Type=<%s>",
	    PRTS(Mnt->DevPath), PRTS(Mnt->MntName), PRTS(Mnt->Type));

    /*
     * Is this a local filesystem?
     * Check the list of known local fs types
     */
    if (StrArray(Mnt->Type, LocalFStypes) > 0) {
	Usage = PIU_FILESYS;
#if defined(MNTTYPE_SWAP)
    } else if (EQ(Mnt->Type, MNTTYPE_SWAP)) {
	Usage = PIU_SWAP;
#endif
    } else if (EQ(Mnt->Type, "swap")) {
	Usage = PIU_SWAP;
    } else {
	SImsg(SIM_DBG, "MntInfoFill: Type <%s> is not of interest to us.",
	      PRTS(Mnt->Type));
    }

    if (Usage) {
	McDebug("MntInfoFill:\t Setting Usage=%d", Usage);
	Mnt->Usage = Usage;
    }

    return 0;
}

/*
 * Return pointer to MntInfoList
 */
extern MntInfo_t *MntInfoGet(void)
{
    struct stat		st;
    MntInfo_t *		Mnt;

    if (MntInfoList)
	return MntInfoList;

    /* Call OS provided function to find MntInfo and set it to MntInfoList */
    (void) MntInfoSet(&MntInfoList);

    /*
     * Perform post processing.
     * Add rdev value to each Mnt member.
     */
    for (Mnt = MntInfoList; Mnt; Mnt = Mnt->Next) {
	McDebug("MntInfoGet: Post processing DevPath=<%s> MntName=<%s>",
		    PRTS(Mnt->DevPath), PRTS(Mnt->MntName));
	if (!Mnt->DevPath || *Mnt->DevPath != '/') {
	    McDebug("MntInfoGet:\t Skipping stat");
	    continue;
	}
	if (stat(Mnt->DevPath, &st) == 0) {
	    McDebug("MntInfoGet:\t rdev=%d,%d",
		    major(st.st_rdev), minor(st.st_rdev));
	    Mnt->rdev = st.st_rdev;
	} else {
	    McDebug("MntInfoGet:\t %s: Stat failed.", Mnt->DevPath);
	}
    }

    return MntInfoList;
}

/*
 * Get Mount info for partition Part
 */
extern int PartSetFilesys(DevInfo_t *	DevInfo,
			  char *	DevFile,
			  PartInfo_t *	Part)
{
    static MntInfo_t   	       *MntInfo;
    MntInfo_t       	       *Mnt;
    static char			Name[128];
    char		      **Argv;
    int				l;

    if (!DevInfo || !DevFile || !Part)
	return -1;

    McDebug("PartSetFilesys: DevInfo=<%s> File=<%s>",
	    PRTS(DevInfo->Name), PRTS(DevFile));

    MntInfo = MntInfoGet();

    /*
     * If DevFile ends with a digit, then add "p" + Part#
     * Otherwise just add Part#
     */
    l = strlen(DevFile);
    if (isdigit(DevFile[l-1])) 
	(void) snprintf(Name, sizeof(Name), "%sp%d", 
			DevFile, Part->Num);
    else
	(void) snprintf(Name, sizeof(Name), "%s%d", 
			DevFile, Part->Num);

    if (!Part->DevPath)
	Part->DevPath = strdup(Name);

    for (Mnt = MntInfo; Mnt; Mnt = Mnt->Next)
	if (eq(Mnt->DevPath, Name) || eq(Mnt->DevPath, Part->DevPath)) {
	    McDebug("PartSetFilesys: \tFound MntName=<%s> Type=<%s> Size=%ld",
		    PRTS(Mnt->MntName), PRTS(Mnt->Type), (long)Mnt->Size);
	    Part->Usage = PIU_FILESYS;
	    Part->MntName = Mnt->MntName;
	    Part->Type = Mnt->Type;
	    if (Part->AmtUsed <= 0)
		Part->AmtUsed = Mnt->AmtUsed;
	    if (Part->Size <= 0)
		Part->Size = Mnt->Size;
	    if (Mnt->Opts && StrToArgv(Mnt->Opts, ",", &Argv, NULL, 0) > 0)
		Part->MntOpts = Argv;
	    return 0;
	}

    return -1;
}

/*
 * Get Mount info for partition Part
 */
extern int GetPartUsage(DevInfo_t *	DevInfo,
			char *		DevFile,
			DiskPart_t *	DiskPart)
{
    if (!DiskPart || !DiskPart->PartInfo)
	return -1;

    return PartSetFilesys(DevInfo, DevFile, DiskPart->PartInfo);
}

/*
 * Build partition information using mount info.
 * This is intended to augment normal disk partition probes
 * and should catch RAID/volume manager type partitions that
 * otherwise would not be identified.
 */
extern Opaque_t BuildPartInfoMnt(PartInfo_t **	PartInfoTree,
				 char **	SearchExp)
{
    MntInfo_t *			MntInfo;
    MntInfo_t *			Mnt;
    PartInfo_t *		New;
    int				IsNew = TRUE;

    if (!PartInfoTree)
	return (Opaque_t) -1;

    SImsg(SIM_DBG, "BuildPartInfoMnt: Checking MntInfo for partitions ...");

    MntInfo = MntInfoGet();
    if (!MntInfo) {
	SImsg(SIM_DBG, "BuildPartInfoMnt: Cannot fetch MntInfo");
	return (Opaque_t) -1;
    }

    for (Mnt = MntInfo; Mnt; Mnt = Mnt->Next) {
	/* Only local filesystems are of interest to us */
	if (Mnt->Usage != PIU_FILESYS && Mnt->Usage != PIU_SWAP) {
	    SImsg(SIM_DBG, "BuildPartInfoMnt: Mnt <%s> Dir=<%s> not a local fs - SKIPPED",
		  Mnt->DevPath, Mnt->MntName);
	    continue;
	}

	/*
	 * Make sure this entry wasn't added by someone else.
	 * If it's already present, we'll merge the data.
	 */
	New = PartFindByDev(PartInfoTree, Mnt);
	if (New) {
	    IsNew = FALSE;
	    SImsg(SIM_DBG, "BuildPartInfoMnt: Mnt <%s> Dir=<%s> already in Part Table - Merging",
		  Mnt->DevPath, Mnt->MntName);
	}

	if (!New) {
	    /* Commit to creation */
	    McDebug("BuildPartInfoMnt: Adding Part from Mnt=<%s> Dir=<%s>",
		    Mnt->DevPath, Mnt->MntName);
	    New = PartInfoCreate(NULL);
	}

	if (!New->BaseName && IsNew)
	    New->BaseName = BaseName(Mnt->DevPath);
	if (!New->DevPath)    	New->DevPath = Mnt->DevPath;
	if (!New->Type)		New->Type = Mnt->Type;
	if (New->Usage <= 0)	New->Usage = Mnt->Usage;
	if (!New->MntName)	New->MntName = Mnt->MntName;
	if (!New->Size)		New->Size = Mnt->Size;
	if (!New->AmtUsed)	New->AmtUsed = Mnt->AmtUsed;
	if (!New->MntOpts && Mnt->Opts)
	    StrToArgv(Mnt->Opts, ",", &(New->MntOpts), NULL, 0);

	if (IsNew)
	    PartInfoAdd(PartInfoTree, New);
    }

    return (Opaque_t) 0;
}
