/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.7 $";
#endif

/*
 * Printer class information
 */

#include "defs.h"

/*
 * Add Printer to PrinterTree
 */
extern int PrinterAdd(PrinterTree, Printer)
     Printer_t		      **PrinterTree;
     Printer_t		       *Printer;
{
    Printer_t		       *Ptr;

    if (!PrinterTree)
	return -1;

    if (*PrinterTree) {
	for (Ptr = *PrinterTree; Ptr && Ptr->Next; Ptr = Ptr->Next);
	Ptr->Next = Printer;
    } else {
	*PrinterTree = Printer;
    }

    return 0;
}

/*
 * Free all members of Array
 */
static int FreeArray(Array)
     char		      **Array;
{
    register char	      **cpp;

    if (!Array)
	return -1;

    for (cpp = Array; cpp && *cpp; ++cpp)
	if (*cpp)
	    (void) free(*cpp);

    return 0;
}

/*
 * Destroy a Printer_t
 */
extern int PrinterDestroy(Printer)
     Printer_t		       *Printer;
{
    if (!Printer)
	return -1;

    /* XXX need to free DescList */
    if (Printer->Queue)		(void) free(Printer->Queue);
    if (Printer->Desc)		(void) free(Printer->Desc);
    if (Printer->Vendor)	(void) free(Printer->Vendor);
    if (Printer->Model)		(void) free(Printer->Model);
    if (Printer->Device)	(void) free(Printer->Device);
    if (Printer->Server)	(void) free(Printer->Server);
    if (Printer->RemoteQueue)	(void) free(Printer->RemoteQueue);
    if (Printer->SpoolDir)	(void) free(Printer->SpoolDir);
    if (Printer->Aliases)	FreeArray(Printer->Aliases);
    if (Printer->Langs)		FreeArray(Printer->Langs);

    (void) free(Printer);

    return 0;
}

/*
 * Create a new Printer_t
 */
extern Printer_t *PrinterCreate(Old)
     Printer_t		       *Old;
{
    Printer_t		       *New;

    New = (Printer_t *) xcalloc(1, sizeof(Printer_t));
    New->MaxJobSize = -1;

    return New;
}

/*
 * Interface function between mcSysInfo() and underlying probes.
 */
extern int PrinterMCSI(Query)
     MCSIquery_t	       *Query;
{
    static Printer_t 	       *Root;
    Printer_t	 	       *Part;
    PSI_t		       *PSIptr;

    if (!Query) {
	errno = ENXIO;
	return -1;
    }

    switch (Query->Op) {
    case MCSIOP_CREATE:
	if (!Root) {
	    if (!PsiSwitch.BuildPrinter || !PsiSwitch.BuildPrinter->Get) {
		SImsg(SIM_DBG, 
		      "Support for `Printer' class information is not available on this platform.");
		return 0;
	    }

	    SImsg(SIM_DBG, "BUILDING Printer Tree ...");
	    /*
	     * Call each defined Build function
	     */
	    for (PSIptr = PsiSwitch.BuildPrinter;
		 PSIptr && PSIptr->Get; ++PSIptr) {
		(void) (PSIptr->Get)(&Root, Query->SearchExp);
	    }
	}

	if (!Root) {
	    SImsg(SIM_DBG, "No printer information was found.");
	    errno = ENOENT;
	    return -1;
	}

	Query->Out = (Opaque_t) Root;
	Query->OutSize = sizeof(Printer_t *);
	break;
    case  MCSIOP_DESTROY:
	if (Query->Out && Query->OutSize)
	    return PrinterDestroy((Printer_t *) Query->Out);
	break;
    }

    return 0;
}
