/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

/*
 * This code is based on "cpuid" by Phil Karn.
 * Other references:
 * http://www.sandpile.org/ia32/cpuid.htm
 * http://www.cpuid.com
 */

/* Intel and AMD x86 CPUID display program v 2.0 (24 Dec 1999)
 * Copyright 1999 Phil Karn, KA9Q
 * May be used under the terms of the GNU Public License (GPL)

 * Reference documents:
 * ftp://download.intel.com/design/pro/applnots/24161809.pdf  (AP-485)
 * http://developer.intel.com/design/pentiumiii/applnots/24512501.pdf (AP-909)
 * http://www.amd.com/K6/k6docs/pdf/20734j.pdf
 */

#include <stdio.h>
#include "defs.h"

/*
 * Edit the CPU Manufacturer name
 */
extern char *CpuManEdit(Name)
     char 		       *Name;
{
    char		       *Man;

    if (!Name)
	return (char *) NULL;

    Man = Name;
    if (EQN("Authentic", Man, 9))
	Man += 9;
    else if (EQN("Genuine", Man, 7))
	Man += 7;

    return Man;
}

/*
 * Edit the CPU Model name and remove extra stuff.
 */
extern char *CpuModelEdit(Name)
     char 		       *Name;
{
    char		       *Model;
    char		       *cp;

    if (!Name)
	return (char *) NULL;

    Model = Name;
    if (EQN(Name, "AMD ", 4))
	Model = Name+4;
    else if (EQN(Name, "Intel ", 6))
	Model = Name+6;
#if 0
    if ((cp = strchr(Model, '(')) && EQN(cp, "(tm)", 4))
	*cp = CNULL;
#endif

    return Model;
}

#if	defined(IS_X86) && defined(HAVE_CPUID)
void cpuid(int, long *, long *, long *, long *);

/*
 * SubTypes of processor
 */
#define IAST_CELERON		1		/* Celeron */
#define IAST_PENTIUMXEON	2		/* Pentium Xeon */

/*
 * Get the subtype of Intel CPU based on cache flags
 */
static int IntelSubType(unsigned long eax, unsigned long ebx, 
			unsigned long ecx, unsigned long edx)
{
    unsigned long		Val;

    Val = eax & 0xFF000000;
    if (Val == 0x40000000)
	return IAST_CELERON;
    if ((Val >= 0x44000000) && (Val <= 0x45000000))
	return IAST_PENTIUMXEON;

    Val = eax & 0xFF0000;
    if (Val == 0x400000)
	return IAST_CELERON;
    if ((Val >= 0x440000) && (Val <= 0x450000))
	return IAST_PENTIUMXEON;

    Val = eax & 0xFF00;
    if (Val == 0x4000)
	return IAST_CELERON;
    if ((Val >= 0x4400) && (Val <= 0x4500))
	return IAST_PENTIUMXEON;

    Val = ebx & 0xFF000000;
    if (Val == 0x40000000)
	return IAST_CELERON;
    if ((Val >= 0x44000000) && (Val <= 0x45000000))
	return IAST_PENTIUMXEON;

    Val = ebx & 0xFF0000;
    if (Val == 0x400000)
	return IAST_CELERON;
    if ((Val >= 0x440000) && (Val <= 0x450000))
	return IAST_PENTIUMXEON;

    Val = ebx & 0xFF00;
    if (Val == 0x4000)
	return IAST_CELERON;
    if ((Val >= 0x4400) && (Val <= 0x4500))
	return IAST_PENTIUMXEON;

    Val = ebx & 0xFF;
    if (Val == 0x40)
	return IAST_CELERON;
    if ((Val >= 0x44) && (Val <= 0x45))
	return IAST_PENTIUMXEON;

    Val = ecx & 0xFF000000;
    if (Val == 0x40000000)
	return IAST_CELERON;
    if ((Val >= 0x44000000) && (Val <= 0x45000000))
	return IAST_PENTIUMXEON;

    Val = ecx & 0xFF0000;
    if (Val == 0x400000)
	return IAST_CELERON;
    if ((Val >= 0x440000) && (Val <= 0x450000))
	return IAST_PENTIUMXEON;

    Val = ecx & 0xFF00;
    if (Val == 0x4000)
	return IAST_CELERON;
    if ((Val >= 0x4400) && (Val <= 0x4500))
	return IAST_PENTIUMXEON;

    Val = ecx & 0xFF;
    if (Val == 0x40)
	return IAST_CELERON;
    if ((Val >= 0x44) && (Val <= 0x45))
	return IAST_PENTIUMXEON;

    Val = edx & 0xFF000000;
    if (Val == 0x40000000)
	return IAST_CELERON;
    if ((Val >= 0x44000000) && (Val <= 0x45000000))
	return IAST_PENTIUMXEON;

    Val = edx & 0xFF0000;
    if (Val == 0x400000)
	return IAST_CELERON;
    if ((Val >= 0x440000) && (Val <= 0x450000))
	return IAST_PENTIUMXEON;

    Val = edx & 0xFF00;
    if (Val == 0x4000)
	return IAST_CELERON;
    if ((Val >= 0x4400) && (Val <= 0x4500))
	return IAST_PENTIUMXEON;

    Val = edx & 0xFF;
    if (Val == 0x40)
	return IAST_CELERON;
    if ((Val >= 0x44) && (Val <= 0x45))
	return IAST_PENTIUMXEON;

    return -1;
}

/* 
 * Decode Intel TLB and cache info descriptors 
 */
static void IntelDecodeTLB (CpuInfo_t *CpuInfo, int x)
{
    switch (x & 0xff){
    case 0:
	break;
    case 0x1:
	if (!CpuInfo->L1iCache)
	    CpuInfo->L1iCache = CacheInfoNew(NULL);
	CpuInfo->L1iCache->TLB = 32;
	AddDesc(&CpuInfo->Features, "Instruction TLB",
		"4KB pages, 4-way set assoc", 0);
	break;
    case 0x2:
	if (!CpuInfo->L1iCache)
	    CpuInfo->L1iCache = CacheInfoNew(NULL);
	CpuInfo->L1iCache->TLB = 2;
	AddDesc(&CpuInfo->Features, "Instruction TLB",
		"4MB pages, fully assoc", 0);
	break;
    case 0x3:
	if (!CpuInfo->L1dCache)
	    CpuInfo->L1dCache = CacheInfoNew(NULL);
	CpuInfo->L1dCache->TLB = 64;
	AddDesc(&CpuInfo->Features, "Data TLB",
		"4KB pages, 4-way set assoc", 0);
	break;
    case 0x4:
	if (!CpuInfo->L1dCache)
	    CpuInfo->L1dCache = CacheInfoNew(NULL);
	CpuInfo->L1dCache->TLB = 8;
	AddDesc(&CpuInfo->Features, "Data TLB",
		"4MB pages, 4-way set assoc", 0);
	break;
    case 0x6:
	if (!CpuInfo->L1iCache)
	    CpuInfo->L1iCache = CacheInfoNew(NULL);
	CpuInfo->L1iCache->Size = 8 * KBYTES;
	CpuInfo->L1iCache->Assoc = 4;
	CpuInfo->L1iCache->LineSize = 32;
	break;
    case 0x8:
	if (!CpuInfo->L1iCache)
	    CpuInfo->L1iCache = CacheInfoNew(NULL);
	CpuInfo->L1iCache->Size = 16 * KBYTES;
	CpuInfo->L1iCache->Assoc = 4;
	CpuInfo->L1iCache->LineSize = 32;
	break;
    case 0xA:
	if (!CpuInfo->L1dCache)
	    CpuInfo->L1dCache = CacheInfoNew(NULL);
	CpuInfo->L1dCache->Size = 8 * KBYTES;
	CpuInfo->L1dCache->Assoc = 2;
	CpuInfo->L1dCache->LineSize = 32;
	break;
    case 0xC:
	if (!CpuInfo->L1dCache)
	    CpuInfo->L1dCache = CacheInfoNew(NULL);
	CpuInfo->L1dCache->Size = 16 * KBYTES;
	CpuInfo->L1dCache->Assoc = 2;
	CpuInfo->L1dCache->LineSize = 32;
	break;
    case 0x40:
	AddDesc(&CpuInfo->Features, "L2 unified cache",
		"No L2 cache", 0);
	break;
    case 0x41:
	if (!CpuInfo->L2Cache)
	    CpuInfo->L2Cache = CacheInfoNew(NULL);
	CpuInfo->L2Cache->Size = 128 * KBYTES;
	CpuInfo->L2Cache->Assoc = 4;
	CpuInfo->L2Cache->LineSize = 32;
	break;
    case 0x42:
	if (!CpuInfo->L2Cache)
	    CpuInfo->L2Cache = CacheInfoNew(NULL);
	CpuInfo->L2Cache->Size = 256 * KBYTES;
	CpuInfo->L2Cache->Assoc = 4;
	CpuInfo->L2Cache->LineSize = 32;
	break;
    case 0x43:
	if (!CpuInfo->L2Cache)
	    CpuInfo->L2Cache = CacheInfoNew(NULL);
	CpuInfo->L2Cache->Size = 512 * KBYTES;
	CpuInfo->L2Cache->Assoc = 4;
	CpuInfo->L2Cache->LineSize = 32;
	break;
    case 0x44:
	if (!CpuInfo->L2Cache)
	    CpuInfo->L2Cache = CacheInfoNew(NULL);
	CpuInfo->L2Cache->Size = 1 * MBYTES;
	CpuInfo->L2Cache->Assoc = 4;
	CpuInfo->L2Cache->LineSize = 32;
	break;
    case 0x45:
	if (!CpuInfo->L2Cache)
	    CpuInfo->L2Cache = CacheInfoNew(NULL);
	CpuInfo->L2Cache->Size = 2 * MBYTES;
	CpuInfo->L2Cache->Assoc = 4;
	CpuInfo->L2Cache->LineSize = 32;
	break;
    case 0x50:
	if (!CpuInfo->L1iCache)
	    CpuInfo->L1iCache = CacheInfoNew(NULL);
	CpuInfo->L1iCache->TLB = 64;
	AddDesc(&CpuInfo->Features, "Instruction TLB",
		"4K, 2M or 4M pages, fully associative", 0);
	break;
    case 0x51:
	if (!CpuInfo->L1iCache)
	    CpuInfo->L1iCache = CacheInfoNew(NULL);
	CpuInfo->L1iCache->TLB = 128;
	AddDesc(&CpuInfo->Features, "Instruction TLB",
		"4K, 2M or 4M pages, fully associative", 0);
	break;
    case 0x52:
	if (!CpuInfo->L1iCache)
	    CpuInfo->L1iCache = CacheInfoNew(NULL);
	CpuInfo->L1iCache->TLB = 256;
	AddDesc(&CpuInfo->Features, "Instruction TLB",
		"4K, 2M or 4M pages, fully associative", 0);
	break;
    case 0x5B:
	if (!CpuInfo->L1iCache)
	    CpuInfo->L1iCache = CacheInfoNew(NULL);
	CpuInfo->L1iCache->TLB = 64;
	AddDesc(&CpuInfo->Features, "Data TLB",
		"4K or 4M pages, fully associative", 0);
	break;
    case 0x5C:
	if (!CpuInfo->L1iCache)
	    CpuInfo->L1iCache = CacheInfoNew(NULL);
	CpuInfo->L1iCache->TLB = 128;
	AddDesc(&CpuInfo->Features, "Data TLB",
		"4K or 4M pages, fully associative", 0);
	break;
    case 0x5D:
	if (!CpuInfo->L1iCache)
	    CpuInfo->L1iCache = CacheInfoNew(NULL);
	CpuInfo->L1iCache->TLB = 256;
	AddDesc(&CpuInfo->Features, "Data TLB",
		"4K or 4M pages, fully associative", 0);
	break;
    case 0x66:
	AddDesc(&CpuInfo->Features, "Data Cache",
		"Sectored, 64 byte cache line, 4 way set associative, 8K", 0);
	break;
    case 0x67:
	AddDesc(&CpuInfo->Features, "Data Cache",
		"Sectored, 64 byte cache line, 4 way set associative, 16K", 0);
	break;
    case 0x68:
	AddDesc(&CpuInfo->Features, "Data Cache",
		"Sectored, 64 byte cache line, 4 way set associative, 32K", 0);
	break;
    case 0x70:
	AddDesc(&CpuInfo->Features, "Instruction Trace Cache",
		"4 way set associative, 12K uOps", 0);
	break;
    case 0x71:
	AddDesc(&CpuInfo->Features, "Instruction Trace Cache",
		"4 way set associative, 16K uOps", 0);
	break;
    case 0x72:
	AddDesc(&CpuInfo->Features, "Instruction Trace Cache",
		"4 way set associative, 32K uOps", 0);
	break;
    case 0x7A:
	if (!CpuInfo->L2Cache)
	    CpuInfo->L2Cache = CacheInfoNew(NULL);
	CpuInfo->L2Cache->Size = 256 * KBYTES;
	CpuInfo->L2Cache->Assoc = 8;
	CpuInfo->L2Cache->LineSize = 64;
	break;
    case 0x7B:
	if (!CpuInfo->L2Cache)
	    CpuInfo->L2Cache = CacheInfoNew(NULL);
	CpuInfo->L2Cache->Size = 512 * KBYTES;
	CpuInfo->L2Cache->Assoc = 8;
	CpuInfo->L2Cache->LineSize = 64;
	break;
    case 0x7C:
	if (!CpuInfo->L2Cache)
	    CpuInfo->L2Cache = CacheInfoNew(NULL);
	CpuInfo->L2Cache->Size = 1 * MBYTES;
	CpuInfo->L2Cache->Assoc = 8;
	CpuInfo->L2Cache->LineSize = 64;
	break;
    case 0x82:
	if (!CpuInfo->L2Cache)
	    CpuInfo->L2Cache = CacheInfoNew(NULL);
	CpuInfo->L2Cache->Size = 256 * KBYTES;
	CpuInfo->L2Cache->Assoc = 8;
	CpuInfo->L2Cache->LineSize = 32;
	break;
    case 0x83:
	if (!CpuInfo->L2Cache)
	    CpuInfo->L2Cache = CacheInfoNew(NULL);
	CpuInfo->L2Cache->Size = 512 * KBYTES;
	CpuInfo->L2Cache->Assoc = 8;
	CpuInfo->L2Cache->LineSize = 32;
	break;
    case 0x84:
	if (!CpuInfo->L2Cache)
	    CpuInfo->L2Cache = CacheInfoNew(NULL);
	CpuInfo->L2Cache->Size = 1 * MBYTES;
	CpuInfo->L2Cache->Assoc = 8;
	CpuInfo->L2Cache->LineSize = 32;
	break;
    case 0x85:
	if (!CpuInfo->L2Cache)
	    CpuInfo->L2Cache = CacheInfoNew(NULL);
	CpuInfo->L2Cache->Size = 1 * MBYTES;
	CpuInfo->L2Cache->Assoc = 8;
	CpuInfo->L2Cache->LineSize = 32;
	break;

    default:
	SImsg(SIM_DBG, "IntelDecodeTLB: TLB/cache descriptor: 0x%X", x);
	break;
    }
}

/*
 * Get the Intel CPU Model name
 */
static char *IntelModel(CpuInfo_t *CpuInfo, int family, int extfamily,
			int model, int type, int SubType)
{
    static char			Buff[128];
    char		       *Arch = NULL;
    char		       *ArchVer = NULL;
    int				BitSize = 32;

    SImsg(SIM_DBG, "IntelModel: Family=%d Model=%d Type=%d", 
	  family, model, type);

    /*
     * Some of this comes from Linux: /usr/src/linux/arch/i386/setup.c
     */
    switch (family) {
    case 0:
	strcpy(Buff, "8086/8088");
	break;
    case 2:
	strcpy(Buff, "i286");
	Arch = "i286";
	ArchVer = "IA16";
	BitSize = 16;
	break;
    case 3:
	strcpy(Buff, "i386");
	Arch = "i386";
	ArchVer = "IA32";
	break;
    case 4:
	Arch = "i486";
	ArchVer = "IA32";
	strcpy(Buff, "i486");
	switch (model){
	case 0:
	case 1:
	    strcat(Buff, " DX-25/33");
	    break;
	case 2:
	    strcat(Buff, " SX");
	    break;
	case 3:
	    strcat(Buff, " DX2");
	    break;
	case 4:
	    strcat(Buff, " SL");
	    break;
	case 5:
	    strcat(Buff, " SX2");
	    break;
	case 7:
	    strcat(Buff, " DX2WB");
	    break;
	case 8:
	    strcat(Buff, " DX4");
	    break;
	case 9:
	    strcat(Buff, " DX4WB");
	    break;
	}
	break;
    case 5:
	Arch = "i586";
	ArchVer = "IA32";
	strcpy(Buff, "Pentium");
	switch (model){
	case 0:
	    strcat(Buff, " 60/66 A-step");
	    break;
	case 1:
	    strcat(Buff, " 60/66");
	    break;
	case 2:
	    strcat(Buff, " 75-200");
	    break;
	case 3:
	    strcat(Buff, " for 486 system");
	    break;
	case 4:
	    strcat(Buff, " MMX");
	    break;
	case 7:
	    strcat(Buff, " 75-200 Mobile");
	    break;
	case 8:
	    strcat(Buff, " MMX Mobile");
	    break;
	}
	break;
    case 6:
	Arch = "i686";
	ArchVer = "IA32";
	strcpy(Buff, "Pentium II/Pro");
	switch (model) {
	case 0:
	    strcpy(Buff, "Pentium Pro A-step");
	    break;
	case 1:
	    strcpy(Buff, "Pentium Pro");
	    break;
	case 3:
	    strcpy(Buff, "Pentium II (Klamath)");
	    break;
	case 5:
	    switch (SubType) {
	    case IAST_CELERON:
		strcpy(Buff, "Celeron Model 5");
		break;
	    case IAST_PENTIUMXEON:
		strcpy(Buff, "Pentium II Xeon");
		break;
	    default:
		strcpy(Buff, "Pentium II (Deschutes)");
		break;
	    }
	    break;
	case 6:
	    strcpy(Buff, "Pentium II Mobile");
	    break;
	case 7:
	    switch (SubType) {
	    case IAST_CELERON:
		strcpy(Buff, "Celeron Model 7");
		break;
	    case IAST_PENTIUMXEON:
		strcpy(Buff, "Pentium III Xeon");
		break;
	    default:
		strcpy(Buff, "Pentium III (Katmai)");
		break;
	    }
	    break;
	case 8:
	    switch (SubType) {
	    case IAST_CELERON:
		strcpy(Buff, "Celeron Model 8");
		break;
	    case IAST_PENTIUMXEON:
		strcpy(Buff, "Pentium III Xeon");
		break;
	    default:
		strcpy(Buff, "Pentium III (Coppermine)");
		break;
	    }
	    break;
	case 10:
	    switch (SubType) {
	    case IAST_CELERON:
		strcpy(Buff, "Celeron Model 10");
		break;
	    case IAST_PENTIUMXEON:
		strcpy(Buff, "Pentium III Xeon");
		break;
	    default:
		strcpy(Buff, "Pentium III");
		break;
	    }
	    break;
	default:
	    SImsg(SIM_DBG, 
		  "IntelModel: Model %d is unknown member of family %d",
		  model, family);
	    /*
	     * Make a guess
	     */
	    if (model >= 11)
		strcpy(Buff, "Pentium III");
	}
	break;
    case 0xF:
	/* Look in extended family */
	switch (extfamily) {
	case 1:
	    strcpy(Buff, "Celeron");
	    Arch = "i686";
	    ArchVer = "IA32";
	    break;
	case 2:
	    Arch = "i686";
	    ArchVer = "IA32";
	    strcpy(Buff, "Pentium III");
	    break;
	case 3:
	    Arch = "i686";
	    ArchVer = "IA32";
	    strcpy(Buff, "Pentium III Xeon");
	    break;
	case 8:
	    Arch = "i786";
	    ArchVer = "IA32";
	    strcpy(Buff, "Pentium 4");
	    break;
	case 9:		/* 0x9 */
	    Arch = "i786";
	    ArchVer = "IA32";
	    strcpy(Buff, "Pentium 4");
	    break;
	case 11:	/* 0xB */
	    Arch = "i786";
	    ArchVer = "IA32";
	    strcpy(Buff, "Xeon");
	    break;
	case 14:
	    Arch = "i786";
	    ArchVer = "IA32";
	    strcpy(Buff, "Pentium 4 Xeon");
	    break;
	default:
	    SImsg(SIM_DBG, 
		  "IntelModel: Unknown Extended Family Value 0x%X",
		  extfamily);
	}
	break;
    default:
	SImsg(SIM_DBG, "IntelModel: Unknown family value 0x%X", family);
    }
    CpuInfo->BitSize = BitSize;

    SImsg(SIM_DBG, "IntelModel: Model=<%s> Arch=<%s> ArchVer=<%s> Bits=%d", 
	  PRTS(Buff), PRTS(Arch), PRTS(ArchVer), CpuInfo->BitSize);

    if (Arch && !CpuInfo->Arch)
	CpuInfo->Arch = Arch;
    if (ArchVer && !CpuInfo->ArchVer)
	CpuInfo->ArchVer = ArchVer;

    return (Buff[0]) ? Buff : (char *) NULL;
}

/*
 * Intel-specific information 
 */
void IntelInfo(CpuInfo_t *CpuInfo, int maxi)
{
    register int 		i;
    unsigned long 		eax,ebx,ecx,edx;
    int 			stepping,model,family,type,reserved,ntlb;
    int				extfamily;
    int				SubType;
    static char			Buff[128];

    SImsg(SIM_DBG, "GetCpuIntel: Intel-specific function");

    if (maxi >= 1) {
	/* Family/model/type etc */
	cpuid(1, &eax, &ebx, &ecx, &edx);
	stepping 	= eax & 0xf;
	model 		= (eax >> 4) & 0xf;
	family 		= (eax >> 8) & 0xf;
	type 		= (eax >> 12) & 0x3;
	reserved 	= (eax >> 14);
	extfamily	= (ebx & 0xff);

	CpuInfo->Stepping = stepping;

	if (family == 5 || family == 6) {
	    /* Some kind of Pentium II/III Xenon or Celeron */
	    SubType = IntelSubType(eax, ebx, ecx, edx);
	    SImsg(SIM_DBG, "IntelSubType is %d", SubType);
	}
	if (!CpuInfo->Model)
	    CpuInfo->Model = IntelModel(CpuInfo, family, extfamily,
					model, type, SubType);

	switch (type){
	case 0:
	    AddDesc(&CpuInfo->Features, NULL, "Original OEM", 0);
	    break;
	case 1:
	    AddDesc(&CpuInfo->Features, NULL, "Overdrive", 0);
	    break;
	case 2:
	    AddDesc(&CpuInfo->Features, NULL, "Dual-capable", 0);
	    break;
	case 3:
	    /* Reserved */
	    break;
	}

	/* Feature Flags */
	if (edx & (1<<0)) {
	    CpuInfo->HasFPU = TRUE;
	}
	if (edx & (1<<1))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "VME - Virtual 8086 Mode Enhancements", 0);
	if (edx & (1<<2))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "DE - Debugging Extensions", 0);
	if (edx & (1<<3))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "PSE - Page Size Extensions", 0);
	if (edx & (1<<4))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "TSC - Time Stamp Counter", 0);
	if (edx & (1<<5))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "MSR - Model Specific Registers", 0);
	if (edx & (1<<6))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "PAE - Physical Address Extension", 0);
	if (edx & (1<<7))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "MCE - Machine Check Exception", 0);
	if (edx & (1<<8))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "CX8 - COMPXCHG8B Instruction", 0);
	if (edx & (1<<9))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "APIC - On-chip Advanced Programmable Interrupt Controller present and enabled", 0);

	if (edx & (1<<11))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "SEP - Fast System Call", 0);
	if (edx & (1<<12))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "MTRR   Memory Type Range Registers", 0);
	if (edx & (1<<13))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "PGE - PTE Global Flag", 0);
	if (edx & (1<<14))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "MCA - Machine Check Architecture", 0);
	if (edx & (1<<15))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "CMOV - Conditional Move and Compare Instructions", 0);
	if (edx & (1<<16))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "FGPAT - Page Attribute Table", 0);
	if (edx & (1<<17))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "PSE - 36-bit Page Size Extension", 0);
	if (edx & (1<<18))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "PN - Processor Serial Number present and enabled", 0);

	if (edx & (1<<23))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "MMX - MMX instruction set", 0);
	if (edx & (1<<24))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "FXSR - Fast FP/MMX Streaming SIMD Extensions save/restore", 0);
	if (edx & (1<<25))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "XMM - Streaming SIMD Extensions instruction set", 0);
    }
    
    if (maxi >= 2){
	/* Decode TLB and cache info */
	ntlb = 255;
	for(i = 0; i< ntlb; i++){
	    cpuid(2, &eax, &ebx, &ecx, &edx);
	    ntlb =  eax & 0xff;
	    IntelDecodeTLB(CpuInfo, eax >> 8);
	    IntelDecodeTLB(CpuInfo, eax >> 16);
	    IntelDecodeTLB(CpuInfo, eax >> 24);
	    
	    if ((ebx & 0x80000000) == 0){
		IntelDecodeTLB(CpuInfo, ebx);
		IntelDecodeTLB(CpuInfo, ebx >> 8);
		IntelDecodeTLB(CpuInfo, ebx >> 16);
		IntelDecodeTLB(CpuInfo, ebx >> 24);
	    }
	    if ((ecx & 0x80000000) == 0){
		IntelDecodeTLB(CpuInfo, ecx);
		IntelDecodeTLB(CpuInfo, ecx >> 8);
		IntelDecodeTLB(CpuInfo, ecx >> 16);
		IntelDecodeTLB(CpuInfo, ecx >> 24);
	    }
	    if ((edx & 0x80000000) == 0){
		IntelDecodeTLB(CpuInfo, edx);
		IntelDecodeTLB(CpuInfo, edx >> 8);
		IntelDecodeTLB(CpuInfo, edx >> 16);
		IntelDecodeTLB(CpuInfo, edx >> 24);
	    }
	}
    }

    if (maxi >= 3){
	/* Pentium III and later CPU serial number */
	unsigned long 		signature;
	cpuid(1, &eax, NULL, NULL, NULL);
	signature = eax;
	
	cpuid(3, &eax, &ebx, &ecx, &edx);
	(void) snprintf(Buff, sizeof(Buff), "%04X-%04X-%04X-%04X-%04X-%04X",
			signature >> 16,
			signature & 0xffff,
			edx >> 16,
			edx & 0xffff,
			ecx >> 16,
			ecx & 0xffff);
	SImsg(SIM_DBG, "IntelInfo: Serial=<%s>", Buff);
	CpuInfo->Serial = strdup(Buff);
    }
}

/*
 * Get and set AMD Model info
 */
static char *AMDmodel(CpuInfo_t *CpuInfo, int family, int model)
{
    static char			Buff[128];
    char		       *Arch = NULL;
    char		       *ArchVer = NULL;
    int				BitSize = 32;

    SImsg(SIM_DBG, "AMDmodel: family=%d Model=%d", family, model);

    Buff[0] = CNULL;
    switch (family){
    case 4:
	strcpy(Buff, "486");
	Arch = "i486";
	ArchVer = "IA32";
	break;
    case 5:
	Arch = "i586";
	ArchVer = "IA32";
	/* K6 */
	switch (model) {
	case 0:
	case 1:
	case 2:
	case 3:
	    strcpy(Buff, "K5");
	    break;
	case 4: /* Not used */
	case 5: /* Not used */
	case 6:
	case 7:
	    strcpy(Buff, "K6");
	    break;
	case 8:
	    strcpy(Buff, "K6-2");
	    break;
	case 9:
	    strcpy(Buff, "K6-III");
	    break;
	case 10:
	case 11:
	case 12:
	case 13:
	case 14:
	case 15:
	    strcpy(Buff, "K6-3");
	    break;
	default:
	    SImsg(SIM_DBG, "AMDmodel: Family %d Model %d undefined.",
		  family, model);
	}
	break;
    case 6:
	Arch = "i686";
	ArchVer = "IA32";
	switch (model) {
	case 1:
	case 2:
	case 4:
	    strcpy(Buff, "Athlon");
	    break;
	case 3:
	    strcpy(Buff, "Duron");
	    break;
	case 6:
	    strcpy(Buff, "Athlon MP/Mobile Athlon");
	    break;
	case 7:
	    strcpy(Buff, "Mobile Duron");
	    break;
	default:
	    SImsg(SIM_DBG, "AMDmodel: Family %d Model %d undefined.",
		  family, model);
	    strcpy(Buff, "Athlon/Duron");
	    break;
	}
    default:
 	SImsg(SIM_DBG, "AMDmodel: Family %d undefined.", family);
    }
    CpuInfo->BitSize = BitSize;

    SImsg(SIM_DBG, "AMDmodel: Family=<%s> Arch=<%s> ArchVer=<%s> Bits=%d", 
	  Buff, Arch, ArchVer, BitSize);

    if (Arch && !CpuInfo->Arch)
	CpuInfo->Arch = Arch;
    if (ArchVer && !CpuInfo->ArchVer)
	CpuInfo->ArchVer = ArchVer;

    return (Buff[0]) ? Buff : (char *) NULL;
}

/* 
 * AMD-specific information 
 */
void AMDInfo(CpuInfo_t *CpuInfo, int maxi)
{
    int 			i;
    int				extended;
    unsigned long 		maxei,eax,ebx,ecx,edx;
    int 			stepping,model,family,reserved,ntlb;
    static char			Buff[256];
    char		       *FamilyName = NULL;
    char		       *Model;
    char		       *cp;
    
    SImsg(SIM_DBG, "AMDInfo: Start");

    cpuid(0x80000000, &maxei, NULL, NULL, NULL);
    if (maxei != 0){
	/* Dump extended info in raw hex */
	for(i = 0x80000000; i<= maxei; i++){
	    cpuid(i, &eax, &ebx, &ecx, &edx);
	    SImsg(SIM_DBG, 
		  "eax in: 0x%x, eax = %08x ebx = %08x ecx = %08x edx = %08x",
		  i, eax, ebx, ecx, edx);
	}
    }
    
    /* Do standard stuff */
    if (maxi >= 1){
	cpuid(1, &eax, &ebx, &ecx, &edx);
	stepping 	= eax & 0xf;
	model 		= (eax >> 4) & 0xf;
	family 		= (eax >> 8) & 0xf;
	reserved 	= (eax >> 12);
	
	FamilyName = AMDmodel(CpuInfo, family, model);

	CpuInfo->Stepping = stepping;

	/* Standard feature flags */
	if (edx & (1<<0))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Floating Point Unit", 0);
	if (edx & (1<<1))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Virtual Mode Extensions", 0);
	if (edx & (1<<2))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Debugging Extensions", 0);
	if (edx & (1<<3))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Page Size Extensions", 0);
	if (edx & (1<<4))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Time Stamp Counter (with RDTSC and CR4 disable bit)", 0);
	if (edx & (1<<5))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Model Specific Registers", 0);

	if (edx & (1<<7))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Machine Check Exception", 0);
	if (edx & (1<<8))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "COMPXCHG8B Instruction", 0);
	if (edx & (1<<9)){
	    if (family == 5 && model == 0)
		AddDesc(&CpuInfo->Features, NULL, 
			"Global Paging Extensions", 0);
	    else
		AddDesc(&CpuInfo->Features, NULL, 
			"APIC - On-chip Advanced Programmable Interrupt Controller present and enabled", 0);
	}

	if (edx & (1<<12))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Memory Type Range Registers", 0);
	if (edx & (1<<13)){
	    if (!(family == 5 && model == 0))
		AddDesc(&CpuInfo->Features, NULL, 
			"Global Paging Extension", 0);
	}

	if (edx & (1<<15))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Conditional Move Instruction", 0);
	if (edx & (1<<23))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "MMX instructions", 0);
    }

    /* Check for presence of extended info */
    if (maxei == 0)
	return;
    
    /* Extended feature flags */
    if (maxei >= 0x80000001){
	cpuid(0x80000001, &eax, &ebx, &ecx, &edx);
	stepping = eax & 0xf;
	model = (eax >> 4) & 0xf;
	family = (eax >> 8) & 0xf;
	reserved = eax >> 12;

	if (!FamilyName)
	    FamilyName = AMDmodel(CpuInfo, family, model);

	CpuInfo->Stepping = stepping;

	if (edx & (1<<0))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Floating Point Unit", 0);
	if (edx & (1<<1))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Virtual Mode Extensions", 0);
	if (edx & (1<<2))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Debugging Extensions", 0);
	if (edx & (1<<3))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Page Size Extensions", 0);
	if (edx & (1<<4))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Time Stamp Counter (with RDTSC and CR4 disable bit)", 0);
	if (edx & (1<<5))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "K86 family Model Specific Registers with RDMSR & WRMSR", 0);

	if (edx & (1<<7))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Machine Check Exception", 0);
	if (edx & (1<<8))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "COMPXCHG8B Instruction", 0);
	if (edx & (1<<11))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Syscall and Sysret instructions", 0);
	if (edx & (1<<13))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Global paging extension", 0);
	if (edx & (1<<15))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Integer Conditional Move Instruction", 0);
	if (edx & (1<<16))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "Floating-point conditional move instructions", 0);
	if (edx & (1<<23))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "MMX instructions", 0);
	if (edx & (1<<31))
	    AddDesc(&CpuInfo->Features, NULL, 
		    "3DNow instructions", 0);
    }

    if (maxei >= 0x80000002){
	/* Processor identification string */
	char 		       *cp;
	int 			j;

	cp = Buff;
	for(j = 0x80000002; j <= 0x80000004; j++){
	    cpuid(j, &eax, &ebx, &ecx, &edx);
	    
	    for (i = 0; i < 4; i++)
		*cp++ = eax >> (8*i);
	    for (i = 0; i < 4; i++)
		*cp++ = ebx >> (8*i);
	    for (i = 0; i < 4; i++)
		*cp++ = ecx >> (8*i);
	    for (i = 0; i < 4; i++)
		*cp++ = edx >> (8*i);
	}
	*cp = CNULL;
	SImsg(SIM_DBG, "CPU Retrieved Model=<%s>", Buff);

	Model = CpuModelEdit(Buff);
	SImsg(SIM_DBG, "CPU Model=<%s>", Model);
	if (!CpuInfo->Model)
	    CpuInfo->Model = strdup(Model);
    }

    if (!CpuInfo->Model && FamilyName)
	CpuInfo->Model = strdup(FamilyName);
      
    if (maxei >= 0x80000005){
	/* TLB and cache info */
	cpuid(0x80000005, &eax, &ebx, &ecx, &edx);

	if (!CpuInfo->L1dCache)
	    CpuInfo->L1dCache = CacheInfoNew(NULL);
	if (!CpuInfo->L1iCache)
	    CpuInfo->L1iCache = CacheInfoNew(NULL);

	CpuInfo->L1dCache->TLB = (ebx >> 16) & 0xff;
	CpuInfo->L1dCache->TLBAssoc = ebx >> 24;

	CpuInfo->L1iCache->TLB = ebx & 0xff;
	CpuInfo->L1iCache->TLBAssoc = (ebx >> 8) & 0xf;

	CpuInfo->L1dCache->Size = (ecx >> 24) * KBYTES;
	CpuInfo->L1dCache->Assoc = (ecx >> 16) & 0xff;
	CpuInfo->L1dCache->Lines = (ecx >> 8) & 0xff;
	CpuInfo->L1dCache->LineSize = ecx & 0xff;

	CpuInfo->L1iCache->Size = (edx >> 24) * KBYTES;
	CpuInfo->L1iCache->Assoc = (edx >> 16) & 0xff;
	CpuInfo->L1iCache->Lines = (edx >> 8) & 0xff;
	CpuInfo->L1iCache->LineSize = edx & 0xff;
    }
    
    /* check K6-III (and later?) on-chip L2 cache size */
    if (maxei >= 0x80000006) {
	cpuid(0x80000006, &eax, &ebx, &ecx, &edx);
	if (!CpuInfo->L2Cache)
	    CpuInfo->L2Cache = CacheInfoNew(NULL);
	CpuInfo->L2Cache->Size = (ecx >> 16) * KBYTES;
	CpuInfo->L2Cache->Assoc = (ecx >> 12) & 0x0f;
	CpuInfo->L2Cache->Lines = (ecx >> 8) & 0x0f;
	CpuInfo->L2Cache->LineSize = ecx & 0xff;
    }
}
#endif	/* IS_X86 && HAVE_CPUID */

/*
 * Use x86 instructions to get CPU Info
 */
extern Opaque_t GetCpuInfoX86() {
#if	defined(IS_X86) && defined(HAVE_CPUID)
    unsigned long 		maxi,eax,ebx,ecx,edx;
    static CpuInfo_t		CpuInfo;
    static char			Buff[256];
    char		       *Man;
    char		       *cp;
    register int		i;
    register int		n;
    
    SImsg(SIM_DBG, "GetCpuInfoX86() Start");
    
    if (CpuInfo.Model || CpuInfo.Man) 
	return &CpuInfo;
    
    /* Vendor ID and max CPUID level supported */
    cpuid(0, &eax, &ebx, &ecx, &edx);
    n = 0;
    for(i = 0; i < 4; i++)
	Buff[n++] = ebx >> (8*i);
    for(i = 0; i < 4; i++)
	Buff[n++] = edx >> (8*i);
    for(i = 0; i < 4; i++)
	Buff[n++] = ecx >> (8*i);
    Buff[n] = CNULL;

    Man = CpuManEdit(Buff);
    SImsg(SIM_DBG, "GetCpuInfoX86: Manufacturer=<%s>", Man);
    CpuInfo.Man = strdup(Man);
    
    cpuid(0, &maxi, NULL, NULL, NULL);
    AddDesc(&(CpuInfo.Features), "CPUID", itoa(maxi), 0);
    
    switch (ebx){
    case 0x756e6547: /* Intel */
	IntelInfo(&CpuInfo, maxi);
	break;
    case 0x68747541: /* AMD */
	AMDInfo(&CpuInfo, maxi);
	break;
    case 0x69727943: /* Cyrix */
	SImsg(SIM_DBG, "GetCpuInfoX86: Cyrix detected, but unsupported.");
	break;
    default:
	SImsg(SIM_DBG, "GetCpuInfoX86: Unknown CPU Manufacturer: 0x%X",
	      ebx);
	break;
    }

    return &CpuInfo;
#else	/* IS_X86 && HAVE_CPUID) */
    return (CpuInfo_t *) NULL;
#endif	/* IS_X86 && HAVE_CPUID */
}
