/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.37 $";
#endif

/*
 * Common SysInfo DMI functions
 */

#include "defs.h"
#include "dmi.h"

static Dmi_t		       *Dmi;

/*
 * Strings which indicate the value is too generic or not really set
 */
static char *NotSetList[] = {
    "System Manufacturer",
    "System Name",
    "System Version",
    "SYS-123",
    "xxxxxxx",
    "Chassis",
    "Asset-1234",
    "REV 1.xx",
    NULL
};

/*
 * Find entry in Array and return it.
 */
extern char *GetArrayEntry(Array, Entry)
     char		      **Array;
     int			Entry;
{
    register int		i;

    for (i = 0; Array[i]; ++i) 
	if (i == Entry)
	    return (Array[i][0]) ? Array[i] : (char *) NULL;

    return (char *) NULL;
}

/*
 * Does String indicate a generic (not set) value?
 */
extern int DmiIsGeneric(String)
     char		       *String;
{
    register int		i;

    if (strlen(String) == 1)
	return TRUE;

    for (i = 0; NotSetList[i]; ++i) 
	if (EQN(String, NotSetList[i], strlen(NotSetList[i])))
	    return TRUE;

    return FALSE;
}

/*
 * Lookup IntVal in Table and return associated string.
 */
extern char *GetNameTab(IntVal, Table)
     int			IntVal;
     NameTab_t		       *Table;
{
    NameTab_t		       *Tab;

    for (Tab = Table; Tab && Tab->name; ++Tab)
	if (Tab->value == IntVal)
	    return Tab->name;

    return (char *) NULL;
}

/*
 * Return a string from the DMI data at Ref.
 *
 * String sets are located after Hdr starting at Hdr->Length bytes.
 * Each string set is NULL terminated and the last string set is
 * followed by 2 NULL's.  Ref indicates which of the string sets
 * we want to retrieve.
 */
extern char *DmiString(Hdr, Ref)
     DmiHeader_t	       *Hdr;
     Byte_t			Ref;
{
    Byte_t		       *bp;
    Byte_t			r = Ref;

    if (Ref <= 0)
	return (char *) NULL;

    bp = (Byte_t *) Hdr;
    bp += Hdr->Length;		/* Position at start of string sets */
    while (r > 1) {
	bp += strlen(bp);	/* Advance to next end of string set */
	bp++;			/* Position at start of next string set */
	r--;
    }

    /* Extra debugging */
    SImsg(SIM_DBG, "\tDMI STRING: Type=%d Length=%d Ref=%d String=<%s>",
	  Hdr->Type, Hdr->Length, Ref, (char *) bp);

    /* Return newly scrubbed and alloc'ed version */
    return (bp && isprint(*bp)) ? 
	CleanString((char *)bp, strlen((char *)bp), 0) : (char *) NULL;
}

/*
 * Read in the DMI Table
 *
 * The allocated space is never freed so that future callers can use
 * the data.
 */
extern void *DmiGetTable(Dmi)
     Dmi_t		       *Dmi;
{
    DmiEps_t		       *Eps;
    char		       *Buffer;

    if (!Dmi || !Dmi->Eps) {
	SImsg(SIM_DBG, "DmiGetTable: Bad parameters");
	return (void *) NULL;
    }

    Eps = Dmi->Eps;

    if (lseek(Dmi->_FileDesc, EPS(Eps, TableAddr), 0) < 0) {
	SImsg(SIM_DBG, "%s: lseek to start of table at 0x%X failed: %s",
	      _PATH_DEV_MEM, EPS(Eps, TableAddr), SYSERR);
	return (void *) NULL;
    }

    Buffer = (char *) xcalloc(1, EPS(Eps, TableLen));
    if (read(Dmi->_FileDesc, Buffer, EPS(Eps, TableLen)) !=
	EPS(Eps,TableLen)) {
	SImsg(SIM_DBG, "%s: read %d bytes of DMI table at 0x%X failed: %s",
	      _PATH_DEV_MEM, EPS(Eps, TableLen), EPS(Eps, TableAddr), SYSERR);
	(void) free(Buffer);
	return (void *) NULL;
    }

    Dmi->Table = Buffer;

    return Dmi->Table;
}
	
/*
 * Find DMI entry with Handle and return its Data
 */
extern char *DmiFindHandle(Handle, Dmi)
     Word_t			Handle;
     Dmi_t		       *Dmi;
{
    register int		i;
    char		       *Table;
    char		       *Data;
    DmiHeader_t		       *Hdr;
    DmiEps_t		       *Eps;

    if (!Handle || !Dmi || !Dmi->Eps) {
	SImsg(SIM_DBG, "DmiFindHandle: Bad parameters");
	return (char *) NULL;
    }

    Eps = Dmi->Eps;

    Data = Table = DmiGetTable(Dmi);
    if (!Table)
	return (char *) NULL;

    for (i = 0; i < EPS(Eps, NumStructs); ++i) {
	Hdr = (DmiHeader_t *) Data;

	if (Hdr->Handle == Handle)
	    return Data;

	/* Skip to next header */
	Data += Hdr->Length;
	while (*Data || Data[1])
	    Data++;
	Data += 2;
    }

    return (char *) NULL;
}

/*
 * Decode DMI tables and add to TreePtr
 */
static int DmiDecode(Dmi, Query, TreePtr, SearchNames)
     Dmi_t		       *Dmi;
     DmiQuery_t		       *Query;
     DevInfo_t		      **TreePtr;
     char		      **SearchNames;
{
    register int		i;
    char		       *Table;
    char		       *Data;
    DmiHeader_t		       *Hdr;
    DmiEps_t		       *Eps;
    DevInfo_t		       *DevInfo;

    if (!Dmi || !Dmi->Eps) {
	SImsg(SIM_DBG, "DmiDecode: Bad parameters");
	return -1;
    }

    Eps = Dmi->Eps;

    Data = Table = DmiGetTable(Dmi);
    if (!Table)
	return -1;

    /* Make tree available for functions we call */
    if (!Dmi->_DevInfoTreePtr)
	Dmi->_DevInfoTreePtr = TreePtr;

    for (i = 0; i < EPS(Eps, NumStructs); ++i) {
	Hdr = (DmiHeader_t *) Data;
	if (!Query)
	    SImsg(SIM_DBG, "DMI: Handle=0x%04X Type=%d Length=%d (0x%X)",
		  Hdr->Handle, Hdr->Type, Hdr->Length, Hdr->Length);

	/*
	 * If this is a specific query, only decode if the Type's match
	 * If it's not a query, decode away.
	 */
	if (!(Query && Query->Type != Hdr->Type)) {
	    /* Get DevInfo from specific decode function then add it */
	    DevInfo = NULL;
	    switch (Hdr->Type) {
		/* These values are handled by other DmiDecode*() functions */
	    case DMI_STRUCT_CACHE:
		break;
	    case DMI_STRUCT_BIOS:
		DevInfo = DmiDecodeBios(Hdr, Data, Dmi, Query);
		break;
	    case DMI_STRUCT_SYSINFO:
		DevInfo = DmiDecodeSysInfo(Hdr, Data, Dmi, Query);
		break;
	    case DMI_STRUCT_BOARD:
		DevInfo = DmiDecodeBoard(Hdr, Data, Dmi, Query);
		break;
	    case DMI_STRUCT_MEMCTL:
		DevInfo = DmiDecodeMemctl(Hdr, Data, Dmi, Query);
		break;
	    case DMI_STRUCT_MEMMOD:
		DevInfo = DmiDecodeMemmod(Hdr, Data, Dmi, Query);
		break;
	    case DMI_STRUCT_MEMARRAY:
		DevInfo = DmiDecodeMemarray(Hdr, Data, Dmi, Query);
		break;
	    case DMI_STRUCT_MEMDEV:
		DevInfo = DmiDecodeMemdev(Hdr, Data, Dmi, Query);
		break;
	    case DMI_STRUCT_ENCLOSURE:
		DevInfo = DmiDecodeEnclosure(Hdr, Data, Dmi, Query);
		break;
	    case DMI_STRUCT_PROC:
		DevInfo = DmiDecodeProc(Hdr, Data, Dmi, Query);
		break;
	    case DMI_STRUCT_PORT:
		DevInfo = DmiDecodePort(Hdr, Data, Dmi, Query);
		break;
	    case DMI_STRUCT_SLOT:
		DevInfo = DmiDecodeSlot(Hdr, Data, Dmi, Query);
		break;
	    case DMI_STRUCT_OEM:
		DevInfo = DmiDecodeOem(Hdr, Data, Dmi, Query);
		break;
	    case DMI_STRUCT_SYSCONFIG:
		DevInfo = DmiDecodeSysconfig(Hdr, Data, Dmi, Query);
		break;
	    default:
		SImsg(SIM_DBG, "DMI:\tType %d found", Hdr->Type);
	    }
	    if (!Query && DevInfo) {
		DevInfo->NodeID = Hdr->Handle;
		AddDevice(DevInfo, TreePtr, SearchNames);
	    }
	}

	/* Skip to next header */
	Data += Hdr->Length;
	while (*Data || Data[1])
	    Data++;
	Data += 2;
    }

    return 0;
}

/*
 * Detect devices via DMI
 */
extern Dmi_t *DmiGet()
{
    int				fd;
    long			fp;
    static DmiEps_t		Eps;
    static int			Tried;
    static unsigned char	Buffer[sizeof(DmiEps_t) + 1];
    register int		i;

    SImsg(SIM_DBG, "DMI: sizeof(DmiEps_t)=%d", sizeof(DmiEps_t));

    if (Dmi)
	return Dmi;

    if (Tried)	/* Already tried and failed */
	return (Dmi_t *) NULL;
    Tried = TRUE;

    /* 
     * Safety check 
     * DmiEps_t must be 2X SMBIOS_BLOCK_SIZE or we'll have buffer 
     * overrun problems.
     */
    if ((sizeof(DmiEps_t)) != (SMBIOS_BLOCK_SIZE * 2)) {
	SImsg(SIM_DBG, "DMI: sizeof(DmiEps_t)=%d != (SMBIOS_BLOCK_SIZE*2)=%d",
	      sizeof(DmiEps_t), (SMBIOS_BLOCK_SIZE * 2));
	return (Dmi_t *) NULL;
    }
    if (sizeof(Byte_t) != 1) {
	SImsg(SIM_DBG, "DMI: sizeof(Byte_t)=%d (needs to be 1)",
	      sizeof(Byte_t));
	return (Dmi_t *) NULL;
    }
    if (sizeof(Word_t) != 2) {
	SImsg(SIM_DBG, "DMI: sizeof(Word_t)=%d (needs to be 2)",
	      sizeof(Word_t));
	return (Dmi_t *) NULL;
    }
    if (sizeof(Dword_t) != 4) {
	SImsg(SIM_DBG, "DMI: sizeof(Dword_t)=%d (needs to be 4)",
	      sizeof(Dword_t));
	return (Dmi_t *) NULL;
    }

    fd = open(_PATH_DEV_MEM, O_RDONLY);
    if (fd < 0) {
	SImsg(SIM_DBG, "%s: DMI: Open for read failed: %s",
	      _PATH_DEV_MEM, SYSERR);
	return (Dmi_t *) NULL;
    }

    /*
     * Position at start of range to search
     */
    fp = SMBIOS_ADDR_START;
    if (lseek(fd, (off_t)fp, 0) < 0) {
	SImsg(SIM_DBG, "%s: lseek to SMBIOS start failed: %s",
	      _PATH_DEV_MEM, SYSERR);
	(void) close(fd);
	return (Dmi_t *) NULL;
    }

    Dmi = (Dmi_t *) xcalloc(1, sizeof(Dmi_t));
    if (Dmi)
	Dmi->_FileDesc = fd;

    SImsg(SIM_DBG, 
	  "DMI: Searching for start of Entry Point Structure (EPS)");

    /*
     * Search through the memory range in increments of SMBIOS_BLOCK_SIZE
     * in search of the Entry Point Structure (EPS) which tells us where
     * the real data is.
     */
    for ( ; fp < SMBIOS_ADDR_END; fp += SMBIOS_BLOCK_SIZE) {
	if (read(fd, Buffer, SMBIOS_BLOCK_SIZE) != SMBIOS_BLOCK_SIZE) {
	    SImsg(SIM_DBG, "%s: Read (looking for EPS) failed: %s",
		  _PATH_DEV_MEM, SYSERR);
	    (void) close(fd);
	    (void) free(Dmi);
	    Dmi = NULL;
	    return (Dmi_t *) NULL;
	}

	/* Some debug info */
	if (DebugOn && Buffer[0] == '_') {
	    for (i = 0; isprint(Buffer[i]); ++i);
	    if (i)
		SImsg(SIM_DBG, "DMI: Anchor=<%.*s>", i, Buffer);
	}

	/*
	 * Check to see if this is an EPS v2 or later
	 */
	if (memcmp(Buffer, "_SM_", 4) == 0) {
	    /*
	     * EPS v2 entry
	     */
	    /* Read in the remainer of the EPS */
	    if (read(fd, &Buffer[SMBIOS_BLOCK_SIZE], SMBIOS_BLOCK_SIZE) != 
		SMBIOS_BLOCK_SIZE) {
		SImsg(SIM_DBG, 
		      "%s: Read (looking for EPS 2nd half) failed: %s",
		      _PATH_DEV_MEM, SYSERR);
		(void) close(fd);
		(void) free(Dmi);
		Dmi = NULL;
		return (Dmi_t *) NULL;
	    }
	    /* Skip over this entry for future reads (if any) */
	    fp += SMBIOS_BLOCK_SIZE;

	    /* Copy the buffer to the Eps buffer */
	    copybytes(Buffer, sizeof(Buffer), &Eps.v2, sizeof(Eps.v2));
	    Eps.Version = 2;

	    SImsg(SIM_DBG, "DMI: EPS v2 Found at 0x%lX Length=%d",
		  fp, Eps.v2.Length);
	    if (Eps.v2.Length == 31) {
		/* See note in SMBIOS section 2.2.1 regarding this bug */
		SImsg(SIM_DBG, 
		      "DMI: EPS Length bug detected. Resetting to 32");
		Eps.v2.Length = 32;
	    }

	    SImsg(SIM_DBG, "DMI: SMBIOS %d.%d Revision %d",
		  Eps.v2.MajVer, Eps.v2.MinVer, Eps.v2.EPrev);
	    Dmi->SmbiosMajVer = Eps.v2.MajVer;
	    Dmi->SmbiosMinVer = Eps.v2.MinVer;
	    Dmi->SmbiosEpsRev = Eps.v2.EPrev;
	    /* Calculate SMBIOS Version for internal use */
	    Dmi->SmbiosVersion = (Dmi->SmbiosMajVer * 1000) + 
		(Dmi->SmbiosMinVer * 10);

	    SImsg(SIM_DBG, "DMI: SMBIOS Version=%d (for internal use)",
		  Dmi->SmbiosVersion);
	    SImsg(SIM_DBG, 
		  "DMI: Int Anchor=<%.*s> TabLen=%d TabAddr=0x%X",
		  (isprint((Eps.v2.InterAnchor)[0])) ?
		  sizeof(Eps.v2.InterAnchor) : 0, 
		  (isprint((Eps.v2.InterAnchor)[0])) ?
		  (char *)Eps.v2.InterAnchor : "", 
		   Eps.v2.TableLen, Eps.v2.TableAddr);
	    SImsg(SIM_DBG, "DMI: # Structures=%d BCDrev=%d",
		  Eps.v2.NumStructs, Eps.v2.BCDrev);

	    /* Set the real data */
	    Dmi->Eps = &Eps;
	    break;
	} else if (memcmp(Buffer, "_DMI_", 5) == 0) {
	    /*
	     * This is an EPS v1 entry
	     */
	    /* Read in the remainer of the EPS */
	    if (read(fd, &Buffer[SMBIOS_BLOCK_SIZE], SMBIOS_BLOCK_SIZE) != 
		SMBIOS_BLOCK_SIZE) {
		SImsg(SIM_DBG, 
		      "%s: Read (looking for EPS 2nd half) failed: %s",
		      _PATH_DEV_MEM, SYSERR);
		(void) close(fd);
		(void) free(Dmi);
		Dmi = NULL;
		return (Dmi_t *) NULL;
	    }
	    /* Skip over this entry for future reads (if any) */
	    fp += SMBIOS_BLOCK_SIZE;

	    /* Copy the buffer to the Eps buffer */
	    copybytes(Buffer, sizeof(Buffer), &Eps.v1, sizeof(Eps.v1));
	    Eps.Version = 1;

	    SImsg(SIM_DBG, "DMI: EPS v1 Found at 0x%lX",
		  fp);

	    SImsg(SIM_DBG, "DMI: SMBIOS %d.%d",
		  Eps.v1.MajVer, Eps.v1.MinVer);
	    Dmi->SmbiosMajVer = Eps.v1.MajVer;
	    Dmi->SmbiosMinVer = Eps.v1.MinVer;
	    /* Calculate SMBIOS Version for internal use */
	    Dmi->SmbiosVersion = (Dmi->SmbiosMajVer * 1000) + 
		(Dmi->SmbiosMinVer * 10);

	    SImsg(SIM_DBG, "DMI: SMBIOS Version=%d (for internal use)",
		  Dmi->SmbiosVersion);
	    SImsg(SIM_DBG, "DMI: TabLen=%d TabAddr=0x%lX",
		  Eps.v1.TableLen, Eps.v1.TableAddr);
	    SImsg(SIM_DBG, "DMI: # Structures=%d",
		  Eps.v1.NumStructs);

	    /* Set the real data */
	    Dmi->Eps = &Eps;
	    break;
	} else if (memcmp(Buffer, "$PnP", 4) == 0) {
 	    SImsg(SIM_DBG, "DMI: PNP BIOS available");
	    Dmi->PnpBios = 1;
	} else if (memcmp(Buffer, "_SYSID_", 7) == 0) {
 	    SImsg(SIM_DBG, "DMI: SYSID available");
	    Dmi->SysId = 1;
	}
    }

    /* Leave fd open for later callers */

    if (Dmi && Dmi->Eps)
	return Dmi;
    else {
	SImsg(SIM_DBG, "DMI: Failed to find DMI data");
	return (Dmi_t *) NULL;
    }
}

/*
 * Build device info based on DMI
 */
extern int BuildDmi(TreePtr, SearchNames)
     DevInfo_t		      **TreePtr;
     char		      **SearchNames;
{
    if (!Dmi)
	Dmi = DmiGet();

    if (!Dmi) {
	SImsg(SIM_DBG, "BuildDmi: No DMI data present.");
	return -1;
    }

    return DmiDecode(Dmi, NULL, TreePtr, SearchNames);
}

/*
 * Look for Query->Type in the DMI data
 */
static int DmiQuery(Query)
     DmiQuery_t		       *Query;
{
    if (!Dmi)
	Dmi = DmiGet();

    if (!Dmi) {
	SImsg(SIM_DBG, "DmiQuery: No DMI data present.");
	return -1;
    }

    return DmiDecode(Dmi, Query, NULL, NULL);
}

/*
 * Get System Model from DMI
 */
extern Opaque_t GetModelDmi()
{
    static DmiQuery_t		Query;

    memset(&Query, CNULL, sizeof(Query));
    Query.Type = DMI_STRUCT_SYSINFO;
    if (DmiQuery(&Query) != 0) {
	SImsg(SIM_DBG, "GetModelDmi: DmiQuery failed");
	return (Opaque_t) NULL;
    }

    if (!Query.ValStr2) {
	SImsg(SIM_DBG, "GetModelDmi: No model value found");
	return (Opaque_t) NULL;
    }

    if (DmiIsGeneric(Query.ValStr2)) {
	SImsg(SIM_DBG, "GetModelDmi: No model value set");
	return (Opaque_t) NULL;
    }

    return (Opaque_t) Query.ValStr2;
}

/*
 * Get CPU speed (in MHz) from DMI
 */
extern int DmiGetCpuSpeed()
{
    static DmiQuery_t		Query;
    static int			Speed;

    if (Speed)
	return Speed;

    memset(&Query, CNULL, sizeof(Query));
    Query.Type = DMI_STRUCT_PROC;
    if (DmiQuery(&Query) != 0) {
	SImsg(SIM_DBG, "DmiGetCpuSpeed: DmiQuery for DMI_STRUCT_PROC failed");
	return 0;
    }

    if (Query.ValInt1 <= 0) {
	SImsg(SIM_DBG, 
	      "DmiGetCpuSpeed: No CpuSpeed value found in DMI_STRUCT_PROC");
	return 0;
    }
    Speed = Query.ValInt1;

    SImsg(SIM_DBG, "DmiGetCpuSpeed: Speed=%d", Speed);

    return Speed;
}

/*
 * Get system serial from DMI type Type
 */
static char *GetSerialDmiType(Type, TypeStr)
     int			Type;
     char		       *TypeStr;
{
    static DmiQuery_t		Query;

    memset(&Query, CNULL, sizeof(Query));
    Query.Type = Type;
    if (DmiQuery(&Query) != 0) {
	SImsg(SIM_DBG, "GetSerialDmi: DmiQuery for %s failed", TypeStr);
	return (char *) NULL;
    }

    if (!Query.ValStr4) {
	SImsg(SIM_DBG, "GetSerialDmi: No serial value found in %s", TypeStr);
	return (char *) NULL;
    }

    if (DmiIsGeneric(Query.ValStr4)) {
	SImsg(SIM_DBG, "GetSerialDmi: No serial value set for %s", TypeStr);
	return (char *) NULL;
    }

    return (char *) Query.ValStr4;
}

/*
 * Get System Serial from DMI
 */
extern Opaque_t GetSerialDmi()
{
    char		       *Serial = NULL;

    if (Serial = GetSerialDmiType(DMI_STRUCT_SYSINFO, "DMI_STRUCT_SYSINFO"))
	return (Opaque_t) Serial;

    if (Serial = GetSerialDmiType(DMI_STRUCT_ENCLOSURE, 
				  "DMI_STRUCT_ENCLOSURE"))
	return (Opaque_t) Serial;

    return (Opaque_t) NULL;
}

/*
 * Get System Manufacturer (long) from DMI
 */
extern Opaque_t GetManLongDmi()
{
    static DmiQuery_t		Query;

    memset(&Query, CNULL, sizeof(Query));
    Query.Type = DMI_STRUCT_SYSINFO;
    if (DmiQuery(&Query) != 0) {
	SImsg(SIM_DBG, "GetManLongDmi: DmiQuery failed");
	return (Opaque_t) NULL;
    }

    if (!Query.ValStr1) {
	SImsg(SIM_DBG, "GetManLongDmi: No Manufacturer value found");
	return (Opaque_t) NULL;
    }

    if (DmiIsGeneric(Query.ValStr1)) {
	SImsg(SIM_DBG, "GetManLongDmi: No Manufacturer value set");
	return (Opaque_t) NULL;
    }

    return (Opaque_t) Query.ValStr1;
}

/*
 * Get System Manufacturer (short) from DMI
 */
extern Opaque_t GetManShortDmi()
{
    char		       *Man;
    char		       *cp;

    cp = (char *) GetManLongDmi();
    if (!cp)
	return (Opaque_t) NULL;

    Man = strdup(cp);

    if (cp = strchr(Man, ' '))
	*cp = CNULL;

    return (Opaque_t) Man;
}

/*
 * Get Memory from DMI
 */
extern Opaque_t GetMemoryDmi()
{
    int				MemSize = 0;
    static char		       *MemStr = NULL;
    char		       *Data;
    DmiHeader_t		       *Hdr;
    static DmiQuery_t		Query;
    DmiEps_t		       *Eps;
    DevInfo_t		       *DevInfo;
    register int		i;
    int				FoundMemmod = 0;

    if (MemStr)
	return (Opaque_t) MemStr;

    SImsg(SIM_DBG, "GetMemoryDmi: Searching DMI for memory modules");

    if (!Dmi)
	Dmi = DmiGet();
    if (!Dmi || !Dmi->Eps) {
	SImsg(SIM_DBG, "GetMemoryDmi: Cannot find DMI information.");
	return (Opaque_t) NULL;
    }

    Eps = Dmi->Eps;

    Data = DmiGetTable(Dmi);
    if (!Data)
	return (Opaque_t) NULL;

    for (i = 0; i < EPS(Eps, NumStructs); ++i) {
	Hdr = (DmiHeader_t *) Data;

	/*
	 * It's common for both MEMMOD and MEMDEV data to be present.
	 * Since they both indicate the same data, we need to avoid
	 * adding both sets of data to MemSize.
	 */
	if (Hdr->Type == DMI_STRUCT_MEMMOD) {
	    memset(&Query, CNULL, sizeof(Query));
	    Query.Type = DMI_STRUCT_MEMMOD;
	    (void) DmiDecodeMemmod(Hdr, Data, Dmi, &Query);
	    MemSize += Query.ValInt1;
	    if (Query.ValInt1)
		/* Indicate we shouldn't also add in MEMDEV data */
		FoundMemmod = 1;
	} else if (Hdr->Type == DMI_STRUCT_MEMDEV && !FoundMemmod) {
	    memset(&Query, CNULL, sizeof(Query));
	    Query.Type = DMI_STRUCT_MEMDEV;
	    (void) DmiDecodeMemdev(Hdr, Data, Dmi, &Query);
	    MemSize += Query.ValInt1;
	}

	/* Skip to next header */
	Data += Hdr->Length;
	while (*Data || Data[1])
	    Data++;
	Data += 2;
    }

    if (MemSize)
	MemStr = GetSizeStr((Large_t) MemSize, (Large_t)MBYTES, 0);

    SImsg(SIM_DBG, "GetMemoryDmi: Size=%d Memory=<%s>",
	  MemSize, PRTS(MemStr));

    return (Opaque_t) MemStr;
}
