/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.8 $";
#endif

#include "defs.h"
#include "dmi.h"

static char		       *s = "Supports";
static DevDesc_t	      **lp;

/*
 * Values for Cache ECC Type (DmiStructCache_t)
 * See section 3.3.8.2 of SMBIOS Ref Spec.
 */
static char *EccTypes[] = {
    "",
    "",
    "",
    "Parity",
    "Single-bit ECC",
    "Multi-bit ECC",
    NULL
};

/*
 * Values for Cache Associativity (DmiStructCache_t)
 * See section 3.3.8.4 of SMBIOS Ref Spec.
 */
static char *AssocTypes[] = {
    "",
    "",
    "",
    "Direct Mapped",
    "2-way Set",
    "4-way Set",
    "Fully",
    NULL
};

/*
 * Return the size of a Dmi Cache in bytes
 */
static Large_t DmiCacheSize(Size, Step)
     Word_t			Size;
     Word_t			Step;
{
    Large_t			Amount = 0;

    /* Per DMI Spec */
    if (Step == (Word_t)0)
	Amount = (Large_t) ((Large_t)Size * 1 * KBYTES);
    else if (Step == (Word_t)1)
	Amount = (Large_t) ((Large_t)Size * 64 * KBYTES);
    else /* Something is wrong */
	Amount = 0;

    SImsg(SIM_DBG, "DmiCacheSize: Size=%d Step=%d Result=%d", 
	  Size, Step, Amount);

    return Amount;
}

/*
 * Find and decode DMI Cache info (DMI_STRUCT_CACHE) which has
 * handle of Handle.
 */
extern int DmiGetCache(Handle, Level, Dmi, Query, DevInfo)
     Word_t			Handle;
     int			Level;
     Dmi_t		       *Dmi;
     DmiQuery_t		       *Query;
     DevInfo_t		       *DevInfo;
{
    DmiStructCache_t	       *Cache;
    DmiHeader_t		       *Hdr;
    CpuInfo_t		       *CpuInfo;
    CacheInfo_t		       *ci = NULL;
    Large_t			MaxSize;
    Large_t			InstSize;
    char		       *cp;

    if (!Handle || Handle == 0xFF)
	return -1;

    Cache = (DmiStructCache_t *) DmiFindHandle(Handle, Dmi);
    if (!Cache) {
	SImsg(SIM_DBG, "DmiGetCache: Cannot located cache with handle=0x%04X",
	      Handle);
	return -1;
    }
    Hdr = (DmiHeader_t *) Cache;
    cp = (char *) Cache;	/* Debugging aid */

    InstSize = DmiCacheSize(Cache->InstSize, Cache->InstSizeStep);
    MaxSize = DmiCacheSize(Cache->MaxSize, Cache->MaxSizeStep);

    SImsg(SIM_DBG, "DMI CACHE: Handle=0x%04X Level=%d Size=%d/%d Speed=%dns",
	  Handle, 
	  Cache->Level + 1, 
	  InstSize, MaxSize, Cache->Speed);
    SImsg(SIM_DBG, 
	  "DMI CACHE:\tsizeof(DmiStructCache_t)=%d (0x%X) Hdr->Length=%d (0x%X)",
	  sizeof(DmiStructCache_t), sizeof(DmiStructCache_t), 
	  Hdr->Length, Hdr->Length);

    /* XXX This isn't quite working right, so we skip setting data for now */
    return -1;

    if (DevInfo->DevSpec)
	CpuInfo = (CpuInfo_t *) DevInfo->DevSpec;
    else
	CpuInfo = (CpuInfo_t *) xcalloc(1, sizeof(CpuInfo_t));

    /*
     * Select and set ci
     */
    switch (Level) {
    case 1:	
	if (!(ci = CpuInfo->L1iCache))
	    ci = CpuInfo->L1iCache = CacheInfoNew(NULL);
	break;
    case 2:	
	if (!(ci = CpuInfo->L2Cache))
	    ci = CpuInfo->L2Cache = CacheInfoNew(NULL);
	break;
    default:
	SImsg(SIM_DBG, "DmiGetCache: Unknown cache level=%d", Level);
	return -1;
    }

    ci->Enabled = Cache->Enabled;
    ci->Socketed = Cache->Socketed;
    ci->Size = InstSize;
    if (cp = DmiString(Hdr, Cache->Socket))
	ci->Location = strdup(cp);

    /* Cache->Level is offset by +1 */
    ci->Level = Cache->Level + 1;

    switch (Cache->OpMode) {
    case 0:	ci->Mode = "Write Through";		break;
    case 1:	ci->Mode = "Write Back";		break;
    case 2:	ci->Mode = "Varies with Address";	break;
    }

    if (Cache->CurSramNonBurst)
	ci->SramType = "Non-Burst";
    else if (Cache->CurSramPipeBurst)
	ci->SramType = "Pipeline Burst";
    else if (Cache->CurSramSync)
	ci->SramType = "Synchronous";
    else if (Cache->CurSramAsync)
	ci->SramType = "Asynchronous";
    else if (!Cache->CurSramOther && !Cache->CurSramUnknown)
	SImsg(SIM_DBG, "DMI CACHE:\tCannot determine SRAM Type");

    if (Dmi->SmbiosVersion >= 2010 && Hdr->Length >= 0x0F) {
	ci->Speed = Cache->Speed;
	if (cp = GetArrayEntry(EccTypes, Cache->EccType))
	    ci->EccType = cp;

	switch (Cache->SysType) {
	case 0x00: /* Unset */
	case 0x01: /* Other */
	case 0x02: /* Unknown */
	    break;
	case 0x03:	ci->Type = CACHE_TYPE_INST;		break;
	case 0x04:	ci->Type = CACHE_TYPE_DATA;		break;
	case 0x05:	ci->Type = CACHE_TYPE_UNIFIED;		break;
	default:
	    SImsg(SIM_DBG, "DMI CACHE: Unknown System Cache Type=0x%X",
		  Cache->SysType);
	}
    }

    return 0;
}

