/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.7 $";
#endif

#include "defs.h"
#include "dmi.h"
#include <math.h>

static char		       *s = "Supports";
static DevDesc_t	      **lp;

/*
 * Values for Memory Controller Error Detection Methods (DmiStructMemctl_t)
 * See section 3.3.6.1 of SMBIOS Ref Spec.
 */
static char *ErrDetectMethods[] = {
    "",
    "",
    "",
    "8-bit Parity",
    "32-bit ECC",
    "64-bit ECC",
    "128-bit ECC",
    "CRC",
    NULL,
};

/*
 * Values for DmiStructMemctl_t.SupInter
 * See section 3.3.6.3 of SMBIOS Ref Spec.
 */
static char *InterleaveTypes[] = {
    "",
    "",
    "",
    "1-way",
    "2-way",
    "4-way",
    "8-way",
    "16-way",
    NULL,
};

/*
 * Decode DMI Memory Controller (DMI_STRUCT_MEMCTL)
 */
extern DevInfo_t *DmiDecodeMemctl(Hdr, Data, Dmi, Query)
     DmiHeader_t	       *Hdr;
     void		       *Data;
     Dmi_t		       *Dmi;
     DmiQuery_t		       *Query;
{
    DevInfo_t		       *DevInfo;
    DevInfo_t		       *Child;
    DevInfo_t		       *Last = NULL;
    DmiStructMemctl_t	       *Memctl;
    char		       *cp;
    char		       *ModData;
    static char			Buff[128];
    static int			Unit;
    Large_t			MaxModSize;
    Word_t			Handle;
    register int		i;

    Memctl = (DmiStructMemctl_t *) Data;
    MaxModSize = (Large_t) pow((double)2, (double)Memctl->MaxModSize);

    SImsg(SIM_DBG, 
	  "DMI MEMCTL: Handle=0x%04X Slots=%d MaxModSize=%d (%d MB)",
	  Hdr->Handle, Memctl->Slots, Memctl->MaxModSize, MaxModSize);
    SImsg(SIM_DBG, 
	  "DMI MEMCTL:\tsizeof(DmiStructMemctl_t)=%d (0x%X) Hdr->Length=%d (0x%X)",
	  sizeof(DmiStructMemctl_t), sizeof(DmiStructMemctl_t), 
	  Hdr->Length, Hdr->Length);

    /*
     * If this is a specific query, then set the values now and return
     */
    if (Query && Query->Type == Hdr->Type) {
	/* Return nothing for now */
	return DevInfo;
    }

    DevInfo = NewDevInfo(NULL);
    DevInfo->Type = DT_MEMCTL;
    DevInfo->Unit = Unit++;
    snprintf(Buff, sizeof(Buff), "memctl%d", DevInfo->Unit);
    DevInfo->Name = strdup(Buff);

    lp = &(DevInfo->DescList);

    AddDesc(lp, "# Memory Slots", itoa((int)Memctl->Slots), 0);

    if (cp = GetSizeStr(MaxModSize, MBYTES, 0))
	AddDesc(lp, "Max Supported/Module", cp, 0);
    if (cp = GetSizeStr(MaxModSize * (Large_t)Memctl->Slots, MBYTES, 0))
	AddDesc(lp, "Max Supported Memory", cp, 0);

    
    if (Memctl->SupSpeed70)	AddDesc(lp, "Supports Speed", "70 ns", 0);
    if (Memctl->SupSpeed60)	AddDesc(lp, "Supports Speed", "60 ns", 0);
    if (Memctl->SupSpeed50)	AddDesc(lp, "Supports Speed", "50 ns", 0);

    if (Memctl->TypeStandard)	AddDesc(lp, "Supports Type", "Standard", 0);
    if (Memctl->TypeFPM)	AddDesc(lp, "Supports Type", "FPM", 0);
    if (Memctl->TypeEDO)	AddDesc(lp, "Supports Type", "EDO", 0);
    if (Memctl->TypeParity)	AddDesc(lp, "Supports Type", "Parity", 0);
    if (Memctl->TypeECC)	AddDesc(lp, "Supports Type", "ECC", 0);
    if (Memctl->TypeSIMM)	AddDesc(lp, "Supports Type", "SIMM", 0);
    if (Memctl->TypeDIMM)	AddDesc(lp, "Supports Type", "DIMM", 0);
    if (Memctl->TypeBurstEDO)	AddDesc(lp, "Supports Type", "Burst-EDO", 0);
    if (Memctl->TypeSDRAM)	AddDesc(lp, "Supports Type", "SDRAM", 0);

    if (Memctl->Volt5)		AddDesc(lp, "Supports Voltage", "5v", 0);
    if (Memctl->Volt33)		AddDesc(lp, "Supports Voltage", "3.3v", 0);
    if (Memctl->Volt29)		AddDesc(lp, "Supports Voltage", "2.9v", 0);

    if (cp = GetArrayEntry(ErrDetectMethods, Memctl->ErrDetect))
	AddDesc(lp, "Error Detection", cp, 0);

    if (Memctl->EccNone)
	AddDesc(lp, "ECC Support", "None", 0);
    if (Memctl->EccSingle)
	AddDesc(lp, "ECC Support", "Single Bit", 0);
    if (Memctl->EccDouble)
	AddDesc(lp, "ECC Support", "Double Bit", 0);
    if (Memctl->EccScrub)
	AddDesc(lp, "ECC Support", "Error Scrubbing", 0);

    if (cp = GetArrayEntry(InterleaveTypes, Memctl->SupInter))
	AddDesc(lp, "Interleave Support", cp, 0);
    if (cp = GetArrayEntry(InterleaveTypes, Memctl->CurInter))
	AddDesc(lp, "Current Interleave", cp, 0);

    /*
     * Look for and add all attached memory modules
     */
    for (i = 0, cp = (char *) &(Memctl->Handles); i < Memctl->Slots; ++i) {
	Handle = (Word_t) *cp;
	cp += sizeof(Word_t);
	SImsg(SIM_DBG, "DMI MEMCTL:	Slot=%d Module Handle=0x%04X",
	      i, Handle);
	if (ModData = DmiFindHandle(Handle, Dmi))
	    if (Child = DmiDecodeMemmod((DmiHeader_t *)ModData, ModData, 
					Dmi, NULL)) {
		/* Add the child to our list */
		Child->Master = DevInfo;
		if (Last) {
		    Last->Next = Child;
		    Last = Child;
		} else {
		    DevInfo->Slaves = Child;
		    Last = Child;
		}
	    }
    }

    return DevInfo;
}
