/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.13 $";
#endif

#include "defs.h"
#include "dmi.h"
#include <math.h>

/*
 * Decode DMI Memory Module (DMI_STRUCT_MEMMOD)
 */
extern DevInfo_t *DmiDecodeMemmod(Hdr, Data, Dmi, Query)
     DmiHeader_t	       *Hdr;
     void		       *Data;
     Dmi_t		       *Dmi;
     DmiQuery_t		       *Query;
{
    DevInfo_t		       *DevInfo = NULL;
    DevInfo_t		       *Found = NULL;
    static DevFind_t		Find;
    Ident_t			Ident;
    DmiStructMemmod_t	       *Mem;
    char		       *cp;
    char		       *Location = NULL;
    static char			Buff[128];
    static int			Unit;
    static Word_t		History[512];
    int				HistoryIndex;
    Large_t			InstSize = 0;
    Large_t			EnableSize = 0;
    Word_t			Handle;
    register int		i;
    DevDesc_t		      **lp;

    /* Make sure we haven't checked this handle yet */
    for (i = 0; i < sizeof(History)/sizeof(Word_t) && History[i];
	 ++i)
	if (History[i] == Hdr->Handle) {
	    SImsg(SIM_DBG, 
		  "DMI MEMMOD: Handle=0x%04X Already decoded this handle.",
		  Hdr->Handle);
	    return (DevInfo_t *) NULL;
	}
    HistoryIndex = i;

    Mem = (DmiStructMemmod_t *) Data;
    if (cp = DmiString(Hdr, Mem->Location))
	Location = cp;
    /*
     * 0x7F per spec indicates nothing detected
     */
    if (Mem->InstSize != 0x7F)
	InstSize = (Large_t) pow((double)2, (double)Mem->InstSize);
    if (Mem->EnabledSize != 0x7F)
	EnableSize = (Large_t) pow((double)2, (double)Mem->EnabledSize);

    SImsg(SIM_DBG, 
	  "DMI MEMMOD: Handle=0x%04X Location=<%s> Speed=%dns Size=%d(%d MB)",
	  Hdr->Handle, PRTS(Location), Mem->CurSpeed, 
	  Mem->InstSize, InstSize);
    SImsg(SIM_DBG, 
	  "DMI MEMMOD:\tsizeof(DmiStructMemmod_t)=%d (0x%X) Hdr->Length=%d (0x%X)",
	  sizeof(DmiStructMemmod_t), sizeof(DmiStructMemmod_t), 
	  Hdr->Length, Hdr->Length);

    /*
     * If this is a specific query, then set the values now and return
     */
    if (Query && Query->Type == Hdr->Type) {
	Query->ValInt1 = (int) InstSize;
	Query->ValInt2 = (int) EnableSize;
	return DevInfo;
    }

    /* Save Handle in history after Query */
    History[HistoryIndex] = Hdr->Handle;

    /*
     * See if we can find an existing entry to use
     */
    (void) memset(&Ident, CNULL, sizeof(Ident));
    Ident.Type = IDT_ASCII;
    Ident.Identifier = (u_char *) Location;
    if (Location)
	Ident.Length = strlen(Location);
    DevFindInit(&Find);
    Find.Ident = &Ident;
    Find.DevType = DT_MEMORY;
    Find.Flags = DFE_AND;
    if (Dmi->_DevInfoTreePtr)
	Find.Tree = *Dmi->_DevInfoTreePtr;
    Found = DevInfo = DevFind(&Find);

    if (!DevInfo) {
	DevInfo = NewDevInfo(NULL);
	DevInfo->Type = DT_MEMORY;
    }
    if (DevInfo->Unit < 0)
	DevInfo->Unit = Unit++;
    if (!DevInfo->Name) {
	snprintf(Buff, sizeof(Buff), "memory%d", DevInfo->Unit);
	DevInfo->Name = strdup(Buff);
    }

    lp = &(DevInfo->DescList);

    cp = NULL;
    if (Mem->TypeStandard)		cp = "Standard";
    else if (Mem->TypeFPM)		cp = "FPM";
    else if (Mem->TypeEDO)		cp = "EDO";
    else if (Mem->TypeParity)		cp = "Parity";
    else if (Mem->TypeECC)		cp = "ECC";
    else if (Mem->TypeSIMM)		cp = "SIMM";
    else if (Mem->TypeDIMM)		cp = "DIMM";
    else if (Mem->TypeBurstEDO)		cp = "Burst-EDO";
    else if (Mem->TypeSDRAM)		cp = "SDRAM";

    if (!DevInfo->Model && cp)
	DevInfo->Model = strdup(cp);

    if (Location) {
	DevInfo->Ident = (Ident_t *) xcalloc(1, sizeof(Ident_t));
	DevInfo->Ident->Type = IDT_ASCII;
	DevInfo->Ident->Identifier = (u_char *) Location;
	DevInfo->Ident->Length = strlen(Location);
    }

    if (Mem->CurSpeed) {
	(void) snprintf(Buff, sizeof(Buff), "%d ns", Mem->CurSpeed);
	DevInfo->Speed = strdup(Buff);
    }

    /*
     * Decode Bank connection info
     */
    memset(Buff, CNULL, sizeof(Buff));
    if ((Mem->BankCon & 0xF0) != 0xF0)
	snprintf(Buff, sizeof(Buff), "%d", Mem->BankCon >> 4);
    if ((Mem->BankCon & 0x0F) != 0x0F) {
	i = Mem->BankCon & 0x0F;
	if (Buff[0])
	    snprintf(&Buff[1], sizeof(Buff), " %d", i);
	else
	    snprintf(Buff, sizeof(Buff), "%d", i);
    }
    if (Buff[0])
	AddDesc(lp, "Banks", strdup(Buff), 0);

    if (InstSize && InstSize != 0x7D && InstSize != 0x7E && InstSize != 0x7F)
	if (cp = GetSizeStr(InstSize, MBYTES, 0)) {
	    DevInfo->Capacity = strdup(cp);
	    AddDesc(lp, "Installed Size", DevInfo->Capacity, 0);
	}

    if (EnableSize && EnableSize != 0x7D && EnableSize != 0x7E && 
	EnableSize != 0x7F)
	if (cp = GetSizeStr(EnableSize, MBYTES, 0)) {
	    if (!DevInfo->Capacity)
		DevInfo->Capacity = strdup(cp);
	    AddDesc(lp, "Enabled Size", strdup(cp), 0);
	}

    /*
     * Return nothing if this entry was found and therefor already exists.
     */
    return (Found) ? (DevInfo_t *) NULL : DevInfo;
}
