/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.5 $";
#endif

/*
 * Authentication
 */

#include "defs.h"
#include <netdb.h>

static char *AuthStateNames[] = AUTH_STATE_NAMES;
#define AUTH_STATE_NAME_MAX 	(sizeof(AuthStateNames)/sizeof(char *))

/*
 * Return name of State
 */
extern char *AuthStateName(int		State)
{
    if (State < 0 || (State > AUTH_STATE_NAME_MAX))
	return (char *) NULL;
    else
	return AuthStateNames[State];
}

/*
 * Return the list of enabled Auth Types
 */
extern char **AuthTypesEnabled(int *ListCntPtr)
{
    static char **		List;
    static int			ListCnt;
    char *			AuthStr;

    if (List && ListCnt) {
	if (ListCntPtr)
	    *ListCntPtr = ListCnt;
	return List;
    }

    AuthStr = CFgetValue(CF_AUTH);
    if (!AuthStr) {
	Debug("AuthTypesEnabled: No %s entry was set in %s",
	      CF_AUTH, CFfileName(NULL));
	return (char **) NULL;
    } else {
	Debug("AuthTypesEnabled: %s=%s", CF_AUTH, AuthStr);
    }

    ListCnt = StrToArgv(AuthStr, ",", &List, NULL, 0);

    if (ListCntPtr)
	*ListCntPtr = ListCnt;

    if (ListCnt >= 1)
	return List;
    else
	return (char **) NULL;
}

/*
 * Check to see if session s is from Host
 * Return an AUTH_STATE_* value.
 */
extern int AuthHost(SSPsession_t *	s,
		    char *		Host)
{
    struct hostent *		hp;

    if (!s || !Host)
	return AUTH_STATE_NONE;

    /* Get canonical name of Host */
    hp = gethostbyname(Host);
    if (!hp) {
	Debug("AuthHost: Hostname <%s> is not resolvable.", Host);
	return AUTH_STATE_NONE;
    }

    if (EQ(hp->h_name, s->ClientName)) {
	Debug("AuthHost: Hostname <%s> (%s) is authorized", Host, hp->h_name);
	return AUTH_STATE_GOOD;
    }

    return AUTH_STATE_NONE;
}

/*
 * Check to see if session s is member of netgroup NetGroup
 * Return an AUTH_STATE_* value.
 */
extern int AuthNetGroup(SSPsession_t *	s,
			char *		NetGroup)
{
#if	defined(HAVE_INNETGR)
    if (!s || !s->ClientName || !NetGroup)
	return AUTH_STATE_NONE;

    Debug("AuthNetGroup: Checking to see if <%s> is member of netgroup <%s>",
	  PRTS(s->ClientName), PRTS(NetGroup));

    /*
     * Use innetgr(3c) to check to see if s->ClientName is a member of
     * NetGroup.
     */
    if (innetgr(NetGroup, s->ClientName, NULL, NULL) == 1) {
	Debug("AuthNetGroup: Host <%s> is member of netgroup <%s>; authorized",
	      s->ClientName, NetGroup);
	return AUTH_STATE_GOOD;
    }
#else
    Debug("AuthNetGroup: innetgr(3c) function not available; cannot check netgroup <%s>",
	  PRTS(NetGroup));
#endif	/* HAVE_INNETGR */
    return AUTH_STATE_NONE;
}

/*
 * Check the authorization for session s.
 * Return an AUTH_STATE_* value.
 */
extern int Authorized(SSPsession_t *s)
{
    char *			AllowStr = NULL;
    char **			AuthTypes = NULL;
    int				AuthTypesCnt = 0;
    char **			Argv;
    int				Argc;
    int				State = AUTH_STATE_NONE;
    int				i;
    char *			cp;

    Debug("Authorized: Checking client=<%s> ip=<%s> port=%d",
	  PRTS(s->ClientName), PRTS(s->ClientIP), s->ClientPort);

    /*
     * If AuthTypes is set, then check to see if Host auth is
     * required.  Other AuthTypes (i.e. passwd) will be handled after client
     * connection is setup.
     */
    if (AuthTypes = AuthTypesEnabled(&AuthTypesCnt)) {
	if (StrArray(AUTH_TYPE_HOST, AuthTypes))
	    AllowStr = CFgetValue(CF_ALLOW);
    } else {
	/*
	 * No Auth entry was found so check to see if Allow was set.
	 */
	AllowStr = CFgetValue(CF_ALLOW);
    }

    if (AllowStr) {
	/*
	 * Check each name found in the comma seperated AllowStr
	 */
	Argc = StrToArgv(AllowStr, ",", &Argv, NULL, 0);
	for (i = 0; i < Argc && State == AUTH_STATE_NONE; ++i) {
	    if (State == AUTH_STATE_NONE)
		State = AuthHost(s, Argv[i]);
	    if (State == AUTH_STATE_NONE)
		State = AuthNetGroup(s, Argv[i]);
	}
	DestroyArgv(&Argv, Argc);

	/*
	 * If there is no other AUTH type set and Allow failed to
	 * authorize access, then deny access outright now.
	 */
	if (State == AUTH_STATE_NONE && AuthTypesCnt <= 1) {
	    Debug("Authorized: Only %s auth is enabled and that failed; denying access",
		  CF_ALLOW);
	    State = AUTH_STATE_DENIED;
	}
    } else if (AuthTypesCnt == 0) {
	/*
	 * No "Auth" and no "Allow" entry were set.  This implies full access
	 * by everyone.
	 */
	Debug("Authorized: No \"%s\" was set in %s; allowing access from all",
	      CF_AUTH, CFfileName(NULL));
	State = AUTH_STATE_GOOD;
    }

    s->AuthState = State;

    SImsg(SIM_INFO, "%s authorized state now %s (%d)\n",
	  SSPclientStr(s), 
	  (cp = AuthStateName(s->AuthState)) ? cp : "UNKNOWN",
	  s->AuthState);

    return State;
}
