/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.10 $";
#endif

/*
 * SysInfo Server Configuration File functions
 */

#include "defs.h"
#include "cfkv.h"

/*
 * List of config directories relative to the executing program which
 * we look for.
 */
static char *ConfDirs[] = {
#if 0
    /* Debugging */		".", "..",
#endif
    /* System file */		"/etc",
    /* Relative to Program */	"../config", "../../config",
    				"../../../config",
    /* Mark end of list */	NULL
};

static CFKVhandle_t *			CFKVhandle;

/*
 * Set or return name of the CF file we read
 */
extern char *CFfileName(char *Path)
{
    static char *		File;

    if (Path) {
	if (File)
	    (void) free(File);
	File = strdup(Path);
    }

    return File;
}

/*
 * Read a configuration File
 */
extern int CFreadFile(char *	File)
{
    char *			Err = NULL;

    Debug("CFread: Reading config file %s", File);

    /*
     * Parse our config file
     */
    CFKVhandle = CFKVread(File, ProgName, &Err, (DebugOn) ? CFKV_DEBUG : 0);
    if (!CFKVhandle) {
	if (Err && *Err)
	    SImsg(SIM_CERR, "%s", Err);
	SImsg(SIM_CERR, "%s: Error reading the configuration file.", File);
	return -1;
    }

    CFfileName(File);

    return 0;
}

/*
 * Search for a config file and then call the read function on that file
 */
extern int CFread(char *	ConfigFile,	/* User specified file */
		  char *	ProgramPath)
{
    char **			Dir;
    char *			ProgramDir;
    static char			File[MAXPATHLEN];

    if (ConfigFile)			/* User specified file */
	return CFreadFile(ConfigFile);

    if (!ProgramPath)
	return -1;
    ProgramDir = DirName(ProgramPath);

    for (Dir = ConfDirs; Dir && *Dir; ++Dir) {
	if (eqn(*Dir, "../", 3))	/* Relative directory path */
	    snprintf(File, sizeof(File), "%s/%s/%s",
		     ProgramDir, *Dir, CF_BASENAME);
	else				/* Absolute directory path */
	    snprintf(File, sizeof(File), "%s/%s",
		     *Dir, CF_BASENAME);

	Debug("CFread: Checking for config file %s", File);

	if (FileExists(File, FF_SILENCE))
	    return CFreadFile(File);
    }

    SImsg(SIM_CERR, "No configuration file was found.");
    SImsg(SIM_CERR, "Re-run with --debug for more information.");

    return -1;
}

/*
 * Return a value for Key
 */
extern char *CFgetValue(char *Key)
{
    return CFKVgetValue(CFKVhandle, Key);
}

/*
 * Set up to the first entry
 */
extern int CFsetEnt()
{
    return CFKVsetEnt(CFKVhandle);
}

/*
 * Get the next entry in the table
 */
extern int CFgetEnt(char **		KeyPtr,
		    char **		ValuePtr)
{
    return CFKVgetEnt(CFKVhandle, KeyPtr, ValuePtr);
}
