/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

/*
 * $Revision: 1.20 $
 */
#ifndef __mysysinfod_defs__
#define __mysysinfod_defs__

#if defined(HAVE_AUTOCONFIG_H)
# include "mcsysinfo-autoconfig.h"
#endif	/* HAVE_AUTOCONFIG_H */

#include <stdio.h>
#include <string.h>
#include <sys/param.h>

#include "ssp.h"
#include "mcbase.h"
#include "mcsim.h"

#ifndef SYSINFOD_PID_FILE
#define SYSINFOD_PID_FILE	"/var/run/mcsysinfod.pid"
#endif
#ifndef SYSINFO_CACHE_EXPIRE
#define SYSINFO_CACHE_EXPIRE	60*60			/* Cache expire time */
#endif
#ifndef SYSINFO_CACHE_DIR
#define SYSINFO_CACHE_DIR 	"/var/tmp/mcsysinfocache"/* Cached data dir */
#endif
#ifndef SYSINFO_PATH		/* Where to look for mcsysinfo cmd */
#define SYSINFO_PATH		"${RUNDIR}/../../../bin/mcsysinfo:${CFDIR}/../bin/mcsysinfo"
#endif
#ifndef SYSINFO_BASE_ARGS
#define SYSINFO_BASE_ARGS	"--nw --siedirect"	/* Base args to sysinfo cmd */
#endif
#ifndef SYSINFO_TIMEOUT
#define SYSINFO_TIMEOUT		60*60		/* 1 hour timeout */
#endif
#ifndef SYSINFO_MAX_LINE
#define SYSINFO_MAX_LINE	100*1024	/* Max length of 1 sysinfo   */
#endif						/* line			     */
#define MAX_CLASSLIST		32		/* Max # of -class values */

/*
 * ConfigFile keys
 */
#define CF_SYSINFOPATH		"SysInfoPath"	/* Path to sysinfo(1) */
#define CF_AUTH			"Auth"		/* List of auth types */
#define CF_ALLOW		"Allow"		/* Authorized access list */
#define CF_USER			"User"		/* Auth user entry */
#define CF_PORT			"Port"		/* Listen port */
#define CF_TIMEOUT		"Timeout"	/* Session idle timeout */
#define CF_BASENAME		"mcsysinfod.cf"	/* Base name of a .cf file */

/*
 * Auth types
 */
#define AUTH_TYPE_HOST		"host"		/* Host/IP based */
#define AUTH_TYPE_USER		"user"		/* User+Pwd in mcsysinfod.cf */
#define AUTH_TYPE_PASSWD	"passwd"	/* User+Pwd from passwd(5) */

/*
 * Authentication States
 */
#define AUTH_STATE_NONE		0		/* Not yet authorized */
#define AUTH_STATE_GOOD		1		/* Auth is verified + good */
#define AUTH_STATE_DENIED	2		/* Auth is refused/denied */

/*
 * Make sure to match values up against AUTH_STATE_* above
 */
#define AUTH_STATE_NAMES	\
{ \
    "NONE", 		/* AUTH_STATE_NONE */ \
    "GOOD",		/* AUTH_STATE_GOOD */ \
    "DENIED"		/* AUTH_STATE_DENIED */ \
} \

/*
 * Command Dispatch entry
 */
typedef struct {
    char *			Name;		/* Name of cmd. i.e. "SEND" */
    char *			Usage;		/* Help string */
    int			      (*Func)();	/* Handler function */
    int				AuthReq;	/* Cmd requires auth? */
} CmdDispatch_t;

/*
 * Class IO handle
 */
typedef struct {
    char *			Class;		/* Name of class */
    char *			CacheFile;	/* Name of cache file */
    size_t			CacheSize;	/* Size of CacheFile */
    FILE *			SieReadFH;	/* SIE read file handle */
    FILE *			CacheReadFH;	/* Cache read file handle */
    FILE *			CacheWriteFH;	/* Cache write file handle */
    int				GetFirst;	/* Internal flag */
} CIOhandle_t;

/*
 * Include declarations after all definetions
 */
#include "sysinfod-declare.h"

#endif	/* __mysysinfod_defs__ */
