/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

/*
 * $Revision: 1.7 $
 */

/*
 * SysInfo Service Protocoal (SSP)
 *
 * NOTE: Keep in sync with lib/mcSysInfo/mcSSP.pm
 */
#ifndef __ssp_h__
#define __ssp_h__

#include <stdio.h>

/*
 * Default TCP service port
 */
#define SSP_TCP_PORT		11967

/*
 * Current SSP protocol version
 */
#define SSP_PROTO_VERSION	1

/*
 * SSP Commands
 */
#define SSP_CMD_AUTH		"AUTH"		/* Send authorization info */
#define SSP_CMD_BYE		"BYE"		/* Shutdown connection */
#define SSP_CMD_CLASSLIST	"CLASSLIST"	/* List available classes */
#define SSP_CMD_LICENSE		"LICENSE"	/* Show info about license */
#define SSP_CMD_MEMBERS		"MEMBERS"	/* Show member info */
#define SSP_CMD_NOOP		"NOOP"		/* Do nothing */
#define SSP_CMD_SEND		"SEND"		/* Send data */
#define SSP_CMD_SETOPTION	"SETOPTION"	/* Set options */
#define SSP_CMD_TYPES		"TYPES"		/* Show types info */
#define SSP_CMD_VERSION		"VERSION"	/* Show version info */

/*
 * Authorization types
 */
#define SSP_AUTH_PASSWORD	"PASSWORD"	/* Username + password */

/*
 * SSP Response Codes
 */
#define SSP_RC_OK		"OK"		/* Success */
#define SSP_RC_ERR		"ERR"		/* Error */
#define SSP_RC_AUTHREQ		"AUTHREQ"	/* Authentication required */
#define SSP_RC_START		"START"		/* Start data transmit */
#define SSP_RC_BYE		"BYE"		/* End session */

/*
 * Session type
 */
typedef struct {
    int				ReadFD;		/* Read descriptor */
    FILE *			ReadFP;		/* Read file ptr */
    int				WriteFD;	/* Write descriptor */
    FILE *			WriteFP;	/* Write file ptr */
    char *			ClientName;	/* Client's hostname */
    char *			ClientIP;	/* Client's IP address */
    int				ClientPort;	/* Port # of connection */
    int				AuthState;	/* Authorization State */
} SSPsession_t;

/*
 * Response Continuation Characters
 */
#define SSP_RCC_SINGLE		' '		/* Single line response */
#define SSP_RCC_MULTI		'-'		/* Multi line response */

#endif	/* __ssp_h__ */

