/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.17 $";
#endif

/*
 * SysInfo Server
 */

#include "defs.h"
#include "mcl.h"
#include "options.h"
#include "mcsim.h"
#if HAVE_LIBGEN_H
# include <libgen.h>	/* for basename() */
#endif

char *					ProgPath;
char *					ProgName;
static char *				ConfigFile;
static char *				SysInfoPath;
static char *				LogFac;
static int				DebugFlag = 0;
static int				DoPrintVersion = 0;
static int				Port = 0;
static int				Timeout = SYSINFO_TIMEOUT;
int					MsgClassFlags = SIM_DEFAULT;
int					ForkFlag = 1;
int					SiUseStderr = 0;

/*
 * Command Line Options
 */
OptionDescRec Opts[] = {
    { "configfile",  	SepArg,	OptStr,	(OptPtr_t) &ConfigFile,		NULL,
      "<cfFile>", 	"Configuration File",				'c' },
    { "debug",  	NoArg,	OptBool,(OptPtr_t) &DebugFlag,		"1",
      NULL,  	"Enable debugging", 					'd' },
    { "fg",  		NoArg,	OptBool,(OptPtr_t) &ForkFlag,		"0",
      NULL,  	"Run in foreground (no fork)",				'f' },
    { "logfac", 	SepArg,	OptStr,	(OptPtr_t) &LogFac,	"daemon",
      "<facility>",  	"syslog facility name",				'L' },
    { "sysinfo",  	SepArg,	OptStr,	(OptPtr_t) &SysInfoPath,	NULL,
      "<path>",  	"Path to sysinfo(1) command",			's' },
    { "port",  		SepArg,	OptInt,	(OptPtr_t) &Port,		NULL,
      "<port>",  	"Port to listen on",				'p' },
    { "timeout",  SepArg,OptInt,(OptPtr_t) &Timeout,			NULL,
      "<secs>",  	"Number of seconds for connection timeout",	't' },
    { "version",NoArg,		OptBool,(OptPtr_t) &DoPrintVersion,	"1",
      NULL,	"Print version of this program", 			'V' },
};

/*
 * The start of all good things ...
 */
main(int 	Argc,
     char **	Argv)
{
    int				Status = 0;
    char *			cp;
    static char			Path[MAXPATHLEN];

    ProgPath = Argv[0];
    ProgName = basename(ProgPath);

    /*
     * Parse command line arguments.
     */
    if (ParseOptions(Opts, Num_Opts(Opts), Argc, Argv) < 0)
	exit(1);

    if (DebugFlag)
	MsgClassFlags |= SIM_DBG;

    SImsgSetup(ProgName);
    if (LogFac && SImsgLogFac(LogFac) < 0) {
	exit(1);
    }

    /* Read configuration file */
    if (CFread(ConfigFile, ProgPath) < 0)
	exit(1);

    /*
     * Show version info
     */
    if (DoPrintVersion || DebugOn) {
	if (DoPrintVersion)
	    SiUseStderr = TRUE; /* XXX hack to force output to stderr */ 
	VersionPrint(FALSE, DebugOn);
	if (DoPrintVersion)
	    exit(0);
    }

    /*
     * Set parameters from config file if not specified on command line
     */
    if (!Port && (cp = CFgetValue(CF_PORT)))
	Port = atoi(cp);
    if ((Timeout == SYSINFO_TIMEOUT) && (cp = CFgetValue(CF_TIMEOUT)))
	Timeout = atoi(cp);

    /*
     * Make sure we can find the sysinfo(1) command or there's not much
     * we'll be able to do.
     */
    if (!SIEsysinfoPath(SysInfoPath)) {
	snprintf(Path, sizeof(Path), "%s/%s", CFDIR, CF_BASENAME);
	SImsg(SIM_CERR,
	      "Add \"SysInfoPath=...\" to %s or use \"--sysinfo <path>\" option.",
	      (ConfigFile) ? ConfigFile : Path);
	exit(1);
    } 

    /*
     * Start service
     */
    Status = ServerInit(Port, Timeout);

    exit(Status);
}
