/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.6 $";
#endif

/*
 * Message functions
 */

#include "defs.h"
#include "mcmacros.h"
#include "mcdefs.h"
#include "mcsim.h"
#include <syslog.h>

#define Match(m)	(FLAGS_ON(MsgType, m) && FLAGS_ON(MsgClassFlags, m))

/*
 * Output file
 */
char *				OutputFile = NULL;
static FILE *			OutputFH;
static int			UseSysLog = FALSE;
static char *			ProgramName;
extern int			MsgClassFlags;
static int			SysLogFac = LOG_DAEMON;

extern int			SiUseStderr;

/*
 * syslog(3) facility name -> value table
 */
typedef struct {
    char *			Name;		/* Facility name */
    int				Value;		/* Numeric value */
} SysLogFac_t;
static SysLogFac_t SysLogFacTab[] = {
    { "auth", LOG_AUTH },
#if LOG_AUTHPRIV
    { "authpriv", LOG_AUTHPRIV },
#endif
    { "cron", LOG_CRON },
    { "daemon", LOG_DAEMON },
#if LOG_FTP
    { "ftp", LOG_FTP },
#endif
    { "kern", LOG_KERN },
    { "lpr", LOG_LPR },
    { "mail", LOG_MAIL },
    { "news", LOG_NEWS },
    { "security", LOG_AUTH },		/* DEPRECATED */
    { "syslog", LOG_SYSLOG },
    { "user", LOG_USER },
    { "uucp", LOG_UUCP },
    { "local0", LOG_LOCAL0 },
    { "local1", LOG_LOCAL1 },
    { "local2", LOG_LOCAL2 },
    { "local3", LOG_LOCAL3 },
    { "local4", LOG_LOCAL4 },
    { "local5", LOG_LOCAL5 },
    { "local6", LOG_LOCAL6 },
    { "local7", LOG_LOCAL7 },
    { NULL, -1 }
};

/*
 * Do setup for SImsg()
 */
extern int SImsgSetup(Program)
     char		       *Program;
{
    uid_t			uid = 0;
    uid_t			euid = 0;

    if (Program)
	ProgramName = Program;
    if (!ProgramName)
	ProgramName = "<unknown>";

    /* Always test for this */
    if (SiUseStderr) {
	OutputFH = stderr;
	UseSysLog = FALSE;
    }

    if (!OutputFH) {
	if (OutputFile) {
	    /*
	     * If we're setuid, then drop privs temporarily so we don't
	     * allow the user to write files they normally can't
	     */
	    uid = getuid();
	    euid = geteuid();
	    if (uid != euid)
		seteuid(uid);

	    OutputFH = fopen(OutputFile, "w");

	    /*
	     * Raise privs back to previous level
	     */
	    if (uid != euid)
		setuid(euid);

	    if (!OutputFH) {
		fprintf(stderr, "%s%s%s: Open for writing failed: %s\n",
			(ProgramName) ? ProgramName : "", 
			(ProgramName) ? ": " : "", 
			OutputFile, SYSERR);
		return -1;
	    }
	} else if (SiUseStderr || (DebugOn && !ForkFlag)) {
	    /* Use stderr */
	    OutputFH = stderr;
	} else {
	    /* Use syslog */
	    UseSysLog = TRUE;
	    openlog(ProgramName, LOG_PID, SysLogFac);
	}
    }

    return 0;
}

/*
 * End SImsg calls
 */
extern int SImsgEnd()
{
    if (OutputFH && OutputFH != stdout)
	fclose(OutputFH);

    if (UseSysLog)
	closelog();

    return 0;
}

/*
 * Return the syslog Priority to use for MsgType
 */
static int SysLogGetPriority(int MsgType)
{
    int				Prio = 0;

    if (Match(SIM_INFO)) 
	Prio = LOG_INFO;
    else if (Match(SIM_WARN))
	Prio = LOG_WARNING;
    else if (Match(SIM_GERR))
	Prio = LOG_ERR;
    else if (Match(SIM_CERR))
	Prio = LOG_CRIT;
    else if (Match(SIM_HINT))
	Prio = LOG_INFO;
    else if (Match(SIM_DBG))
	Prio = LOG_DEBUG;
    else if (Match(SIM_UNKN))
	Prio = LOG_NOTICE;
    else
	/* The value being passed is not set */
	return -1;

    return Prio;
}

/*
 * SysInfo Message functions
 *
 * SIM_INFO messages will include \n (newline) when needed by calling
 * function.  All other SIM_* will have newline provided by SImsg().
 */
#if	ARG_TYPE == ARG_STDARG
#include <stdarg.h>
/*
 * StdArg version of SImsg()
 */
extern void vSImsg(int MsgType, char *fmt, va_list args)
{
    int				Prio = 0;
    static char			Buff[8192];

    if (MsgType == 0 || MsgClassFlags == 0)
	/* No output */
	return;

    if (SImsgSetup(NULL) < 0)
	return;

    if (UseSysLog) {
	Prio = SysLogGetPriority(MsgType);
	if (Prio < 0)
	    return;

	/* Format msg */
	vsnprintf(Buff, sizeof(Buff), fmt, args);
	/* Send syslog msg */
	syslog(Prio, "%s", Buff);
    } else {
	/*
	 * Write to a file or stdout/stderr
	 */
	if (Match(SIM_INFO) && FLAGS_ON(MsgClassFlags, SIM_INFO)) {
	    /* Normal output */
	    (void) vfprintf(OutputFH, fmt, args);
	} else if (Match(SIM_WARN)) {
	    (void) fflush(OutputFH);
	    if (FLAGS_OFF(MsgType, SIM_NOLBL))
		(void) fprintf(stderr, "%s: WARNING: ", ProgramName);
	    (void) vfprintf(stderr, fmt, args);
	    if (FLAGS_OFF(MsgType, SIM_NONL))
		(void) fprintf(stderr, "\n");
	} else if (Match(SIM_UNKN)) {
	    (void) fflush(OutputFH);
	    if (FLAGS_OFF(MsgType, SIM_NOLBL))
		(void) fprintf(stderr, "%s: UNKNOWN: ", ProgramName);
	    (void) vfprintf(stderr, fmt, args);
	    if (FLAGS_OFF(MsgType, SIM_NONL))
		(void) fprintf(stderr, "\n");
	} else if (Match(SIM_GERR)) {
	    (void) fflush(OutputFH);
	    if (FLAGS_OFF(MsgType, SIM_NOLBL))
		(void) fprintf(stderr, "%s: GERROR: ", ProgramName);
	    (void) vfprintf(stderr, fmt, args);
	    if (FLAGS_OFF(MsgType, SIM_NONL))
		(void) fprintf(stderr, "\n");
	} else if (Match(SIM_CERR)) {
	    (void) fflush(OutputFH);
	    if (FLAGS_OFF(MsgType, SIM_NOLBL))
		(void) fprintf(stderr, "%s: CERROR: ", ProgramName);
	    (void) vfprintf(stderr, fmt, args);
	    if (FLAGS_OFF(MsgType, SIM_NONL))
		(void) fprintf(stderr, "\n");
	} else if (Match(SIM_DBG)) {
	    (void) fflush(OutputFH);
	    (void) vfprintf(stderr, fmt, args);
	    if (FLAGS_OFF(MsgType, SIM_NONL))
		(void) fprintf(stderr, "\n");
	} else if (Match(SIM_HINT)) {
	    (void) fflush(OutputFH);
	    (void) vfprintf(stderr, fmt, args);
	    if (FLAGS_OFF(MsgType, SIM_NONL))
		(void) fprintf(stderr, "\n");
	}
    }
}

/*
 * StdArg version of SImsg()
 */
extern void SImsg(int MsgType, char *fmt, ...)
{
    va_list 			args;

    va_start(args, fmt);
    vSImsg(MsgType, fmt, args);
    va_end(args);
}

/*
 * StdArg version of Debug()
 */
extern void Debug(char *fmt, ...)
{
    va_list 			args;

    va_start(args, fmt);
    vSImsg(SIM_DBG, fmt, args);
    va_end(args);
}

/*
 * StdArg version of Fatal()
 */
extern void Fatal(int Status, char *fmt, ...)
{
    va_list 			args;

    va_start(args, fmt);
    vSImsg(SIM_CERR, fmt, args);
    va_end(args);
    exit(Status);
}
#endif	/* ARG_STDARG */
#if	ARG_TYPE == ARG_VARARG
#include <varargs.h>
/*
 * Varargs version of SImsg()
 */
extern void vSImsg(MsgType, fmt, args)
     int			MsgType;
     char		       *fmt;
     va_list 			args;
{
    int				Prio = 0;

    if (MsgType == 0 || MsgClassFlags == 0)
	/* No output */
	return;

    if (SImsgSetup(NULL) < 0)
	return;

    if (UseSysLog) {
	Prio = SysLogGetPriority(MsgType);
	if (Prio < 0)
	    return;

	/* Send syslog msg */
	vsyslog(Prio, fmt, args);
    } else {
	/*
	 * Write to a file or stdout/stderr
	 */
	if (Match(SIM_INFO)) {
	    (void) vfprintf(OutputFH, fmt, args);
	} else if (Match(SIM_WARN)) {
	    (void) fflush(OutputFH);
	    if (FLAGS_OFF(MsgType, SIM_NOLBL))
		(void) fprintf(stderr, "%s: WARNING: ", ProgramName);
	    (void) vfprintf(stderr, fmt, args);
	    if (FLAGS_OFF(MsgType, SIM_NONL))
		(void) fprintf(stderr, "\n");
	} else if (Match(SIM_UNKN)) {
	    (void) fflush(OutputFH);
	    if (FLAGS_OFF(MsgType, SIM_NOLBL))
		(void) fprintf(stderr, "%s: UNKNOWN: ", ProgramName);
	    (void) vfprintf(stderr, fmt, args);
	    if (FLAGS_OFF(MsgType, SIM_NONL))
		(void) fprintf(stderr, "\n");
	} else if (Match(SIM_GERR)) {
	    (void) fflush(OutputFH);
	    if (FLAGS_OFF(MsgType, SIM_NOLBL))
		(void) fprintf(stderr, "%s: GERROR: ", ProgramName);
	    (void) vfprintf(stderr, fmt, args);
	    if (FLAGS_OFF(MsgType, SIM_NONL))
		(void) fprintf(stderr, "\n");
	} else if (Match(SIM_CERR)) {
	    (void) fflush(OutputFH);
	    if (FLAGS_OFF(MsgType, SIM_NOLBL))
		(void) fprintf(stderr, "%s: CERROR: ", ProgramName);
	    (void) vfprintf(stderr, fmt, args);
	    if (FLAGS_OFF(MsgType, SIM_NONL))
		(void) fprintf(stderr, "\n");
	} else if (Match(SIM_DBG)) {
	    (void) fflush(OutputFH);
	    (void) vfprintf(stderr, fmt, args);
	    if (FLAGS_OFF(MsgType, SIM_NONL))
		(void) fprintf(stderr, "\n");
	} else if (Match(SIM_HINT)) {
	    (void) fflush(OutputFH);
	    (void) vfprintf(stderr, fmt, args);
	    if (FLAGS_OFF(MsgType, SIM_NONL))
		(void) fprintf(stderr, "\n");
	}
    }
}

/*
 * Varargs version of SImsg()
 */
extern void SImsg(MsgType, fmt, va_alist)
     int			MsgType;
     char		       *fmt;
     va_dcl
{
    va_list 			args;

    va_start(args);
    vSImsg(MsgType, fmt, args);
    va_end(args);
}

/*
 * Varargs version of Debug()
 */
extern void Debug(fmt, va_alist)
     char		       *fmt;
     va_dcl
{
    va_list 			args;

    va_start(args);
    vSImsg(SIM_DBG, fmt, args);
    va_end(args);
}

/*
 * Varargs version of Fatal()
 */
extern void Fatal(Status, fmt, va_alist)
     int			Status;
     char		       *fmt;
     va_dcl
{
    va_list 			args;

    va_start(args);
    vSImsg(SIM_CERR, fmt, args);
    va_end(args);

    exit(Status);
}
#endif	/* ARG_VARARG */

/*
 * Set the syslog facility to use to be Name.
 */
extern int SImsgLogFac(char *Name)
{
    register int		i;

    for (i = 0; SysLogFacTab[i].Name; ++i)
	if (EQ(Name, SysLogFacTab[i].Name)) {
	    SysLogFac = SysLogFacTab[i].Value;
	    return 0;
	}

    SImsg(SIM_CERR, "No such syslog(3) facility as \"%s\".", Name);

    return -1;
}
