/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.11 $";
#endif

/*
 * SysInfo Server
 */

#include "defs.h"
#include "version.h"
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/time.h>		/* for FD_SET */

/*
 * Return a string showing server version
 */
extern char *ServerVersion()
{
    static char			Buff[64];

    snprintf(Buff, sizeof(Buff), "%s.%d.%d-%s",
	     SYSINFO_VERSION_STR, SYSINFO_RELEASE,
	     SYSINFO_PATCHLEVEL, SYSINFO_VERSION_STATUS);

    return Buff;
}

/*
 * Initialize server to accept connections and await a connection.
 */
extern int ServerInit(unsigned short 	Port,
		      int		Timeout)
{
    unsigned long       	Addr = INADDR_ANY;
    struct sockaddr_in		ServAddr;
    struct sockaddr_in		ClientAddr;
    fd_set			FdSet;
    fd_set			FdSetRead;
#if defined(_XOPEN_SOURCE_EXTENDED)
    socklen_t			sl;
#else
    int				sl;
#endif
    pid_t			p;
    int				i;
    int				r;
    int				SockFd;
    int				SockFdClient;

    if (!Port)
	Port = SSP_TCP_PORT;

    /* Ensure network order */
    Port = htons(Port);
    Addr = htonl(Addr);

    if (ForkFlag) {
	p = fork();
	if (p == -1) {
	    SImsg(SIM_CERR, "ServerInit(): fork() failed: %s", SYSERR);
	    return -1;
	}
	if (p > 0) {
	    /* We're the parent */
	    Debug("Server forked PID %d", p);
	    exit(0);
	}

	p = setsid();		/* Disassociate from controlling terminal */
	if (p == -1)
	    SImsg(SIM_CERR, "setsid() failed: %s", SYSERR);
    }

    /*
     * Be friendly by insuring we don't hold open any directories.
     */
    (void) chdir("/");

    /*
     * Close all file descriptors including stdin, stdout, stderr which may
     * have been redirected by parent.
     */
    if (ForkFlag && !DebugOn)
	for (i = sysconf(_SC_OPEN_MAX); i >= 0; i--)
	    close(i);

    /*
     * Setup listening socket
     */
    SockFd = socket(AF_INET, SOCK_STREAM, 0);
    if (SockFd < 0) {
	SImsg(SIM_CERR, "Create stream socket failed: %s", SYSERR);
	return -1;
    }

    i = 1;
    if (setsockopt(SockFd, SOL_SOCKET, SO_REUSEADDR,
		   (void *) &i, sizeof(i)) < 0) {
	SImsg(SIM_GERR, "Set socket option SO_REUSEADDR failed: %s", SYSERR);
	/* Not Fatal */ 
    }

    /*
     * Setup Adddress block
     */
    memset(&ServAddr, 0, sizeof(ServAddr));
    ServAddr.sin_family = AF_INET;
    ServAddr.sin_addr.s_addr = Addr;
    ServAddr.sin_port = Port;

    /*
     * Bind the socket to the port
     */
    if (bind(SockFd, (struct sockaddr *) &ServAddr, sizeof(ServAddr)) < 0) {
	if (errno == EADDRINUSE)
	    SImsg(SIM_CERR,
		  "Bind socket failed: Address %s:%d already in use.",
		  inet_ntoa(ServAddr.sin_addr),
		  ntohs(ServAddr.sin_port));
	else
	    SImsg(SIM_CERR, "Bind socket to %s:%d failed: %s",
		  inet_ntoa(ServAddr.sin_addr),
		  ntohs(ServAddr.sin_port), SYSERR);
	close(SockFd);
	return -1;
    }

    /*
     * Save our PID
     */
    (void) PidSave();

    /*
     * Service is now ready
     */
    Debug("Listening on %s:%d",
	  inet_ntoa(ServAddr.sin_addr), ntohs(ServAddr.sin_port));

    if (listen(SockFd, 5) < 0) {
	SImsg(SIM_CERR, "listen on %s:%d failed: %s",
		  inet_ntoa(ServAddr.sin_addr),
		  ntohs(ServAddr.sin_port), SYSERR);
	close(SockFd);
	return -1;
    }

    /*
     * Setup non-blocking I/O on SockFd
     */
    i = fcntl(SockFd, F_GETFL, 0);
    if (fcntl(SockFd, F_SETFL, i | O_NONBLOCK) < 0) {
	SImsg(SIM_GERR, "Set nonblocking I/O via fcntl failed: %s", SYSERR);
	close(SockFd);
	return -1;
    }

    /*
     * Setup Signal Handling
     */
    SigSetup();

    /*
     * Build template descriptors
     */
    memset(&FdSet, 0, sizeof(FdSet));
    FD_SET(SockFd, &FdSet);

    for ( ; ; ) {	/* Loop */
	/* Make copy since select() modifies the set we pass it */
	FdSetRead = FdSet;

	/*
	 * Use select() to see if the new connection is ready since accept()
	 * does not return on signals on some platforms.
	 */
	r = select(SockFd + 1, &FdSetRead, NULL, NULL, NULL);
	if (r == -1 && errno != EINTR) {
	    SImsg(SIM_CERR, "select() error: %s", SYSERR);
	    return -1;
	}
	if (r == 0 || (FD_ISSET(SockFd, &FdSetRead) == FALSE)) {
	    SImsg(SIM_CERR, "detected select() unexpected error");
	    return -1;
	}

	sl = sizeof(ClientAddr);
	SockFdClient = accept(SockFd, (struct sockaddr *) &ClientAddr, &sl);
	if (SockFdClient <= 0) {
	    SImsg(SIM_GERR, "accept() failed: %s", SYSERR);
	} else {
	    /* Connection established with client */
	    ClientInit(SockFdClient, SockFd, Timeout);
	}
    }

    return 0;
}
