/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.6 $";
#endif

/*
 * SIE Class functions
 */

#include "defs.h"
#include <time.h>
#include <sys/stat.h>

#define CACHE_INFO_EXT		".cinfo"	/* cache info file extension */
#define CACHE_DATA_EXT		".cdata"	/* cache data file extension */

/*
 * Save cache info from h
 */
static int CacheSave(CIOhandle_t *h)
{
    static char 		File[MAXPATHLEN];
    FILE *			FH;

    if (!h || !h->Class || !h->CacheFile)
	return -1;

    snprintf(File, sizeof(File), "%s/%s%s",
	     SYSINFO_CACHE_DIR, h->Class, CACHE_INFO_EXT);

    Debug("CacheSave: Saving cache to <%s>", File);

    unlink(File);
    FH = fopen(File, "w");
    if (!FH) {
	SImsg(SIM_GERR, "%s: Open for write failed: %s", File, SYSERR);
	return -1;
    }

    fprintf(FH, "cachefile=%s\n", h->CacheFile);
    fprintf(FH, "size=%ld\n", h->CacheSize);

    (void) fclose(FH);
    (void) chmod(File, 0644);

    return 0;
}

/*
 * Load basic cache info into h
 */
static int CacheLoad(CIOhandle_t *h)
{
    static char			Line[512];
    static char 		File[MAXPATHLEN];
    FILE *			FH;
    char *			cp;
    char *			Key;
    char *			Value;

    if (!h || !h->Class)
	return -1;

    snprintf(File, sizeof(File), "%s/%s%s",
	     SYSINFO_CACHE_DIR, h->Class, CACHE_INFO_EXT);

    Debug("CacheLoad: Loading cache from <%s>", File);

    FH = fopen(File, "r");
    if (!FH) {
	if (errno != ENOENT)
	    SImsg(SIM_GERR, "%s: Open for read failed: %s", File, SYSERR);
	return -1;
    }

    while (fgets(Line, sizeof(Line), FH)) {
	if (cp = strchr(Line, '\n'))
	    *cp = CNULL;
	Key = Line;
	cp = strchr(Key, '=');
	if (!cp)
	    continue;

	*cp = CNULL;
	Value = cp + 1;

	Debug("CacheLoad:\t <%s>=<%s>", Key, Value);

	/*
	 * Lookup Key
	 * Ignore unknown keys
	 */
	if (EQ(Key, "cachefile")) {
	    if (!h->CacheFile)
		h->CacheFile = strdup(Value);
	} else if (EQ(Key, "size")) {
	    h->CacheSize = atol(Value);
	}
    }

    fclose(FH);

    return 0;
}

/*
 * Is the CacheFile in h valid?
 * Returns TRUE if it is.
 */ 
static int CacheValid(CIOhandle_t *h)
{
    struct stat			st;
    time_t			GmTime;
    time_t			LocalTime;
    int				Valid = -1;

    if (!h || !h->CacheFile)
	return FALSE;

    CacheLoad(h);

    Debug("CacheValid: class=<%s> file=<%s> size=%ld",
	  PRTS(h->Class), PRTS(h->CacheFile), h->CacheSize);

    if (stat(h->CacheFile, &st) == 0) {
	/*
	 * Check when cache was made
	 */
	time(&GmTime);
	LocalTime = mktime(localtime(&GmTime));
	Debug("CacheValid:\t localtime=%ld mtime=%ld size=%ld",
	      LocalTime, st.st_mtime, st.st_size);
	if (Valid < 0 && LocalTime - st.st_mtime > SYSINFO_CACHE_EXPIRE) {
	    Debug("CacheValid: Cache file <%s> expired (%d secs old)",
		  h->CacheFile, LocalTime - st.st_mtime);
	    Valid = FALSE;
	}
	/*
	 * Check size of cache
	 */
	if (Valid < 0 && (st.st_size != h->CacheSize || st.st_size == 0)) {
	    Debug("CacheValid: Cache file <%s> size mismatch (CacheSize=%ld Actual=%ld)",
		  h->CacheFile, h->CacheSize, st.st_size);
	    Valid = FALSE;
	}
	/*
	 * Past all the tests?
	 */
	if (Valid < 0) {
	    Valid = TRUE;
	}
    } else {
	Debug("CacheValid: stat <%s> failed: %s", h->CacheFile, SYSERR);
	return FALSE;
    }

    if (Valid == TRUE) {
	/*
	 * Valid
	 */
	Debug("CacheValid: Cache VALID file <%s>", h->CacheFile);
	return TRUE;
    } else {
	/*
	 * Not valid
	 */
	Debug("CacheValid: Clearing cache for <%s>", h->Class);
	if (unlink(h->CacheFile) < 0) {
	    /* Not fatal */
	    SImsg(SIM_GERR, "%s: Remove cache file failed: %s",
		  h->CacheFile, SYSERR);
	}
	h->CacheSize = 0;
	return FALSE;
    }
}

/*
 * Create a new handle for Class
 */
extern CIOhandle_t *SIEclassSet(char *ClassName)
{
    CIOhandle_t *		h;
    static char			Cmd[MAXPATHLEN];
    char *			Class;
    char *			SysInfo;
    static char 		Path[MAXPATHLEN];
    char *			cp;
    int				DirOK = FALSE;
    struct stat			st;

    if (!ClassName)
	return (CIOhandle_t *) NULL;

    /* Make sure we're consistant with class name */
    if (cp = strlower(ClassName))
	Class = cp;
    else
	Class = ClassName;

    /* Create handle */
    h = (CIOhandle_t *) xcalloc(1, sizeof(CIOhandle_t));
    h->Class = strdup(Class);

    /* Check for and create if necessary the cache directory */
    if (stat(SYSINFO_CACHE_DIR, &st) != 0) {
	Debug("SIEclassSet: mkdir %s", SYSINFO_CACHE_DIR);
	if (MakeDir(SYSINFO_CACHE_DIR, 0700) != 0) {
	    SImsg(SIM_GERR, "%s: make directory failed: %s",
		  SYSINFO_CACHE_DIR, SYSERR);
	} else {
	    DirOK = TRUE;
	}
    } else {
	DirOK = TRUE;
    }
    if (DirOK) {
	snprintf(Path, sizeof(Path), "%s/%s%s",
		 SYSINFO_CACHE_DIR, h->Class, CACHE_DATA_EXT);
	h->CacheFile = strdup(Path);
    }

    /*
     * If the cache is valid, then open the cache file
     */
    if (h->CacheFile && CacheValid(h)) {
	h->CacheReadFH = fopen(h->CacheFile, "r");
	if (!h->CacheReadFH) {
	    SImsg(SIM_GERR, "%s: Open cache file for read failed: %s",
		  h->CacheFile, SYSERR);
	} else {
	    Debug("SIEclassSet: Using cache file <%s>", h->CacheFile);
	}
    } else {
	/*
	 * Open pipe to sysinfo engine and set file handle
	 */
	SysInfo = SIEsysinfoPath(NULL);
	if (!SysInfo) {
	    SImsg(SIM_GERR, "Cannot find sysinfo engine command.");
	    return (CIOhandle_t *) NULL;
	}

	/*
	 * Create the SIE command.  We're careful in how we set --repsep
	 * to '|' since we pass this on to popen() which uses $SHELL to
	 * run this command.
	 */
	snprintf(Cmd, sizeof(Cmd),
		 "%s %s --repsep \\%c --format report --msglevel all --class %s",
		 SysInfo, SIEoptionStr(), '|', h->Class);
	Debug("SIEclassSet: Run: %s", Cmd);

	h->SieReadFH = popen(Cmd, "r");
	if (!h->SieReadFH) {
	    SImsg(SIM_GERR, "Open pipe to command \"%s\" failed: %s",
		  Cmd, SYSERR);
	    return (CIOhandle_t *) NULL;
	}

	/*
	 * Setup CacheWriteFH to allow caller to write to new cache
	 */
	if (h->CacheFile) {
	    h->CacheWriteFH = fopen(h->CacheFile, "w");
	    if (!h->CacheWriteFH) {
		SImsg(SIM_GERR, "%s: Open cache file for write failed: %s",
		      h->CacheFile, SYSERR);
	    } else {
		Debug("SIEclassSet: Cache file opened for write <%s>",
		      h->CacheFile);
	    }
	}
    }

    return h;
}

/*
 * Get one line of data from Handle's file stream
 */
extern char *SIEclassGet(CIOhandle_t *h)
{
    static char			Line[SYSINFO_MAX_LINE];
    FILE *			ReadFH = NULL;
    FILE *			CacheFH = NULL;
    char *			cp = NULL;
    size_t			len;

    if (!h || !h->Class) {
	Debug("SIEclassGet: Missing params");
	return (char *) NULL;
    }

    /* Select read FH */
    if (h->CacheReadFH) {
	cp = "Using cached data";
	ReadFH = h->CacheReadFH;
    } else if (h->SieReadFH) {
	cp = "Reading from SIE directly";
	ReadFH = h->SieReadFH;
    } else {
	SImsg(SIM_GERR, "SIEclassGet: No file handle is available to read.");
	return (char *) NULL;
    }

    if (cp && !h->GetFirst) {
	h->GetFirst = TRUE;
	Debug("SIEclassGet: %s for <%s>", cp, h->Class);
    }

    /*
     * Read and return one line of data (including newline)
     */
    if (fgets(Line, sizeof(Line), ReadFH)) {
	if (h->CacheWriteFH) {
	    /*
	     * Save this Line to the cache
	     */
	    len = strlen(Line);
	    if (fprintf(h->CacheWriteFH, "%.*s", len, Line) != len) {
		SImsg(SIM_GERR,
		      "%s: Write error occured writing to cache file: %s",
		      h->CacheFile, SYSERR);
		h->CacheSize = 0;		/* Invalidate cache */
		fclose(h->CacheWriteFH);	/* Invalidate cache */
		h->CacheWriteFH = NULL;		/* Invalidate cache */
	    }
	    h->CacheSize += len;
	}
	return Line;
    }

    return (char *) NULL;
}

/*
 * Close done this handle
 */
extern int SIEclassEnd(CIOhandle_t *h)
{
    if (!h)
	return -1;

    if (h->CacheWriteFH)	/* Only save if we created the cache */
	CacheSave(h);

    if (h->CacheReadFH) {
	(void) fclose(h->CacheReadFH);
	h->CacheReadFH = NULL;
    }
    if (h->CacheWriteFH) {
	(void) fclose(h->CacheWriteFH);
	h->CacheWriteFH = NULL;
    }
    if (h->SieReadFH) {
	(void) fclose(h->SieReadFH);
	h->SieReadFH = NULL;
    }
    if (h->Class)
	(void) free(h->Class);
    if (h->CacheFile)
	(void) free(h->CacheFile);

    (void) free(h);

    return 0;
}
