/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.6 $";
#endif

/*
 * SSP Authentication commands
 */

#include "defs.h"
#ifndef _XOPEN_SOURCE
#define _XOPEN_SOURCE		/* for crypt(3) */
#endif
#include <unistd.h>
#include <pwd.h>
#if defined(HAVE_SHADOW_H)
# include <shadow.h>
#endif
#if defined(HAVE_CRYPT_H)
# include <crypt.h>
#endif
#include <sys/types.h>

/*
 * Authenticate client using a username + password compared against
 * the system's passwd(5) database.
 */
static int AuthPwdPasswd(SSPsession_t *		s,
			 char *			User,
			 char *			Password)
{
    struct passwd *		pw;
#if defined(HAVE_GETSPNAM)
    struct spwd *		sp;
#endif
    char *			encpwd = NULL;
    char *			SysPwd = NULL;

    Debug("AuthPwdPasswd: Checking user <%s> via getpwnam(3)", User);

    pw = getpwnam(User);
    if (!pw) {
	Debug("AuthPwdPasswd: User <%s> was not found via getpwnam(3)", User);
	/* Allow other auth methods to be tried */
	return AUTH_STATE_NONE;
    }

    SysPwd = pw->pw_passwd;
#if defined(HAVE_GETSPNAM)
    /*
     * Lookup the actually encrypted system passwd in the shadow db
     */
    if (EQ(pw->pw_passwd, "x")) {
	sp = getspnam(User);
	if (!sp) {
	    /*
	     * This often fails if we don't have privs to read /etc/shadow
	     */
	    Debug("AuthPwdPasswd: User <%s> was not found via getspnam(3)",
		  User);
	    Debug("AuthPwdPasswd: Make sure you have permissions to read /etc/shadow");
	    /* Allow other auth methods to be tried */
	    return AUTH_STATE_NONE;
	}
	SysPwd = sp->sp_pwdp;
    }
#endif	/* HAVE_GETSPNAM */

    encpwd = crypt(Password, SysPwd);

    Debug("AuthPwdPasswd:\t pw_passwd=<%s> SysPwd=<%s> client enc pwd=<%s>",
	  PRTS(pw->pw_passwd), PRTS(SysPwd), PRTS(encpwd));

    if (eq(SysPwd, encpwd)) {
	Debug("AuthPwdPasswd: User <%s> matched passwd from getpwnam(3)",
	      User);
	return AUTH_STATE_GOOD;
    } else {
	Debug("AuthPwdPasswd: User <%s> did not match passwd from getpwnam(3)",
	      User);
	/* Hard error - no more auth type tries */
	return AUTH_STATE_DENIED;
    }
}

/*
 * Authenticate client using a username + password compared against
 * the "User" entries in the sysinfod.cf file.
 */
static int AuthPwdUser(SSPsession_t *		s,
		       char *			User,
		       char *			Password)
{
    char *			Key;
    static char			Value[256];
    char *			cp;
    char *			user;
    char *			pwd;

    CFsetEnt();

    while (CFgetEnt(&Key, &cp) > 0) {
	/*
	 * We're looking for "user" or "user_XXXX"
	 * The Value should be "username password" with a single
	 * space as the seperator.
	 */
	if (EQ(Key, "user") || EQN(Key, "user_", 5)) {
	    /* Save Value into a disposible buffer */
	    snprintf(Value, sizeof(Value), "%s", cp);

	    Debug("AuthPwdUser: Checking <%s>", PRTS(Value));
	    user = Value;
	    if (cp = strchr(user, ' ')) {
		*cp = CNULL;
		pwd = cp + 1;
	    } else {
		Debug("AuthPwdUser:\t No password; looking for: user pwd");
		continue;
	    }

	    if (eq(User, user)) {
		Debug("AuthPwdUser:\t Username matches");
		if (eq(Password, pwd)) {
		    Debug("AuthPwdUser:\t Password matches");
		    return AUTH_STATE_GOOD;
		} else {
		    Debug("AuthPwdUser:\t Password does not match");
		    /* Hard error - no more auth type tries */
		    return AUTH_STATE_DENIED;
		}
	    }
	}
    }

    return AUTH_STATE_NONE;
}

/*
 * Authenticate client using a username + password
 */
static int AuthPassword(SSPsession_t *		s,
			char *			User,
			char *			Password)
{
    char **			AuthTypes;
    int				AuthTypesCnt = 0;
    int				State = AUTH_STATE_NONE;
    int				i;

    if (!s || !User || !Password) {
	SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE,
		"Internal server error - missing params in AuthPassword()");
	return -1;
    }
	
    AuthTypes = AuthTypesEnabled(&AuthTypesCnt);
    if (!AuthTypes) {
	SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE,
		"Internal server error - cannot find any authtypes");
	return -1;
    }

    /*
     * Try each type of password authentication until we run out
     * of types or one type returns something other than AUTH_STATE_NONE.
     */
    for (i = 0; State == AUTH_STATE_NONE && i < AuthTypesCnt; ++i) {
	if (EQ(AuthTypes[i], AUTH_TYPE_USER)) {
	    State = AuthPwdUser(s, User, Password);
	} else if  (EQ(AuthTypes[i], AUTH_TYPE_PASSWD)) {
	    State = AuthPwdPasswd(s, User, Password);
	}
    }

    s->AuthState = State;

    if (State == AUTH_STATE_GOOD) {
	SImsg(SIM_INFO, "%s auth password user %s GOOD\n",
	      SSPclientStr(s), PRTS(User));
	SSPsend(s, SSP_RC_OK, SSP_RCC_SINGLE, "Authentication succeeded");
    } else {
	SImsg(SIM_INFO, "%s auth password user %s DENIED\n",
	      SSPclientStr(s), PRTS(User));
	SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE,
		"Your authentication information is invalid");
    }

    return State;
}

/*
 * Authenticate the client
 */
extern int SSPauth(SSPsession_t *		s,
		   char **			Argv,
		   int				Argc,
		   CmdDispatch_t *		Cmd)
{
    char *			AuthType;

    if (Argc <= 1) {
	SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE,
		"Missing arguments; Usage: %s %s", 
		Cmd->Name, Cmd->Usage);
	return -1;
    }

    AuthType = Argv[1];

    if (EQ(AuthType, SSP_AUTH_PASSWORD)) {
	if (Argc < 4) {
	    SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE,
		    "Missing auth type arguments; Usage: %s %s", 
		    Cmd->Name, Cmd->Usage);
	    return -1;
	}

	return AuthPassword(s,
			    Argv[2],		/* User */
			    Argv[3]);		/* Password */
    } else {
	SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE,
		"Auth type \"%s\" is not recognized", AuthType);
	return -1;
    }
}
