/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.15 $";
#endif

/*
 * SSP commands
 */

#include "defs.h"

/*
 * Command Dispatch Table
 */
static CmdDispatch_t DispatchTable[] = {
    /* CmdName		Usage			Function	AuthReq? */
    { SSP_CMD_AUTH,	"PASSWORD user pwd",	SSPauth,	FALSE },
    { SSP_CMD_BYE,	"No arguments",		SSPbye,		FALSE },
    { SSP_CMD_CLASSLIST,"No arguments",		SSPclasslist,	TRUE },
    { SSP_CMD_NOOP,	"No arguments",		SSPnoop,	FALSE },
    { SSP_CMD_LICENSE,	"No arguments",		SSPlicense,	TRUE },
    { SSP_CMD_MEMBERS,	"No arguments",		SSPmembers,	TRUE },
    { SSP_CMD_TYPES,	"No arguments",		SSPtypes,	TRUE },
    { SSP_CMD_SEND,	"No arguments",		SSPsendCmd,	TRUE },
    { SSP_CMD_SETOPTION,"option1,option2,...",	SSPsetoption,	TRUE },
    { SSP_CMD_VERSION,	"[ client-version ]",	SSPversion,	TRUE },
    { 0 },
};

/*
 * If the client specified Argv[1] take that as the client telling us
 * what it's version number is.
 * If Argv[1] is not set, return our (server) version.
 */
extern int SSPversion(SSPsession_t *		s,
		      char **			Argv,
		      int			Argc,
		      CmdDispatch_t *		Cmd)
{
    char *			sv = NULL;
    char *			cv = NULL;

    if (Argc >= 2 && Argv[1] && Argv[1][0]) {
	SImsg(SIM_INFO, "%s version is %s\n",
	      SSPclientStr(s), PRTS(Argv[1]));
	SSPsend(s, SSP_RC_OK, SSP_RCC_SINGLE, "I always wanted to know that");
    } else {
	sv = ServerVersion();
	cv = SIEversion();
	SSPsend(s, SSP_RC_OK, SSP_RCC_SINGLE,
		"SSP=%d SERVER=%s SIE=%s",
		SSP_PROTO_VERSION, PRTS(sv), PRTS(cv));
    }

    return 0;
}

/*
 * Set options
 */
extern int SSPsetoption(SSPsession_t *		s,
			char **			Argv,
			int			Argc,
			CmdDispatch_t *		Cmd)
{
    int				i;

    if (Argc <= 1) {
	SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE, "No options where specified.");
	return -1;
    }

    /*
     * Try to set each option specified by client
     */
    for (i = 1; i < Argc; ++i) {
	if (SIEoptionSet(Argv[i]) < 0) {
	    SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE,
		    "Option \"%s\" is not recognized", Argv[i]);
	    return -1;
	}
    }

    SSPsend(s, SSP_RC_OK, SSP_RCC_SINGLE, "Those options look good to me");

    return 0;
}

/*
 * The CLASSLIST command returns a comma list of supported class names.
 */
extern int SSPclasslist(SSPsession_t *		s,
			char **			Argv,
			int			Argc,
			CmdDispatch_t *		Cmd)
{
    char **			List;
    static char *		ListStr;

    if (!ListStr) {
	/* Fetch the list */
	if (List = SIEclassList())
	    /* Format it into a comma seperated string */
	    if (ArgvToStr(&ListStr, List, 0, ",", 0) != 0)
		/* Failure */
		ListStr = NULL;
    }

    if (ListStr) {
	SSPsend(s, SSP_RC_OK, SSP_RCC_SINGLE, "%s", ListStr);
	return 0;
    } else {
	SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE,
		"class information is not available");
	return -1;
    }
}

/*
 * The NOOP command does nothing but return success.
 */
extern int SSPnoop(SSPsession_t *		s,
		   char **			Argv,
		   int				Argc,
		   CmdDispatch_t *		Cmd)
{
    SSPsend(s, SSP_RC_OK, SSP_RCC_SINGLE, "I am a slacker");

    return 0;
}

/*
 * The BYE command ends the session
 */
extern int SSPbye(SSPsession_t *		s,
		  char **			Argv,
		  int				Argc,
		  CmdDispatch_t *		Cmd)
{
    /* SSPclose() sends the BYE response */
    SSPclose(s);

    return 0;	/* NOTREACHED */
}

/*
 * The LICENSE command sends the client info about the server side license's
 */
extern int SSPlicense(SSPsession_t *		s,
		      char **			Argv,
		      int			Argc,
		      CmdDispatch_t *		Cmd)
{
    char **		List = NULL;
    int			ListCnt;
    int			i;

    ListCnt = SIElicense(&List);
    if (ListCnt > 0) {
	/*
	 * Use the START protocol even though we're only returning a
	 * single line so that we can add additional license lines
	 * later without a protocol change.
	 */
	SSPsend(s, SSP_RC_START, SSP_RCC_SINGLE, "By your command");
	for (i = 0; i < ListCnt; ++i) {
	    SSPwrite(s, List[i], strlen(List[i]));
	    SSPwrite(s, "\n", 1);
	    (void) free(List[i]);
	}
	(void) free(List);
	SSPsend(s, SSP_RC_OK, SSP_RCC_SINGLE, "That's all she wrote");
	return 0;
    } else {
	SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE, "No license info was found.");
	return -1;
    }
}

/*
 * The MEMBERS command sends the client info about class members
 */
extern int SSPmembers(SSPsession_t *		s,
		      char **			Argv,
		      int			Argc,
		      CmdDispatch_t *		Cmd)
{
    char **		List = NULL;
    int			ListCnt;
    int			i;

    ListCnt = SIEmembers(&List);
    if (ListCnt > 0) {
	/*
	 * Use the START protocol even though we're only returning a
	 * single line so that we can add additional license lines
	 * later without a protocol change.
	 */
	SSPsend(s, SSP_RC_START, SSP_RCC_SINGLE, "By your command");
	for (i = 0; i < ListCnt; ++i) {
	    SSPwrite(s, List[i], strlen(List[i]));
	    SSPwrite(s, "\n", 1);
	    (void) free(List[i]);	/* Free as we go */
	}
	(void) free(List);
	SSPsend(s, SSP_RC_OK, SSP_RCC_SINGLE, "What more do you want?");
	return 0;
    } else {
	SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE, "No members info was found.");
	return -1;
    }
}

/*
 * The TYPES command sends the client info about types
 */
extern int SSPtypes(SSPsession_t *		s,
		    char **			Argv,
		    int				Argc,
		    CmdDispatch_t *		Cmd)
{
    char **		List = NULL;
    int			ListCnt;
    int			i;

    ListCnt = SIEtypes(&List);
    if (ListCnt > 0) {
	/*
	 * Use the START protocol even though we're only returning a
	 * single line so that we can add additional license lines
	 * later without a protocol change.
	 */
	SSPsend(s, SSP_RC_START, SSP_RCC_SINGLE, "By your command");
	for (i = 0; i < ListCnt; ++i) {
	    SSPwrite(s, List[i], strlen(List[i]));
	    SSPwrite(s, "\n", 1);
	    (void) free(List[i]);	/* Free as we go */
	}
	(void) free(List);
	SSPsend(s, SSP_RC_OK, SSP_RCC_SINGLE, "What more do you want?");
	return 0;
    } else {
	SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE, "No types info was found.");
	return -1;
    }
}

/*
 * Find command Name in the dispatch table and return it.
 */
extern CmdDispatch_t *SSPcmdGet(SSPsession_t *	s,
				char *		Name)
{
    CmdDispatch_t *		Cmd;

    for (Cmd = DispatchTable; Cmd && Cmd->Name; ++Cmd)
	if (EQ(Cmd->Name, Name))
	    return Cmd;

#if 0
    Debug("SSPcmdGet: Command <%s> is unknown.", Name);
#endif

    return (CmdDispatch_t *) NULL;
}

