/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.4 $";
#endif

/*
 * SSP SEND command
 */

#include "defs.h"

/*
 * Send the client each class of data specified in Argv.
 */
extern int SSPsendCmd(SSPsession_t *		s,
		      char **			Argv,
		      int			Argc,
		      CmdDispatch_t *		Cmd)
{
    int				i;
    char *			Class;
    char *			Line;
    char **			ValidClasses;
    char **			Classes;
    int				ClassCnt;
    int				Status = 0;
    CIOhandle_t *		Handle;

    if (Argc <= 1) {
	SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE,
		"You must specify a comma seperated list of class names to send");
	return -1;
    }

    /*
     * Get a comma seperated list of class names
     */
    ClassCnt = StrToArgv(Argv[1], ",", &Classes, NULL, 0);
    if (ClassCnt <= 0) {
	SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE, "Could not parse class list");
	return -1;
    }

    /*
     * Get list of valid class names for later validation against request
     */
    ValidClasses = SIEclassList();
    if (!ValidClasses) {
	SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE,
		"Server side error - Could not get list of valid classes");
	return -1;
    }

    /*
     * Validate each Class requested
     */
    for (i = 0; i < ClassCnt; ++i) {
	Class = Classes[i];
	if (EQ(Class, "hardware"))
	    Class = "device";		/* XXX special hack check */
	if (!StrArray(Class, ValidClasses)) {
	    SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE,
		    "Class \"%s\" is not valid.", Class);
	    return -1;
	}
    }

    SSPsend(s, SSP_RC_START, SSP_RCC_SINGLE, "Hope you're hungry");

    for (i = 0; i < ClassCnt; ++i) {
	Class = Classes[i];
	if (!Class) {
	    SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE,
		    "A zero length class name is not acceptable");
	    Status = -1;
	    break;
	}

	Handle = SIEclassSet(Class);
	if (!Handle) {
	    SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE,
		    "Class \"%s\" is not valid", Class);
	    Status = -1;
	    break;
	}

	SImsg(SIM_INFO, "%s send class %s\n", SSPclientStr(s), Class);

	while ((Status >= 0) && (Line = SIEclassGet(Handle))) {
	    if (SSPwrite(s, Line, strlen(Line)) < 0) {
		SImsg(SIM_GERR,
		      "SSPwrite() failed while writing <%s> from class <%s>",
		      PRTS(Line), Class);
		SSPsend(s, SSP_RC_ERR, SSP_RCC_SINGLE,
			"A write error occured while sending class data");
		SIEclassEnd(Handle);
		Status = -1;
		break;
	    }
	}

	/* We're done with this Handle */
	SIEclassEnd(Handle);
    }

    if (Status == 0)
	SSPsend(s, SSP_RC_OK, SSP_RCC_SINGLE, "That's all there is");

    /* Free memory */
    DestroyArgv(&Classes, ClassCnt);

    return Status;
}
