/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.9 $";
#endif

/*
 * SSP Start
 */

#include "defs.h"
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>

/*
 * Start an SSP session
 */
extern SSPsession_t *SSPstart()
{
    SSPsession_t *		s;
    struct sockaddr_in		cs;
    struct hostent *		hp;
    char *			cp;
    int				sp = 0;		/* Socket Ptr */
#if defined(_XOPEN_SOURCE_EXTENDED)
    socklen_t			sl;
#else
    int				sl;
#endif
    int				State;
    int				NeedAuth = FALSE;

    s = (SSPsession_t *) xcalloc(1, sizeof(SSPsession_t));

    /*
     * File descriptors
     */
    s->ReadFD = fileno(stdin);
    s->ReadFP = stdin;
    s->WriteFD = fileno(stdout);
    s->WriteFP = stdout;

    Debug("SSPsession: ReadFD=%d ReadFP=0x%x WriteFD=%d WriteFP=0x%x",
	  s->ReadFD, s->ReadFP, s->WriteFD, s->WriteFP);

    sl = sizeof(cs);
    if (getpeername(sp, (struct sockaddr *) &cs, &sl) < 0) {
	SImsg(SIM_GERR, "Unable to obtain client address from socket: %s",
	      SYSERR);
    } else {
	/* Success */
	if (cp = inet_ntoa(cs.sin_addr))
	    s->ClientIP = strdup(cp);
	s->ClientPort = ntohs(cs.sin_port);

	if (hp = gethostbyaddr((char *) &cs.sin_addr, sizeof(cs.sin_addr),
			       AF_INET)) {
	    if (hp->h_name)
		s->ClientName = strdup(hp->h_name);
	} else {
	    SImsg(SIM_GERR, "Client's IP address %s does not resolve",
		  s->ClientIP);
	}
    }

    if (!s->ClientIP) {
	SImsg(SIM_CERR,
	      "Cannot determine IP address of client; terminating connection");
	SSPsend(s, SSP_RC_ERR, SSP_RCC_MULTI,
		"Cannot determine your IP address");
	SSPclose(s);	/* Sends SSP_RC_BYE */
    }

    /*
     * Authenticate connection
     */
    State = Authorized(s);

    /*
     * Terminate connection if auth is denied
     */
    if (State == AUTH_STATE_DENIED) {
	SImsg(SIM_CERR, "%s unauthorized terminating connection",
	      SSPclientStr(s));
	SSPsend(s, SSP_RC_ERR, SSP_RCC_MULTI,
		"%s You are not authorized",
		(s->ClientName) ? s->ClientName : s->ClientIP);
	SSPclose(s);	/* Sends SSP_RC_BYE */
	return (SSPsession_t *) NULL;
    }

    /*
     * If we're not yet authorized tell the client that she'll
     * need to authorize herself via SSP_CMD_AUTH
     */
    if (State != AUTH_STATE_GOOD)
	NeedAuth = TRUE;

    SImsg(SIM_INFO, "%s connection setup %s\n",
	  SSPclientStr(s), (NeedAuth) ? "NEEDAUTH" : "");

    if (NeedAuth) {
	SSPsend(s, SSP_RC_OK, SSP_RCC_MULTI,
		"SSP MagniComp SysInfo Server %s",
		ServerVersion());
	SSPsend(s, SSP_RC_AUTHREQ, SSP_RCC_SINGLE,
		"You won't be able to do anything until you talk %s with me",
		SSP_CMD_AUTH);
    } else {
	SSPsend(s, SSP_RC_OK, SSP_RCC_SINGLE,
		"SSP MagniComp SysInfo Server %s",
		ServerVersion());
    }

    return s;
}
