/*
 * $Id: sample1.c,v 1.9 2003/03/11 18:34:25 mcooper Exp $
 *
 * Sample of how to use the mcsysinfoc(3) API.
 *
 * Compile with:
 *
 *	./configure
 *	make
 *
 * NOTES:
 * - Make sure to use the rules in ./configure to set LARGE_T or else you
 *   will experience core dumps due to a mismatch in what -lmcsysinfo uses
 *   for the sizeof(Large_t) and what your program uses.
 */

#include <stdio.h>
#include "mcsysinfo.h"

#define PRTS(s)	( s && *(s) ) ? s : ""
#define Eq(a,b)	(a && b && (strcmp(a,b) == 0))

/*
 * Directory containing SysInfo .cf files
 */
static char *ConfigDir = "/opt/sysinfo/config";

#if MCSYSINFO_VERSION_NUMBER < 50000	/* Pre SysInfo 5.0 */
char * ProgramName = "sample1";
#endif

/*
 * Show general class info.
 * This function obtains a sampling of GENERAL class data.  Many other
 * GENERAL class types are available beyond those shown below.
 */
void General()
{
    int s;
    char **pp = NULL;
    static MCSIquery_t Query;

    printf("\nGENERAL class data\n");

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Cmd = MCSI_HOSTNAME;
    s = mcSysInfo(&Query);
    if (s == 0)
	printf ("Hostname is <%s>\n", PRTS((char *) Query.Out));
    else
	fprintf(stderr, "mcSysInfo get MCSI_HOSTNAME failed with status %d.\n",s);

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Cmd = MCSI_HOSTALIASES;
    s = mcSysInfo(&Query);
    if (s == 0) {
	printf ("Host Alias(es) is");
	for (pp = (char **) Query.Out; pp && *pp && **pp; ++pp)
	    printf (" %s", PRTS(*pp));
	printf ("\n");
    }

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Cmd = MCSI_HOSTADDRS;
    s = mcSysInfo(&Query);
    if (s == 0) {
	printf ("Host Address(es) is");
	for (pp = (char **) Query.Out; pp && *pp && **pp; ++pp)
	    printf (" %s", PRTS(*pp));
	printf ("\n");
    }

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Cmd = MCSI_MAN;
    s = mcSysInfo(&Query);
    if (s == 0) {
	printf ("System Mfg is <%s>\n", PRTS((char *) Query.Out));
    } else {
	fprintf(stderr, "mcSysInfo get MCSI_MAN failed with status %d.\n",s);
    }

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Cmd = MCSI_MODEL;
    s = mcSysInfo(&Query);
    if (s == 0) {
	printf ("System Model is <%s>\n", PRTS((char *) Query.Out));
    } else {
	fprintf(stderr, "mcSysInfo get MCSI_MODEL failed with status %d.\n",s);
    }
}

/*
 * Set our options for mcSysInfo() calls.
 */
void SetOptions(char * Program, int DebugFlag, char *CfDir)
{
    static MCSIquery_t Query;
    static McsiOpt_t McsiOpts;
    int s;

    /* Enable debugging */
    if (DebugFlag)
	McsiOpts.Flags |= MCSIOPT_DEBUG;
    /* Set path to this program */
    McsiOpts.ProgramPath = Program;
    /* Directory containing sysinfo .cf files */
    McsiOpts.ConfigDir = (CfDir) ? CfDir : ConfigDir;
    /* Our own .cf file if any */
    McsiOpts.ConfigFile = NULL;

    memset(&Query, 0, sizeof(Query));
    Query.Op = MCSIOP_SETOPTS;
    Query.In = &McsiOpts;
    Query.InSize = sizeof(McsiOpts);
    s = mcSysInfo(&Query);
    if (s != 0) {
	fprintf(stderr, "Set MCSI Options failed with status %d.\n", s);
    }
}

/*
 * Show info about a disk drive
 */
void ShowDisk(DiskDrive_t *Disk, char *Type)
{
    if (!Disk)
	return;

    printf("\t%s DataCyl=%ld PhyCyl=%ld Tracks=%ld Sects=%ld Size=%.0fMB\n",
	   PRTS(Type),
	   Disk->DataCyl, Disk->PhyCyl, Disk->Tracks,
	   Disk->Sect, Disk->Size);
}

/*
 * Show info for device DevInfo then recursively call on peers/children.
 * --RECURSE--
 */
void HardwareShow(DevInfo_t *DevInfo)
{
    DiskDriveData_t *DriveData;
    CpuInfo_t *Cpu;
    NetIF_t *NetIf;

    if (DevInfo->Name) {
	printf("Name=<%s> Vendor=<%s> Model=<%s> ModelDesc=<%s> Capacity=<%s> Speed=<%s>\n",
	       PRTS(DevInfo->Name),
	       PRTS(DevInfo->Vendor),
	       PRTS(DevInfo->Model),
	       PRTS(DevInfo->ModelDesc),
	       PRTS(DevInfo->Capacity),
	       PRTS(DevInfo->Speed));
	switch (DevInfo->Type) {
	case DT_DISKDRIVE:
	    if (DriveData = (DiskDriveData_t *) DevInfo->DevSpec) {
		ShowDisk(DriveData->HWdata, "HW");
		ShowDisk(DriveData->OSdata, "OS");
	    }
	    break;
	case DT_CPU:
	    if (Cpu = (CpuInfo_t *) DevInfo->DevSpec) {
		printf("\tMan=<%s> Model=<%s> Arch=<%s> Bits=%d Speed=%ldMHz\n",
		       PRTS(Cpu->Man), PRTS(Cpu->Model),
		       PRTS(Cpu->Arch), Cpu->BitSize, Cpu->Speed);
	    }
	    break;
	case DT_NETIF:
	    if (NetIf = (NetIF_t *) DevInfo->DevSpec) {
		printf("\tCurrent MAC addr=<%s> MAC name=<%s>\n",
		       PRTS(NetIf->MACaddr), PRTS(NetIf->MACname));
	    }
	    break;
	}
    }

    /* Descend */
    if (DevInfo->Slaves) {
	HardwareShow(DevInfo->Slaves);
    }

    /* Traverse */
    if (DevInfo->Next) {
	HardwareShow(DevInfo->Next);
    }
}

/*
 * Get Hardware (Device) data
 */
void Hardware()
{
    static MCSIquery_t Query;
    DevInfo_t *RootDev;
    int s;

    printf("\nHARDWARE class data\n");

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Op = MCSIOP_CREATE;
    Query.Cmd = MCSI_DEVTREE;
    s = mcSysInfo(&Query);
    if (s != 0) {
	fprintf(stderr, "mcSysInfo MCSI_DEVTREE failed with status %d.\n", s);
	return;	/* Error */
    }

    /* Extract the root of the hardware tree */
    RootDev = (DevInfo_t *) Query.Out;
    HardwareShow(RootDev);
}

/*
 * Show partition info
 */
void Partition()
{
    static MCSIquery_t Query;
    PartInfo_t *Root;
    PartInfo_t *Part;
    int s;

    printf("\nPARTITION class data\n");

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Op = MCSIOP_CREATE;
    Query.Cmd = MCSI_PARTITION;
    s = mcSysInfo(&Query);
    if (s != 0) {
	fprintf(stderr, "mcSysInfo MCSI_PARTITION failed with status %d.\n", s);
	return;	/* Error */
    }

    /* Extract the root of the hardware tree */
    Root = (PartInfo_t *) Query.Out;

    for (Part = Root; Part; Part = Part->Next) {
	printf("BaseName=<%s> DevPath=<%s> Size=%lld MB\n",
	       PRTS(Part->BaseName), PRTS(Part->DevPath),
	       Part->Size);
    }
}

/*
 * Main
 */
main(int Argc, char ** Argv)
{
    int i;
    int DebugFlag = 0;
    char *CfDir = NULL;

    if (getuid() != 0) {
	fprintf(stderr, "warning: You must run as root (uid=0) to get full data from mcSysInfo().\n");
    }

    for (i = 1; i < Argc; ++i) {
	if (Eq(Argv[i], "--debug"))
	    DebugFlag = 1;
	else if (Eq(Argv[i], "--cfdir"))
	    CfDir = Argv[++i];
	else {
	    fprintf(stderr, "argument \"%s\" is unknown.\n", Argv[i]);
	    exit(1);
	}
    }

    SetOptions(Argv[0], DebugFlag, CfDir);

    General();
    Hardware();
    Partition();
}
