/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.13 $";
#endif

/*
 * Check Space functions
 */

#include "defs.h"

/*
 * Get available disk space for Dir
 */
static char *GetAvail(Dir, OSname)
     char		       *Dir;
     char		       *OSname;
{
    static char			Cmd[8192];
    char		       *Cmds[2];
    char		       *Avail = NULL;

    SetupDebug("GetAvail: OSname=<%s> Dir=<%s>",
	       OSname, Dir);

    if (EQ(OSname, "SunOS")) 
	snprintf(Cmd, sizeof(Cmd), 
		 "/usr/bin/df -b %s | /bin/tail -1 |/bin/awk '{print $2}'",
		 Dir);
    else if (EQ(OSname, "AIX")) 
	snprintf(Cmd, sizeof(Cmd), 
		 "/usr/bin/df -k %s | /bin/tail -1 |/bin/awk '{print $3}'",
		 Dir);
    else if (EQ(OSname, "Linux")) 
	snprintf(Cmd, sizeof(Cmd), 
		 "/bin/df -P %s | /usr/bin/tail -1 |/bin/awk '{print $4}'",
		 Dir);
    else if (EQ(OSname, "HPUX") || EQ(OSname, "HP-UX")) 
	snprintf(Cmd, sizeof(Cmd), 
		 "/bin/df -k %s | /bin/grep free | /bin/awk '{print $1}'",
		 Dir);
    else if (EQ(OSname, "IRIX") || EQ(OSname, "IRIX64"))
	snprintf(Cmd, sizeof(Cmd), 
		 "/bin/df %s | /bin/tail -1 | /bin/awk '{print $5}'",
		 Dir);
    else if (EQ(OSname, "FreeBSD"))
	snprintf(Cmd, sizeof(Cmd), 
		 "/bin/df %s | /usr/bin/tail -1 | /usr/bin/awk '{print $4}'",
		 Dir);
    else if (EQ(OSname, "BSD/OS"))
	snprintf(Cmd, sizeof(Cmd), 
		 "/bin/df %s | /usr/bin/tail -1 | /usr/bin/awk '{print $4}'",
		 Dir);
    else if (REmatch(OSname, ".*BSD.*", NULL) > 0)
	snprintf(Cmd, sizeof(Cmd), 
		 "/bin/df %s | /bin/tail -1 | /bin/awk '{print $4}'",
		 Dir);
    else if (EQ(OSname, "MacOSX"))
	snprintf(Cmd, sizeof(Cmd), 
		 "/bin/df %s | /usr/bin/tail -1 | /usr/bin/awk '{print $4}'",
		 Dir);
    else {
	Error("CheckSpace: No method defined for <%s>.", OSname);
	return (char *) NULL;
    }

    Cmds[0] = Cmd;
    Cmds[1] = NULL;
    if (Avail = (char *) RunCmds(Cmds, 0))
	return Avail;
    else
	SetupDebug("Cannot determine amount of available disk space for <%s>.",
		   Dir);

    return (char *) NULL;
}

/*
 * Check to see if there's suffucient disk space
 */
extern int CheckSpace(Product)
     McdProduct_t	       *Product;
{
    char		       *AvailStr = NULL;
    char		       *Dir;
    char		       *OSname;
    char		       *cp;
    Large_t			AvailSpace;
    Large_t			NeedSpace;
    struct stat			Stat;

    Msg(0, "Checking available disk space . . . ");

    OSname = GetSysInfo(MCSI_OSNAME);
    if (!OSname) {
	Error("Cannot determine OS name to check disk space.");
	return -1;
    }

    for (Dir = strdup(Product->InstallDir);
	 Dir && strlen(Dir) && !AvailStr; ) {

	SetupDebug("CheckSpace looking for <%s>", Dir);

	if (stat(Dir, &Stat) == 0) {
	    if (S_ISDIR(Stat.st_mode))
		AvailStr = GetAvail(Dir, OSname);
	    else
		SetupDebug("CheckSpace: <%s> not a directory", Dir);
	} else {
	    SetupDebug("CheckSpace: <%s> Stat failed: %s", Dir, SYSERR);
	}
	/* Remove trailing path entry */
	if (strlen(Dir) > 1 && (cp = strrchr(Dir, '/'))) {
	    *cp = CNULL;
	    if (strlen(Dir) == 0)
		Dir = "/";
	} else {
	    break;
	}
    }

    if (!AvailStr) {
	Error("Unable to determine amount of available disk space.");
	return -1;
    }

    /* AvailSpace is in KB */
    AvailSpace = (Large_t) strtol(AvailStr, NULL, 10);
    NeedSpace = Product->Size / KBYTES;

#if HAVE_PRINTF_LL
    SetupDebug("CheckSpace: Dir=<%s> Avail=%lld KB Need=%lld KB",
#else
    SetupDebug("CheckSpace: Dir=<%s> Avail=%ld KB Need=%ld KB",
#endif
	       Dir, AvailSpace, NeedSpace);

    if ((AvailSpace - NeedSpace) < 100) {
	Error("There is insufficient disk space to install into %s",
	      Dir);
#if HAVE_PRINTF_LL
	Error("This package requires %lld KB of disk space.", NeedSpace);
	Error("There is only %lld KB of disk space available on %s.",
	      AvailSpace, Dir);
#else
	Error("This package requires %ld KB of disk space.", NeedSpace);
	Error("There is only %ld KB of disk space available on %s.",
	      AvailSpace, Dir);
#endif
	return -1;
    }

    Msg(0, "OK\n");

    return 0;
}
