/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.8 $";
#endif

/*
 * Install files functions
 */

#include "defs.h"

static int SymLink(McdMap_t *Entry, char *InstallDir)
{
    static char 		StartDir[MAXPATHLEN];
    char *			Src;
    char *			Dest;

    Src = Entry->Src;
    Dest = Entry->Dest;

    SetupDebug("Symlink %s -> %s [InstallDir=<%s>]",
	       PRTS(Src), PRTS(Dest), PRTS(InstallDir));

    if (!Src) {
	Error("SymLink: No source file was given for destination <%s>",
	      PRTS(Dest));
	return -1;
    }
    if (!Dest) {
	Error("SymLink: No destination file was given for source <%s>",
	      PRTS(Src));
	return -1;
    }

    /* Save current dir */
    if (!getcwd(StartDir, sizeof(StartDir))) {
	Error("SymLink: Cannot getcwd: %s", SYSERR);
	return -1;
    }

    /*
     * Go to the install dir in case the links are relative (very likely)
     */
    if (chdir(InstallDir) != 0) {
	Error("SymLink: Cannot chdir to %s: %s", InstallDir, SYSERR);
	return -1;
    }

    /* Insure the link file we want to make doesn't exist or symlink fails */
    (void) unlink(Src);

    /*
     * Make the actual link
     */
    if (symlink(Dest, Src) != 0) {
	Error("Symlink %s -> %s failed: %s", Src, Dest, SYSERR);
	return -1;
    }

    /* Go back to where we started */
    if (chdir(StartDir) != 0) {
	Error("SymLink: Cannot chdir to %s: %s", StartDir, SYSERR);
	return -1;
    }

    return 0;
}

/*
 * Install the actual files
 */
extern int InstallFiles(McdProduct_t *	Product,
			char *		DistDir)
{
    McdMap_t *		        Entry;
    McdPackage_t *		Pkg;
    char		       *Src;
    static char		        Dest[MAXPATHLEN];
    static char			Prompt[128];
    struct stat			Stat;
    int				ErrCount = 0;
    int				Padding = 0;
    static Meter_t		MeterInfo;

    /* Calculate Padding based on longest Pkg->Name */
    Padding = McdPkgPadding(Product->Packages);

    for (Pkg = Product->Packages; Pkg; Pkg = Pkg->Next) {
	if (!Pkg->Selected)
	    continue;

	memset(&MeterInfo, 0, sizeof(MeterInfo));
	MeterInfo.TermWidth = TermWidth;

	for (MeterInfo.Total = 0, Entry = Pkg->MapList; Entry;
	     Entry = Entry->Next)
	    ++(MeterInfo.Total);

	snprintf(Prompt, sizeof(Prompt), "Installing %-*s . . . ",
		 Padding, Pkg->Name);
	MeterInfo.Prompt = Prompt;

	if (DebugFlag)
	    McdMapDump(Pkg->MapList);

	for (Entry = Pkg->MapList; Entry; Entry = Entry->Next) {
	    ++(MeterInfo.Current);
	    Meter(&MeterInfo);

	    snprintf(Dest, sizeof(Dest), "%s/%s",
		     Product->InstallDir, Entry->Dest);
	    if (Entry->Src && Entry->Src[0] == '/')
		/* Entry->Src is absolute path */
		Src = Entry->Src;
	    else if (Entry->Src)
		/* Path is relative to DistDir */
		Src = GetDistPath(Entry->Src, DistDir, Product, Pkg);
	    else
		Src = NULL;

	    switch (Entry->Type) {
	    case MCDT_FILE:
		(void) unlink(Dest);
		if (FLAGS_ON(Entry->Flags, MCDF_EDIT)) {
		    if (CopyEdit(Src, Dest, Product) != 0)
			Bail();
		} else {
		    if (Copy(Src, Dest) != 0)
			Bail();
		}
		break;
	    case MCDT_DIR:
		if (MakeDir(Dest, Entry->Mode) != 0)
		    Bail();
		break;
	    case MCDT_SYMLINK:
		if (SymLink(Entry, Product->InstallDir) != 0)
		    Bail();
		continue;	/* Skip SetPerms() */
		break;
	    default:
		Error("Type %d for entry \"%s\" is unknown.",
		      Entry->Type, (Entry->Src) ? Entry->Src : Entry->Dest);
		Bail();
	    }
	    if (SetPerms(Dest, Entry->Owner, Entry->Group, Entry->Mode) < 0) 
		Bail();
	}
	Msg(0, "\n");
    }

    Msg(0, "\n");
    if (ErrCount) {
	Msg(0, "\nThere were %d errors during installation.\n", ErrCount);
	return -1;
    } else {
	return 0;
    }
}
