/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.23 $";
#endif

/*
 * Setup installation functions
 */

#include "defs.h"
#include <unistd.h>

static char			       *Answer = NULL;
static char			       *AnswersList[3] = {
    A_ALL, A_NONE, NULL
};

/*
 * Add special file to mcd list
 */
static int AddMcdFile(char *		File,
		      int		Optional,
		      char *		PkgName,
		      char *		Dir,
		      char *		DstDir,
		      McdProduct_t *	Product)
{
    static char		Path[MAXPATHLEN];
    static char		Dst[MAXPATHLEN];

    snprintf(Path, sizeof(Path), "%s/%s/%s", Dir, PkgName, File);

    if (!FileExists(Path, FALSE)) {
	if (Optional) {
	    SetupDebug("AddMcdFile: Optional file does not exist: %s", Path);
	    return 0;
	} else {
	    Error("AddMcdFile: File <%s> does not exist.", Path);
	    return -1;
	}
    }

    /* Files are placed under a product subdir */
    snprintf(Dst, sizeof(Dst), "%s/%s/%s/%s",
	     DstDir,
	     FN_PACKAGES_DIR,
	     PkgName,
	     File);

    /* Add to list */
    McdPkgAdd(Product->Packages, MCDT_FILE, strdup(Path),
	      strdup(Dst), 0444);

    return 0;
}

/*
 * Add mcd files to list of files to install as part of first package.
 */
static int AddMcdFiles(McdProduct_t *	Product,
		       McdData_t *	McdData)
{
    static char		Cwd[MAXPATHLEN];
    static char		Path[MAXPATHLEN];
    static char		DstDir[MAXPATHLEN];
    char *		Dir;
    char *		cp;
    char **		PkgNames;
    char **		cpp;

    if (!Product || !McdData) {
	Error("AddMcdFiles: Missing params.");
	return -1;
    }

    /* Top level destination dir */
    snprintf(DstDir, sizeof(DstDir), "%s/%s",
	     FN_MCD, Product->Name);

    /*
     * General stuff
     */
    McdPkgAdd(Product->Packages, MCDT_DIR, NULL, FN_MCD, 0755);

    /*
     * Add ourself, but make sure to use full pathname.
     */
    if (McdData->ProgramPath) {
	if (McdData->ProgramPath[0] != '/') {
	    if (!getcwd(Cwd, sizeof(Cwd))) {
		Error("AddMcdFiles: Get current directory failed: %s", SYSERR);
		return -1;
	    }
	    snprintf(Path, sizeof(Path), "%s/%s", Cwd, McdData->ProgramPath);
	    cp = Path;
	} else {
	    cp = McdData->ProgramPath;
	}
	SetupDebug("AddMcdFiles: adding progpath=<%s> name=<%s>",
		   PRTS(cp), PRTS(McdData->ProgramName));
	if (cp)
	    McdPkgAdd(Product->Packages, MCDT_FILE, strdup(cp),
		      McdData->ProgramName, 0555);
    }

    snprintf(Path, sizeof(Path), "%s/%s", DstDir, FN_MCDPRODUCT);
    McdPkgAdd(Product->Packages, MCDT_FILE, Product->ProductFile,
	      strdup(Path), 0444);

    /*
     * Scan the "mcd" dir in the distribution directory for all files/dirs
     * and add those to the base (first) package for installation.
     */
    Dir = McdPath(Product, NULL, McdData->DistDir, FN_PACKAGES_DIR);

    /*
     * Get list of all names found in Dir.
     * This should be a list of package names for this product.
     */
    PkgNames = DirGetNames(Dir, FALSE, NULL);

    for (cpp = PkgNames; cpp && *cpp; ++cpp) {
	AddMcdFile(FN_MCDPKG_FILE, FALSE, *cpp, Dir, DstDir, Product);
	AddMcdFile(FN_MCDMAP_FILE, FALSE, *cpp, Dir, DstDir, Product);
	AddMcdFile(FN_MCDPOST, TRUE, *cpp, Dir, DstDir, Product);
    }

    DirFreeNames(PkgNames);

    return 0;
}

/*
 * Update Product->Size based on selected packages
 */
extern int UpdateSize(McdProduct_t *Product)
{
    McdPackage_t *			Pkg;
    Large_t				Size = 0;

    if (!Product)
	return -1;

    for (Pkg = Product->Packages; Pkg; Pkg = Pkg->Next) {
	if (Pkg->Selected)
	    Size += Pkg->Size;
    }

    if (Size)
	Product->Size = Size;

    return 0;
}

/*
 * Installation
 */
extern int InstallProduct(McdData_t *		McdData,
			  McdProduct_t *	Product)
{
    char *			DistDir;
    static char			Path[MAXPATHLEN];
    McdPackage_t *		Pkg;
    int				i;
    int				Err = 0;
    int				DoIt = FALSE;

    DistDir = McdData->DistDir;

    for (i = 0; i < SETUP_MAX_PROMPTS; ++i) {
	/* Select required, unselect non-required */
	(void) McdPkgSelectSet(Product, McdData, FALSE, A_ALL, TRUE);

	Msg(0, "\nThis product is composed of one or more packages.  Some\n");
	Msg(0, "packages are required (REQ=YES) while others are optional.\n");
	Msg(0, "You may elect not to install any package which is not required.\n");

	ShowSummaryPkgs(Product->Packages, Product, FALSE, 0);
	Msg(0, "\n");
	Answer = Ask("Which packages would you like to install?",
		     AnswersList,
		     A_ALL,
		     "Enter comma seperated list of PKG#'s (i.e. \"1,2,4\") or \"all\" or \"none\".",
		     AF_NUMLIST);
	SetupDebug("Ask() user returned <%s>", PRTS(Answer));
	if (EQ(Answer, A_NONE)) {
	    Error("You have choosen not to proceed with installation.");
	    return -1;
	}

	/* Select the packages per the user's answer */
	if (McdPkgSelectSet(Product, McdData, TRUE, Answer, TRUE) < 0)
	    continue;

	/* Insure that all Required packages are selected */
	for (Err = 0, Pkg = Product->Packages; Pkg; Pkg = Pkg->Next) {
	    if (Pkg->Required && !Pkg->Selected) {
		Error("PKG# %d (%s) is required, but was not selected.",
		      Pkg->Number, PRTS(Pkg->Name));
		++Err;
	    }
	}

	if (Err) {
	    /*
	     * If the user specify a PkgList, then this is immediately fatal
	     */
	    if (McdData->PkgList) {
		if (!OverRide) {
		    Error("One or more required packages was not selected.");
		    return -1;
		} else {
		    /* OverRide is on so proceed */
		    DoIt = TRUE;
		    break;
		}
	    }
	    /* Re-prompt user */
	    continue;
	}

	UpdateSize(Product);

	/* Show summary of selected packages */
	ShowSummaryPkgs(Product->Packages, Product, TRUE, 0);
	Msg(0, "\n");
	Answer = Ask("Are the packages selected what you want?",
		     AnswersYN, A_YES, NULL, 0);
	if (EQ(Answer, A_YES)) {
	    /* Do it */
	    DoIt = TRUE;
	    break;
	}
    }

    if (Err && !OverRide) {
	Error("One or more required packages was not selected.");
	return -1;
    }

    if (!DoIt) {
	Error("Installation is canceled..");
	return -1;
    }

    if (InventoryProduct(Product, McdData) < 0)
	return -1;

    if (LicenseAccept(Product, McdData) < 0)
	return -1;

    if (SysReq(Product) != 0) {
	Error("Your system does not meet the requirements for this software.");
	if (OverRide) {
	    Msg(0, "Continue? YES\n");
	} else {
	    Answer = Ask("Continue?", AnswersYN, NULL, NULL, 0);
	    if (!ConfirmFlag && !EQ(Answer, A_YES))
		UserBail();
	}
    }

    if (McdData->InstallDir)
	Product->InstallDir = McdData->InstallDir;
    else {
	Answer = Ask("What directory do you want to install into?", 
		     NULL, Product->InstallDir, NULL, 0);
	if (Answer)
	    McdData->InstallDir = Product->InstallDir = strdup(Answer);
    }

    if (CheckSpace(Product) < 0) {
	Answer = Ask("Do you wish to continue anyway?",
		     AnswersYN, A_NO, NULL, 0);
	if (!EQ(Answer, A_YES))
	    return -1;
    }

    Msg(0, "\n");
    ShowSummary(Product, TRUE, 0);
    ShowSummaryPkgs(Product->Packages, Product, TRUE, 0);
    Msg(0, "\n");
    Answer = Ask("Proceed and install all files now?",
		 AnswersYN, A_YES, NULL, 0);
    if (!EQ(Answer, A_YES)) {
	Error("You have choosen not to proceed with installation.");
	return -1;
    }

    Line(LT_SINGLE);
    Msg(0, "Preparing to install into %s . . . ", Product->InstallDir);
    MakeDir(Product->InstallDir, 0755);
    Msg(0, "OK\n");

    /*
     * Add mcd files to list of files to install
     */
    AddMcdFiles(Product, McdData);

    if (InstallFiles(Product, DistDir) < 0)
	return -1;

    if (McdPkgUpdate(Product, McdData) < 0)
	return -1;

    if (McdPost(Product, FALSE) < 0) 
	return -1;

    if (InstallLicense(Product) < 0)
	return -1;

    Msg(0, "\nInstallation is now complete.\n");

    Msg(0, "\nTo uninstall this software run the following:\n");
    Msg(0, "    %s/%s --uninstall\n\n", 
	Product->InstallDir, McdData->ProgramName);

    /* Show finish message */
    MsgShowFile(FN_FINISH, Product);

    return 0;
}

/*
 * Install products
 */
extern int Install(McdData_t *McdData)
{
    McdProduct_t *		Prod;
    int				i;
    int				Status = -1;
    int				Count;

    for (i = 0; i < SETUP_MAX_PROMPTS; ++i) {
	/*
	 * Ask user to select each product for installation
	 */
	for (Prod = McdData->Products, Count = 0; Prod; Prod = Prod->Next) {
	    ShowSummary(Prod, FALSE, 0);
	    Msg(0, "\n");
	    
	    Answer = Ask("Select this product for installation?",
			 AnswersYN, A_YES, NULL, 0);
	    if (EQ(Answer, A_YES)) {
		Prod->Selected = TRUE;
		++Count;
	    } else {
		Prod->Selected = FALSE;
	    }
	}

	if (!Count) {
	    /* The user didn't select anything */
	    Msg(0, "\nYou have not selected any products for installation.\n");
	    Answer = Ask("Do you wish to select products to install?",
		     AnswersYN, A_YES, NULL, 0);
	    if (!EQ(Answer, A_YES)) {
		UserBail();
		return -1;
	    }
	    Msg(0, "\n");
	    continue;
	}

	SummaryProductList(McdData->Products, TRUE, 0);
	Msg(0, "\n");

	Answer = Ask("Is the selection of products to be installed correct?",
		     AnswersYN, A_YES, NULL, 0);
	if (EQ(Answer, A_YES)) {
	    Status = 0;
	    break;
	}
    }

    /* Check to see if any products were selected */
    if (Status < 0) {
	Error("No products were selected for installation.");
	return -1;
    }

    for (Prod = McdData->Products; Prod; Prod = Prod->Next) {
	if (InstallProduct(McdData, Prod) < 0)
	    --Status;
    }

    return Status;
}
