/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.8 $";
#endif

/*
 * Setup Inventory functions
 */

#include "defs.h"

/*
 * Perform an inventory of Product in the distribution dir
 */
extern int InventoryProduct(McdProduct_t *	Product,
			    McdData_t *		McdData)
{
    McdMap_t *			Entry;
    McdPackage_t *		Pkg;
    struct stat			Stat;
    static char			Path[MAXPATHLEN];
    int				ErrCount = 0;
    int				Success = 0;
    char *		        Src;
    char *		        cp;
    static Meter_t		MeterInfo;
    struct stat			st;

    if (!Product || !Product->Packages || !McdData->DistDir) {
	Error("Inventory: Missing params.");
	return -1;
    }

    /*
     * Test to see if this looks like the distribution dir
     */
    cp = GetDistPath(NULL, McdData->DistDir, Product, Product->Packages);
    if (!cp) {
	Error("Could not get path to inventory test path.");
	return -1;
    }
    SetupDebug("Inventory: Checking for test path <%s>", PRTS(cp));
    if (lstat(cp, &st) != 0) {
	Error("Directory \"%s\" does not appear to be the distribution directory.",
	      McdData->DistDir);
	Error("You can only use the --inventory option from the distribution directory.");
	return -1;
    }

    memset(&MeterInfo, 0, sizeof(MeterInfo));
    MeterInfo.TermWidth = TermWidth;
    MeterInfo.Prompt = "Inventory check . . . ";
    /* Calculate MeterInfo.Total */
    for (Pkg = Product->Packages; Pkg; Pkg = Pkg->Next) {
	for (Entry = Pkg->MapList; Entry; Entry = Entry->Next)
	    ++(MeterInfo.Total);
    }

    for (Pkg = Product->Packages; Pkg; Pkg = Pkg->Next) {
	for (Entry = Pkg->MapList; Entry; Entry = Entry->Next) {
	    ++(MeterInfo.Current);
	    Meter(&MeterInfo);

	    if (!Entry->Src || EQ(Entry->Src, "-") ||
		Entry->Type == MCDT_SYMLINK)
		continue;

	    Src = GetDistPath(Entry->Src, McdData->DistDir, Product, Pkg);

	    SetupDebug("Checking <%s>", Src);
	    if (stat(Src, &Stat) != 0) {
		Error("%s: Missing file detected", Src);
		++ErrCount;
		continue;
	    }
	    if (FLAGS_OFF(Entry->Flags, MCDF_EDIT) &&
		(Entry->Size > 0) &&
		(Stat.st_size != Entry->Size)) {
		Error("%s: File size mismatch - file is %d bytes expecting %d",
		      Src, Stat.st_size, Entry->Size);
		++ErrCount;
		continue;
	    }

	    /* Count our success */
	    ++Success;
	}
    }

    if (ErrCount) {
	Msg(0, "\nThere were %d errors during inventory.\n", ErrCount);
	return -1;
    } else if (Success == 0) {
	Msg(0, "\nIt does not appear we found anything to inventory.\n");
	return -1;
    } else {
	Msg(0, "\nInventory succeeded.\n");
	return 0;
    }
}

/*
 * Perform an inventory of all products in the distribution dir
 */
extern int Inventory(McdData_t *McdData)
{
    McdProduct_t *		Prod;
    int				Status = 0;

    for (Prod = McdData->Products; Prod; Prod = Prod->Next) {
	if (InventoryProduct(Prod, McdData) < 0)
	    --Status;
    }

    return Status;
}

