/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.4 $";
#endif

/*
 * License functions
 */

#include "mcbase.h"
#include "defs.h"
#include "mcl.h"

static char		       *Answer = NULL;
static char		       *MCLprogramName = "sysinfo";
extern char		       *LicenseFile;

/*
 * Return path where license file is installed
 */
extern char *GetLicensePath(McdProduct_t *Product)
{
    static char			LicFile[MAXPATHLEN];

    if (Product->InstallLic[0] == '/' || Product->InstallLic[0] == '.')
	(void) snprintf(LicFile, sizeof(LicFile), "%s", Product->InstallLic);
    else
	(void) snprintf(LicFile, sizeof(LicFile), "%s/%s", 
			Product->InstallDir, Product->InstallLic);

    return LicFile;
}

/*
 * Prompt user for name of license file to be installed.
 */
extern char *LicenseFilePrompt(McdProduct_t *Product, mcl_t *Query)
{
    int				Status;
    int				i;

    Msg(0, "This software requires a license file for permanent operation.\n");
    Msg(0, "Without a license file the software will work properly for\n");
    Msg(0, "%d days for demonstration purposes.\n\n", DEMODAYSNUM);
    Msg(0, "You may install the license file now or later by running:\n");
    Msg(0, "    %s/%s --license --licensefile <file>\n\n",
	Product->InstallDir, BaseName(ProgramName));

    for (i = 0; i < SETUP_MAX_PROMPTS; ++i) {
	Answer = Ask("Do you wish to install the license file now?", 
		     AnswersYN, A_YES, NULL, 0);
	if (!EQ(Answer, A_YES))
	    return (char *) NULL;
	    
	Msg(0, "The license file you name will be copied to:\n");
	Msg(0, "    %s\n", GetLicensePath(Product));
	Answer = Ask("Pathname to license file (e.g. license.mcl): (none)",
		     NULL, NULL, NULL, 0);

	if (EQ(Answer, "none"))
	    return Answer;

	if (FileExists(Answer, 0)) {
	    /* Valid license file */
	    Query->File = Answer;
	    Status = MCLcheck(Query);
	    if (Status != 0 || Query->Status != MCL_STAT_OK) {
		Error("%s: File does not contain a valid license.", Answer);
		continue;
	    }
	    /* Everything's fine */
	    return strdup(Answer);
	} else
	    Error("%s: The file does not exist.", Answer);
    }

    Error("Maximum Prompt limit reached.");
    return (char *) NULL;
}

/*
 * License installation
 */
extern int InstallLicense(McdProduct_t *Product)
{
    static mcl_t		Query;
    int				Status;
    char		       *FileToInstall = LicenseFile;
    char		       *LicFile;

    Msg(0, "Checking license status . . . ");

    if (!Product->InstallLic) {
	Error("Distribution error - no license file location was found.");
	Bail();
    }

    LicFile = GetLicensePath(Product);

    (void) memset(&Query, 0, sizeof(Query));
    Query.File = LicFile;
    Query.Product = MCLprogramName;
    Query.ProdVers = SYSINFO_VERSION_STR;
    if (DebugFlag)
	Query.Flags = MCLF_DEBUG;

    Status = MCLcheck(&Query);
    if (Status == 0 && (Query.Type == MCL_FULL || Query.Type == MCL_UPGRADE)) {
	Msg(0, "You already have a license.\n");
	return 0;
    }

    Msg(0, "A license file is required.\n");

    if (!FileToInstall) {
	FileToInstall = LicenseFilePrompt(Product, &Query);
	if (EQ(FileToInstall, "none"))
	    return 0;
    }

    if (!FileToInstall)
	return 0;

    Msg(0, "Installing license file %s . . . ", FileToInstall);

    if (Copy(FileToInstall, LicFile) != 0) {
	Error("Copy of %s to %s failed: %s", 
	      FileToInstall, LicFile, SYSERR);
	return -1;
    }
    if (chmod(LicFile, 0644) != 0) {
	Error("WARNING: %s: chmod failed: %s", LicFile, SYSERR);
	/* NOT FATAL */
    }

    Msg(0, "OK\n");

    return 0;
}

/*
 * Install each Product license
 */
extern int InstallLicenses(McdData_t *McdData)
{
    McdProduct_t *		Prod;
    int				Status = 0;

    if (!McdData)
	return -1;

    for (Prod = McdData->Products; Prod; Prod = Prod->Next) {
	if (InstallLicense(Prod) < 0)
	    --Status;
    }

    return Status;
}
