/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.21 $";
#endif

/*
 * MCD functions
 */

#include "defs.h"

/*
 * Create a new mcd_t
 */
extern McdMap_t *McdNew(void)
{
    McdMap_t		       *New;

    New = (McdMap_t *) calloc(1, sizeof(McdMap_t));
    if (!New) {
	Error("calloc %d bytes failed: %s", sizeof(McdMap_t), SYSERR);
	exit(1);
    }

    return New;
}

/*
 * Read mcdmap data from FileName and add to Package
 * Returns 0 on success and sets Package->MapList.
 * Returns -1 on error.
 */
static int McdMapReadFile(McdProduct_t *	Product,
			  McdData_t *		McdData,
			  McdPackage_t *	Package,
			  char *		FileName)
{
    FILE		       *FH;
    static char			Line[8192];
    int				Argc;
    static char		      **Argv;
    char		       *cp;
    int				LineNo = 0;
    int				ErrCount = 0;
    int				McdVer;
    int				ReqFields = -1;
    McdMap_t		       *Map = NULL;
    McdMap_t		       *New = NULL;
    McdMap_t		       *Last = NULL;
    McdPackage_t *		Pkg;
    char		       *File;

    File = McdPath(Product, NULL, McdData->DistDir, FileName);
    if (!File) {
	if (FileName)
	    Error("%s: This %s file doesn't exist.", FileName, MCDMAP);
	else
	    Error("Cannot locate %s in %s.", FileName, McdData->DistDir);
	return -1;
    }

    SetupDebug("Reading %s contents from <%s> ...", MCDMAP, File);

    FH = fopen(File, "r");
    if (!FH) {
	Error("%s: Open for read failed: %s", File, SYSERR);
	return -1;
    }

    /*
     * Validate header
     */
    if (!fgets(Line, sizeof(Line), FH)) {
	Error("%s: Cannot read first line: %s", File, SYSERR);
	return -1;
    }
    if (cp = strchr(Line, '\n'))
	*cp = CNULL;
    if (strncasecmp(Line, "#@MCDMAP", 8) != 0) {
	Error("%s: This doesn't appear to be a valid %s file.",
	      File, MCDMAP);
	return -1;
    }
    McdVer = atoi(&Line[8]);
    SetupDebug("MCD Version %d", McdVer);

    switch (McdVer) {
    case 1: ReqFields = 6;  break;
    case 2: ReqFields = 7;  break;
    case 3: ReqFields = 8;  break;
    default:
	Error("Unknown MCD Version found: %d", McdVer);
	return -1;
    }

    /*
     * Read contents
     */
    while (fgets(Line, sizeof(Line), FH)) {
	++LineNo;
	if (Line[0] == '#')
	    continue;

	if (cp = strchr(Line, '\n'))
	    *cp = CNULL;

	Argc = StrToArgv(Line, " ", &Argv, NULL, 0);
	if (Argc != ReqFields) {
	    Error("%s: Line %d: Wrong number of fields detected - found %d require %d",
		  File, LineNo, Argc, ReqFields);
	    ++ErrCount;
	    continue;
	}

	New = (McdMap_t *) calloc(1, sizeof(McdMap_t));
	if (!New) {
	    Error("calloc %d bytes failed: %s", sizeof(McdMap_t), SYSERR);
	    return -1;
	}

	/* Set Type */
	switch (Argv[0][0]) {
	case 'f':
	    New->Type = MCDT_FILE;
	    break;
	case 'F':
	    New->Type = MCDT_FILE;
	    New->Flags |= MCDF_EDIT;
	    break;
	case 'd':
	    New->Type = MCDT_DIR;
	    break;
	case 'l':
	    New->Type = MCDT_SYMLINK;
	    break;
	default:
	    Error("%s: Line %d: Unknown type <%c>", File, LineNo, Argv[0][0]);
	    ++ErrCount;
	}

	if (!EQ(Argv[1], "-")) New->Src = Argv[1];
	if (!EQ(Argv[2], "-")) New->Dest = Argv[2];
	if (!EQ(Argv[3], "-")) New->Owner = Argv[3];
	if (!EQ(Argv[4], "-")) New->Group = Argv[4];
	if (!EQ(Argv[5], "-")) New->Mode = strtol(Argv[5], NULL, 8);
	if (ReqFields >= 7) {
	    if (!EQ(Argv[6], "-")) New->Size = strtol(Argv[6], NULL, 0);
	}
	if (ReqFields >= 8) {
	    if (!EQ(Argv[7], "-")) {
		New->Package = Argv[7];
		/* Add the size of this entry to the package total */
		if (New->Size)
		    Package->Size += New->Size;
	    }
	}

	/* Add it to the list */
	if (!Map) {
	    Last = Map = New;
	} else {
	    Last->Next = New;
	    Last = New;
	}
    }

    fclose(FH);

    if (ErrCount)
	return -1;

    if (Map) {
	Package->MapList = Map;
	return 0;
    }

    return -1;
}

/*
 * Read in all mcdmap data.
 * Returns 0 on success and sets McdData->MapData.
 * Returns -1 on error.
 */
extern int McdMapRead(McdData_t *	McdData)
{
    McdProduct_t *		Prod;
    McdPackage_t *		Pkg;
    static char			File[MAXPATHLEN];

    for (Prod = McdData->Products; Prod; Prod = Prod->Next) {
	for (Pkg = Prod->Packages; Pkg; Pkg = Pkg->Next) {
	    snprintf(File, sizeof(File), "%s/%s/%s",
		     FN_PACKAGES_DIR, Pkg->Name, FN_MCDMAP_FILE);
	    McdMapReadFile(Prod, McdData, Pkg, File);
	}
    }

    return 0;
}

/*
 * Dump (for debug) all of MapList
 */
extern void McdMapDump(McdMap_t *MapList)
{
    McdMap_t *			Entry;
    char 			t;

    SetupDebug("MAP LIST:");

    for (Entry = MapList; Entry; Entry = Entry->Next) {
	switch (Entry->Type) {
	case MCDT_FILE:		t = 'f'; break;
	case MCDT_DIR:		t = 'd'; break;
	case MCDT_SYMLINK:	t = 'l'; break;
	default:		t = '?'; break;
	}

	SetupDebug(" %c %s -> %s %s:%s %o",
		   t, PRTS(Entry->Src), PRTS(Entry->Dest),
		   PRTS(Entry->Owner), PRTS(Entry->Group),
		   Entry->Mode);
    }
}
