/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.26 $";
#endif

/*
 * MCD functions
 */

#include "defs.h"

/*
 * Return path to the PkgName/mcdpkg file
 */
extern char *McdPkgPath(McdProduct_t *	Product,
			char *		PkgName,
			char *		TopDir)
{
    static char			File[MAXPATHLEN];

    if (!Product || !PkgName)
	return (char *) NULL;

    snprintf(File, sizeof(File), "%s/%s/%s",
	     FN_PACKAGES_DIR, PkgName, FN_MCDPKG_FILE);

    return McdPath(Product, NULL, TopDir, File);
}

extern int McdPkgSelectSetProduct(McdProduct_t *	Product,
				  McdData_t *		McdData,
				  int			Enable,
				  char **		List,
				  int			All,
				  int			SelectReq)
{
    McdPackage_t *		Pkg;

    if (!Product)
	return -1;

    for (Pkg = Product->Packages; Pkg && Pkg->Name; Pkg = Pkg->Next) {
	/*
	 * If PkgList is set, it overrides all
	 */
	if (McdData->PkgList) {
	    /* Is this Pkg in PkgList? */
	    if (StrArray(Pkg->Name, McdData->PkgList) > 0)
		Pkg->Selected = TRUE;
	    else
		Pkg->Selected = FALSE;
	    continue;
	}
	
	/* Required packages must always be Selected */
	if (SelectReq && Pkg->Required) {
	    Pkg->Selected = TRUE;
	    continue;
	}
	if (All || StrArray(itoa(Pkg->Number), List) > 0) {
	    if (Enable)
		Pkg->Selected = TRUE;
	    else 
		Pkg->Selected = FALSE;
	}
    }

    return 0;
}

/*
 * Take SelectStr and apply it to Packages
 */
extern int McdPkgSelectSet(McdProduct_t *	Product,
			   McdData_t *		McdData,
			   int			Enable,
			   char *		SelectStr,
			   int			SelectReq)
{
    McdProduct_t *		Prod;
    int				All = FALSE;
    int				ListCnt;
    char **			List = NULL;
    int				Status = 0;

    SetupDebug("McdPkgSelectSet: Str=<%s>", PRTS(SelectStr));

    /*
     * If PkgList is set, ignore other params
     */
    if (!McdData->PkgList) {
	if (EQ(SelectStr, A_ALL))
	    All = TRUE;
	else if (SelectStr) {
	    ListCnt = StrToArgv(SelectStr, ",", &List, NULL, 0);
	    if (ListCnt <= 0) {
		Error("The selection string \"%s\" cannot be parsed.",
		      SelectStr);
		return -1;
	    }
	}
    }

    if (Product) {
	/* Just do the indicated product */
	Status = McdPkgSelectSetProduct(Product, McdData, Enable, List,
					All, SelectReq);
    } else {
	/* Do all products */
	for (Prod = McdData->Products; Prod; Prod = Prod->Next) {
	    if (McdPkgSelectSetProduct(Prod, McdData, Enable, List,
				       All, SelectReq) < 0)
		--Status;
	}
    }

    return Status;
}

/*
 * Find a package named Name and return its entry
 */
extern McdPackage_t *McdPkgFind(McdPackage_t *	List,
				char *		Name)
{
    McdPackage_t *		Pkg;

    if (!List || !Name)
	return (McdPackage_t *) NULL;

    /*
     * XXX We use a linear list search to keep it simple since we
     * do not expect to have more than half a dozen entries ever.
     * If we do, we should change this code to setup and search a
     * hash table.
     */
    for (Pkg = List; Pkg && Pkg->Name; Pkg = Pkg->Next)
	if (EQ(Pkg->Name, Name))
	    return Pkg;

    return (McdPackage_t *) NULL;
}

/*
 * Create a new McdPackage_t
 */
extern McdPackage_t *McdPkgNew(void)
{
    McdPackage_t	       *New;

    New = (McdPackage_t *) calloc(1, sizeof(McdPackage_t));
    if (!New) {
	Error("calloc %d bytes failed: %s", sizeof(McdPackage_t), SYSERR);
	exit(1);
    }

    return New;
}

/*
 * Read McdPackage data from File and return McdPackage_t
 */
static McdPackage_t *McdPkgReadFile(char *File)
{
    FILE *			FH;
    static char			Line[8192];
    int				LineNo = 0;
    char *			cp;
    McdPackage_t *		Pkg;
    int				Argc;
    char **			Argv;

    SetupDebug("McdPkgReadFile: Reading <%s>", File);

    FH = fopen(File, "r");
    if (!FH) {
	Error("%s: Open package to read failed: %s", File, SYSERR);
	return (McdPackage_t *) NULL;
    }

    Pkg = McdPkgNew();

    /*
     * File looks like:
     *		Key=Value
     * where Value is often quoted.
     */
    while (fgets(Line, sizeof(Line), FH)) {
	++LineNo;
	if (Line[0] == '#')
	    continue;		/* Skip comments */
	if (cp = strchr(Line, '\n'))
	    *cp = CNULL;	/* Zap newline */

	Argc = StrToArgv(Line, "=", &Argv, NULL, 0);
	if (Argc != 2) {
	    Error("%s: line %d: Expecting 2 fields of form \"key=value\"; got %d fields.",
		  File, LineNo, Argc);
	    continue;
	}

	if (EQ(Argv[0], "PKGNAME"))
	    Pkg->Name = DeQuote(Argv[1]);
	else if (EQ(Argv[0], "SUMMARY"))
	    Pkg->Summary = DeQuote(Argv[1]);
	else if (EQ(Argv[0], "POSTINSTALL"))
	    Pkg->PostInstall = DeQuote(Argv[1]);
	else if (EQ(Argv[0], "POSTREMOVE"))
	    Pkg->PostRemove = DeQuote(Argv[1]);
	else if (EQ(Argv[0], "REQUIRED")) {
	    if (EQ(Argv[1], "1") || EQ(Argv[1], "TRUE") || EQ(Argv[1], "YES"))
		Pkg->Required = TRUE;
	} else if (EQ(Argv[0], "INSTALLED")) {
	    if (EQ(Argv[1], "1") || EQ(Argv[1], "TRUE") || EQ(Argv[1], "YES"))
		Pkg->Installed = TRUE;
	} else {
	    SetupDebug("%s: line %d: Unknown keyword <%s>",
		       File, LineNo, Argv[0]);
	}
    }

    SetupDebug("\tMcdPkg: Name=<%s> Required=%d Install=<%s> Remove=<%s>",
	       PRTS(Pkg->Name), Pkg->Required,
	       PRTS(Pkg->PostInstall), PRTS(Pkg->PostRemove));
    SetupDebug("\tMcdPkg: Summary=<%s>",
	       PRTS(Pkg->Summary));

    if (Pkg->Name)
	return Pkg;
    else
	return (McdPackage_t *) NULL;
}

/*
 * Read in all package (*.mcdpkg)
 * Returns 0 on success and sets McdData->Packages.
 * Returns -1 on error.
 */
extern int McdPkgRead(McdData_t *	McdData)
{
    static char			File[MAXPATHLEN];
    char **			pname;
    int				Status = 0;
    McdProduct_t *		Prod;
    McdPackage_t *		Pkg;
    McdPackage_t *		Last = NULL;
    int				PkgNumber = 0;

    if (!McdData)
	return -1;

    SetupDebug("Reading MCD package files ...");

    for (Prod = McdData->Products; Prod; Prod = Prod->Next) {
	for (pname = Prod->PkgList, Last = NULL; pname && *pname; ++pname) {
	    /*
	     * The distribution directory is layed out without the
	     * product name under the "mcd" dir so we create the File
	     * path by hand here rather than call McdPkgPath().
	     */
	    snprintf(File, sizeof(File), "%s/%s/%s/%s/%s/%s",
		     McdData->DistDir,
		     FN_MCD,
		     Prod->Name,
		     FN_PACKAGES_DIR,
		     *pname,
		     FN_MCDPKG_FILE);
	    
	    if (!FileExists(File, 0)) {
		Error("The mcd package file \"%s\" is missing.", File);
		Status = -1;
		continue;
	    }
	    
	    /* Read the file */
	    if (Pkg = McdPkgReadFile(File)) {
		Pkg->Number = ++PkgNumber;
		/* Add the package data */
		if (Last) {
		    Last->Next = Pkg;
		    Last = Pkg;
		} else {
		    Last = Prod->Packages = Pkg;
		}
	    }
	}
    }

    if (Status < 0)
	return -1;

    return Status;
}

/*
 * Add entry to Package
 */
extern int McdPkgAdd(McdPackage_t *	Package,
		     int		Type,
		     char *		Src,
		     char *		Dst,
		     mode_t		Mode)
{
    McdMap_t *		        New;
    McdMap_t *			Map;

    SetupDebug("McdPkgAdd: src=<%s> dst=<%s> type=%d mode=%o",
	       PRTS(Src), PRTS(Dst), Type, Mode);

    New = McdNew();
    New->Src = Src;
    New->Dest = Dst;
    New->Mode = Mode;
    New->Type = Type;

    if (!Package->MapListLast && Package->MapList) {
	/* Find the last entry and set MapListLast */
	for (Map = Package->MapList; Map && Map->Next; Map = Map->Next);
	Package->MapListLast = Map;
    }

    if (Package->MapListLast) {
	Package->MapListLast->Next = New;
	Package->MapListLast = New;
    } else
	Package->MapList = Package->MapListLast = New;

    return 0;
}

/*
 * Add Key=Value to File
 */
static int McdPkgFileAdd(char *File, char *Key, char *Value)
{
    FILE *			RFH;
    FILE *			TFH;
    static char			Line[8192];
    static char			TmpFile[MAXPATHLEN];
    static char			LineKey[256];
    char *			cp;
    int				Added = FALSE;

    SetupDebug("Updating <%s> with <%s>=<%s>",
	       PRTS(File), PRTS(Key), PRTS(Value));

    RFH = fopen(File, "r");
    if (!RFH) {
	Error("%s: Cannot open for read: %s", File, SYSERR);
	return -1;
    }

    snprintf(TmpFile, sizeof(TmpFile), "%s.tmp%d", File, getpid());
    TFH = fopen(TmpFile, "w");
    if (!TFH) {
	Error("%s: Cannot open for write: %s", TmpFile, SYSERR);
	fclose(RFH);
	return -1;
    }

    while (fgets(Line, sizeof(Line), RFH)) {
	if (cp = strchr(Line, '\n'))
	    *cp = CNULL;

	if (Line[0] == '#') {
	    fprintf(TFH, "%s\n", Line);
	    continue;
	}

	/*
	 * If this is a key=value line, then see if this line contains the
	 * same key as we are adding and if so, replace it with our new Value
	 */
	if (cp = strchr(Line, '=')) {
	    snprintf(LineKey, sizeof(LineKey), "%.*s", cp-Line, Line);
	    if (EQ(LineKey, Key)) {
		fprintf(TFH, "%s=%s\n", Key, Value);
		Added = TRUE;
		continue;
	    }
	}

	fprintf(TFH, "%s\n", Line);
    }

    /* If we didn't find an existing entry to update, add it now */
    if (!Added)
	fprintf(TFH, "%s=%s\n", Key, Value);

    fclose(RFH);
    fclose(TFH);

    /* Set perms */
    (void) chmod(TmpFile, 0444);

    /* Install new file */
    if (rename(TmpFile, File) != 0) {
	Error("Rename %s -> %s failed: %s", TmpFile, File, SYSERR);
	return -1;
    }

    return 0;
}

/*
 * Update the installed *.mcdpkg files with their status.
 */
extern int McdPkgUpdate(McdProduct_t *	Product,
			McdData_t *	McdData)
{
    char *			Path;
    McdPackage_t *		Pkg;
    int				Err = 0;

    for (Pkg = Product->Packages; Pkg; Pkg = Pkg->Next) {
	if (!Pkg->Selected)
	    continue;

	Path = McdPkgPath(Product, Pkg->Name, McdData->InstallDir);

	/* Make any required directories */
	MakeDir(DirName(Path));

	if (McdPkgFileAdd(Path, "INSTALLED", "1") < 0)
	    ++Err;
    }

    if (Err)
	return -1;
    else
	return 0;
}

/*
 * Fetch Key from package PkgName installed in Installdir
 */
extern char *McdPkgGetKey(McdProduct_t *	Product,
			  char *		PkgName,
			  char *		InstallDir,
			  char *		Key)
{
    FILE *			FH;
    char *			File;
    static char			Line[1024];
    char *			cp;
    char *			LineKey = NULL;
    char *			Value = NULL;

    File = McdPkgPath(Product, PkgName, InstallDir);
    FH = fopen(File, "r");
    if (!FH) {
	Error("%s: Open mcdpkg file for reading failed: %s", File, SYSERR);
	return (char *) NULL;
    }

    /*
     * Search standard Key=Value lines
     */
    while (fgets(Line, sizeof(Line), FH)) {
	if (Line[0] == '#')
	    continue;
	if (cp = strchr(Line, '\n'))
	    *cp = CNULL;

	if (cp = strchr(Line, '=')) {
	    *cp = CNULL;
	    LineKey = Line;
	    if (EQ(Key, LineKey)) {
		Value = cp + 1;
		break;
	    }
	}
    }

    fclose(FH);

    return Value;
}

/*
 * Set selected packages according to what's installed in InstallDir
 */
extern int McdPkgSelectInstalled(McdData_t *	McdData)
{
    McdProduct_t *		Prod;
    McdPackage_t *		Pkg;
    char *			cp;

    if (!McdData) {
	Error("McdPkgSelectInstalled: No McdData param.");
	return -1;
    }

    SetupDebug("Selecting packages installed in <%s> ...",
	       McdData->InstallDir);

    for (Prod = McdData->Products; Prod; Prod = Prod->Next) {
	for (Pkg = Prod->Packages; Pkg; Pkg = Pkg->Next) {
	    cp = McdPkgGetKey(Prod, Pkg->Name, Prod->InstallDir,
			      "INSTALLED");
	    if (EQ(cp, "1"))
		Pkg->Selected = TRUE;
	    else
		Pkg->Selected = FALSE;
	    SetupDebug("SelectInstalled: Pkg <%s> is %s",
		       Pkg->Name,
		       (Pkg->Selected) ? "INSTALLED" : "NOT-INSTALLED");
	}
    }

    return 0;
}

/*
 * Return max width for padding of all Packages's names.
 */
extern int McdPkgPadding(McdPackage_t *Packages)
{
    McdPackage_t *		Pkg;
    int				Padding = 0;
    int				l;

    for (Pkg = Packages; Pkg; Pkg = Pkg->Next) {
	if (!Pkg->Selected && !Pkg->Installed)
	    continue;
	l = strlen(Pkg->Name);
	if (l > Padding)
	    Padding = l;
    }

    return Padding;
}
