/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.5 $";
#endif

/*
 * MCD Post functions
 */

#include "defs.h"
#include <sys/wait.h>
#if HAVE_STDLIB_H
# include <stdlib.h>
#endif

/*
 * Perform post install|uninstall operations.
 * All Post programs are called with these CLI options:
 *	--installdir <dir>		Install directory
 *	--install|--uninstall		Operation
 */
extern int McdPost(McdProduct_t *	Product,
		   int			UnInstall)
{
    McdPackage_t *		Pkg;
    static char			Cmd[MAXPATHLEN];
    char *			PostCmd;
    char *			InstallDir;
    int				Failed = 0;
    int				Total = 0;
    int				Status;

    InstallDir = Product->InstallDir;
    if (!InstallDir) {
	Error("McdPost: Cannot determine installation directory.");
	return -1;
    }

    /*
     * Check for and run mcdpost for each package
     */
    for (Pkg = Product->Packages; Pkg; Pkg = Pkg->Next) {
	PostCmd = McdPath(Product, Pkg, InstallDir, FN_MCDPOST);

	if (!FileExists(PostCmd, FALSE)) {
	    SetupDebug("McdPost: No post program <%s>", PostCmd);
	    continue;
	}
	
	Msg(0, "Performing post %s tasks for package %s . . . \n",
	    (UnInstall) ? "uninstall" : "install", Pkg->Name);
	
	/* Insure PostCmd is executable */
	if (chmod(PostCmd, 0755) != 0)
	    SetupDebug("McdPost: chmod 755 %s failed: %s", PostCmd, SYSERR);
	
	snprintf(Cmd, sizeof(Cmd), "%s %s --installdir %s",
		 PostCmd,
		 (UnInstall) ? "--uninstall" : "--install",
		 InstallDir);
	
	SetupDebug("Running Post Command: %s", Cmd);
	Status = system(Cmd);
	SetupDebug("Post Command returned %d (%d)", Status, WEXITSTATUS(Status));
	if (Status != 0)
	    ++Failed;
	++Total;
    }

    SetupDebug("McdPost: Total=%d Failed=%d", Total, Failed);

    if (Failed != 0) {
	Msg(0, "%d of %d post tasks failed.", Failed, Total);
	return -1;
    } else if (Total) {
	Msg(0, "Post tasks completed OK\n");
	return 0;
    } else {
	/* Nothing happened so be silent */
	return 0;
    }
}
