/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.5 $";
#endif

/*
 * McdProduct functions
 */

#include "defs.h"

/*
 * Read mcdinfo file
 */
extern McdProduct_t *McdProductRead(char *File)
{
    FILE		       *FH;
    char			Line[8192];
    char		       *cp;
    int				Argc;
    char		      **Argv;
    int				LineNo = 0;
    int				ErrCount = 0;
    McdProduct_t *		Product;

    if (!File)
	return (McdProduct_t *) NULL;

    SetupDebug("Reading product contents from <%s> ...", File);

    FH = fopen(File, "r");
    if (!FH) {
	Error("%s: Open for read failed: %s", File, SYSERR);
	return (McdProduct_t *) NULL;
    }

    Product = (McdProduct_t *) xcalloc(1, sizeof(McdProduct_t));
    Product->ProductFile = strdup(File);

    /*
     * Read contents
     */
    while (fgets(Line, sizeof(Line), FH)) {
	++LineNo;
	if (Line[0] == '#')
	    continue;
	if (cp = strchr(Line, '\n'))
	    *cp = CNULL;

	Argc = StrToArgv(Line, "=", &Argv, NULL, 0);
#define REQ_FIELDS 2
	if (Argc != REQ_FIELDS) {
	    Error("%s: Line %d: Wrong number of fields detected - found %d require %d",
		  File, LineNo, Argc, REQ_FIELDS);
	    ++ErrCount;
	    continue;
#undef REQ_FIELDS
	}
	if (EQ(Argv[0], "NAME"))	Product->Name = DeQuote(Argv[1]);
	if (EQ(Argv[0], "VERSION"))	Product->Version = DeQuote(Argv[1]);
	if (EQ(Argv[0], "RELEASE"))	Product->Release = DeQuote(Argv[1]);
	if (EQ(Argv[0], "VENDORNAME"))	Product->VendorName = DeQuote(Argv[1]);
	if (EQ(Argv[0], "URL"))	    	Product->URL = DeQuote(Argv[1]);
	if (EQ(Argv[0], "SUMMARY"))	Product->Summary = DeQuote(Argv[1]);
	if (EQ(Argv[0], "OSNAME"))	Product->OSname = DeQuote(Argv[1]);
	if (EQ(Argv[0], "MINOSVERSION")) Product->OSverMin = DeQuote(Argv[1]);
	if (EQ(Argv[0], "OSVERSION"))	Product->OSver = DeQuote(Argv[1]);
	if (EQ(Argv[0], "ARCH"))	Product->Arch = DeQuote(Argv[1]);
	if (EQ(Argv[0], "LIBCNAME"))	Product->LibCname = DeQuote(Argv[1]);
	if (EQ(Argv[0], "INSTALLDIR"))	Product->InstallDir = DeQuote(Argv[1]);
	if (EQ(Argv[0], "INSTALLLIC"))	Product->InstallLic = DeQuote(Argv[1]);
	if (EQ(Argv[0], "MINLIBCVERSION")) 
	    Product->LibCverMin = DeQuote(Argv[1]);
	if (EQ(Argv[0], "ACCEPTLICENSEFILE")) 
	    Product->LicFile = DeQuote(Argv[1]);
	if (EQ(Argv[0], "ASKFORLICENSEFILE") && atoi(DeQuote(Argv[1])))
	    Product->AskForLic = TRUE;
	if (EQ(Argv[0], "DISTBTIME"))	    
	    Product->DistBuilt = strtol(DeQuote(Argv[1]), NULL, 0);
	if (EQ(Argv[0], "SIZE"))	
	    Product->Size = (Large_t) strtol(DeQuote(Argv[1]), NULL, 0);
	if (EQ(Argv[0], "PKGLIST")) {
	    int 	ListCnt;
	    char	**List;

	    ListCnt = StrToArgv(DeQuote(Argv[1]), " ", &List, NULL, 0);
	    if (ListCnt > 0)
		Product->PkgList = List;
	}
    }

    fclose(FH);

    if (ErrCount || !Product->Name)
	return (McdProduct_t *) NULL;

    return Product;
}

/*
 * Set all products we find to McdData
 */
extern int McdProductSet(McdData_t *McdData)
{
    char **			Names;
    char **			cpp;
    static char			McdDir[MAXPATHLEN];
    static char			File[MAXPATHLEN];
    McdProduct_t *		Product;
    McdProduct_t *		Last = NULL;
    int				Found = 0;
    int				ProdNumber = 0;

    if (!McdData)
	return -1;

    snprintf(McdDir, sizeof(McdDir), "%s/%s", McdData->DistDir, FN_MCD);

    /* Get the list of Names found in McdDir */
    Names = DirGetNames(McdDir, FALSE, NULL);

    /* Try each one */
    for (cpp = Names; cpp && *cpp; ++cpp) {
	snprintf(File, sizeof(File), "%s/%s/%s", McdDir, *cpp, FN_MCDPRODUCT);

	SetupDebug("McdProductSet: Checking for <%s>", File);

	if (!FileExists(File, FALSE))
	    /* Must not be a product dir */
	    continue;

	if (Product = McdProductRead(File)) {
	    SetupDebug("McdProductSet:\t Found product <%s>", Product->Name);

	    /* Count how many we find */
	    ++Found;

	    /* Set the product's number */
	    Product->Number = ++ProdNumber;

	    /* Add Product to McdData->Products */
	    if (Last) {
		Last->Next = Product;
		Last = Product;
	    } else {
		McdData->Products = Last = Product;
	    }
	}
    }

    DirFreeNames(Names);

    SetupDebug("McdProductSet: Found %d Products", Found);

    if (!Found) {
	/* Failure */
	Error("Could not find any MCD products in %s", McdDir);
	return -1;
    } else {
	/* Success */
	return 0;
    }
}

