/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.46 $";
#endif

/*
 * Setup and installation program
 */

#if defined(HAVE_AUTOCONFIG_H)
# include "mcsysinfo-autoconfig.h"
#endif	/* HAVE_AUTOCONFIG_H */
#if defined(HAVE_STDLIB_H)
# include <stdlib.h>
#endif
#include <stdio.h>
#include "options.h"
#include "version.h"
#include "defs.h"

int				DebugFlag = 0;
int				AcceptLicense = 0;
int				ConfirmFlag = 0;
int				OverRide = 0;
int				DoPrintVersion = 0;
int				IgnoreErrors = 0;
int				OpInstall = 0;
int				OpLicense = 0;
int				OpUnInstall = 0;
int				OpVerify = 0;
int				OpInventory = 0;
int				OpInfo = 0;
int				TermWidth = 80;
char			       *InstallDir = NULL;
char			       *LicenseFile = NULL;
char			       *Answer = NULL;
char			       *PkgString = NULL;
char			       *ProgramPath = NULL;
char			       *DistDir = NULL;
static Meter_t			MeterInfo;

/*
 * Command line options table.
 */
OptionDescRec Opts[] = {
    {"acceptlicense", NoArg,	OptBool,(OptPtr_t) &AcceptLicense,	"1",
     	NULL,  "Accept the terms of the license without prompting" },
    {"debug", NoArg,	OptBool,	(OptPtr_t) &DebugFlag,		"1",
     	NULL,  "Enable debugging" },
    {"distdir", SepArg,OptStr,	(OptPtr_t) &DistDir,		NULL,
     	NULL,  "Distribution directory" },
    {"confirm", NoArg,OptBool,	(OptPtr_t) &ConfirmFlag,	"1",
     	NULL,  "Confirm all default answers (no prompts)" },
    {"ignoreerrors", NoArg,OptBool,	(OptPtr_t) &IgnoreErrors,	"1",
     	NULL,  "Ignore normally fatal errors" },
    {"install", NoArg,OptBool,	(OptPtr_t) &OpInstall,		"1",
     	NULL,  "Install software" },
    {"installdir", SepArg,OptStr,	(OptPtr_t) &InstallDir,		NULL,
     	NULL,  "Installation directory" },
    {"inventory", NoArg,OptBool,	(OptPtr_t) &OpInventory,	"1",
     	NULL,  "Inventory distribution files" },
    {"info", 	NoArg,OptBool,	(OptPtr_t) &OpInfo,		"1",
     	NULL,  "Show info about product and packages in distribution" },
    {"license", NoArg,OptBool,	(OptPtr_t) &OpLicense,		"1",
     	NULL,  "Install license" },
    {"licensefile", SepArg,OptStr,	(OptPtr_t) &LicenseFile,        NULL,
     	NULL,  "Name of license file to be installed" },
    {"override", NoArg,OptBool,	(OptPtr_t) &OverRide,		"1",
     	NULL,  "Override errors which prompt for confirmation" },
    {"packages", SepArg,OptStr,	(OptPtr_t) &PkgString,        NULL,
     	NULL,  "List of packages (comma seperated)" },
    {"uninstall", NoArg,OptBool,	(OptPtr_t) &OpUnInstall,	"1",
     	NULL,  "UnInstall software" },
    {"verify", NoArg,OptBool,		(OptPtr_t) &OpVerify,		"1",
     	NULL,  "Verify installed software" },
    {"version",NoArg,	OptBool,	(OptPtr_t) &DoPrintVersion,	"1",
	 NULL,	"Print version of this program" },
};

/*
 * Get program version
 */
static char *GetVersion(void)
{
    char		       *cp;
    char		       *Version = NULL;
    static char			Buff[128];
    static char			String[128];

    if (String[0])
	return String;

    strncpy(Buff, RCSid, sizeof(Buff));
    if (cp = strchr(Buff, ' ')) {
	Version = cp+1;
	if (cp = strchr(Version, ' '))
	    *cp = CNULL;
    }

    if (Version) {
	(void) snprintf(String, sizeof(String), "%s version %s",
			ProgramName, Version);
	return String;
    }

    return (char *) NULL;
}

/*
 * Bail out for some kind of fatal condition
 */
extern void Bail(void)
{
    if (!IgnoreErrors) {
	Msg(0, "\nThe installation has been aborted due to an error.\n");
	exit(1);
    }
}

/*
 * Bail out at user request
 */
extern void UserBail(void)
{
    Msg(0, "\nThe installation has been aborted at your request.\n");
    exit(1);
}

/*
 * Get pathname relative to distribution directory
 */
extern char *GetDistPath(char *		File,
			 char *		DistDir,
			 McdProduct_t *	Product,
			 McdPackage_t *	Pkg)
{
    static char			Path[MAXPATHLEN];

    if (FileExists(File, 0))
	snprintf(Path, sizeof(Path), "%s", File);
    else if (DistDir && Product && Pkg)
	snprintf(Path, sizeof(Path), "%s/%s/%s/%s/%s/%s/%s",
		 DistDir, FN_MCD, PRTS(Product->Name), 
		 FN_PACKAGES_DIR, PRTS(Pkg->Name), FN_FILES_DIR, PRTS(File));
    else if (DistDir)
	snprintf(Path, sizeof(Path), "%s/files/%s", DistDir, File);
    else
	snprintf(Path, sizeof(Path), "files/%s", File);

    return Path;
}

/*
 * Get an MCD data file
 */
extern char *GetMcdFile(char *		File,
			char *		Base,
			char *		DistDir)
{
    static char		        Path[MAXPATHLEN];

    if (File) {
	if (FileExists(File, 0))
	    return File;
	else {
	    Error("%s: File does not exist.", File);
	    return (char *) NULL;
	}
    } else if (FileExists(Base, 0)) {
	return Base;
    } else if (DistDir) {
	snprintf(Path, sizeof(Path), "%s/%s", DistDir, Base);
	if (FileExists(Path, 0))
	    return Path;

	snprintf(Path, sizeof(Path), "%s/mcd/%s", DistDir, Base);
	if (FileExists(Path, 0))
	    return Path;

	snprintf(Path, sizeof(Path), "%s/files/%s", DistDir, Base);
	if (FileExists(Path, 0))
	    return Path;
    } else {
	snprintf(Path, sizeof(Path), "files/%s", Base);
	if (FileExists(Path, 0))
	    return Path;
    }

    return (char *) NULL;
}

/*
 * Figure out Installation Directory based on ProgramPath
 */
extern int SetInstallDir(McdData_t *	McdData)
{
    static char			InstallDir[MAXPATHLEN];
    char *			DistDir;
    char   		       *cp;

    DistDir = McdData->DistDir;

    if (DistDir && IsDir(DistDir))
	(void) snprintf(InstallDir, sizeof(InstallDir), "%s", DistDir);
    else {
	(void) snprintf(InstallDir, sizeof(InstallDir), "%s",
			McdData->ProgramPath);
	if (cp = strrchr(InstallDir, '/')) {
	    *cp = CNULL;
	    if (EQ(InstallDir, ".") || EQN(InstallDir, "./", 2)) {
		if (!getcwd(InstallDir, sizeof(InstallDir))) {
		    Error("Get current working directory failed: %s", SYSERR);
		    return -1;
		}
	    }
	} else {
	    if (!getcwd(InstallDir, sizeof(InstallDir))) {
		Error("Get current working directory failed: %s", SYSERR);
		return -1;
	    }
	}
    }

    SetupDebug("SetInstallDir: InstallDir=<%s>", InstallDir);

    McdData->InstallDir = InstallDir;

    return 0;
}

/*
 * Generate name of LogFile to use
 */
static char *MakeLogFile(char *SwName)
{
    char		       *Tmp = "/var/tmp";
    char		       *cp;
    static char			File[MAXPATHLEN];

    if (File[0] || !SwName)
	return File;

#ifdef notdef /* disable for security reasons */
    if (cp = (char *) getenv("TMP"))
	Tmp = cp;
#endif

    (void) snprintf(File, sizeof(File), "%s/%s_%s_log_%d",
		    Tmp, ProgramName, SwName, getpid());

    return File;
}

/*
 * Get Distribution directory relative to McdFile
 */
extern char *GetDistDir(char *McdFile)
{
    char		       *cp;
    char		       *Dir = NULL;

    if (!McdFile)
	return (char *) NULL;

    if (strrchr(McdFile, '/')) {
	Dir = strdup(McdFile);
	if (cp = strrchr(Dir, '/'))
	    *cp = CNULL;
    }

    return Dir;
}

/*
 * Display banner to user
 */
static void Banner(McdProduct_t *McdProduct)
{
    static char			Buff[512];

    Line(LT_DOUBLE);

    Msg(0, "\n");

    snprintf(Buff, sizeof(Buff), "Setup program for %s %s",
	     McdProduct->Name, McdProduct->Version);
    Msg(0, "%s\n\n", Center(Buff));

    Msg(0, "%s\n\n", Center(COPYRIGHT_MSG));

    Line(LT_DOUBLE);
}

/*
 * Ask the user to accept/decline the License
 */
extern int LicenseAccept(McdProduct_t *		Product,
			 McdData_t *		McdData)
{
    char		       *Answers[8];
    char *			cp;

    if (!Product || !McdData)
	return -1;

    if (!Product->AskForLic)
	return 0;

    if (!Product->LicFile) {
	Error("Distribution error - AskForLic is setup, but no license file was specified.");
	return -2;
    }

    Msg(0, "\nThe following is the license for this software:\n");
    Line(LT_SINGLE);
    /* XXX Assume the first package is where LicFile resides */
    cp = GetDistPath(Product->LicFile, McdData->DistDir,
		     Product, Product->Packages);
    Msg(MSGF_FILE, cp);
    Line(LT_SINGLE);

#define LICPROMPT "Do you accept the terms of this license?"
    if (AcceptLicense) {
	Msg(0, "%s %s\n", LICPROMPT, A_ACCEPT);
	return 0;
    } else {
	Answers[0] = A_ACCEPT;
	Answers[1] = A_DECLINE;
	Answers[2] = NULL;
	Answer = Ask(LICPROMPT, Answers, NULL, NULL, 0);
	if (!EQ(Answer, A_ACCEPT)) {
	    Msg(0, "Installation is being aborted because you declined the terms of the license.\n");
	    UserBail();
	}
    }
#undef LICPROMPT

    return 0;
}

/*
 * Copy distirbution files into install dir
 */
extern int DistCopy(char *		File,
		    McdProduct_t *	Product,
		    mode_t		Mode)
{
    static char			Path[MAXPATHLEN];

    (void) snprintf(Path, sizeof(Path), "%s/%s", 
		    Product->InstallDir, BaseName(File));

    (void) unlink(Path);

    if (Copy(File, Path) != 0) {
	Error("%s -> %s: Copy failed: %s", File, Path, SYSERR);
	return -1;
    }

    (void) SetPerms(Path, NULL, NULL, Mode);

    return 0;
}

/*
 * The beginning
 */
main(int Argc, char **Argv)
{
    static char			MyDir[MAXPATHLEN];
    static char			Path[MAXPATHLEN];
    McdData_t			McdData;
    McdProduct_t *		Prod;
    char		       *cp;
    int				Status = 0;
    char		       *What = NULL;
    char **			PkgList = NULL;
    int				PkgListCnt = 0;
    int				MsgFlags = 0;

    memset(&McdData, 0, sizeof(McdData));
    ProgramPath = strdup(Argv[0]);

    /*
     * Parse command line arguments.
     */
    if (ParseOptions(Opts, Num_Opts(Opts), Argc, Argv) < 0)
	exit(1);

    if (PkgString) {
	PkgListCnt = StrToArgv(PkgString, ",", &PkgList, NULL, 0);
	if (PkgListCnt > 0)
	    McdData.PkgList = PkgList;
    }

    McdData.InstallDir = InstallDir;
    McdData.ProgramName = BaseName(ProgramPath);
    /*
     * Set ProgramPath
     * We need to make sure it's an absolute pathname.
     */
    if (ProgramPath && ProgramPath[0] != '/') {
	if (!getcwd(MyDir, sizeof(MyDir))) {
	    Error("Get my current working directory failed: %s", SYSERR);
	    exit(1);
	}
	snprintf(Path, sizeof(Path), "%s/%s", MyDir, ProgramPath);
	McdData.ProgramPath = strdup(Path);
    } else {
	McdData.ProgramPath = strdup(ProgramPath);
    }
    SetupDebug("ProgramPath=<%s> ProgramName=<%s>",
	       PRTS(McdData.ProgramPath), PRTS(McdData.ProgramName));

    if (DebugFlag)
	SetupDebug("%s", GetVersion());
    else if (DoPrintVersion) {
	Msg(0, "%s\n", GetVersion());
	exit(0);
    }

    /*
     * Calculate DistDir - the location of the distribution being installed
     */
    if (!DistDir)
	DistDir = GetDistDir(NULL);
    if (!DistDir)
	DistDir = GetDistDir(ProgramPath);

    /*
     * We should not run from the install directory, especially if we're
     * doing --uninstall so we set DistDir to getcwd and "cd /tmp" (someplace
     * which still allows us to core dump if that has to happen).
     */
    if (EQ(DistDir, ".")) {
	SetupDebug("DistDir is <.> so we need to chdir /tmp");
	if (!getcwd(MyDir, sizeof(MyDir))) {
	    Error("Get current working directory failed: %s", SYSERR);
	    exit(1);
	}
	DistDir = MyDir;
	if (chdir("/tmp") != 0) {
	    Error("chdir to \"/tmp\" failed: %s", SYSERR);
	    exit(1);
	}
    }

    McdData.DistDir = DistDir;
    SetupDebug("DistDir = <%s>", PRTS(DistDir));

    /*
     * Get list of all products
     */
    if (McdProductSet(&McdData) < 0 || !McdData.Products) {
	Error("Cannot find any MCD products.");
	exit(1);
    }

    if (McdPkgRead(&McdData) < 0) {
	Error("Cannot find MCD package data.");
	exit(1);
    }

    if (cp = MakeLogFile(McdData.Products->Name)) {
	SetLogFile(cp);
	Msg(0, "Logging results to %s\n", cp);
    }

    Banner(McdData.Products);

    if (McdMapRead(&McdData) < 0)
	exit(1);

    if (!OpInstall && !OpLicense && !OpUnInstall && !OpVerify && 
	!OpInventory && !OpInfo)
	OpInstall++;

    if (!OpInstall && DistDir)
	for (Prod = McdData.Products; Prod; Prod = Prod->Next)
	    Prod->InstallDir = DistDir;

    What = "Undefined operation";
    Status = -1;

    if (OpInstall) {
	Status = Install(&McdData);
	What = "Installation";
    } else if (OpUnInstall) {
	Status = UnInstall(&McdData);
	What = "UnInstall";
    } else if (OpInventory) {
	Status = Inventory(&McdData);
	What = "Inventory";
    } else if (OpVerify) {
	Status = Verify(&McdData);
	What = "Verify";
    } else if (OpLicense) {
	Status = InstallLicenses(&McdData);
	What = "License installation";
    } else if (OpInfo) {
	What = "Show Information";
	(void) McdPkgSelectSet(NULL, &McdData, FALSE, NULL, TRUE);
	MsgFlags |= MSGF_NOLOG;		/* Don't log to file */
	ShowSummaryAll(&McdData, MsgFlags);
	Status = 0;
    }

    if (Status < 0)
	Error("%s failed.", What);

    if (FLAGS_OFF(MsgFlags, MSGF_NOLOG))
	Msg(0, "A log of this %s action is available as\n    %s\n", 
	    What, MakeLogFile(NULL));

    exit(Status);
}
