/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.14 $";
#endif

/*
 * Summary functions
 */
#include <stdio.h>
#include "defs.h"

/*
 * Show one line summary info
 */
extern void ShowSummaryLine(char *Label, char *Desc)
{
    Msg(0, "%30s: %s\n", Label, Desc);
}

/*
 * Show summary info for Packages
 */
extern void ShowSummaryPkgs(McdPackage_t *	Packages,
			    McdProduct_t *	Product,
			    int			OnlySelected,
			    int			MsgFlags)
{
    McdPackage_t *		Pkg;
    int				f = MsgFlags;
    static char			Buff[128];

    if (!Packages)
	return;

    snprintf(Buff, sizeof(Buff), "PACKAGES CONTAINED IN %s PRODUCT",
	     Product->Name);
    Msg(f, "\n%s\n\n", Center(Buff));
    Msg(f, "%4s %-8s %-3s %8s %s\n",
	"PKG#", "NAME", "REQ", "SIZE", "SUMMARY");
    Msg(f, "%4s %-8s %-3s %8s %s\n",
	"----", "--------", "---", "--------",
	"---------------------------------------------------");
    for (Pkg = Packages; Pkg && Pkg->Name; Pkg = Pkg->Next) {
	if (OnlySelected && !Pkg->Selected)
	    continue;
	Msg(f, "%c%3d %-8s %-3s %8s %s\n",
	    (Pkg->Selected) ? '*' : ' ',
	    Pkg->Number, PRTS(Pkg->Name),
	    (Pkg->Required) ? "YES" : "NO",
	    GetSizeStr(Pkg->Size, BYTES),
	    PRTS(Pkg->Summary));
    }
    
    Msg(f, "\nNOTES:\n");
    Msg(f, "REQ = Package is required\n");
    Msg(f, "*   = Package is selected\n");
}

/*
 * Show a summary of the software
 */
extern void ShowSummary(McdProduct_t *	Product,
			int		OnlySelected,
			int		MsgFlags)
{
    static char			Buff[1024];
    int				f = MsgFlags;

    if (!Product)
	return;

    snprintf(Buff, sizeof(Buff), "%s %s %s",
	     PRTS(Product->Name), PRTS(Product->Version),
	     PRTS(Product->Release));
    ShowSummaryLine("Software", Buff);
    if (Product->Summary)
	ShowSummaryLine("Summary", Product->Summary);

    if (Product->OSverMin && Product->OSverMax)
	snprintf(Buff, sizeof(Buff), "%s %s - %s %s",
		 PRTS(Product->OSname), PRTS(Product->OSverMin),
		 PRTS(Product->OSverMax), PRTS(Product->Arch));
    else if (Product->OSverMin)
	snprintf(Buff, sizeof(Buff), "%s %s and later %s",
		 PRTS(Product->OSname), PRTS(Product->OSverMin),
		 PRTS(Product->Arch));
    else
	snprintf(Buff, sizeof(Buff), "%s %s %s",
		 PRTS(Product->OSname), PRTS(Product->OSver),
		 PRTS(Product->Arch));
    ShowSummaryLine("Runs On", Buff);

    if (Product->LibCname && Product->LibCverMin) {
	snprintf(Buff, sizeof(Buff), "%s %s or later",
		 Product->LibCname, Product->LibCverMin);
	ShowSummaryLine("Requires", Buff);
    }

    if (Product->InstallDir)
	ShowSummaryLine("Installation Directory", Product->InstallDir);
    if (Product->Size)
	ShowSummaryLine("Amount to Install", GetSizeStr(Product->Size, BYTES));
}

/*
 * Show summary for all products
 */
extern void ShowSummaryAll(McdData_t *		McdData,
			   int			MsgFlags)
{
    McdProduct_t *		Prod;

    for (Prod = McdData->Products; Prod; Prod = Prod->Next) {
	ShowSummary(Prod, FALSE, MsgFlags);
	ShowSummaryPkgs(Prod->Packages, Prod, FALSE, MsgFlags);
    }
}

/*
 * Show a product list summary
 */
extern void SummaryProductList(McdProduct_t *	Products,
			       int		OnlySelected,
			       int		MsgFlags)
{
    McdProduct_t *		Prod;
    int				f = MsgFlags;

    if (!Products)
	return;

    Msg(f, "\n%s\n\n", Center("PRODUCTS SUMMARY"));
    Msg(f, "%4s %-3s %-16s %-12s %s\n",
	"PRO#", "REQ", "NAME", "VERSION", "SUMMARY");
    Msg(f, "%4s %-3s %-16s %-12s %s\n",
	"----", "---", "----------------", "------------",
	"---------------------------------------");
    for (Prod = Products; Prod && Prod->Name; Prod = Prod->Next) {
	if (OnlySelected && !Prod->Selected)
	    continue;
	Msg(f, "%c%3d %-3s %-16s %-12s %s\n",
	    (Prod->Selected) ? '*' : ' ',
	    Prod->Number,
	    (Prod->Required) ? "YES" : "NO",
	    PRTS(Prod->Name),
	    PRTS(Prod->Version),
	    PRTS(Prod->Summary));
    }
    
    Msg(f, "\nNOTES:\n");
    Msg(f, "REQ = Product is required\n");
    Msg(f, "*   = Product is selected\n");
}
