/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.13 $";
#endif

/*
 * System Requirements
 */

#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/param.h>
#include "mcsysinfo-autoconfig.h"
#include "mcsysinfo.h"
#include "defs.h"
#include "sysreq.h"

extern char		       *Answer;

/*
 * Cleanup a string and remove unwanted characters.
 * Returns pointer to newly allocated buffer.
 */
static char *Clean(char *String)
{
    char		       *Buffer;
    char		       *cp;
    char		       *bp;

    Buffer = (char *) calloc(1, strlen(String)+1);
    if (!Buffer)
	return (char *) NULL;

    for (cp = String, bp = Buffer; cp && *cp; ++cp) {
	if (*cp != '-' && *cp != '/')
	    *bp++ = *cp;
    }

    return Buffer;
}

/*
 * Get specified info from mcSysInfo()
 */
extern char *GetSysInfo(int Cmd)
{
    MCSIquery_t			Query;

    memset(&Query, CNULL, sizeof(Query));
    Query.Op = MCSIOP_CREATE;
    Query.Cmd = Cmd;
    if (mcSysInfo(&Query) == 0) {
	return (char *) Query.Out;
    }

    return (char *) NULL;
}

/*
 * Take a version string and convert it to a numeric value with padding
 */
extern long GetVerNum(char *String)
{
    int				Argc;
    char		      **Argv;
    long			Num = 0;
    char			NumBuff[16];
    int				r1 = 0;
    int				r2 = 0;
    int				r3 = 0;

    if (!String)
	return 0;

    Argc = StrToArgv(String, ".", &Argv, NULL, 0);

    if (Argc >= 1)
	r1 = atoi(Argv[0]);
    if (Argc >= 2)
	r2 = atoi(Argv[1]);
    if (Argc >= 3)
	r3 = atoi(Argv[2]);

    (void) snprintf(NumBuff, sizeof(NumBuff), "%2.2d%2.2d%2.2d", r1, r2, r3);

    Num = strtol(NumBuff, NULL, 10);
    SetupDebug("GetVerNum(%s) = %d", String, Num);

    return Num;
}

/*
 * Set information about system in SysReq
 */
static int SysReqSet(SysReq_t *SysReq)
{
    MCSIquery_t			Query;
    char		       *cp;
    char		       *sp;

    if (!SysReq)
	return -2;

    /*
     * One time setup
     */
    memset(&Query, CNULL, sizeof(Query));
    Query.Op = MCSIOP_PROGRAM;
    Query.In = ProgramName;
    Query.InSize = strlen((char *)Query.In);
    mcSysInfo(&Query);

    if (cp = GetSysInfo(MCSI_OSNAME)) {
	if (EQ(cp, "IRIX64"))
	    /* Special hack for IRIX */
	    SysReq->OSname = strdup("IRIX");
	else
	    SysReq->OSname = Clean(cp);
    } else {
	Error("Cannot deterine OS name.");
	return -1;
    }

    if (cp = GetSysInfo(MCSI_OSVER)) {
	if (sp = strchr(cp, '-'))
	    *sp = CNULL;
	SysReq->OSver = strdup(cp);
	SysReq->OSverNum = GetVerNum(cp);
	if (sp = strchr(cp, '.')) {
	    *sp++ = CNULL;
	    SysReq->OSverMaj = atoi(cp);
	    /* Zap any further dot numbers */
	    if (cp = strchr(sp, '.'))
		*cp = CNULL;
	    SysReq->OSverMin = atoi(sp);
	}
    } else {
	Error("Cannot deterine OS version.");
	return -1;
    }

    if (cp = GetSysInfo(MCSI_APPARCH))
	SysReq->Arch = Clean(cp);
    else {
	Error("Cannot deterine system architecture.");
	return -1;
    }

    if (cp = GetSysInfo(MCSI_LIBCNAME))
	SysReq->LibcName = strdup(cp);
    if (cp = GetSysInfo(MCSI_LIBCVER))
	SysReq->LibcVer = strdup(cp);

    SetupDebug("SysReqSet: OSname=<%s> Ver=<%s> VerNum=%d Maj=%d Min=%d Arch=<%s>",
	       SysReq->OSname, SysReq->OSver, SysReq->OSverNum,
	       SysReq->OSverMaj, SysReq->OSverMin, SysReq->Arch);
    SetupDebug("SysReqSet: LibcName=<%s> LibcVer=<%s>",
	       PRTS(SysReq->LibcName), PRTS(SysReq->LibcVer));

    return 0;
}

/*
 * Check to see if this system meets system requirements of Product
 */
extern int SysReq(McdProduct_t *Product)
{
    static SysReq_t		SysReq;
    long			Ver1;
    long			Ver2;
    int				ErrCount = 0;

    Msg(0, "Checking system requirements . . . ");
    if (SysReqSet(&SysReq) != 0) {
	Error("Cannot determine system information.");
	return -1;
    }

    if (getuid() != 0) {
	Msg(0, "\nYou must run %s as \"root\" for the installation to properly\n",
	    ProgramName);
	Msg(0, "succeed.  You may still proceed with the installation if you\n");
	Msg(0, "wish, but the resulting installation may not function properly.\n\n");
	if (OverRide) {
	    Msg(0, "Continue anyway? YES\n");
	} else {
	    Answer = Ask("Continue anyway?", AnswersYN, NULL, NULL, 0);
	    if (!ConfirmFlag && !EQ(Answer, A_YES))
		UserBail();
	}
    }

    /* 
     * The second part of this test is a special hack for Linux x86.
     * Some Linux systems report i586 for AMD CPU's.
     */
    if (!EQ(Product->Arch, SysReq.Arch) && 
	!(EQ(Product->Arch, "i686") && EQ(SysReq.Arch, "i586"))) {
	Error("This software runs on %s systems and this system is a %s.",
	      Product->Arch, SysReq.Arch);
	++ErrCount;
    }

    if (!EQ(Product->OSname, SysReq.OSname)) {
	Error("This software runs on the %s OS and this system runs %s.",
	      Product->OSname, SysReq.OSname);
	++ErrCount;
    }

    if (Product->OSverMin) {
	Ver1 = GetVerNum(Product->OSverMin);
	if (SysReq.OSverNum < Ver1) {
	    Error("The minimum supported OS version is %s and this systemm is running %s.",
		  Product->OSverMin, SysReq.OSver);
	    ++ErrCount;
	}
    }

    if (Product->OSverMax) {
	Ver1 = GetVerNum(Product->OSverMax);
	if (SysReq.OSverNum > Ver1) {
	    Error("The latest supported OS version is %s and this systemm is running %s.",
		  Product->OSverMax, SysReq.OSver);
	    ++ErrCount;
	}
    }

    if (Product->LibCname && Product->LibCverMin &&
	SysReq.LibcName && SysReq.LibcVer) {
	Ver1 = GetVerNum(SysReq.LibcVer);
	Ver2 = GetVerNum(Product->LibCverMin);
	if (EQ(Product->LibCname, SysReq.LibcName) &&
	    Ver1 < Ver2) {
	    Error("The software requires %s %s or later and this system runs %s %s.",
		  Product->LibCname, Product->LibCverMin,
		  SysReq.LibcName, SysReq.LibcVer);
	    ++ErrCount;
	}
    }

    if (Product->LibCname && SysReq.LibcName) {
	if (!EQ(Product->LibCname, SysReq.LibcName)) {
	    Error("The software requires a libc named \"%s\" and this system has \"%s\".",
		  Product->LibCname, SysReq.LibcName);
	    ++ErrCount;
	}
    }

    if (ErrCount) {
	Msg(0, "\n");
	return -1;
    } else {
	Msg(0, "OK\n");
	return 0;
    }
}
