/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.11 $";
#endif

/*
 * Uninstall functions
 */

#include "defs.h"

static char *				Answer;
static char			       *AnswerList[3] = {
    A_ALL, A_NONE, NULL
};
static char			       *AnswerList2[4] = {
    A_ALL, A_NONE, A_SELECTED, NULL
};

/*
 * Count number of directories found in Path
 */
static int DirCount(char *Path)
{
    char		       *cp;
    int				Count = 0;

    for (cp = strchr(Path, '/'); cp; cp = strchr(cp+1, '/'))
	++Count;

    return Count;
}

/*
 * qsort directory comparision
 * Sort in descending order
 */
static int DirCmp(const void *p1, const void *p2)
{
    char		      **c1 = (char **) p1;
    char		      **c2 = (char **) p2;
    int				l1 = DirCount(*c1);
    int				l2 = DirCount(*c2);

    if (l1 < l2)
	return 1;
    if (l1 > l2)
	return -1;

    return 0;
}

/*
 * Un install distribution
 */
extern int UnInstallProduct(McdProduct_t *	Product,
			    McdData_t *		McdData)
{
    McdMap_t *		        Entry;
    McdPackage_t *		Pkg;
    struct stat			Stat;
    int				ErrCount = 0;
    static char			Path[MAXPATHLEN];
    static char			Dir[MAXPATHLEN];
    static char			Prompt[128];
    char *			Answer;
    char		       *cp;
    char		      **Dirs = NULL;
    int				DirCount = 0;
    int				i;
    int				DoIt = FALSE;
    int				Padding = 0;
    static Meter_t		MeterInfo;

    for (i = 0; i < SETUP_MAX_PROMPTS; ++i) {
	/* Reset selection */
	(void) McdPkgSelectSet(Product,
			       McdData,
			       FALSE, 		/* Enable */
			       A_ALL,		/* Selection String */
			       FALSE);		/* Select Required */

	ShowSummaryPkgs(Product->Packages, Product, FALSE, 0);
	Msg(0, "\n");
	Answer = Ask("Which packages would you like to remove?",
		     (McdData->PkgList) ? AnswerList2 : AnswerList,
		     (McdData->PkgList) ? A_SELECTED : A_ALL,
		     "Enter comma seperated list of PKG#'s (i.e. \"1,2,4\") or \"all\" or \"none\".",
		     AF_NUMLIST);
	SetupDebug("Ask() user returned <%s>", PRTS(Answer));
	if (EQ(Answer, A_NONE)) {
	    Error("You have choosen not to proceed with installation.");
	    return -1;
	}

	/*
	 * The A_SELECTED answer means go with what is already selected.
	 */
	if (!EQ(Answer, A_SELECTED)) {
	    /* Select the packages per the user's answer */
	    if (McdPkgSelectSet(Product, McdData, TRUE, Answer, FALSE) < 0)
		continue;
	}

	UpdateSize(Product);

	/* Show summary of selected packages */
	ShowSummaryPkgs(Product->Packages, Product, TRUE, 0);
	Msg(0, "\n");
	Answer = Ask("Would you like to proceed with removal?",
		     AnswersYN, A_YES, NULL, 0);
	if (EQ(Answer, A_YES)) {
	    /* Do it */
	    DoIt = TRUE;
	    break;
	}
    }

    if (!DoIt) {
	Error("Installation is canceled..");
	return -1;
    }

    /*
     * Explicitly remove the license file
     */
    snprintf(Path, sizeof(Path), "%s/%s",
	     Product->InstallDir, Product->InstallLic);
    SetupDebug("Removing license file <%s>", Path);
    (void) unlink(Path);

    /*
     * Perform any post actions now prior to the scripts being
     * removed.
     */
    if (McdPost(Product, TRUE) < 0) 
	return -1;

    /* Calculate Padding based on longest Pkg->Name */
    Padding = McdPkgPadding(Product->Packages);

    /* Count number of directories */
    for (Pkg = Product->Packages; Pkg; Pkg = Pkg->Next) {
	if (!Pkg->Selected)
	    continue;
	for (Entry = Pkg->MapList; Entry; Entry = Entry->Next) {
	    if (Entry->Type == MCDT_DIR)
		++DirCount;
	}
    }

    /* Alloc space for all the directories */
    Dirs = (char **) calloc(DirCount+1, sizeof(char *));
    if (!Dirs) {
	Error("calloc %d bytes failed: %s", sizeof(char *)*DirCount+1);
	Bail();
    }
    DirCount = 0;

    /*
     * Perform the removals
     */
    for (Pkg = Product->Packages; Pkg; Pkg = Pkg->Next) {
	if (!Pkg->Selected)
	    continue;

	memset(&MeterInfo, 0, sizeof(MeterInfo));
	MeterInfo.TermWidth = TermWidth;

	/* Calculate total */
	for (MeterInfo.Total = 0, Entry = Pkg->MapList; Entry;
	     Entry = Entry->Next)
	    ++(MeterInfo.Total);

	/* Set the Meter prompt */
	snprintf(Prompt, sizeof(Prompt), "Removing %-*s . . . ",
		 Padding, Pkg->Name);
	MeterInfo.Prompt = Prompt;

	/* Remove all the files and list directories for later deletion */
	for (Entry = Pkg->MapList; Entry; Entry = Entry->Next) {
	    ++(MeterInfo.Current);
	    Meter(&MeterInfo);
	    
	    if (!Entry->Dest || EQ(Entry->Dest, "-"))
		continue;
	    
	    if (Entry->Type == MCDT_SYMLINK) {
		SetupDebug("Link: Src=<%s> Dst=<%s> InstallDir=<%s>",
			   PRTS(Entry->Src), PRTS(Entry->Dest),
			   Product->InstallDir);
		(void) snprintf(Path, sizeof(Path), "%s/%s",
				Product->InstallDir, Entry->Src);
	    } else if (Entry->Dest[0] != '/')
		(void) snprintf(Path, sizeof(Path), "%s/%s", 
				Product->InstallDir, Entry->Dest);
	    else
		(void) snprintf(Path, sizeof(Path), "%s", 
				Entry->Dest);
	    
	    SetupDebug("Removing <%s>", Path);
	    if (lstat(Path, &Stat) != 0) {
		Error("%s: Missing file detected", Path);
		++ErrCount;
		continue;
	    }
	    switch (Entry->Type) {
	    case MCDT_SYMLINK:
	    case MCDT_FILE:
		if (unlink(Path) != 0) {
		    Error("%s: Remove file failed: %s", Path, SYSERR);
		    ++ErrCount;
		}
		/* Try to remove the parent directory without checking */
		(void) rmdir(DirName(Path));
		break;
	    case MCDT_DIR:
		Dirs[DirCount++] = strdup(Path);
		break;
	    default:
		Error("%s: File type %d is unknown.", Path, Entry->Type);
		++ErrCount;
	    }
	}
	Msg(0, "\n");
    }

    /*
     * Explicitly remove myself (the executing program).
     * Look in DistDir since we may have done a chdir() at startup
     * which invalidates ProgramPath.
     */
    snprintf(Path, sizeof(Path), "%s/%s",
	     McdData->DistDir, McdData->ProgramName);
    if (FileExists(Path, FALSE)) {
	SetupDebug("Removing myself <%s>", Path);
	if (unlink(Path) < 0) {
	    Error("%s: Cannot remove myself: %s", Path, SYSERR);
	    ++ErrCount;
	}
    }

    /*
     * Nuke all the MCD data files after doing basic sanity checks.
     */
    if (Product->InstallDir && *Product->InstallDir &&
	strlen(Product->InstallDir) > 4) {
	snprintf(Path, sizeof(Path), "/bin/rm -rf %s/%s",
		 Product->InstallDir, FN_MCD);
	SetupDebug("Removing MCD data <%s>", Path);
	system(Path);
    }

    /*
     * Sort Dirs with deepest entries first
     */
    qsort((void *) Dirs, DirCount, sizeof(char **), DirCmp);

    /*
     * Now wack all the Dirs
     */
    memset(&MeterInfo, 0, sizeof(MeterInfo));
    MeterInfo.TermWidth = TermWidth;
    MeterInfo.Prompt = "Removing directories . . . ";
    MeterInfo.Total = DirCount;
    for (i = 0; i < DirCount; ++i) {
	++(MeterInfo.Current);
	Meter(&MeterInfo);

	if (!Dirs[i])
	    break;

	SetupDebug("Removing <%s>", Dirs[i]);
	if (rmdir(Dirs[i]) != 0) {
	    if (errno != ENOENT) {
		Error("%s: Remove directory failed: %s", Dirs[i], SYSERR);
		++ErrCount;
	    }
	}
    }

    /* 
     * Make sure we're not in the installation directory when we try 
     * to remove it 
     */
    if (getcwd(Dir, sizeof(Dir))) {
	chdir("/");
	SetupDebug("Removing <%s>", Product->InstallDir);
	if (rmdir(Product->InstallDir) != 0) {
	    Error("%s: Remove installation directory failed: %s",
		  Product->InstallDir, SYSERR);
	    ++ErrCount;
	}
    } else {
	Error("Cannot determine current working directory: %s", SYSERR);
	++ErrCount;
    }

    if (ErrCount) {
	Msg(0, "\nThere were %d errors during UnInstall.\n", ErrCount);
	return -1;
    } else {
	Msg(0, "\nUnInstall succeeded.\n");
	return 0;
    }
}

/*
 * UnInstall products
 */
extern int UnInstall(McdData_t *McdData)
{
    McdProduct_t *		Prod;
    int				i;
    int				Status = -1;

    for (i = 0; i < SETUP_MAX_PROMPTS; ++i) {
	/*
	 * Ask user to select each product for installation
	 */
	for (Prod = McdData->Products; Prod; Prod = Prod->Next) {
	    Msg(0, "\n");
	    ShowSummary(Prod, FALSE, 0);
	    Msg(0, "\n");
	    
	    Answer = Ask("Select this product for removal?",
			 AnswersYN, A_YES, NULL, 0);
	    if (EQ(Answer, A_YES))
		Prod->Selected = TRUE;
	    else
		Prod->Selected = FALSE;
	}

	SummaryProductList(McdData->Products, TRUE, 0);
	Msg(0, "\n");

	Answer = Ask("Is the selection of products to be removed correct?",
		     AnswersYN, A_YES, NULL, 0);
	if (EQ(Answer, A_YES)) {
	    Status = 0;
	    break;
	}
    }

    /* Check to see if any products were selected */
    if (Status < 0) {
	Error("No products were selected for installation.");
	return -1;
    }

    for (Prod = McdData->Products; Prod; Prod = Prod->Next) {
	if (UnInstallProduct(Prod, McdData) < 0)
	    --Status;
    }

    return Status;
}
