/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.9 $";
#endif

/*
 * Setup Verify functions
 */

#include "defs.h"

/*
 * Verify installed distribution is correct
 */
extern int Verify(McdData_t *		McdData)
{
    McdProduct_t *		Product;
    McdMap_t *		        Entry;
    McdPackage_t *		Pkg;
    struct stat			Stat;
    int				ErrCount = 0;
    int				Padding = 0;
    int				l;
    static char			Path[MAXPATHLEN];
    static char			Buff[MAXPATHLEN];
    static Meter_t		MeterInfo;
    static char			Prompt[128];
    char *			cp;

    /*
     * Figure out Installation Directory based on ProgramPath
     */
    SetInstallDir(McdData);

    for (Product = McdData->Products; Product; Product = Product->Next) {
	Msg(0, "Verifying Product %s . . .\n", Product->Name);

	/* Calculate Padding based on longest Pkg->Name */
	Padding = McdPkgPadding(Product->Packages);
	
	for (Pkg = Product->Packages; Pkg; Pkg = Pkg->Next) {
	    if (!Pkg->Installed)
		continue;
	    
	    memset(&MeterInfo, 0, sizeof(MeterInfo));
	    MeterInfo.TermWidth = TermWidth;
	    
	    /* Calculate total */
	    for (MeterInfo.Total = 0, Entry = Pkg->MapList; Entry;
		 Entry = Entry->Next)
		++(MeterInfo.Total);
	    
	    /* Set the Meter prompt */
	    snprintf(Prompt, sizeof(Prompt), "    Verifying %-*s . . . ",
		     Padding, Pkg->Name);
	    MeterInfo.Prompt = Prompt;
	    
	    /* Do the verification */
	    for (Entry = Pkg->MapList; Entry; Entry = Entry->Next) {
		++(MeterInfo.Current);
		Meter(&MeterInfo);
		
		if (!Entry->Src || EQ(Entry->Src, "-"))
		    continue;
		
		(void) snprintf(Path, sizeof(Path), "%s/%s", 
				Product->InstallDir, Entry->Dest);
		
		if (Entry->Type == MCDT_SYMLINK) {
		    /*
		     * Verify symlink
		     */
		    (void) snprintf(Path, sizeof(Path), "%s/%s", 
				    Product->InstallDir, Entry->Src);
		    
		    SetupDebug("Checking <%s>", Path);
		    
		    /* Read the link contents into Buff */
		    l = readlink(Path, Buff, sizeof(Buff));
		    if (l < 0) {
			Error("%s: Readlink failed: %s", Path, SYSERR);
			++ErrCount;
			continue;
		    }
		    /* Need to null terminate since readlink() doesn't */
		    Buff[l] = CNULL;
		    
		    SetupDebug("Readlink(%s) returned <%s>", Path, Buff);
		    SetupDebug("    Looking for <%s>", Entry->Dest);
		    
		    /* 
		     * Check to see if the contents of the symlink match
		     * the destination Entry.
		     */
		    if (!eq(Buff, Entry->Dest)) {
			Error("%s: Link points at <%s> should point to <%s>",
			      Path, Buff, Entry->Dest);
			++ErrCount;
		    }
		} else {
		    /*
		     * Verify file or directory.
		     */
		    SetupDebug("Checking <%s>", Path);
		    if (stat(Path, &Stat) != 0) {
			Error("%s: Missing file detected", Path);
			++ErrCount;
			continue;
		    }
		    
		    /*
		     * We can't check size of MCDF_EDIT files because their size
		     * will have changed due to the file edit
		     */
		    if (FLAGS_OFF(Entry->Flags, MCDF_EDIT) &&
			(Entry->Size > 0) &&
			(Stat.st_size != Entry->Size)) {
			Error("%s: File size mismatch - file is %d should be %d", 
			      Path, Stat.st_size, Entry->Size);
			++ErrCount;
			continue;
		    }
		}
	    }
	    Msg(0, "\n");
	}
    }

    if (ErrCount) {
	Msg(0, "\nThere were %d errors during verification.\n", ErrCount);
	return -1;
    } else {
	Msg(0, "\nVerification succeeded.\n");
	return 0;
    }
}
