/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.51 $";
#endif

/*
 * Class Info
 */
#include "defs.h"

/*
 * Generic headers
 */
#define HeaderNamesNorm	{ "DESCRIPTION", "VALUE", NULL }
#define HeaderNamesAll	{ "DESCRIPTION", "KEY", "VALUE", NULL }

/*
 * Header format definetions
 */
HeaderFormat_t HeadersNorm = {
    HeaderNamesNorm,	2,
    { 0, 0 },		2,
    { 0, 0 },		2,
};
HeaderFormat_t HeadersAll = {
    HeaderNamesAll,	3,
    { 0, 0, 0 },	3,
    { 0, 0, 0 },	3,
};
HeaderFormat_t PartHeader = {
    { "PART", "CAPACITY", "USED", "TYPE", "USAGE", NULL },	5,
    { 10, 10, 10, 10, 0 },	5,
    { 13, 13, 13, 13, 46 },	5,
};
HeaderFormat_t HardwareHeader = {
    { "DEVICE", "VENDOR", "MODEL", "TYPE", "CLASS", "SPEED", "CAPACITY",
      NULL },				7,
    { 12, 8, 19, 8, 10, 8, 9 },		7,
    { 16, 11, 25, 11, 14, 11, 12 },	7,
};
HeaderFormat_t SoftwareHeader = {
    { "PACKAGE", "VERSION", "DESCRIPTION", NULL },	3,
    { 20, 15, 0 },	3,
    { 30, 12, 58 },	3,
};
HeaderFormat_t PrinterHeader = {
    { "QUEUE", "SERVER", "DESCRIPTION", NULL },	3,
    { 20, 15, 0 },	3,
    { 30, 12, 58 },	3,
};

/*
 * Class information
 */
ClassInfo_t ClassInfo[] = {
    { CN_ALL,			{ NULL }, 
      NULL,
      NULL,
      NULL,
      NULL,
      "All classes of information",
      NULL,
      NULL, NULL,
      FALSE },
    { CN_GENERAL, 					/* Name */
      { NULL },						/* Aliases */
      GeneralShow,					/* Show function */
      ShowInfoList,					/* List Function */
      GeneralShowInfo,					/* ShowInfo data */
      "--class General --show Name1,Name2,...",		/* Show Usage */
      "General Information",				/* Label */
      NULL,						/* Banner */
      &HeadersNorm,					/* Normal Headers */
      &HeadersAll,					/* All Headers */
      TRUE },						/* Enabled by default*/
    { CN_KERNEL,
      { NULL },
      KernelShow,
      KernelList,
      NULL,
      "--class Kernel --show Name1,Name2,...",
      "Kernel Information",
      NULL,
      &HeadersNorm,
      &HeadersAll,
      FALSE },
    { CN_SYSCONF,
      { NULL },
      SysConfShow,
      SysConfList,
      NULL,
      "--class SysConf --show Name1,Name2,...",
      "SysConf Information",
      NULL,
      &HeadersNorm,
      &HeadersAll,
      FALSE },
    { CN_PRINTER,
      { NULL },
      ShowPrinter,
      ListPrinter,
      NULL,
      "--class printer --show prt1,prt2...",
      "Printer Information",
      NULL,
      &PrinterHeader,
      &PrinterHeader,
      FALSE },
    { CN_PARTITION,
      { NULL },
      ShowPartition,
      ListPartition,
      NULL,
      "--class partition --show part1,part2...",
      "Partition Information",
      NULL,
      &PartHeader,
      &PartHeader,
      FALSE },
    { CN_DEVICE,
      { CN_HARDWARE, NULL },
      DeviceShow,
      DeviceList,
      NULL,
      "--class device --show Dev1,Dev2...",
      "Hardware Information",
      NULL,
      &HardwareHeader,
      &HardwareHeader,
      TRUE },
    { CN_NETIF,
      { NULL },
      ShowNetIf,
      ListNetIf,
      NULL,
      "--class NetIf --show if1,if2...",
      "Network Interfaces",
      NULL,
      &HeadersNorm,
      &HeadersAll,
      FALSE },
    { CN_NETWORK,
      { NULL },
      ShowNetwork,
      ShowInfoList,
      NetworkShowInfo,
      "--class network --show Name1,Name2,...",
      "Network Information",
      NULL,
      &HeadersNorm,
      &HeadersAll,
      FALSE },
    { CN_SOFTINFO,
      { NULL },
      ShowSoftInfo,
      ListSoftInfo,
      NULL,
      "--class software --show pkg1,pkg2...",
      "Software Information",
      NULL,
      &SoftwareHeader,
      &SoftwareHeader,
      FALSE },
    { CN_LICENSE,
      { NULL },
      ShowLicense,
      ListLicense,
      NULL,
      "--class license --show license1,license2...",
      "License Information",
      NULL,
      &HeadersNorm,
      &HeadersAll,
      FALSE },
    { 0 },
};

/*
 * List class names.
 */
extern void ClassList(void)
{
    register ClassInfo_t       *ClassPtr;
    static char			Buff[BUFSIZ];
    char		       *Name;
    register char	      **cpp;

    if (!VL_TERSE) {
	SImsg(SIM_INFO, 
"The following values may be specified with the `-class' option:\n");
	SImsg(SIM_INFO, "%-20s %s\n", "VALUE", "DESCRIPTION");
    }

    for (ClassPtr = &ClassInfo[0]; ClassPtr->Name; ++ClassPtr) {
	if (ClassPtr->Aliases) {
	    snprintf(Buff, sizeof(Buff), ClassPtr->Name);
	    for (cpp = ClassPtr->Aliases; cpp && *cpp; ++cpp) {
		Name = &Buff[strlen(Buff)];
		snprintf(Name, sizeof(Buff)-strlen(*cpp)-3, ",%s", *cpp);
	    }
	    Name = Buff;
	} else {
	    Name = ClassPtr->Name;
	}
	SImsg(SIM_INFO, "%-20s %s\n", Name, ClassPtr->Label);
    }
}

/*
 * Lookup a class named Name.
 */
extern ClassInfo_t *ClassGetByName(char *Name)
{
    register ClassInfo_t       *ClassPtr;
    register char	      **cpp;

    for (ClassPtr = &ClassInfo[0]; ClassPtr->Name; ++ClassPtr) {
	if (EQ(Name, ClassPtr->Name))
	    return(ClassPtr);
	for (cpp = ClassPtr->Aliases; cpp && *cpp; ++cpp)
	    if (EQ(Name, *cpp))
		return(ClassPtr);
	    
    }

    return((ClassInfo_t *) NULL);
}

/*
 * Convert String into an array of class names.
 */
extern char **ClassGetNames(char *String)
{
    char **			List;
    char **			cpp;
    char *			cp;
    int				i;
    int				x;

    if (!String)
	return (char **) NULL;

    List = Split(",", String);

    /*
     * Check for "all" and if found, rebuild with the actual list of all
     * class names.
     */
    for (cpp = List; cpp && *cpp; ++cpp)
	if (EQ(*cpp, CN_ALL)) {
	    /* Destroy old list */
	    DestroyArgv(&List, -1);
	    /*
	     * Create a new List with all class names
	     */
	    List = (char **) xcalloc(sizeof(ClassInfo)/sizeof(ClassInfo_t),
				     sizeof(char **));
	    for (i = 0, x = 0; ClassInfo[i].Name; ++i) {
		if (EQ(ClassInfo[i].Name, CN_ALL))
		    continue;
		List[x++] = ClassInfo[i].Name;
	    }
	}

    if (DebugOn) {
	cp = Join(",", List, 0);
	McDebug("ClassGetNames: <%s> -> <%s>", String, cp);
	(void) free(cp);
    }

    return List;
}

/*
 * Set class information
 */
extern void ClassSetInfo(char *Names)
{
    register ClassInfo_t       *ClassPtr;
    register char	       *cp;
    char		       *String;

    if (!Names)
	return;

    if (EQ(Names, CN_ALL))
	/*
	 * Enable all classes
	 */
	for (ClassPtr = &ClassInfo[0]; ClassPtr->Name; ++ClassPtr)
	    ClassPtr->Enabled = TRUE;
    else {
	/*
	 * Enable a specific list of classes.
	 * The Names variable is a `,' seperated list
	 * of class names.
	 */
	String = (char *) strdup(Names);
	for (cp = (char *) strtok(String, ","); cp;
	     cp = (char *) strtok((char *)NULL, ",")) {
	    ClassPtr = ClassGetByName(cp);
	    if (!ClassPtr) {
		SImsg(SIM_GERR, "The class name `%s' is invalid.", cp);
		ClassList();
		exit(1);
	    }
	    ClassPtr->UserEnabled = TRUE;
	}
	(void) free(String);

	/* Since user specified class, disable defaults */
	for (ClassPtr = &ClassInfo[0]; ClassPtr->Name; ++ClassPtr)
	    ClassPtr->Enabled = FALSE;
    }
}

/*
 * Call each enabled show function
 */
extern int ClassCall(char *		NameStr,
		     MCSIopts_t *	MCSIopts)
{
    ClassInfo_t		       *ClassPtr;
    char		      **Argv;
    int				Argc;
    int				Status;
    int				ErrCount = 0;
    int				First = 1;

    Argc = StrToArgv(NameStr, ",", &Argv, NULL, 0);

    for (ClassPtr = &ClassInfo[0]; ClassPtr->Name; ++ClassPtr)
	if ((ClassPtr->Enabled || ClassPtr->UserEnabled) && ClassPtr->Show) {
	    if (First) {
		if (EncodeType == OE_HTML)
		    HtmlStart();
		First = 0;
	    }

	    ClassPtr->Options = MCSIopts;

	    Status = (*ClassPtr->Show)(ClassPtr,
				       (Argc > 0) ? Argv : (char **) NULL);

	    if (Status == CLASS_UNSUPPORTED && ClassPtr->UserEnabled) {
		SImsg(SIM_CERR, "Class `%s' is unsupported on this system.",
		      ClassPtr->Name);
		++ErrCount;
	    }
	}

    if (!First && EncodeType == OE_HTML)
	HtmlEnd();

    return (ErrCount == 0) ? 0 : -1;
}

/*
 * Call each Class List function
 */
extern void ClassCallList(void)
{
    register ClassInfo_t       *ClassPtr;

    for (ClassPtr = &ClassInfo[0]; ClassPtr->Name; ++ClassPtr)
	if (ClassPtr->List)
	    (*ClassPtr->List)(ClassPtr);
}

/*
 * Print class members.
 * If ClassNames is set, limit listings to specified classes.
 */
extern int ClassMembers(char **		ClassNames)
{
    ClassInfo_t *		clp;

    for (clp = &ClassInfo[0]; clp->Name; ++clp) {
	if (ClassNames && StrArray(clp->Name, ClassNames) <= 0)
	    continue;
	if (clp->List)
	    (*clp->List)(clp);
    }

    return 0;
}
