/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.2 $";
#endif

/*
 * SIE List functions
 */
#include "defs.h"

/*
 * List possible values for List
 */
static void ListInfoDesc(char *		OptStr,
			 ListInfo_t *	List)
{
    register ListInfo_t	       *ListPtr;

    if (OptStr && !VL_TERSE)
	SImsg(SIM_INFO, 
	"\nThe following values may be specified to the `%s' option:\n",
	OptStr);

    if (!VL_TERSE)
        SImsg(SIM_INFO, "%-20s %s\n", "VALUE", "DESCRIPTION");

    for (ListPtr = List; ListPtr->Name; ++ListPtr)
	SImsg(SIM_INFO, "%-20s %s\n", ListPtr->Name,
	       (ListPtr->Desc) ? ListPtr->Desc : "");
}

/*
 * List type names.
 */
static void ListType()
{
    register DevType_t	       *TypePtr;

    if (FormatType == FT_REPORT) {
	for (TypePtr = DevTypes; TypePtr->Name; ++TypePtr)
	    ReportType(TypePtr->Name, TypePtr->Desc);
    } else {
	SImsg(SIM_INFO, 
"The following values may be specified with the `-type' option:\n");
	SImsg(SIM_INFO, "%-20s %s\n", "VALUE", "DESCRIPTION");

	for (TypePtr = DevTypes; TypePtr->Name; ++TypePtr)
	    SImsg(SIM_INFO, "%-20s %s\n", PRTS(TypePtr->Name), PRTS(TypePtr->Desc));
    }
}

/*
 * List Format values
 */
static void ListFormat(void)
{
    ListInfoDesc("--format", FormatNames);
}

/*
 * List Encode values
 */
static void ListEncode(void)
{
    ListInfoDesc("--encode", EncodeNames);
}

/*
 * List MsgClass values
 */
static void ListMsgClass(void)
{
    ListInfoDesc("--msgclass", MsgClassNames);
}

/*
 * List MsgLevel values
 */
static void ListMsgLevel(void)
{
    ListInfoDesc("--msglevel", MsgLevelNames);
}

ListInfo_t ListInfo[] = {
    { 0, "class",	"Values for `--class' option",	ClassList },
    { 0, "encode",	"Values for `--encode' option",	ListEncode },
    { 0, "format",	"Values for `--format' option",	ListFormat },
    { 0, "msgclass",	"Values for `--msgclass' option",ListMsgClass },
    { 0, "msglevel",	"Values for `--msglevel' option",ListMsgLevel },
    { 0, "show",	"Values for `--show' option",	ClassCallList },
    { 0, "type",	"Values for `--type' option",	ListType },
    { 0 },
};

/*
 * List list values
 */
static void ListList(void)
{
    ListInfoDesc("--list", ListInfo);
}

/*
 * List information about each word found in Str.
 */
extern void List(char *		Str,
		 char **	ClassNames)
{
    register ListInfo_t	*       ListPtr;
    char *		        Word;
    int				Found;

    if (EQ(Str, "-")) {
	ListList();
	return;
    }

    for (Word = strtok(Str, ","); Word; Word = strtok((char *)NULL, ",")) {
	for (ListPtr = ListInfo, Found = FALSE; 
	     ListPtr->Name && !Found; ++ListPtr)
	    if (EQ(Word, ListPtr->Name)) {
		Found = TRUE;
		(*ListPtr->List)(NULL);
	    }

	if (!Found) {
	    SImsg(SIM_CERR, "The word \"%s\" is invalid.", Word);
	    ListList();
	    return;
	}
    }
}
