/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.66 $";
#endif

/*
 * Show Devices related functions.
 */

#include "defs.h"
#include "html.h"

extern void DetailsDiskDrive();
extern void DetailsFrameBuffer();
extern void DetailsMonitor();
extern void DetailsNetIf();
extern void DetailsCPU();
extern void PrintDevice();
extern void PrintGeneric();

static float 			TotalDisk = 0;
extern DevType_t		DevTypes[];

/*
 * List valid arguments for Devices class. 
 */
extern void DeviceList(ClassInfo_t *Class)
{
    if (!VL_TERSE && (FormatType == FT_TREE || FormatType == FT_COLUMNS))
	SImsg(SIM_INFO, 
	      "\n\nTo show specific devices use `%s'.\n\n",
	      Class->ShowUsage);
}

/*
 * Reset Print* values for Info
 */
extern void PrintInfoReset(Info)
     PrintInfo_t	       *Info;
{
    Info->DetailsTreeText = Info->DetailsTreeHtml = 
	Info->DetailsColumnsText = Info->DetailsColumnsHtml =
	Info->DetailsReport = NULL;
}

/*
 * Initialize the Print (show) functions for each device type
 */
static void DevTypesInit()
{
    register int		i;

    for (i = 0; DevTypes[i].Name; ++i)
	switch (DevTypes[i].Type) {
	case DT_CPU:
	    DevTypes[i].Print = DetailsCPU;		break;
	case DT_DISKDRIVE:
	    DevTypes[i].Print = DetailsDiskDrive;	break;
	case DT_FRAMEBUFFER:
	    DevTypes[i].Print = DetailsFrameBuffer;	break;
	case DT_NETIF:
	    DevTypes[i].Print = DetailsNetIf;		break;
	case DT_MONITOR:
	    DevTypes[i].Print = DetailsMonitor;		break;
	case DT_NONE:
	    break;
	default:
	    DevTypes[i].Print = PrintGeneric;		break;
	}
}

/*
 * Print all device info
 */
extern int DeviceShow(MyClass, Names)
    ClassInfo_t		       *MyClass;
    char		      **Names;
{
    static DevInfo_t 	       *RootDev;
    static char		       *RptData[2];
    static MCSIquery_t		Query;
    static char			Buff[256];
    static PrintInfo_t		PrintInfo;

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Op = MCSIOP_CREATE;
    Query.Cmd = MCSI_DEVTREE;
    if (mcSysInfo(&Query) != 0) {
	return -1;
    }
    RootDev = (DevInfo_t *) Query.Out;

    if (!RootDev)
	return CLASS_UNSUPPORTED;

    ShowStart(MyClass, (int *)NULL);

    TotalDisk = (float) 0;

    DevTypesInit();

    memset(&PrintInfo, CNULL, sizeof(PrintInfo));
    PrintInfo.DevInfo = RootDev;
    /*
     * Recursively print info
     */
    PrintDevice(&PrintInfo);
    /*
     * Terminate any remaining lines
     */
    ShowLineEnd();

    ShowEnd(MyClass);

    if (VL_DESC && TotalDisk > (float) 0)
	switch (FormatType) {
	case FT_TREE:
	case FT_COLUMNS:
	    snprintf(Buff, sizeof(Buff), "Total Disk Capacity is %s", 
		     GetSizeStr((Large_t) TotalDisk, 
				(Large_t) MBYTES));

	    switch (EncodeType) {
	    case OE_TEXT:
		SImsg(SIM_INFO, "\n%s\n", Buff);
		break;
	    case OE_HTML:
		HtmlNewLine();
		HtmlLine(-1, HtmlBold(Buff));
		break;
	    }
	    break;
	case FT_REPORT:
	    RptData[0] = GetSizeStr((Large_t) TotalDisk, 
				    (Large_t) MBYTES);
	    Report(CN_DEVICE, R_TOTALDISK, NULL, RptData, 1);
	    break;
	}

    return 0;
}

/*
 * --RECURSE--
 * Print info about a device.  Recursively calls itself for slaves and
 * next elements
 */
extern void PrintDevice(Info)
     PrintInfo_t	       *Info;
{
    DevInfo_t 		       *DevInfo;
    int 			OffSet;
    DevType_t 		       *DevType = NULL;

    if (!Info)
	return;

    DevInfo = Info->DevInfo;
    OffSet = Info->OffSet;
    /* Reset */
    PrintInfoReset(Info);

    DevType = TypeGetByType(DevInfo->Type);
    if (!DevType)
	DevType = TypeGetByType(DT_GENERIC);

    /*
     * If device->Name is not set, this is the root of the device tree
     */
    if (DevInfo->Name && DevType->Enabled) {
	Info->DevType = DevType;
	if (DevType->Print)
	    (*DevType->Print)(Info);
	else
	    PrintGeneric(Info);
    }

    /*
     * Descend
     */
    if (DevInfo->Slaves) {
	Info->DevInfo = DevInfo->Slaves;
	Info->OffSet = (DevInfo->Name) ? OffSet+OffSetAmt : 0;
	PrintDevice(Info);
    }

    /*
     * Traverse
     */
    if (DevInfo->Next) {
	Info->DevInfo = DevInfo->Next;
	Info->OffSet = (DevInfo->Name) ? OffSet : 0;
	PrintDevice(Info);
    }
}

char *GetDescLabel(DevDesc)
    DevDesc_t		       *DevDesc;
{
    if (DevDesc && DevDesc->Label)
	return(DevDesc->Label);
    else
	return("Description");
}

/*
 * Get a DevDesc_t pointer to the prime description of a device.
 */
extern DevDesc_t *PrimeDescPtr(DevInfo)
    DevInfo_t		       *DevInfo;
{
    register DevDesc_t	       *DevDesc;

    if (!DevInfo)
	return((DevDesc_t *) NULL);

    for (DevDesc = DevInfo->DescList; DevDesc; DevDesc = DevDesc->Next)
	if (FLAGS_ON(DevDesc->Flags, DA_PRIME))
	    return(DevDesc);

    return((DevDesc_t *) NULL);
}

/*
 * Print Generic device info
 */
extern void DetailsGeneric(Info)
     PrintInfo_t	       *Info;
{
    DevInfo_t 		       *DevInfo;
    DevType_t 		       *DevType;
    int 			OffSet;
    ClassType_t		       *ClassType = NULL;
    static char			Buff[1024];

    if (FormatType == FT_REPORT)
	return;

    if (!Info)
	return;

    if (!VL_CONFIG)
	return;

    DevInfo = Info->DevInfo;
    DevType = Info->DevType;
    OffSet = Info->OffSet;

    if (DevInfo->Vendor)
	ShowLabel("Vendor", DevInfo->Vendor, OffSet);
    if (DevInfo->Model)
	ShowLabel("Model", DevInfo->Model, OffSet);
    if (DevInfo->Capacity)
	ShowLabel("Capacity", DevInfo->Capacity, OffSet);
    if (DevInfo->Speed)
	ShowLabel("Speed", DevInfo->Speed, OffSet);
    if (DevInfo->Part)
	ShowLabel("Part", DevInfo->Part, OffSet);
    if (DevInfo->Serial)
	ShowLabel("Serial", DevInfo->Serial, OffSet);
    if (DevInfo->Ident)
	ShowLabel("Identifier", IdentString(DevInfo->Ident), OffSet);
    if (DevInfo->Revision)
	ShowLabel("Revision", DevInfo->Revision, OffSet);
    if (DevInfo->Unit >= 0)
	ShowLabel("Unit", itoa(DevInfo->Unit), OffSet);
    if (DevInfo->Bus >= 0)
	ShowLabel("Bus", itoa(DevInfo->Bus), OffSet);
    if (DevInfo->DevNum >= 0)
	ShowLabel("Device Number", itoa(DevInfo->DevNum), OffSet);
    if (DevInfo->Port >= 0)
	ShowLabel("Port", itoa(DevInfo->Port), OffSet);
    if (DevInfo->Addr >= 0) {
	snprintf(Buff, sizeof(Buff), "0x%x", DevInfo->Addr);
	ShowLabel("Address", Buff, OffSet);
    }
    if (DevInfo->Prio >= 0)
	ShowLabel("Priority", itoa(DevInfo->Prio), OffSet);
    if (DevInfo->Vec >= 0)
	ShowLabel("Vector", itoa(DevInfo->Vec), OffSet);
    if (DevType && DevType->Desc)
	ShowLabel("Device Type", DevType->Desc, OffSet);
    if (ClassType = ClassTypeGetByType(DevInfo->Type, DevInfo->ClassType))
	ShowLabel("Class",
		  (ClassType->Desc) ? ClassType->Desc : ClassType->Name, 
		  OffSet);
}

/*
 * Print the details about a generic device in OE_TEXT and OE_HTML.
 * The ShowLabel() call takes care of formating for OE_TEXT or OE_HTML.
 */
static void DetailsDeviceTextHtml(Info)
     PrintInfo_t	       *Info;
{     
    DevInfo_t 		       *DevInfo;
    DevType_t 		       *DevType;
    DevType_t 		       *mdt;
    static ClassInfo_t	       *HwClass;
    register DevDesc_t	       *DevDesc;
    int 			OffSet = 0;
    static char			Buff[8192];
    register char	      **cpp;
    register DevInfo_t 	       *pd;

    if (!VL_DESC && !VL_CONFIG)
	return;

    if (!Info)
	return;

    DevInfo = Info->DevInfo;
    DevType = Info->DevType;
    OffSet = Info->OffSet;

    if (!DevInfo->Name)
	return;

    if (FormatType == FT_COLUMNS && EncodeType == OE_HTML) {
	/*
	 * We need to wrap the details in a table row/cell since we're
	 * part of a larger table.
	 */
	if (!HwClass)
	    HwClass = ClassGetByName(CN_HARDWARE);
	if (!HwClass) {
	    SImsg(SIM_DBG, "DetailsDeviceTextHtml: Cannot find class <%s>",
		  CN_HARDWARE);
	    return;
	}
	snprintf(Buff, sizeof(Buff), "colspan=%d",
		 HwClass->Headers->HeadersCount);

	HtmlTrStart(NULL);
	HtmlTdStart(Buff);
    }

    ShowLabelStart();

    DetailsGeneric(Info);

    if (VL_DESC || VL_CONFIG) {
	if (DevInfo->Aliases && DevInfo->Aliases[0]) {
	    Buff[0] = CNULL;
	    for (cpp = DevInfo->Aliases; cpp && *cpp; ++cpp) {
		if (Buff[0])
		    StrAppend(Buff, sizeof(Buff), " ");
		StrAppend(Buff, sizeof(Buff), *cpp);
	    }
	    ShowLabel("Alias Names", Buff, OffSet);
	}
	if (DevInfo->DescList)
	    for (DevDesc = DevInfo->DescList; DevDesc; 
		 DevDesc = DevDesc->Next)
		ShowLabel(GetDescLabel(DevDesc), DevDesc->Desc, OffSet);
	if (DevInfo->Files && DevInfo->Files[0]) {
	    Buff[0] = CNULL;
	    for (cpp = DevInfo->Files; cpp && *cpp; ++cpp) {
		if (Buff[0])
		    StrAppend(Buff, sizeof(Buff), " ");
		StrAppend(Buff, sizeof(Buff), *cpp);
	    }
	    ShowLabel("Device Files", Buff, OffSet);
	}
	if (DevInfo->Master && DevInfo->Master->Name) {
	    Buff[0] = CNULL;
	    if (DevInfo->Master->Name) {
		if (Buff[0])
		    StrAppend(Buff, sizeof(Buff), " ");
		StrAppend(Buff, sizeof(Buff), DevInfo->Master->Name);
	    } else if (DevInfo->Master->Model) {
		if (Buff[0])
		    StrAppend(Buff, sizeof(Buff), " ");
		StrAppend(Buff, sizeof(Buff), DevInfo->Master->Model);
		if (mdt = TypeGetByType(DevInfo->Master->Type)) {
		    StrAppend(Buff, sizeof(Buff), " ");
		    StrAppend(Buff, sizeof(Buff), mdt->Desc);
		}
	    }
	    ShowLabel("Connected to", Buff, OffSet);
	}
    }

    if (VL_CONFIG) {
	if (DevInfo->Slaves && (DevInfo->Name || DevInfo->Model || 
				  (DevType && DevType->Desc))) {
	    Buff[0] = CNULL;
	    for (pd = DevInfo->Slaves; pd; pd = pd->Next) {
		if (Buff[0])
		    StrAppend(Buff, sizeof(Buff), " ");
		StrAppend(Buff, sizeof(Buff), pd->Name);
	    }
	    ShowLabel("Attached Device(s)", Buff, OffSet);
	}
    }

    switch (FormatType) {
    case FT_COLUMNS:
	if (Info->DetailsColumnsText)
	    (*Info->DetailsColumnsText)(Info);
	break;
    case FT_TREE:
	if (Info->DetailsTreeText)
	    (*Info->DetailsTreeText)(Info);
	break;
    }

    ShowLabelEnd();

    if (FormatType == FT_COLUMNS && EncodeType == OE_HTML) {
	HtmlTdEnd();
	HtmlTrEnd();
    }
}

/*
 * Print device info details
 */
static void DetailsDevice(Info)
     PrintInfo_t	       *Info;
{
    switch (EncodeType) {
    case OE_HTML:
    case OE_TEXT:
	DetailsDeviceTextHtml(Info);
	break;
    }
}

/*
 * Print general device information in FT_TREE format
 */
static void DetailsDeviceTree(Info)
     PrintInfo_t	       *Info;
{     
    DevInfo_t 		       *DevInfo;
    DevType_t 		       *DevType;
    int 			OffSet;
    ClassType_t		       *ClassType = NULL;
    register char	       *cp;
    register char	      **cpp;
    static char			Buff[8192];
    static char			Buff2[8192];

    if (!Info)
	return;

    DevInfo = Info->DevInfo;
    DevType = Info->DevType;
    OffSet = Info->OffSet;

    if (!DevInfo->Name)
	return;

    snprintf(Buff, sizeof(Buff), ShowEncodeName(DevInfo->Name));

    if (DevInfo->AltName) {
	StrAppend(Buff, sizeof(Buff), " (");
	StrAppend(Buff, sizeof(Buff), DevInfo->AltName);
	StrAppend(Buff, sizeof(Buff), ")");
    }
    
    if (DevInfo->Vendor || DevInfo->Model || DevInfo->ModelDesc ||
	DevInfo->Capacity || DevInfo->Speed ||
	(DevType && DevType->Desc)) {
	StrAppend(Buff, sizeof(Buff), " is a");

	if (DevInfo->Vendor || DevInfo->Model) {
	    if (EncodeType == OE_TEXT) {
		snprintf(Buff2, sizeof(Buff2), " \"%s%s%s\"", 
			 (DevInfo->Vendor) ? DevInfo->Vendor : "",
			 (DevInfo->Vendor && DevInfo->Model) ? " " : "",
			 (DevInfo->Model) ? DevInfo->Model : "");
		StrAppend(Buff, sizeof(Buff), Buff2);
	    } else if (EncodeType == OE_HTML) {
		snprintf(Buff2, sizeof(Buff2), " %s%s%s ", 
			 (DevInfo->Vendor) ? DevInfo->Vendor : "",
			 (DevInfo->Vendor && DevInfo->Model) ? " " : "",
			 (DevInfo->Model) ? DevInfo->Model : "");
		StrAppend(Buff, sizeof(Buff), HtmlItalic(Buff2));
	    }
	}

	if (ClassType = ClassTypeGetByType(DevInfo->Type, DevInfo->ClassType)){
	    StrAppend(Buff, sizeof(Buff), " ");
	    StrAppend(Buff, sizeof(Buff), 
		      (ClassType->Desc) ? ClassType->Desc : ClassType->Name);
	}

	/* Make sure ModelDesc != ClassType */
	if (DevInfo->ModelDesc &&
	    (!ClassType || !EQ(ClassType->Name, DevInfo->ModelDesc))) {
	    StrAppend(Buff, sizeof(Buff), " ");
	    StrAppend(Buff, sizeof(Buff), DevInfo->ModelDesc);
	}

	if (DevInfo->Capacity &&
	    (!DevInfo->ModelDesc || !EQ(DevInfo->Capacity, DevInfo->ModelDesc))) {
	    StrAppend(Buff, sizeof(Buff), " ");
	    StrAppend(Buff, sizeof(Buff), DevInfo->Capacity);
	}

	if (DevInfo->Speed &&
	    (!DevInfo->ModelDesc || !EQ(DevInfo->Speed, DevInfo->ModelDesc))) {
	    StrAppend(Buff, sizeof(Buff), " ");
	    StrAppend(Buff, sizeof(Buff), DevInfo->Speed);
	}

	if (DevType && DevType->Desc) {
	    StrAppend(Buff, sizeof(Buff), " ");
	    StrAppend(Buff, sizeof(Buff), DevType->Desc);
	} else if ((cp = DescPrime(DevInfo->DescList)) &&
		   !EQ(cp, DevInfo->Model)) {
	    StrAppend(Buff, sizeof(Buff), " ");
	    StrAppend(Buff, sizeof(Buff), cp);
	}
    } else if (cp = DescPrime(DevInfo->DescList)) {
	StrAppend(Buff, sizeof(Buff), " is a ");
	StrAppend(Buff, sizeof(Buff), cp);
    }

    ShowLine(OffSet, Buff);

    DetailsDevice(Info);
}

/*
 * Print general device information in FT_REPORT format
 */
static void DetailsDeviceReport(Info)
     PrintInfo_t	       *Info;
{     
    DevInfo_t 		       *DevInfo;
    DevType_t 		       *DevType;
    register DevDesc_t	       *DevDesc;
    ClassType_t		       *ClassType = NULL;
    DevInfo_t		       *di;
    register char	      **cpp;
    static char		       *RptData[26];
    static char			FileBuff[BUFSIZ];
    static char			SlaveBuff[BUFSIZ];
    static char			UnitBuff[50];
    static char			AddrBuff[50];
    static char			PrioBuff[50];
    static char			VecBuff[50];
    static char			NodeIDBuff[50];

    if (!Info)
	return;

    DevInfo = Info->DevInfo;
    DevType = Info->DevType;

    if (!DevInfo->Name)
	return;

    memset(RptData, CNULL, sizeof(RptData));

    RptData[0] = PRTS(DevInfo->AltName);

    if (DevInfo->Master)
	RptData[1] = PRTS(DevInfo->Master->Name);
    else if (DevInfo->MasterName)
	RptData[1] = PRTS(DevInfo->MasterName);

    FileBuff[0] = CNULL;
    for (cpp = DevInfo->Files; cpp && *cpp; ++cpp) {
	if (FileBuff[0])
	    strcat(FileBuff, ",");
	if (strlen(*cpp) + strlen(FileBuff) + 3 >= sizeof(FileBuff))
	    break;
	strcat(FileBuff, *cpp);
    }
    if (FileBuff[0])
	RptData[2] = FileBuff;

    if (DevType) {
	RptData[3] = PRTS(DevType->Name);
	RptData[4] = PRTS(DevType->Desc);
    }
    RptData[5] = PRTS(DevInfo->Model);
    RptData[6] = PRTS(DevInfo->ModelDesc);

    if (DevInfo->Unit >= 0) {
	(void) snprintf(UnitBuff, sizeof(UnitBuff), "%d", DevInfo->Unit);
	RptData[7] = UnitBuff;
    }
    if (DevInfo->Addr != -1 && DevInfo->Addr != 0) {
	(void) snprintf(AddrBuff, sizeof(AddrBuff), "0x%x", DevInfo->Addr);
	RptData[8] = AddrBuff;
    }
    if (DevInfo->Prio != 0 && DevInfo->Prio != -1) {
	(void) snprintf(PrioBuff, sizeof(PrioBuff), "%d", DevInfo->Prio);
	RptData[9] = PrioBuff;
    }
    if (DevInfo->Vec != 0 && DevInfo->Vec != -1) {
	(void) snprintf(VecBuff, sizeof(VecBuff), "%d", DevInfo->Vec);	
	RptData[10] = VecBuff;
    }
    if (DevInfo->NodeID != 0 && DevInfo->NodeID != -1 && 
	DevInfo->NodeID != -2) {
	(void) snprintf(NodeIDBuff, sizeof(NodeIDBuff), "%d", DevInfo->NodeID);
	RptData[11] = NodeIDBuff;
    }
    if (ClassType = ClassTypeGetByType(DevInfo->Type, DevInfo->ClassType)) {
	RptData[12] = ClassType->Name;
	RptData[13] = (ClassType->Desc) ? ClassType->Desc : "";
    } 
    if (DevInfo->Vendor)
	RptData[14] = DevInfo->Vendor;
    if (DevInfo->Serial)
	RptData[15] = DevInfo->Serial;
    if (DevInfo->Revision)
	RptData[16] = DevInfo->Revision;
    if (DevInfo->Ident)
	RptData[17] = IdentString(DevInfo->Ident);
    if (DevInfo->Part)
	RptData[18] = DevInfo->Part;
    if (DevInfo->Next)
	RptData[19] = DevInfo->Next->Name;
    if (DevInfo->Slaves) {
	SlaveBuff[0] = CNULL;
	for (di = DevInfo->Slaves; di; di = di->Next) {
	    if (SlaveBuff[0])
		strcat(SlaveBuff, ",");
	    if (strlen(di->Name) + strlen(SlaveBuff) + 3 >= sizeof(SlaveBuff))
		break;
	    strcat(SlaveBuff, di->Name);
	}
	if (SlaveBuff[0])
	    RptData[20] = SlaveBuff;
    }
    if (DevInfo->Capacity)
	RptData[21] = DevInfo->Capacity;
    if (DevInfo->Speed)
	RptData[22] = DevInfo->Speed;
    if (DevInfo->Bus >= 0)
	RptData[23] = itoa(DevInfo->Bus);
    if (DevInfo->DevNum >= 0)
	RptData[24] = itoa(DevInfo->DevNum);
    if (DevInfo->Port >= 0)
	RptData[25] = itoa(DevInfo->Port);

    Report(CN_DEVICE, R_NAME, PRTS(DevInfo->Name), RptData, 
	   sizeof(RptData) / sizeof(char *));

    /*
     * Report description info
     */
    for (DevDesc = DevInfo->DescList; DevDesc; DevDesc = DevDesc->Next) {
	RptData[0] = PRTS(DevDesc->Label);
	RptData[1] = PRTS(DevDesc->Desc);
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 2);
    }

    if (Info->DetailsReport)
	(*Info->DetailsReport)(Info);
}

/*
 * Print general device information in FT_COLUMNS format as OE_TEXT
 */
extern void DetailsDeviceColumnsText(Info)
     PrintInfo_t	       *Info;
{     
    DevInfo_t 		       *DevInfo;
    DevType_t 		       *DevType;
    ClassInfo_t		       *Class;
    ClassType_t		       *ClassType = NULL;
    static char			Vendor[128];
    static char		       *FormatStr;
    char		      **Format;
    char		       *cp;

    if (!Info)
	return;

    DevInfo = Info->DevInfo;
    DevType = Info->DevType;
    ClassType = ClassTypeGetByType(DevInfo->Type, DevInfo->ClassType);

    if (!DevInfo->Name)
	return;

    /*
     * Shorten Vendor by removing everything after first space
     */
    if (DevInfo->Vendor) {
	snprintf(Vendor, sizeof(Vendor), "%s", DevInfo->Vendor);
	if (cp = strchr(Vendor, ' '))
	    *cp = CNULL;
    } else {
	Vendor[0] = '-';
	Vendor[1] = CNULL;
    }

    if (!FormatStr) {
	Class = ClassGetByName(CN_HARDWARE);
	if (!Class) {
	    SImsg(SIM_DBG, "DetailsDeviceColumns: Cannot find %s class",
		  CN_HARDWARE);
	    return;
	}
	Format = ColGetFmt(Class->Headers);
	if (!Format)
	    return;

	FormatStr = ColGetFmtStr(Format);
	if (!FormatStr)
	    return;
    }

    SImsg(SIM_INFO, FormatStr,
	  CPRT(DevInfo->Name),
	  Vendor,
	  CPRT(DevInfo->Model),
	  (DevType) ? CPRT(DevType->Name) : "-",
	  (ClassType) ? CPRT(ClassType->Name) : "-",
	  CPRT(DevInfo->Speed),
	  CPRT(DevInfo->Capacity)
	  );

    DetailsDevice(Info);
}

/*
 * Print general device information in FT_COLUMNS format as OE_HTML
 */
extern void DetailsDeviceColumnsHtml(Info)
     PrintInfo_t	       *Info;
{     
    DevInfo_t 		       *DevInfo;
    DevType_t 		       *DevType;
    ClassType_t		       *ClassType = NULL;
    static char			Vendor[128];
    char		       *cp;
    char		       *DataArgv[7];
    char		       *FormArgv[7];
    static int			Counter;
    char		        Options[128];

    if (!Info)
	return;

    DevInfo = Info->DevInfo;
    DevType = Info->DevType;
    ClassType = ClassTypeGetByType(DevInfo->Type, DevInfo->ClassType);

    if (!DevInfo->Name)
	return;

    /*
     * Shorten Vendor by removing everything after first space
     */
    if (DevInfo->Vendor) {
	snprintf(Vendor, sizeof(Vendor), "%s", DevInfo->Vendor);
	if (cp = strchr(Vendor, ' '))
	    *cp = CNULL;
    } else {
	snprintf(Vendor, sizeof(Vendor), "%s", HTML_SPACE);
    }

    DataArgv[0] = PRTS(DevInfo->Name);
    FormArgv[0] = "align=left";
    DataArgv[1] = Vendor;
    FormArgv[1] = "align=left";
    DataArgv[2] = PRTS(DevInfo->Model);
    FormArgv[2] = "align=left";
    DataArgv[3] = (DevType) ? PRTS(DevType->Name) : HTML_SPACE;
    FormArgv[3] = "align=left";
    DataArgv[4] = (ClassType) ? PRTS(ClassType->Name) : HTML_SPACE;
    FormArgv[4] = "align=center";
    DataArgv[5] = PRTS(DevInfo->Speed);
    FormArgv[5] = "align=center";
    DataArgv[6] = PRTS(DevInfo->Capacity);
    FormArgv[6] = "align=right";

    /* Rows alternate colors */
    if (Counter++ % 2)
	snprintf(Options, sizeof(Options), "bgcolor=\"%s\"",
		 HTML_BGLINE_1A);
    else
	snprintf(Options, sizeof(Options), "bgcolor=\"%s\"",
		 HTML_BGLINE_1B);

    HtmlTableData(DataArgv, sizeof(DataArgv)/sizeof(char *),
		  FormArgv, Options);

    DetailsDevice(Info);
}

/*
 * Print general device information in FT_COLUMNS format
 */
extern void DetailsDeviceColumns(Info)
     PrintInfo_t	       *Info;
{
    switch (EncodeType) {
    case OE_TEXT:
	DetailsDeviceColumnsText(Info);
	break;
    case OE_HTML:
	DetailsDeviceColumnsHtml(Info);
	break;
    }
}

/*
 * Print general device information
 */
extern void PrintGeneric(Info)
     PrintInfo_t	       *Info;
{
    switch (FormatType) {
    case FT_TREE:
	DetailsDeviceTree(Info);
	break;
    case FT_COLUMNS:
	DetailsDeviceColumns(Info);
	break;
    case FT_REPORT:
	DetailsDeviceReport(Info);
	break;
    }
}

/*
 * Print info about disk partitioning in FT_TREE format as OE_TEXT
 */
static void DetailsPartTreeText(Info, Disk)
     PrintInfo_t	       *Info;
     DiskDrive_t	       *Disk;
{
    int 			OffSet;
    register DiskPart_t        *dp;
    PartInfo_t		       *PartInfo;
    DiskPart_t		       *DiskPart;
    static char			Buff[128];
    static char			TitleDef[] = "Partition Information";
    int				SecSize = 512;

    if (!Info || !Disk || !Disk->DiskPart || !Disk->DiskPart->PartInfo)
	return;

    OffSet = Info->OffSet;
    DiskPart = Disk->DiskPart;
    PartInfo = DiskPart->PartInfo;

    if (PartInfo->SecSize)
	SecSize = PartInfo->SecSize;
    else if (Disk->SecSize)
	SecSize = Disk->SecSize;

    SImsg(SIM_INFO, "\n");
    ShowOffSet(OffSet);
    if (PartInfo->Title)
	(void) snprintf(Buff, sizeof(Buff), "%s %s", 
			PartInfo->Title, TitleDef);
    else
	(void) snprintf(Buff, sizeof(Buff), "%s", TitleDef);
    SImsg(SIM_INFO, "%40s\n", Buff);

    ShowOffSet(OffSet);
    SImsg(SIM_INFO, "%8s %10s %10s %9s\n",
	   "", "START", "NUMBER OF", "SIZE");

    ShowOffSet(OffSet);
    SImsg(SIM_INFO, "%8s %10s %10s %9s %8.8s %s\n",
	   "PART", "SECTOR", "SECTORS", "(MB)", "TYPE", "USAGE");

    for (dp = DiskPart; dp; dp = dp->Next) {
	ShowOffSet(OffSet);
#if	defined(HAVE_PRINTF_LL)
	SImsg(SIM_INFO, "%8s %10lld %10lld %9.2f %8.8s %s\n",
#else
	SImsg(SIM_INFO, "%8s %10ld %10ld %9.2f %8.8s %s\n",
#endif	/* HAVE_PRINTF_LL */
	       (dp->PartInfo->Name) ? dp->PartInfo->Name : "",
#if	defined(HAVE_PRINTF_LL)
	       dp->PartInfo->StartSect,
	       dp->PartInfo->NumSect,
#else
	       (long) dp->PartInfo->StartSect,
	       (long) dp->PartInfo->NumSect,
#endif	/* HAVE_PRINTF_LL */
	       bytes_to_mbytes(nsect_to_bytes(dp->PartInfo->NumSect, SecSize)),
	       (dp->PartInfo->Type) ? dp->PartInfo->Type : "",
	       (dp->PartInfo->MntName) ? dp->PartInfo->MntName : ""
	       );
    }
}

/*
 * Print info about disk partitioning in FT_TREE + OE_HTML format.
 */
static void DetailsPartTreeHtml(Info, Disk)
     PrintInfo_t	       *Info;
     DiskDrive_t	       *Disk;
{
    int 			OffSet;
    register DiskPart_t        *dp;
    PartInfo_t		       *PartInfo;
    DiskPart_t		       *DiskPart;
    static HtmlTable_t		Table;
    static char			Buff[128];
    static char		       *Argv[8];
    static char		       *FormatArgv[8];
    static char			TitleDef[] = "Partition Information";
    int				SecSize = 512;
    int				Count = 0;

    if (!Info || !Disk || !Disk->DiskPart || !Disk->DiskPart->PartInfo)
	return;

    OffSet = Info->OffSet;
    DiskPart = Disk->DiskPart;
    PartInfo = DiskPart->PartInfo;

    if (PartInfo->SecSize)
	SecSize = PartInfo->SecSize;
    else if (Disk->SecSize)
	SecSize = Disk->SecSize;

    SImsg(SIM_INFO, "\n");
    ShowOffSet(OffSet);
    if (PartInfo->Title)
	(void) snprintf(Buff, sizeof(Buff), "%s %s", 
			PartInfo->Title, TitleDef);
    else
	(void) snprintf(Buff, sizeof(Buff), "%s", TitleDef);
    
    Table.Title = Buff;
    Table.Style = "margin-left: .5in";
    Table.Width = 200;
    Table.CellPadding = 3;
    Table.CellSpacing = 1;
    HtmlTableStart(&Table);

    Argv[0] = "Part";
    Argv[1] = "Start Sector";
    Argv[2] = "Number of Sectors";
    Argv[3] = "Size (MB)";
    Argv[4] = "Type";
    Argv[5] = "Usage";

    FormatArgv[0] = "valign=bottom";
    FormatArgv[1] = "valign=bottom";
    FormatArgv[2] = "valign=bottom";
    FormatArgv[3] = "valign=bottom";
    FormatArgv[4] = "valign=bottom align=right";
    FormatArgv[5] = "valign=bottom align=left";
    snprintf(Buff, sizeof(Buff), "bgcolor=\"%s\"", HTML_BGHEAD_2);
    HtmlTableHeader(Argv, 6, FormatArgv, Buff);

    FormatArgv[0] = "align=right";
    FormatArgv[1] = "align=right";
    FormatArgv[2] = "align=right";
    FormatArgv[3] = "align=right";
    FormatArgv[4] = "align=right";
    FormatArgv[5] = "align=left";

    memset(Argv, CNULL, sizeof(Argv));

    for (dp = DiskPart; dp; dp = dp->Next) {
	Argv[0] = (dp->PartInfo->Name) ? dp->PartInfo->Name : "";
	if (Argv[1]) free(Argv[1]);
	Argv[1] = strdup(Ltoa(dp->PartInfo->StartSect));
	if (Argv[2]) free(Argv[2]);
	Argv[2] = strdup(Ltoa(dp->PartInfo->NumSect));

	if (Argv[3]) free(Argv[3]);
	snprintf(Buff, sizeof(Buff), "%9.2f",
		 bytes_to_mbytes(nsect_to_bytes(dp->PartInfo->NumSect, 
						SecSize)));
	Argv[3] = strdup(Buff);

	Argv[4] = (dp->PartInfo->Type) ? dp->PartInfo->Type : "";
	Argv[5] = (dp->PartInfo->MntName) ? dp->PartInfo->MntName : "";
	snprintf(Buff, sizeof(Buff), "bgcolor=\"%s\"",
		 (Count++ % 2) ? HTML_BGLINE_2A : HTML_BGLINE_2B);
	HtmlTableData(Argv, 6, FormatArgv, Buff);
    }

    HtmlTableEnd();
}

/*
 * Print info about disk partitioning in FT_REPORT format.
 *
 * Output is in format:
 *
 *	device|desc|$Name|part|$PartName|$StartSect|$NumSect|$Size|$Use
 */
static void DetailsPartReport(Info, Disk)
    PrintInfo_t		       *Info;
    DiskDrive_t		       *Disk;
{
    register DiskPart_t        *dp;
    PartInfo_t		       *pi;
    static char		       *RptData[7];
    static char			StartBuff[50];
    static char			NumBuff[50];
    static char			SizeBuff[50];
    int				SecSize = 512;

    if (!Disk || !Info || !Info->DevInfo)
	return;

    if (Disk->SecSize)
	SecSize = Disk->SecSize;

    for (dp = Disk->DiskPart; dp; dp = dp->Next) {
	pi = dp->PartInfo;
	if (pi->SecSize) 
	    SecSize = pi->SecSize;
#if	defined(HAVE_PRINTF_LL)
	(void) snprintf(StartBuff, sizeof(StartBuff), "%lld", pi->StartSect);
	(void) snprintf(NumBuff, sizeof(NumBuff), "%lld", pi->NumSect);
#else
	(void) snprintf(StartBuff, sizeof(StartBuff), "%ld", (long) pi->StartSect);
	(void) snprintf(NumBuff, sizeof(NumBuff), "%ld", (long) pi->NumSect);
#endif	/* HAVE_PRINTF_LL */
	(void) snprintf(SizeBuff, sizeof(SizeBuff), "%9.2f", 
			bytes_to_mbytes(nsect_to_bytes(pi->NumSect, SecSize)));
	RptData[0] = R_PART;
	RptData[1] = PRTS(pi->Name);
	RptData[2] = StartBuff;
	RptData[3] = NumBuff;
	RptData[4] = SizeBuff;
	RptData[5] = PRTS(pi->Type);
	RptData[6] = PRTS(pi->MntName);
	Report(CN_DEVICE, R_DESC, PRTS(Info->DevInfo->Name), RptData, 7);
    }
}
/*
 * Assemble the "Label" piece of disk info for output.
 */
static char *PrDkLbl(Disk, Label)
     DiskDrive_t	       *Disk;
     char		       *Label;
{
    static char			Buff[256];
    int				Len;

    if (!Disk || !Label)
	return((char *) NULL);

    Buff[0] = CNULL;

    if (Disk->DataType)
	(void) snprintf(Buff, sizeof(Buff), "%s ", Disk->DataType);

    Len = strlen(Buff);
    (void) snprintf(&Buff[Len], sizeof(Buff)-Len, "%s", Label);

    return(Buff);
}

/*
 * Get the capacity of a disk drive
 */
extern char *GetDiskSize(DevInfo, DiskDrive)
    DevInfo_t		       *DevInfo;
    DiskDrive_t		       *DiskDrive;
{
    float			Amt = 0;
    static char			Buff[32];
    u_long			Cyls = 0;

    if (!DevInfo || !DiskDrive)
	return((char *) NULL);

    SImsg(SIM_DBG, 
	 "%s: GetDiskSize size=%.2f dcyl=%d pcyl=%d sect=%d hd=%d secsize=%d",
	  DevInfo->Name,
	  (float)DiskDrive->Size,
	  DiskDrive->DataCyl, DiskDrive->PhyCyl, 
	  DiskDrive->Sect, DiskDrive->Tracks,
	  DiskDrive->SecSize);

    if (DiskDrive->DataCyl)
	Cyls = DiskDrive->DataCyl;
    else if (DiskDrive->PhyCyl)
	Cyls = DiskDrive->PhyCyl;

    if (DiskDrive->Size == 0 &&
	Cyls && DiskDrive->Sect && DiskDrive->Tracks) {

	DiskDrive->Size = (float) nsect_to_mbytes(Cyls * 
						  DiskDrive->Sect * 
						  DiskDrive->Tracks, 
						  DiskDrive->SecSize);
    }

    if (DiskDrive->Size > 0)
	Amt = DiskDrive->Size;

    /* If it's small enough, we care about the precision so we do it here */
    if (Amt > 0 && Amt < 5) {
	(void) snprintf(Buff, sizeof(Buff), "%.1f MB", Amt);
	return(Buff);
    } else if (Amt)
	return(GetSizeStr((Large_t) Amt, MBYTES));
    else
	return((char *) NULL);
}

/*
 * Print Disk Drive information in FT_TREE format.
 */
static void DetailsDiskDriveTree(Info, Disk)
     PrintInfo_t	       *Info;
     DiskDrive_t	       *Disk;
{
    DevInfo_t 		       *DevInfo;
    int 			OffSet;
    static char			Buff[16];
    char		       *DiskSize;

    if (!Info || !Info->DevInfo || !Disk)
	return;

    DevInfo = Info->DevInfo;

    if (DiskSize = GetDiskSize(DevInfo, Disk))
	AddDevDesc(DevInfo, DiskSize,
		   PrDkLbl(Disk, "Capacity"), DA_APPEND);
    if (Disk->Label)
	AddDevDesc(DevInfo, Disk->Label, 
		   PrDkLbl(Disk, "Disk Label"), DA_APPEND);
    if (Disk->Unit >= 0) {
	if (FLAGS_ON(Disk->Flags, DF_HEXUNIT))
	    (void) snprintf(Buff, sizeof(Buff), "%.3x", Disk->Unit);
	else
	    (void) snprintf(Buff, sizeof(Buff), "%d", Disk->Unit);
	AddDevDesc(DevInfo, Buff, 
		   PrDkLbl(Disk, "Unit Number"), DA_APPEND);
    }
    if (Disk->Slave >= 0)
	AddDevDesc(DevInfo, itoa(Disk->Slave), 
		   PrDkLbl(Disk, "Slave Number"), DA_APPEND);

    /* Cylinders */
    if (Disk->DataCyl > 0)
	AddDevDesc(DevInfo, itoa(Disk->DataCyl), 
		   PrDkLbl(Disk, "Data Cylinders"), DA_APPEND);
    if (Disk->PhyCyl > 0)
	AddDevDesc(DevInfo, itoa(Disk->PhyCyl), 
		   PrDkLbl(Disk, "Physical Cylinders"), DA_APPEND);
    if (Disk->AltCyl > 0)
	AddDevDesc(DevInfo, itoa(Disk->AltCyl), 
		   PrDkLbl(Disk, "Alt Cylinders"), DA_APPEND);
    if (Disk->CylSkew > 0)
	AddDevDesc(DevInfo, itoa(Disk->CylSkew), 
		   PrDkLbl(Disk, "Cylinder Skew"), DA_APPEND);
    if (Disk->APC > 0)
	AddDevDesc(DevInfo, itoa(Disk->APC), 
		   PrDkLbl(Disk, "Alternates / Cylinder"), DA_APPEND);

    /* Tracks */
    if (Disk->Tracks > 0)
	AddDevDesc(DevInfo, itoa(Disk->Tracks), 
		   PrDkLbl(Disk, "Tracks"), DA_APPEND);
    if (Disk->AltTracksPerZone > 0)
	AddDevDesc(DevInfo, itoa(Disk->AltTracksPerZone), 
		   PrDkLbl(Disk, "Alt Tracks/Zone"), DA_APPEND);
    if (Disk->AltTracksPerVol > 0)
	AddDevDesc(DevInfo, itoa(Disk->AltTracksPerVol), 
		   PrDkLbl(Disk, "Alt Tracks/Volume"), DA_APPEND);
    if (Disk->TrackSkew > 0)
	AddDevDesc(DevInfo, itoa(Disk->TrackSkew), 
		   PrDkLbl(Disk, "Track Skew"), DA_APPEND);

    /* Floppy related */
    if (Disk->StepsPerTrack > 0)
	AddDevDesc(DevInfo, itoa(Disk->StepsPerTrack), 
		   PrDkLbl(Disk, "Steps/Tracks"), DA_APPEND);

    /* Sectors */
    if (Disk->Sect > 0)
	AddDevDesc(DevInfo, itoa(Disk->Sect), 
		   PrDkLbl(Disk, "Sectors/Track"), DA_APPEND);
    if (Disk->SecSize > 0)
	AddDevDesc(DevInfo, itoa(Disk->SecSize), 
		   PrDkLbl(Disk, "Sector Size"), DA_APPEND);
    if (Disk->PhySect > 0)
	AddDevDesc(DevInfo, itoa(Disk->PhySect), 
		   PrDkLbl(Disk, "Physical Sectors/Track"), DA_APPEND);
    if (Disk->AltSectPerZone > 0)
	AddDevDesc(DevInfo, itoa(Disk->AltSectPerZone), 
		   PrDkLbl(Disk, "Alt Sectors/Zone"), DA_APPEND);
    if (Disk->SectGap > 0)
	AddDevDesc(DevInfo, itoa(Disk->SectGap), 
		   PrDkLbl(Disk, "Sector Gap Length"), DA_APPEND);

    /* General Misc */
    if (Disk->RPM > 0)
	AddDevDesc(DevInfo, itoa(Disk->RPM), 
		   PrDkLbl(Disk, "RPM"), DA_APPEND);
    if (Disk->IntrLv > 0)
	AddDevDesc(DevInfo, itoa(Disk->IntrLv), 
		   PrDkLbl(Disk, "Interleave"), DA_APPEND);
    if (Disk->PROMRev > 0)
	AddDevDesc(DevInfo, itoa(Disk->PROMRev), 
		   PrDkLbl(Disk, "PROM Revision"), DA_APPEND);

}

/*
 * Print Disk Drive information in FT_REPORT format.
 *
 * Output format is:
 *
 *	device|desc|$Name|$Label|$Desc|$Flags|$DataType
 */
static void DetailsDiskDriveReport(Info, Disk)
    PrintInfo_t 	       *Info;
    DiskDrive_t		       *Disk;
{
    DevInfo_t 		       *DevInfo;
    static char		       *RptData[4];
    static char			Buff[50];
    char		       *DiskSize = NULL;

    if (!Info || !Info->DevInfo || !Disk)
	return;

    DevInfo = Info->DevInfo;

    /* Set the DataType for all calls below */
    RptData[2] = Disk->DataType;

    if (DiskSize = GetDiskSize(DevInfo, Disk)) {
	(void) snprintf(Buff, sizeof(Buff), "%s", DiskSize);
	RptData[0] = "capacity";
	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }

    if (Disk->Label) {
	RptData[0] = "disklabel";
	RptData[1] = Disk->Label;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }
    if (Disk->Unit >= 0) {
	if (FLAGS_ON(Disk->Flags, DF_HEXUNIT))
	    (void) snprintf(Buff, sizeof(Buff), "%.3x", Disk->Unit);
	else
	    (void) snprintf(Buff, sizeof(Buff), "%d", Disk->Unit);
	RptData[0] = "unit";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }
    if (Disk->Slave >= 0) {
	(void) snprintf(Buff, sizeof(Buff), "%d", Disk->Slave);
	RptData[0] = "slave";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }

    /* Cylinders */
    if (Disk->DataCyl > 0) {
	(void) snprintf(Buff, sizeof(Buff), "%ld", Disk->DataCyl);
	RptData[0] = "datacyl";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }
    if (Disk->PhyCyl > 0) {
	(void) snprintf(Buff, sizeof(Buff), "%ld", Disk->PhyCyl);
	RptData[0] = "phycyl";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }
    if (Disk->AltCyl > 0) {
	(void) snprintf(Buff, sizeof(Buff), "%ld", Disk->AltCyl);
	RptData[0] = "altcyl";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }
    if (Disk->CylSkew > 0) {
	(void) snprintf(Buff, sizeof(Buff), "%ld", Disk->CylSkew);
	RptData[0] = "cylskew";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }
    if (Disk->APC > 0) {
	(void) snprintf(Buff, sizeof(Buff), "%ld", Disk->APC);
	RptData[0] = "apc";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }

    /* Tracks */
    if (Disk->Tracks > 0) {
	(void) snprintf(Buff, sizeof(Buff), "%ld", Disk->Tracks);
	RptData[0] = "tracks";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }
    if (Disk->AltTracksPerZone > 0) {
	(void) snprintf(Buff, sizeof(Buff), "%ld", Disk->AltTracksPerZone);
	RptData[0] = "alttracksperzone";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }
    if (Disk->AltTracksPerVol > 0) {
	(void) snprintf(Buff, sizeof(Buff), "%ld", Disk->AltTracksPerVol);
	RptData[0] = "alttrackspervol";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }
    if (Disk->TrackSkew > 0) {
	(void) snprintf(Buff, sizeof(Buff), "%ld", Disk->TrackSkew);
	RptData[0] = "trackskew";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }

    /* Sectors */
    if (Disk->Sect > 0) {
	(void) snprintf(Buff, sizeof(Buff), "%ld", Disk->Sect);
	RptData[0] = "sectors";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }
    if (Disk->SecSize > 0) {
	(void) snprintf(Buff, sizeof(Buff), "%d", Disk->SecSize);
	RptData[0] = "sectsize";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }
    if (Disk->PhySect > 0) {
	(void) snprintf(Buff, sizeof(Buff), "%ld", Disk->PhySect);
	RptData[0] = "hardsectors";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }
    if (Disk->AltSectPerZone > 0) {
	(void) snprintf(Buff, sizeof(Buff), "%ld", Disk->AltSectPerZone);
	RptData[0] = "altsectperzone";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }

    /* General Misc */
    if (Disk->RPM > 0) {
	(void) snprintf(Buff, sizeof(Buff), "%ld", Disk->RPM);
	RptData[0] = "rpm";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }
    if (Disk->IntrLv > 0) {
	(void) snprintf(Buff, sizeof(Buff), "%ld", Disk->IntrLv);
	RptData[0] = "interleave";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }
    if (Disk->PROMRev > 0) {
	(void) snprintf(Buff, sizeof(Buff), "%ld", Disk->PROMRev);
	RptData[0] = "promrev";	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 3);
    }
}

/*
 * Print info about a disk device.
 */
extern void DetailsDiskDrive(Info)
     PrintInfo_t	       *Info;
{
    DevInfo_t 		       *DevInfo;
    DevType_t 		       *DevType;
    int 			OffSet;
    DiskDriveData_t	       *Data = NULL;
    DiskDrive_t		       *Disk = NULL;
    char		       *DiskSize = NULL;
    char		       *cp;
    size_t			len;

    if (!Info)
	return;

    DevInfo = Info->DevInfo;
    DevType = Info->DevType;

    if (DevInfo && DevInfo->DevSpec)
	Data = (DiskDriveData_t *) DevInfo->DevSpec;

    /*
     * Make sure DataType is set
     */
    if (Data && Data->HWdata && !Data->HWdata->DataType)
	Data->HWdata->DataType = DK_DTYPE_HW;
    if (Data && Data->OSdata && !Data->OSdata->DataType)
	Data->OSdata->DataType = DK_DTYPE_OS;
	
    /*
     * Disk is what we should use for general info about this disk.
     * We prefer the HW data over the OS data.
     */
    if (Data && Data->HWdata)
	Disk = Data->HWdata;
    else if (Data && Data->OSdata)
	Disk = Data->OSdata;

    if (Disk) {
	if (DiskSize = GetDiskSize(DevInfo, Disk))
	    DevInfo->Capacity = strdup(DiskSize);
	TotalDisk += Disk->Size;
    }

    if (Data) {
	if (VL_CONFIG) {
	    switch (FormatType) {
	    case FT_COLUMNS:
	    case FT_TREE:
		if (Data->HWdata)
		    DetailsDiskDriveTree(Info, Data->HWdata);
		if (Data->OSdata)
		    DetailsDiskDriveTree(Info, Data->OSdata);
		break;
	    case FT_REPORT:
		if (Data->HWdata)
		    DetailsDiskDriveReport(Info, Data->HWdata);
		if (Data->OSdata)
		    DetailsDiskDriveReport(Info, Data->OSdata);
		break;
	    }
	}
    }

    PrintGeneric(Info);

    if (VL_ALL && Data && Data->HWdata && Data->HWdata->DiskPart)
	switch (FormatType) {
	case FT_COLUMNS: 
	case FT_TREE: 
	    switch (EncodeType) {
	    case OE_TEXT:
		DetailsPartTreeText(Info, Data->HWdata);
		break;
	    case OE_HTML: 
		DetailsPartTreeHtml(Info, Data->HWdata);
		break;
	    }
	    break;
	case FT_REPORT:	
	    DetailsPartReport(Info, Data->HWdata);
	    break;
	}
    if (VL_ALL && Data && Data->OSdata && Data->OSdata->DiskPart)
	switch (FormatType) {
	case FT_COLUMNS: 
	case FT_TREE: 
	    switch (EncodeType) {
	    case OE_TEXT:
		DetailsPartTreeText(Info, Data->OSdata);
		break;
	    case OE_HTML:
		DetailsPartTreeHtml(Info, Data->OSdata);
		break;
	    }
	    break;
	case FT_REPORT:	
	    DetailsPartReport(Info, Data->OSdata);
	    break;
	}
}

/*
 * Print info about a frame buffer.
 */
extern void DetailsFrameBuffer(Info)
     PrintInfo_t	       *Info;
{
    DevInfo_t 		       *DevInfo;
    FrameBuffer_t 	       *fb = NULL;
    static char		       *RptData[3];

    if (!Info)
	return;

    DevInfo = Info->DevInfo;

    if (DevInfo && DevInfo->DevSpec)
	fb = (FrameBuffer_t *) DevInfo->DevSpec;

    if (fb && VL_CONFIG) {
	switch (FormatType) {
	case FT_COLUMNS:
	case FT_TREE:
	    if (fb->CMSize)
		AddDevDesc(DevInfo, itoa(fb->CMSize), "Color Map Size",
			   DA_INSERT);
	    if (fb->Size)
		AddDevDesc(DevInfo, 
			   GetSizeStr((Large_t)fb->Size, BYTES),
			   "Total Size", DA_INSERT);
	    if (fb->VMSize)
		AddDevDesc(DevInfo, 
			   GetSizeStr((Large_t)fb->VMSize, BYTES),
			   "Video Memory", DA_INSERT);
	    if (fb->Height)
		AddDevDesc(DevInfo, itoa(fb->Height), "Height", DA_APPEND);
	    if (fb->Width)
		AddDevDesc(DevInfo, itoa(fb->Width), "Width", DA_APPEND);
	    if (fb->Depth)
		AddDevDesc(DevInfo, itoa(fb->Depth), "Depth (bits)", 
			   DA_APPEND);
	    break;
	case FT_REPORT:
	    if (fb->CMSize > 0) {
		RptData[0] = "colormapsize";	RptData[1] = itoa(fb->CMSize);
		Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 2);
	    }
		   
	    if (fb->VMSize > 0) {
		RptData[0] = "videomemsize";	RptData[1] = itoa(fb->VMSize);
		Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 2);
	    }

	    if (fb->Size > 0) {
		RptData[0] = "videomemsize";	RptData[1] = itoa(fb->Size);
		Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 2);
	    }

	    if (fb->Height > 0) {
		RptData[0] = "height";	RptData[1] = itoa(fb->Height);
		Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 2);
	    }

	    if (fb->Width > 0) {
		RptData[0] = "width";	RptData[1] = itoa(fb->Width);
		Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 2);
	    }

	    if (fb->Depth > 0) {
		RptData[0] = "depth";	RptData[1] = itoa(fb->Depth);
		Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 2);
	    }
	    break;
	}
    }

    PrintGeneric(Info);
}

/*
 * Print monitor info (FT_TREE)
 */
extern void DetailsMonitorTree(Info)
     PrintInfo_t	       *Info;
{
    Monitor_t	 	       *Mon = NULL;
    static char			Buff[BUFSIZ];
    register char	      **cpp;
    int				OffSet;

    if (!Info)
	return;

    OffSet = Info->OffSet;
    if (Info->DevInfo && Info->DevInfo->DevSpec)
	Mon = (Monitor_t *) Info->DevInfo->DevSpec;

    if (Mon->MaxHorSize)
	ShowLabel("Max Horizontal Image Size (cm)", 
		  itoa(Mon->MaxHorSize), OffSet);
    if (Mon->MaxVerSize)
	ShowLabel("Max Vertical Image Size (cm)", 
		  itoa(Mon->MaxVerSize), OffSet);
    if (Mon->Resolutions) {
	Buff[0] = CNULL;
	for (cpp = Mon->Resolutions; cpp && *cpp; ++cpp) {
	    if (Buff[0])
		StrAppend(Buff, sizeof(Buff), " ");
	    StrAppend(Buff, sizeof(Buff), *cpp);
	}
	ShowLabel("Supported Resolutions", Buff, OffSet);
    }
}

/*
 * Print monitor info (FT_REPORT)
 */
extern void DetailsMonitorReport(Info)
     PrintInfo_t	       *Info;
{
    Monitor_t	 	       *Mon = NULL;
    static char			Buff[BUFSIZ];
    static char		       *RptData[3];
    register char	      **cpp;

    if (!Info)
	return;

    if (Info->DevInfo && Info->DevInfo->DevSpec)
	Mon = (Monitor_t *) Info->DevInfo->DevSpec;

    if (!Mon)
	return;

    if (Mon->Resolutions) {
	Buff[0] = CNULL;
	for (cpp = Mon->Resolutions; cpp && *cpp; ++cpp) {
	    if (Buff[0])
		strcat(Buff, ",");
	    if (strlen(*cpp) + strlen(Buff) + 3 >= sizeof(Buff))
		break;
	    strcat(Buff, *cpp);
	}
	RptData[0] = "resolutions";
	RptData[1] = Buff;
	Report(CN_DEVICE, R_DESC, Buff, RptData, 2);
    }
    if (Mon->MaxHorSize) {
	RptData[0] = "maxhorsize";
	RptData[1] = itoa(Mon->MaxHorSize);
	Report(CN_DEVICE, R_DESC, PRTS(Info->DevInfo->Name), RptData, 2);
    }
    if (Mon->MaxVerSize) {
	RptData[0] = "maxversize";
	RptData[1] = itoa(Mon->MaxVerSize);
	Report(CN_DEVICE, R_DESC, PRTS(Info->DevInfo->Name), RptData, 2);
    }
}

/*
 * Print info about a Monitor
 */
extern void DetailsMonitor(Info)
     PrintInfo_t	       *Info;
{
    if (!Info || !Info->DevInfo)
	return;

    Info->DetailsReport = DetailsMonitorReport;

    if  (Info->DevInfo->DevSpec && VL_CONFIG) {
	Info->DetailsTreeText = DetailsMonitorTree;
	Info->DetailsTreeHtml = DetailsMonitorTree;
	Info->DetailsColumnsText = DetailsMonitorTree;
    }

    PrintGeneric(Info);
}

/*
 * Print info about a network interface in FT_TREE format
 */
static void DetailsNetIfTree(Info)
     PrintInfo_t	       *Info;
{
    DevInfo_t 		       *DevInfo;
    int 			OffSet;
    register NetIF_t	       *ni;
    static char			Buff[512];

    if (!Info)
	return;

    DevInfo = Info->DevInfo;
    OffSet = Info->OffSet;

    for (ni = (NetIF_t *) DevInfo->DevSpec; ni; ni = ni->NextAddr) {
	if (ni->AddrType)
	    ShowLabel("Address Type", ni->AddrType, OffSet);

	if (ni->HostAddr) {
	    snprintf(Buff, sizeof(Buff), "%-18s [%s]", ni->HostAddr,
		     (ni->HostName) ? ni->HostName : "<unknown>");
	    ShowLabel("Host Address", Buff, OffSet);
	}

	if (ni->NetAddr) {
	    snprintf(Buff, sizeof(Buff), "%-18s [%s]", ni->NetAddr, 
		     (ni->NetName) ? ni->NetName : "<unknown>");
	    ShowLabel("Network Address", Buff, OffSet);
	}

	if (ni->NetMask)
	    ShowLabel("Network Mask", ni->NetMask, OffSet);

	if (ni->MACaddr) {
	    snprintf(Buff, sizeof(Buff), "%-18s [%s]", ni->MACaddr,
		     (ni->MACname && ni->MACname[0]) 
		     ? ni->MACname : "<unknown>");
	    ShowLabel("Current MAC Addr", Buff, OffSet);
	}

	if (ni->FacMACaddr) {
	    snprintf(Buff, sizeof(Buff), "%-18s [%s]", ni->FacMACaddr,
		     (ni->FacMACname && ni->FacMACname[0]) 
		     ? ni->FacMACname : "<unknown>");
	    ShowLabel("Factory MAC Addr", Buff, OffSet);
	}
    }
}

/*
 * Print info about a network interface in FT_REPORT format
 *
 * XXX Need to keep ShowIf() in shownetif.c in sync with this.
 *
 * Output format is:
 *
 *	device|desc|$Name|$AddrType|$HostAddr|$HostName|\
 *		$NetAddr|$NetName|$MACaddr|$MACname|
 *		$FacMACaddr|$FacMACname|$NetMask|$Broadcast
 */
static void DetailsNetIfReport(Info)
     PrintInfo_t	       *Info;
{
    DevInfo_t 		       *DevInfo;
    register NetIF_t	       *ni;
    static char		       *RptData[11];

    if (!Info)
	return;

    DevInfo = Info->DevInfo;

    for (ni = (NetIF_t *) DevInfo->DevSpec; ni; ni = ni->NextAddr) {
	RptData[0] = PRTS(ni->AddrType);
	RptData[1] = PRTS(ni->HostAddr);
	RptData[2] = PRTS(ni->HostName);
	RptData[3] = PRTS(ni->NetAddr);
	RptData[4] = PRTS(ni->NetName);
	RptData[5] = PRTS(ni->MACaddr);
	RptData[6] = PRTS(ni->MACname);
	RptData[7] = PRTS(ni->FacMACaddr);
	RptData[8] = PRTS(ni->FacMACname);
	RptData[9] = PRTS(ni->NetMask);
	RptData[10] = PRTS(ni->BroadAddr);
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData,
	       sizeof(RptData)/sizeof(char *));
    }
}

/*
 * Print info about a network interface
 */
extern void DetailsNetIf(Info)
     PrintInfo_t	       *Info;
{
    if (!Info || !Info->DevInfo)
	return;

    Info->DetailsReport = DetailsNetIfReport;

    if (VL_CONFIG) {
	Info->DetailsTreeText = DetailsNetIfTree;
	Info->DetailsTreeHtml = DetailsNetIfTree;
	Info->DetailsColumnsText = DetailsNetIfTree;
    }

    PrintGeneric(Info);
}

/*
 * Print Cache Info in TREE format.
 */
static void PrintCacheTree(Cache, Prefix, OffSet)
     CacheInfo_t	       *Cache;
     char		       *Prefix;
     int 			OffSet;
{
    static char			Buff[512];

    if (Cache->Enabled >= 0)
	ShowLabel(StrCat(Prefix, " Enabled?"), 
		  (Cache->Enabled) ? "TRUE" : "FALSE", OffSet);
    if (Cache->Size)
	ShowLabel(StrCat(Prefix, " Cache Size"), 
		  GetSizeStr(Cache->Size, BYTES), OffSet);
    if (Cache->Speed) {
	snprintf(Buff, sizeof(Buff), "%d ns", Cache->Speed);
	ShowLabel(StrCat(Prefix, " Speed"), Buff, OffSet);
    }
    if (Cache->Assoc)
	ShowLabel(StrCat(Prefix, " Associativity"), 
		  itoa(Cache->Assoc), OffSet);
    if (Cache->Lines)
	ShowLabel(StrCat(Prefix, " # Lines"), itoa(Cache->Lines), OffSet);
    if (Cache->LineSize)
	ShowLabel(StrCat(Prefix, " Line Size"), itoa(Cache->LineSize), OffSet);
    if (Cache->TLB)
	ShowLabel(StrCat(Prefix, " TLB Entries"), itoa(Cache->TLB), OffSet);
    if (Cache->TLBAssoc)
	ShowLabel(StrCat(Prefix, " TLB Associativity"), 
		  itoa(Cache->TLBAssoc), OffSet);
    if (Cache->Socketed >= 0)
	ShowLabel(StrCat(Prefix, " Socketed?"), 
		  (Cache->Socketed) ? "TRUE" : "FALSE", OffSet);
    if (Cache->Location)
	ShowLabel(StrCat(Prefix, " Location"), Cache->Location, OffSet);
    if (Cache->SramType)
	ShowLabel(StrCat(Prefix, " SRAM Type"), Cache->SramType, OffSet);
    if (Cache->EccType)
	ShowLabel(StrCat(Prefix, " ECC Type"), Cache->EccType, OffSet);
    if (Cache->Mode)
	ShowLabel(StrCat(Prefix, " Mode"), Cache->Mode, OffSet);
}

/*
 * Print info about a CPU in FT_TREE format
 */
static void DetailsCPUTree(Info)
     PrintInfo_t	       *Info;
{
    DevInfo_t 		       *DevInfo;
    int 			OffSet;
    CpuInfo_t		       *Cpu = NULL;
    Desc_t		       *Desc;

    if (!Info)
	return;

    DevInfo = Info->DevInfo;
    OffSet = Info->OffSet;

    if (DevInfo && DevInfo->DevSpec)
	Cpu = (CpuInfo_t *) DevInfo->DevSpec;

    if (!Cpu)
	return;

    if (Cpu->Arch)
	ShowLabel("Architecture", Cpu->Arch, OffSet);
    if (Cpu->ArchVer)
	ShowLabel("Architecture Version", Cpu->ArchVer, OffSet);
    if (Cpu->Speed)
	ShowLabel("Clock Speed", GetClockStr(Cpu->Speed, (Large_t)MHERTZ),
		  OffSet);
    if (Cpu->BitSize)
	ShowLabel("Bit Size", itoa(Cpu->BitSize), OffSet);
    if (Cpu->L1iCache)
	PrintCacheTree(Cpu->L1iCache, "L1 Inst", OffSet);
    if (Cpu->L1dCache)
	PrintCacheTree(Cpu->L1dCache, "L1 Data", OffSet);
    if (Cpu->L2Cache)
	PrintCacheTree(Cpu->L2Cache, "L2", OffSet);

    if (Cpu->Stepping)
	ShowLabel("Stepping", itoa(Cpu->Stepping), OffSet);
    if (Cpu->HasFPU)
	ShowLabel("HasFPU", "TRUE", OffSet);

    for (Desc = Cpu->Features; Desc; Desc = Desc->Next)
	ShowLabel(GetDescLabel(Desc), Desc->Desc, OffSet);
}

/*
 * Print info about a cache in FT_REPORT format.
 *
 * Output is in format:
 *
 *	device|desc|$CpuName|cacheinfo|$CacheName|...
 */
static void PrintCacheInfoReport(Info, Cache, CacheName)
     PrintInfo_t	       *Info;
     CacheInfo_t	       *Cache;
     char		       *CacheName;
{
    static char		       *RptData[16];

    if (!Info || !Info->DevInfo || !Cache)
	return;

    memset(&RptData, 0, sizeof(RptData));

    RptData[0] = CacheName;
    if (Cache->Size)	RptData[1] = strdup(GetSizeStr(Cache->Size, BYTES));
    if (Cache->Assoc)		RptData[2] = strdup(itoa(Cache->Assoc));
    if (Cache->Lines)		RptData[3] = strdup(itoa(Cache->Lines));
    if (Cache->LineSize)	RptData[4] = strdup(itoa(Cache->LineSize));
    if (Cache->TLB)		RptData[5] = strdup(itoa(Cache->TLB));
    if (Cache->TLBAssoc)	RptData[6] = strdup(itoa(Cache->TLBAssoc));
    if (Cache->Enabled >= 0)	RptData[7] = strdup(itoa(Cache->Enabled));
    if (Cache->Type >= 0)	RptData[8] = strdup(itoa(Cache->Type));
    if (Cache->Level >= 0)	RptData[9] = strdup(itoa(Cache->Level));
    if (Cache->Speed >= 0)	RptData[10] = strdup(itoa(Cache->Speed));
    if (Cache->Socketed >= 0)	RptData[11] = strdup(itoa(Cache->Socketed));
    if (Cache->Location)	RptData[12] = strdup(Cache->Location);
    if (Cache->SramType)	RptData[13] = strdup(Cache->SramType);
    if (Cache->EccType)		RptData[14] = strdup(Cache->EccType);
    if (Cache->Mode)		RptData[15] = strdup(Cache->Mode);

    Report(CN_DEVICE, R_CACHEINFO, PRTS(Info->DevInfo->Name), RptData,
	   sizeof(RptData)/sizeof(char *));
}

/*
 * CpuInfo_t Reporting
 */
static void DetailsCPUReport(Info)
     PrintInfo_t	       *Info;
{
    DevInfo_t 		       *DevInfo;
    CpuInfo_t		       *Cpu = NULL;
    static char		       *RptData[25];
    register DevDesc_t	       *DevDesc;

    if (!Info)
	return;

    DevInfo = Info->DevInfo;
    if (DevInfo && DevInfo->DevSpec)
	Cpu = (CpuInfo_t *) DevInfo->DevSpec;

    if (!Cpu)
	return;

    (void) memset(RptData, CNULL, sizeof(RptData));
    RptData[0] = Cpu->Man;
    RptData[1] = Cpu->Model;
    RptData[2] = Cpu->Arch;
    RptData[3] = Cpu->ArchVer;
    RptData[4] = Cpu->Serial;
    if (Cpu->BitSize)
	RptData[5] = strdup(itoa(Cpu->BitSize));
    if (Cpu->Speed)
	RptData[6] = (char *) strdup(GetClockStr(Cpu->Speed, (Large_t)MHERTZ));

    /*
     * Fields 7-22 previously contained L1 and L2 cache data.  These fields
     * are obsoleted by the "cacheinfo" data reported seperately.  The
     * fields are set to empty to preserve compatability.
     */

    if (Cpu->HasFPU)	RptData[23] = strdup(itoa(Cpu->HasFPU));
    if (Cpu->Stepping)	RptData[24] = strdup(itoa(Cpu->Stepping));

    Report(CN_DEVICE, R_CPUINFO, PRTS(DevInfo->Name), RptData, 
	   sizeof(RptData)/sizeof(char *));

    /*
     * Report CPU Features
     */
    for (DevDesc = Cpu->Features; DevDesc; DevDesc = DevDesc->Next) {
	McDebug("CpuReport: Feature Label=<%s> Desc=<%s>",
		PRTS(DevDesc->Label), PRTS(DevDesc->Desc));
	RptData[0] = PRTS(DevDesc->Label);
	RptData[1] = PRTS(DevDesc->Desc);
	Report(CN_DEVICE, R_DESC, PRTS(DevInfo->Name), RptData, 2);
    }

    /*
     * Report Cache info
     */
    if (Cpu->L1iCache)
	PrintCacheInfoReport(Info, Cpu->L1iCache, "L1 Inst");
    if (Cpu->L1dCache)
	PrintCacheInfoReport(Info, Cpu->L1dCache, "L1 Data");
    if (Cpu->L2Cache)
	PrintCacheInfoReport(Info, Cpu->L2Cache, "L2");
}

/*
 * Print info about a CPU
 */
extern void DetailsCPU(Info)
     PrintInfo_t	       *Info;
{
    if (!Info || !Info->DevInfo)
	return;

    Info->DetailsReport = DetailsCPUReport;

    if  (Info->DevInfo->DevSpec && VL_CONFIG) {
	Info->DetailsTreeText = DetailsCPUTree;
	Info->DetailsTreeHtml = DetailsCPUTree;
	Info->DetailsColumnsText = DetailsCPUTree;
    }

    PrintGeneric(Info);
}
