/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.19 $";
#endif

/*
 * Show Kernel related functions.
 */

#include "defs.h"

/*
 * Check to see if a symbol named String appears
 * in Argv.
 * Return 1 if found.
 * Return 0 if not found.
 */
static int HasName(String, Argv)
    char		       *String;
    char		      **Argv;
{
    register char	      **cpp;
    register char	       *cp;

    cp = String;
    if (*cp == '_')
	++cp;

    for (cpp = Argv; cpp && *cpp; ++cpp)
	if (EQ(cp, *cpp))
	    return(1);

    return(0);
}

/*
 * Show kernel variables
 */
extern int KernelShow(MyClass, Names)
    ClassInfo_t		       *MyClass;
    char		      **Names;
{
    Define_t		       *KernDef;
    Define_t		       *DefPtr;
    static char		       *RptData[3];
    int				Len;
    int				Max[3];
    static MCSIquery_t		Query;

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Op = MCSIOP_CREATE;
    Query.Cmd = MCSI_KERNELVAR;
    if (mcSysInfo(&Query) != 0)
	return -1;

    KernDef = (Define_t *) Query.Out;
    if (!KernDef) {
	SImsg(SIM_WARN, "No kernel variables are defined.");
	return CLASS_UNSUPPORTED;
    }

    /*
     * Find the longest descriptive string for later use.
     */
    Max[0] = Max[1] = Max[2] = -1;
    for (DefPtr = KernDef; DefPtr; DefPtr = DefPtr->Next) {
	if (!DefPtr->ValStr3 || (Names && HasName(DefPtr->KeyStr, Names)))
	    continue;

	/* Description */
	if (DefPtr->ValStr2) {
	    Len = (int) strlen(DefPtr->ValStr2);
	    if (Len > Max[0])
		Max[0] = Len;
	}

	if (VL_ALL) {
	    /* Key/Name */
	    if (DefPtr->KeyStr) {
		Len = (int) strlen(DefPtr->KeyStr);
		if (Len > Max[1])
		    Max[1] = Len;
	    }
	}
    }

    ShowStart(MyClass, Max);

    /*
     * For each variable in the definetion list that we have an
     * address for, read the value from the kernel using a data type
     * specific function.
     */
    for (DefPtr = KernDef; DefPtr; DefPtr = DefPtr->Next) {
	if (!DefPtr->ValStr3 || (Names && HasName(DefPtr->KeyStr, Names)))
	    continue;

	switch (FormatType) {
	case FT_TREE:
	    ShowKeyValueTree(DefPtr->ValStr2, DefPtr->KeyStr, 
			     DefPtr->ValStr3, Max);
	    break;
	case FT_COLUMNS:
	    ColShowValue(MyClass,
			 DefPtr->ValStr2, DefPtr->KeyStr, DefPtr->ValStr3);
	    break;
	case FT_REPORT:
	    RptData[0] = DefPtr->ValStr2;
	    RptData[1] = DefPtr->ValStr3;
	    Report(CN_KERNEL, NULL, DefPtr->KeyStr, RptData, 2);
	    break;
	}
    }

    ShowEnd(MyClass);

    return 0;
}

/*
 * List valid arguments for the Kernel class.
 * XXX Maybe we should nlist the variables first and only list
 * those variables we find?
 */
extern void KernelList(ClassInfo_t *Class)
{
    Define_t		       *KernDef;
    char		       *SymName;

    KernDef = DefGetList(DL_KERNEL); 
    if (!KernDef) {
	McDebug("No kernel variables are defined.");
	return;
    }

    if (FormatType == FT_REPORT) {
	for ( ; KernDef; KernDef = KernDef->Next) {
	    SymName = KernDef->KeyStr;
	    if (*SymName == '_')
		++SymName;
	    ReportMember(Class->Name, SymName, KernDef->ValStr2);
	}
    } else {
	if (!VL_TERSE) {
	    SImsg(SIM_INFO,
		  "\n\nThe following are valid arguments for `%s':\n\n",
		  Class->ShowUsage);
	    SImsg(SIM_INFO, "%-25s %s\n", "NAME", "DESCRIPTION");
	}

	for ( ; KernDef; KernDef = KernDef->Next) {
	    SymName = KernDef->KeyStr;
	    if (*SymName == '_')
		++SymName;
	    SImsg(SIM_INFO, "%-25s %s\n", SymName, KernDef->ValStr2);
	}
    } 
}
