/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.9 $";
#endif

/*
 * Show License related functions.
 */

#include "defs.h"

/*
 * List valid arguments for License class. 
 */
extern void ListLicense(ClassInfo_t *Class)
{
    if (!VL_TERSE && (FormatType == FT_TREE || FormatType == FT_COLUMNS))
	SImsg(SIM_INFO, 
	  "\n\nTo show specific license use `%s'.\n\n", Class->ShowUsage);
}

/*
 * Report output
 */
static void LicenseReport(License, Level)
     License_t		       *License;
     int			Level;
{    
    /* Report primary name */
    Report(CN_LICENSE, R_NAME, PRTS(License->SoftName), NULL, 0);

    /*
     * Report properties
     */
    ReportProp(CN_LICENSE, License->SoftName, "softver", 
	       "Software Version", License->SoftVer);
    ReportProp(CN_LICENSE, License->SoftName, "type", 
	       "License Type", License->TypeStr);
    ReportProp(CN_LICENSE, License->SoftName, "typedesc", 
	       "License Type Description", License->TypeDescStr);
    if (License->Port)
	ReportProp(CN_LICENSE, License->SoftName, "port", 
		   "Server Port", itoa(License->Port));
    if (License->Count)
	ReportProp(CN_LICENSE, License->SoftName, "count", 
		   "# Licenses", itoa(License->Count));
    ReportProp(CN_LICENSE, License->SoftName, "file", 
	       "License File", License->File);
    ReportProp(CN_LICENSE, License->SoftName, "serversoft", 
	       "License Server Software", License->ServerSoft);
    ReportProp(CN_LICENSE, License->SoftName, "serverver", 
	       "License Server Version", License->ServerVer);
    ReportProp(CN_LICENSE, License->SoftName, "vendorserversoft", 
	       "Vendor License Server Software", License->VendorServerSoft);
    ReportProp(CN_LICENSE, License->SoftName, "vendorserverver", 
	       "Vendor License Server Version", License->VendorServerVer);
    ReportProp(CN_LICENSE, License->SoftName, "issuer", 
	       "Issuer", License->Issuer);
    ReportProp(CN_LICENSE, License->SoftName, "hostid", 
	       "Authorized Host ID", License->Hostid);
    ReportProp(CN_LICENSE, License->SoftName, "issuedate", 
	       "Issue Date", License->IssueDate);
    ReportProp(CN_LICENSE, License->SoftName, "startdate", 
	       "Start Date", License->StartDate);
    ReportProp(CN_LICENSE, License->SoftName, "expiredate", 
	       "Expire Date", License->ExpireDate);
    ReportProp(CN_LICENSE, License->SoftName, "serial", 
	       "Serial #", License->Serial);
    ReportProp(CN_LICENSE, License->SoftName, "key", 
	       "Key", License->Key);
}

/*
 * Display License info
 */
extern int ShowLicense(MyClass, Names)
    ClassInfo_t		       *MyClass;
    char		      **Names;
{
    static License_t 	       *Root = NULL;
    static MCSIquery_t		Query;
    License_t		       *License;

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Op = MCSIOP_CREATE;
    Query.Cmd = MCSI_LICENSE;
    Query.Options = MyClass->Options;
    if (mcSysInfo(&Query) != 0)
	return CLASS_UNSUPPORTED;

    Root = (License_t *) Query.Out;

    if (!Root || !Root->SoftName)
	return CLASS_UNSUPPORTED;

    ShowStart(MyClass, (int *)NULL);

    /*
     * Traverse list
     */
    for (License = Root; License; License = License->Next) {
	if (!License->SoftName)
	    continue;
	LicenseReport(License, 0);
    }

    ShowEnd(MyClass);

    return 0;
}
