/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.14 $";
#endif

/*
 * Show Printer related functions.
 */

#include "defs.h"

/*
 * List valid arguments for NetIf class. 
 */
extern void ListNetIf(ClassInfo_t *Class)
{
    if (!VL_TERSE && (FormatType == FT_TREE || FormatType == FT_COLUMNS))
	SImsg(SIM_INFO, 
	  "\n\nTo show specific NetIf use `%s'.\n\n", Class->ShowUsage);
}

/*
 * Show NetIf entry
 * XXX Need to keep DetailsNetIfReport() in showdevices.c in sync with this.
 */
static int ShowIf(NetIF_t *		NetIf)
{
    char *			cp;
    NetIfFlag_t *		f;

    /* Report primary name */
    Report(CN_NETIF, R_NAME, PRTS(NetIf->Name), NULL, 0);

    ReportProp(CN_NETIF, NetIf->Name,
	       "addrtype", "Address Type", NetIf->AddrType);
    ReportProp(CN_NETIF, NetIf->Name,
	       "iftype", "Interface Type", NetIf->IfType);
    ReportProp(CN_NETIF, NetIf->Name,
	       "hostaddr", "Host Address", NetIf->HostAddr);
    ReportProp(CN_NETIF, NetIf->Name,
	       "hostname", "Host Name", NetIf->HostName);
    ReportProp(CN_NETIF, NetIf->Name,
	       "macaddr", "Current MAC Address", NetIf->MACaddr);
    ReportProp(CN_NETIF, NetIf->Name,
	       "macname", "Current MAC Name", NetIf->MACname);
    ReportProp(CN_NETIF, NetIf->Name,
	       "facmacaddr", "Factory MAC Address", NetIf->FacMACaddr);
    ReportProp(CN_NETIF, NetIf->Name,
	       "facmacname", "Factory MAC Name", NetIf->FacMACname);
    ReportProp(CN_NETIF, NetIf->Name,
	       "netaddr", "Network Address", NetIf->NetAddr);
    ReportProp(CN_NETIF, NetIf->Name,
	       "netname", "Network Name", NetIf->NetName);
    ReportProp(CN_NETIF, NetIf->Name,
	       "netmask", "Network Mask", NetIf->NetMask);
    ReportProp(CN_NETIF, NetIf->Name,
	       "broadcast", "Broadcast Address", NetIf->BroadAddr);
    ReportProp(CN_NETIF, NetIf->Name,
	       "mediatype", "Media Type", NetIf->MediaType);
    ReportProp(CN_NETIF, NetIf->Name,
	       "mediasubtype", "Media Sub Type", NetIf->MediaSubType);
    ReportProp(CN_NETIF, NetIf->Name,
	       "speed", "Link Speed", NetIf->Speed);
    ReportProp(CN_NETIF, NetIf->Name,
	       "duplex", "Link Duplex", NetIf->Duplex);
    if (NetIf->MtuSize > 0)
	ReportProp(CN_NETIF, NetIf->Name,
		   "mtu", "MTU Size", itoa(NetIf->MtuSize));
    for (f = NetIf->Flags; f; f = f->Next) {
	ReportProp(CN_NETIF, NetIf->Name, "flag", f->Name, PRTS(f->Desc));
    }

    return 0;
}

/*
 * Display NetIf info
 */
extern int ShowNetIf(ClassInfo_t *	MyClass,
		     char **		Names)
{
    static NetIF_t 	       *Root = NULL;
    static MCSIquery_t		Query;
    NetIF_t		       *NetIf;

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Op = MCSIOP_CREATE;
    Query.Cmd = MCSI_NETIF;
    if (mcSysInfo(&Query) != 0)
	return CLASS_UNSUPPORTED;

    Root = (NetIF_t *) Query.Out;

    if (!Root || !Root->Name)
	return CLASS_UNSUPPORTED;

    ShowStart(MyClass, (int *)NULL);

    /*
     * Traverse list
     */
    for (NetIf = Root; NetIf; NetIf = NetIf->Next) {
	if (!NetIf->Name)
	    continue;
	ShowIf(NetIf);
    }

    ShowEnd(MyClass);

    return 0;
}
