/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/5.0/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.11 $";
#endif

/*
 * Show Printer related functions.
 */

#include "defs.h"

/*
 * Master table of Network info
 */
ShowInfo_t NetworkShowInfo[] = {
    { "defgwip",	"Default Gateway IP",		MCNI_DEFGWIP },
    { "defgwname",	"Default Gateway Name",		MCNI_DEFGWNAME },
    { "dnsdomain",	"DNS Domain Name",		MCNI_DNSDOMAIN },
    { "dnsserverips",	"DNS Server IP Addresses",	MCNI_DNSSERVERIP },
    { "dnsservernames",	"DNS Server Names",		MCNI_DNSSERVERNAME },
    { "nisdomain",	"NIS Domain Name",		MCNI_NISDOMAIN },
    { "nisservermethod","NIS Server Method",		MCNI_NISSERVERMETHOD },
    { "nisserverips",	"NIS Server IP Addresses",	MCNI_NISSERVERIP },
    { "nisservernames",	"NIS Server Names",		MCNI_NISSERVERNAME },
    { "nisserver",	"NIS Current Server",		MCNI_NISSERVERCUR },
    { "nismaster",	"NIS Master Server",		MCNI_NISMASTER },
    { 0 },
};

/*
 * Show Network
 */
static int ShowNetworkInfo(Network_t *	Net,
			  char **	Names)
{
    static MCSIquery_t		Query;
    static char		       *RptData[3];
    int				Max[3];
    register char	      **cpp;
    char		       *cp;
    char		       *Value;
    register ShowInfo_t	       *ShowPtr;
    int				Found = 0;
    register int		Len = 0;

    /*
     * Calculate Max length for Label and Key (Name)
     */
    Max[0] = Max[1] = Max[2] = -1;
    for (ShowPtr = NetworkShowInfo; ShowPtr->Name; ++ShowPtr) {
	if (!ShowPtr->Label || !ShowPtr->Name)
	    continue;
	Len = strlen(ShowPtr->Label);
	if (Len > Max[0])
	    Max[0] = Len;

	if (VL_ALL) {
	    Len = strlen(ShowPtr->Name);
	    if (Len > Max[1])
		Max[1] = Len;
	}
    }

    for (ShowPtr = NetworkShowInfo; ShowPtr->Name; ++ShowPtr)
	if (ShowPtr->Enabled) {
	    switch (ShowPtr->GetType) {
	    case MCNI_DEFGWIP:		Value = Net->DefGatewayIP;	break;
	    case MCNI_DEFGWNAME:	Value = Net->DefGatewayName;	break;
	    case MCNI_DNSDOMAIN:	Value = Net->DnsDomain;		break;
	    case MCNI_DNSSERVERIP:
		Value = Join(",", Net->DnsServerIPs, 0);		break;
	    case MCNI_DNSSERVERNAME:
		Value = Join(",", Net->DnsServerNames, 0);		break;
	    case MCNI_NISDOMAIN:	Value = Net->NisDomain;		break;
	    case MCNI_NISSERVERMETHOD:	Value = Net->NisServerMethod;	break;
	    case MCNI_NISSERVERIP:
		Value = Join(",", Net->NisServerIPs, 0);		break;
	    case MCNI_NISSERVERNAME:
		Value = Join(",", Net->NisServerNames, 0);		break;
	    case MCNI_NISSERVERCUR:	Value = Net->NisServerCurrent;	break;
	    case MCNI_NISMASTER:	Value = Net->NisMaster;		break;
	    default:
		McDebug("ShowNetwork: Show type 0x%X is unknown",
			ShowPtr->GetType);
		continue;
	    }

	    /* Remove trailing newline */
	    if (Value && (cp = strchr(Value, '\n')))
		*cp = CNULL;

	    RptData[0] = PRTS(ShowPtr->Label);
	    RptData[1] = PRTS(Value);
	    Report(CN_NETWORK, NULL, ShowPtr->Name, RptData, 2);
	}

    return 0;
}

/*
 * Display Network info
 */
extern int ShowNetwork(ClassInfo_t *	MyClass,
		       char **		Names)
{
    static Network_t * 	        Root = NULL;
    static MCSIquery_t		Query;
    ShowInfo_t *	        ShowPtr;
    Network_t *		        Network;
    int				Found = FALSE;
    char **			cpp;
    int				Valid = 0;
    int				Types = 0;

    if (Names) {
	/*
	 * Enable only the specific items specified by user
	 */
	for (cpp = Names; cpp && *cpp; ++cpp) {
	    Valid = FALSE;
	    for (ShowPtr = NetworkShowInfo; ShowPtr->Name; ++ShowPtr)
		if (EQ(*cpp, ShowPtr->Name)) {
		    Valid = TRUE;
		    break;
		}
	    if (!Valid) {
		SImsg(SIM_CERR, 
		      "%s: Not a valid -show value.  Use `-list show` for a list of values.",
		      *cpp);
		exit(1);
	    }
	    if (ShowPtr->Name && ShowPtr->GetType) {
		ShowPtr->Enabled = TRUE;
		Types |= ShowPtr->GetType;
		Found = TRUE;
	    }
	}
    } else {
	Found = TRUE;
	/*
	 * No items specified so enable everything
	 */
	for (ShowPtr = NetworkShowInfo; ShowPtr->Name; ++ShowPtr) {
	    ShowPtr->Enabled = TRUE;
	    Types |= ShowPtr->GetType;
	}
    }

    if (!Found) {
	SImsg(SIM_DBG, "ShowNetwork: Nothing found.");
	return -1;
    }

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Op = MCSIOP_CREATE;
    Query.Cmd = MCSI_NETWORK;
    Query.SubClasses = Types;		/* Tell what info we want */
    if (mcSysInfo(&Query) != 0)
	return CLASS_UNSUPPORTED;

    Root = (Network_t *) Query.Out;

    if (!Root)
	return CLASS_UNSUPPORTED;

    ShowStart(MyClass, (int *)NULL);

    ShowNetworkInfo(Root, Names);

    ShowEnd(MyClass);

    return 0;
}
